/*Macro function to generate a variance-mean table and a subset of most variable genes;*/
%global StudyName InData testaddgene InGenes inputGenes testanova ExpInData topGenes minFeature maxFeature minCell MtPercent 
CalculationMethod min_disp max_disp min_mean max_mean OutData nullqclog nullqc basicinfo OutPath ServerOutPath MacroPath VariableGeneGraphScript 
varName OutDataScript OutDataScriptName syserr exiterror clockrunning rmMt rmRib;

%macro ODSOff(); /* Call prior to BY-group processing */
ods graphics off;
ods exclude all;
ods noresults;
%mend;
 
%macro ODSOn(); /* Call after BY-group processing */
ods graphics on;
ods exclude none;
ods results;
%mend;

/*subset gene tables based on gene list saved in a separate table*/
%macro subsetGenes(genesToKeep,TBtoSubset,outdatname,outputdir);
	
	libname tmpLib "&outputdir";
	/*Check duplicate in genesToKeep*/
	proc sql;
	create table &genesToKeep._nodup as
	select distinct(Variable) from &genesToKeep;
	quit;
	
	
	/*transpose the dataset to subset*/
	
	proc transpose data=tmpLib.&TBtoSubset name=Features
	out=TBtoSubset_ttmp(drop=_LABEL_);
	id Barcode;
	run;

	proc datasets lib=work nolist;
	   modify TBtoSubset_ttmp;
	   attrib _all_ label=' ';
	quit;

	/*Remove genes through inner join*/
	proc sql noprint;
		create table tmpLib.&outdatname._t as
		select x.* from TBtoSubset_ttmp as x, &genesToKeep._nodup as y
		where upcase(x.Features)=upcase(y.Variable);
	quit;
	


	/*transpose it back*/
	proc transpose data=tmpLib.&outdatname._t name=Barcode
	out=tmpLib.&outdatname;
	id Features;
	run;

	proc datasets lib=tmpLib nolist;
	   modify &outdatname;
	   attrib _all_ label=' ';
	quit;
	
	
	proc sql;
	drop table TBtoSubset_ttmp;
	quit;
	libname tmpLib clear;
%mend;


/*Remove mito and ribsome genes*/
%macro rmMtRibGenes(inputData,outpath,rmMt, rmRib);
	libname tmpLib "&outpath";
		/*mt_/mt-*/
	
	proc transpose data=tmpLib.&inputData name=Features
	out=TBtoClean_ttmp(drop=_LABEL_);
	id Barcode;
	run;
	
	%if %upcase(&rmMt)=YES %then %do;
		%put Remove mt genes;
		Data TBtoClean_ttmp;
		set TBtoClean_ttmp;
		IF find(Features,"mt","i")=1 AND find(Features,'_')=3 THEN DELETE;
		Run;
	%end;

	/*^Rp[sl][[:digit:]*/
	%if %upcase(&rmRib)=YES %then %do;
		%put Remove rib genes;
		Data TBtoClean_ttmp;
		set TBtoClean_ttmp;
		IF prxmatch("/^RP[SL][0-9]|^RPLP[0-9]|^RPSA/i",Features)>0 THEN DELETE;
		Run;	
	%end;
	
	/*transpose it back*/
	proc transpose data=TBtoClean_ttmp name=Barcode
	out=tmpLib.&inputData;
	id Features;
	run;

	proc datasets lib=tmpLib nolist;
	   modify &inputData;
	   attrib _all_ label=' ';
	quit;
	
	proc sql;
		drop table TBtoSubset_ttmp;
	quit;
	libname tmpLib clear;
%mend;


* Check if user provided data name is a valid SAS data set name;
%macro CheckDataNameLen(_name=,_default=,_SDS=0,NameLen=0);
	%if (&SASMode ne SDD) or (&_SDS=1) %then %do;
		%if ^%length(&&&_name) %then %let &_name = %trim(%left(&_default));
		data _null_; 
			name = trim(left(lowcase("&&&_name")));
			j=0;
			do until(j=0);
				j=notname(name,j+1);
				if j>0 then substr(name,j,1)=' ';
			end;
			name = translate(compbl(name),'_','');
			if substr(name,1,1) in ('0','1','2','3','4','5','6','7','8','9') then
				name2='_'||name;
			else name2=name;
			if length(name2) < &NameLen then call symput("&_name",trim(name2));
			else call symput("&_name",trim(substr(name2,1,min(length(name2),&NameLen))));

		run;
	%end;
%mend CheckDataNameLen;

/*Modify or create edf*/
%macro GenerateEdf;
	/*%if %upcase(&testanova)=YES %then %do;
		proc transpose data=outLib.&OutData._cnt name=Features
		out=outLib.&OutData._ttmp(drop=_LABEL_);
		id Barcode;
		run;
		proc datasets lib=outLib nolist;
		   modify &OutData._ttmp;
		     attrib _all_ label=' ';
		quit;

		options nosource nonotes errors=1;
		data outLib.&OutData._t(drop=i);
		set outLib.&OutData._ttmp;
		array _n(*) _numeric_;
		do i=1 to dim(_n);
			if _n(i)=. then _n(i)=0;
		end;
		run;

		proc sql;
		  drop table outLib.&OutData._ttmp;
		quit;
		*/
		%if %length(&ExpInData) %then %do;
			%let nameloc=%sysfunc(countw(&ExpInData,'\'));
			%put &nameloc;
			%let edftmp=%scan(&ExpInData,&nameloc,'\');
			%let edfname=%scan(&edftmp,1,'.');
			%put &edfname;
			%let edfpath = %sysfunc(tranwrd(&ExpInData., &edftmp., %str()));
			
			%CheckDataNameLen(_name=edfname,_default=,_SDS=1,NameLen=28);
			
			libname pathlib "&edfpath";
			data outLib.edftmp;
			set pathlib.&edfname;
			ColumnName=tranwrd(ColumnName,'-','_');
			run;

			Proc sql;
			create table outLib.&edfname._sub as
			Select x.* from outLib.edftmp as x, outLib.&OutData._cnt as y
			where x.ColumnName=y.Barcode;
			quit;

			proc sql;
			  drop table outLib.edftmp;
			quit;

			%let ExpInData=&OutPath.&edfname._sub.sas7bdat;
		%end;
		%else %do;
			proc sql;
			  create table outLib.&OutData._edf as
			  select Barcode as ColumnName, monotonic() as Array
			  from outLib.&OutData._cnt;
			quit;
			%let ExpInData=&OutPath.&OutData._edf.sas7bdat;
		%end;
	/*%end;*/
%mend;



/*Main function of variable gene selection*/
%macro VariableGeneSelection;
	OPTIONS MPRINT;
	OPTIONS SPOOL;
	options dkricond=nowarning dkrocond=nowarning ;
	%let sizeFactor=10000;
	%let ProcessName = VariableGeneSelection; 
	%let FullAPName = Variable Gene Selection;
	%local geneexist;
	%let numInputGenes=0;
	libname outLib "&OutPath";
	
	%include "&MacroPath./UtilityMacros.sas" / nosource;
	%if %length(&InGenes) AND &testaddgene=Yes %then %do;
		%PathName(&InGenes);
		%let genePath=&TmpPath;
		%let geneName=&TmpName;
		
		libname genLib "&genePath";
		%let geneRef=genLib.&geneName;
		%if %sysfunc(exist(&geneRef)) %then %do;
			%let geneexist=1;
			
			proc contents
			     data = genLib.&geneName
			      noprint
			      out = data_info
			     (keep = name varnum);
			run;

			proc sort
			     data = data_info
			     out = variable_names(keep = name);
			     by varnum;
			run;

			proc sql noprint;
			select NAME into :genevarname separated by ' '
			from variable_names;
			quit;
			/*Modify this part to make it subset gene column into a table*/
			%let geneVarCnt=%sysfunc(countw(&genevarname));
			%if &geneVarCnt ge 1 %then %do;
				%let genevar=%qscan(&genevarname,1,%str( ));
				
				proc sql noprint;
					select trim(&genevar), count(*)
					into :inputGenes separated by ' ', :numInputGenes
					from genLib.&geneName;
				quit;
				
				data outLib.&geneName;
					set genLib.&geneName(keep=&genevar);
				run;
				
				%put number of input gene is &numInputGenes;
			%end;
			%else %do; No coumn exist; %end;
		%end;
		%else %do; %let geneexist=0; %end;
	%end;
	%else %do; %put no gene file is provided; %end;
	
	%GetOrigFileName(InData,OrigInData);
	%PathName(&InData);
	%if &exiterror %then %do;
		%put ERROR: Input data set specification is invalid.;
		%goto exit;
	%end;
	
	%let subsetID=TopVariableGenesID;
	
	
	libname inLib "&TmpPath";
	
	

	/*check name for OutData;*/
	/*%if ^%length(&OutData) %then %AddSuffixToDataName(_dap,&OrigInData,OutData,_SDS=1);
		%else %CheckDataName(_name=OutData,_default=,_SDS=1);
	*/
	
	%if ^%length(&OutData) %then %AddSuffixToDataName(_dap,&OrigInData,OutData,_SDS=1);
		%else %CheckDataNameLen(_name=OutData,_default=,_SDS=1,NameLen=21);
	
	%let varName=&OutData._var;
	options nosource nonotes errors=1;
	%let suffix=_nll;
	%let prefix=&OutData._dns;
	%let densename=&prefix.&suffix;

	data &densename(drop=i);
	set inLib.&TmpName;
	array _n(*) _numeric_;
	do i=1 to dim(_n);
		if _n(i)=0 then _n(i)=.;
	end;
	run;
	options source notes;
	
	/**Transpose to calculate count of cells per gene;*/
	%let denseT=&prefix._T;
	proc transpose data=&densename name=Variable
	out=&denseT;
	id Barcode;
	run;

	/**Calculate number of nonmissing cells for each gene;
	**Suppress the error message, change from 0 to other number to turn it off;*/

	options nosource nonotes errors=1;
	data &denseT;
	set &denseT;
	array _n(*) _numeric_;
	ncells = N(of _all_);
	run;
	options source notes;

	/**Drop genes expressed in less than 3 cells;*/
	data &denseT (drop=ncells);
	set &denseT (where=(ncells>=&minCell));
	run;
	
	/**Transpose back after low count genes removed;*/
	options nosource nonotes errors=1;
	%let densecell=&prefix._mncl;
	proc transpose data=&denseT name=Barcode
	out=&densecell;
	id Variable;
	**idlabel _LABEL_;
	run;
	options source notes;

	/**Get read, feature, and Mt read counts Version 3;
	**Suppress the error message, change from 0 to other number to turn it off;*/
	data &densecell;
	set &densecell;
	array _n(*) _numeric_;
	readCount=sum(of _n[*]);
	nfeatures = N(of _all_)-1;
	run;
	
	/**Extract mt columns only;*/
	%let densemt=&prefix._mt;
	proc sql;
	create table &densemt as
	select * from &densecell(keep=Barcode MT_: mt_:);
	quit;

	/*Check how many mt genes are found*/	
	proc contents data=&densemt noprint out=_mtcontents_ (keep=name);
	run;
	proc sql noprint;
	select strip(name) into :mtNames separated by ' '
	from _mtcontents_
	where upcase(name) like 'MT_%' AND find(name,'_')=3;
	quit;
	%let mt_count=%sysfunc(countw(&mtNames,%str( )));
	%put &mt_count;
	
	%if &mt_count>1 %then %do;
		/**Calculate mtCount;*/
		options nosource nonotes errors=1;
		data &densemt;
		set &densemt;
		array _n(*) _numeric_;
		mtCount=sum(of _n[*]);
		run;
		options source notes;
	%end;
	%else %do;
		data &densemt;
		set &densemt;
		mtCount=0;
		run;
	%end;

	/**Calculate mtPercent and merge back to original table;*/
	%let densecount=&prefix._cnt;
	proc sql;
	create table &densecount as
	select x.*, (y.mtCount/x.readCount)*100 as mtPercent from &densecell as x,
	&densemt as y
	where x.Barcode=y.Barcode;
	quit;

	/**Save read, feature, Mt percent data to a new file, will be in the report;*/
	%let basicinfo=&prefix._sts;
	data outLib.&basicinfo;
	set &densecount(keep=Barcode nfeatures readCount mtPercent);
	run;
	
	proc datasets lib=outLib nolist;
	   modify &basicinfo;
	   attrib _all_ label=' ';
	quit;
	
	/**Remove cells with extreme RNA count and high MT percentage;*/
	%let nullqc=&prefix._QCed;
	data outLib.&nullqc(drop=nfeatures mtPercent mtCount);
	set &densecount(where=(nfeatures>=&minFeature and nfeatures<=&maxFeature and mtPercent<&MtPercent));
	run;
	
	/*Remove mito and ribsome genes*/
	%rmMtRibGenes(&nullqc,&OutPath,&rmMt, &rmRib);
	
	/*Convert null back to zero to calculate mean and variance*/
	%let zeroqc=&prefix._zero;
	options nosource nonotes errors=1;
	data &zeroqc(drop=i readCount);
	set outLib.&nullqc;
	array _n(*) _numeric_;
	do i=1 to dim(_n);
		if _n(i)=. then _n(i)=0;
	end;
	run;
	
	/*Summary stats for zero filled & QCed matrix*/
	options source notes;
	ods exclude all;
	proc means data=&zeroqc nolabels
		   Var STD Mean N  
		   STACKODSOUTPUT;        
	ods output Summary=varmeanzero;
	run;
	ods exclude none;
	
	/*creat mean variance table with raw UMI counts; watch out for readCount*/
	ods exclude all;
	proc means data=outLib.&nullqc(drop=readCount) nolabels
		   Var STD Mean N  
		   STACKODSOUTPUT;        
	ods output Summary=varmeanraw;
	run;
	ods exclude none;
	%let nullqclog=&prefix._lg;
	%put &nullqclog;
	
	/*try to replace . with 0*/
	options nosource nonotes errors=1;
	data outLib.&nullqclog(drop=i readCount);
	set outLib.&nullqc;
	array _n(*) _numeric_;
	array _counts(*) readCount;
	do i=1 to dim(_n)-1;
		if _n(i)=. then _n(i)=0;
		_n(i)=log1px((_n(i)/_counts(1))*10000);
	end;
	run;
	
	options source notes;
	/*Standardize qced-log-transformed data*/
	%let qcscale=&prefix._std;
	proc stdize data=outLib.&nullqclog out=outLib.&qcscale method=STD Missing=0;
	run;
	
	/*Remove mito and ribsome genes*/
	/*%rmMtRibGenes(&nullqc,&OutPath,&rmMt, &rmRib);
	%rmMtRibGenes(&nullqclog,&OutPath,&rmMt, &rmRib);
	%rmMtRibGenes(&qcscale,&OutPath,&rmMt, &rmRib);*/
	
	
	%if "&CalculationMethod"="Dispersion" %then %do;
		
		%let varName=&varName._disp;
		/*Replace zero means with small number*/
		data varmeanzero;
		set varmeanzero;
		array _n(*) Mean;
		if _n(1)=0 then _n(1)=1e-12;
		run;
		
		/*Use log1px for Mean, log for Dispersion*/
		data varmeanzero;
		set varmeanzero;
		Dispersion=log(Var/Mean);
		logxMean=log1px(Mean);
		run;
		
		
		/*Standardize Dispersion by Bins*/
		proc rank data=varmeanzero
		out=varmeanzero
		groups=20;
		var logxMean;
		ranks MeanGrp;
		run;
		
		ods exclude all;
		proc means data=varmeanzero nolabels
			   STD Mean;
			   class MeanGrp;
			   var Dispersion;
			   **STACKODSOUTPUT;        
		ods output Summary=varmeanzero_binstats;
		run;
		ods exclude none;
		
		proc sql;
			create table varmeanzero_bins as
			select x.*, y.Dispersion_StdDev, y.Dispersion_Mean from varmeanzero as x,
			varmeanzero_binstats as y
			where x.MeanGrp=y.MeanGrp;
		quit;
		
		data outLib.&varName;
		set varmeanzero_bins;
		DispersionScale=(Dispersion-Dispersion_Mean)/Dispersion_StdDev;
		run;
		
		/**sort mean variance table by variance;*/
		proc sort data=outLib.&varName;
		by descending DispersionScale;
		run;
		
		/*Select genes based on dispersion and mean values provided by user*/
		%if %length(&min_disp)=0 %then %let min_disp=0.5;
		%if %length(&max_disp)=0 %then %let max_disp=100;
		%if %length(&min_mean)=0 %then %let min_mean=0.0125;
		%if %length(&max_mean)=0 %then %let max_mean=3;
		
		/*Create table instead of a list*/
		/*proc sql noprint;
		select trim(Variable), count(*)
		into :geneList separated by ' ', :numGenes
		from outLib.&varname
		where DispersionScale>&min_disp & DispersionScale<&max_disp & logxMean>&min_mean & logxMean<&max_mean;
		quit;*/
		
		proc sql noprint;
			select count(*) into :numGenes from outLib.&varname;
			create table geneList as
			select Variable from outLib.&varname
			where DispersionScale>&min_disp & DispersionScale<&max_disp & logxMean>&min_mean & logxMean<&max_mean;
		quit;
		
		%if &numGenes<10 %then %do;
			%put ERROR: There are too few genes selected, please adjust selection criteria.;
			%goto exit;
		%end;
		/*Merge user selected genes with variable genes*/
		%if  &geneexist=1 AND &numInputGenes>0 %then %do;
			/*%let inputGenes = %sysfunc(tranwrd(%quote(&inputGenes),%str(.),%str(_)));
			%let inputGenes = %sysfunc(tranwrd(%quote(&inputGenes),%str(-),%str(_)));
			%let geneList=Barcode &inputGenes &geneList;*/
			DATA genes2keep;
			keep Variable;
			length Variable $ 30;
			SET outLib.&geneName(rename=(&genevar=Variable)) geneList; 
			RUN;
		%end;
		%else %do;
			/*%let geneList=Barcode &geneList;*/
			Data genes2keep;
			Set geneList;
			Run;
		%end;
		
		
		/*Call function to subset three tables*/
		%if %eval(&numGenes+&numInputGenes)>0 %then %do;
			%let OutData=&OutData._disp;
			%subsetGenes(genes2keep,&nullqc,&OutData._cnt,&OutPath);
			%subsetGenes(genes2keep,&nullqclog,&OutData._lg,&OutPath);
			%subsetGenes(genes2keep,&qcscale,&OutData._std,&OutPath);
		%end;
		%else %do;
			%put ERROR: There are too few genes selected, please adjust selection criteria.;
			%goto exit;
		%end;
		
		%let topGenes=%trim(&numGenes);
		
		
		%GenerateEdf;
		
		proc sql;
			drop table varmeanzero;
			drop table varmeanraw;
			drop table varmeanzero_bins;
			drop table varmeanzero_binstats;
		quit;
	%end;
	%else %if "&CalculationMethod"="VST" %then %do;
	/**VST method;*/
		%put VST method!;
		%let varName=&varName._vst;
		/*Compare topGenes to total gene number*/
		%if %length(&topGenes)=0 %then %let topGenes=3000;
		/*Log10 transform variance and mean*/
		data varmeanzero_log10tmp;
		set varmeanzero;
		logVar=log10(Var);
		logMean=log10(Mean);
		run;
		
		/*Remove missing records*/
		data varmeanzero_log10;
		set varmeanzero_log10tmp;
		if cmiss(of _all_) then delete;
		run;
		
		%ODSOff;
		/**Perform loess regression on mean var table;*/
		proc loess data=varmeanzero_log10;
		   ods output OutputStatistics = varmeanzero_Loess;
		              /*FitSummary=Summary;*/
		   model logVar = logMean / degree=2 smooth=0.3;
		   ID Variable;
		run;
		
		/*Convert predicted variance back to original scale and take sqrt of it;*/
		data varmeanzero_Loess;
		set varmeanzero_Loess;
		PredVar=sqrt(10**Pred);
		Mean=10**logMean;
		run;
		
		/**Transpose the QCed data;*/
		proc transpose data=&zeroqc (drop=readCount) name=Variable
		out=outLib.&zeroqc._T;
		id Barcode;
		run;
		
		/**Merge transposed count data with means and predVar;*/
		proc sql;
		create table tmp_loess as
		select x.*, y.PredVar, y.Mean from outLib.&zeroqc._T (drop=_LABEL_) as x,
		varmeanzero_Loess as y
		where x.Variable=y.Variable;
		quit;
		
		/**Standardize the count data with loess results;*/
		data tmp_loessscaledT;
		set  tmp_loess;
		array Nums(*) _Numeric_;
		array Means(*) Mean;
		array PredVars(*) PredVar;
		do i=1 to dim(Nums)-2;
			if (Nums[i]-Means[1])/PredVars[1] < sqrt(dim(Nums)-2) then 
			Nums[i]=(Nums[i]-Means[1])/PredVars[1];
			else Nums[i]=sqrt(dim(Nums)-2);
		end;
		drop i PredVar Mean;
		run;
		
		/**Transpose back the standardized data;*/
		proc transpose data=tmp_loessscaledT name=Barcode
		out=tmp_LoessScaled;
		id Variable;
		run;
		
		/**Calculte new variance;*/
		ods exclude all;                  /* suppress display to open ODS destinations */
		proc means data=tmp_LoessScaled nolabels
		           Var STD Mean N  /* type every statistic you want */
		           STACKODSOUTPUT;        /* preserve table form of output */
		ods output Summary=varmean_Loess;  /* write statistics to data set */
		run;
		ods exclude none; /* restore normal display of tables */
		
		/**Merge two variable-mean tables;*/
		proc sql;
		create table outLib.&varName as
		select x.Variable, x.Var, x.StdDev, y.Mean from varmean_Loess as x,
		varmeanzero_Loess as y
		where x.Variable=y.Variable;
		quit;

		/**sort mean variance table by variance;*/
		proc sort data=outLib.&varName;
		by descending StdDev;
		run;
		
		data outLib.&varName;
			rename var=Variance;
			set outLib.&varName;
		run;

	
		%if &topGenes<10 %then %do;
			%put ERROR: There are too few genes selected, please adjust selection criteria.;
			%goto exit;
		%end;
		
		proc sql inobs=&topGenes noprint;
		create table geneList as
		select Variable from outLib.&varname;
		quit;
		
		%if  &geneexist=1 AND &numInputGenes>0 %then %do;
			/*%let inputGenes = %sysfunc(tranwrd(%quote(&inputGenes),%str(.),%str(_)));
			%let inputGenes = %sysfunc(tranwrd(%quote(&inputGenes),%str(-),%str(_)));
			%let geneList=Barcode &inputGenes &geneList;*/
			DATA genes2keep;
			keep Variable;
			length Variable $ 30;
			SET outLib.&geneName(rename=(&genevar=Variable)) geneList; 
			RUN;
		%end;
		%else %do;
			Data genes2keep;
				Set geneList;
			Run;
		%end;
		
		/**Subset input table with the list of variable genes;*/
		%if %eval(&topGenes+&numInputGenes)>0 %then %do;
			%let OutData=&OutData._vst;
			%subsetGenes(genes2keep,&nullqc,&OutData._cnt,&OutPath);
			%subsetGenes(genes2keep,&nullqclog,&OutData._lg,&OutPath);
			%subsetGenes(genes2keep,&qcscale,&OutData._std,&OutPath);
		%end;
		%else %do;
			%put ERROR: There are too few genes selected, please adjust selection criteria.;
			%goto exit;
		%end;
		
		
		%GenerateEdf;
		proc sql;
			drop table varmeanzero;
			drop table varmeanraw;
			drop table varmeanzero_log10;
			drop table varmeanzero_log10tmp;
			drop table varmean_loess;
			drop table varmeanzero_loess;
		quit;
	%end;
	
	proc sql;
	drop table geneList;
	drop table genes2keep;
	drop table &prefix._nll;
	drop table &prefix._t;
	drop table &prefix._mt;
	drop table outLib.&prefix._mncl;
	drop table outLib.&prefix._cnt;
	drop table &prefix._zero;
	quit;
	
	
%exit:;
%mend;

%VariableGeneSelection;

