### Note 1: Arguments are passed from SAS to Python in sys.argv[]
### Note 2: XGBoost arguments description can be found at https://xgboost.readthedocs.io/en/latest/parameter.html

### Customized python functions ###
def CheckForPackage(package_name):
	spec = importlib.util.find_spec(package_name)
	if spec is None:
	    print(package_name +" is not installed")


def FillInParamLst(ParamTypeIn, ParamListOut):
	for i in range(len(ParamTypeIn)): 
		#print(i+1)
		if (sys.argv[i+1] != "None"):
			ps= ParamTypeIn[i].split(':')
			#print(ps[0])
			if (ps[1] == 'int'):
				ParamListOut[ps[0]] = int(sys.argv[i+1]) #ps[1]+"("+ sys.argv[i+1] +")"
			elif (ps[1] == 'float'):
				ParamListOut[ps[0]] = float(sys.argv[i+1]) #ps[1]+"("+ sys.argv[i+1] +")"
			elif (ps[1] == 'str'):
				ParamListOut[ps[0]] = sys.argv[i+1] #ps[1]+"("+ sys.argv[i+1] +")"
			elif (ps[1] == 'strs'):
				ParamListOut[ps[0]] = sys.argv[i+1].split(',') #sys.argv[i+1].split(',')
	return(ParamListOut)

def create_feature_map(features):
    outfile = open(os.path.join(sys.argv[46],sys.argv[50]+"_xgb.fmap"), 'w')
    i = 0
    for feat in features:
        outfile.write('{0}\t{1}\tq\n'.format(i, feat))
        i = i + 1
    outfile.close()
    
### Load libraries
import importlib.util
import sys
import os

CheckForPackage('xgboost')
CheckForPackage('pandas')
CheckForPackage('numpy')
CheckForPackage('pickle')
CheckForPackage('operator')
CheckForPackage('sklearn')

import xgboost as xgb
import pandas as pd
import numpy as np
import pickle 
import operator
from sklearn.preprocessing import LabelEncoder
#from sklearn.preprocessing import OneHotEncoder    

### Print files paths and target variable
#print(len(sys.argv))
print("Python File: " + sys.argv[0])
print("Train File: " + sys.argv[44])
print("Target Variable " + sys.argv[45])
print("Output Path: " + sys.argv[46])
print("Output Pred File: " + sys.argv[47])
print("Test File: " + sys.argv[48])
print("Output Test File: " + sys.argv[49])
print("OutName: " + sys.argv[50])

### Define parameter dictionary
param = {}

"""
### General Parameters, see comment for each definition 
param['booster'] = sys.argv[1]
param['verbosity'] = int(sys.argv[2])
if sys.argv[3] != "None": 
	param['nthread'] = int(sys.argv[3])
param['disable_default_eval_metric'] = int(sys.argv[4])

### Learning Task Parameters
param['objective'] =  sys.argv[5]
param['num_class'] = int(sys.argv[6])

param['base_score'] =  float(sys.argv[6])
param['eval_metric'] = sys.argv[7].split(',')
param['seed'] = int(sys.argv[8])

if (sys.argv[1] == "gbtree" or sys.argv[1] == "dart"):
	### Tree Booster Parameters
	param['eta'] = float(sys.argv[9])
	param['gamma'] = float(sys.argv[10])
	param['max_depth'] = int(sys.argv[11])
	param['min_child_weight'] = float(sys.argv[12])
	param['max_delta_step'] = float(sys.argv[13])
	param['subsample'] = float(sys.argv[14])
	param['colsample_bytree'] = float(sys.argv[15])
	param['colsample_bylevel'] = float(sys.argv[16])
	param['colsample_bynode'] = float(sys.argv[17])
	param['lambda'] = float(sys.argv[18]) #lambda is defined for both tree and linear boosters
	param['alpha'] = float(sys.argv[19]) #alpha is defined for both tree and linear boosters
	param['tree_method'] = sys.argv[20]
	param['sketch_eps'] = float(sys.argv[21])
	param['scale_pos_weight'] = float(sys.argv[22])
	if (sys.argv[23] != "None"):
		param['updater'] = sys.argv[23] #updater is defined for both tree and linear boosters
	param['refresh_leaf'] = int(sys.argv[24])
	param['process_type'] = sys.argv[25]
	param['grow_policy'] = sys.argv[26]
	if (sys.argv[27] != "None"):
		param['max_leaves'] = int(sys.argv[27])
	if (sys.argv[28] != "None"):	
		param['max_bin'] = int(sys.argv[28])
	param['predictor'] = sys.argv[29]
	param['num_parallel_tree'] = int(sys.argv[30])	
elif (sys.argv[1] == "dart"):
	### Dart Booster Parameters
	if (sys.argv[31] != "None"):
		param['sample_type'] = sys.argv[31]
	if (sys.argv[32] != "None"):
		param['normalize_type'] = sys.argv[32]
	if (sys.argv[33] != "None"):
		param['rate_drop'] = float(sys.argv[33])
	if (sys.argv[34] != "None"):
		param['one_drop'] = int(sys.argv[34])
	if (sys.argv[35] != "None"):
		param['skip_drop'] = float(sys.argv[35])
elif (sys.argv[1] == "gblinear"):
	### Linear Booster Parameters
	if (sys.argv[36] != "None"):
		param['lambda'] = float(sys.argv[36]) #lambda is defined for both tree and linear boosters
	if (sys.argv[37] != "None"):
		param['alpha'] = float(sys.argv[37]) #alpha is defined for both tree and linear boosters
	if (sys.argv[38] != "None"):
		param['updater'] = sys.argv[38] #updater is defined for both tree and linear boosters
	if (sys.argv[39] != "None"):
		param['feature_selector'] = sys.argv[39]
	if (sys.argv[40] != "None"):
		param['top_k'] = int(sys.argv[40])

if (sys.argv[5] == "reg:tweedie"):
	### Tweedie Regression Parameters
	if (sys.argv[41] != "None"):
		param['tweedie_variance_power'] = sys.argv[41]
"""	

ParamLstType=[
	'booster:str', 
	'verbosity:int', 
	'nthread:int', 
	'disable_default_eval_metric:int', 
	'objective:str', 
	'num_class:int', 
	'base_score:float', 
	'eval_metric:strs', 
	'seed:int', 
	'eta:float', 
	'gamma:float', 
	'max_depth:int', 
	'min_child_weight:float', 
	'max_delta_step:float', 
	'subsample:float', 
	'colsample_bytree:float', 
	'colsample_bylevel:float', 
	'colsample_bynode:float', 
	'lambda:float', 
	'alpha:float', 
	'tree_method:str', 
	'sketch_eps:float', 
	'scale_pos_weight:float', 
	'updater:str', 
	'refresh_leaf:int', 
	'process_type:str', 
	'grow_policy:str', 
	'max_leaves:int', 
	'max_bin:int', 
	'predictor:str', 
	'num_parallel_tree:int', 
	'sample_type:str', 
	'normalize_type:str', 
	'rate_drop:float', 
	'one_drop:int', 
	'skip_drop:float', 
	'lambda:float', 
	'alpha:float', 
	'updater:str', 
	'feature_selector:str', 
	'top_k:int', 
	'tweedie_variance_power:str'
	]
	
#print(ParamLstType[1])
#print(ParamLstType[1].split(':'))

FillInParamLst(ParamLstType, param)
#print(param)

### Command Line Parameters
## Number of round to do boosting
num_round = int(sys.argv[43])
#print("Rounds: " + str(num_round))

### Load CSV train file 
# raw_dtrain = pd.read_csv(os.path.join(sys.argv[1], sys.argv[2]))
raw_dtrain = pd.read_csv(sys.argv[44])
## Preview the first 5 lines of the train data 
#print(raw_dtrain.head())

### Target variable name
DepVar= sys.argv[45]
#print(raw_dtrain[DepVar][0:5])

### Weight variable
WVar= sys.argv[51]
WValues = None #default

### Predictor variables
if (WVar == "None"):
	PredVars = raw_dtrain.columns.drop([DepVar])	
else:
	PredVars = raw_dtrain.columns.drop([DepVar, WVar])
	#print(raw_dtrain[WVar].values)
	
#SomeVars = raw_dtrain.columns[5:-1]
#print("PredVars: " + PredVars)

### Type of target variable (numeric or string)
DepVarType= raw_dtrain[DepVar].dtype
#print(DepVarType)

### Create flag for the presence of missing target values
YMissFlag = raw_dtrain[DepVar].isnull().values.any()
#print(YMissFlag)

### Dealing missing target values
if (YMissFlag):	
	### Take rows with nonmissing target values
	#print("Missing Target Values")
	dtrain = raw_dtrain.copy()
	#print(dtrain[pd.notnull(dtrain[DepVar])][DepVar].values)
	dtrain = dtrain[pd.notnull(dtrain[DepVar])]
	#print(dtrain.head())
	
	### Weigtht values
	if (WVar != "None"):
		WValues = dtrain[WVar].values

	### Encode string target variable class values as integers
	if (DepVarType == 'object'):
		label_encoder = LabelEncoder()
		#label_encoder = label_encoder.fit(dtrain[pd.notnull(dtrain[DepVar])][DepVar].values)
		#label_encoded_y = label_encoder.transform(dtrain[pd.notnull(dtrain[DepVar])][DepVar].values)
		label_encoder = label_encoder.fit(dtrain[DepVar].values)
		label_encoded_y = label_encoder.transform(dtrain[DepVar].values)
		#print(label_encoded_y[1:5])	

		### Define training data without samples without missing values of encoded target variable
		dtrain = xgb.DMatrix(dtrain[PredVars].values, label=label_encoded_y, feature_names= PredVars, missing=np.nan, weight= WValues)
	else:
		### Define training data without samples without missing values of original target variable
		dtrain = xgb.DMatrix(dtrain[PredVars].values, label=dtrain[DepVar].values, feature_names= PredVars, missing=np.nan, weight= WValues)
else:
	### Weigtht values
	if (WVar != "None"):
		WValues = raw_dtrain[WVar].values

	### Encode string target variable class values as integers
	if (DepVarType == 'object'):
		label_encoder = LabelEncoder()
		label_encoder = label_encoder.fit(raw_dtrain[DepVar])
		label_encoded_y = label_encoder.transform(raw_dtrain[DepVar])
		#print(label_encoded_y)	

		### Define training data with encoded target variable
		dtrain = xgb.DMatrix(raw_dtrain[PredVars].values, label=label_encoded_y, feature_names= PredVars, missing=np.nan, weight= WValues)
	else:
		### Define training data original target variable
		dtrain = xgb.DMatrix(raw_dtrain[PredVars].values, label=raw_dtrain[DepVar].values, feature_names= PredVars, missing=np.nan, weight= WValues)
	
### Create map of feature names
#print(dtrain.feature_names)
create_feature_map(dtrain.feature_names)

### Training model  
EvalResults= {}
model = xgb.train(param, dtrain, num_round, evals=[(dtrain,'train')], evals_result= EvalResults)

## Saving metric evaluatin history over iterations
EvalResults = pd.DataFrame.from_dict(EvalResults['train'])
#print(EvalResults)
#print(EvalResults.keys())
np.savetxt(os.path.join(sys.argv[46], sys.argv[50]+"_xgb_fit.csv"), EvalResults, header= ",".join(EvalResults.keys()), comments="", delimiter=",", fmt='%f')

if (param["booster"] == "gbtree" or param["booster"] == "dart"):
	#print("Importance Train: ")
	
	## Importance using get_score()
	impgain = model.get_score(fmap=os.path.join(sys.argv[46],sys.argv[50]+"_xgb.fmap"), importance_type='gain')
	#impgain = sorted(impgain.items(), key=operator.itemgetter(0))
	impgain = pd.DataFrame(impgain.items(), columns=['feature', 'gain'])	
	#print(impgain.head())
	
	impcover = model.get_score(fmap=os.path.join(sys.argv[46],sys.argv[50]+"_xgb.fmap"), importance_type='cover')
	#impcover = sorted(impcover.items(), key=operator.itemgetter(0))
	impcover = pd.DataFrame(impcover.items(), columns=['feature', 'cover'])	
	#print(impcover.head())
	
	## Importance using get_fscore()
	impfdf = model.get_fscore(fmap=os.path.join(sys.argv[46],sys.argv[50]+"_xgb.fmap"))
	#impfdf = sorted(impfdf.items(), key=operator.itemgetter(0))
	impfdf = pd.DataFrame(impfdf.items(), columns=['feature', 'nsplits'])
	impfdf['weight'] = impfdf['nsplits'] / impfdf['nsplits'].sum()
	#print(impfdf.head())

	## Join data frame
	impfdf= impfdf.join(impgain['gain'])
	impfdf= impfdf.join(impcover['cover'])

	## Sort by gain
	impfdf = impfdf.sort_values(by='gain', ascending=False)
	impfdf = pd.DataFrame(impfdf, columns=['feature', 'nsplits', 'weight', 'gain', 'cover'])
	#print(impfdf.head())

	## Save importance
	impfdf.to_csv(os.path.join(sys.argv[46], sys.argv[50]+"_xgb_imp.csv"), index = None, header=True) 

### Save trained model
model.save_model(os.path.join(sys.argv[46],sys.argv[50]+"_xgb_sm.model"))

### Dump model
model.dump_model(os.path.join(sys.argv[46],sys.argv[50]+"_xgb_dm.txt"))

### Pickle and dump model
pickle.dump(model, open(os.path.join(sys.argv[46],sys.argv[50]+"_xgb_pm.dat"), "wb"))

### Create data frame with all samples again for prediction when there are missing values of target variables
if (YMissFlag):	
	### Weigtht values
	if (WVar != "None"):
		WValues = raw_dtrain[WVar].values
	dtrain = xgb.DMatrix(raw_dtrain[PredVars].values, feature_names= PredVars, missing=np.nan, weight= WValues)
	#dtrain = xgb.DMatrix(raw_dtrain[PredVars].values, feature_names= PredVars, missing=np.nan)	

### Make predictions for the training data
predtrain = model.predict(dtrain)
#print(np.shape(predtrain))
#print(predtrain[0:5,])
if (predtrain.ndim == 2):
	colnames = ["pred"+str(i) for i in range(1,np.shape(predtrain)[1]+1)]
else:
	colnames = ["pred1"]

if (sys.argv[48] == "None"):
	### Save predictions from training data in a CSV file 
	np.savetxt(os.path.join(sys.argv[46], sys.argv[47]+".csv"), predtrain, header= ",".join(colnames), comments="", delimiter=",", fmt='%f')
	print("Testing data absent")
else:
	### Test file
	print("Testing data present")
	
	##Load CSV test file
	dtest = pd.read_csv(sys.argv[48])
	
	##Define test data
	#dtest = xgb.DMatrix(dtest[PredVars].values, feature_names= PredVars, missing=np.nan, weight= dtest[WVar].values)
	dtest = xgb.DMatrix(dtest[PredVars].values, feature_names= PredVars, missing=np.nan)
	#print(dtest.feature_names)

	## Load saved model
	#loaded_model = xgb.Booster()
	#loaded_model.load_model(os.path.join(sys.argv[46],sys.argv[50]+"_xgb_sm.model")
	
	"""
	## Load Pickle model from file
	loaded_model = pickle.load(open(os.path.join(sys.argv[46],sys.argv[50]+"_xgb_pm.dat"), "rb"))
	print(loaded_model.feature_names)
	if (param["booster"] == "gbtree" or param["booster"] == "dart"):
		print("Importance Test: ")
		print(loaded_model.get_fscore())
		
	## Save predictions from training data in a CSV file 
	predtrainpickle= loaded_model.predict(dtrain)
	np.savetxt(os.path.join(sys.argv[46], sys.argv[47]+"_xgb_pickle.csv"), predtrainpickle, header= ",".join(colnames), comments="", delimiter=",", fmt='%f')	
	"""
	
	if (param['booster'] == "dart"): 
		## Predictions with loaded model
		#predtest= loaded_model.predict(dtest, ntree_limit=num_round)
		predtest= model.predict(dtest, ntree_limit=num_round)
	else:
		## Predictions with loaded model
		#predtest= loaded_model.predict(dtest)
		predtest= model.predict(dtest)	
	
	## Save predictions from testing data in a CSV file 
	np.savetxt(os.path.join(sys.argv[46], sys.argv[49]+".csv"), predtest, header= ",".join(colnames), comments="", delimiter=",", fmt='%f')

