/*
JMP Life Sciences Analytical Process

Name: LearningCurveModelComparison
Author: Russ Wolfinger, SAS Institute Inc.

Description: 
	This process compares a set of models using learning curves.
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath SettingsFilter Workflow
LC_InnerNum LC_NMin LC_NMax LC_OuterNum LC_InnerLoop LC_TestSet LC_FixedTestSet LC_GroupName LC_Seed
CVMethod CV_GroupName CV_BlockName CV_HOSize CV_KFold CV_LOut CV_HOPct CV_HONum CV_HOStart CV_HOEnd CV_Seed 
Priors CustomPriors CustomCosts AsymmetricLossEvalProp 
CV_UseGrid CV_GMax CV_ListAll CV_OutName LCMCScript OutPath MacroPath;

*ProcessBody;

%macro LearningCurveModelComparison;

* initialization and argument checking;
%let ProcessName = LearningCurveModelComparison;
%let FullAPName = %str(Learning Curve Model Comparison);

%include "&MacroPath./UtilityMacros.sas" / nosource;

* S1566147: Creatin two JLS files, a setup and a run file. The run file is immediately deleted after running. 
* create JSL setup file;
%global LCMCSetupScript;
%PrepOutFile(LCMCSetupScript,LCMCSetupScriptName,&ProcessName Setup Script.jsl);
/*
%global _mainjslfile_;
%let _mainjslfile_ = &LCMCSetupScript;
*/

data _null_;
	file "&LCMCSetupScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "include(ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/Model Comparison Functions.jsl"");";
	put "fullapname = ""&FullAPName"";";	
	put "apname = ""&ProcessName"";";
	put "inpath = ""&InPath"";";
	put "workflow = words(""&Workflow"",""|"");";
	put "outpath = ""&ClientOutPath"";";
	put "lc_innernum = ""&LC_InnerNum"";";
	put "lc_nmin = ""&LC_NMin"";";
	put "lc_nmax = ""&LC_NMax"";";
	put "lc_outernum = ""&LC_OuterNum"";";
	put "lc_innerloop = ""&LC_InnerLoop"";";
	put "lc_testset = ""&LC_TestSet"";";
	put "lc_fixedtestset = ""&LC_FixedTestSet"";";
	put "lc_groupname = ""&LC_GroupName"";";
	put "lc_seed = ""&LC_Seed"";";
	put "cvmethod = ""&CVMethod"";";
	put "cv_groupname = ""&CV_GroupName"";";
	put "cv_blockname = ""&CV_BlockName"";";
	put "cv_hosize = ""&CV_HOSize"";";
	put "cv_kfold = ""&CV_KFold"";";
	put "cv_lout = ""&CV_LOut"";";
	put "cv_hopct = ""&CV_HOPct"";";
	put "cv_honum = ""&CV_HONum"";";
	put "cv_seed = ""&CV_Seed"";";
	put "cv_usegrid = ""&CV_UseGrid"";";
	put "cv_gmax = ""&CV_GMax"";";
	put "cv_hostart = ""&CV_HOStart"";";
	put "cv_hoend = ""&CV_HOEnd"";";
	put "priors = ""&Priors"";";
	put "custompriors = ""&CustomPriors"";";
	put "customcosts = ""&CustomCosts"";";
	put "asymmetriclossevalprop = ""&AsymmetricLossEvalProp"";";
	put "cv_listall = ""&CV_ListAll"";";
	put "cv_outname = ""&CV_OutName"";";
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &LCMCSetupScriptName, , text/plain, yes, no);

* create JSL run file;
%PrepOutFile(LCMCScript,LCMCScriptName,&ProcessName Script.jsl);
%global _mainjslfile_;
%let _mainjslfile_ = &LCMCScript;

data _null_;
	file "&LCMCScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "include(""&ClientOutPath.&LCMCSetupScriptName"");";
	put "_RunLCMC;"; 
	put "Delete File(""&ClientOutPath.&LCMCScriptName"");";
	put "// throw to prevent blank tabbed report";
	put "throw();";	
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &LCMCScriptName, , text/plain, yes, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend LearningCurveModelComparison;

%LearningCurveModelComparison;
