/*
JMP Life Sciences Analytical Process

Name: TestSetModelComparison
Author: Russ Wolfinger, SAS Institute Inc.

Description: 
	This process compares a set of models using cross validation.
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath SettingsFilter Workflow TS_InPath FileFilter TS_Names
Priors CustomPriors CustomCosts AsymmetricLossEvalProp 
TS_ChartBy TS_ChartByTestSet TS_ChartByModel TS_ListAll TS_OutName TSMCScript OutPath MacroPath;

*ProcessBody;

%macro TestSetModelComparison;

* initialization and argument checking;
%let ProcessName = TestSetModelComparison;
%let FullAPName = %str(Test Set Model Comparison);

%include "&MacroPath./UtilityMacros.sas" / nosource;

%let TS_ChartByTestSet= No;
%let TS_ChartByModel= No;
%if %index(%quote(&TS_ChartBy),ByTestSet) %then %let TS_ChartByTestSet= Yes;
%if %index(%quote(&TS_ChartBy),ByModel) %then %let TS_ChartByModel= Yes;

* S1566147: Creatin two JLS files, a setup and a run file. The run file is immediately deleted after running. 
* create JSL setup file;
%global TSMCSetupScript;
%PrepOutFile(TSMCSetupScript,TSMCSetupScriptName,&ProcessName Setup Script.jsl);
/*
%global _mainjslfile_;
%let _mainjslfile_ = &TSMCSetupScript;
*/

data _null_;
	file "&TSMCSetupScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "include(ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/Model Comparison Functions.jsl"");";
	put "fullapname = ""&FullAPName"";";	
	put "apname = ""&ProcessName"";";
	put "inpath = ""&InPath"";";
	put "workflow = words(""&Workflow"",""|"");";
	put "testinpath = ""&TS_InPath"";";
	put "testindatas = words(""&TS_Names"",""|"");";
	put "outpath = ""&ClientOutPath"";";
	put "priors = ""&Priors"";";
	put "custompriors = ""&CustomPriors"";";
	put "customcosts = ""&CustomCosts"";";
	put "asymmetriclossevalprop = ""&AsymmetricLossEvalProp"";";
	put "chartbytestset = ""&TS_ChartByTestSet"";";
	put "chartbymodel = ""&TS_ChartByModel"";";	
	put "listall = ""&TS_ListAll"";";
	put "outname = ""&TS_OutName"";";
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &TSMCSetupScriptName, , text/plain, yes, no);


* create JSL run file;
%PrepOutFile(TSMCScript,TSMCScriptName,&ProcessName Script.jsl);
%global _mainjslfile_;
%let _mainjslfile_ = &TSMCScript;

data _null_;
	file "&TSMCScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "include(""&ClientOutPath.&TSMCSetupScriptName"");";
	put "_RunTSMC;"; 
	put "Delete File(""&ClientOutPath.&TSMCScriptName"");";
	put "// throw to prevent blank tabbed report";
	put "throw();";	
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &TSMCScriptName, , text/plain, yes, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);


%mend TestSetModelComparison;

%TestSetModelComparison;
