/*
JMP Genomics Analytical Process

Name: XGBoost Regression
Author: Luciano Silva, SAS Institute Inc.
History:
	sbrlsi 01April19 initial coding
*/

/*
-----------------------------------------------------------
Copyright (C) 2019 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData ClassVar PredictorVars ClassPredVars WeightVar LabelVar 
ListContPredVars ListClassPredVars LockContPredVars LockClassPredVars ListLockContPredVars ListLockClassPredVars OutPath ServerOutPath
PredWhere PredClassWhere DataStepStatements TransformationMethod 
StandardizationMethod NumArg ImputeZeros KMeans KM_Max KM_CorrRadius KM_Usage 
StatTest ST_Max ST_Method Nominalize ST_AsymmetricLossFitProp ST_QuantileLevel MultipleTestingMethod NSample NegLog10pCutoff AbsMeanDiffCutoff AbsPropDiffCutoff
RG_SelectMethod RG_Adaptive RG_StopCriterion RG_SLEntry RG_SLStay 
RG_CVPMethod RG_CVKFold RG_Seed RG_MaxStep RG_OrderInt RG_Poolwise RG_PoolSize RG_PoolMax 
RG_ModelAveraging RG_MA_NSamples RG_MA_Max RG_MaxRounds
FR_ImpVars FR_Inds FR_NImpVars FR_MaxDepth FR_LeafSize FR_Seed FR_MaxTrees FR_NInputs
Optimize Opt_Similarity Opt_Criterion Opt_Algorithm StandardizeRows 
TraitType 

Booster Verbosity DisableDefEvalMet NThread NumBoostRound Objective BaseScore EvalMetric Seed
EtaReg GammaReg MaxDepth MinChildWeight MaxDeltaStep SubSample ColSampleByTree
ColSampleByLevel ColSampleByNode LambdaTree AlphaTree TreeMethodReg SketchEps 
ScalePosWeight UpdaterTree RefreshLeaf ProcessType GrowPolicy MaxLeaves MaxBin 
Predictor NumParallelTree SampleType NormalizeType RateDrop OneDrop SkipDrop
LambdaLinear AlphaLinear UpdaterLinear FeatureSelector TopK TweedieVarPower
NumRound

/*EventTrait AsymmetricLossFitProp NPred HetVar Priors VarSelect OrderInt Kernel BLUPWhere
NumVar MaxIter VarFreq StopVal Seed BJ_Est*/
CensorVar /*CensorValues CensorWeight BJ_Maxiter BJ_RMSE_Tol*/ 
TestInData /*ProcOption ClassVariables FixedEffects ModOption ProcMixedStatements*/ 
AsymmetricLossEvalProp CustomCosts XGBoostReg_Script XGBoostReg_OutData 
XGBoostReg_OutDataVars MacroPath;

*ProcessBody;

%macro XGBoostReg;

	%*put TraitType= &TraitType;

	*** initialization and argument checking;
	%let ProcessName = XGBoostReg;
	%let FullAPName = %str(XGBoost Regression); 
	
	****;
	%global DepVar Mode VarSelect Priors;
	%let DepVar = &ClassVar;
	%let Mode = Automated;
	%let VarSelect = ;
	%let Priors= ;	
	
	%include "&MacroPath./UtilityMacros.sas" / nosource;

	%GetOrigFileName(InData,OrigInData);
	
	%*put XGBoostReg_OutData= &XGBoostReg_OutData;
	%if %length(&XGBoostReg_OutData) %then %do;
		%let prefix = &XGBoostReg_OutData;
		%let XGBoostReg_OutData = ;
	%end;
	%else %let prefix= &OrigInData;
	%if %length(&prefix) > 24 %then %let prefix = %substr(&prefix,1,24);
	%put prefix= &prefix;	
	
	
	*** XGBooster not available in the dialog are set to None 
	*** and in the XGBoostReg.py they are set to default 
	*** by skiping them in the definition param;
  	%global BoosterPar;
  	%if %length(&Booster) = 0 %then %let BoosterPar= gbtree;
  	%else %let BoosterPar= &Booster;
  	
  	*not declared in the xml file (setting to default)*;
  	%global VerbosityPar; 
  	%if %length(&Verbosity) = 0 %then %let VerbosityPar= 1;
  	%else %let VerbosityPar= &Verbosity;
  	
  	*dependency = none, and needs re-parametrization  (0 = all threads = none)*;
  	%global NThreadPar;
	%if (%length(&NThread) = 0 or &NThread = 0) %then %let NThreadPar= None;
	%else %let NThreadPar= &NThread;

	*not declared in the xml file (setting to default)*;
  	%global DisableDefEvalMetPar; 
  	%if %length(&DisableDefEvalMet) = 0 %then %let DisableDefEvalMetPar= 0;
  	%else %let DisableDefEvalMet= &DisableDefEvalMet;

  	%global ObjectivePar;
  	%if %length(&Objective) = 0 %then %let ObjectivePar= %str(reg:squarederror);
  	%else %let ObjectivePar= &Objective;	

	%global BaseScorePar;
	%if %length(&BaseScore) = 0 %then %let BaseScorePar= 0.5;
	%else %let BaseScorePar= &BaseScore;	
  	
  	*not declared in the xml file (setting to default);
  	%global EvalMetricPar EvalMetricLab EvalMetricPlotY; 
  	/*
  	%if %length(&EvalMetric) = 0 %then %do;
  		*depends on the objective*;
  		%if &ObjectivePar = %str(reg:squarederror) %then %do;
  			%let EvalMetricPar= %str(rmse,logloss);
  			%let EvalMetricPlotY= &EvalMetricPar;
  			%let EvalMetricLab= %str(Root mean square error,Negative log-likelihood);
  			%let TraitType= Continuous;
  		%end;
  		%if &ObjectivePar = %str(reg:logistic) or 
  		    &ObjectivePar = %str(binary:logistic) or 
  		    &ObjectivePar = %str(binary:logitraw) or 
  		    &ObjectivePar = %str(binary:hinge) %then %do;
  			%let EvalMetricPar= %str(error,auc);
  			%let EvalMetricPlotY= &EvalMetricPar;
  			%let EvalMetricLab= %str(Classification error rate,Area under the curve);
  			%let TraitType= Binary;	
  		%end;
  		%if &ObjectivePar = %str(count:poisson) %then %do;
  			%let EvalMetricPar= poisson-nloglik;
  			%let EvalMetricPlotY= poisson_nloglik;
  			%let EvalMetricLab= %str(Negative log-likelihood for Poisson regression);
  			%let TraitType= Continuous;
  		%end;
  		%if &ObjectivePar = %str(survival:cox) %then %do;
  			%let EvalMetricPar= cox-nloglik;
  			%let EvalMetricPlotY= cox_nloglik;
  			%let EvalMetricLab= %str(Negative partial log-likelihood for Cox proportional hazards regression);
  			%let TraitType = Continuous;
  		%end;
  		%if &ObjectivePar = %str(multi:softmax) or 
  		    &ObjectivePar = %str(multi:softprob) %then %do;
  			%let EvalMetricPar= %str(merror,mlogloss);
  			%let EvalMetricPlotY= &EvalMetricPar;
  			%let EvalMetricLab= %str(Multiclass classification error rate,Multiclass logloss);
  			%let TraitType= Nominal;
  		%end;
  		%if &ObjectivePar = %str(rank:pairwise) or 
  		    &ObjectivePar = %str(rank:ndcg) or 
  		    &ObjectivePar = %str(rank:map) %then %do;
  			%let EvalMetricPar= map;
  			%let EvalMetricPlotY= &EvalMetricPar;
  			%let EvalMetricLab= %str(Mean average precision);
  			%let TraitType = Continuous;
  		%end;
  		%if &ObjectivePar = %str(reg:gamma) %then %do;
  			%let EvalMetricPar= %str(gamma-nloglik,gamma-deviance);
  			%let EvalMetricPlotY= %str(gamma_nloglik,gamma_deviance);
  			%let EvalMetricLab= %str(Negative log-likelihood for gamma regression,Residual deviance for gamma regression);
  			%let TraitType= Continuous;
  		%end;
  		%if &ObjectivePar = %str(reg:tweedie) %then %do;
  			%let EvalMetricPar= tweedie-nloglik;
  			%let EvalMetricPlotY= tweedie_nloglik;;
  			%let EvalMetricLab= %str(Negative log-likelihood for Tweedie regression);
  			%let TraitType= Continuous;
  		%end;
  	%end;
  	%else %let EvalMetricPar= &EvalMetric;
  	*/
  	
  	%global SeedPar;
  	%if %length(&Seed) = 0 %then %let SeedPar= 923748;
  	%else %let SeedPar= &Seed;
  	
  	%global EtaRegPar;
  	%if %length(&EtaReg) = 0 %then %let EtaRegPar= 0.3;
  	%else %let EtaRegPar= &EtaReg;  	
  	
  	%global GammaRegPar;
  	%if %length(&GammaReg) = 0 %then %let GammaRegPar= 0.0;
  	%else %let GammaRegPar= &GammaReg;
  	
  	%global MaxDepthPar;
  	%if %length(&MaxDepth) = 0 %then %let MaxDepthPar= 6;
  	%else %let MaxDepthPar= &MaxDepth;
  	
  	%global MinChildWeightPar;
  	%if %length(&MinChildWeight) = 0 %then %let MinChildWeightPar= 1;
  	%else %let MinChildWeightPar= &MinChildWeight;
  	
  	%global MaxDeltaStepPar;
  	%if %length(&MaxDeltaStep) = 0 %then %let MaxDeltaStepPar= 0;
  	%else %let MaxDeltaStepPar= &MaxDeltaStep;
  	
  	%global SubSamplePar;
  	%if %length(&SubSample) = 0 %then %let SubSamplePar= 1;
  	%else %let SubSamplePar= &SubSample;  	
  	
  	%global ColSampleByTreePar;
  	%if %length(&ColSampleByTree) = 0 %then %let ColSampleByTreePar= 1;
  	%else %let ColSampleByTreePar= &ColSampleByTree;
  	
  	%global ColSampleByLevelPar;
  	%if %length(&ColSampleByLevel) = 0 %then %let ColSampleByLevelPar= 1;
  	%else %let ColSampleByLevelPar= &ColSampleByLevel;
  	
  	%global ColSampleByNodePar;
  	%if %length(&ColSampleByNode) = 0 %then %let ColSampleByNodePar= 1;
  	%else %let ColSampleByNodePar= &ColSampleByNode;
  	
   	%if &BoosterPar= gblinear %then %let LambdaTree= &LambdaLinear;
  	
  	%global LambdaTreePar;
  	%if %length(&LambdaTree) = 0 %then %let LambdaTreePar= 1;
  	%else %let LambdaTreePar= &LambdaTree;  	

  	%global AlphaTreePar;
  	%if %length(&AlphaTree) = 0 %then %let AlphaTreePar= 0;
  	%else %let AlphaTreePar= &AlphaTree;

  	%global TreeMethodRegPar;
  	%if %length(&TreeMethodReg) = 0 %then %let TreeMethodRegPar= auto;
  	%else %let TreeMethodRegPar= &TreeMethodReg;

  	%global SketchEpsPar;
  	%if %length(&SketchEps) = 0 %then %let SketchEpsPar= 0.03;
  	%else %let SketchEpsPar= &SketchEps;
  	
  	%global ScalePosWeightPar;
  	%if %length(&ScalePosWeight) = 0 %then %let ScalePosWeightPar= 1;
  	%else %let ScalePosWeightPar= &ScalePosWeight;  	
 
	*not declared in the xml file (setting to default);
	%global UpdaterTreePar; 
  	%if %length(&UpdaterTree) = 0 %then %let UpdaterTreePar= None /*%str(grow_colmaker,prune)*/; 
  	%else %let UpdaterTreePar= &UpdaterTree;
  	
  	%global RefreshLeafPar;
  	%if %length(&RefreshLeaf) = 0 %then %let RefreshLeafPar= 1;
  	%else %let RefreshLeafPar= &RefreshLeaf;  	
  	
  	*not declared in the xml file (setting to default);
  	%global ProcessTypePar; 
 	%if %length(&ProcessType) = 0 %then %let ProcessTypePar= default;
 	%else %let ProcessTypePar= &ProcessType;  	
  	
  	%global GrowPolicyPar;
  	%if %length(&GrowPolicy) = 0 %then %let GrowPolicyPar= depthwise;
  	%else %let GrowPolicyPar= &GrowPolicy;  	
  	
  	*dependency = none*;
  	%global MaxLeavesPar;
  	%if %length(&MaxLeaves) = 0 %then %let MaxLeavesPar= None;
  	%else %let MaxLeavesPar= &MaxLeaves;  	
  	
  	*dependency = none*;
  	%global MaxBinPar;
  	%if %length(&MaxBin) = 0 %then %let MaxBinPar= None;
  	%else %let MaxBinPar= &MaxBin;  	
  	
  	%global PredictorPar;
  	%if %length(&Predictor) = 0 %then %let PredictorPar= cpu_predictor;
  	%else %let PredictorPar= &Predictor;  	
  	
  	%global NumParallelTreePar;
  	%if %length(&NumParallelTree) = 0 %then %let NumParallelTreePar= 1;
  	%else %let NumParallelTreePar= &NumParallelTree;  	
  	
  	*dependency = none*;
  	%global SampleTypePar;
  	%if %length(&SampleType) = 0 %then %let SampleTypePar= None;
  	%else %let SampleTypePar= &SampleType;  	
  	
  	*dependency = none*;
  	%global NormalizeTypePar;
  	%if %length(&NormalizeType) = 0 %then %let NormalizeTypePar= None;
  	%else %let NormalizeTypePar= &NormalizeType;  	
  	
  	*dependency = none*;
  	%global RateDropPar;
  	%if %length(&RateDrop) = 0 %then %let RateDropPar= None;
  	%else %let RateDropPar= &RateDrop;  	
  	
  	*dependency = none, needs re-parametrization (No=0, Yes=1)*;
   	%global OneDropPar; 
 	%if %length(&OneDrop) = 0 %then %let OneDropPar= None;
	%else %if &OneDrop = No %then %let OneDropPar= 0;	
	%else %let OneDropPar= 1; 	
  	
  	*dependency = none*;
  	%global SkipDropPar;
  	%if %length(&SkipDrop) = 0 %then %let SkipDropPar= None;
  	%else %let SkipDropPar= &SkipDrop;  	
  	
  	*dependency = none*;
  	%global LambdaLinearPar;
  	%if %length(&LambdaLinear) = 0 %then %let LambdaLinearPar= None;
  	%else %let LambdaLinearPar= &LambdaLinear;  	
  	
  	*dependency = none*;
  	%global AlphaLinearPar;
  	%if %length(&AlphaLinear) = 0 %then %let AlphaLinearPar= None;
  	%else %let AlphaLinearPar= &AlphaLinear;  	
  	
  	*dependency = none*;
  	%global UpdaterLinearPar;
  	%if %length(&UpdaterLinear) = 0 %then %let UpdaterLinearPar= None;
  	%else %let UpdaterLinearPar= &UpdaterLinear;  	
  	
  	*dependency = none*;
  	%global FeatureSelectorPar;
  	%if %length(&FeatureSelector) = 0 %then %let FeatureSelectorPar= None;
  	%else %let FeatureSelectorPar= &FeatureSelector;  	
  	
  	*dependency = none*;
  	%global TopKPar;
  	%if %length(&TopK) = 0 %then %let TopKPar= None;
  	%else %let TopKPar= &TopK;  	
  	
  	*dependency = none*;
  	%global TweedieVarPowerPar;
  	%if %length(&TweedieVarPower) = 0 %then %let TweedieVarPowerPar= None;
  	%else %let TweedieVarPowerPar= &TweedieVarPower;  	
  	
  	*dependency = none*;
  	%global NumBoostRoundPar;
  	%if %length(&NumBoostRound) = 0 %then %let NumBoostRoundPar= None;
  	%else %let NumBoostRoundPar= &NumBoostRound;
  	

	*** JAVA LIBRARIES/CLASS FILES LOCATION;
	%*put LS_HOME= &LS_HOME;
	%global JavaClassPath; 
	%let JavaClassPath= &LS_HOME.%str(JavaLib\Class);
        %*put JavaClassPath= &JavaClassPath;
        
        *** SET CLASSPATH;
        /*proc javainfo all; run;*/
        Options Set=CLASSPATH "&JavaClassPath";
	
	*** VALIDATE JAVA CLASSPATH; 
	%let FlagJavaPathError= 0;
	data _null_; 
		length _x1 $32767; 
		_x1 = sysget("CLASSPATH");  
		_x3 = %upcase("&JavaClassPath");
		*_x4 = sysget('PATH'); 
		_x2 = index(upcase(trim(_x1)), %upcase("&JavaClassPath")); 
		if _x2 = 0 then do;
			call symputx("FlagJavaPathError",_x2+1);
		end;
	run; 
	
	%if &FlagJavaPathError ^= 0 %then %do;
		%put ERROR: Invalid Java CLASSPATH.;
		%let exiterror = 1;
		%goto XGBRegExit;
	%end;	
	
	*** Test Python connection ***;
	%let FlagPythonConnectError= 0;
	data _null_;
		length rtn_val 8;
		declare javaobj j("dev.SASJavaExec", "python", "--version");
		j.callIntMethod("executeProcess", rtn_val);
		call symputx("FlagPythonConnectError",rtn_val);
	run;	

	%if &FlagPythonConnectError ^= 0 %then %do;
		%put ERROR: Python connection failed. Please, check if Python software is installed.;
		%let exiterror = 1;
		%goto XGBRegExit;
	%end;	
	
	%*let exiterror= 1;
	%*goto XGBRegExit;

	%PrepOutDS(XGBoostReg_OutData,XGBoostReg_OutDataName,&prefix,_xgb);
	%PrepOutDS(XGBoostReg_OutDataVars,XGBoostReg_OutDataVarsName,&prefix,_xgbv);
	%PrepOutFile(XGBoostReg_Script,XGBoostReg_ScriptName,&ProcessName Script for &prefix..jsl);		
	
	/*
	%if %symexist(CVing) %then %let CVFlag = 1;
	%else %let CVFlag = 0;
	%put CVFlag= &CVFlag;
	
	%PathName(&InData); 
	%let trainpath=&TmpPath;
	%let InDataTmp = %trim(&TmpName);
	libname InLibTmp "&TmpPath";
	
	%global VType;
	%put InData= &InData;
	%put ClassVar= &ClassVar;
	%VarAttributes(_Ds=InLibTmp.&InDataTmp,_Var=&ClassVar,_Len=,_Type=VType);
	%put VType= &VType;
	%if &VType ^= N %then %do;
		%put ERROR: Dependent variable (&ClassVar) is not a numeric variable.;
		%let exiterror=1;
		%goto XGBRegExit;
	%end;			

	%if &KMeans= Yes %then %do;
		%let KM_Usage= Select One Representative Predictor from Each Cluster;
	%end;

	%if ^%length(&OrderInt) %then %let OrderInt = 1;
	%if ^%length(&NPred) %then %let NPred = 0;
	*/
		
	*** call PredictiveAnalysis macro;
	%include "&MacroPath./PredMacros.sas" / nosource;	

	%PredictiveAnalysis(
		XGBoostReg_OutData, XGBoostReg_OutDataName,
		XGBoostReg_OutDataVars, XGBoostReg_OutDataVarsName,
		XGBoostReg_Script, XGBoostReg_ScriptName
	);
	
	%XGBRegExit:;
	%if &exiterror %then %do;
		%global OutName;
		%let OutName = &prefix;
		%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
	%end;

%mend XGBoostReg;

%XGBoostReg;
