%macro compute_roc(data=, isevent=, pevent=, outdata=roc);

	 data sorted;
	 	set &data;
	 	if missing(&pevent) or missing(&isevent) then delete;
	 run;
	 
	 %NObsVars(sorted);
	 %if (&nobs = 0) %then %do;
	 	%put ROC Statistics are not computed because input data set has no nonmissing observations.;
	 	data &outdata;
	 		set sorted;
	 	run;
	 %end;
	 %else %do;
	 
		 proc sort data=sorted; 
			by &pevent ;
		 run;

		 * first pass to compute negatives and totals;
		 data sorted;
			retain _FALNEG_ _NEG_ n_events n_not 0;

			label _NEG_    = 'True Negatives';
			label _FALNEG_ = 'False Negatives';

			set sorted end=the_end ;

			if &isevent then do;
				n_events = n_events + 1;
				_FALNEG_ = _FALNEG_ + 1;
			end;
			else do;
				n_not = n_not + 1;
				_NEG_ = _NEG_ + 1;
			end;

			if the_end ne 0 then do;
				call symput('n_events', trim(left(n_events)));
				call symput('n_not', trim(left(n_not)));
				call symput('n_total',trim(left(_n_)));
			end;
		 run;

		 %put n_events = &n_events;
		 %put n_not = &n_not;
		 %put n_total = &n_total;

		 * second pass, each value is computed assuming the cutoff equals pevent at the current observation;
		 data &outdata;
			retain maxacc 0;
			label _POS_    = 'True Positives';
			label _FALPOS_ = 'False Positives';
			label _SENSIT_ = 'Sensitivity';
			label _SPEC_   = 'Specificity';
			label _1MSPEC_ = '1 - Specificity';
			label _ACC_    = 'Total Accuracy';
			label _PPV_    = 'Positive Predictive Value';
			label _NPV_    = 'Negative Predictive Value';
			label _MCC_    = 'Matthews Correlation Coefficient';
			label _J_      = 'Youden Index';
			label _01_     = 'Zero-One Index';

			set sorted;
			_POS_    = &n_events - _FALNEG_;
			_FALPOS_ = &n_not - _NEG_;
			_SENSIT_ = _POS_ / &n_events;
			_1MSPEC_ = _FALPOS_ / &n_not;
			_SPEC_   = 1 - _1MSPEC_;
			_ACC_    = (_POS_ + _NEG_)/(&n_events + &n_not);
			if (_ACC_ > maxacc) then do;
				maxacc = _ACC_;
				call symput('maxacc_n_',trim(left(_n_)));
			end;
			_PPV_    = _POS_ / (_POS_ + _FALPOS_);
			_NPV_    = _NEG_ / (_NEG_ + _FALNEG_);
			_J_      = _SENSIT_ + _SPEC_ - 1;
			_01_     = 1 - sqrt((1-_SENSIT_)**2 + (1-_SPEC_)**2);
			mccden = (_POS_+_FALNEG_)*(_POS_+_FALPOS_)*(_NEG_+_FALPOS_)*(_NEG_+_FALNEG_);
			*if (mccden > 0) then _MCC_ = ((_POS_*_NEG_)-(_FALPOS_*_FALNEG_))/sqrt(mccden);
			*else _MCC_ = .;
			if(_POS_ > 0 and (_FALPOS_ + _FALNEG_ + _NEG_) = 0) then do;
				_MCC_= 1;
			end;
			else if(_NEG_ > 0 and (_FALPOS_ + _FALNEG_ + _POS_) = 0) then do;
				_MCC_= 1;
			end;
			else if(_FALNEG_ > 0 and (_FALPOS_ + _POS_ + _NEG_) = 0) then do;
				_MCC_= -1;
			end;
			else if(_FALPOS_ > 0 and (_FALNEG_ + _POS_ + _NEG_) = 0) then do;
				_MCC_= -1;
			end;
			else if((((_POS_ + _FALPOS_) = 0) and (_FALNEG_ > 0 and _NEG_ > 0)) or 
				(((_FALNEG_ + _NEG_) = 0) and (_FALPOS_ > 0 and _POS_ > 0)) or 
				(((_POS_ + _FALNEG_) = 0) and (_NEG_ > 0 and _FALPOS_ > 0)) or 
				(((_FALPOS_ + _NEG_) = 0) and (_FALNEG_ > 0 and _POS_ > 0))) then do;
				_MCC_= 0;
			end;
			else do;
				nummcc= (_POS_ * _NEG_ - _FALPOS_ * _FALNEG_);
				denmcc= (sqrt((_POS_ + _FALPOS_)*(_POS_ + _FALNEG_) * (_NEG_ + _FALPOS_)*(_NEG_ + _FALNEG_)));
				_MCC_= ((round(nummcc / denmcc,0.0001)));
			end;			
			drop maxacc mccden nummcc denmcc;
		 run;

		 proc sort data=&outdata;
			by &pevent _SPEC_;
		 run;
	%end;

%mend;
