/*
JMP Genomics Analytical Process 

Name: ANOVA
Author: Russ Wolfinger, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
	sasrdw 25Oct04 initial coding from MixedModelAnalysis

Description: 
ANOVA fits a linear model to sequential
sets of observations of a pre-normalized intensity 
variable and creates numerous output displays. 
You can conduct similar and more advanced analyses
using MixedModelAnalysis. This process can be 
computationally intensive for large data sets.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/



*ProcessVariables;


%let InData=%nrbquote($HOME\JMPLS\WorkflowResults\affyinputengine_med.sas7bdat);
%let ByVars= Probe_Set_ID;
%let BaseMergeVars= Probe_Set_ID;
%let InClassVars= Probe;
%let OutPath = %nrbquote($HOME\JMPLS\WorkflowResults\);
%let MacroPath= $LS_HOME\MacroLib;
%let AnnoInData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Latin Square\u95a_trim.sas7bdat);
%let AnnoMergeVars= Probe_Set_ID;
%let AnnoLabel= Probe_Set_ID;
%let AnnoAccession= Sequence_Derived_From;
%let AnnoGeneSymbol= Gene_Symbol;
%let AnnoDescription= Title;
%let AnnoGeneId= LocusLink;
%let AnnoSnpId= ;
%let AnnoOrganism= Homo sapiens;
%let ExpInData=%nrbquote($HOME\JMPLS\WorkflowResults\affyinputengine_exp.sas7bdat);
%let ClassVars= Experiment Array;
%let DesignByVars= ;
%let SplitResults= ;
%let FixedEffects=%nrbquote(Experiment Probe);
%let LsmeansEffects=%nrbquote(Experiment);
%let EstInFile=%nrbquote();
%let RandomEffects=%nrbquote(Array);
%let LSMeansDiffSet= Differences with a Control;
%let LSMeansControlValues=%nrbquote("a");
%let LSMeanStdMethod= STD;
%let TStats= no;
%let MultipleTestingMethod= ;
%let Alpha= ;
%let NegLog10pCutoff= 15;
%let PlotResiduals= no;
%let OutDataRes= ;
%let AnnoLabResid= ;
%let FilterType= False Positive Rate;
%let ResidualFalsePositive= 0.05;
%let ResidualCutoff= ;
%let MaxLoop= 3;
%let VolcanoUnifiedY= yes;
%let Speak= no;
%let Unbounded= no;
%let KenwardRoger= no;
%let ChunkSize= 1000;
%let OutData=%nrbquote();
%let JSLFile=%nrbquote();
%let MMEIData=%nrbquote(affylatin_mmei);


*ProcessBody;

%include "$GENOMICS_HOME\ProcessLibrary\ANOVA.sas"; 