/**
SAS macros for JMP Genomics Genetics Analytical Processes
**/

%macro CheckGenotypeFmt;
	%if %index(%qupcase(&MarkerFmt),NUMERIC GENOTYPES)>0 %then %do;
		data chkgeno; set Inlib.&Indata(keep=&AlleleVars obs = 1);
		run;
		
		%let dsid = %sysfunc(open(chkgeno));
		%let GType = %sysfunc(vartype(&dsid,1));
		%let dsid = %sysfunc(close(&dsid));
		%if &GType = C %then %do;
			%put ERROR: The SNP variables in the Input SAS Data Set are character format.  The Format of SNP Variables on the Options tab cannot be set to Numeric Genotypes.;
			%let exiterror =1;
		%end;
	%end;
%mend;

%macro GenInit(name,maxnamelength);

*initialization and argument checking;
%global ProcessName;
%let ProcessName = &name; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
options nolabel;

%GetOrigFileName(InData,OrigInData);

%PathName(&InData);
%let InData = &TmpName;
libname InLib "&TmpPath";

libname OutLib "&OutPath";

* check name for OutName;
%CheckDataName(_name=OutName,_default=&OrigInData);
%if %length(&OutName)>&maxnamelength %then %let OutName=%substr(&OutName,1,&maxnamelength);

%global AnnoMergeVars AnnoGeneSymbol AnnoDescription AnnoGeneId AnnoSnpId AnnoOrganism;
%let AnnoMergeVars=&AnnoLabel;
%let AnnoGeneSymbol=;
%let AnnoDescription=;
%let AnnoGeneId=;
%let AnnoOrganism=;
%let AnnoSnpId = &AnnoAccession;
%CheckAnno(_checkindata=0);

/* GENOCOL option for SAS/Genetics procs */
%global GenoCol;
%let GenoCol = ;

/* Need to accommodate possible NUMERIC GENOTYPES for SNP-Trait Association */
%if %index(%qupcase(&MarkerFmt),GENOTYPES)>0 %then %do;

	%if %qupcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
		%let GenoDelim=%str(/);
 		proc format;
			value numtochar 0='A/A'
	  	            		2='B/B'
	  	            		1='A/B'
	  	            		other=''
			;
		run;
	%end;
	%let GenoCol = %str(genocol delimiter="&GenoDelim");
	
%end;

/* to use in pvalue plot macro */
%global GenAP;
%let GenAP=1;

%mend;

%macro PrepMapandBy;
/******************
Sorts data set by BY variables.
Creates a list of BY variables
without DESCENDING in list.
Counts number of BY groups.
*******************/

%global Nbyval NByVars ByVarsOnly Jby ds lastby annolen OrigNByVal NonTraitByVars JNonTraitByVars;

%if %length(&ByVars) %then %do;
	options label;
	proc sort data=InLib.&InData out=a;
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
		by &ByVars;
	run;
	options nolabel;
	
	%Nobsvars(a);
	%if &nobs=0 %then %do;
		%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
		%let exiterror=1;
		%goto exerr;
	%end;
	
	%let ds = a;
	
	%let temp=%sysfunc(tranwrd(&ByVars,DESCENDING, ));
	%let ByVarsOnly=%sysfunc(compbl(&temp)); 
	%let JBy=%sysfunc(translate(&ByVarsOnly,","," "));
	%let i=1;
	%do %while (%length(%scan(&ByVarsOnly, &i))>0);
		%let i=%eval(&i+1);
	%end;
	%let NByVars = %eval(&i-1);
	data countby;
		set &ds;
		by &ByVars;
		%let lastby=%scan(&ByVarsOnly, &NByVars);
		if first.&lastby;
	run;
	%let dsid=%sysfunc(open(countby));
	%let Nbyval=%sysfunc(attrn(&dsid,NOBS));
	%let ret=%sysfunc(close(&dsid));	
	%if &Nbyval=1 and &ProcessName ne %str(ImputedSNPTraitAssociation)%then %do;
		%let ByVars=;
		%let ByVarsOnly=;
		%let JBy=;
	%end;
%end;
%else %do;
 %let ds = InLib.&InData;
 %let Nbyval=1;
 %let ByVarsOnly=;
 %let NByVars = 0;
%end;

/* Need this for Venns */
%let OrigNByVal = &NByVal;
%if &ProcessName=%str(ImputedSNPTraitAssociation) %then %do;
	%let NonTraitByVars = &OldByVars;
	
	%if %length(&OldByVars) %then %do;
		proc sort data=&ds out=countby2;
			by &OldByVars;
		run;

		data countby2;
			set countby2;
			by &OldByVars;
			%let lastby=%scan(&OldbyVars, %eval(&NByVars-1));
			if first.&lastby;
		run;
		%let dsid=%sysfunc(open(countby2));
		%let OrigNbyval=%sysfunc(attrn(&dsid,NOBS));
		%let ret=%sysfunc(close(&dsid));		
	%end;
	%else %let OrigNByVal = 1;
%end;	
%else %let NonTraitByVars = &ByVars;

%let b=1;
%do %while(%length(%scan(&NonTraitByVars,&b)));
	%let JNonTraitByVars = &JNonTraitByVars :%scan(&NonTraitByVars,&b),;
	%let b=%eval(&b+1);
%end;

%if &ProcessName=%str(ImputedSNPTraitAssociation) %then %do;

	proc sort data=&ds out=snpids(keep=&SNPIDVar) nodupkey;
		by &SNPIDVar;
	run;
	
	%if %length(&AnnoInData) %then %do;
	
		proc sort data=AnnoLib.&AnnoData out=mapdata;
			by &AnnoLabel;
		run;
		
		%if %length(&AnnoWhereStmt)%then %do;
			data mapdata;
				set mapdata;
				_order_=_n_;
				if %unquote(&AnnoWhereStmt);
			run;
			
			%Nobsvars(mapdata);
			%if &syserr>4 or &nobs=0 %then %do;
				%put ERROR: You have entered an invalid Filter to Include Markers or no markers remained after filtering.;
				%let exiterror=1;
				%goto exerr;
			%end;
			
			data &ds;
				merge &ds(rename=(&SNPIDVar=&AnnoLabel)) mapdata(in=annofilter keep=&AnnoLabel);
				by &AnnoLabel;
				if annofilter;
				rename &AnnoLabel = &SNPIDVar;
			run;
		%end;
		
		/* Check if there any matched with anno, so can issue an error msg if none match in case wrong Label Var used */
		%let anyannomatch=0;
		data snpids;
			merge snpids(in=allsnps rename=(&SNPIDVar=&AnnoLabel)) mapdata(in=annofilter);
			by &AnnoLabel;
			%if %length(&AnnoWhereStmt) %then %do;
				if annofilter;
			%end;
			%else %do;
				if allsnps;
			%end;
			if allsnps and annofilter then call symputx("anyannomatch","1");
		run;
		
		%if &anyannomatch=0 %then %do;
			%put ERROR: There were no SNP IDs in the Annotation Data Set that matched the ones in the Input Data Set.  Check that you have selected variables with the same ID values.;
		%end;
		
	%end;
	%else %let AnnoLabel = &SNPIDVar;

	options errors=100000;
	
	data mapdata;
		set snpids;
		%if %length(&AnnoGroup)=0 %then %do;
			_group_=1;
			%let AnnoGroup=_group_;
			%if %length(&AnnoLoc)=0 %then %do;
				_Position_=_n_;
				%let AnnoLoc=_Position_;
			%end; 			
		%end;
		%if %length(&AnnoLoc) %then %do;
			n&AnnoLoc = &AnnoLoc + 0;
  			drop &AnnoLoc;
  			rename n&AnnoLoc = &AnnoLoc;
  		%end;		
		_order_=_n_;	
  		/* Make sure AnnoLabel is character */
  		&AnnoLabel.c = strip(&AnnoLabel||'');
  		drop &AnnoLabel;
  		rename &AnnoLabel.c=&AnnoLabel;	
  	run;
  	options errors=20;
 
	%if %length(&AnnoLoc)=0 and %quote(&AnnoGroup) ne %str(_group_) %then %do;
		proc sort data=mapdata;
			by &AnnoGroup;
	  	run;

	  	data mapdata;
			set mapdata;
			by &AnnoGroup;
			if first.&AnnoGroup then _Position_=1;
			else _Position_+1;
	  	run;
	  	proc sort data=mapdata;
			 by _order_;
	  	run;
	  	%let AnnoLoc=_Position_;
	%end; 	
	
	%global NumGps nummark;  	
	
   %let dsid=%sysfunc(open(mapdata));
   %let annolen=%sysfunc(varlen(&dsid,%sysfunc(varnum(&dsid,&AnnoLabel))));
 	%let annogptype=%sysfunc(vartype(&dsid,%sysfunc(varnum(&dsid,&AnnoGroup))));    
 	%let nummark=%sysfunc(attrn(&dsid,NOBS));
  	%let ret=%sysfunc(close(&dsid));
  	
  	%goto more;
	
%end;

options label; /* So if no annotation ds, can use labels of marker variables */
data count; 
 set &ds;
 keep &AlleleVars;
 if _n_=1 then output;
 else stop;
run;

/* Gets number of markers */
%let dsid=%sysfunc(open(count));
%global n2snps;
%let n2snps=%sysfunc(attrn(&dsid,NVARS));
%if %length(&GenoCol) %then %do;
 %let n2snps = %eval(2*&n2snps);
%end;
%let ret=%sysfunc(close(&dsid));
%global nummark;
%let nummark=%eval(&n2snps/2);

options nolabel;

%if &nummark=0 %then %do;
	%if %length(&GenoCol) %then %do;
		%put ERROR: One or more Marker Variables must be specified when a Genotype format is used.;
	%end;
	%else %do;
		%put ERROR: Two or more Marker Variables must be specified when the Allele format is used.;
	%end;
	%let exiterror=1;
%end;

%if %quote(&ProcessName)=HTSNPSelection and %length(&AnnoInData) %then %do;
   %let dsid=%sysfunc(open(AnnoLib.&AnnoData));
   %let nummark=%sysfunc(attrn(&dsid,NOBS));
   %let ret=%sysfunc(close(&dsid));
%end;

/***************************
Create MAP data set if
necessary, or MAP variables
that don't exist.
****************************/
%global NumGps;
%if %length(&AnnoGroup)=0 %then %let NumGps=1;
%else %let NumGps = ;

%if %symexist(AnnoMName)=0 %then %let AnnoMName=;
%if %qupcase(&MarkerFmt)=%str(ALLELES) %then %let AnnoMName=;

/*********************************************************************
	If AnnoMName is specified, check that the first 100 markers match in 
	input and annotation data sets - if not exit and launch Subset Reorder 
*********************************************************************/	
%if %length(&AnnoData) %then %do;
	%if %length(&AnnoMName) %then %do; 

		%let allmatch=1;
		data _null_;
			set AnnoLib.&AnnoData end=_l;
			retain md nv;
			if _n_=1 then do;
				md = open("count");
				nv = attrn(md,"nvars");
			end;
			if _n_ le nv then do;
				if upcase(strip(&AnnoMName)) ne upcase(varname(md,_n_)) then do;
					call symputx("allmatch",0);
					call symputx("snpmm",varname(md,_n_));
					call symputx("annomm",strip(&AnnoMName));
					call symputx("mmnum",strip(_n_));
					stop;
				end;
			end;
			if _n_=_l then rc=close(md);	
		run;
		
		%if &allmatch=0 %then %do;
			%put ERROR: The order of markers in the input data set does not match the annotation data set according to the variable selected as Marker Names Variable.;
			%put The first mis-match occurs at marker &mmnum, with marker column name &snpmm and Marker Names Variable &annomm in the annotation data set.;
			%put You must run the Subset and Reorder Genetic Data process to order these data sets correctly.;
			
			* create JSL file;
			%global SRButtonsScript;
			%let SRButtonsScript=;
			%PrepOutFile(SRButtonsScript,SRButtonsScriptName,launchAP_buttons_SR.jsl);
			filename jslbtns "&SRButtonsScript";

			options noquotelenmax;
			data _null_;
				file jslbtns lrecl=32767;
				put "//!";
				put "Names Default to Here(1);";
				put "wait(0.5);";
				put "if(host is(Windows),";
				put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
				put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
				put "	";
				put "srbuttonswindow=New Window(""Launch AP to Reorder Markers"",";
				put "		VListBox(";
				put "			TextBox(""Click on the button below to launch the Subset and Reorder Genetic Data process to reorder marker columns properly.""),";
				put "			TextBox(""Use the output data sets from that process as the new input and annotation data sets to .""),";

				put "			TextBox("" ""),";
				put "			ButtonBox(""Subset and Reorder Genetic Data"",";
				put "				if(host is(Windows),";
				put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
				put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
				put "	";

				/* Set input data set and Output Folder */
				put "				jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";				
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""%sysfunc(pathname(Inlib))&_slash_.&InData..sas7bdat"");";
				put "				jg:SubsetandReorderGeneticData_updateDeps;";
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&AlleleVars"");";	
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""Genotypes"");";
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),""0"");";		
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""%sysfunc(pathname(AnnoLib))&_slash_.&AnnoData..sas7bdat"");";					
				put "				jg:SubsetandReorderGeneticData_updateDeps;";				
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&AnnoMName"");";	
				put "			)";;

				put "		)";
				put ");";
			run;

			%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SRButtonsScriptName, , text/plain, yes);			
			
			%let exiterror=1;
			%goto exerr;
		%end;
					
	%end;				 
%end;
/****** end check with AnnoMName *******/

%if &NumGps=1 %then %do;
 %let AnnoGroup=_group_;
 
 %if %length(&AnnoInData) %then %do;
   data mapdata;
    set AnnoLib.&AnnoData(obs=&nummark);
    _group_=1;
    %if %length(&AnnoLoc)=0 %then %do;
     _Position_=_n_;
     %let AnnoLoc=_Position_;
    %end; 
    _order_=_n_;
    /* Make sure AnnoLabel is character */
    &AnnoLabel.c = strip(&AnnoLabel||'');
    drop &AnnoLabel;
    rename &AnnoLabel.c=&AnnoLabel;
   run;

   %let dsid=%sysfunc(open(mapdata));
   %let nmapobs=%sysfunc(attrn(&dsid,NOBS));
   %let annolen=%sysfunc(varlen(&dsid,%sysfunc(varnum(&dsid,&AnnoLabel))));

   %let ret=%sysfunc(close(&dsid));

   %if (&nmapobs<&nummark) %then %do;

   	options label;
     	data map2;
     		set count;
     		array _var_{*} &AlleleVars;
			retain maxl &annolen;
   		do i=%eval(&nmapobs+1) to &nummark;
    	  		%if %length(&GenoCol) %then %do;
     	    		&AnnoLabel=strip(vlabel(_var_{i}));  
    	  		%end;
    	  		%else %do;
        		    &AnnoLabel="Marker"||left(i);
    	  		%end;
    	  		maxl=max(maxl,length(&AnnoLabel));
    	  		&AnnoLoc=i;
    	  		_group_=1;
    	  		_order_=i;
    	  		output;
   		end;
   		call symput("AnnoLen",strip(maxl));
      	keep &AnnoLabel &AnnoLoc _group_ _order_;
		run;
		options nolabel;

		 data mapdata;
			length &AnnoLabel $&AnnoLen;
			set mapdata map2;
		 run;

	%end;
 %end;
 
 %else %do;

  %let AnnoLoc=_Position_;
  %let AnnoLabel=Marker;
  options label;
  
  /* Fix for S0835179 - using ma: for list-style spec of marker vars */
  %let countid=%sysfunc(open(count,i));
  %let countfv = %sysfunc(varname(&countid,1));
  %let ncv = %sysfunc(attrn(&countid,nvars));
  %let countlv = %sysfunc(varname(&countid,&ncv));
  %let rc = %sysfunc(close(&countid));
  
  data mapdata;
   length Marker $32;
   set count;
   array _var_{*} &countfv--&countlv;
   %if %length(&GenoCol)=0 %then Marker="Marker&nummark";;
   retain maxalen 0;
   do i=1 to &nummark;
    %if %length(&GenoCol) %then %do;
     Marker=strip(vlabel(_var_{i}));
     Marker_Name=strip(vname(_var_{i})); /*sbrlsi: including name of markers in the mapdata as well for the output*/
    %end;
    %else %do;
     Marker="Marker"||left(i);
    %end;
    _Position_=i;
    _group_=1;
    _order_=i;
    if lengthn(marker) > maxalen then maxalen = lengthn(marker);
    call symput("annolen",maxalen);
    output;
   end;
   /*sbrlsi: including Marker_Name in the keep statement for the output*/
   keep Marker %if %length(&GenoCol) %then Marker_Name; _Position_ _group_ _order_;
  run;
  options nolabel;

 %end;
%end;

%else %do;

 options errors=100000;
 data mapdata;
  set AnnoLib.&AnnoData(obs=&nummark);
  _order_=_n_;
  %if %length(&AnnoLoc) %then %do;
  	n&AnnoLoc = &AnnoLoc + 0;
  	drop &AnnoLoc;
  	rename n&AnnoLoc = &AnnoLoc;
  %end;
  /* Make sure AnnoLabel is character */
  &AnnoLabel.c = strip(&AnnoLabel||'');
  drop &AnnoLabel;
  rename &AnnoLabel.c=&AnnoLabel;  
 run; 
 options errors=20;
 
 %if %length(&AnnoLoc)=0 %then %do;
  proc sort data=mapdata;
   by &AnnoGroup;
  run;
 	
  data mapdata;
   set mapdata;
   by &AnnoGroup;
   if first.&AnnoGroup then _Position_=1;
   else _Position_+1;
  run;
  proc sort data=mapdata;
    by _order_;
  run;
  %let AnnoLoc=_Position_;
 %end;
 
 %let dsid=%sysfunc(open(mapdata));
 %let nmapobs=%sysfunc(attrn(&dsid,NOBS));
 %let annolen=%sysfunc(varlen(&dsid,%sysfunc(varnum(&dsid,&AnnoLabel))));
 %let annogptype=%sysfunc(vartype(&dsid,%sysfunc(varnum(&dsid,&AnnoGroup)))); 
 %let ret=%sysfunc(close(&dsid));

 %if (&nmapobs<&nummark) %then %do;
 		options label;
		data map2;
     		set count;
     		array _var_{*} &AlleleVars;
     		retain maxl &annolen;
    		do i=%eval(&nmapobs+1) to &nummark;
    	  		%if %length(&GenoCol) %then %do;
     	    		&AnnoLabel=strip(vlabel(_var_{i}));  
    	  		%end;
			  %else %do;
				 &AnnoLabel="Marker"||left(i);
			  %end;
			  maxl=max(maxl,length(&AnnoLabel));
			  &AnnoLoc=i;
			  %if %quote(&annogptype)=%str(C) %then %do; 
				&AnnoGroup="";
			  %end;
			  %else &AnnoGroup=.;;
			  _order_=i;
			  output;
			end;
			call symput("AnnoLen",strip(maxl));
      	keep &AnnoLabel &AnnoLoc &AnnoGroup _order_;
   	run;
   	options nolabel;
 
    data mapdata;
     	length &AnnoLabel $&AnnoLen;
     	set mapdata map2;
    run;
 %end;
%end;

%if %length(&AnnoInData)=0 %then %let AnnoWhereStmt=;

%if ((%symexist(AnnoWhereStmt) and %length(&AnnoWhereStmt)>0) 
		or (%symexist(XLVarStmt) and %length(&XLVarStmt)>0)
		or (%symexist(ReorderingVars) and %length(&ReorderingVars)>0)) %then %GenSubset(%quote(&AnnoWhereStmt));

/* Change for 9.2 
%if (%upcase(&MarkerFmt)=GENOTYPES or %upcase(&MarkerFmt)=%str(NONDELIMITED GENOTYPES)) and %length(&GenoDelim)=0 %then %do;
	data a;
		set &ds; 
		array __g__ &AlleleVars;
		array _dgeno{&nummark} $3;
		do ii=1 to &nummark;
			if lengthn(strip(__g__{ii}))>1 then do;
				_dgeno{ii}=cats(substr(strip(__g__{ii}),1,1),"/",substr(strip(__g__{ii}),2,1));
			end;
		end;
		drop &AlleleVars ii;
	run;
	
	%let ds=a;
	
	%let AlleleVars=_dgeno1-_dgeno&nummark;
	%let MarkerFmt = GENOTYPES;
	%let GenoDelim=%str(/);
	%let GenoCol = %str(genocol delimiter="&GenoDelim");
	
	data count; 
	 set &ds;
	 keep &AlleleVars;
	 if _n_=1 then output;
	 else stop;
	run;	
	
%end;
*/
%more:

data mapdatat;
	set mapdata;
	_ag = &AnnoGroup;
	%if &ProcessName=JMPGenomicsBrowser %then %do;
		if vtype(_ag)="C" then do;
			do i=0 to 9;	
				if _ag = "chr"||strip(i) then _ag = "chr0"||strip(i);
			end;
		end;
		drop i;
	%end;
run;
	
proc sort data=mapdatat out=mapgp(drop=_ag) nodupkey;
 by _ag;
run;

/************************
Retrieve number of groups in
MAP ds.  Get type of Group var
for plotting.  Find which markers
variables belong to which map
group.
**************************/
%let dsid=%sysfunc(open(mapgp));
%let NumGps=%sysfunc(attrn(&dsid,NOBS));
%let ret=%sysfunc(close(&dsid));

%let dsid=%sysfunc(open(mapdata));
%let GpPos=%sysfunc(varnum(&dsid,&AnnoGroup));
%global GpType GpLen;
%let GpType=%sysfunc(vartype(&dsid,&GpPos)); 
%let GpLen=%sysfunc(varlen(&dsid,&GpPos)); 
%let rc1=%sysfunc(close(&dsid));  

data _null_;
 set mapgp end=last_one;
 retain maxlenG maxlenL 0;
 lenG = lengthn(strip(&AnnoGroup));
 lenL = lengthn(strip(&AnnoLabel));
 if (lenG > maxlenG) then maxlenG = lenG;
 if (lenL > maxlenL) then maxlenL = lenL;
 %if &GpType=C %then %do;
 	call symput('Groupn'||left(_n_),trimn(&AnnoGroup));
	 _sasag=trimn(&AnnoGroup);
	 j=0;
	 do until(j=0);  
			j=notname(trimn(_sasag));  
			if j > 0 then substr(_sasag,j,1)="_";  
	 end;
 	call symput('SGroupn'||left(_n_),lowcase(trimn(_sasag)));
 %end;
 %else %do;
  call symput('Groupn'||left(_n_),trimn(left(&AnnoGroup)));
  if &AnnoGroup ne . then call symput('SGroupn'||left(_n_),trimn(left(&AnnoGroup)));
  else call symput('SGroupn'||left(_n_),"_");
 %end;
 if last_one then do;
 	call symputx("maxlenAG",strip(maxlenG));
 	call symputx("maxlenAL",strip(maxlenL));
 end;

 %exerr:
 
%mend;

%macro Psmooth(Tests, _datatable, _datatablename, _lsp=0);
/*********************
Create names of pvalue columns.
Run Proc PSMOOTH.  Create
macro vars to be used in JSL.
**********************/
/* _lsp = 1 when LS Means pvalues exist.  They have already been mult. test corrected, don't want p-value adjustment, but want
to use name creation stuff */

%let i=1;
%global PlotVars;
%let PlotVars=;
%let Prefixes=;
%let PPlotVars=;
%do %while (%length(%scan(&Tests,&i))>0);
 %let new=%scan(&Tests,&i);
 /* Exclude LSDiff pvalues for PSMOOTH */
 %if &_lsp=0 or %index(&new,LSDiff)=0 %then %let PPlotVars = %str(&PPlotVars Prob&new);
 %let PlotVars = %str(&PlotVars Prob&new);
 %let Prefixes = %str(&Prefixes Prob&new:);
 %let i=%eval(&i+1);
%end;
%global NumPlotVars;
%let NumPlotVars=%eval(&i-1);

%global TestNames;
%let TestNames=&Tests;

%global bwfrom bwto bwby;
%let bwfrom=%scan(&Bandwidth,1);
%if %length(&bwfrom)=0 %then %let bwfrom=0;

%if %length(%scan(&Bandwidth,3))>0 %then %do;
 %let bwto=%scan(&Bandwidth, 3);
%end;
%else %let bwto=&bwfrom;

%if %length(%scan(&Bandwidth,5))>0 %then %do;
 %let bwby=%scan(&Bandwidth, 5);
%end;
%else %let bwby=1; 

%global PvalAdjOrg; 
%let PvalAdjOrg= &PvalAdj;
%if %length(&PvalAdj)=0 %then %let PvalAdj=Simes;
%else %if %upcase(&PvalAdj)=TPM %then %let PvalAdj=&PvalAdj tau=&Tau;

%if %substr(&MTC,1,3)=%quote(FDR) %then %let MTC=FDR;

data temp1;
	set &_datatable;
	_mrgord_ = _n_;
run;

title;
proc psmooth data=temp1 &Conv bw=&bwfrom to &bwto by &bwby &PvalAdj &MTC out=temp;
 var &PPlotVars;
 %if %length(&ByVars) %then by &ByVars;;
 id _mrgord_;
run;
%if &syserr>4 %then %goto endps;

%let MTC=%substr(&MTC,1,3);

/*data OutLib.temp1; set temp1; run;*/
/*data OutLib.temp; set temp; run;*/

data &_datatable;
 merge temp1 temp;
 by _mrgord_;
 drop _mrgord_;
run;

%global r ALabel;
%if %upcase(&Conv)=NEGLOG %then %do;
 	%let r=%sysevalf(-%sysfunc(log(&alpha)));
 	%*let ALabel=%str(-log%(p%));
 	%let ALabel=%str(Negative Log P-Value);
%end;
%else %if %upcase(&Conv)=NEGLOG10 %then %do;
 	%let r=%sysevalf(-%sysfunc(log10(&alpha)));
 	%*let ALabel=%str(-log10%(p%));
 	%let ALabel=%str(Negative Log10 P-Value);
%end;
%else %do;
	%let r=%sysevalf(&alpha);
	%*let ALabel=%str(p-value);
	%let ALabel=%str(P-Value);
%end;

%put bwfrom bwto MTC= &bwfrom &bwto &MTC;
%global fl;
%if (&bwfrom=0 and &bwto=0 and %length(&MTC)=0) %then %do;
	%let fl=%substr(&PValAdj,1,1);
	%let dropvars=;
	%do i=1 %to &NumPlotVars; 
	 	%let dropvars=&dropvars %scan(&PlotVars,&i)_&fl.0;
	%end;
 
	data &_datatable;
		set &_datatable;
		drop &dropvars;
	run;

 	%let fl=;
%end;
%else %do;
	%let fl=%substr(&PValAdj,1,1);
%end;
%put fl= &fl;

%if %substr(%upcase(&Conv),1,3)=NEG %then %let Conv=&Conv._;

/*data OutLib._datatable; set &_datatable; run;*/

data &_datatable;
	set &_datatable;
	%if %length(&ByVars) %then %do;
		by &ByVars;
	%end;
	retain _max_ 0;
	array pvals{*} &Prefixes;
	%if %upcase(&Conv)=NEGLOG_ %then %do;
		%if %symexist(Censor) %then %do;
			maxp = &Censor;
		%end;
		%else %do;
			maxp=74;
		%end;
	%end;
	%else %if %upcase(&Conv)=NEGLOG10_ %then %do;
		%if %symexist(Censor) %then %do;
			maxp = &Censor;
		%end;
		%else %do;
			maxp=32;
		%end;
	%end;
	%else %do;
		maxp=1;
	%end;
	do i=1 to dim(pvals);
		if pvals{i} ne . then pvals{i}=min(pvals{i},maxp);
		if pvals{i} > _max_ then _max_ = pvals{i};
	end;
	%global AxisMax;
	%if %length(&Conv)=0 %then %let AxisMax=1.25;
	%else call symput("AxisMax",strip(_max_+0.25));;
	%if %length(&ByVars) %then %do;
		/* make sure have right lastby  */
		%let lastby=%scan(&ByVars,-1);
		if first.&lastby then CumOrder=1;
		else CumOrder+1;
	%end;
	%else %do;
		CumOrder = _n_;
	%end;
	drop i maxp _max_;
	%global PVars NPVars lsPVars;
  	%let PVars = ;
  	%let NPVars = 0;
  	%let lsPVars =;
	rename 
	%do i=&bwfrom %to &bwto %by &bwby;
		%if %length(&fl)=0 %then %do;
			%let oldsuffix=;
			%if %length(&MTC) %then %let newsuffix=_&MTC;
			%else %let newsuffix=;
		%end;
		%else %do;
			%let oldsuffix=_&fl.&i;
			%if %length(&MTC) %then %let newsuffix=_&fl.&i._&MTC;
			%else %let newsuffix=&oldsuffix;
			%if &bwfrom=0 and &bwto=0 and &fl=S %then %let newsuffix=_&MTC;
			/*%if %length(&PValAdjOrg)=0 and &fl=S %then %let newsuffix=_&MTC;*/
		%end;
		%do k=1 %to %eval(&NumPlotVars);
			%if &_lsp=0 or %index(%scan(&PlotVars,&k),LSDiff)=0 %then %do;
				%let PVars = &PVars %unquote(&Conv%scan(&PlotVars,&k)&newsuffix);
				%let NPVars = %eval(&NPVars + 1);
				%unquote(%scan(&PlotVars,&k)&oldsuffix) = %unquote(&Conv%scan(&PlotVars,&k)&newsuffix)
				/* S0942566,S0943182 the previous statement (witout %unquote()) casued problem TC 02/11/13 */
				/*begin: when conv and mct were both true, the variable name was being 
				reported as ProbGenotype and ProbTrend, but it should have been &conv.ProbGenotype 
				and &conv.ProbTrend to account for the log transformation. sbrlsi*/
				%if %length(&oldsuffix) and %length(&conv) %then %do;
					%unquote(%scan(&PlotVars,&k)) = %unquote(&Conv%scan(&PlotVars,&k))
				%end;
				/*end: sbrlsi*/
			%end;
			%else %do; 
				/* LSDiff p-values */
				%if &i=&bwfrom %then %do;
					%if %length(&MTC) %then %do;
						%let LSPVars = &LSPVars &Conv%scan(&PlotVars,&k)_&MTC;
						%unquote(%scan(&PlotVars,&k)) = %unquote(&Conv%scan(&PlotVars,&k)_&MTC)	
					%end;
					%else %do;
						%unquote(%let LSPVars = &LSPVars &Conv%scan(&PlotVars,&k);					
						%scan(&PlotVars,&k)) = %unquote(&Conv%scan(&PlotVars,&k))
					%end;
				%end;
			%end;
		%end;
	%end;
	; /* end rename stmt*/
run;
%put PVars= &PVars;

%if &ProcessName ne HaplotypeEstimation and &ProcessName ne ImputedSNPTraitAssociation 
 and &ProcessName ne MultipleSNPTraitAssociation and &ProcessName ne SNPSNPInteractions
 and &ProcessName ne RareVariantAssociation and &ProcessName ne HaplotypeQKMixedModel
 and &ProcessName ne HaplotypeTrendRegression
%then %do;
	%if (%symexist(AnnoWhereStmt) and %length(&AnnoWhereStmt)) or (&ProcessName=%str(PCAforPS) and %length(&NullSNPs)) %then %do;
		
		/*
		proc sort data=&_datatable out=&_datatable;
			by _oldorder_;
		run;
		*/
		
		%let JoinBy= %str(a._oldorder_ = b._oldorder_);

		/* Need to do this so don't have missing values for BY variables for excluded markers */
		%if %length(&ByVars) %then %do;
			proc sort data=&_datatable out=byvars(keep=&ByVars) nodupkey;
				by &ByVars;
			run;
			
			/*data OutLib.gm_mapdata_orig; set mapdata_orig; run;*/

			%NObsvars(mapdata_orig);
			%let mapobs=&nobs;
			%NObsvars(byvars);

			data mapdata_orig_by;
				set byvars;
				do _i=1 to &mapobs;
					output;
				end;
				drop _i;
			run;

			data mapdata_orig_by;
				set mapdata_orig_by;
				set %do i=1 %to &nobs; mapdata_orig %end;;
			run;

			%if &syserr>4 %then %goto endps;

			proc sort data=mapdata_orig_by out=mapdata_orig;
				by _oldorder_ &ByVars;
			run;

			proc datasets nolist;
				delete mapdata_orig_by;
			quit;
			
			/*data OutLib.datatable; set &_datatable;run;*/
			/*data OutLib.mapdata_orig; set mapdata_orig;run;*/
			
			*S1561109: Creating list of by-variables to join data sets*;
		  	%let k=1;
		        %do %while(%length(%scan(&ByVars,&k,' ')));
		        	%let currby= %scan(&ByVars,&k,' ');
		        	%let JoinBy= &JoinBy %str(and a.&currby = b.&currby);
		        	%let k=%eval(&k+1);
         		%end;
         		%let nby= %eval(&k-1);
         		%put nby= &nby;
		%end;
		
		/*data OutLib.ps_mapdata_orig; set mapdata_orig; run;*/
		/*data OutLib.ps_datatable0; set &_datatable; run;*/
		
		/*
		proc sort data=&_datatable out=&_datatable;
			by _oldorder_;
		run;
		
		%put JoinBy= &JoinBy;
		**S1561109: merge data sets here to avoid creating duplicated rows*;
		proc sql UNDO_POLICY=NONE;
			create table &_datatable as 
			select *
			from mapdata_orig(drop=keepthis _xlvar) as a left join &_datatable as b
			on &JoinBy
			;
		quit;	
		
		data &_datatable;
			set &_datatable;
			%if %symexist(MarkerFmt) %then %do;
				%if &MarkerFmt = %str(Numeric Genotypes) %then %do;
					retain _ads_;
					if _n_=1 then do;
						if exist("count_old") then  _ads_=open("count_old","i"); 
						else _ads_=open("count","i");
					end;
					_Column_Name_ = varname(_ads_,_oldorder_);
					if _closeds then _rc_=close(_ads_);
				%end;
			%end;
			drop _oldorder_ _order_ _ads_ _rc_;
		run;
		*/
		
		proc sort data= mapdata_orig out= mapdata_orig;
			by _oldorder_ &ByVars;
		run;

		proc sort data= &_datatable out=&_datatable;
			by _oldorder_ &ByVars;
		run;	

		/*
		*Checking variables with same name in the mapdata_orig and _datatable*;
		proc contents data=mapdata_orig out=varsmap(keep= MEMNAME NAME) noprint;
		run;
		
		proc contents data=&_datatable out=varstable(keep= MEMNAME NAME) noprint;
		run;
		
		proc sql UNDO_POLICY=NONE;
			create table commonvars as 
			select a.*, b.*
			from varsmap(rename=(memname= memnamemap name=namemap)) as a
			full join varstable as b
			on upcase(a.namemap)=upcase(b.name)
			;
		quit;
		
		data _null_;
			set commonvars end=_eof_;
			retain count;
			if _n_= 1 then count=0;
			if not missing(namemap) and not missing(name) then do;
				count= count + 1;
				call symputx("rename"||trim(left(count)),trim(left(namemap)));
			end;
			if _eof_ then call symputx("nrename", count);
		run;
		%put nrename= &nrename;
		*/
		
		**S1574488: before this defect, the code used proc sql, now it is using merge.*;
		data &_datatable;
			merge mapdata_orig(drop=keepthis _xlvar) &_datatable end=_closeds;
			by _oldorder_ &ByVars;
			%if %symexist(MarkerFmt) %then %do;
				%if &MarkerFmt = %str(Numeric Genotypes) %then %do;
					retain _ads_;
					if _n_=1 then do;
						if exist("count_old") then  _ads_=open("count_old","i"); 
						else _ads_=open("count","i");
					end;
					_Column_Name_ = varname(_ads_,_oldorder_);
					if _closeds then _rc_=close(_ads_);
				%end;
			%end;
			drop _oldorder_ _order_ _ads_ _rc_;
		run;
				
		/*data OutLib.ps_datatable1; set &_datatable; run;*/
		
	%end;
	%else %do;
	
		data %unquote(&_datatable);
			set %unquote(&_datatable) end=_closeds;
			retain _ads_;
			%if %symexist(MarkerFmt) %then %do;
				%if &MarkerFmt = %str(Numeric Genotypes) %then %do;
					if _n_=1 then do;
						if exist("count_old") then  _ads_=open("count_old","i"); 
						else _ads_=open("count","i");
					end;
					_Column_Name_ = varname(_ads_,_order_);
					if _closeds then _rc_=close(_ads_);
				%end;
			%end;
			drop _ads_ _rc_;
		run;
		
		proc sort data=&_datatable out=&_datatable(drop=_order_);
		 by _order_;
		run;	
	%end;
%end;

%endps:;
%mend;

%macro JSLPvalMap(TestName, _JSLFile, _JSLFileName, _OutDataName, _pve=0, _overlay=1, _OutDataName2=, mod=0, _summchart=1, _volplot=0, _forestplot=0, _pvalplots=1, _subsetdd = 0, _lsvp=0, _plotinput=0);
/* mod should only be 1 from PCA for PS to append this code to existing JSL */
/* _summchart should only be 0 when from JMP Genomics Browser for non-pvalue variables - to prevent anything related to summary chart from running */
/* set _volplot to 1 to create volcano plot - must have Estimate_Trend and a variable with ProbTrend */
/* _forestplot=1 only for GWAS Meta-Analysis when inverse variance method is used */
/* _pvalplots=0 for Marker Properties to suppress HWE plots when HWEType=None */
/* _subsetdd=1 to include Subset and Reorder drill-down */
/* _lsvp to create volcano plots of genotype ls diffs */
/* _plotinput to plot trait vs. genotype from input data set */

/*
%put NumGps= &NumGps;
%put AnnoOrganism= &AnnoOrganism;
%put AnnoAccession_All= &AnnoAccession_All;
%put AnnoGroup= &AnnoGroup;
%put AnnoLoc= &AnnoLoc;
%put AnnoLabel= &AnnoLabel;
%put ByVars= &ByVars;
%put JBy= &JBy;
%put NbyVal= &NbyVal;
%put ByVarsOnly= &ByVarsOnly;
%put NByVars= &NByVars;
%put OrigNbyval= &OrigNbyval;
%put JNONTRAITBYVARS= &JNONTRAITBYVARS;
*/

%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
	%let _suf_= _&nCallstoJSPV;
%end;
%else %do;
	%let _suf_= ;
%end;

%let maxon = %eval(31-&maxlenAG);
%if %length(%trim(&_OutDataName)) > &maxon %then 
	%let _name = %substr(&_OutDataName,1,&maxon);
%else %let _name = &_OutDataName;

options label;

%if %symexist(NPVars)=0 %then %let NPVars=0;

%if %symexist(PlotFitVars)=0 %then %let PlotFitVars=0;

%global MUnit;
%if %lowcase(&ProcessName)=multiplesnptraitassociation or %lowcase(&ProcessName)=rarevariantassociation %then %let MUnit = &AnnoGroup2.s;
%else %if %lowcase(&ProcessName)=haplotypeestimation %then %let MUnit = %str(Marker Windows);
%else %let MUnit = Markers;

%if ^%symexist(AxisMin) %then %let AxisMin = -0.05;

%if &NPVars>0 and &_summchart %then %do;
	%if %length(&Conv)=0 %then %let ltgt=%str(<);
	%else %let ltgt=%str(>);

	data sigind;
		set OutLib.&_OutDataName;
		array pvalvars{*} &PVars;
		array sigind{&NPvars} ;
		do _i=1 to &NPvars;
			if missing(pvalvars{_i})=0 and pvalvars{_i} &ltgt &r then sigind{_i}=1;
			else sigind{_i}=0;
		end;
	run;
	
	proc sort data=sigind;
		by &ByVars &AnnoGroup;
	run;

	proc means data=sigind noprint;
		var sigind1-sigind&Npvars;
		by &ByVars &AnnoGroup;
		output out=OutLib.%substr(&_name,1,28)_sbg sum(sigind1-sigind&Npvars)=
			%do p=1 %to &Npvars; %substr(%scan(&Pvars,&p),%sysfunc(max(1,%index(%scan(&Pvars,&p),%str(Prob))))) %end;;
	run;
	
	options label;
	proc datasets nolist library=OutLib;
		modify %substr(&_name,1,28)_sbg;
		label %do p=1 %to &NPVars; 
			%if %index(%scan(&Pvars,&p),%str(Prob))>0 %then %do;
				%if %index(%scan(&Pvars,&p),%str(HapCC))=0 and 
					%index(%scan(&Pvars,&p),%str(Rare))=0 and
					%index(%scan(&Pvars,&p),%str(RaoScott))=0 %then %do;
					%let bg= %sysevalf(%index(%scan(&Pvars,&p),%str(Prob)) + 4);
					%let ed= %sysfunc(find(%scan(&Pvars,&p), %str(_), &bg + 1));
					%if &ed %then %do;
						%let st= %substr(%scan(&Pvars,&p), &bg, &ed - &bg);
					%end;
					%else %do;
						%let st= %substr(%scan(&Pvars,&p), &bg);
					%end;			
					%*let ctlabel&p=%quote(Number of Significant &MUnit for %substr(%scan(&Pvars,&p),%index(%scan(&Pvars,&p),%str(Prob))+4) Test);
					%let ctlabel&p=%quote(Number of Significant &MUnit for &st Test);
				%end;
				%else %if %index(%scan(&Pvars,&p),%str(Rare))>0 %then %do;
					%let ctlabel&p=%quote(Number of Significant &MUnit for Rare Variant Test);
				%end;				
				%else %if %index(%scan(&Pvars,&p),%str(HapCC))>0 %then %do;
					%let ctlabel&p=%quote(Number of Significant &MUnit for Haplotype Case Control Test);
				%end;
				%else %if %index(%scan(&Pvars,&p),%str(RaoScott))>0 %then %do;
					%let ctlabel&p=%quote(Number of Significant &MUnit for RaoScott Test);
				%end;				
				%else %do;
					%let ctlabel&p=%quote(Number of Significant &MUnit for %scan(&Pvars,&p));
				%end;
			%end;			
			%else %do;
				%let ctlabel&p=%quote(Number of Significant &MUnit for %scan(&Pvars,&p));
			%end;
			%substr(%scan(&Pvars,&p),%sysfunc(max(1,%index(%scan(&Pvars,&p),%str(Prob)))))="&&ctlabel&p" 
		%end;;
	quit;
	%if (&ProcessName ne MissingGenotypeSummary and &ProcessName ne JMPGenomicsBrowser) %then options nolabel;;

%end;

/*
data %do i=1 %to &NumGps; OutLib.&_name._&&SGroupn&i %end;;
	set OutLib.&_OutDataName;
	%if (&GpType = N) %then %do;
		if &AnnoGroup = &Groupn1 then output OutLib.&_name._&SGroupn1;
	%end;
	%else %do;
		if trimn(&AnnoGroup) = "&Groupn1" then output OutLib.&_name._&SGroupn1;
	%end;				
	%do i=2 %to &NumGps;
		%if (&GpType = N) %then %do;
			else if &AnnoGroup = &&Groupn&i then output OutLib.&_name._&&SGroupn&i;
		%end;
		%else %do;
			else if trimn(&AnnoGroup) = "&&Groupn&i" then output OutLib.&_name._&&SGroupn&i;
		%end;			
	%end;
run;
*/

%let dsid=%sysfunc(open(Outlib.%substr(&_name,1,28)_sbg,i));
%let AnnoGpLab=%sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&AnnoGroup))));
%if %length(&AnnoGpLab) %then %let AnnoGroup=%quote(&AnnoGpLab);
%let rc=%sysfunc(close(&dsid));

%if &_pve=0 %then %do;
	%global _mainjslfile_;
	%let _mainjslfile_ = %quote(&_jslfile);	
%end;

%if %symexist(SummaryDescription)=0 %then %let SummaryDescription=;

/* Get var labels - for PValue Viewer */
%let dsid=%sysfunc(open(Outlib.&_OutDataName,i));
%let AnnoLabLab=%sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&AnnoLabel))));
%if %length(&AnnoLabLab) %then %let AnnoLabel=%quote(&AnnoLabLab);
%let AnnoLocLab=%sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&AnnoLoc))));
%if %length(&AnnoLocLab) %then %let AnnoLoc=%quote(&AnnoLocLab);
%if %symexist(AnnoLoc2) %then %do;
	%let AnnoLocLab2=%sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&AnnoLoc2))));
	%if %length(&AnnoLocLab2) %then %let AnnoLoc2=%quote(&AnnoLocLab2); 
%end;
%let AnnoAccLab=%sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&AnnoAccession))));
%if %length(&AnnoAccLab) %then %let AnnoAccession=%quote(&AnnoAccLab);   	

%do p=1 %to &NPvars;
	%let CurrPVar=%scan(&PVars,&p);
	%let vn=%sysfunc(varnum(&dsid,&CurrPVar));
	%let PVarLab=%sysfunc(varlabel(&dsid,&vn));
	%if %length(&PVarLab)=0 %then %let PVarLab=&CurrPVar;
	%let PVar&p=%quote(&PVarLab);
%end;

/*
%if %upcase(&ProcessName) = HAPLOTYPEESTIMATION %then %do;
	%let k=1;
	%let l=1;
	%let origNPvars= &NPvars;
	%do p=1 %to &NPVars;
		%let origPVar&p= &&PVar&p;
		%if %sysfunc(find(&&PVar&p, HapCC)) %then %do;
			%let PVar&k= &&PVar&p; 
			%let k= %sysevalf(&k + 1);
		%end;
		%else %if %sysfunc(find(&&PVar&p, LD)) %then %do;
			%let LDPVar&l= &&PVar&p;
			%let l= %sysevalf(&l + 1);
		%end;
	%end;
	%let NPvars= %sysevalf(&k - 1);
	%let LDNPvars= %sysevalf(&l - 1);
%end;
*/

/*MTC correction*/
%let currSubPVar = &PVar1;
%let mtcmethod= None;
%if %sysfunc(find(&PVar1, _Bon)) %then %do;
	%let mtcmethod= %str(Bonferroni);
	%let currSubPVar = %sysfunc(tranwrd(&PVar1, _Bon, ));
%end;
%else %if %sysfunc(find(&PVar1, _FDR)) %then %do;
	%let mtcmethod= %str(FDR);
	%let currSubPVar = %sysfunc(tranwrd(&PVar1, _FDR, ));
%end;
%else %if %sysfunc(find(&PVar1, _Sid)) %then %do;
	%let mtcmethod= %str(Sidak);
	%let currSubPVar = %sysfunc(tranwrd(&PVar1, _Sid, ));
%end;	

/*p-value adjustment*/					
%let adjmethod= None;
%let adjbandth= ;	
%let adjtau= ;
%if %sysfunc(find(&currSubPVar, _F)) %then %do;

	%let bg= %sysfunc(find(&currSubPVar, _F));
	%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
	%if &ed %then %do;
		%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
	%end;
	%else %do;
		%let bwth= %substr(&currSubPVar, &bg + 2);
	%end;
	%let adjbandth= %str(Bandwith= &bwth);
	%let adjmethod= %str(Fisher / &adjbandth);
%end;
%else %if %sysfunc(find(&currSubPVar, _S)) %then %do;

	%let bg= %sysfunc(find(&currSubPVar, _S));
	%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
	%if &ed %then %do;
		%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
	%end;
	%else %do;
		%let bwth= %substr(&currSubPVar, &bg + 2);
	%end;
	%let adjbandth= %str(Bandwith= &bwth);					
	%let adjmethod= %str(Simes / &adjbandth);
%end;
%else %if %sysfunc(find(&currSubPVar, _T)) %then %do;
		
	%let bg= %sysfunc(find(&currSubPVar, _T));
	%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
	%if &ed %then %do;
		%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
	%end;
	%else %do;
		%let bwth= %substr(&currSubPVar, &bg + 2);
	%end;
	%let adjbandth= %str(Bandwith= &bwth);
	%if %symexist(Tau) %then %do;
		%let adjmethod= %str(TPM / &adjbandth / Tau= &Tau);	
	%end;
	%else %do;
		%let adjmethod= %str(TPM / &adjbandth);	
	%end;
%end;

%if &PlotFitVars %then %do;
	%macro GetPlotParmsFitVars(OverlayVars=, PrefixVar=, NVars=, minVal=, maxVal=);
		%if %length(&&&OverlayVars) %then %do;
			%let p= 1;
			%let CurrVar=%scan(&&&OverlayVars, &p, %str( ));
			%do %while(%length(&CurrVar));
				%let vn=%sysfunc(varnum(&dsid,&CurrVar));
				%let FitVarLab=%sysfunc(varlabel(&dsid,&vn));
				%if %length(&FitVarLab)=0 %then %let FitVarLab=&CurrVar;
				%global &PrefixVar&p;
				%let &PrefixVar&p=%quote(&FitVarLab);
				%let p= %sysevalf(&p + 1);
				%let CurrVar=%scan(&&&OverlayVars,&p, %str( ));
			%end;
			%let &NVars= %sysevalf(&p - 1);

			proc means data=Outlib.&_OutDataName max min noprint;
				var &&&OverlayVars;
				output out=FitStatMaxMin;
			run;

			proc transpose data=FitStatMaxMin out=FitStatMaxMin_t;
				var &&&OverlayVars;
				id _stat_;
			run;

			proc sort data=FitStatMaxMin_t;
				by min;
			run;

			data _null_;
				set FitStatMaxMin_t(obs=1);
				call symputx("&minVal", trim(left(min)));
			run;

			proc sort data= FitStatMaxMin_t;
				by descending max;
			run;

			data _null_;
				set FitStatMaxMin_t(obs=1);
				call symputx("&maxVal", trim(left(max)));
			run;

			%put minVal= &minVal;
			%put maxVal= &maxVal;
		%end;
	%mend;
	%global Group1Prefix NGroup1Val minGroup1Val maxGroup1Val;
	%GetPlotParmsFitVars(OverlayVars= Group1OverlayVars, PrefixVar= Group1Var, NVars= NGroup1Val, minVal= minGroup1Val, maxVal= maxGroup1Val);
	%put NGroup1Val_= &NGroup1Val;
	%put minGroup1Val_= &minGroup1Val;
	%put maxGroup1Val_= &maxGroup1Val;

	%if %length(&Group2OverlayVars) %then %do;
		%global Group2Prefix NGroup2Val minGroup2Val maxGroup2Val;
		%GetPlotParmsFitVars(OverlayVars= Group2OverlayVars, PrefixVar= Group2Var, NVars= NGroup2Val, minVal= minGroup2Val, maxVal= maxGroup2Val);
		%put NGroup2Val_= &NGroup2Val;
		%put minGroup2Val_= &minGroup2Val;
		%put maxGroup2Val_= &maxGroup2Val;
	%end;
	
	%if %length(&Group3OverlayVars) %then %do;
		%global Group3Prefix NGroup3Val minGroup3Val maxGroup3Val;
		%GetPlotParmsFitVars(OverlayVars= Group3OverlayVars, PrefixVar= Group3Var, NVars= NGroup3Val, minVal= minGroup3Val, maxVal= maxGroup3Val);
		%put NGroup3Val_= &NGroup3Val;
		%put minGroup3Val_= &minGroup3Val;
		%put maxGroup3Val_= &maxGroup3Val;
	%end;
%end;

%let rc=%sysfunc(close(&dsid));
%if &_pve=0 %then %let AnnoAccession_All=&AnnoAccession;

* main script now just opens the journal;
data _null_;
	%if &mod=1 %then %do;
		file "&_JSLFile" mod lrecl=32767;
	%end;
	%else %do;
		file "&_JSLFile" lrecl=32767;
		put "//!";
		put "Names Default to Here(1);";

		put "jg:drill_down = 0;";

		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";
		
		%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
			put "Here:ActionButtonDesc= {};";
			put "Here:ActionButtonList= {};";
			put "Here:ActionCodeList= {};";
		%end;
	%end;
	
	/*S1149704 sbrlsi %*if &NPVars>0 and &NumGps>1 and &_summchart %then %do;*/	
	%if (&NPVars>0 and &_summchart) and (&NumGps>1 or %upcase(&ProcessName) = QKMIXEDMODEL) %then %do; 
		
		put "InsertInto(DataList,{{""Here:sbg""}});";
		put "InsertInto(Here:TabButtonList,""Summary Chart"");";

		%if &NByVal<=1 %then %do;
			put "Here:sbgchart=Function({},{default local},";
			put "if(isEmpty(Here:sbg),";
			put " Here:sbg=open(""&ClientOutPath.%substr(&_name,1,28)_sbg.sas7bdat"", invisible);";
			put " foundit=Try(agcol = Column(""&AnnoGroup"");1,0);";
			put " if(foundit,agcol << Set Property(""Value Ordering"",{""chr0"",""chr1"",""chr2"",""chr3"",""chr4"",""chr5"",";
			put "    ""chr6"",""chr7"",""chr8"",""chr9""});";
			put "    agcol << label(1);";
			put " );";
			put ",//else";
			put " CurrentDataTable(Here:sbg);";
			put ");";
			put "Vlistbox(";
			%if %length(&SummaryDescription) %then %do;
				put " Textbox(""&SummaryDescription""),";			
			%end;
			%if ^%index(&ProcessName,MetaAnalysis) %then %do;
				put "   HListBox(";
				put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
				put "	),";
				put "   HListBox(";
				put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
				put "	),";
			%end;
			%if %length(&SummaryDescription) or ^%index(&ProcessName,MetaAnalysis) %then %do;
				put "   HListBox(";
				put "   	Text Box( "" "" ),";
				put "	),";
			%end;
			put " Here:sbg_gb= Graph Builder(";
			put "  Show Control Panel( 0 ),";
			put "  Graph Spacing( 3 ),";
			put "  Spacing Borders( 1 ),";	
			put "  Legend Position( ""Bottom""),";
			put "  Variables("; 
			put "  X( :&AnnoGroup),";
			%let k=0;
			%do p=1 %to &Npvars;
				put "  Y( ";
				put "  :&&ctlabel&p, Position(&k)" ;
				put "  ),";
				%let k=1;
			%end;
			put " ),";
			put " Elements("@; 
			put " Bar("@;
			put " X,"@;
			%do p=1 %to &Npvars;
				put " Y(&p),"@;
			%end;			
			put ")";
			put "),"; 
			/*S1149704 sbrlsi*/
			/*%if &NumGps = 1 %then %do; 
				put "   //Category Axis << {Axis Name( """" ), Label( ""none"" )},";
			%end;
			*/
			put " SendToReport(";
			put "		Dispatch(";
			put "			{},";
			put "	   		""Graph Builder"",";
			put "			OutlineBox,";
			/*%if %symexist(TraitVar) %then %do;
				%if &NumGps > 1 %then %do; 
					put "   Set Title( ""Number of Significant &MUnit by &AnnoGroup for &TraitVar"" )";
				%end; 
				%else %do;
					put "   Set Title( ""Number of Significant &MUnit for &TraitVar"" )";  *S1149704 sbrlsi*
				%end;
			%end;
			%else %do;*/
				%if &NumGps > 1 %then %do; 
					put "   Set Title( ""Number of Significant &MUnit by &AnnoGroup"" )";
				%end; 
				%else %do;
				        /**S1149704 sbrlsi**/
					put "   Set Title( ""Number of Significant &MUnit"" )";  
				%end;			
			/*%end;*/
			put "		),";
			put "   Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Number of Significant &MUnit"" )} ),";
			/*put "   Dispatch( {}, """", AxisBox( 2 ), Add Axis Label( ""Number of Significant &MUnit"" ) ),";*/			
			/*
			%if &ProcessName = MarkerProperties and &_pvalplots %then %do;
				put "   Dispatch( {}, """", AxisBox( 2 ), Add Axis Label( ""Number of Significant &MUnit for HWE Test"" ) ),";
			%end;
			%else %do;
				put "   Dispatch( {}, """", AxisBox( 2 ), Add Axis Label( ""Number of Significant &MUnit"" ) ),";
			%end;
			*/
			put "   Dispatch( {}, ""Graph Builder"", FrameBox, {Frame Size( 398, 207 )} ),";
			put "   Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
			put "	Dispatch(";
			put "		{},";
			put "		""400"",";
			put "		LegendBox,";
			put "		{Orientation( ""Horizontal"" ), Sides( ""Left"" ), Set Wrap( 1 )}";
			put "	),";
			put "	Dispatch(";
			put "		{},";
			put " 		""&ctlabel1"", ";
			put "		ScaleBox,";
			put "		{Format( ""Fixed Dec"", 12, 0 )}";
			put "	),";			
			put ")";
			put ")));";			
		%end;
		%else %do;
			put "Here:sbgchart=Function({},{default local},";
			put "if(isEmpty(Here:sbg),";
			put " Here:sbg=open(""&ClientOutPath.%substr(&_name,1,28)_sbg.sas7bdat"", invisible);";
			put " foundit=Try(agcol = Column(""&AnnoGroup"");1,0);";
			put " if(foundit,agcol << Set Property(""Value Ordering"",{""chr0"",""chr1"",""chr2"",""chr3"",""chr4"",""chr5"",";
			put "    ""chr6"",""chr7"",""chr8"",""chr9""});";
			put "    agcol << label(1);";
			put " );";
			put ",//else";
			put " CurrentDataTable(Here:sbg);";
			put ");";			
			put " Vlistbox(";
			%if %length(&SummaryDescription) %then %do;
				put "  Textbox(""&SummaryDescription""),";				
			%end;
			%if ^%index(&ProcessName,MetaAnalysis) %then %do;
				put "   HListBox(";
				put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
				put "	),";
				put "   HListBox(";
				put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
				put "	),";
			%end;
			%if %length(&SummaryDescription) or ^%index(&ProcessName,MetaAnalysis) %then %do;
				put "   HListBox(";
				put "   	Text Box( "" "" ),";
				put "	),";
			%end;
			put "  Here:sbg_gb= Graph Builder(";
			put "   Show Control Panel( 0 ),";
			put "	Graph Spacing( 3 ),";
			put "	Spacing Borders( 1 ),";	
			put "   Legend Position( ""Bottom""),";			
			put "	Variables(";
			put "   X(:&AnnoGroup ),";
			%let k=0;
			%do p=1 %to &NPvars;
				put "  Y( ";
				put "  :&&ctlabel&p, Position(&k)" ;
				put "  ),";
				%let k=1;
			%end;
			%if %length(&ByVars)>0 %then %do;
				%let jb= %scan(&ByVars,1," ");
				put "   Group Y(:&jb),";
			%end;
			put "),";
			%if %length(&ByVars)>0 and &NByVal > 1 %then %do;
				put "   By(";
				%let k=2;
				%let jb= %scan(&ByVars,&k," ");
				%do %while(%length(&jb));
					put "           &jb,";
					%let k= %sysevalf(&k + 1);
					%let jb= %scan(&ByVars,&k," ");
				%end;	
				put "   ),";
			%end;
			
			put " Elements("@; 
			put "	Bar("@;
			put "		X,"@;
			%do p=1 %to &Npvars;
				put "  Y(&p),"@;
			%end;			
			put ")";
			put "),";
			put " SendToReport(";
			put "		Dispatch(";
			put "			{},";
			put "	   		""Graph Builder"",";
			put "			OutlineBox,";
			/*
			%if %symexist(TraitVar) %then %do;
				%if &NumGps > 1 %then %do; 
					put "   Set Title( ""Number of Significant &MUnit by &AnnoGroup for &TraitVar"" )";
				%end; 
				%else %do;
					put "   Set Title( ""Number of Significant &MUnit for &TraitVar"" )";  **S1149704 sbrlsi**
				%end;
			%end;
			%else %do;
				%if &NumGps > 1 %then %do; 
					put "   Set Title( ""Number of Significant &MUnit by &AnnoGroup"" )";
				%end; 
				%else %do;
					put "   Set Title( ""Number of Significant &MUnit"" )";  **S1149704 sbrlsi**
				%end;			
			%end;	
			*/
			put "		),";
			put "   Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Number of Significant &MUnit"" )} ),";
			/*put "   Dispatch( {}, """", AxisBox( 2 ), Add Axis Label( ""Number of Significant &MUnit"" ) ),";*/			
			/*
			%if &ProcessName ^= MarkerProperties and &NPvars > 1 %then %do;
				%do p=1 %to &Npvars;
					put "   Dispatch( {}, """", AxisBox( &p + 1 ), Add Axis Label( ""Number of Significant &MUnit"" ) ),";
				%end;
			%end;
			*/
			put "   Dispatch( {}, ""Graph Builder"", FrameBox, {Frame Size( 398, 207 )} ),";
			put "   Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
			put "	Dispatch(";
			put "		{},";
			put "		""400"",";
			put "		LegendBox,";
			put "		{Orientation( ""Horizontal"" ), Sides( ""Left"" ), Set Wrap( 1 )}";
			put "	),";	
			put "	Dispatch(";
			put "		{},";
			put " 		""&ctlabel1"", ";
			put "		ScaleBox,";
			put "		{Format( ""Fixed Dec"", 12, 0 )}";
			put "	),";			
			put ")";			
			put ")";			
			put "));";
		%end;
		
		put "////////sbgchart_after ";
		put "Here:sbgchart_after=function({},{default local},";
		put "Here:dt = Today();";
		put "Eval(Parse(EvalInsert(""\[sbgr^dt^ = Here:sbg_gb << Report;]\"")));";
		put "Eval(Parse(EvalInsert(""\[";
  		put "        for(_b=1, _b<=&NbyVal, _b++,"; 		
  		put "            //try() will return false if NbyVal=1;";
  		put "            try(";
  		put "            //show(sbgr^dt^[_b][OutLineBox(1)]);";
  		put "            gtitle= sbgr^dt^[_b][OutLineBox(1)]<< Get Text;";
  		put "            //show(gtitle);";
  		put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
  		put "            //show(gtitle);";
      		put "            pos= Munger(gtitle, 1, ""Where("");";
      		put "            //show(pos);";
      		put "            if(pos, ";
      		put "            	del= Trim(Substr(gtitle, pos - 6));";
      		put "            	sbgr^dt^[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
      		put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";
      		put "            );";
      		put "            //show(gtitle);";
      		/*%if %symexist(TraitVar) %then %do;
      			%if &NumGps > 1 %then %do; 
  				put "            sbgr^dt^[_b][OutLineBox(1)] << {Set Title(""Number of Significant &MUnit by &AnnoGroup for &TraitVar for ""||gtitle), Image Export Display( Normal )};";
  			%end;
  			%else %do;
				put "            sbgr^dt^[_b][OutLineBox(1)] << {Set Title(""Number of Significant &MUnit for &TraitVar for ""||gtitle), Image Export Display( Normal )};";
  			%end;
  		%end;
  		%else %do;*/
  		      	%if &NumGps > 1 %then %do; 
		  		put "            sbgr^dt^[_b][OutLineBox(1)] << {Set Title(""Number of Significant &MUnit by &AnnoGroup for ""||gtitle), Image Export Display( Normal )};";
		  	%end;
		  	%else %do;
				put "            sbgr^dt^[_b][OutLineBox(1)] << {Set Title(""Number of Significant &MUnit for ""||gtitle), Image Export Display( Normal )};";
  			%end;
  		/*%end;*/
  		put "        ));";		
		put ']\"';				
		put ")));";
		put ");";		
		
		put "InsertInto(Here:TabCodeList,expr(sbgchart()));";
		put "InsertInto(TabAfterCodeList,expr(Here:sbgchart_after));"; 
		put "InsertInto(Here:TabOpenList,1);";
	%end;
  	
  	/*** Venn drill-down function ***/
  	%if %symexist(NTrait) %then %do;
  	%if &NTrait > 1 /*and %quote(&ByVars) = Trait*/ %then %do;
		put "Here:VennCompTraits&_suf_ = Function({_currpvar,_currtestname},{default local},";
		%if &NumGps > 1 or %upcase(&ProcessName) = QKMIXEDMODEL %then %do; /*S1149704 sbrlsi %if &NumGps > 1 %then %do;*/
			put " aglist = { ""_All_"" " @;
			%do jj=1 %to &NumGps;
				put ", ""&&Groupn&jj"" " @;
			%end;
			put "};";
			put " dlgg = New Window(""Select a &AnnoGroup to View"", <<ReturnResult, <<modal,";
			put "   vlistbox(";
			put "    Here:cglistb=listBox(aglist,maxselected(1)),";			
			put "    buttonbox(""OK"", cglist=Here:cglistb<<Get Selected;)));";
			put " if (dlgg[""button""] == -1, throw());";
			put " if(Nitems(cglist)==0,cglist=aglist);";
			put " whichgpl = Loc(aglist,cglist[1]);";
			put " //show(whichgpl, cglist, aglist);";
			put " whichgp = Num(whichgpl[1]) - 1;";
		%end;
		%else %do;
			put " whichgp = 0;";
		%end;	
		%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
			put " vennei = EvalInsert(""\[ Here:cc^whichgp^&_suf_(); ]\""); ";
		%end;
		%else %do;
			put " vennei = EvalInsert(""\[ Here:cc^whichgp^(); ]\""); ";
		%end;		
		put " Eval(parse(vennei));";

		put " ncSigName = ""Sig_""||_currpvar;";
		put " nsigcol = New Column(ncSigName,Numeric);";
		put " for each row(";
		%if %length(&Conv)=0 %then %do;
			put " if(Column(_currpvar)[] < &r ,";  
		%end;
		%else %do;
			put " if(Column(_currpvar)[] > &r ,";  
		%end;		
		put "    Column(ncSigName)[]=1,Column(ncSigName)[]=0)";
		put " );";
		put "	nsigcol << Set Modeling Type(""Nominal"");";
		put " venn2ei = EvalInsert(""\[ ";
		%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
			put "  NumCol = NCol(Here:data^whichgp^&_suf_);";
			put "  Here:split_data = Here:data^whichgp^&_suf_ << Split(";
		%end;
		%else %do;
			put "  NumCol = NCol(Here:data^whichgp^);";
			put "  Here:split_data = Here:data^whichgp^ << Split(";
		%end;		
		put "   Split By( :Trait ),";
		put "   Split( nsigcol ),";
		
		%if %symexist(IntEff)=0 %then %do;
			%let IntEff=;
		%end;
		put "   Remaining Columns( Keep (:&AnnoGroup, :&AnnoLoc, :&AnnoLabel, &JNonTraitByVars) ),";
		put "   Output Table( _currtestname ||"" Venn"" )";
		put "  );";
		%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
			put "  Here:data^whichgp^&_suf_ << Delete Columns(nsigcol);";
		%end;
		%else %do;
			put "  Here:data^whichgp^ << Delete Columns(nsigcol);";
		%end;
		put " ]\""); //end EvalInsert ";
		put " Eval(parse(venn2ei));";
		put " Here:split_data<< Minimize Window;";
		put " Current Data Table(Here:split_data);";

		put " InsertInto(Here:DataList,{{""Here:split_data""}});";
		put " NumColNew = NCol(Here:split_data);";
		%if %length(&IntEff)>0 %then %let TotByVars = %eval(&NByVars + 1);
		%else %let TotByVars = &NByVars;
		%if &ProcessName=%str(ImputedSNPTraitAssociation) %then %let TotByVars = %eval(&TotByVars - 1);
		
		%if &NTrait le 5 %then %do;
			put " allcols = Here:split_data<<GetColumnNames(String);";
			put " jg:col_list = allcols[%eval(4+&TotByVars)::NumColNew];";
		
			/* use NumCol-1 instead of NumCol + 1 since the Sig column hasn't been saved to table and Trait was removed during Split
			put " for(i=4, i<=NumColNew, i++,";
			put "   InsertInto(col_list,char(ColumnName(i)));";
			put " );";*/
		%end;
		%else %do;
			put " allcols = Here:split_data<<GetColumnNames(String);";
			put " jg:col_list = allcols[%eval(4+&TotByVars)::NumColNew];";		
			put " Here:Orig_col_list = {}; ";
			put " for(i=%eval(4+&TotByVars), i<=NumColNew, i++,";
			put "   InsertInto(Here:orig_col_list,char(ColumnName(i)));";
			put " );";		
			put " dlg = New Window(""Select the Traits to Compare"",<<modal,";
			put "   vlistbox(";
			put "    clistb=listBox(Here:orig_col_list),";
			put "		TextBox("" ""),";
			put "		pacb=CheckBox("" Proportional Areas""),";
			put "		TextBox("" ""),";
			put "    buttonbox(""OK"", clist=clistb<<Get Selected; jg:areaprop = pacb<<Get;)";
			put "	  )";
			put "	);";
			put " jg:col_list = clist;";
		%end;
		
		put " jg:freq_list = {};";
		put " jg:ExtraTitle = "": ""||_currtestname || "" Test"";";
		/*S1149704 sbrlsi %if &NumGps > 1 %then %do;*/
		%if &NumGps > 1 or %upcase(&ProcessName) = QKMIXEDMODEL %then %do; 
			put " jg:ExtraTitle = jg:ExtraTitle || "" for &AnnoGroup "" || aglist[whichgp+1];";
		%end;
		
		%if &OrigNbyval > 1 %then %do;
			put "split_data << Subset(By(&JNonTraitByVars), Output Table Name(_currtestname ||"" VennSub ""||char(Here:dt)));";
			%do b=1 %to &OrigNByVal;
				put " Here:currby = ""&b"";";
				put " jg:ExtraTitle = "": ""||_currtestname || "" Test"";";
				%if &NumGps > 1 or %upcase(&ProcessName) = QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
				%*if &NumGps > 1 %then %do;
					put " jg:ExtraTitle = jg:ExtraTitle || "" for &AnnoGroup "" || aglist[whichgp+1];";
				%end;				

				put "Current Data Table(Data Table(_currtestname ||"" VennSub ""||char(Here:dt)||"" &b""));";
				%do bv=1 %to &TotByVars;
					put "  jg:ExtraTitle = jg:ExtraTitle || "" %scan(&NonTraitByVars,&bv)="" || (Current Data Table()<<Get Table Variable(""%scan(&NonTraitByVars,&bv)""));";
				%end;

				put " wait(0.1);";
				put " jg:vdt = Data Table(_currtestname ||"" VennSub ""||char(Here:dt)||"" &b"");";
				put " jg:LSindex = 2;";
				put " include(""$LS_HOME/JSLFiles/VennScript.jsl"", << new context, << ""Names Default To Here"");";
				put " Current Data Table() << Minimize Window;";
				put " wait(0.1);";
				put " InsertInto(Here:TabWinList,0);";
			   put " Here:TabWinList[Here:nwcounter]=Window(""Venn Diagram""||jg:ExtraTitle);";
			   put "	Here:nwcounter++;";
			%end;	
		%end;
		%else %do;
			put " jg:vdt = Here:split_data;";
			put " jg:LSindex = 2;";			
			put " include(""$LS_HOME/JSLFiles/VennScript.jsl"", << new context, << ""Names Default To Here"");";		
			put " InsertInto(Here:TabWinList,0);";
			put " Here:TabWinList[Here:nwcounter]=Window(""Venn Diagram""||jg:ExtraTitle);";
			put "	Here:nwcounter++;";			
		%end;

		put ");";
	
	%end;
	%end;
	/*** END Venn drill-down function ***/
		
	/*** Forest Plot drill-down for Meta-Analysis ***/
	%if &_forestplot=1 %then %do;
		%let studylab= Study Name;
		%let refvalue= 0;
		%if %symexist(OddsRatioVar) %then %do;
			%if %upcase(&OddsRatioVar) = YES %then %do;
				%let refvalue= 1;
			%end;
		%end;

		put "Here:ForestPlotDD = Function({},{default local},";
		
		put "if(Contains(Current Data Table()<<Get Column Names(String),""&AnnoLabel"")==0,";
		put " throw(""You must select a different tab, e.g. the Summary Chart tab cannot be the current tab."");";
		put ");";		
		
		put "include(""$LS_HOME/JSLFiles/ForestPlot.jsl"");";			
		
		put "selrows = Current Data Table()<<Get Selected Rows();";
		put "if(Nrows(selrows)==0,throw(""You must select points in the plot before clicking on the Forest Plot button""));";
		
		put "selSNPs = Column(""&AnnoLabel"")[selrows];";
		put "if(islist(selSNPs),nSNPs = N Items(selSNPs), nSNPs= Nrows(selSNPs));";
		put "cip = 100*(1-&alpha);";
		
		put "selSNPSc = char(selSNPS);";
		put "selSNPsp = Munger(selSNPsc,2,length(selSNPsc)-2);";

		put 'SASCode = EvalInsert("\[';
			put "libname outlib ""&OutPath"";";

			put "data sub;";
			put "   set OutLib.&_OutDataName;";
			put "   where &AnnoLabel in (^selSNPsp^);";
			put "run;";
			
			put "proc sort data=sub;";
			put " by &AnnoLabel;";
			put "run;";

			put "proc transpose data=sub(rename=(&EffectVar.1-&EffectVar.&ndata=Study1-Study&ndata)) out=sub1(drop=_label_) name=_Study_ prefix=Effect_;";
			put "  var Study1-Study&ndata;";
			put "  by &AnnoLabel;";
			put "run;";

			put "proc transpose data=sub(rename=(&StdErrVar.1-&StdErrVar.&ndata=Study1-Study&ndata)) out=sub2(drop=_label_) name=_Study_ prefix=SE_;";
			put "  var Study1-Study&ndata;";
			put "  by &AnnoLabel;";
			put "run;";		
			
			put "data overall;";
			put " length _Study_ $32;";			
			put " set sub(keep=&AnnoLabel OverallEffect OverallSE);";
			put " _Study_=""_Overall_"";";
			put " rename OverallEffect = &EffectVar OverallSE = SE_1;";
			put "run;";

			put "data fp;";
			put " length _Study_ $32;";
			put " set sub1; set sub2;";
			put " if mod(_n_,&NData)=1 then _Study_ = ""&InData1"";";
			%do w=2 %to &NData;
				put " else if mod(_n_,&NData)=mod(&w,&NData) then _Study_ = ""&&InData&w"";";
			%end;
			put " rename Effect_1 = &EffectVar;";
			put "run;";
			
			put "data fp;";
			put " set overall fp;";
			put " lci = &EffectVar + probit(&alpha/2)*SE_1;";
			put " uci = &EffectVar + probit(1-&alpha/2)*SE_1;";
			put " wt = 1/SE_1**2;";
			put " label _study_=""&studylab"";";
			put " studynum = _n_;";
			put "	rename SE_1 = Standard_Error lci = Lower_Confidence_Bound uci=Upper_Confidence_Bound wt = Weight;";
			put "run;";		
			
			put "proc sort data=fp out=outlib.&GWASMA_OutDataNameFP(drop=studynum);";
			put " by &AnnoLabel Studynum;";
			put "run;";
			
		put "]\"");";
		put "serveroutpath = ""&OutPath"";";
		put "jg:CheckSASConnection;";
		put "SASSubmit(SASCode, NoOutputWindow);";
		put "if(jg:sasserver !=""Local"",";
		put "  jg:SASGetFile2(serveroutpath||""&GWASMA_OutDataNameFP""||"".sas7bdat"",""&clientoutpath""||""&GWASMA_OutDataNameFP""||"".sas7bdat"");";
		put ");";
		put "jg:SASLog=SASGetLog();";	
		put "Here:fst_data=open(""&ClientOutPath.&GWASMA_OutDataNameFP..sas7bdat"", invisible);";	
		put "if(nSNPs>1,";
		put "  pb = New Window(""JMP Life Sciences Message"",Textbox(""Please wait while the forest plots are being generated.""));";
		put "	 for(s=1,s<=nSNPs,s++,";
		put "    try(Close(Data Table(""_fp ""||char(s)), nosave));";
		put "  );";
		put "  Here:fst_data<<Subset(By(:Name(""&AnnoLabel"")),Keep by columns( 1 ), Selected rows(0),Output table name(""_fp""),invisible);";
		put "  wait(2);";
		put "  fpnw = New Window( ""Meta-Analysis Forest Plots"",";
		put "	   Here:rrvlb1 = V List Box();";
		put "  );";
		put "  currtime = today();";
		put "	 for(s=1,s<=nSNPs,s++,";
		put "    //wait(1);";
		put "    cdt = Data Table(""_fp ""||char(s));";
		put "    Current Data Table(cdt);";
		put "    currsnp = cdt << Get Table Variable(""&AnnoLabel"") ;";
		put "    Here:CreateForestPlot(""&studylab"",""&EffectVar"",""Lower_Confidence_Bound"",""Upper_Confidence_Bound"",""Weight"",cip,""&AnnoLabel ""||currsnp, char(currtime+s), 1, &refvalue);";
		put "    InsertInto(Here:TabWinList,0);";
		put "    Here:TabWinList[Here:nwcounter]=Data Table(""_fp ""||char(s));";
		put "    Here:nwcounter++; ";		
		put "    //cdt << Show window(0);";
		put "  );";
		put "  pb << Close Window();";
		put ", //else 1 snp";
		put "  fpnw = New Window( ""Meta-Analysis Forest Plot"",";
		put "  	 Here:rrvlb1 = V List Box();";
		put "  );";		
		put "  Here:CreateForestPlot(""&studylab"",""&EffectVar"",""Lower_Confidence_Bound"",""Upper_Confidence_Bound"",""Weight"",cip,""&AnnoLabel ""||char(parse(selSNPsp)),char(today()), 1, &refvalue);";
		put ");";
		put "jg:SetIcon(fpnw,""GWASMetaAnalysis"");";
		put "InsertInto(Here:TabWinList,0);";
		put "Here:TabWinList[Here:nwcounter]=fpnw;";
		put "Here:nwcounter++; ";
		put ");  //end func";

	%end;
	/*** END Forest Plot drill-down for Meta-Analysis ***/

	%do i=0 %to &NumGps;
		%if &i=1 and &NumGps = 1 and &_summchart %then %do;
			%if %upcase(&ProcessName) = QKMIXEDMODEL %then %goto outofloop0; /*S1149704 sbrlsi*/ 
			%else %goto outofloop;
		%end;
		%*if &i=1 and &NumGps = 1 and &_summchart %then %goto outofloop; 
		%if &i>0 %then %do;
			put " _ag_ = ""&&Groupn&i"";";		
			%if &_summchart and %upcase(&ProcessName) ne QTLSINGLEMARKERANALYSIS %then %do;
				put "InsertInto(Here:TabButtonList,""&AnnoGroup "" || _ag_ || "" Results"", Loc(TabButtonList,""Summary Chart"")+2+&i);";		
				put "InsertInto(Here:TabOpenList,0,Loc(TabButtonList,""Summary Chart"")+2+&i);";
			%end;
			%let tNumGps = 1;
		%end;
		%else %if %upcase(&ProcessName) ne METAANALYSIS and &_pvalplots %then %do;
			%let tNumGps = &NumGps; 
			%if &NumGps = 1 %then %do;
				%if %upcase(&ProcessName) ^= QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
					put "InsertInto(TabOpenList,1);";
				%end;
				%else %do; 
					put "Here:TabsSectionFlag = 1;"; /*S1149704 sbrlsi*/
					put "InsertInto(TabOpenList,0);"; /*S1149704 sbrlsi*/				
				%end; 
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					put "InsertInto(Here:TabButtonList,""P-Value Plot &HaploTest"");";
				%end;
				%else %do;
					put "InsertInto(Here:TabButtonList,""P-Value Plot"");";	
				%end;
			%end;
			%else %do;
				%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
					put "InsertInto(TabOpenList,1);";
				%end;
				%else %do;
					put "Here:TabsSectionFlag = 1;";
					put "InsertInto(TabOpenList,0);";
				%end;
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					put "InsertInto(Here:TabButtonList,""All P-Value Plot &HaploTest"");";						
				%end;
				%else %do;
					put "InsertInto(Here:TabButtonList,""All P-Value Plots"");";
				%end;
			%end;
		%end;
		%else %let tNumGps = &NumGps;

		put '//data<<Minimize Window;';
		put "dt = Today();";					
		
		%if %symexist(TrackPath)=0 %then %let TrackPath=;
		
		%if &i=0 %then %do;
		
			%if %length(&AnnoInData) or %length(&AnnoAccession_All) or %length(&TrackPath) %then %do;
				%include "&MacroPath./WebSearchJSL.sas";
				%if &_pve=1 %then %WebSearchJSL(Ds=InLib.&InData,clientPath=,MakeWindow=0);
				%else %WebSearchJSL(Ds=,clientPath=,MakeWindow=0);
			%end;

			/* web and drill down buttons */
			%if %length(&AnnoAccession_All) or %length(&TrackPath) %then %do;

				put "Here:ActionButtonDesc = {""Select Point in Graphs or in Linked Tables, then Click a Button to View Annotation""};";

				%if &_pve=1 %then %do;
					%WebButtons;
				%end;

				%WebButtons_individual;
	
				%if %symexist(NTrait) %then %do;
				%if &NTrait > 1 /*and %quote(&ByVars) = Trait */%then %do;		
					put "InsertInto(Here:ActionButtonDesc,""View Venn Diagram of Significant &MUnit by Trait for the Test Below"");";
					put "InsertInto(Here:ActionButtonList, {{""%scan(&TestNames,1)"" " @;
					%do p=2 %to &NPvars; 
						put ", ""%scan(&TestNames,&p)"" " @;
					%end;
					put "}});";
					put "InsertInto(Here:ActionCodeList, {{Here:VennCompTraits&_suf_(""%scan(&PVars,1)"",""%scan(&TestNames,1)"")" @;
					%do p=2 %to &NPvars;
						put ", Here:VennCompTraits&_suf_(""%scan(&PVars,&p)"",""%scan(&TestNames,&p)"")" @;
					%end;
					put "}});";
					put "include(""$LS_HOME/ProcessLibrary/VennDiagram.jsl"");";		
				%end;
				%end;		
			%end;
			%else %do;
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					%if %symexist(NTrait) %then %do;
						%if &NTrait > 1 /*and %quote(&ByVars) = Trait */ %then %do;	
							%*put TestNames= &TestNames;
							%let TestNamesNew= ;
							%if %index(&TestNames, ChiSq) %then %do;
								%do p=1 %to &NPvars; 
									%let TestNamesNew= &TestNamesNew %sysfunc(substr(%scan(&TestNames,&p), 6));
								%end;
							%end;	
							%else %do;
								%do p=1 %to &NPvars; 
									%let TestNamesNew= &TestNamesNew %sysfunc(substr(%scan(&TestNames,&p), 2));
								%end;
							%end;	
							%let TestNames= &TestNamesNew;
							%*put TestNames= &TestNames;
							put "InsertInto(Here:ActionButtonDesc, {""View Venn Diagram of Significant &MUnit by Trait for the Test Below""});";
							put "InsertInto(Here:ActionButtonList, {{ ""%scan(&TestNames,1)"" " @;
							%do p=2 %to &NPvars; 
								put ", ""%scan(&TestNames,&p)"" " @;
							%end;
							put "}});";
							put "InsertInto(Here:ActionCodeList, {{Here:VennCompTraits&_suf_(""%scan(&PVars,1)"",""%scan(&TestNames,1)"")" @;
							%do p=2 %to &NPvars;
								put ", Here:VennCompTraits&_suf_(""%scan(&PVars,&p)"",""%scan(&TestNames,&p)"")" @;
							%end;
							put "}});";
							put "include(""$LS_HOME/ProcessLibrary/VennDiagram.jsl"");";
						%end;
					%end;
				%end;
				%else %do;
					%if %symexist(NTrait) %then %do;
						%if &NTrait > 1 /*and %quote(&ByVars) = Trait */ %then %do;		
							put "Here:ActionButtonDesc= {""View Venn Diagram of Significant &MUnit by Trait for the Test Below""};";
							put "Here:ActionButtonList= {{ ""%scan(&TestNames,1)"" " @;
							%do p=2 %to &NPvars; 
								put ", ""%scan(&TestNames,&p)"" " @;
							%end;
							put "}};";
							put "Here:ActionCodeList= {{Here:VennCompTraits&_suf_(""%scan(&PVars,1)"",""%scan(&TestNames,1)"")" @;
							%do p=2 %to &NPvars;
								put ", Here:VennCompTraits&_suf_(""%scan(&PVars,&p)"",""%scan(&TestNames,&p)"")" @;
							%end;
							put "}};";
							put "include(""$LS_HOME/ProcessLibrary/VennDiagram.jsl"");";		
						%end;
					%end;
				%end;				
			%end;

			%if &_ForestPlot=1 %then %do;
			
				put "if(isEmpty(ActionButtonList) | NItems(ActionButtonList)==0,";
		
				put "   Here:ActionButtonList = {{""Forest Plot""}};";
				put "   Here:ActionCodeList = {{""""}};";
				put "   Here:ActionCodeList[1][1]=expr(Here:ForestPlotDD());";
				put "   Here:ActionButtonDesc = {""View Forest Plot for Selected Values of &AnnoLabel""};";		
				put ", //else";
				put "   InsertInto(Here:ActionButtonDesc,""View Forest Plot for Selected Values of &AnnoLabel"");";
				put "   InsertInto(Here:ActionButtonList, {{""Forest Plot""}}); " ;					
				put "   InsertInto(Here:ActionCodeList, {{Here:ForestPlotDD()}}); " ;	
				put ");";
			%end;
			
			%if &_subsetdd %then %do;
				%if %symexist(r) %then %do;
					put "    Here:DefAlpha = ""&r"";";
				%end;
				%else %do;
					put "    Here:DefAlpha = ""0"";";
					%let ltgt = %str(<);
					%let PVar1 = ProbHWE;
				%end;				
				/* Define function for launching subset & reorder, both from all APs and for filtering from Marker Properties */
				put "    Here:launchSubRe = Function({filter},{default local},";
			
				put "      if(filter,";
				put "       if(length(trim(maftb<<GetText))==0, mafco = ""0"", mafco=maftb<<GetText);";
				put "       if(length(trim(nmgtb<<GetText))==0, nmgco = ""1"", nmgco=nmgtb<<GetText);";
				%if %length(&Conv) %then %do;
					put "       if(length(trim(hwetb<<GetText))==0, hweco = ""100"", hweco=hwetb<<GetText);";
				%end;
				%else %do;
					put "       if(length(trim(hwetb<<GetText))==0, hweco = ""0"", hweco=hwetb<<GetText);";				
				%end;
				
				put "       FilterExpr = ""MAF gt ""||mafco||"" and propmissing le ""||nmgco " @;
				put "        ||"" and ""||hweco||"" &ltgt &PVar1""; "; 

				put "       if(length(trim(nmgitb<<GetText))==0, nmgitco = ""1"", nmgitco=nmgitb<<GetText);";
				put "       minpercent = char(1-num(nmgitco));";
				
				put "     , // else filter=0";
				put "       if(Contains(Current Data Table()<<Get Column Names(String),""&AnnoLabel"")==0,";
				put "        throw(""You must select a different tab, e.g. the Summary Chart tab cannot be the current tab."");";
				put "       );";
				
				put "       minpercent = ""0"";";
				put "       nsel=0;";
				put "       WhereClause = ""&AnnoLabel in ("";";
				put "       selsub = {};";
				put "       scol = :Name(""&AnnoLabel"");";
				put "       for each row(";
				put "       	if(selected(rowstate())==1, ";
				put "       		if(!Contains(selsub,:Name(""&AnnoLabel"")[]),";
				put "       			nsel++;";
				put "       			InsertInto(selsub,:Name(""&AnnoLabel"")[]);";
				put "       			if(nsel>1,";
				put "       				WhereClause ||= "", "" ";
				put "       			);";
				put "       			WhereClause = WhereClause||""\[""]\""||:Name(""&AnnoLabel"")[]||""\[""]\"";";
				put "       		)";
				put "       	)";
				put "       );";
				
				put "       WhereClause ||= "")"";";
				put "       FilterExpr = WhereClause;";
				put "       if(nsel==0,throw(""There are no markers selected.""));";
				
				put "     ); //end filter";

				put "			if(host is(Windows),";
				put "				Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
				put "				Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
				put "	";

				/* Set parameters for Subset and Reorder */
				put "			jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";	
				%if %symexist(Recode)=0 %then %let Recode=;
				%if %upcase(&Recode)=YES %then %do;
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""&ClientOutPath.&NumGenoDataName..sas7bdat"");";
					put "			jg:SubsetandReorderGeneticData_updateDeps;";
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""numgeno:"");";	
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""Numeric Genotypes"");";
				%end;
				%else %do;
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""%sysfunc(pathname(Inlib))&_slash_.&InData..sas7bdat"");";
					put "			jg:SubsetandReorderGeneticData_updateDeps;";
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AlleleVars""),""&OldAlleleVars"");";	
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&ListMarkerVars"");";	
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""&MarkerFmt"");";
					%if %symexist(GenoDelim) %then %do;
						%if %quote(&GenoDelim) = %str( ) %then %do;
							put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""GenoDelim""),""%nrstr(%str( ))"");";
						%end;
						%else %do;
							put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""GenoDelim""),""&GenoDelim"");";					
						%end;
					%end;
					put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""WhereStmt""),""\[%nrbquote(&WhereStmt)]\"");";						
				%end;
				put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),minpercent);";	

				put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
				put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""&ClientOutPath.&_OutDataName..sas7bdat"");";					
				put "			jg:SubsetandReorderGeneticData_updateDeps;";				
				put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&AnnoMName"");";	
				put "			jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoWhereStmt""),FilterExpr);";	
				put "       jg:runProcess(2,""SubsetandReorderGeneticData"");";
				put " );";			
			
			 	/* End launchsubre function */			
			
				put "if(isEmpty(ActionButtonList) | NItems(ActionButtonList)==0,";
		
				put "   Here:ActionButtonList = {{""Create Subset Genotype and Annotation Data Sets""}};";
				put "   Here:ActionCodeList = {{""""}};";
				put "   Here:ActionCodeList[1][1]=expr(Here:launchSubRe(0));";
				put "   Here:ActionButtonDesc = {""Select Markers in Graphs or Linked Tables""};";		
				put ", //else";
				put "   ArryIndex=NItems(Here:ActionButtonList)+1;";
				put "   InsertInto(Here:ActionButtonList,0);";
				put "   InsertInto(Here:ActionCodeList,{{""""}});";
				put "   InsertInto(Here:ActionButtonDesc,0);";

				put "   Here:ActionButtonList[ArryIndex]={""Create Subset Genotype and Annotation Data Sets""};";
				put "   Here:ActionCodeList[ArryIndex][1]=expr(Here:launchSubRe(0));";
				put "   Here:ActionButtonDesc[ArryIndex]=""Select Markers in Graphs or Linked Tables"";";
				put ");";	
			%end;			
			%if &_plotinput %then %do;
			/* Plot input data drill-down */
				%let annovarlist = %bquote("&AnnoLabel", "&AnnoGroup", "&AnnoLoc");
			/*
				%let _danno_ = %sysfunc(open(Outlib.&_OutDataName));
				%let nv2 = %sysfunc(attrn(&_danno_,NVARS));
				%do v=1 %to &nv2;
					%let vname_&v=%sysfunc(varname(&_danno_,&v));
					%if (%length(&&vname_&v) > 30) %then %let vname_&v = %substr(&&vname_&v,1,30);
					%if %lowcase(&&vname_&v) ne %str(_order_) and %lowcase(&&vname_&v) ne %lowcase(&AnnoLabel) %then 
						%let annovarlist = %bquote(&annovarlist, "&&vname_&v");
				%end;
				%let rc=%sysfunc(close(&_danno_));			
				*/
				
				put "Here:InPath = ""%sysfunc(pathname(Inlib))"";";
				put "Here:InData = ""&InData"";";
				put "Here:PWhereStmt = ""\[%nrbquote(&WhereStmt)]\"";";
				put "Here:OutPath= ""&OutPath"";";
				put "Here:ClientOutPath=""&ClientOutPath"";";
				%if %symexist(CensorVar)=0 %then %let CensorVar=;
				put "Here:CensorVar = ""&CensorVar"";";		
				put "Here:AnnoLab = ""&AnnoLabel"";";
				put "Here:ColList = {&AnnoVarlist};";
				put "Here:AnnoAcc = ""&AnnoAccession"";";
				put "Here:ByVars = ""&ByVars"";";
				put "Here:TraitType = ""&TraitType"";";
				%if %symexist(IntEff) %then %do;
					put "Here:IntEff = ""&IntEff"";";
				%end;
				%else %do;
					put "Here:IntEff = """";";
				%end;

				/* Need this for Case-Control with 1 trait */
				put "Here:TVar = ""&TraitVar"";";
				
				%if %length(%scan(&PVars,2)) %then %do;
					put "Here:PValVars = {""%scan(&Pvars,1)"",""%scan(&Pvars,2)""};";
				%end;
				%else %do;
					put "Here:PValVars = {""%scan(&Pvars,1)""};";
				%end;					
				
				put " include(""$GENOMICS_HOME\JSLFiles\PlotTraitGenotype.JSL"");";
				
				put "if(isEmpty(ActionButtonList) | NItems(ActionButtonList)==0,";
		
				put "   Here:ActionButtonList = {{""Plot Trait by Genotype""}}; ";

				put "   Here:ActionCodeList = {{""""}};";
				put "   Here:ActionCodeList[1][1]=expr(Here:fmdd());";
				put "   Here:ActionButtonDesc = {""Select Markers in Graphs or Linked Tables""};";		
				put ", //else";
				put "   if(Contains(Here:ActionButtonDesc,""Select Markers in Graphs or Linked Tables""),";
				put "     ArryIndex = Contains(Here:ActionButtonDesc,""Select Markers in Graphs or Linked Tables"");";
				put "     InsertInto(Here:ActionButtonList[ArryIndex],""Plot Trait by Genotype"" );" ;
				put "     InsertINto(Here:ActionCodeList[ArryIndex],expr(Here:fmdd()));";	
				put "     jg:ActionButtonsperRow = 1;";
				put "   ,//else ";
				put "    ArryIndex=NItems(Here:ActionButtonList)+1;";
				put "    InsertInto(Here:ActionButtonList,0);";
				put "    InsertInto(Here:ActionCodeList,{{""""}});";
				put "    InsertInto(Here:ActionButtonDesc,0);";

				put "    Here:ActionButtonList[ArryIndex]={""Plot Trait by Genotype"" };" ;
				put "    Here:ActionCodeList[ArryIndex][1]=expr(Here:fmdd());";
				put "    Here:ActionButtonDesc[ArryIndex]=""Select Markers in Graphs or Linked Tables"";";
				put "   );";
				put ");";		
		
			%end;
		
			%if %symexist(OutputSurvFunc) %then %do;
				%if %upcase(&OutputSurvFunc)=YES %then %do;
				/* Survival curve drill-down */

					put "Here:SurvCurveFunc = Function({testtype},{default local},";

					put "    if(Contains(Current Data Table()<<Get Column Names(String),""&AnnoLabel"")==0,";
					put "     throw(""You must select a different tab, e.g. the Summary Chart tab cannot be the current tab."");";
					put "    );";				
					put "    nsel=0;";
					put "    selsub = {};";
					put "    scol = :Name(""&AnnoLabel"");";
					put "    for each row(";
					put "    	if(selected(rowstate())==1, ";
					put "      		if(!Contains(selsub,:Name(""&AnnoLabel"")[]),";
					put "      			nsel++;";
					put "      			InsertInto(selsub,:Name(""&AnnoLabel"")[]);";
					put "      		)";
					put "      	)";
					put "     );";

					put "     if(nsel==0,throw(""There are no markers selected.""));";
					put "     if(Contains(testtype,""Trend""),";
					put "      Here:datasv = open(""&ClientOutPath.&SNPTraitAssoc_sftName..sas7bdat"");";
					put "     , //else genotype";
					put "      Here:datasv = open(""&ClientOutPath.&SNPTraitAssoc_sfgName..sas7bdat"");";
					put "     );";		
					put "     InsertInto(Here:TabWinList,0);";
					put "     Here:TabWinList[Here:nwcounter]=Here:datasv;";
					put "     Here:nwcounter++; ";

					put "     Here:datasv << Select Where(Contains(selsub,:Name(""&AnnoLabel"")));";
					put "     Here:datasvs = Here:datasv << Subset(Output table name(""Survival Function Data (""||testtype||"")""));";
					put "     Here:sop = Graph Builder(";
					put "        Show Control Panel( 0 ),";
					put "        Show Legend( 1 ),";
					put "		Graph Spacing( 3 ),";
					put "		Spacing Borders( 1 ),";					
					put "        Variables(X(:Name(""&TraitVar"")),Y(:SurvivalFunctionEst),Overlay(:Genotype)),";
					put "        Elements( Line( X, Y, Legend( 1 ), Connection( ""Step"" ) ) ),";
					put "        By(:Name(""&AnnoLabel"")),";
					put "     );";					
					put "     Here:datasv << Close Window;";
					put "     Here:datasvs << Minimize Window;";				
					put "     InsertInto(Here:TabWinList,0);";
					put "     Here:TabWinList[Here:nwcounter]=Here:sop;";
					put "     Here:nwcounter++; ";				
					put "); //end func ";

					put "if(isEmpty(ActionButtonList) | NItems(ActionButtonList)==0,";

					put "   Here:ActionButtonList = {{""%scan(&TestNames,1)"" "@;
					%if %length(%scan(&TestNames,2)) %then %do;
						put ",""%scan(&TestNames,2)"" "@;
					%end;
					put "}};";
					put "   Here:ActionCodeList = {{"""",""""}};";
					put "   Here:ActionCodeList[1][1]=expr(Here:SurvCurveFunc(""%scan(&TestNames,1)""));";
					%if %length(%scan(&TestNames,2)) %then %do;
						put "   Here:ActionCodeList[1][2]=expr(Here:SurvCurveFunc(""%scan(&TestNames,2)""));";
					%end;				
					put "   Here:ActionButtonDesc = {""Select Markers in Graphs or Linked Tables, then Click to View Survival Curves""};";		
					put ", //else";
					put "   ArryIndex=NItems(Here:ActionButtonList)+1;";
					put "   InsertInto(Here:ActionButtonList,0);";
					put "   InsertInto(Here:ActionCodeList,{{""""},{""""}});";
					put "   InsertInto(Here:ActionButtonDesc,0);";

					put "   Here:ActionButtonList[ArryIndex]={""%scan(&TestNames,1)"" " @;
					%if %length(%scan(&TestNames,2)) %then %do;
						put ",""%scan(&TestNames,2)"" " @;
					%end;
					put "};";
					put "   Here:ActionCodeList[ArryIndex][1]=expr(Here:SurvCurveFunc(""%scan(&TestNames,1)""));";
					%if %length(%scan(&TestNames,2)) %then %do;
						put "   Here:ActionCodeList[ArryIndex][2]=expr(Here:SurvCurveFunc(""%scan(&TestNames,2)""));";
					%end;
					put "   Here:ActionButtonDesc[ArryIndex]=""Select Markers in Graphs or Linked Tables, then Click to View Survival Curves"";";
					put ");";			
				%end; 
			%end;
		%end;
		
		%if &i>0 and &_summchart %then %do;
			%if &_summchart and %upcase(&ProcessName) ne QTLSINGLEMARKERANALYSIS %then %do;
				put "InsertInto(DataList,{{""Here:data&i&_suf_""}},Loc(TabButtonList,""Summary Chart"")+2+&i);";
			%end;
		%end;
		%else %if %upcase(&ProcessName) ne METAANALYSIS and &_pvalplots %then %do;
			put "InsertInto(DataList,{{""Here:data&i&_suf_""}});";
		%end;
		
		/* put "print(""creating Here:cc&i&_suf_"");"; */
		put "Here:cc&i&_suf_=function({},{default local}," ;
		/*********** kemicl and sbrlsi Add message about nonconvergence from Logistic ************/
		%if &_pvalplots and &NumGps = 1 %then %do;
			%if %symexist(Flag_NonConvergenceG) or %symexist(Flag_NonConvergenceT) %then %do;
				%if (&Flag_NonConvergenceG = 1 or &Flag_NonConvergenceT=1) %then %do;
					put "Here:_nw00_= New Window(""&ProcessName Warning Message"",showmenu(0),showtoolbars(0),";
					put " TextBox(""NOTE: Some Association Tests did not converge. Results shown are based on the last maximum likelihood ""|| ";
					put "			""iteration. Validity of the model fits is questionable. \!N\!N"",";
					put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400)";
					put " ),";
					put " TextBox("" A data set containing the results for Non-convergent Tests is included in the Output Data."", ";
					put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400)";
					put " )";					
					put ");";
					put "if(!IsEmpty(Here:TabWinList),";
					put "    InsertInto(Here:TabWinList,0);";
					put "    Here:TabWinList[Here:nwcounter]=Here:_nw00_;";
					put "    Here:nwcounter++; ";		
					put "	);";
				%end;
			%end;	
		%end;
		put "jg:dt=Today();";
		
		/* expressions to set frame size */
		put "uf&i&_suf_=EvalInsert(""\[";	
		%if %symexist(GenAP) %then %do;
			%if &GenAP=1 %then %do;
				put "hfsize&i&_suf_ = 350;";
				put "vfsize&i&_suf_ = 175;";
			%end;
			%else %do;
				put "hfsize&i&_suf_ = 800;";
				put "vfsize&i&_suf_ = 200;";	
			%end;
		%end;
		%else %do;
			put "hfsize&i&_suf_ = 800;";
			put "vfsize&i&_suf_ = 200;";					
		%end;
		put "Here:update_Frames&i&_suf_ = Expr(";
		put "]\"");";

		%if &i=0 %then %do;
			%let startloop=1;
			%let endloop=&tNumGps;
		%end;
		%else %do;
			%let startloop = &i;
			%let endloop = &i;
		%end;

		%do j=&startloop %to &endloop;

			/* For PValue Browser since don't overlay p-values */			
			put "uf&i&_suf_||=EvalInsert(""\[";
			%if &_overlay=0 %then %do;
				put "  for(_i=1, _i<=&NPVars, _i++,";
				put '   try(';
				put "     rop&i.&j&_suf_[framebox(_i)] << FrameSize(hfsize&i&_suf_,vfsize&i&_suf_);";   
				put '  ));';
			%end;
			%else %do;
				put "  for(_b=1, _b<=&NbyVal, _b++,";
				put '   try(';
				put "     rop&i&j&_suf_[_b][framebox(1)] << FrameSize(hfsize&i&_suf_,vfsize&i&_suf_);";   
				%if &ProcessName = MissingGenotypeSummary %then %do;
					put "     rdiff&i&j&_suf_[_b][framebox(1)] << FrameSize(hfsize&i&_suf_,vfsize&i&_suf_);";
				%end;
				put '));';
			%end;		
			put ']\");';

		%end;
		put "uf&i&_suf_||="");"";"; 
		put "Eval(Parse(uf&i&_suf_));";

		put "uf&i._2&_suf_ = EvalInsert(""\[";
		put "update_HFS_TextEditBox&i&_suf_ = Expr(";
		put "   value = Round(Num(hfsize&i&_suf_),0);";
		put "   HFS_TEB&i&_suf_ << SetText(char(num(value)));";
		put "   HFS_TEB&i&_suf_ << Reshow;";
		put "   hfsize&i&_suf_ = value;";
		put "   update_Frames&i&_suf_;";
		put ");";
		put "update_HFS_SliderBox&i&_suf_ = Expr(";
		put "   hfsize&i&_suf_ = num(HFS_TEB&i&_suf_<<GetText);";
		put "   HFS_SB&i&_suf_ << Reshow;";
		put "   update_HFS_TextEditBox&i&_suf_;";
		put ");";
		put "update_VFS_TextEditBox&i&_suf_ = Expr(";
		put "   value = Round(Num(vfsize&i&_suf_),0);";
		put "   VFS_TEB&i&_suf_ << SetText(char(num(value)));";
		put "   VFS_TEB&i&_suf_ << Reshow;";
		put "   vfsize&i&_suf_ = value;";
		put "   update_Frames&i&_suf_;";
		put ");";
		put "update_VFS_SliderBox&i&_suf_ = Expr(";
		put "   vfsize&i&_suf_ = num(VFS_TEB&i&_suf_<<GetText);";
		put "   VFS_SB&i&_suf_ << Reshow;";
		put "   update_VFS_TextEditBox&i&_suf_;";
		put ");";
		put ']\");';
		put "Eval(Parse(uf&i._2&_suf_));";

		put "vfsteb&i&_suf_ = EvalInsert(""\[";
		put "   VFS_TEB&i&_suf_ = TextEditBox(char(num(vfsize&i&_suf_)),";
		put "      <<Script(update_VFS_SliderBox&i&_suf_)";
		put "   )";
		put "]\"");";					
		put "vfssb&i&_suf_ = EvalInsert(""\[";
		put "   VFS_SB&i&_suf_ = SliderBox(50,800,vfsize&i&_suf_,";
		put "      update_VFS_TextEditBox&i&_suf_";
		put "   )";
		put "]\"");";	
		put "vfssb&i._2&_suf_ = EvalInsert(""\[";
		put "   VFS_SB&i&_suf_ << SetValue(num(vfsize&i&_suf_));";
		put "   VFS_SB&i&_suf_ << SetWidth(120);";
		put "]\"");";	

		put "hfsteb&i&_suf_ = EvalInsert(""\[";
		put "   HFS_TEB&i&_suf_ = TextEditBox(char(num(hfsize&i&_suf_)),";
		put "      <<Script(update_HFS_SliderBox&i&_suf_)";
		put "   )";
		put "]\"");";					
		put "hfssb&i&_suf_ = EvalInsert(""\[";
		put "   HFS_SB&i&_suf_ = SliderBox(100,1200,hfsize&i&_suf_,";
		put "      update_HFS_TextEditBox&i&_suf_";
		put "   )";
		put "]\"");";	
		put "hfssb&i._2&_suf_ = EvalInsert(""\[";
		put "   HFS_SB&i&_suf_ << SetValue(num(hfsize&i&_suf_));";
		put "   HFS_SB&i&_suf_ << SetWidth(120);";
		put "]\"");";
		
		put "if(isempty(jg:drill_down),";
		put " jg:drill_down = 0";
		put ");";
		
		put "if(isempty(Here:data&i&_suf_),";

		put "if(jg:drill_down==0 & isEmpty(Here:fromCCP),";
		%if &i>0 %then %do;
			
			put 'Here:SASCode = EvalInsert("\[';
			put "libname outlib ""&OutPath"";";
			put "data OutLib.&_name._&&SGroupn&i;";
			put "   set OutLib.&_OutDataName;";
			%if (&GpType = N) %then %do;
				%if %length(&&Groupn&i) %then %do;
					put "   if &AnnoGroup = &&Groupn&i;";
				%end;
				%else %do;
					put "   if &AnnoGroup = .;";
				%end;				
			%end;
			%else %do;
				put "   if trimn(&AnnoGroup) = ""&&Groupn&i"";";
			%end;		  
			put "run;";
			put "]\"");";
				
			put "jg:serveroutpath = ""&OutPath"";";
			put "Here:clientoutpath = ""&ClientOutPath"";";
			put "jg:CheckSASConnection;";
			put "SASSubmit(SASCode, NoOutputWindow);";
			put "if(jg:sasserver !=""Local"",";
			put "  jg:SASGetFile2(jg:serveroutpath||""&_name._&&SGroupn&i""||"".sas7bdat"",Here:clientoutpath||""&_name._&&SGroupn&i""||"".sas7bdat"");";
			put ");";
			put "jg:SASLog=SASGetLog();";			
			put "Here:data&i&_suf_=open(""&ClientOutPath.&_name._&&SGroupn&i...sas7bdat"", invisible);";
		%end;
		%else %do;	
			put " Here:data&i&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
		%end;
		put ");";

		%if %upcase(&OnlySig)=YES %then %do;
			%if %length(&Conv)=0 %then %let gtlt=%str(>);
			%else %let gtlt=%str(<);
			put "Here:data&i&_suf_<<select where(" @;
			put ":Name(""&PVar1"") &gtlt &r" @;
			%do p=2 %to &NPVars;
			put "& :Name(""&&PVar&p"") &gtlt &r" @;
			%end;

			put ")<<hide;";
		%end;  
		put "colnames = Here:data&i&_suf_<<GetColumnNames(String);";
		put 'if(Contains(colnames,"_excluded")>0,';
		put "	Here:data&i&_suf_<<select where(:_excluded==1)<<hide);";
		put "Here:data&i&_suf_ << exclude;";
		put "nhidden=0;";
		put "for each row(";
		put "  if(hidden(), nhidden=nhidden+1));";
		put "if(nhidden==nrow(Here:data&i&_suf_),";
		put " throw(""All p-values non-significant in this data set.""));";
		put "Here:data&i&_suf_ << Clear Select;";
		%if %length(&AnnoAccession) %then %do;
			put "column(""&AnnoAccession"")<<data type(character);";
		%end;
		%if %length(&AnnoGroup) %then %do;
			put "column(""&AnnoGroup"")<<data type(character);";
		%end;
		put "column(""&AnnoLoc"")<<data type(numeric);";
		put "column(""&AnnoLabel"") << label(1);";
		%if &_pvalplots %then %do;
			put "column(""CumOrder"") << modeling type(nominal);";
		%end;
		%if &ProcessName=%str(HaplotypeEstimation) or &ProcessName=%str(HaplotypeQKMixedModel) or &ProcessName=%str(HaplotypeTrendRegression) %then %do;
			put "column(""WindowID"")<<label(1);";
			put "column(""WindowID"")<< Set Modeling Type( ""Nominal"" );";
		%end;
		put ", //else";
		put "  currentdatatable(Here:data&i&_suf_);";
		%if %length(&AnnoGroup) %then %do;
			put " column(""&AnnoGroup"")<<data type(character);";
		%end;		
		put "); //end isempty(data&i&_suf_)";
		put "dt = jg:dt;";
		put "jslcode=EvalInsert(""\[";
		put "   try(if(jg:drill_down,currentdatatable(Here:subdata^dt^)));";
		put ']\");';					
		put "Eval(Parse(jslcode));";

		%if %symexist(TrackPath) %then %do;
			%if ^%symexist(TrackAdd) %then %let TrackAdd = 1000;
			%if %length(&TrackPath) %then %do;
				put "show(jg:dt);";
				put "dt = jg:dt;";
				put "include(""$GENOMICS_HOME\JSLFiles\TrackPlot.jsl"");";
				put "if(IsEmpty(trackadd),trackadd = &TrackAdd);";
				put "Here:tfcb&i&_suf_=EvalInsert(""\[";
				put "   jg:int&icb = CheckBox(""Plot Track Data for All Intervening Genes in Gene Tracks"",<<Set(1,1))";
				put ']\");';					
				put "Here:tftb&i&_suf_=EvalInsert(""\[";
				put "   HListBox(";
				put "      TextBox(""Additional Bandwidth on Each Side of Tracks ""),";
				put "      jg:int&itb = TextEditBox(char(trackadd))";
				put "   )";
				put ']\");';					
				put "Here:tfbb&i&_suf_=EvalInsert(""\[";
				put "  HListBox(";
				put "   ButtonBox(""Drill Down on Selected Points and Display Tracks"",";
				put "      anysel = CurrentDataTable()<<Get Selected Rows;";
				put "      if(ncol(anysel)==0,";
				put "         New Window(""Drill Down Message"", showmenu(0),showtoolbars(0), TextBox(""You must select at least one point in the plot.""))";
				put "         , // else";
				put "         if(!length(trim(jg:trackfiles)),";
				put "            New Window(""Drill Down Message"", showmenu(0),showtoolbars(0),";
				put "               TextBox(""You must select at least one track setting file.  Click Change Track Parameters to choose one."")";
				put "            )";
				put "            , // else";
				put " 	        jg:drill_down = 1;";
				put " 	        jg:dt = Today();";					
				put " 	        dt = jg:dt;";					
				put " 	        Eval(Parse(""Here:subdata""||char(dt)||"" = CurrentDataTable()<<Subset(output table name(\!""Subset Selected for Drill Down\!""),invisible);""));";
				put " 	        _ag_ = char(Column(""&AnnoGroup"")[1]);";
				put " 			  nwtrack = New Window(""&AnnoGroup "" || _ag_, showmenu(1),showtoolbars(1),Here:cc&i&_suf_());";
				put " 	        nwtrack << SetWindowIcon(""$LS_HOME/Documentation/Icons/GLM.ico"");";
				put " 	        jslcode = ""nwtrack << OnClose("";";
				put " 	        jslcode ||= ""Try(Close(Here:subdata""||char(dt)||"",NoSave));"";";
				put " 	        for(t=1,t<=NItems(Words(jg:trackfiles,""|"")),t++,";
				put " 	           jslcode ||= ""Try(Close(tdgene""||char(t)||"",NoSave));"";";
				put " 	           jslcode ||= ""Try(Close(tdsnp""||char(t)||"",NoSave));"";";
				put " 	           jslcode ||= ""Try(Close(tdsas""||char(t)||"",NoSave));"";";
				put " 	        );";
				put " 	        jslcode ||= "");"";";
				put " 	        Eval(Parse(jslcode));";
				put " 	        Here:ropfunc&i&_suf_();";
				put " 	        jg:drill_down = 0;";
				put "         )";
				put "      )";
				put "   ),";
				put "   ButtonBox(""Change Track Parameters"",";
				put "      include(""$GENOMICS_HOME/JSLFiles/TrackChangeParms.jsl"");";
				put "   )";
				put "  )";
				put ']\");';					
			%end;
		%end;
		%else %do;
			%let TrackPath = ;
		%end;
		put "VListbox(";
		%if ^%index(&ProcessName,MetaAnalysis) %then %do;
			put "   HListBox(";
			put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
			put "	),";
			put "   HListBox(";
			put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
			put "	),";
		%end;
		%if &ProcessName = MissingGenotypeSummary %then %do;
			%do kk=1 %to &nrectraits;
				put "   HListBox(";
				put "   	Text Box( "" &&TextSummary&kk "", << set wrap(800) ),";
				put "	),";
			%end;	
		%end;
		%if ^%index(&ProcessName,MetaAnalysis) or &ProcessName = MissingGenotypeSummary %then %do;
			put "   HListBox(";
			put "   	Text Box( "" "" ),";
			put "	),";
		%end;
		put "	HListBox(";
		put "   Text Box( "" Vertical Frame Size "" ),";
		put "   HListBox(";
		put "      eval(parse(vfsteb&i&_suf_)),";
		put "      eval(parse(vfssb&i&_suf_)),";
		put "      eval(parse(vfssb&i._2&_suf_))";
		put "   ),";
		put "   Text Box( "" Horizontal Frame Size "" ),";
		put "   HListBox(";
		put "      eval(parse(hfsteb&i&_suf_)),";
		put "      eval(parse(hfssb&i&_suf_)),";
		put "      eval(parse(hfssb&i._2&_suf_))";
		put "   )";
		put " ),";						
		%if %length(&TrackPath) %then %do;
			put "    eval(parse(Here:tfbb&i&_suf_)),";
		%end;
		%if &NumGps=1 and %upcase(&ProcessName) ^= QKMIXEDMODEL %then %do; /*S1149704 sbrlsi %if &NumGps=1 %then %do;*/
			%if %length(&SummaryDescription) %then %do;
				put " Textbox(""""),";				
				put " Textbox(""&SummaryDescription""),";
				put " Textbox(""""),";				
			%end;
		%end;		
		
		/* overlay plots */
		put "  ScrollBox(size(1100,800),";
		%if &tNumGps>1 %then %do;
			
			%if &ProcessName = MarkerProperties %then %do;
				%if &_pvalplots %then %do;
					put '   Outlinebox("Test for HWE",';
				%end;	
			%end;
			put "  hlistbox(";
		%end;
		
		%do j=&startloop %to &endloop;
		
			%if &tNumGps>1 %then %do;
				put "  o=OutlineBox(""&AnnoGroup &&Groupn&j"",";
				%if %quote(&&Groupn&j)=%quote(.) %then %let Groupn&j=%str();
			%end;
			put '   vlistbox(';
			%if &ProcessName = MarkerProperties and &_pvalplots %then %do;
				%if &tnumGps = 1 %then %do;
					put "  Outlinebox(""Test for HWE"",";
				%end;
			%end;
			%if &_pvalplots %then %do;	
				
				put "Here:op&i._&j&_suf_= Graph Builder(";
				put "	Size(500,300),";
				put "	Show Control Panel( 0 ),";
				put "	Show Legend( 1 ),";
				put "	Legend Position( ""Bottom"" ),";				
				put "	Graph Spacing( 3 ),";
				put "	Spacing Borders( 1 ),";
				put "	Variables(";
				put "		X( ""&AnnoLoc"" ),";
				put "		Y(Column(""&&PVar1"")),";
				%do p=2 %to &NPVars;
					put "		Y(Column(""&&PVar&p""), Position(1)),";
				%end;
				put "	),";
				%if %length(&ByVars)>0 %then %do;
					put "           By(&JBy),";
				%end;
				%if &tNumGps>1 %then %do;
					put "	Where(:&AnnoGroup==""&&Groupn&j""),";
				%end;				
				%let l= 20;
				put "	Elements(Points( X"@;
				%do p=1 %to &NPVars;
					put ",Y( &p )"@;
					%let l= %eval(&l + 1);
				%end;
				put "	,Legend( 22 ) )";
				put "	,Line( X"@;
				%do p=1 %to &NPVars;
					put "		,Y( &p )"@;
					%let l= %eval(&l + 1);
				%end;				
				put "	,Legend( 23 ) )";
				put "	),";
				put "	SendToReport(";
				%do p=1 %to &NPVars;	
					%if &p=1 %then %let k=;
					%else %let k= %sysevalf(&p - 1); 

					/*Test*/
					%let testflag= 0;
					%let ytest=;
					%if %sysfunc(find(&&PVar&p, Genotype)) %then %do;
						%let ytest= %str(Genotype Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, Trend)) %then %do;
						%let ytest= %str(Trend Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, Allele)) %then %do;
						%let ytest= %str(Allele Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, Dom)) %then %do;
						%let ytest= %str(Dominant Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, Rec)) %then %do;
						%let ytest= %str(Recessive Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, MAX)) %then %do;
						%let ytest= %str(MAX Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, HWE)) %then %do;
						%let ytest= %str(HWE Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, HapCC)) %then %do;
						%let ytest= %str(Haplotype Case Control Test);
						%let testflag= 1;
					%end;	
					%else %if %sysfunc(find(&&PVar&p, ProbFGlobal)) %then %do;
						%let ytest= %str(Global Haplotype Trend Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbFSingle)) %then %do;
						%let ytest= %str(Single Haplotype Trend Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbMissing)) %then %do;
						%let ytest= %str(Missing Genotype Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbRareVar)) %then %do;
						%let ytest= %str(Rare Variant Test);
						%let testflag= 1;
					%end;	
					%else %if %sysfunc(find(&&PVar&p, ProbOQTDT)) %then %do;
						%let ytest= %str(QTD Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbTDTq5)) %then %do;
						%let ytest= %str(TDTq5 Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbTqps)) %then %do;
						%let ytest= %str(Tqps Test);
						%let testflag= 1;
					%end;					
					%else %if %sysfunc(find(&&PVar&p, ProbSibQTDT)) %then %do;
						%let ytest= %str(SibQTDT Test);
						%let testflag= 1;
					%end;						
					%else %if %sysfunc(find(&&PVar&p, ProbTDT)) %then %do;
						%let ytest= %str(TD Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbSTDT)) %then %do;
						%let ytest= %str(STD Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbSDT)) %then %do;
						%let ytest= %str(SD Test);
						%let testflag= 1;
					%end;					
					%else %if %sysfunc(find(&&PVar&p, ProbRCTDT)) %then %do;
						%let ytest= %str(RCTDT Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbRaoScott_C)) %then %do;
						%let ytest= %str(RaoScott Chi-Square Test);
						%let testflag= 1;
					%end;					
					%else %if %sysfunc(find(&&PVar&p, ProbRaoScott_F)) %then %do;
						%let ytest= %str(RaoScott F Test);
						%let testflag= 1;
					%end;
					%else %if %sysfunc(find(&&PVar&p, ProbMetaIV)) %then %do;
						%let ytest= %str(Inverse Variance Test);
						%let testflag= 1;
					%end;	
					%else %if %sysfunc(find(&&PVar&p, ProbMetaWSS)) %then %do;
						%let ytest= %str(Weighted Square Root Test);
						%let testflag= 1;
					%end;						
					%else %do;
						%let ytest= &&PVar&p;
						%let testflag= 0;
					%end;			

					/*MTC Correction*/
					%let ymtc=;
					
					/*p-value adjustment*/						
					%let yadjs= ;
					%let ybwth= ;

					/*p-value conversion*/
					%let yconv=;
					
					put "			Dispatch(";
					put "			{},";
					put "			""400"",";
					put "			ScaleBox,";
   					put "			{Legend Model(";
   					put "				23,";
   					put "				Level Name(&p,"" "@;
					%if &testflag %then %do;
						put "&ytest "@;
						%if %length(&ymtc) %then %do;
							put "&ymtc "@;
						%end;
						%if %length(&yadjs) %then %do;
							put "&yadjs "@;
						%end;
						%if %length(&yconv) %then %do;
							put "&yconv "@;
						%end;					
					%end;
					%else %do;
						put "&ytest "@;
					%end;
					put " "" ,Item ID( ""&&PVar&p"", 1 )),";
					put "			 )}";
					put "			),";
				%end;
				put "			Dispatch(";
				put "				{},";
				put "				""400"",";
				put "				LegendBox,";
				put "				{Orientation( ""Horizontal"" ), Sides( ""Left"" ), Set Wrap( 1 )}";
				put "			),";				
				%if &NPVars < 2 %then %do;
					put "			Dispatch(";
					put "			{},";
					put "			""400"",";
					put "			LegendBox,";
   					put "			{Legend Position({22, [-1], 23, [0]}), Position({-1,0})}";
   					put "		),";								
				%end;
				put "		Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
				put "	),";
				put "),";	
			%end;
			
			%if &ProcessName = MissingGenotypeSummary %then %do;	
				put "   Here:diffs&i._&j&_suf_ = Graph Builder(";
				put "				Size(500,300),";
				put "  				Show Control Panel( 0 ),";
				put "				Graph Spacing( 3 ),";
				put "				Spacing Borders( 1 ),";
				put "				Variables(";
				put "   			X( :&AnnoLoc),";
				put "   			Y(:Name(""Number of Missing for Trait=&cval1"")),"; 
				put "   			Y(:Name(""Number of Missing for Trait=&cval2""), Position(1), Side(""Right""))),";
				put "            		Elements("; 
				put "            		Line( X, Y( 1 ), Legend( 4 ), Missing Values( ""No Connection"" ) ),"; 
				put "            		Points( X, Y( 1 ), NoLegend( 1 ) ),"; 
				put "            		Line( X, Y( 2 ), Legend( 8 ), Missing Values( ""No Connection"" ) ),"; 
				put "            		Points( X, Y( 2 ), NoLegend( 5 ) )"; 	
				put "            )";
				%if &tNumGps>1 %then %do;
					put "            , Where(:&AnnoGroup==""&&Groupn&j"")";
				%end;
				%if %length(&ByVars)>0 %then %do;
					put "            , By(&JBy)";
				%end; 	
				put "            	,SendToReport(";
				/*
				put "            		Dispatch(";
				put "            			{},";
				put "            			""400"",";
				put "            			ScaleBox,";
				put "            			{Legend Model(";
				put "            				4,";
				put "            				Properties(";
				put "            					0,";
				put "            					{Marker( ""Circle"" )},";
				put "            					Item ID( ""Number of Missing for Trait=1"", 1 )";
				put "            				)";
				put "            			), Legend Model(";
				put "            				1,";
				put "            				Base( 0, 0, 0, Item ID( ""Number of Missing for Trait=1"", 1 ) ),";
				put "            				Properties(";
				put "            					0,";
				put "            					{Marker( ""Circle"" )},";
				put "            					Item ID( ""Number of Missing for Trait=1"", 1 )";
				put "            				)";
				put "            			), Legend Model(";
				put "            				8,";
				put "            				Properties(";
				put "            					0,";
				put "            					{Marker( ""Plus"" )},";
				put "            					Item ID( ""Number of Missing for Trait=0"", 1 )";
				put "            				)";
				put "            			), Legend Model(";
				put "            				5,";
				put "            				Base( 0, 0, 0, Item ID( ""Number of Missing for Trait=0"", 1 ) ),";
				put "            				Properties(";
				put "            					0,";
				put "            					{Marker( ""Plus"" )},";
				put "            					Item ID( ""Number of Missing for Trait=0"", 1 )";
				put "            				)";
				put "            			)}";
				put "            		),";
				*/
				put "            		Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
				put "            		Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Drawing Mode( ""Normal"" )}, {Marker Size( 3 )}),";
	 			put "				Dispatch(";
	 			put "					{},";
	 			put " 					""Number of Missing for Trait=&cval1"", ";
	 			put "					ScaleBox,";
	 			put "					{Format( ""Fixed Dec"", 12, 0 )}";
				put "				),";
	 			put "				Dispatch(";
	 			put "					{},";
	 			put " 					""Number of Missing for Trait=&cval2"", ";
	 			put "					ScaleBox,";
	 			put "					{Format( ""Fixed Dec"", 12, 0 )}";
				put "				),";				
				put "            	)";				
				put "		)," ;					
			%end;  
			%if &ProcessName = MarkerProperties and &_pvalplots %then %do;
				%if &tnumGps = 1 %then %do;
					put "  ),";
				%end;
			%end;
			
		  	%if &ProcessName = MissingGenotypeSummary and &i>0 %then %do;
		  		put "VListbox(";
		  		put 'Distribution(';
		  		put '  Continuous Distribution(';
		  		put "    Column(""Number of Missing for Trait=&cval1""),";
		  		put '    Quantiles(0),';
		  		put '    Moments(0),';
		  		put '    Vertical(0)';
		  		put '  ),';
		  		put '  Continuous Distribution(';
		  		put "    Column(""Number of Missing for Trait=&cval2""),";
		  		put '    Quantiles(0),';
		  		put '    Moments(0),';
		  		put '    Vertical(0)';
		  		put '  ),' @;
		  		%if %length(&ByVars)>0 or &tNumGps>1 %then %do;
		  			put "  By(&JByAnno),";
		  		%end;
		  		put " SendToReport(";
	 			put "	Dispatch(";
	 			put " 		""Number of Missing for Trait=&cval1"", ";
	 			put "		""1"",";
	 			put "		ScaleBox,";
	 			put "		{Format( ""Fixed Dec"", 12, 0 )}";
				put "	),";
	 			put "		Dispatch(";
	 			put " 		""Number of Missing for Trait=&cval2"", ";
	 			put "		""1"",";
	 			put "		ScaleBox,";
	 			put "		{Format( ""Fixed Dec"", 12, 0 )}";
				put "	),";
				put "),";
				put "),";
				put ");";
			%end;
			
			%if &ProcessName = MarkerProperties and &i>0 %then %do;

				put 'Vlistbox(Distribution(';
				put '  Continuous Distribution(';
				put "    Column(""MAF""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';		
				put '    Vertical(0)';
				put '  ),';
				put '  Continuous Distribution(';
				put "    Column(""PropMissing""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';
				put '    Vertical(0)';
				put '  ),' @;
				%if %length(&ByVars)>0 or &tNumGps>1 %then %do;
					put " By(&JByAnno),";
				%end;
				put ' SendToReport(';
				put ' Dispatch(';
				put ' {},';
				put '"MAF",';
				put '  OutlineBox,';
				put '  Set Title("Minor Allele Freq (MAF)")),';
				put ' Dispatch(';
				put ' {},';
				put '"PropMissing",';
				put '  OutlineBox,';
				put '  Set Title("Missing Genotype Proportion"))';		
				put ')';
				put '));';
			%end;
			
			put " ),";
			%if &tNumGps>1 %then %do;
				put " ),";
			%end;

			%if (&NbyVal<2 and &tNumGps>1) %then %do;
				put 'try(o[TextBox(1)] << Set Text("") << Set Font Size(1));';
		  	%end;	
		  	 			
		%end;	
		
		%if &tNumGps>1 %then %do;
			put "), //close hlist box ";
			%if &ProcessName = MarkerProperties %then %do;
				%if &_pvalplots %then %do;
					put ' ), //end outlinebox in subbox';
				%end;	
			%end;
		%end;
		
		%if %symexist(Odds) %then %do;
			%if &Odds=Yes and &i>0 %then %do;

				/* set Frame Size */
				put "oppfsize&i&_suf_ = 800;";
				put "Vlistbox(";
				put '      HListBox(TextBox(" Frame Size for Graphs Below"),';
				put "       fsso&i&_suf_=EvalInsert(""\[ frameSizeSlidero&i&_suf_ = SliderBox(num(200),num(800),oppfsize&i&_suf_,]\"");";

				/* Since i>0, startloop and endloop are the same value, &i */
				%do j=&startloop %to &endloop;
					put "fsso&i&_suf_||=EvalInsert(""\[";
					put "  for(_b=1, _b<=&NbyVal, _b++,";
					put '   try(';
					put "     rop2_&i&j&_suf_[_b][framebox(1)] << FrameSize(oppfsize&i&_suf_,0.35*oppfsize&i&_suf_);";   
					put '));';
					put ']\"); ';					
				%end;

				put "fsso&i&_suf_||="")"";";
				put "Eval(Parse(fsso&i&_suf_));";
				put "   ),";				

				%do j=&startloop %to &endloop;
					put '   vlistbox(' @;
					put "op2&i._&j&_suf_=Graph Builder(";
					put "	Show Control Panel( 0 ),";
					put "	Show Legend( 0 ),";
					put "	Graph Spacing( 3 ),";
					put "	Spacing Borders( 1 ),";					
					put "	Variables(";
					/*
					%if %length(&MTC) %then %do;
						put "  		X( :Log_OddsRatio_Trend), Y( :&Yoddsplot._&MTC)";
					%end;
					%else %do;
						put "  		X( :Log_OddsRatio_Trend), Y( :&Yoddsplot)";
					%end;
					*/
					put "  		X( :Log_OddsRatio_Trend),";
					%let u=1;
					%do p=1 %to &NPVars;
						%if %index(&&PVar&p,ProbTrend) %then %do;
							put "             Y(:&&PVar&p),";
							%let ProbTrendVar&u = &&PVar&p;
							%let u= %sysevalf(&u + 1); 
						%end;	
					%end;	
					%let nvp= %sysevalf(&u - 1); 
					put "           " @;
					%if &NumGps>1 %then %do;
						put "Color(&AnnoGroup)";
					%end;					
					put "),";
					%if %length(&JByOdds)>0 %then %do;
						put "            By(&JByOdds),";
					%end;					
					/*put "		Elements( Line( X, Y, Legend( 4 ) ), Points( X, Y, Legend( 1 ) ) )";*/
					/*put "		Elements( Points( X, Y, Legend( 1 ) ) ),";*/
					%do p=1 %to &nvp;
						put "      Elements(Position(1,&p), Points( X( 1 )" @;
						%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
							put ", X( 2 )" @;
						%end;
						put ", Y, Legend( &p + 6 ) ) ),";			
					%end;	
					put "		SendToReport(";
					/*
					%do p=1 %to &nvp;
						put "	Dispatch(";
						put "	    {},";
						put "	    ""&&ProbTrendVar&p"",";
						put "	    ScaleBox,";
						put "	    {Format( ""Fixed Dec"", 9, 2 ), Inc(%sysevalf((&AxisMax-&AxisMin)/6)), Min( &AxisMin), Max( &AxisMax ),  Minor Ticks( 0 ),Add Ref Line( &r, Dashed, ""Red"" ),";
						put "	    Rotated Labels( ""Automatic"" )}";
						put "    ),";
					%end;
					*/
					%let u=0;
					%do p=1 %to &NPVars;
						%if %index(&&PVar&p,ProbTrend) %then %do;
							%if &u=0 %then %let k=;

							/*Test*/
							%let testflag= 0;
							%let ytest=;
							%if %sysfunc(find(&&PVar&p, Trend)) %then %do;
								%let ytest= %str(Trend Test);
								%let testflag= 1;
							%end;
							%else %do;
								%let ytest= &&PVar&p;
								%let testflag= 0;
							%end;			

							/*MTC Correction*/
							%let ymtc=;
							/*
							%let currSubPVar = &&PVar&p;
							%if %sysfunc(find(&&PVar&p, _Bon)) %then %do;
								%let ymtc= %str(Bonferroni Corrected);
								%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _Bon, ));
							%end;
							%else %if %sysfunc(find(&&PVar&p, _FDR)) %then %do;
								%let ymtc= %str(FDR Corrected);
								%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _FDR, ));
							%end;
							%else %if %sysfunc(find(&&PVar&p, _Sid)) %then %do;
								%let ymtc= %str(Sidak Corrected);
								%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _Sid, ));
							%end;
							*/

							/*p-value adjustment*/						
							%let yadjs= ;
							%let ybwth= ;
							/*			
							%if %sysfunc(find(&currSubPVar, _F)) %then %do;

								%let bg= %sysfunc(find(&currSubPVar, _F));
								%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
								%if &ed %then %do;
									%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
								%end;
								%else %do;
									%let bwth= %substr(&currSubPVar, &bg + 2);
								%end;
								%let ybwth= %str(Bandwith &bwth);
								%let yadjs= %str(Fisher &ybwth Adj);
							%end;
							%else %if %sysfunc(find(&currSubPVar, _S)) %then %do;

								%let bg= %sysfunc(find(&currSubPVar, _S));
								%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
								%if &ed %then %do;
									%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
								%end;
								%else %do;
									%let bwth= %substr(&currSubPVar, &bg + 2);
								%end;
								%let ybwth= %str(Bandwith &bwth);					
								%let yadjs= %str(Simes &ybwth Adj);
							%end;
							%else %if %sysfunc(find(&currSubPVar, _T)) %then %do;

								%let bg= %sysfunc(find(&currSubPVar, _T));
								%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
								%if &ed %then %do;
									%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
								%end;
								%else %do;
									%let bwth= %substr(&currSubPVar, &bg + 2);
								%end;
								%let ybwth= %str(Bandwith &bwth);
								%let yadjs= %str(TPM &ybwth Adj);					
							%end;
							*/

							/*p-value conversion*/
							%let yconv=;
							%if %sysfunc(find(%upcase(&&PVar&p), NEGLOG10)) %then %do;
								%let yconv= %str(Negative Log10 P-Value );
							%end;
							%else %if %sysfunc(find(%upcase(&&PVar&p), NEGLOG)) %then %do; 
								%let yconv= %str(Negative Log P-Value );
							%end;
							%else %do;
								%let yconv= %str(P-Value );
							%end;				

							put "			Dispatch(";
							put "			{},";
							put "			""Y &k title"",";
							put "			TextEditBox,";
							put "			{Set Text("" ";
							%if &testflag %then %do;
								put "			&ytest ";
								%if %length(&ymtc) %then %do;
									put "			&ymtc ";
								%end;
								%if %length(&yadjs) %then %do;
									put "			&yadjs ";
								%end;
								%if %length(&yconv) %then %do;
									put "			&yconv ";
								%end;					
							%end;
							%else %do;
								put "			&ytest ";
							%end;
							put "			 "")}";
							put "			),";

							%let k= %sysevalf(&k + 1); 
							%let u= %sysevalf(&u + 1); 
						%end;
					%end;						
					
					put '   ),';  
					put '   ),';    
					put "rop2ag&i&_suf_=EvalInsert(""\[";
					put " try(o2&i._&j&_suf_[TextBox(1)] << Delete());";
					put " if(type(op2&i._&j&_suf_)==""Scriptable"" | type(op2&i._&j&_suf_)==""List"",";
					put "  rop2_&i&j&_suf_ = op2&i._&j&_suf_ << report;"; 
					%if &NbyVal > 1 %then %do;
						put "  for(_b=1, _b<=&NbyVal, _b++,";
						put '   try(';
						put "   	rop2_&i&j&_suf_[_b][framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
						%if %symexist(minusr) %then %do;
							put "   	rop2_&i&j&_suf_[_b][framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
						%end;
						put "   	rop2_&i&j&_suf_[_b][framebox(1)] << Marker Size(3);";
						put "   	rop2_&i&j&_suf_[_b][framebox(1)] << Marker Drawing Mode(Normal);";    
						put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Remove Axis Label;";
						put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
						put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Min(&AxisMin);";
						put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Max(&AxisMax);";  
						put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
						/*put "	   	rop2_&i&j&_suf_[_b][Axisbox(2)] << RemoveAxisLabel;";*/
						put "//    	rop2_&i&j&_suf_[_b][framebox(1)] << Frame Size(350,175);";   
						put '     ))';
					%end;
					%else %do;
						put '   try(';
						put "   	rop2_&i&j&_suf_[framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
						%if %symexist(minusr) %then %do;
							put "   	rop2_&i&j&_suf_[framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
						%end;
						put "   	rop2_&i&j&_suf_[framebox(1)] << Marker Size(3);";
						put "   	rop2_&i&j&_suf_[framebox(1)] << Marker Drawing Mode(Normal);";    
						put "   	rop2_&i&j&_suf_[Axisbox(2)] << Remove Axis Label;";
						put "   	rop2_&i&j&_suf_[Axisbox(2)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
						put "   	rop2_&i&j&_suf_[Axisbox(2)] << Min(&AxisMin);";
						put "   	rop2_&i&j&_suf_[Axisbox(2)] << Max(&AxisMax);";  
						put "   	rop2_&i&j&_suf_[Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
						/*put "	   	rop2_&i&j&_suf_[Axisbox(2)] << RemoveAxisLabel;";*/
						put "//    	rop2_&i&j&_suf_[framebox(1)] << Frame Size(350,175);";   
						put '     )';										
					%end;
					put ');]\");';
					put "Eval(Parse(rop2ag&i&_suf_));";					
				%end;
				put '));';

				%if &i>0 %then %do;
					%if %symexist(NTrait) %then %do;
						%if &NTrait>1 %then %do;	
							%let j = 1;
							put "ocobexpr&i&_suf_=Parse(ocob&i&_suf_);";
							put "SubstituteInto(ocobexpr&i&_suf_,parse(EvalInsert(""rop&i&j&_suf_"")),parse(EvalInsert(""rop2_&i&j&_suf_"")));";
							put "Eval(ocobexpr&i&_suf_);";
						%end;
					%end;
				%end;	
		
			%end;
		%end; /* end for odds ratio plots */
		
		put "),";
		put "),";
		put "); //end cc&i function";

		%if &_pvalplots %then %do;
			put "Here:ropfunc&i&_suf_=function({},{default local},";
			put "// print(""Entering ropfunc&i&_suf_"");";

			%do j=&startloop %to &endloop;

				%if &i>0 or &j=1 %then %do;
					put "Here:ropag&i&_suf_="" "";";
				%end;	

				%if &i=0 %then %do;
					put "if(!IsEmpty(_ag_),_ag0_=_ag_,_ag0_=""&&SGroupn&j"");";
					put "_ag_=""C_&&SGroupn&j"";";
				%end;

				put "Here:ropag&i&_suf_||=EvalInsert(""\[";
				put " try(o2&i._&j&_suf_[TextBox(1)] << Delete());";
				put " if(type(op&i._&j&_suf_)==""Scriptable"" | type(op&i._&j&_suf_)==""List"",";
				put "  try(Here:rop&i&j&_suf_ = op&i._&j&_suf_ << report);";
				%if &ProcessName = MissingGenotypeSummary %then %do;
					put "  try(rdiff&i&j&_suf_ = diffs&i._&j&_suf_ << report);"; 
				%end;
				%if &_overlay=0 %then %do;
				/* For PValue Browser since don't overlay p-values */
					put "  for(_i=1, _i<=&NPVars, _i++,";
					put '   try(';
					put "   	rop&i&j&_suf_[framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
					%if %symexist(minusr) %then %do;
						put "   	rop&i&j&_suf_[framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
					%end;
					put "   	rop&i&j&_suf_[framebox(_i)] << Marker Size(3);";
					put "   	rop&i&j&_suf_[framebox(_i)] << Marker Drawing Mode(Normal);";    
					/*put "   	rop&i&j&_suf_[Axisbox(_i)] << Remove Axis Label;";*/
					put "   	rop&i&j&_suf_[Axisbox(_i)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
					put "   	rop&i&j&_suf_[Axisbox(_i)] << Min(&AxisMin);";
					put "   	rop&i&j&_suf_[Axisbox(_i)] << Max(&AxisMax);";   
					put "   	rop&i&j&_suf_[Axisbox(_i)] << Format( ""Fixed Dec"", 9, 2 );";
					put "	   	rop&i&j&_suf_[Axisbox(_i)] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";
					/*put "     	rop&i&j&_suf_[framebox(_i)] << Frame Size(350,175);";*/  
					/*put "     	rop&i&j&_suf_[outlinebox(1)] << SetTitle(""Test for HWE"");"; */		
				%end;
				%else %do;
					%if &NbyVal > 1 %then %do;
						put "  for(_b=1, _b<=&NbyVal, _b++,";
						put "   // show(_b);";
						put '   try(';
						put "   	rop&i&j&_suf_[_b][framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
						%if %symexist(minusr) %then %do;
							put "   	rop&i&j&_suf_[_b][framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
						%end;
						/* kemicl added to change marker format for qtl */
						/*%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
							put "   	rop&i&j&_suf_[_b][framebox(1)] << Marker Size(3);";
							put "   	rop&i&j&_suf_[_b][framebox(1)] << Marker Drawing Mode(Normal);";  
						%end;
						%else %do;*/
							put "   	rop&i&j&_suf_[_b][framebox(1)] << Marker Size(3);";
							put "   	rop&i&j&_suf_[_b][framebox(1)] << Marker Drawing Mode(Normal);";  
						/*%end;*/
						/*
						put "   	rop&i&j&_suf_[_b][Axisbox(1)] << RemoveAxisLabel;";
						put "   	rop&i&j&_suf_[_b][Axisbox(1)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
						put "   	rop&i&j&_suf_[_b][Axisbox(1)] << Min(&AxisMin);";
						put "   	rop&i&j&_suf_[_b][Axisbox(1)] << Max(&AxisMax);";  
						put "   	rop&i&j&_suf_[_b][Axisbox(1)] << Format( ""Fixed Dec"", 9, 2 );";
						put "	   	rop&i&j&_suf_[_b][Axisbox(1)] << AddAxisLabel(""&ALabel"");";
						put "     	rop&i&j&_suf_[_b][framebox(1)] << FrameSize(350,175);";  
						*/
						put "   	rop&i&j&_suf_[_b] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";
						put "   	rop&i&j&_suf_[_b] << Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} );";
		      				put "      	//show(rop&i&j&_suf_[_b][OutLineBox(1)]);";
						put "      	gtitle= rop&i&j&_suf_[_b][OutLineBox(1)]<< Get Text;";
      						put "      	//show(gtitle);";
      						put "      	gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
      						put "      	//show(gtitle);";
						put "      	pos= Munger(gtitle, 1, ""Where("");";
						put "      	//show(pos);";
						put "      	if(pos, ";
      						put "      	     	del= Trim(Substr(gtitle, pos - 6));";
      						put "      	     	rop&i&j&_suf_[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );"; 					
						put "      		gtitle= Trim(Substr(gtitle, 1, pos - 6));";						
						put "      	);";      			
      						put "      	rop&i&j&_suf_[_b][OutLineBox(1)] << {Set Title(""Significance Test for ""||gtitle), Image Export Display( Normal )};";					
						put "   	rop&i&j&_suf_[_b][Axisbox(2)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
						put "   	rop&i&j&_suf_[_b][Axisbox(2)] << Min(&AxisMin);";
						put "   	rop&i&j&_suf_[_b][Axisbox(2)] << Max(&AxisMax);";  
						put "   	rop&i&j&_suf_[_b][Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
						/*put "     	rop&i&j&_suf_[_b][framebox(1)] << FrameSize(350,175);";*/ 
						/*put "     rop&i&j&_suf_[_b][outlinebox(1)] << SetTitle(""Test for HWE"");";*/   
						
						%if &ProcessName=%str(HaplotypeQKMixedModel) or &ProcessName=%str(HaplotypeTrendRegression) %then %do;
							put "   	rop&i&j&_suf_[_b][Axisbox(2)] << Inc(1);";
						%end;
						%if &ProcessName = MissingGenotypeSummary %then %do;
							/*put "     rdiff&i&j&_suf_[_b][framebox(1)] << FrameSize(350,175);";*/
							put "     rdiff&i&j&_suf_[_b][framebox(1)] << MarkerDrawingMode(Normal);";
			      				put "      //show(rdiff&i&j&_suf_[_b][OutLineBox(1)]);";
							put "      gtitle= rdiff&i&j&_suf_[_b][OutLineBox(1)]<< Get Text;";
	      						put "      //show(gtitle);";
	      						put "      gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
	      						put "      //show(gtitle);";
							put "      pos= Munger(gtitle, 1, ""Where("");";
							put "      //show(pos);";
							put "      if(pos, ";
	      						put "           del= Trim(Substr(gtitle, pos - 6));";
	      						put "           rdiff&i&j&_suf_[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );"; 					
							put "      	gtitle= Trim(Substr(gtitle, 1, pos - 6));";						
							put "      );";      			
	      						put "      rdiff&i&j&_suf_[_b][OutLineBox(1)] << {Set Title(""Number of missing genotypes for ""||gtitle), Image Export Display( Normal )};";												
						%end;
						put '     )));';		
					%end;
					%else %do;
						put '   try(';
						put "   	rop&i&j&_suf_[framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
						%if %symexist(minusr) %then %do;
							put "   	rop&i&j&_suf_[framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
						%end;
						/* kemicl added to change marker format for qtl */
						/*%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
							put "   	rop&i&j&_suf_[framebox(1)] << Marker Size(3);";
							put "   	rop&i&j&_suf_[framebox(1)] << Marker Drawing Mode(Normal);";  
						%end;
						%else %do;*/
							put "   	rop&i&j&_suf_[framebox(1)] << Marker Size(3);";
							put "   	rop&i&j&_suf_[framebox(1)] << Marker DrawingMode(Normal);";  
						/*%end;*/
						/*
						put "   	rop&i&j&_suf_[Axisbox(1)] << RemoveAxisLabel;";
						put "   	rop&i&j&_suf_[Axisbox(1)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
						put "   	rop&i&j&_suf_[Axisbox(1)] << Min(&AxisMin);";
						put "   	rop&i&j&_suf_[Axisbox(1)] << Max(&AxisMax);";  
						put "   	rop&i&j&_suf_[Axisbox(1)] << Format( ""Fixed Dec"", 9, 2 );";
						put "	   	rop&i&j&_suf_[Axisbox(1)] << AddAxisLabel(""&ALabel"");";
						put "     	rop&i&j&_suf_[framebox(1)] << FrameSize(350,175);";  
						*/
						put "   	rop&i&j&_suf_ << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";
						put "   	rop&i&j&_suf_ << Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} );";	
						%if %symexist(TraitVar) %then %do;
							put "      	rop&i&j&_suf_[OutLineBox(1)] << {Set Title(""Significance Test for &TraitVar""), Image Export Display( Normal )};";
						%end;
						%else %do;
							put "      	rop&i&j&_suf_[OutLineBox(1)] << {Set Title(""Significance Test""), Image Export Display( Normal )};";
						%end;
						put "   	rop&i&j&_suf_[Axisbox(2)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
						put "   	rop&i&j&_suf_[Axisbox(2)] << Min(&AxisMin);";
						put "   	rop&i&j&_suf_[Axisbox(2)] << Max(&AxisMax);";  
						put "   	rop&i&j&_suf_[Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
						/*put "     	rop&i&j&_suf_[framebox(1)] << FrameSize(350,175);";*/  
						/*put "         rop&i&j&_suf_[outlinebox(1)] << SetTitle(""Test for HWE"");";*/
						%if &ProcessName=%str(HaplotypeQKMixedModel) or &ProcessName=%str(HaplotypeTrendRegression) %then %do;
							put "   	rop&i&j&_suf_[Axisbox(2)] << Inc(1);";
						%end;
						%if &ProcessName = MissingGenotypeSummary %then %do;
							/*put "     rdiff&i&j&_suf_[framebox(1)] << FrameSize(350,175);";*/
							put "     rdiff&i&j&_suf_[framebox(1)] << MarkerDrawingMode(Normal);";
							%if %symexist(TraitVar) %then %do;
								put "      	rdiff&i&j&_suf_[OutLineBox(1)] << {Set Title(""Number of missing genotypes for &TraitVar""), Image Export Display( Normal )};";
							%end;
							%else %do;
								put "      	rdiff&i&j&_suf_[OutLineBox(1)] << {Set Title(""Number of missing genotypes""), Image Export Display( Normal )};";
							%end;							
						%end;
						put '     ));';
					%end;	
				%end;

				%if %symexist(TrackPath) %then %do;
					%if %length(&TrackPath) %then %do;
						put "// show(jg:drill_down);";
						put "if(jg:drill_down==0,";
						put "   jg:trackfolder = ""&TrackPath"";";
						put "   jg:trackfiles = ""&TrackFiles"";";
						put "   jg:trackadd = &TrackAdd;";
						put "   jg:intervening = 1;";
						put ");";
						put "plotreport=rop&i&j&_suf_;";
						put "chrvar = ""&AnnoGroup"";";
						put "posvar = ""&AnnoLoc"";";
						put "yvar = {};";
						%do p=1 %to &NPVars;
							put "InsertInto(yvar,""&&PVar&p"");";
						%end;
						put "chrvalues = {};";
						%if &i=0 %then %do;
							put "chrvalues[1] = char(_ag0_);";
						%end;
						%else %do;
							put "chrvalues[1] = char(_ag_);";
						%end;
						/*
						put "if(Munger(char(_ag_),1,""chr"")==1,";
						put " chrvalues[1] = _ag_,";
						put " chrvalues[1] = ""chr""||char(_ag_);";	
						put ");";
						*/
						/*
						put "if(Munger(""&&Groupn&j"",1,""chr"")==1,";
						put " chrvalues = {""&&Groupn&j""},";
						put " chrvalues = {""chr&&Groupn&j""});";
						*/
						put "if(jg:drill_down==1,";
						put "   trackfolder = jg:trackfolder;";
						put "   trackfiles = jg:trackfiles;";
						put "   trackadd = jg:trackadd;";
						put "   intervening = jg:intervening;";
						put "   TrackPlot;";
						put ");";
					%end;
				%end;
				put ']\"); ';

			%end;
			/*
			S1561428: Fixing frame size of p-value plots not working.
			put "Here:ropag&i&_suf_||=EvalInsert(""\[";
			put "   Here:update_Frames&i&_suf_();";
			put ']\"); ';
			*/
			put "// show(Here:ropag&i&_suf_);";
			put "Eval(Parse(Here:ropag&i&_suf_));";

			%if &i>0 %then %do;
				%if %symexist(NTrait) %then %do;
					%if &NTrait>1 %then %do;
						%let j = &i;
						/* Only open outline boxes for selected rows (traits) */
						put "ocob&i&_suf_=EvalInsert(""\[";
						%if %quote(&ByVars) ne Trait %then %do;
							put "try(rop&i&j&_suf_[1][""Graph Builder Trait=&_tv1,?""]<<CloseAllLikeThis);";					
						%end;		
						%else %do;
							put "try(rop&i&j&_suf_[1][""Graph Builder Trait=&_tv1""]<<CloseAllLikeThis);";
						%end;
						put "]\"");";
						put "sbgsellist={};";
						put "sbgR={};";
						put "CurrentDataTable(Here:sbg);";
						put "for each row(";
						put " if(selected(rowstate())==1,";
						put "  InsertInto(sbgsellist,:Trait);";
						put "  _rc=ceiling(row()/&NumGps);";
						put "  InsertInto(sbgR,_rc)";
						put "	);";
						put ");";
						put "for(s=1,s<=NItems(sbgsellist),s++,";
						put " ocob&i&_suf_||=EvalInsert(""\[";
						%if %quote(&ByVars) ne Trait %then %do;
							put "  try(rop&i&j&_suf_[^sbgR[s]^][""Graph Builder Trait=^sbgsellist[s]^,?""]<<Close(0));";					
						%end;
						%else %do;
							put "  try(rop&i&j&_suf_[^sbgR[s]^][""Graph Builder Trait=^sbgsellist[s]^""]<<Close(0));";
						%end;
						put " ]\"");";
						put ");";
						put "Eval(Parse(ocob&i&_suf_));";
						%if &ProcessName = MissingGenotypeSummary %then %do;
							put "ocobexpr&i&_suf_=Parse(ocob&i&_suf_);";
							put "SubstituteInto(ocobexpr&i&_suf_,";
							put " parse(EvalInsert(""rop&i&j&_suf_"")),parse(EvalInsert(""rdiff&i&j&_suf_"")),";
							put " parse(""CloseAllLikeThis""),parse(""Close(1)"")";
							put ");";
							put "Eval(ocobexpr&i&_suf_);";	
						%end;
						put "CurrentDataTable(Here:data&i&_suf_);";
					%end;
				%end;	
			%end;
			put "// print(""Exiting ropfunc&i&_suf_"");";
			put "); //end ropfunc&i&_suf_ function";
		%end;
		
		%if &i>0 and &_summchart %then %do;
			%if &_summchart and %upcase(&ProcessName) ne QTLSINGLEMARKERANALYSIS  %then %do;
				put "InsertInto(TabAfterCodeList,expr(ropfunc&i&_suf_()), Loc(TabButtonList,""Summary Chart"")+2+&i);";
				put "InsertInto(TabCodeList,expr(cc&i&_suf_), Loc(TabButtonList,""Summary Chart"")+2+&i);";
			%end;
		%end;
		%else %if %upcase(&ProcessName) ne METAANALYSIS and &_pvalplots %then %do;
			put "InsertInto(TabAfterCodeList,expr(ropfunc&i&_suf_()));";
			put "InsertInto(TabCodeList,expr(cc&i&_suf_));";
		%end;
		
		/*kemicl - Added Odds plots that are drawn for SNP-trait if Odds Ratios are requested*/
		%if %symexist(Odds) %then %do;
			%if &Odds=Yes %then %do;
				%let Yoddsplot = &Conv.ProbTrend;
				%if %symexist(PerfPerm) %then %do;
					%if &PerfPerm %then %do;
						%let Yoddsplot = &Yoddsplot.Pm;
						%if %upcase(&AdjPerm)=YES %then %let Yoddsplot = &Yoddsplot.Adj;
					%end;
				%end;

				%if &i=0 %then %do;
					put "Here:oddsplots&_suf_=Function({},{default local},";
					put "if(isempty(Here:data0&_suf_),";
					put " Here:data0&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
					%if %upcase(&OnlySig)=YES %then %do;
						%if %length(&Conv)=0 %then %let gtlt=%str(>);
						%else %let gtlt=%str(<);
						put "Here:data&i&_suf_<<select where(" @;
						put ":Name(""&PVar1"") &gtlt &r" @;
						%do p=2 %to &NPVars;
						put "& :Name(""&&PVar&p"") &gtlt &r" @;
						%end;

						put ")<<hide;";
					%end;  
					put "colnames = Here:data&i&_suf_<<GetColumnNames(String);";
					put 'if(Contains(colnames,"_excluded")>0,';
					put "	Here:data&i&_suf_<<select where(:_excluded==1)<<hide);";
					put "Here:data&i&_suf_ << exclude;";
					put "nhidden=0;";
					put "for each row(";
					put "  if(hidden(), nhidden=nhidden+1));";
					put "if(nhidden==nrow(Here:data&i&_suf_),";
					put " throw(""All p-values non-significant in this data set.""));";
					put "Here:data&i&_suf_ << Clear Select;";
					%if %length(&AnnoAccession) %then %do;
						put "column(""&AnnoAccession"")<<data type(character);";
					%end;
					%if %length(&AnnoGroup) %then %do;
						put "column(""&AnnoGroup"")<<data type(character);";
					%end;
					put "column(""&AnnoLoc"")<<data type(numeric);";		
					put "column(""&AnnoLabel"") << label(1);";
					%if &_pvalplots %then %do;
						put "column(""CumOrder"") << modeling type(nominal);";
					%end;					
					put ", //else";
					put " Current Data Table(Here:data0&_suf_);";
					%if %length(&AnnoGroup) %then %do;
						put " column(""&AnnoGroup"")<<data type(character);";
					%end;
					put ");";
					put "Here:dt = Today();";
					put "oppfsize = 800;";

					/* set Frame Size */
					put "Vlistbox(";
					put '      HListBox(TextBox(" Frame Size for Graphs Below"),';
					put "       fsso&i&_suf_=EvalInsert(""\[ frameSizeSlidero&i&_suf_ = SliderBox(num(200),num(800),oppfsize,]\"");";

					%do j=1 %to &tNumGps;
						%if &i=0 %then %do;
							put "_ag_=""C_&&SGroupn&j"";";
						%end;
						put "fsso&i&_suf_||=EvalInsert(""\[";
						put "  for(_b=1, _b<=&NbyVal, _b++,";
						put '   try(';
						put "     rop2_&i&j&_suf_[_b][framebox(1)] << FrameSize(oppfsize,0.35*oppfsize);";   
						put '));';
						put ']\"); ';					
					%end;

					put "fsso&i&_suf_||="")"";";
					put "Eval(Parse(fsso&i&_suf_));";
					put '   ),';				

					%if &tNumGps>1 %then %do;
						put '  hlistbox(';
					%end;
					%do j=1 %to &tNumGps;

						%if &tNumGps>1 %then %do;
							put "  o2&i._&j&_suf_=OutlineBox(""&AnnoGroup &&Groupn&j"",";
							%if %quote(&&Groupn&j)=%quote(.) %then %let Groupn&j=%str();
						%end;
						put '   vlistbox(' @;
						put "op2&i._&j&_suf_=Graph Builder(";
						put "	Show Control Panel( 0 ),";
						put "	Show Legend( 0 ),";
						put "	Graph Spacing( 3 ),";
						put "	Spacing Borders( 1 ),";						
						put "	Variables(";
						put "  		X( :Log_OddsRatio_Trend),";
						/*
						%if %length(&MTC) %then %do;
							put "  		X( :Log_OddsRatio_Trend), Y( :(&Yoddsplot._&MTC)";
							put '            ),' @;
						%end;
						%else %do;
							put "  		X( :Log_OddsRatio_Trend), Y( :(&Yoddsplot)";
							put '            ),' @;
						%end;
						*/
						%let u=1;
						%do p=1 %to &NPVars;
							%if %index(&&PVar&p,ProbTrend) or &_forestplot %then %do;
								put "             Y(:&&PVar&p),";
								%let ProbTrendVar&u = &&PVar&p;
								%let u= %sysevalf(&u + 1); 
							%end;	
						%end;	
						%let nvp= %sysevalf(&u - 1); 
						put '            ' @;
						%if &NumGps>1 %then %do;
							put "Color(&AnnoGroup)";
						%end;						
						put "),";
						%if &tNumGps>1 %then %do;
							put "            Where(:&AnnoGroup==""&&Groupn&j""),";
						%end;
						%if %length(&JByOdds)>0 %then %do;
							put "            By(&JByOdds),";
						%end;					
						/*put "		Elements( Line( X, Y, Legend( 4 ) ), Points( X, Y, Legend( 1 ) ) )";*/
						/*put "		Elements( Points( X, Y, Legend( 1 ) ) ),";*/
						%do p=1 %to &nvp;
							put "      Elements(Position(1,&p), Points( X( 1 )" @;
							%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
								put ", X( 2 )" @;
							%end;
							put ", Y, Legend( &p + 6 ) ) ),";			
						%end;						
						put "		SendToReport(";
						%let u=0;
						%do p=1 %to &NPVars;
							%if %index(&&PVar&p,ProbTrend) %then %do;
								%if &u=0 %then %let k=;

								/*Test*/
								%let testflag= 0;
								%let ytest=;
								%if %sysfunc(find(&&PVar&p, Trend)) %then %do;
									%let ytest= %str(Trend Test);
									%let testflag= 1;
								%end;
								%else %do;
									%let ytest= &&PVar&p;
									%let testflag= 0;
								%end;			

								/*MTC Correction*/
								%let ymtc=;
								/*
								%let currSubPVar = &&PVar&p;
								%if %sysfunc(find(&&PVar&p, _Bon)) %then %do;
									%let ymtc= %str(Bonferroni Corrected);
									%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _Bon, ));
								%end;
								%else %if %sysfunc(find(&&PVar&p, _FDR)) %then %do;
									%let ymtc= %str(FDR Corrected);
									%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _FDR, ));
								%end;
								%else %if %sysfunc(find(&&PVar&p, _Sid)) %then %do;
									%let ymtc= %str(Sidak Corrected);
									%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _Sid, ));
								%end;
								*/

								/*p-value adjustment*/						
								%let yadjs= ;
								%let ybwth= ;
								/*			
								%if %sysfunc(find(&currSubPVar, _F)) %then %do;

									%let bg= %sysfunc(find(&currSubPVar, _F));
									%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
									%if &ed %then %do;
										%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
									%end;
									%else %do;
										%let bwth= %substr(&currSubPVar, &bg + 2);
									%end;
									%let ybwth= %str(Bandwith &bwth);
									%let yadjs= %str(Fisher &ybwth Adj);
								%end;
								%else %if %sysfunc(find(&currSubPVar, _S)) %then %do;

									%let bg= %sysfunc(find(&currSubPVar, _S));
									%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
									%if &ed %then %do;
										%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
									%end;
									%else %do;
										%let bwth= %substr(&currSubPVar, &bg + 2);
									%end;
									%let ybwth= %str(Bandwith &bwth);					
									%let yadjs= %str(Simes &ybwth Adj);
								%end;
								%else %if %sysfunc(find(&currSubPVar, _T)) %then %do;

									%let bg= %sysfunc(find(&currSubPVar, _T));
									%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
									%if &ed %then %do;
										%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
									%end;
									%else %do;
										%let bwth= %substr(&currSubPVar, &bg + 2);
									%end;
									%let ybwth= %str(Bandwith &bwth);
									%let yadjs= %str(TPM &ybwth Adj);					
								%end;
								*/

								/*p-value conversion*/
								%let yconv=;
								%if %sysfunc(find(%upcase(&&PVar&p), NEGLOG10)) %then %do;
									%let yconv= %str(Negative Log10 P-Value );
								%end;
								%else %if %sysfunc(find(%upcase(&&PVar&p), NEGLOG)) %then %do; 
									%let yconv= %str(Negative Log P-Value );
								%end;
								%else %do;
									%let yconv= %str(P-Value );
								%end;				
								
								put "			Dispatch(";
								put "			{},";
								put "			""Y &k title"",";
								put "			TextEditBox,";
								put "			{Set Text("" ";
								%if &testflag %then %do;
									put "			&ytest ";
									%if %length(&ymtc) %then %do;
										put "			&ymtc ";
									%end;
									%if %length(&yadjs) %then %do;
										put "			&yadjs ";
									%end;
									%if %length(&yconv) %then %do;
										put "			&yconv ";
									%end;					
								%end;
								%else %do;
									put "			&ytest ";
								%end;
								put "			 "")}";
								put "			),";

								%let k= %sysevalf(&k + 1); 
								%let u= %sysevalf(&u + 1); 

							%end;
						%end;						
						
						/*
						%do p=1 %to &nvp;
							put "		Dispatch(";
							put "	    {},";
							put "	    ""&&ProbTrendVar&p"",";
							put "	    ScaleBox,";
							put "	    {Format( ""Fixed Dec"", 9, 2 ), Inc(%sysevalf((&AxisMax-&AxisMin)/6)), Min( &AxisMin), Max( &AxisMax ),  Minor Ticks( 0 ),Add Ref Line( &r, Dashed, ""Red"" ),";
							put "	    Rotated Labels( ""Automatic"" )}";
							put "    ),";
						%end;
						*/						
						put "			Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
						%if &NbyVal = 1 %then %do;
							put "			Dispatch(";
							put "				{},";
							put "				""Graph Builder"",";
							put "				OutlineBox,";
							put "				{Set Title( ""Trend Odds Ratio for &TraitVar"" )}";
							put "     		),";		
						%end;
						put "             	Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
						put "		),";						
						put '   ),'; 						
						%if &tNumGps>1 %then %do;
							put '  )),';
						%end;
						%else put '   ,';;
						%if &i=0 %then %do;
							put "_ag_=""C_&&SGroupn&j"";";
						%end;
						/*
						%if (&NbyVal<2 and &NumGps>1) %then %do;
							put 'o[TextBox(1)] << Set Text("");';
						%end;
						*/
						put "rop2ag&i&_suf_=EvalInsert(""\[";
						put " try(o2&i._&j&_suf_[TextBox(1)] << Delete());";
						put " if(type(op2&i._&j&_suf_)==""Scriptable"" | type(op2&i._&j&_suf_)==""List"",";
						put "  rop2_&i&j&_suf_ = op2&i._&j&_suf_ << report;";    
						%if &NbyVal > 1 %then %do;
							put "  for(_b=1, _b<=&NbyVal, _b++,";
							put '   try(';
							put "   	rop2_&i&j&_suf_[_b][framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
							%if %symexist(minusr) %then %do;
								put "   	rop2_&i&j&_suf_[_b][framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
							%end;
							put "   	rop2_&i&j&_suf_[_b][framebox(1)] << Marker Size(3);";
							put "   	rop2_&i&j&_suf_[_b][framebox(1)] << Marker Drawing Mode(Normal);";    
							put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Remove Axis Label;";
							/*put "   	rop2_&i&j&_suf_[_b] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";*/
							/*put "   	rop2_&i&j&_suf_[_b] << Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} );";*/						
							put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
							put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Min(&AxisMin);";
							put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Max(&AxisMax);";   
							put "   	rop2_&i&j&_suf_[_b][Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
							put "//    	rop2_&i&j&_suf_[_b][framebox(2)] << Frame Size(350,175);";   
							put '     );';
							put "            try(";
							put "            //show(rop2_&i&j&_suf_[_b][OutLineBox(1)]);";
							put "            gtitle= rop2_&i&j&_suf_[_b][OutLineBox(1)]<< Get Text;";
							put "            //show(gtitle);";
							put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
							put "            //show(gtitle);";
							put "            pos= Munger(gtitle, 1, ""Where("");";
							put "            //show(pos);";
							put "            if(pos, ";
							put "            	del= Trim(Substr(gtitle, pos - 6));";
							put "            	rop2_&i&j&_suf_[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
							put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";							
							put "            );";
							put "            //show(gtitle);";
							put "            rop2_&i&j&_suf_[_b][OutLineBox(1)] << {Set Title(""Trend Odds Ratio for ""||gtitle), Image Export Display( Normal )};";
							put "        );";							
							put '     )';
						%end;
						%else %do;
							put '   try(';
							put "   	rop2_&i&j&_suf_[framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
							%if %symexist(minusr) %then %do;
								put "   	rop2_&i&j&_suf_[framebox(1)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&minusr));";
							%end;
							put "   	rop2_&i&j&_suf_[framebox(1)] << Marker Size(3);";
							put "   	rop2_&i&j&_suf_[framebox(1)] << Marker Drawing Mode(Normal);";    
							put "   	rop2_&i&j&_suf_[Axisbox(2)] << Remove Axis Label;";
							/*put "   	rop2_&i&j&_suf_ << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";*/
							/*put "   	rop2_&i&j&_suf_ << Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} );";*/					
							put "   	rop2_&i&j&_suf_[Axisbox(2)] << Inc(%sysevalf((&AxisMax-&AxisMin)/6));";
							put "   	rop2_&i&j&_suf_[Axisbox(2)] << Min(&AxisMin);";
							put "   	rop2_&i&j&_suf_[Axisbox(2)] << Max(&AxisMax);";   
							put "   	rop2_&i&j&_suf_[Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
							put "//    	rop2_&i&j&_suf_[framebox(2)] << Frame Size(350,175);";   
							put '     );';
							/*
							put "            try(";
							put "            //show(rop2_&i&j&_suf_[OutLineBox(1)]);";
							put "            gtitle= rop2_&i&j&_suf_[OutLineBox(1)]<< Get Text;";
							put "            //show(gtitle);";
							put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
							put "            //show(gtitle);";
							put "            pos= Munger(gtitle, 1, ""Where("");";
							put "            //show(pos);";
							put "            if(pos, ";
							put "            	del= Trim(Substr(gtitle, pos - 6));";
							put "            	rop2_&i&j&_suf_[OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
							put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";						
							put "            );";
							put "            //show(gtitle);";
							put "            rop2_&i&j&_suf_[OutLineBox(1)] << {Set Title(""Trend Odds Ratio for ""||gtitle), Image Export Display( Normal )};";
							put "        );";
							*/
						%end;
						put ');]\");';
						put "Eval(Parse(rop2ag&i&_suf_));";					
					%end;
					put ')));';
																				
					%if &i>0 %then %do;
						%if %symexist(NTrait) %then %do;
							%if &NTrait>1 %then %do;	
								%let j = 1;
								put "ocobexpr&i&_suf_=Parse(ocob&i&_suf_);";
								put "SubstituteInto(ocobexpr&i&_suf_,parse(EvalInsert(""rop&i&j&_suf_"")),parse(EvalInsert(""rop2_&i&j&_suf_"")));";
								put "Eval(ocobexpr&i&_suf_);";
							%end;
						%end;
					%end;	
					/*S1149704 sbrlsi %if &NumGps = 1 %then %do;*/
					%if &NumGps = 1 and %upcase(&ProcessName) ^= QKMIXEDMODEL %then %do; 
						put "InsertInto(TabOpenList,1);";
						put "InsertInto(Here:TabButtonList,""Trend Odds Ratio Plot"");";				
					%end;
					%else %do;
						put "Here:TabsSectionFlag = 1;";
						put "InsertInto(Here:TabButtonList,""All Trend Odds Ratio Plots"");";
						put "InsertInto(TabOpenList,0);";
					%end;						
					put "InsertInto(TabCodeList,expr(oddsplots&_suf_));";
					put "InsertInto(TabAfterCodeList,"" "");";
					put "InsertInto(DataList,{{""Here:data0&_suf_""}});";
				
				%end;
			%end;
		%end;
						  
		/*model fitness plot following p-value plot*/
		%if &tNumGps = 1 and &i=0 and &_pvalplots %then %do;
			%if &PlotFitVars %then %do;
				%if &_summchart and &NumGps > 1 %then %do;
					put "InsertInto(DataList,{{""Here:data0&_suf_""}},Loc(TabButtonList,""Summary Chart"")+2);";
					%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot &HaploTest"",Loc(TabButtonList,""Summary Chart"")+2);";	
					%end;
					%else %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot"",Loc(TabButtonList,""Summary Chart"")+2);";
					%end;
				%end;
				%else %do;
					put "InsertInto(DataList,{{""Here:data0&_suf_""}});";
					%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot &HaploTest"");";
					%end;
					%else %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot"");";
					%end;
				%end;
				
				%macro modelfitnessplot_likep();
					/**S1372145: Frame Size controls don't work consistently across tabs. To fix this, adding the mf suffix**/
					
					put "//model fitness plot following p-value plot;";
					/*put "Here:fit_cc&i&_suf_.mf=function({},{default local}," ;*/
					put "Here:&MainFunName=function({},{default local}," ;
					put "jg:dt=Today();";

					/* expressions to set frame size */
					put "uf&i&_suf_.mf=EvalInsert(""\[";
					put "hfsize&i&_suf_.mf = 800;";
					put "vfsize&i&_suf_.mf = 200;";		
					%if %symexist(GenAP) %then %do;
						%if &GenAP=1 %then %do;
							put "hfsize&i&_suf_.mf = 350;";
							put "vfsize&i&_suf_.mf = 175;";
						%end;
					%end;
					put "Here:update_Frames&i&_suf_.mf = Expr(";
					put ']\");';

					%if &i=0 %then %do;
						%let startloop=1;
						%let endloop=&tNumGps;
					%end;
					%else %do;
						%let startloop = &i;
						%let endloop = &i;
					%end;

					%do j=&startloop %to &endloop;
						put "uf&i&_suf_.mf||=EvalInsert(""\[";
						/* For PValue Browser since don't overlay p-values */
						%if &_overlay=0 %then %do;
							put "  for(_i=1, _i<=&NPVars, _i++,";
							put '   try(';
							put "     rop&i.&j&_suf_.mf[framebox(_i)] << FrameSize(hfsize&i&_suf_.mf,vfsize&i&_suf_.mf);";   
						%end;
						%else %do;
							put "  for(_b=1, _b<=&NbyVal, _b++,";
							put '   try(';
							put "     rop&i&j&_suf_.mf[_b][framebox(1)] << FrameSize(hfsize&i&_suf_.mf,vfsize&i&_suf_.mf);";   
							/*
							%if &ProcessName = MissingGenotypeSummary %then %do;
								put "     rdiff&i&j&_suf_.mf[_b][framebox(1)] << FrameSize(hfsize&i&_suf_.mf,vfsize&i&_suf_.mf);";
							%end;
							*/
						%end;		
						put '));';
						put ']\");';
					%end;
					put "uf&i&_suf_.mf||="");"";"; 
					put "Eval(Parse(uf&i&_suf_.mf));";

					put "uf&i._2&_suf_.mf = EvalInsert(""\[";
					put "update_HFS_TextEditBox&i&_suf_.mf = Expr(";
					put "   value = Round(Num(hfsize&i&_suf_.mf),0);";
					put "   HFS_TEB&i&_suf_.mf << SetText(char(num(value)));";
					put "   HFS_TEB&i&_suf_.mf << Reshow;";
					put "   hfsize&i&_suf_.mf = value;";
					put "   update_Frames&i&_suf_.mf;";
					put ");";
					put "update_HFS_SliderBox&i&_suf_.mf = Expr(";
					put "   hfsize&i&_suf_.mf = num(HFS_TEB&i&_suf_.mf<<GetText);";
					put "   HFS_SB&i&_suf_.mf << Reshow;";
					put "   update_HFS_TextEditBox&i&_suf_.mf;";
					put ");";
					put "update_VFS_TextEditBox&i&_suf_.mf = Expr(";
					put "   value = Round(Num(vfsize&i&_suf_.mf),0);";
					put "   VFS_TEB&i&_suf_.mf << SetText(char(num(value)));";
					put "   VFS_TEB&i&_suf_.mf << Reshow;";
					put "   vfsize&i&_suf_.mf = value;";
					put "   update_Frames&i&_suf_.mf;";
					put ");";
					put "update_VFS_SliderBox&i&_suf_.mf = Expr(";
					put "   vfsize&i&_suf_.mf = num(VFS_TEB&i&_suf_.mf<<GetText);";
					put "   VFS_SB&i&_suf_.mf << Reshow;";
					put "   update_VFS_TextEditBox&i&_suf_.mf;";
					put ");";
					put ']\");';
					put "Eval(Parse(uf&i._2&_suf_.mf));";

					put "vfsteb&i&_suf_.mf = EvalInsert(""\[";
					put "   VFS_TEB&i&_suf_.mf = TextEditBox(char(num(vfsize&i&_suf_.mf)),";
					put "      <<Script(update_VFS_SliderBox&i&_suf_.mf)";
					put "   )";
					put "]\"");";					
					put "vfssb&i&_suf_.mf = EvalInsert(""\[";
					put "   VFS_SB&i&_suf_.mf = SliderBox(50,800,vfsize&i&_suf_.mf,";
					put "      update_VFS_TextEditBox&i&_suf_.mf";
					put "   )";
					put "]\"");";	
					put "vfssb&i._2&_suf_.mf = EvalInsert(""\[";
					put "   VFS_SB&i&_suf_.mf << SetValue(num(vfsize&i&_suf_.mf));";
					put "   VFS_SB&i&_suf_.mf << SetWidth(120);";
					put "]\"");";	

					put "hfsteb&i&_suf_.mf = EvalInsert(""\[";
					put "   HFS_TEB&i&_suf_.mf = TextEditBox(char(num(hfsize&i&_suf_.mf)),";
					put "      <<Script(update_HFS_SliderBox&i&_suf_.mf)";
					put "   )";
					put "]\"");";					
					put "hfssb&i&_suf_.mf = EvalInsert(""\[";
					put "   HFS_SB&i&_suf_.mf = SliderBox(100,1200,hfsize&i&_suf_.mf,";
					put "      update_HFS_TextEditBox&i&_suf_.mf";
					put "   )";
					put "]\"");";	
					put "hfssb&i._2&_suf_.mf = EvalInsert(""\[";
					put "   HFS_SB&i&_suf_.mf << SetValue(num(hfsize&i&_suf_.mf));";
					put "   HFS_SB&i&_suf_.mf << SetWidth(120);";
					put "]\"");";

					put "if(isempty(jg:drill_down),";
					put " jg:drill_down = 0";
					put ");";

					put "if(isempty(Here:data&i&_suf_.mf),";

					put "if(jg:drill_down==0 & isEmpty(Here:fromCCP),";
					%if &i>0 %then %do;

						put 'Here:SASCode = EvalInsert("\[';
						put "libname outlib ""&OutPath"";";
						put "data OutLib.&_name._&&SGroupn&i;";
						put "   set OutLib.&_OutDataName;";
						%if (&GpType = N) %then %do;
							%if %length(&&Groupn&i) %then %do;
								put "   if &AnnoGroup = &&Groupn&i;";
							%end;
							%else %do;
								put "   if &AnnoGroup = .;";
							%end;				
						%end;
						%else %do;
							put "   if trimn(&AnnoGroup) = ""&&Groupn&i"";";
						%end;		  
						put "run;";
						put "]\"");";

						put "jg:serveroutpath = ""&OutPath"";";
						put "Here:clientoutpath = ""&ClientOutPath"";";
						put "jg:CheckSASConnection;";
						put "SASSubmit(SASCode, NoOutputWindow);";
						put "if(jg:sasserver !=""Local"",";
						put "  jg:SASGetFile2(jg:serveroutpath||""&_name._&&SGroupn&i""||"".sas7bdat"",Here:clientoutpath||""&_name._&&SGroupn&i""||"".sas7bdat"");";
						put ");";
						put "jg:SASLog=SASGetLog();";			
						put "Here:data&i&_suf_.mf=open(""&ClientOutPath.&_name._&&SGroupn&i...sas7bdat"", invisible);";
					%end;
					%else %do;	
						put " Here:data&i&_suf_.mf=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
					%end;
					put ");";

					%if %upcase(&OnlySig)=YES %then %do;
						%if %length(&Conv)=0 %then %let gtlt=%str(>);
						%else %let gtlt=%str(<);
						put "Here:data&i&_suf_.mf<<select where(" @;
						put ":Name(""&PVar1"") &gtlt &r" @;
						%do p=2 %to &NPVars;
						put "& :Name(""&&PVar&p"") &gtlt &r" @;
						%end;

						put ")<<hide;";
					%end;  
					put "colnames = Here:data&i&_suf_.mf<<GetColumnNames(String);";
					put 'if(Contains(colnames,"_excluded")>0,';
					put "	Here:data&i&_suf_.mf<<select where(:_excluded==1)<<hide);";
					put "Here:data&i&_suf_.mf << exclude;";
					put "nhidden=0;";
					put "for each row(";
					put "  if(hidden(), nhidden=nhidden+1));";
					put "if(nhidden==nrow(Here:data&i&_suf_.mf),";
					put " throw(""All p-values non-significant in this data set.""));";
					put "Here:data&i&_suf_.mf << Clear Select;";
					%if %length(&AnnoAccession) %then %do;
						put "column(""&AnnoAccession"")<<data type(character);";
					%end;
					%if %length(&AnnoGroup) %then %do;
						put "column(""&AnnoGroup"")<<data type(character);";
					%end;
					put "column(""&AnnoLoc"")<<data type(numeric);";
					put "column(""&AnnoLabel"") << label(1);";
					%if &_pvalplots %then %do;
						put "column(""CumOrder"") << modeling type(nominal);";
					%end;					
					%if &ProcessName=%str(HaplotypeEstimation) or &ProcessName=%str(HaplotypeQKMixedModel) or &ProcessName=%str(HaplotypeTrendRegression) %then %do;
						put "column(""WindowID"")<<label(1);";
						put "column(""WindowID"")<< Set Modeling Type( ""Nominal"" );";
					%end;
					put ", //else";
					put "  currentdatatable(Here:data&i&_suf_.mf);";
					%if %length(&AnnoGroup) %then %do;
						put " column(""&AnnoGroup"")<<data type(character);";
					%end;		
					put "); //end isempty(data&i&_suf_.mf)";
					put "dt = jg:dt;";
					put "jslcode=EvalInsert(""\[";
					put "   try(if(jg:drill_down,currentdatatable(Here:subdata^dt^)));";
					put ']\");';					
					put "Eval(Parse(jslcode));";

					put "VListbox(HListBox(";

					put "   Text Box( "" Vertical Frame Size "" ),";
					put "   HListBox(";
					put "      eval(parse(vfsteb&i&_suf_.mf)),";
					put "      eval(parse(vfssb&i&_suf_.mf)),";
					put "      eval(parse(vfssb&i._2&_suf_.mf))";
					put "   ),";
					put "   Text Box( "" Horizontal Frame Size "" ),";
					put "   HListBox(";
					put "      eval(parse(hfsteb&i&_suf_.mf)),";
					put "      eval(parse(hfssb&i&_suf_.mf)),";
					put "      eval(parse(hfssb&i._2&_suf_.mf))";
					put "   )";
					put " ),";						
					
					%if &NumGps=1 and %upcase(&ProcessName) ^= QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
						%if %length(&SummaryDescription) %then %do;
							put " Textbox(""""),";				
							put " Textbox(""&SummaryDescription""),";
							put " Textbox(""""),";				
						%end;
					%end;		

					/* overlay plots */ 
					%if &tNumGps>1 %then %do;
							%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
								put "  ScrollBox(size(900,600),";
								put "  hlistbox(";
							%end;
							%else %do;
								put "  ScrollBox(size(850,400),";
								put "  hlistbox(";
							%end;
					%end;

					%do j=&startloop %to &endloop;

						%if &tNumGps>1 %then %do;
							put "  o=OutlineBox(""&AnnoGroup &&Groupn&j"",";
							%if %quote(&&Groupn&j)=%quote(.) %then %let Groupn&j=%str();
						%end;

						put "   Here:&MainVLBoxName= vlistbox(";

						%if &_pvalplots %then %do;

							put "   Here:op&i._&j&_suf_.mf=Graph Builder(";
							put "		Size(500,300),";
							put "   	Show Control Panel( 0 ),";
							put "   	Legend Position( ""Right"" ),";
							put "		Graph Spacing( 3 ),";
							put "		Spacing Borders( 1 ),";							
							put "   	Variables(";
							put "   		X(:&AnnoLoc )";
							put "   		, Y(Column(""&&&PrefixGroupVar.1""))";
							%do f=2 %to &NGroup;
								put "           	, Y(Column(""&&&PrefixGroupVar.&f""), Position(1))";
							%end;		
							put "   	)";
							%if &tNumGps>1 %then %do;
								put "            , Where(:&AnnoGroup==""&&Groupn&j"")";
							%end;
							%if %length(&ByVars)>0 %then %do;
								put "            , By(&JBy)";
							%end;
							put "   		, Elements("@;
							put "   			Points( X"@;
							%do f=1 %to &NGroup;
								put "   			, Y( &f )"@;
							%end;
							put "   	, Legend( 3 ) ),";
							put "   			Line( X"@;
							%do f=1 %to &NGroup;
								put "   			, Y( &f )"@;
							%end;
							put "   	, Legend( 4 ) )";
							put "   		),";
							put "   		SendToReport(";
							put "   			Dispatch(";
							put "   				{},";
							put "   				""400"",";
							put "   				ScaleBox,";
							put "   				{Legend Model(";
							put "   					3,";
							/*
							%let k= 0;
							%do f=1 %to &NGroup;
								put "   					Base( &k, 0, 0, Item ID( ""&&&PrefixGroupVar.&f"", 1 ) ),";
								%let k= %eval(&k + 1);
							%end;
							*/
							put "   				)}";
							put "   			),";
							put "   			Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
							put "   			Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Fitness"" )} ),";
							put "				Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";							
							put "   		)";
							put "   	);";
						%end;
						%if &tNumGps>1 %then %do;
							put '  )),';
						%end;
						%else put '   ,';;
					 	%if (&NbyVal<2 and &tNumGps>1) %then %do;
							put 'try(o[TextBox(1)] << Set Text("") << Set Font Size(1));';
					  	%end;	
					%end;		
					%if &tNumGps>1 and &_pvalplots %then %do;
						put ' ), //end outlinebox in subbox';
					%end;
					put "),";
					%if %length(&AppendToVLBoxName) %then %do;
						put "Here:&AppendToVLBoxName << Append(Here:&MainVLBoxName);";
					%end;	
					put ");";
					/*put "); //end fit_cc&i function";*/
					put "); //end &MainFunName function";

					%if &_pvalplots %then %do;
						/*put "Here:fit_ropfunc&i&_suf_.mf=function({},{default local},";*/
						/*put "// print(""Entering fit_ropfunc&i&_suf_.mf"");";*/
						put "Here:&AfterMainFunName=function({},{default local},";
						put "// print(""Entering &AfterMainFunName"");";
					
						%do j=&startloop %to &endloop;

							%if &i>0 or &j=1 %then %do;
								put "Here:ropag&i&_suf_.mf="" "";";
							%end;	

							%if &i=0 %then %do;
								put "if(!IsEmpty(_ag_),_ag0_=_ag_,_ag0_=""&&SGroupn&j"");";
								put "_ag_=""C_&&SGroupn&j"";";
							%end;

							put "Here:ropag&i&_suf_.mf||=EvalInsert(""\[";
							put " if(type(op&i._&j&_suf_.mf)==""Scriptable"" | type(op&i._&j&_suf_.mf)==""List"",";
							put "  try(Here:rop&i&j&_suf_.mf = op&i._&j&_suf_.mf << report);";
				
							%if &_overlay=0 %then %do;
								/* For PValue Browser since don't overlay p-values */
								put "  for(_i=1, _i<=&NPVars, _i++,";
								put '   try(';
								put "   	rop&i&j&_suf_.mf[framebox(_i)] << Marker Size(3);";
								put "   	rop&i&j&_suf_.mf[framebox(_i)] << Marker Drawing Mode(Normal);";    
								put "//   	rop&i&j&_suf_.mf[Axisbox(_i)] << Remove Axis Label;";
								put "   	rop&i&j&_suf_.mf[Axisbox(_i)] << Inc(%sysevalf((&maxGroup-&minGroup)/6));";
								put "   	rop&i&j&_suf_.mf[Axisbox(_i)] << Min(&minGroup - (&maxGroup - &minGroup)*.02 - .5);";
								put "   	rop&i&j&_suf_.mf[Axisbox(_i)] << Max(&maxGroup + (&maxGroup - &minGroup)*.02 + .5);";   
								put "   	rop&i&j&_suf_.mf[Axisbox(_i)] << Format( ""Fixed Dec"", 9, 2 );";
								put "//	   	rop&i&j&_suf_.mf[Axisbox(_i)] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";
								put "//     	rop&i&j&_suf_.mf[framebox(_i)] << Frame Size(350,175);";  	
							%end;
							%else %do;
								%if &NbyVal > 1 %then %do;
									put "  for(_b=1, _b<=&NbyVal, _b++,";
									put "   // show(_b);";
									put '   try(';
									/* kemicl added to change marker format for qtl */
									/*%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
										put "   	rop&i&j&_suf_.mf[_b][framebox(1)] << Marker Size(3);";
										put "   	rop&i&j&_suf_.mf[_b][framebox(1)] << Marker Drawing Mode(Normal);";  
									%end;
									%else %do;*/
										put "   	rop&i&j&_suf_.mf[_b][framebox(1)] << Marker Size(3);";
										put "   	rop&i&j&_suf_.mf[_b][framebox(1)] << Marker Drawing Mode(Normal);";  
									/*%end;*/
									/*put "   	rop&i&j&_suf_.mf[_b][Axisbox(2)] << RemoveAxisLabel;";*/
									put "   	rop&i&j&_suf_.mf[_b][Axisbox(2)] << Inc(%sysevalf((&maxGroup-&minGroup)/6));";
									put "   	rop&i&j&_suf_.mf[_b][Axisbox(2)] << Min(&minGroup - (&maxGroup - &minGroup)*.02 - .5);";
									put "   	rop&i&j&_suf_.mf[_b][Axisbox(2)] << Max(&maxGroup + (&maxGroup - &minGroup)*.02 + .5);";  
									put "   	rop&i&j&_suf_.mf[_b][Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
									/*put "	   	rop&i&j&_suf_.mf[_b][Axisbox(2)] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";*/
									/*put "   	rop&i&j&_suf_.mf[_b] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";*/
									/*put "     	rop&i&j&_suf_.mf[_b][framebox(1)] << FrameSize(350,175);"; */
									%if &ProcessName=%str(HaplotypeQKMixedModel) or &ProcessName=%str(HaplotypeTrendRegression) %then %do;
										put "   	rop&i&j&_suf_.mf[_b][Axisbox(1)] << Inc(1);";
									%end;
									put "            try(";
									put "            //show(rop&i&j&_suf_.mf[_b][OutLineBox(1)]);";
									put "            gtitle= rop&i&j&_suf_.mf[_b][OutLineBox(1)]<< Get Text;";
									put "            //show(gtitle);";
									put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
									put "            //show(gtitle);";
									put "            pos= Munger(gtitle, 1, ""Where("");";
									put "            //show(pos);";
									put "            if(pos, ";
									put "            	del= Trim(Substr(gtitle, pos - 6));";
									put "            	rop&i&j&_suf_.mf[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
									put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";									
									put "            );";
									put "            //show(gtitle);";
									%if %length(&MTestType) %then %do;
										put "            rop&i&j&_suf_.mf[_b][OutLineBox(1)] << {Set Title(""Model Fitness &MTestType for ""||gtitle), Image Export Display( Normal )};";
									%end;
									%else %do;
										put "            rop&i&j&_suf_.mf[_b][OutLineBox(1)] << {Set Title(""Model Fitness for ""||gtitle), Image Export Display( Normal )};";									
									%end;
									put "        );";
									put "    );";
								%end;
								%else %do;
									put '   try(';
									/* kemicl added to change marker format for qtl */
									/*%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
										put "   	rop&i&j&_suf_.mf[framebox(1)] << Marker Size(3);";
										put "   	rop&i&j&_suf_.mf[framebox(1)] << Marker Drawing Mode(Normal);";  
									%end;
									%else %do;*/
										put "   	rop&i&j&_suf_.mf[framebox(1)] << Marker Size(3);";
										put "   	rop&i&j&_suf_.mf[framebox(1)] << Marker Drawing Mode(Normal);";  
									/*%end;*/
									/*put "   	rop&i&j&_suf_.mf[Axisbox(2)] << RemoveAxisLabel;";*/
									put "   	rop&i&j&_suf_.mf[Axisbox(2)] << Inc(%sysevalf((&maxGroup-&minGroup)/6));";
									put "   	rop&i&j&_suf_.mf[Axisbox(2)] << Min(&minGroup - (&maxGroup - &minGroup)*.02 - .5);";
									put "   	rop&i&j&_suf_.mf[Axisbox(2)] << Max(&maxGroup + (&maxGroup - &minGroup)*.02 + .5);";  
									put "   	rop&i&j&_suf_.mf[Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
									/*put "	   	rop&i&j&_suf_.mf[Axisbox(2)] << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";*/
									/*put "   	rop&i&j&_suf_.mf << Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&ALabel"" )} );";*/
									/*put "     	rop&i&j&_suf_.mf[framebox(1)] << FrameSize(350,175);";*/
									%if &ProcessName=%str(HaplotypeQKMixedModel) or &ProcessName=%str(HaplotypeTrendRegression) %then %do;
										put "   	rop&i&j&_suf_.mf[Axisbox(1)] << Inc(1);";
									%end;
									%if %length(&MTestType) %then %do;
										put "            rop&i&j&_suf_.mf[OutLineBox(1)] << {Set Title(""Model Fitness &MTestType for &TraitVar""), Image Export Display( Normal )};";
									%end;
									%else %do;
										put "            rop&i&j&_suf_.mf[OutLineBox(1)] << {Set Title(""Model Fitness for &TraitVar""), Image Export Display( Normal )};";									
									%end;									
								%end;
							%end;
							put '     ));';
							put ']\"); ';
						%end;

						put "Here:ropag&i&_suf_.mf||=EvalInsert(""\[";
						put "   Here:update_Frames&i&_suf_.mf();";
						put ']\"); ';
						put "// show(Here:ropag&i&_suf_.mf);";
						put "Eval(Parse(Here:ropag&i&_suf_.mf));";

						%if &i>0 %then %do;
							%if %symexist(NTrait) %then %do;
								%if &NTrait>1 %then %do;
									%let j = &i;
									/* Only open outline boxes for selected rows (traits) */
									put "ocob&i&_suf_.mf=EvalInsert(""\[";
									%if %quote(&ByVars) ne Trait %then %do;
										put "try(rop&i&j&_suf_.mf[1][""Graph Builder Trait=&_tv1,?""]<<CloseAllLikeThis);";					
									%end;		
									%else %do;
										put "try(rop&i&j&_suf_.mf[1][""Graph Builder Trait=&_tv1""]<<CloseAllLikeThis);";
									%end;
									put "]\"");";
									put "sbgsellist={};";
									put "sbgR={};";
									put "CurrentDataTable(Here:sbg);";
									put "for each row(";
									put " if(selected(rowstate())==1,";
									put "  InsertInto(sbgsellist,:Trait);";
									put "  _rc=ceiling(row()/&NumGps);";
									put "  InsertInto(sbgR,_rc)";
									put "	);";
									put ");";
									put "for(s=1,s<=NItems(sbgsellist),s++,";
									put " ocob&i&_suf_.mf||=EvalInsert(""\[";
									%if %quote(&ByVars) ne Trait %then %do;
										put "  try(rop&i&j&_suf_.mf[^sbgR[s]^][""Graph Builder Trait=^sbgsellist[s]^,?""]<<Close(0));";					
									%end;
									%else %do;
										put "  try(rop&i&j&_suf_.mf[^sbgR[s]^][""Graph Builder Trait=^sbgsellist[s]^""]<<Close(0));";
									%end;
									put " ]\"");";
									put ");";
									put "Eval(Parse(ocob&i&_suf_.mf));";

									put "CurrentDataTable(Here:data&i&_suf_.mf);";
								%end;
							%end;	
						%end;
						/*put "// print(""Exiting fit_ropfunc&i&_suf_.mf"");";*/
						/*put "); //end fit_ropfunc&i&_suf_.mf function";*/
						put "// print(""Exiting &AfterMainFunName"");";
						put "); //end &AfterMainFunName";
					%end;
				%mend;				
				
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					%if %upcase(&HaploTest)= GLOBAL and &global_flag %then %do;
						%put MainFunNameGlobal;
					
						%let MTestType= Global;
						%let MainFunName=fitp1g;
						%let MainVLBoxName=fpvlb1g;
						%let AfterMainFunName=fitp1g_after;
						%let AppendToVLBoxName=;
						%let PrefixGroupVar= Group1Var;		
						%let NGroup= &NGroup1Val;
						%let minGroup= &minGroup1Val;
						%let maxGroup= &maxGroup1Val; 
						%modelfitnessplot_likep();
						%let AfterFunList= &AfterMainFunName;
						%let AnchorVLBoxName= &MainVLBoxName;
						%let AnchorFunName= &MainFunName;

						%if %length(&Group2OverlayVars) %then %do;					
							%let MainFunName=fitp2g;
							%let MainVLBoxName=fpvlb2g;
							%let AfterMainFunName=fitp2g_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group2Var;		
							%let NGroup= &NGroup2Val;
							%let minGroup= &minGroup2Val;
							%let maxGroup= &maxGroup2Val; 
							%modelfitnessplot_likep();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if %length(&Group3OverlayVars) %then %do;					
							%let MainFunName=fitp3g;
							%let MainVLBoxName=fpvlb3g;
							%let AfterMainFunName=fitp3g_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group3Var;		
							%let NGroup= &NGroup3Val;
							%let minGroup= &minGroup3Val;
							%let maxGroup= &maxGroup3Val; 
							%modelfitnessplot_likep();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if &i>0 and &_summchart %then %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName),Loc(TabButtonList,""Summary Chart"")+2);";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}),Loc(TabButtonList,""Summary Chart"")+2);";
							put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+2);";
						%end;
						%else %if %upcase(&ProcessName) ne METAANALYSIS and &_pvalplots %then %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName));";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}));";
							put "InsertInto(TabOpenList,1);";				
						%end;				
					%end;
					%else %if %upcase(&HaploTest)= SINGLE and &single_flag %then %do;
						%put MainFunNameSingle;
					
						%let MTestType= Single;
						%let MainFunName=fitp1s;
						%let MainVLBoxName=fpvlb1s;
						%let AfterMainFunName=fitp1s_after;
						%let AppendToVLBoxName=;
						%let PrefixGroupVar= Group1Var;		
						%let NGroup= &NGroup1Val;
						%let minGroup= &minGroup1Val;
						%let maxGroup= &maxGroup1Val; 
						%modelfitnessplot_likep();
						%let AfterFunList= &AfterMainFunName;
						%let AnchorVLBoxName= &MainVLBoxName;
						%let AnchorFunName= &MainFunName;

						%if %length(&Group2OverlayVars) %then %do;					
							%let MainFunName=fitp2s;
							%let MainVLBoxName=fpvlb2s;
							%let AfterMainFunName=fitp2s_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group2Var;		
							%let NGroup= &NGroup2Val;
							%let minGroup= &minGroup2Val;
							%let maxGroup= &maxGroup2Val; 
							%modelfitnessplot_likep();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if %length(&Group3OverlayVars) %then %do;					
							%let MainFunName=fitp3s;
							%let MainVLBoxName=fpvlb3s;
							%let AfterMainFunName=fitp3s_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group3Var;		
							%let NGroup= &NGroup3Val;
							%let minGroup= &minGroup3Val;
							%let maxGroup= &maxGroup3Val; 
							%modelfitnessplot_likep();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if &i>0 and &_summchart %then %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName),Loc(TabButtonList,""Summary Chart"")+3);";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}),Loc(TabButtonList,""Summary Chart"")+3);";
							put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+3);";
						%end;
						%else %if %upcase(&ProcessName) ne METAANALYSIS and &_pvalplots %then %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName));";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}));";
							put "InsertInto(TabOpenList,1);";				
						%end;				
					%end;				
				%end;
				%else %do;
					%let MTestType=;
					%let MainFunName=fitp1;
					%let MainVLBoxName=fpvlb1;
					%let AfterMainFunName=fitp1_after;
					%let AppendToVLBoxName=;
					%let PrefixGroupVar= Group1Var;		
					%let NGroup= &NGroup1Val;
					%let minGroup= &minGroup1Val;
					%let maxGroup= &maxGroup1Val; 
					%modelfitnessplot_likep();
					%let AfterFunList= &AfterMainFunName;
					%let AnchorVLBoxName= &MainVLBoxName;
					%let AnchorFunName= &MainFunName;

					%if %length(&Group2OverlayVars) %then %do;					
						%let MainFunName=fitp2;
						%let MainVLBoxName=fpvlb2;
						%let AfterMainFunName=fitp2_after;
						%let AppendToVLBoxName= &AnchorVLBoxName;
						%let PrefixGroupVar= Group2Var;		
						%let NGroup= &NGroup2Val;
						%let minGroup= &minGroup2Val;
						%let maxGroup= &maxGroup2Val; 
						%modelfitnessplot_likep();
						%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
					%end;

					%if %length(&Group3OverlayVars) %then %do;					
						%let MainFunName=fitp3;
						%let MainVLBoxName=fpvlb3;
						%let AfterMainFunName=fitp3_after;
						%let AppendToVLBoxName= &AnchorVLBoxName;
						%let PrefixGroupVar= Group3Var;		
						%let NGroup= &NGroup3Val;
						%let minGroup= &minGroup3Val;
						%let maxGroup= &maxGroup3Val; 
						%modelfitnessplot_likep();
						%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
					%end;

					%if &i>0 and &_summchart %then %do;
						put "InsertInto(TabCodeList,expr(&AnchorFunName),Loc(TabButtonList,""Summary Chart"")+2);";
						put "InsertInto(TabAfterCodeList,expr({&AfterFunList}),Loc(TabButtonList,""Summary Chart"")+2);";
						put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+2);";
					%end;
					%else %if %upcase(&ProcessName) ne METAANALYSIS and &_pvalplots %then %do;
						put "InsertInto(TabCodeList,expr(&AnchorFunName));";
						put "InsertInto(TabAfterCodeList,expr({&AfterFunList}));";
						put "InsertInto(TabOpenList,1);";				
					%end;					
				%end;				
			%end;
		%end;

		%outofloop0:; /*S1149704 sbrlsi*/
				
		/* Manhattan plot */
		/*S1149704 sbrlsi %if &tNumGps>1 and &_pvalplots %then %do;*/
		%if (&tNumGps>1 and &_pvalplots) or (%upcase(&ProcessName) = QKMIXEDMODEL and &i=0 and &_pvalplots) %then %do; 
			
			%if &_summchart %then %do;
				put "InsertInto(DataList,{{""Here:data0&_suf_""}},Loc(TabButtonList,""Summary Chart"")+1);";
				put "InsertInto(Here:TabButtonList,""Manhattan Plot"",Loc(TabButtonList,""Summary Chart"")+1);";
			%end;
						
			put "////////// manp   ";
			put "Here:manp=function({},{default local},";
			/*********** kemicl and sbrlsi Add message about nonconvergence from Logistic ************/
			%if %symexist(Flag_NonConvergenceG) or %symexist(Flag_NonConvergenceT) %then %do;
				%if (&Flag_NonConvergenceG = 1 or &Flag_NonConvergenceT=1) %then %do;
					put "Here:_nw01_= New Window(""&ProcessName Warning Message"",showmenu(0),showtoolbars(0),";
					put " TextBox(""NOTE: Some Association Tests did not converge. Results shown are based on the last maximum likelihood ""|| ";
					put "			""iteration. Validity of the model fits is questionable. \!N\!N"",";
					put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400)";
					put " ),";
					put " TextBox("" A data set containing the results for Non-convergent Tests is included in the Output Data."", ";
					put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400)";
					put " )";					
					put ");";
					put "if(!IsEmpty(Here:TabWinList),";
					put "    InsertInto(Here:TabWinList,0);";
					put "    Here:TabWinList[Here:nwcounter]=Here:_nw01_;";
					put "    Here:nwcounter++; ";		
					put "	);";
				%end;
			%end;			
			put "if(isempty(Here:data0&_suf_),";
			put "if(jg:drill_down == 0,";
			put " Here:data0&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
			put ");";
			%if %upcase(&OnlySig)=YES %then %do;
				%if %length(&Conv)=0 %then %let gtlt=%str(>);
				%else %let gtlt=%str(<);
				put "Here:data&i&_suf_<<select where(" @;
				put ":Name(""&PVar1"") &gtlt &r" @;
				%do p=2 %to &NPVars;
				put "& :Name(""&&PVar&p"") &gtlt &r" @;
				%end;
				put ")<<hide;";
			%end;  
			put "colnames = Here:data&i&_suf_<<GetColumnNames(String);";
			put 'if(Contains(colnames,"_excluded")>0,';
			put "	Here:data&i&_suf_<<select where(:_excluded==1)<<hide);";
			put "Here:data&i&_suf_ << exclude;";
			put "nhidden=0;";
			put "for each row(";
			put "  if(hidden(), nhidden=nhidden+1));";
			put "if(nhidden==nrow(Here:data&i&_suf_),";
			put " throw(""All p-values non-significant in this data set.""));";
			put "Here:data&i&_suf_ << Clear Select;";
			%if %length(&AnnoAccession) %then %do;
				put "column(""&AnnoAccession"")<<data type(character);";
			%end;
			%if %length(&AnnoGroup) %then %do;
				put "column(""&AnnoGroup"")<<data type(character);";
			%end;
			put "column(""&AnnoLoc"")<<data type(numeric);";
			put "column(""&AnnoLabel"") << label(1);";			
			%if &_pvalplots %then %do;
				put "column(""CumOrder"") << modeling type(nominal);";
			%end;			
			%if &ProcessName=%str(HaplotypeEstimation) %then put "column(""WindowID"")<<label(1);";;
			%if %upcase(&ProcessName) = QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
				%if (%upcase(&MultiMarkerModel)= YES and %upcase(&SelModelProfile) = YES and &QKSelProf) %then %do;
					%if (%length(&selMarkerVars_g)) and (%length(&selMarkerVars_t)) %then %do;
						put "	Here:data&i&_suf_ << Select Where(!Is Missing(:Name(""Selected_Geno"")[])|!Is Missing(:Name(""Selected_Trend"")[]));";
					%end;
					%else %if (%length(&selMarkerVars_g)) %then %do;
						put "	Here:data&i&_suf_ << Select Where(!Is Missing(:Name(""Selected_Geno"")[]));";
					%end;
					%else %if (%length(&selMarkerVars_t)) %then %do;
						put "	Here:data&i&_suf_ << Select Where(!Is Missing(:Name(""Selected_Trend"")[]));";
					%end;
					put "	Here:data&i&_suf_ << Markers(11);";
					put "	Here:data&i&_suf_ << Clear Select();";
				%end;
			%end;
			
			put ", //else";
			put "  currentdatatable(Here:data0&_suf_);";
			%if &_pvalplots %then %do;
				put "column(""CumOrder"") << modeling type(nominal);";
			%end;
			put "); //end isempty(data&_suf_)";
			
			%if %upcase(&ProcessName) = QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
				%if (%upcase(&MultiMarkerModel)= YES and %upcase(&SelModelProfile) = YES and &QKSelProf) %then %do;
					put "Vlistbox(";
					%if (%length(&selMarkerVars_g)) and (%length(&selMarkerVars_t)) %then %do;
						put "	TextBox(""Markers selected in the multi-marker model of either the Genotype or Trend tests are represented by stars (*) in the plot."",<<FontColor(0),<< Set Wrap(800)),";
						put "	TextBox(""Two types of multi-marker model were fitted: one including all selected markers (shown as star), and the other including all selected markers plus each unselected marker one at the time."",<<FontColor(0),<< Set Wrap(800)),";
					%end;
					%else %if (%length(&selMarkerVars_g)) %then %do;
						put "	TextBox(""Markers selected in the multi-marker model are represented by stars (*) in the plot."",<<FontColor(0),<< Set Wrap(800)),";				
						put "	TextBox(""Two types of multi-marker model were fitted: one including all selected markers (shown as star), and the other including all selected markers plus each unselected marker one at the time."",<<FontColor(0),<< Set Wrap(800)),";
					%end;
					%else %if (%length(&selMarkerVars_t)) %then %do;
						put "	TextBox(""Markers selected in the multi-marker model are represented by stars (*) in the plot."",<<FontColor(0),<< Set Wrap(800)),";				
						put "	TextBox(""Two types of multi-marker model were fitted: one including all selected markers (shown as star), and the other including all selected markers plus each unselected marker one at the time."",<<FontColor(0),<< Set Wrap(800)),";
					%end;
					put "	TextBox("" "",<<FontColor(0),<< Set Wrap(800)),";
				%end;
			%end;
			
			put "Data Filter Context Box(Hlistbox(";
			put "Here:mpvlb=Vlistbox(";
			put "Here:mp= Graph Builder(";
			put "	Show Control Panel( 0 ),";
			put "	Show Legend( 0 ),";
			put "	Graph Spacing( 0 ),";
			put "	Spacing Borders( 1 ),";
			put "	Variables(";
			%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
				put "		X( ""&AnnoGroup"" ),";
				put "		X( :CumOrder, Position( 1 ) ),";
				put "		Color( ""&AnnoGroup"" ),";
			%end;
			%else %do;
				put "		X( :CumOrder ),";
			%end;
			%do p=1 %to &NPVars;
				put "		Y(Column(""&&PVar&p"")),";
			%end;
			/*
			%if %length(&ByVars)>0 %then %do;
				put "             Page(&JBy),";									
			%end;
			*/
			put "	),";
			%if %length(&ByVars)>0 %then %do;
				put "            By(&JBy),";						
			%end;					
			%do p=1 %to &NPVars;
				put "      Elements(Position(1,&p), Points( X( 1 )" @;
				%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
					put ", X( 2 )" @;
				%end;
				put ", Y, Legend( &p + 6 ) ) ),";			
			%end;
			/*
			%if &NumGps > 1 %then %do;
				put "      Local Data Filter(""on right"",";
				put "         Add Filter( Columns(:Name(""&AnnoGroup"")), Display(:Name(""&AnnoGroup""), Size(210, min(400,&NumGps*20)), List Display)),";
				put "		    Mode( Show(1), Include(1) )";
				put "      ),";	
			%end;			
			*/
			put "	SendToReport(";			
			%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
				put "			Dispatch( {}, ""X title"", TextEditBox, {Set Text( ""&AnnoGroup"" )} ),";
			%end;
			%else %do;
				put "			Dispatch( {}, ""X title"", TextEditBox, {Set Text( ""CumOrder"" )} ),";
			%end;
			put "			Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "			Dispatch( {}, ""400"", LegendBox, {Sides( ""Left"" ), Set Wrap( 10 )} ),";			
			%do p=1 %to &NPVars;
				%if &p=1 %then %let k=;
				%else %let k= %sysevalf(&p - 1); 
				
				/*Test*/
				%let testflag= 0;
				%let ytest=;
				%if %sysfunc(find(&&PVar&p, Genotype)) %then %do;
					%let ytest= %str(Genotype Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, Trend)) %then %do;
					%let ytest= %str(Trend Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, Allele)) %then %do;
					%let ytest= %str(Allele Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, Dom)) %then %do;
					%let ytest= %str(Dominant Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, Rec)) %then %do;
					%let ytest= %str(Recessive Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, MAX)) %then %do;
					%let ytest= %str(MAX Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, HWE)) %then %do;
					%let ytest= %str(HWE Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, HapCC)) %then %do;
					%let ytest= %str(Haplotype Case Control Test);
					%let testflag= 1;
				%end;	
				%else %if %sysfunc(find(&&PVar&p, ProbFGlobal)) %then %do;
					%let ytest= %str(Global Haplotype Trend Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbFSingle)) %then %do;
					%let ytest= %str(Single Haplotype Trend Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbMissing)) %then %do;
					%let ytest= %str(Missing Genotype Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbRareVar)) %then %do;
					%let ytest= %str(Rare Variant Test);
					%let testflag= 1;
				%end;	
				%else %if %sysfunc(find(&&PVar&p, ProbOQTDT)) %then %do;
					%let ytest= %str(QTD Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbTDTq5)) %then %do;
					%let ytest= %str(TDTq5 Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbTqps)) %then %do;
					%let ytest= %str(Tqps Test);
					%let testflag= 1;
				%end;					
				%else %if %sysfunc(find(&&PVar&p, ProbSibQTDT)) %then %do;
					%let ytest= %str(SibQTDT Test);
					%let testflag= 1;
				%end;						
				%else %if %sysfunc(find(&&PVar&p, ProbTDT)) %then %do;
					%let ytest= %str(TD Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbSTDT)) %then %do;
					%let ytest= %str(STD Test);
					%let testflag= 1;
				%end;
				%else %if %sysfunc(find(&&PVar&p, ProbSDT)) %then %do;
					%let ytest= %str(SD Test);
					%let testflag= 1;
				%end;					
				%else %if %sysfunc(find(&&PVar&p, ProbRCTDT)) %then %do;
					%let ytest= %str(RCTDT Test);
					%let testflag= 1;
				%end;	
				%else %if %sysfunc(find(&&PVar&p, ProbRaoScott_C)) %then %do;
					%let ytest= %str(RaoScott Chi-Square Test);
					%let testflag= 1;
				%end;					
				%else %if %sysfunc(find(&&PVar&p, ProbRaoScott_F)) %then %do;
					%let ytest= %str(RaoScott F Test);
					%let testflag= 1;
				%end;	
				%else %if %sysfunc(find(&&PVar&p, ProbMetaIV)) %then %do;
					%let ytest= %str(Inverse Variance Test);
					%let testflag= 1;
				%end;	
				%else %if %sysfunc(find(&&PVar&p, ProbMetaWSS)) %then %do;
					%let ytest= %str(Weighted Square Root Test);
					%let testflag= 1;
				%end;				
				%else %do;
					%let ytest= &&PVar&p;
					%let testflag= 0;
				%end;		
				
				/*MTC Correction*/
				%let ymtc=;
				
				/*p-value adjustment*/					
				%let yadjs= ;
				%let ybwth= ;	
				
				/*p-value conversion*/
				%let yconv=;
				%if %sysfunc(find(%upcase(&&PVar&p), NEGLOG10)) %then %do;
					%let yconv= %str(Negative Log10 P-Value );
					%let ymin= -.1;
				%end;
				%else %if %sysfunc(find(%upcase(&&PVar&p), NEGLOG)) %then %do; 
					%let yconv= %str(Negative Log P-Value );
					%let ymin= -.1;
				%end;
				%else %do;
					%let yconv= %str(P-Value );
					%let ymin= -.01;
				%end;
				
				put "			Dispatch(";
				put "			{},";
				put "			""Y &k title"",";
				put "			TextEditBox,";
				put "			{Set Text("" ";
				%if &testflag %then %do;
					put "			&ytest ";
					%if %length(&ymtc) %then %do;
						put "			&ymtc ";
					%end;
					%if %length(&yadjs) %then %do;
						put "			&yadjs ";
					%end;
					%if %length(&yconv) %then %do;
						put "			&yconv ";
					%end;					
				%end;
				%else %do;
					put "			&ytest ";
				%end;
				put "			 "")}";
				put "			),";					
			%end;	
			put "			Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
			put "	)";
			put "),";
			%if (&GpType = N) %then %do;
				put "column(""&AnnoGroup"")<<data type(character);";
			%end;
			%else %do;
				%if &ProcessName=JMPGenomicsBrowser %then %do;
					put "column(""&AnnoGroup"") << Set Property(""Value Ordering"",{""chr0"",""chr1"",""chr2"",""chr3"",""chr4"",""chr5"",";
					put "   ""chr6"",""chr7"",""chr8"",""chr9""});";
				%end;
			%end;			
			put "),";
			%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
				put "Here:data0&_suf_<< Data Filter(";
				put " Conditional(1),";
				%if %length(&ByVars)=0 %then %do;
					put " Local,";			
				%end;
				put " Add Filter(Columns(""&AnnoGroup"")), Display(:Name(""&AnnoGroup""), Size(210, min(400,&NumGps*20)), List Display), Mode(Show(1), Include(1))";
				put ");";
			%end;
			put "),";
			put "), //end Data Filter Context Box";	
			/*S1149704 sbrlsi*/ 
			/*%if &tNumGps>1 %then %do;	
				put "Here:data0&_suf_ << Color by Column(";
				put "&AnnoGroup, Color( 1 ),	Marker( 0 ),Continuous Scale( 0 ))";
			%end;*/
			%if %upcase(&ProcessName) = QKMIXEDMODEL %then %do;
				%if (%upcase(&MultiMarkerModel)= YES and %upcase(&SelModelProfile) = YES and &QKSelProf) %then %do;
					put ");";
				%end;
			%end;
			put "); //end manp function";
			
			put "////////manp_after ";
			put "Here:manp_after= Function({},{default local},";
			put "Here:dt = Today();";
			put "Eval(Parse(EvalInsert(""\[mpr^dt^ = Here:mp << Report;]\"")));";			
			put "agsum = Here:data0&_suf_ << Summary( Group(:&AnnoGroup), Max(:CumOrder), invisible);"; 
			put "Wait( 0.5 );";
			put "Current Data Table( agsum );";
			put "agsum << Sort(By(:Name(""Max(CumOrder)"")),Replace Table);";
			put "Here:_chrvalues = {};";
			put "Here:_maxvalues = {};";
			put "Here:_minvalues = {1};";
			put "For Each Row(";
			put "  Insert Into( Here:_chrvalues, Column(""&AnnoGroup"")[] );";
			put "  Insert Into( Here:_maxvalues, Column(""Max(CumOrder)"")[] );";
			put "  if(row()>1, Insert Into(Here:_minvalues, lag(Column(""Max(CumOrder)"")[],1)+1));";
			put ");";
			put "pvnchr = N Items( Here:_chrvalues );";
			put "Here:_agmax = max(Here:_maxvalues);";		
			put "Here:_lowery = -(.0736* &Axismax);";
			%if &NbyVal > 1 %then %do;
			
				put "Here:ppfsize = 800;";
				put "Eval(Parse(EvalInsert(""\[";
				put "  for(_b=1, _b<=&NbyVal, _b++,";
				put "     for(_i=1, _i<=&NPVars, _i++,";
				put "      try(";
				put "        mpr^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "  )));";
				put "  mpvlb << Prepend(";
				put "   VListBox(";
				%if ^%index(&ProcessName,MetaAnalysis) %then %do;
					put "   HListBox(";
					put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
					put "	),";
					put "   HListBox(";
					put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
					put "	),";
					put "   HListBox(";
					put "   	Text Box( "" "" ),";
					put "	),";	
				%end;
				put "  HListBox(TextBox("" Frame Size for Graphs Below""),";
				put "  frameSizeSlider^dt^ = SliderBox(num(200),num(800),Here:ppfsize,";
				put "  for(_b=1, _b<=&NbyVal, _b++,";
				put "     for(_i=1, _i<=&NPVars, _i++,";
				put "      try(";
				put "        mpr^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "  )));";				
				put ")))";
				put ")";
				put ']\")));';			
			
				put "for(_b=1, _b<=&NbyVal, _b++,";
				put "   for(_i=1, _i<=&NPVars, _i++,";
				put '    try(';
				put "Eval(Parse(EvalInsert(""\[";
				/*put "      mpr^dt^[_b][framebox(_i)] << FrameSize(650,175);";*/
				put "      mpr^dt^[_b][framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
				put "      mpr^dt^[_b][framebox(_i)]  << Marker Size(3);"; 
				put "      mpr^dt^[_b][framebox(_i)]  << Marker Drawing Mode(Normal);"; 
				%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
					put "      mpr^dt^[_b][AxisBox(1)] << Label Row( 1, {Automatic Tick Marks( 0 ), Show Major Labels( 0 )});";
					put "      mpr^dt^[_b][AxisBox(1)] << Label Row( 2, {Automatic Tick Marks( 1 ), Show Major Labels( 1 )});";
				%end;
				%else %do;
					put "      mpr^dt^[_b][AxisBox(1)] << Label Row( 1, {Automatic Tick Marks( 1 ), Show Major Labels( 0 )});";
				%end;		
				%do p=1 %to &NPVars;
					/*put "      mpr^dt^[_b][Axisbox(&p+1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(_lowery-4*((_lowery+&AxisMax)/175)), Max(&AxisMax)};";*/	
					put "      mpr^dt^[_b][Axisbox(&p+1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(&ymin), Max(&AxisMax)};";
					/*put "      mpr^dt^[_b][Axisbox(&p+1)] << {Inc( %sysevalf((&AxisMax-&AxisMin)/6) ), Show Major Ticks( 0 ), Show Minor Ticks( 0 ), Remove Axis Label, Min(-0.5)};";*/
					put "      mpr^dt^[_b][Axisbox(&p+1)] << {Inc( %sysevalf((&AxisMax-&AxisMin)/6) ), Show Major Ticks( 0 ), Show Minor Ticks( 0 ), Remove Axis Label};";
				%end;
				/*put "      mpr^dt^[_b][framebox(_i)] << add graphics script(";*/
				/*put "        TextSize(7);";*/
				/*S1149704 sbrlsi %do j=1 %to &tNumGps %by &byinc;*/
				/*
				%if &NumGps > 1 %then %do;
					%let byinc=1;
					%do j=1 %to &NumGps %by &byinc; 
						put "         Text(Center Justified, {(Here:_minvalues[&j]+Here:_maxvalues[&j])/2 ,_lowery}, ""&&SGroupn&j"");";
					%end;
				%end;
				*/
				/*put "      ); // end graphics script "; */
				put "if(_i==1,";
			      	put "      //show(mpr^dt^[_b][OutLineBox(1)]);";
				put "      gtitle= mpr^dt^[_b][OutLineBox(1)]<< Get Text;";
      				put "      //show(gtitle);";
      				put "      gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
      				put "      //show(gtitle);";
				put "      pos= Munger(gtitle, 1, ""Where("");";
				put "      //show(pos);";
				put "      if(pos, ";
				put "        	del= Trim(Substr(gtitle, pos - 6));";
				put "         	mpr^dt^[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
				put "          	gtitle= Trim(Substr(gtitle, 1, pos - 6));";				
				put "      );";
				put "      //show(gtitle);";
      				put "      mpr^dt^[_b][OutLineBox(1)] << {Set Title(""Significance Test for ""||gtitle), Image Export Display( Normal )};";				
				put ");";
				put ']\")));';	
				put ")));";	
				put "Close( agsum, no save );"; 
				
			%end;
			%else %do;
				put "Here:ppfsize = 800;";
				put "Eval(Parse(EvalInsert(""\[";
				put "     for(_i=1, _i<=&NPVars, _i++,";
				put "      try(";
				put "        mpr^dt^[framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "  ));";
				put "   mpvlb << Prepend(";
				put "   VListBox(";
				%if ^%index(&ProcessName,MetaAnalysis) %then %do;
					put "   HListBox(";
					put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
					put "	),";
					put "   HListBox(";
					put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
					put "	),";
					put "   HListBox(";
					put "   	Text Box( "" "" ),";
					put "	),";	
				%end;
				put "   HListBox(TextBox("" Frame Size for Graphs Below""),";
				put "     frameSizeSlider^dt^ = SliderBox(num(200),num(800),Here:ppfsize,";				
				put "     for(_i=1, _i<=&NPVars, _i++,";
				put "      try(";
				put "        mpr^dt^[framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "  ));";				
				put ");";
				put ")";
				put "))";
				put ']\")));';				
				put "   for(_i=1, _i<=&NPVars, _i++,";
				put '    try(';
				put "Eval(Parse(EvalInsert(""\[";
				/*put "      mpr^dt^[framebox(_i)] << FrameSize(650,175);";*/
				put "      mpr^dt^[framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
				/*%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
					put "   	mpr^dt^[framebox(_i)]  << Marker Size(3);";
					put "   	mpr^dt^[framebox(_i)]  << Marker Drawing Mode(Normal);";  
				%end;
				%else %do;*/
					put "      mpr^dt^[framebox(_i)]  << Marker Size(3);";
					put "      mpr^dt^[framebox(_i)]  << Marker Drawing Mode(Normal);";
				/*%end;*/
				%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
					put "      mpr^dt^[AxisBox(1)] << Label Row( 1, {Automatic Tick Marks( 0 ), Show Major Labels( 0 )});";
					put "      mpr^dt^[AxisBox(1)] << Label Row( 2, {Automatic Tick Marks( 1 ), Show Major Labels( 1 )});";
				%end;
				%else %do;
					put "      mpr^dt^[AxisBox(1)] << Label Row( 1, {Automatic Tick Marks( 1 ), Show Major Labels( 0 )});";
				%end;	
				%do p=1 %to &NPVars;
					/*put "      mpr^dt^[Axisbox(&p+1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(_lowery-4*((_lowery+&AxisMax)/175)), Max(&AxisMax)};";*/
					put "      mpr^dt^[Axisbox(&p+1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(&ymin), Max(&AxisMax)};";					
					/*put "      mpr^dt^[Axisbox(&p+1)] << {Inc( %sysevalf((&AxisMax-&AxisMin)/6) ), Show Major Ticks( 0 ), Show Minor Ticks( 0 ), Remove Axis Label, Min(-0.5)};";*/
					put "      mpr^dt^[Axisbox(&p+1)] << {Inc( %sysevalf((&AxisMax-&AxisMin)/6) ), Show Major Ticks( 0 ), Show Minor Ticks( 0 ), Remove Axis Label};";
				%end;
				/*put "      mpr^dt^[framebox(_i)] << add graphics script(";*/
				/*put "        TextSize(7);";*/
				/*S1149704 sbrlsi %do j=1 %to &tNumGps %by &byinc;*/
				/*
				%if &NumGps > 1 %then %do;
					%let byinc=1;
					%do j=1 %to &NumGps %by &byinc; 
						put "         Text(Center Justified, {(Here:_minvalues[&j]+Here:_maxvalues[&j])/2 ,_lowery}, ""&&SGroupn&j"");";
					%end;
				%end;
				*/
				/*put "      ); // end graphics script "; */
				%if %symexist(TraitVar) %then %do;
      					put "      mpr^dt^[OutLineBox(1)] << {Set Title(""Significance Test for &TraitVar""), Image Export Display( Normal )};";
      				%end;
      				%else %do;
      					put "      mpr^dt^[OutLineBox(1)] << {Set Title(""Significance Test""), Image Export Display( Normal )};";
      				%end;
				put ']\")));';	
				put "));";	
				put "Close( agsum, no save );"; 
			%end;
			put "); //end manp_after function";
							
			%if &_summchart %then %do;
				put "InsertInto(TabCodeList,expr(manp),Loc(TabButtonList,""Summary Chart"")+1);";
				put "InsertInto(TabAfterCodeList,expr(manp_after),Loc(TabButtonList,""Summary Chart"")+1);";
				put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+1);";
			%end;			
		%end;
		
		/*model fitness plot following manhattam plot*/
		%*put tNumGps _pvalplots i= &tNumGps &_pvalplots &i;
		%if (&tNumGps>1 and &_pvalplots and &i=0) %then %do; 
			%if &PlotFitVars %then %do;
				put " ";
				put "//model fitness plot following manhattam plot;";
				%if &_summchart and &NumGps > 1 %then %do;
					put "InsertInto(DataList,{{""Here:data0&_suf_""}},Loc(TabButtonList,""Summary Chart"")+2);";
					%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot &HaploTest"",Loc(TabButtonList,""Summary Chart"")+2);";					
					%end;
					%else %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot"",Loc(TabButtonList,""Summary Chart"")+2);";
					%end;
				%end;
				%else %do;
					put "InsertInto(DataList,{{""Here:data0&_suf_""}});";
					%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot &HaploTest"");";
					%end;
					%else %do;
						put "InsertInto(Here:TabButtonList,""Model Fitness Plot"");";
					%end;					
				%end;
				
				%macro modelfitnessplot_likem();
					put "////////// &MainFunName   ";
					put "Here:&MainFunName=function({},{default local},";			
					put "if(isempty(Here:data0&_suf_),";
					put "if(jg:drill_down == 0,";
					put " Here:data0&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
					put ");";
					%if %upcase(&OnlySig)=YES %then %do;
						%if %length(&Conv)=0 %then %let gtlt=%str(>);
						%else %let gtlt=%str(<);
						put "Here:data&i&_suf_<<select where(" @;
						put ":Name(""&PVar1"") &gtlt &r" @;
						%do p=2 %to &NPVars;
						put "& :Name(""&&PVar&p"") &gtlt &r" @;
						%end;

						put ")<<hide;";
					%end;  
					put "colnames = Here:data&i&_suf_<<GetColumnNames(String);";
					put "if(Contains(colnames,""_excluded"")>0,";
					put "	Here:data&i&_suf_<<select where(:_excluded==1)<<hide);";
					put "Here:data&i&_suf_ << exclude;";
					put "nhidden=0;";
					put "for each row(";
					put "  if(hidden(), nhidden=nhidden+1));";
					put "if(nhidden==nrow(Here:data&i&_suf_),";
					put " throw(""All p-values non-significant in this data set.""));";
					put "Here:data&i&_suf_ << Clear Select;";
					%if %length(&AnnoAccession) %then %do;
						put "column(""&AnnoAccession"")<<data type(character);";
					%end;
					%if %length(&AnnoGroup) %then %do;
						put "column(""&AnnoGroup"")<<data type(character);";
					%end;
					put "column(""&AnnoLoc"")<<data type(numeric);";
					put "column(""&AnnoLabel"") << label(1);";					
					%if &_pvalplots %then %do;
						put "column(""CumOrder"") << modeling type(nominal);";
					%end;					
					%if %upcase(&ProcessName) = QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
						%if (%upcase(&MultiMarkerModel)= YES and %upcase(&SelModelProfile) = YES and &QKSelProf) %then %do;
							%if (%length(&selMarkerVars_g)) and (%length(&selMarkerVars_t)) %then %do;
								put "	Here:data&i&_suf_ << Select Where(!Is Missing(:Name(""Selected_Geno"")[])|!Is Missing(:Name(""Selected_Trend"")[]));";
							%end;
							%else %if (%length(&selMarkerVars_g)) %then %do;
								put "	Here:data&i&_suf_ << Select Where(!Is Missing(:Name(""Selected_Geno"")[]));";
							%end;
							%else %if (%length(&selMarkerVars_t)) %then %do;
								put "	Here:data&i&_suf_ << Select Where(!Is Missing(:Name(""Selected_Trend"")[]));";
							%end;
							put "	Here:data&i&_suf_ << Markers(11);";
							put "	Here:data&i&_suf_ << Clear Select();";
						%end;
					%end;
					put ", //else";
					put "  currentdatatable(Here:data0&_suf_);";
					put "); //end isempty(data&_suf_)";
					%if %upcase(&ProcessName) = QKMIXEDMODEL %then %do; /*S1149704 sbrlsi*/
						%if (%upcase(&MultiMarkerModel)= YES and %upcase(&SelModelProfile) = YES and &QKSelProf) %then %do;
							put "Vlistbox(";
							%if (%length(&selMarkerVars_g)) and (%length(&selMarkerVars_t)) %then %do;
								/*put "	TextBox(""Markers selected in the multi-marker model of either the Genotype or Trend tests are represented by stars (*) in the plot."",<<FontColor(0),<< Set Wrap(800)),";*/
								put "	TextBox(""Two types of multi-marker model were fitted: one including all selected markers, and the other including all selected markers plus each unselected marker one at the time."",<<FontColor(0),<< Set Wrap(800)),";
							%end;
							%else %if (%length(&selMarkerVars_g)) %then %do;
								/*put "	TextBox(""Markers selected in the multi-marker model are represented by stars (*) in the plot."",<<FontColor(0),<< Set Wrap(800)),";*/				
								put "	TextBox(""Two types of multi-marker model were fitted: one including all selected markers, and the other including all selected markers plus each unselected marker one at the time."",<<FontColor(0),<< Set Wrap(800)),";
							%end;
							%else %if (%length(&selMarkerVars_t)) %then %do;
								/*put "	TextBox(""Markers selected in the multi-marker model are represented by stars (*) in the plot."",<<FontColor(0),<< Set Wrap(800)),";*/				
								put "	TextBox(""Two types of multi-marker model were fitted: one including all selected markers, and the other including all selected markers plus each unselected marker one at the time."",<<FontColor(0),<< Set Wrap(800)),";
							%end;
							put "	TextBox("" "",<<FontColor(0),<< Set Wrap(800)),";
						%end;
					%end;
					put "Here:&MainVLBoxName=Vlistbox(";

					put "   Here:ftp=Graph Builder(";
					put "   	Show Control Panel( 0 ),";
					put "   	Legend Position( ""Right"" ),";
					put "		Graph Spacing( 3 ),";
					put "		Spacing Borders( 1 ),";					
					put "   	Variables(";
					%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
						put "		X( ""&AnnoGroup"" )";
						put "		,X( :CumOrder, Position( 1 ) )";
						/*put "		,Color( ""&AnnoGroup"" )";*/
					%end;
					%else %do;
						put "		,X( :CumOrder )";
					%end;							
					put "   		, Y(Column(""&&&PrefixGroupVar.1""))";
					%do f=2 %to &NGroup;
						put "           	, Y(Column(""&&&PrefixGroupVar.&f""), Position(1))";
					%end;		
					put "   	)";
					%if %length(&ByVars)>0 %then %do;
						put "            , By(&JBy)";
					%end;
					put "   		, Elements("@;
					put "Points( X(1)"@;
					%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
						put ", X(2)"@;
					%end;
					%do f=1 %to &NGroup;
						put ", Y( &f )"@;
					%end;
					put ", Legend( 3 ) )";
					/*
					put "   			, Line( X(1)"@;
					%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
						put ", X(2)"@;
					%end;							
					%do f=1 %to &NGroup;
						put ", Y( &f )"@;
					%end;
					put ", Legend( 4 ) )";
					*/
					put "   		),";
					put "   		SendToReport(";						
					put "   			Dispatch(";
					put "   				{},";
					put "   				""400"",";
					put "   				ScaleBox,";
					put "   				{Legend Model(";
					put "   					3,";
					/*
					%let k= 0;
					%do f=1 %to &NGroup;
						put "   					Base( &k, 0, 0, Item ID( ""&&&PrefixGroupVar.&f"", 1 ) ),";
						%let k= %eval(&k + 1);
					%end;
					*/					
					put "   				)}";
					put "   			),";
					put "   			Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
					%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;					
						put "   			Dispatch( {}, ""X title"", TextEditBox, {Set Text( ""&AnnoGroup"" )} ),";
					%end;
					%else %do;
						put "   			Dispatch( {}, ""X title"", TextEditBox, {Set Text( ""CumOrder"" )} ),";
					%end;
					put "   			Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Y"" )} ),";
					put "				Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 1 )} ),";
					put "   		)";
					put "   	);";
					%if (&GpType = N) %then %do;
						put "column(""&AnnoGroup"")<<data type(numeric);";
					%end;
					/*S1149704 sbrlsi*/
					/*%if &tNumGps>1 %then %do; 
						put "Here:data0&_suf_ << Color by Column(";
						put "&AnnoGroup, Color( 1 ),	Marker( 0 ),Continuous Scale( 0 ))";
					%end;*/
					put ");";
					%if %length(&AppendToVLBoxName) %then %do;
						put "Here:&AppendToVLBoxName << Append(Here:&MainVLBoxName);";
					%end;
					%if %upcase(&ProcessName) = QKMIXEDMODEL %then %do;
						%if (%upcase(&MultiMarkerModel)= YES and %upcase(&SelModelProfile) = YES and &QKSelProf) %then %do;
							put ");";
						%end;
					%end;
					put "); //end &MainFunName function";

					put "////////&AfterMainFunName ";
					put "Here:&AfterMainFunName= Function({},{default local},";
					put "Here:dt = Today();";
					put "Eval(Parse(EvalInsert(""\[ftpr^dt^ = Here:ftp << Report;]\"")));";			
					put "agsum = Here:data0&_suf_ << Summary( Group(:&AnnoGroup), Max(:CumOrder), invisible);"; 
					put "Wait( 0.5 );";
					put "Current Data Table( agsum );";
					put "agsum << Sort(By(:Name(""Max(CumOrder)"")),Replace Table);";
					put "Here:_chrvalues = {};";
					put "Here:_maxvalues = {};";
					put "Here:_minvalues = {1};";
					put "For Each Row(";
					put "  Insert Into( Here:_chrvalues, Column(""&AnnoGroup"")[] );";
					put "  Insert Into( Here:_maxvalues, Column(""Max(CumOrder)"")[] );";
					put "  if(row()>1, Insert Into(Here:_minvalues, lag(Column(""Max(CumOrder)"")[],1)+1));";
					put ");";
					put "pvnchr = N Items( Here:_chrvalues );";
					put "Here:_agmaxf = max(Here:_maxvalues);";		
					put "Here:_loweryf = &minGroup - (&maxGroup - &minGroup)/&maxGroup;";
					put "for(_b=1, _b<=&NbyVal, _b++,";
					put "   for(_i=1, _i<=&NGroup, _i++,";
					put "    try(";
					put "Eval(Parse(EvalInsert(""\[";
					put "      ftpr^dt^[_b][framebox(_i)] << FrameSize(650,175);";
					/*put "      ftpr^dt^[_b][framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed""));";*/
					put "      ftpr^dt^[_b][framebox(_i)]  << Marker Size(1);";
					put "      ftpr^dt^[_b][framebox(_i)]  << Marker Drawing Mode(Normal);";  
					%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
						put "      ftpr^dt^[_b][AxisBox(1)] << Label Row( 1, {Automatic Tick Marks( 0 ), Show Major Labels( 0 )});";
						put "      ftpr^dt^[_b][AxisBox(1)] << Label Row( 2, {Automatic Tick Marks( 1 ), Show Major Labels( 1 )});";
					%end;
					%else %do;
						put "      ftpr^dt^[_b][AxisBox(1)] << Label Row( 1, {Automatic Tick Marks( 1 ), Show Major Labels( 0 )});";
					%end;								
					/*put "   	ftpr^dt^[_b][Axisbox(2)] << Inc( 2*^_agmaxf^ );";*/
					put "   	ftpr^dt^[_b][Axisbox(2)] << Min(^_loweryf^);";
					put "   	ftpr^dt^[_b][Axisbox(2)] << Max(&maxGroup + (&maxGroup - &minGroup)/&maxGroup);";   
					put "   	ftpr^dt^[_b][Axisbox(2)] << Format( ""Fixed Dec"", 9, 2 );";
					
					/*put "      ftpr^dt^[_b][Axisbox(2*_i-1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(^_loweryf^), Max(&maxGroup + (&maxGroup - &minGroup)/&maxGroup)};";*/			
					/*put "      ftpr^dt^[_b][Axisbox(2*_i)] << {Inc( %sysevalf((&AxisMax-&AxisMin)/6) ), Show Major Ticks( 0 ), Show Minor Ticks( 0 ), Remove Axis Label, Min(-0.5)};";*/
					/*put "      ftpr^dt^[_b][framebox(_i)] << add graphics script(";*/
					/*put "        TextSize(7);";*/
					/*S1149704 sbrlsi %do j=1 %to &tNumGps %by &byinc;*/
					/*%if &NumGps > 1 %then %do; 
						%let byinc=1;
						%do j=1 %to &NumGps %by &byinc; 
							put "         Text(Center Justified, {(Here:_minvalues[&j]+Here:_maxvalues[&j])/2 ,^_loweryf^ + 0.0}, ""&&SGroupn&j"");";
						%end;
					%end;					*/
					/*put "      ); // end graphics script "; */
					%if %symexist(TraitVar) %then %do;
      						put "      ftpr^dt^[_b][OutLineBox(1)] << {Set Title(""Significance Test for &TraitVar""), Image Export Display( Normal )};";
      					%end;
      					%else %do;
      						put "      ftpr^dt^[_b][OutLineBox(1)] << {Set Title(""Significance Test""), Image Export Display( Normal )};";
      					%end;					
					put "]\"")));";	
					put ")));";			
					%if (&GpType = N) %then %do;
						put "column(""&AnnoGroup"")<<data type(character);";
					%end;
					put "Close( agsum, no save );"; 
					put "Here:ppfsize = 800;";
					put "Eval(Parse(EvalInsert(""\[";
					put "  for(_b=1, _b<=&NbyVal, _b++,";
					put "     for(_i=1, _i<=&NGroup, _i++,";
					put '      try(';
					put "        ftpr^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
					put "  )));";					
					put "   &MainVLBoxName << Prepend(HListBox(TextBox("" Frame Size for Graphs Below""),";
					put "      frameSizeSlider^dt^ = SliderBox(num(200),num(800),Here:ppfsize,";
					put ")))";
					put "]\"")));";
					put "); //end &AfterMainFunName function";
				%mend;
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					/*%put HaploTest= &HaploTest;*/
					/*%put global_flag= &global_flag;*/
					%if %upcase(&HaploTest)= GLOBAL and &global_flag %then %do;
						%put MainFunNameGlobal;
					
						%let MainFunName=fitp1g;
						%let MainVLBoxName=fpvlb1g;
						%let AfterMainFunName=fitp1g_after;
						%let AppendToVLBoxName=;
						%let PrefixGroupVar= Group1Var;		
						%let NGroup= &NGroup1Val;
						%let minGroup= &minGroup1Val;
						%let maxGroup= &maxGroup1Val; 
						%modelfitnessplot_likem();
						%let AfterFunList= &AfterMainFunName;
						%let AnchorVLBoxName= &MainVLBoxName;
						%let AnchorFunName= &MainFunName;

						%if %length(&Group2OverlayVars) %then %do;					
							%let MainFunName=fitp2g;
							%let MainVLBoxName=fpvlb2g;
							%let AfterMainFunName=fitp2g_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group2Var;		
							%let NGroup= &NGroup2Val;
							%let minGroup= &minGroup2Val;
							%let maxGroup= &maxGroup2Val; 
							%modelfitnessplot_likem();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if %length(&Group3OverlayVars) %then %do;					
							%let MainFunName=fitp3g;
							%let MainVLBoxName=fpvlb3g;
							%let AfterMainFunName=fitp3g_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group3Var;		
							%let NGroup= &NGroup3Val;
							%let minGroup= &minGroup3Val;
							%let maxGroup= &maxGroup3Val; 
							%modelfitnessplot_likem();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;
						
						%if &_summchart and &NumGps > 1 %then %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName),Loc(TabButtonList,""Summary Chart"")+2);";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}),Loc(TabButtonList,""Summary Chart"")+2);";
							put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+2);";
						%end;
						%else %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName));";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}));";
							put "InsertInto(TabOpenList,1);";				
						%end;
					%end;
					%else %if %upcase(&HaploTest)= SINGLE and &single_flag %then %do;
						%put MainFunNameSingle;
					
						%let MainFunName=fitp1s;
						%let MainVLBoxName=fpvlb1s;
						%let AfterMainFunName=fitp1s_after;
						%let AppendToVLBoxName=;
						%let PrefixGroupVar= Group1Var;		
						%let NGroup= &NGroup1Val;
						%let minGroup= &minGroup1Val;
						%let maxGroup= &maxGroup1Val; 
						%modelfitnessplot_likem();
						%let AfterFunList= &AfterMainFunName;
						%let AnchorVLBoxName= &MainVLBoxName;
						%let AnchorFunName= &MainFunName;

						%if %length(&Group2OverlayVars) %then %do;					
							%let MainFunName=fitp2s;
							%let MainVLBoxName=fpvlb2s;
							%let AfterMainFunName=fitp2s_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group2Var;		
							%let NGroup= &NGroup2Val;
							%let minGroup= &minGroup2Val;
							%let maxGroup= &maxGroup2Val; 
							%modelfitnessplot_likem();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if %length(&Group3OverlayVars) %then %do;					
							%let MainFunName=fitp3s;
							%let MainVLBoxName=fpvlb3s;
							%let AfterMainFunName=fitp3s_after;
							%let AppendToVLBoxName= &AnchorVLBoxName;
							%let PrefixGroupVar= Group3Var;		
							%let NGroup= &NGroup3Val;
							%let minGroup= &minGroup3Val;
							%let maxGroup= &maxGroup3Val; 
							%modelfitnessplot_likem();
							%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
						%end;

						%if &_summchart and &NumGps > 1 %then %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName),Loc(TabButtonList,""Summary Chart"")+3);";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}),Loc(TabButtonList,""Summary Chart"")+3);";
							put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+3);";
						%end;
						%else %do;
							put "InsertInto(TabCodeList,expr(&AnchorFunName));";
							put "InsertInto(TabAfterCodeList,expr({&AfterFunList}));";
							put "InsertInto(TabOpenList,1);";				
						%end;		
					%end;
				%end;
				%else %do;
					%let MainFunName=fitp1;
					%let MainVLBoxName=fpvlb1;
					%let AfterMainFunName=fitp1_after;
					%let AppendToVLBoxName=;
					%let PrefixGroupVar= Group1Var;		
					%let NGroup= &NGroup1Val;
					%let minGroup= &minGroup1Val;
					%let maxGroup= &maxGroup1Val; 
					%modelfitnessplot_likem();
					%let AfterFunList= &AfterMainFunName;
					%let AnchorVLBoxName= &MainVLBoxName;
					%let AnchorFunName= &MainFunName;

					%if %length(&Group2OverlayVars) %then %do;					
						%let MainFunName=fitp2;
						%let MainVLBoxName=fpvlb2;
						%let AfterMainFunName=fitp2_after;
						%let AppendToVLBoxName= &AnchorVLBoxName;
						%let PrefixGroupVar= Group2Var;		
						%let NGroup= &NGroup2Val;
						%let minGroup= &minGroup2Val;
						%let maxGroup= &maxGroup2Val; 
						%modelfitnessplot_likem();
						%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
					%end;

					%if %length(&Group3OverlayVars) %then %do;					
						%let MainFunName=fitp3;
						%let MainVLBoxName=fpvlb3;
						%let AfterMainFunName=fitp3_after;
						%let AppendToVLBoxName= &AnchorVLBoxName;
						%let PrefixGroupVar= Group3Var;		
						%let NGroup= &NGroup3Val;
						%let minGroup= &minGroup3Val;
						%let maxGroup= &maxGroup3Val; 
						%modelfitnessplot_likem();
						%let AfterFunList= %str(&AfterFunList %str(;) &MainFunName %str(;) &AfterMainFunName);
					%end;

					%if &_summchart and &NumGps > 1 %then %do;
						put "InsertInto(TabCodeList,expr(&AnchorFunName),Loc(TabButtonList,""Summary Chart"")+2);";
						put "InsertInto(TabAfterCodeList,expr({&AfterFunList}),Loc(TabButtonList,""Summary Chart"")+2);";
						put "InsertInto(TabOpenList,1,Loc(TabButtonList,""Summary Chart"")+2);";
					%end;
					%else %do;
						put "InsertInto(TabCodeList,expr(&AnchorFunName));";
						put "InsertInto(TabAfterCodeList,expr({&AfterFunList}));";
						put "InsertInto(TabOpenList,1);";				
					%end;
				%end;
			%end;
		%end;
		
		%if &ProcessName = MissingGenotypeSummary %then %do;
			%if &i=0 %then %do;
				put 'Here:mgdist=Function({},{default local},';
				put "if(isempty(Here:data0&_suf_),";
				put " Here:data0&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
				%if %upcase(&OnlySig)=YES %then %do;
					%if %length(&Conv)=0 %then %let gtlt=%str(>);
					%else %let gtlt=%str(<);
					put "Here:data&i&_suf_<<select where(" @;
					put ":Name(""&PVar1"") &gtlt &r" @;
					%do p=2 %to &NPVars;
					put "& :Name(""&&PVar&p"") &gtlt &r" @;
					%end;

					put ")<<hide;";
				%end;  
				put "colnames = Here:data&i&_suf_<<GetColumnNames(String);";
				put 'if(Contains(colnames,"_excluded")>0,';
				put "	Here:data&i&_suf_<<select where(:_excluded==1)<<hide);";
				put "Here:data&i&_suf_ << exclude;";
				put "nhidden=0;";
				put "for each row(";
				put "  if(hidden(), nhidden=nhidden+1));";
				put "if(nhidden==nrow(Here:data&i&_suf_),";
				put " throw(""All p-values non-significant in this data set.""));";
				put "Here:data&i&_suf_ << Clear Select;";
				%if %length(&AnnoAccession) %then %do;
					put "column(""&AnnoAccession"")<<data type(character);";
				%end;
				%if %length(&AnnoGroup) %then %do;
					put "column(""&AnnoGroup"")<<data type(character);";
				%end;
				put "column(""&AnnoLoc"")<<data type(numeric);";						
				put "column(""&AnnoLabel"") << label(1);";				
				%if &_pvalplots %then %do;
					put "column(""CumOrder"") << modeling type(nominal);";
				%end;				
				put ", //else";
				put " Current Data Table(Here:data0&_suf_);";
				put ");";
				put "VListbox(";
				%do kk=1 %to &nrectraits;
					put "   HListBox(";
					put "   	Text Box( "" &&TextSummary&kk "", << set wrap(800) ),";
					put "	),";
				%end;
				put "   HListBox(";
				put "   	Text Box( "" "" ),";
				put "	),";						
				put 'Distribution(';
				put '  Continuous Distribution(';
				put "    Column(""Number of Missing for Trait=&cval1""),";
				put '    Quantiles(0),';
				put '    Moments(0),';
				put '    Vertical(0)';
				put '  ),';
				put '  Continuous Distribution(';
				put "    Column(""Number of Missing for Trait=&cval2""),";
				put '    Quantiles(0),';
				put '    Moments(0),';
				put '    Vertical(0)';
				put '  ),' @;
				%if %length(&ByVars)>0 or &tNumGps>1 %then %do;
					put "  By(&JByAnno),";
				%end;
		  		put "SendToReport(";
	 			put "	Dispatch(";
	 			put " 		""Number of Missing for Trait=&cval1"", ";
	 			put "		""1"",";
	 			put "		ScaleBox,";
	 			put "		{Format( ""Fixed Dec"", 12, 0 )}";
				put "	),";
	 			put "		Dispatch(";
	 			put " 		""Number of Missing for Trait=&cval2"", ";
	 			put "		""1"",";
	 			put "		ScaleBox,";
	 			put "		{Format( ""Fixed Dec"", 12, 0 )}";
				put "	),";
				put "),";				
				put '));';
				put ');'; /* Ends function */

				put "InsertInto(Here:TabButtonList,""All Distributions"");";
				put "InsertInto(TabCodeList,expr(mgdist));";
				put "InsertInto(TabAfterCodeList,"" "");";
				put "InsertInto(TabOpenList,1);";
				put "InsertInto(DataList,{{""Here:data&i&_suf_""}});";
			%end;
		%end;
		
		/************ Marker Properties only code ************/ 		
		%if &ProcessName = MarkerProperties %then %do;

			%if &i=0 %then %do;
				put 'Here:mpdistfunc=Function({},{default local},';

				put "if(isempty(Here:data0&_suf_),";
				put " Here:data0&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
				%if %upcase(&OnlySig)=YES %then %do;
					%if %length(&Conv)=0 %then %let gtlt=%str(>);
					%else %let gtlt=%str(<);
					put "Here:data&i&_suf_<<select where(" @;
					put ":Name(""&PVar1"") &gtlt &r" @;
					%do p=2 %to &NPVars;
					put "& :Name(""&&PVar&p"") &gtlt &r" @;
					%end;

					put ")<<hide;";
				%end;  
				put "colnames = Here:data&i&_suf_<<GetColumnNames(String);";
				put 'if(Contains(colnames,"_excluded")>0,';
				put "	Here:data&i&_suf_<<select where(:_excluded==1)<<hide);";
				put "Here:data&i&_suf_ << exclude;";
				put "nhidden=0;";
				put "for each row(";
				put "  if(hidden(), nhidden=nhidden+1));";
				put "if(nhidden==nrow(Here:data&i&_suf_),";
				put " throw(""All p-values non-significant in this data set.""));";
				put "Here:data&i&_suf_ << Clear Select;";

				%if %length(&AnnoAccession) %then %do;
					put "column(""&AnnoAccession"")<<data type(character);";
				%end;
				%if %length(&AnnoGroup) %then %do;
					put "column(""&AnnoGroup"")<<data type(character);";
				%end;
				put "column(""&AnnoLoc"")<<data type(numeric);";
						
				put "column(""&AnnoLabel"") << label(1);";		
				%if &_pvalplots %then %do;
					put "column(""CumOrder"") << modeling type(nominal);";
				%end;
				put ", //else";
				put " Current Data Table(Here:data0&_suf_);";
				put ");";			
				put 'Vlistbox(';
				put '_mpd=Distribution(';
				put '  Continuous Distribution(';
				put "    Column(""MAF""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';		
				put '    Vertical(0)';
				put '  ),';
				put '  Continuous Distribution(';
				put "    Column(""PropMissing""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';
				put '    Vertical(0)';
				put '  ),' @;
				%if %length(&ByVars)>0 or &tNumGps>1 %then %do;
					put "  By(&JByAnno),";
				%end;
				put 'SendToReport(';
				put ' Dispatch(';
				put ' {},';
				put '"MAF",';
				put '  OutlineBox,';
				put '  Set Title("Minor Allele Freq (MAF)")),';
				put ' Dispatch(';
				put ' {},';
				put '"PropMissing",';
				put '  OutlineBox,';
				put '  Set Title("Missing Genotype Proportion"))';		
				put ')';
				put ');';
				put '));'; /* Ends new function and vlist */   
			
				put "InsertInto(Here:TabButtonList,""All Distributions"");";
				put "InsertInto(TabCodeList,expr(mpdistfunc));";
				put "InsertInto(TabAfterCodeList,"" "");";
				put "InsertInto(TabOpenList,1);";
				put "InsertInto(DataList,{{""Here:data0&_suf_""}});";			
				
				/* Individual missing genotype proportion distribution */
				
				put 'Here:igdistfunc=Function({},{default local},';

				put "if(isempty(Here:igdist),";
				put " Here:igdist=open(""&ClientOutPath.&PropMissDataName..sas7bdat"", invisible);";
				put ",//else";
				put " Current Data Table(Here:igdist)";
				put ");";
				
				put "Vlistbox(";
				put "Distribution(Continuous Distribution( Column( :PropMissingGenotypes ), Vertical( 0 ) ));";
				put ")); //end vlistbox,igdistfunc";
				put "InsertInto(Here:TabButtonList,""Individual Missing Genotype Proportion"");";
				put "InsertInto(TabCodeList,expr(igdistfunc));";
				put "InsertInto(TabAfterCodeList,"" "");";
				put "InsertInto(TabOpenList,1);";
				put "InsertInto(DataList,{{""Here:igdist""}});";				
			
			%end;

			/*
			put '_rmpd=_mpd<<Report;';
			put '_rmpd[""MAF""]<<SetTitle(""Minor Allele Frequency (MAF)"");';
			put '_rmpd[""PropMissing""]<<SetTitle(""Missing Genotype Proportion"");';
			*/
			%if %upcase(&CreateCellPlot)=YES and &i=0 %then %do;
			 	put "Here:cellplot = Function({},{default local},";
			 	put "Here:datac=open(""&ClientOutPath.&NumGenoDataName..sas7bdat"", invisible);";
				%if (%upcase(&TransposeCellPlotData)=YES) %then %do;
			  		%do ij=1 %to &nobs;
						put "  Column(""Subject&ij"")<<Set Modeling Type(""Ordinal"");";
						put "  Column(""Subject&ij"")<<Set Property(""Value Colors"",{""2""=21,""1""=1,""0""=19});";			
			  		%end;		 
				%end;
				%else %do;
					%do ij=1 %to &nm;
						put "  Column(""&&Label&ij"")<<Set Modeling Type(""Ordinal"");";		 
						put "  Column(""&&Label&ij"")<<Set Property(""Value Colors"",{""2""=21,""1""=1,""0""=19});";
					%end;
				%end;

				put ' cp_uf0=EvalInsert("\[';
				%if (%upcase(&TransposeCellPlotData)=YES) %then %do;
					put "   Here:cp_hfsize0=5*&nobs;";
					put "   Here:cp_vfsize0=max(100,7*&nm);";
				%end;
				%else %do;
					put "   Here:cp_hfsize0=max(800,5*&nm);";
					put "   Here:cp_vfsize0=2*&nobs;";					
				%end;				
				put ' Here:cp_update_Frames0 = Expr(';
				put ' ]\");';
				put ' cp_uf0||=EvalInsert("\[';
				put '   for(j=1,j<=NItems(Here:cpr^dt^),j++,';
				put '    try(';
				put '      Here:cpr^dt^[j][cellplotbox(1)] << FrameSize(cp_hfsize0,cp_vfsize0);';
				put ' ));';
				put ' ]\");';
				put ' cp_uf0||=EvalInsert("\[';
				put '   for(j=1,j<=NItems(Here:cpr^dt^),j++,';
				put '    try(';
				put '      Here:cpr^dt^[j][cellplotbox(1)] << FrameSize(cp_hfsize0,cp_vfsize0);';
				put ' ));';
				put ' ]\");';
				put ' cp_uf0||=");";';
				put ' Eval(Parse(cp_uf0));';
				put ' cp_uf0_2 = EvalInsert("\[';
				put ' cp_update_HFS_TextEditBox0 = Expr(';
				put '    value = Round(Num(cp_hfsize0),0);';
				put '    cp_HFS_TEB0 << SetText(char(num(value)));';
				put '    cp_HFS_TEB0 << Reshow;';
				put '    cp_hfsize0 = value;';
				put '    cp_update_Frames0;';
				put ' );';
				put ' cp_update_HFS_SliderBox0 = Expr(';
				put '    cp_hfsize0 = num(cp_HFS_TEB0<<GetText);';
				put '    cp_HFS_SB0 << Reshow;';
				put '    cp_update_HFS_TextEditBox0;';
				put ' );';
				put ' cp_update_VFS_TextEditBox0 = Expr(';
				put '    value = Round(Num(cp_vfsize0),0);';
				put '    cp_VFS_TEB0 << SetText(char(num(value)));';
				put '    cp_VFS_TEB0 << Reshow;';
				put '    cp_vfsize0 = value;';
				put '    cp_update_Frames0;';
				put ' );';
				put ' cp_update_VFS_SliderBox0 = Expr(';
				put '    cp_vfsize0 = num(cp_VFS_TEB0<<GetText);';
				put '    cp_VFS_SB0 << Reshow;';
				put '    cp_update_VFS_TextEditBox0;';
				put ' );';
				put ' ]\");';
				put ' Eval(Parse(cp_uf0_2));';
				put ' cp_vfsteb0 = EvalInsert("\[';
				put '    cp_VFS_TEB0 = TextEditBox(char(num(cp_vfsize0)),';
				put '       <<Script(cp_update_VFS_SliderBox0)';
				put '    )';
				put ' ]\");';
				put ' cp_vfssb0 = EvalInsert("\[';
				put '    cp_VFS_SB0 = SliderBox(50,20000,cp_vfsize0,';
				put '       cp_update_VFS_TextEditBox0';
				put '    )';
				put ' ]\");';
				put ' cp_vfssb0_2 = EvalInsert("\[';
				put '    cp_VFS_SB0 << SetValue(num(cp_vfsize0));';
				put '    cp_VFS_SB0 << SetWidth(120);';
				put ' ]\");';
				put ' cp_hfsteb0 = EvalInsert("\[';
				put '    cp_HFS_TEB0 = TextEditBox(char(num(cp_hfsize0)),';
				put '       <<Script(cp_update_HFS_SliderBox0)';
				put '    )';
				put ' ]\");';
				put ' cp_hfssb0 = EvalInsert("\[';
				put '    cp_HFS_SB0 = SliderBox(50,20000,cp_hfsize0,';
				put '       cp_update_HFS_TextEditBox0';
				put '    )';
				put ' ]\");';
				put ' cp_hfssb0_2 = EvalInsert("\[';
				put '    cp_HFS_SB0 << SetValue(num(cp_hfsize0));';
				put '    cp_HFS_SB0 << SetWidth(120);';
				put ' ]\");				';

				put " ScrollBox(size(850,800),";
				put " vlistbox(";
				%if &PloidyNum = 2 %then %do;
					put '  panelbox("Legend",';
					put '   t1=Textbox("Homozygous for Major Allele"),';
					put '   t1<<Font Color(3),';
					put '   t1<<Bullet Point(1);';
					put '   t2=Textbox("Heterozygous"),';
					put '   t2<<Font Color(1),';
					put '   t2<<Bullet Point(1);';
					put '   t3=Textbox("Homozygous for Minor Allele"),';
					put '   t3<<Font Color(5),';
					put '   t3<<Bullet Point(1);';
					put '  ),';
				%end;
				put '   HListBox(';
				put '      Text Box( " Vertical Frame Size " ),';
				put '      HListBox(';
				put '         eval(parse(cp_vfsteb0)),';
				put '         eval(parse(cp_vfssb0)),';
				put '         eval(parse(cp_vfssb0_2))';
				put '      ),';
				put '      Text Box( " Horizontal Frame Size " ),';
				put '      HListBox(';
				put '         eval(parse(cp_hfsteb0)),';
				put '         eval(parse(cp_hfssb0)),';
				put '         eval(parse(cp_hfssb0_2))';
				put '      )';
				put '   ),				';
				
				put ' Here:cp= Cell Plot(';
				%if (%upcase(&TransposeCellPlotData)=YES) %then %do;
					  put "  Y(";
					  %do ij=1 %to &nobs-1;
						put "  Column(""Subject&ij""),";
					  %end;
					  put "  Column(""Subject&nobs"")), Scale Uniformly, Label(:Marker),";
					  put "	SendToReport( Dispatch( {}, """", CellPlotBox, {Frame Size(5*&nobs,max(100,7*&nm)), No separator lines( 1 )} ) )";
				%end;
				%else %do;
					put "  Y(";
					%do ij=1 %to &nm-1;
						put "  Column(""&&Label&ij""),";
					%end;
					put "  Column(""&&Label&nm"")), Scale Uniformly,";
		  			put "	SendToReport( Dispatch( {}, """", CellPlotBox, {Frame Size(max(800,5*&nm),2*&nobs), No separator lines( 1 )} ) )";
					%if %length(&ByVars)>0 %then %do;
					  	put " , By(&JBy),";
					%end;
				%end;	
				put ' )';
				put ')';
				put ')';
				put ');';
				
				put ' Here:cellplot_after = Function({},{default local},';
				put '       cef=EvalInsert("\[Here:cpr^dt^=Here:cp<<report;';
				put '       for(j=1,j<=NItems(cpr^dt^),j++,';
				%if (%upcase(&TransposeCellPlotData)=YES) %then %do;				
					put "        cpr^dt^[j][cellplotbox(1)]<<FrameSize(5*&nobs,max(100,7*&nm)));";
				%end;
				%else %do;
					put "        cpr^dt^[j][cellplotbox(1)]<<FrameSize(max(800,5*&nm),2*&nobs));";
				%end;
				put '       ]\");';
				put '       try(Eval(parse(cef)));';
				put ' );			 ';
				put "InsertInto(Here:TabButtonList,""Cell Plot"");";
				put "InsertInto(TabCodeList,expr(cellplot));";
				put "InsertInto(TabAfterCodeList,expr(cellplot_after));";
				put "InsertInto(TabOpenList,1);";
				put "InsertInto(DataList,{{""Here:datac""}});";	
			%end;

			%if %upcase(&ChartFreqs)=YES and &i=0 %then %do;
				 put "Here:chartfunc = Function({},{default local},";
				 put "if(isEmpty(Here:dataa),";
				 put "Here:dataa=open(""&ClientOutPath.&AlleleFreqDataName..sas7bdat"",invisible);";
				 put "Here:datag=open(""&ClientOutPath.&GenotypeFreqDataName..sas7bdat"",invisible);";
				 put ");";
				 put "Here:dt = Today();";
				 put "CurrentDataTable(Here:dataa);";
				 put "fppfsize = 800;";
			 
			 	/* frame size */
				put "      ScrollBox(size(850,800),";
				put "      VListBox(TextBox("" Frame Size for Graphs Below""),";
				put "       fssf&i&_suf_=EvalInsert(""\[frameSizeSlider = SliderBox(num(50),num(500),fppfsize,";
				put "      for(j=1,j<=NItems(Here:afr^dt^),j++,";
				put "       Here:afr^dt^[j][framebox(1)]<<FrameSize(fppfsize,fppfsize));";  
				put "      for(j=1,j<=NItems(Here:gfr^dt^),j++,";
				put "       Here:gfr^dt^[j][framebox(1)]<<FrameSize(fppfsize,fppfsize));";
				put '       )]\");' @@;
				put "Eval(parse(fssf&i&_suf_));";
				put ',';
				put '   hlistbox(';	
				put '      OutlineBox("Allele Frequencies",';
				put '      vlistbox(Here:af=Chart(Freq( :Count), X(Allele), Y(N), Show Y Legend(0),';
				put '               Show Level Legend(0), Bar Chart(1)';
				put "               , By(&AnnoLabel) " @@;
				%if %length(&ByVars)>0 %then %do;
					 put ", Grouping(&JBy),";
				%end;
				put '))), ';

			 	put "      currentdatatable(Here:datag);";
			 	put "	   column(""Genotype"") << modeling type(nominal);";
			 	put '      OutlineBox("Genotype Frequencies",';
			 	put '      vlistbox(Here:gf=Chart(Freq( :Count), X(Genotype), Y(N), Show Y Legend(0),';
				put "               Show Level Legend(0), Bar Chart(1), By(&AnnoLabel) " @@;
				%if %length(&ByVars)>0 %then %do;
					 put ", Grouping(&JBy),";
				%end;
				put "))";
				put ")";
				put ")));";
							
				put ");//end chart function";	
				
				put "Here:chart_after = Function({},{default local},";
				%if &nm > 1 %then %do;
					 put "      aei=EvalInsert(""\[Here:afr^dt^=Here:af<<report;";
					 put "      for(j=1,j<=NItems(afr^dt^),j++,";
					 put "       afr^dt^[j][framebox(1)]<<FrameSize(100,100));";
					 put "      ]\"");";
					 put "      try(Eval(parse(aei)));";
		  		%end;
		  		%if &nm > 1 %then %do;
					 put "      gei=EvalInsert(""\[Here:gfr^dt^=Here:gf<<report;";
					 put "      for(j=1,j<=NItems(gfr^dt^),j++,";
					 put "       gfr^dt^[j][framebox(1)]<<FrameSize(100,100));";
					 put "      ]\"");";
					 put "      try(Eval(parse(gei)));";		 
				%end;
				put ");";
			
				put "InsertInto(Here:TabButtonList,""Marker Frequencies"");";
				put "InsertInto(TabCodeList,expr(chartfunc));";
				put "InsertInto(TabAfterCodeList,expr(chart_after));";
				put "InsertInto(TabOpenList,1);";
				put "InsertInto(Here:DataList,{{""Here:dataa"",""Here:datag""}});";	

			%end;
			
			%if &i=0 and &_subsetdd %then %do;
				put "    Here:PvConv = ""&Conv"";"; 
				
				/* Action button to filter (launch Subset and Reorder) */
				
				put "if(isEmpty(ActionButtonList) | NItems(ActionButtonList)==0,";
				put "   Here:ActionButtonList = {{""OK""}};";
				put "   Here:ActionCodeList = {{""""}};";
				put "   Here:ActionCodeList[1][1]=expr(Here:launchSubRe(1));";
				put "   Here:ActionButtonDesc = {""Filter Markers/Individuals Based on the Criteria Below""};";		
				put ", //else";
				put "   ArryIndex=NItems(Here:ActionButtonList)+1;";
				put "   InsertInto(Here:ActionButtonList,0);";
				put "   InsertInto(Here:ActionCodeList,{{""""}});";
				put "   InsertInto(Here:ActionButtonDesc,0);";

				put "   Here:ActionButtonList[ArryIndex]={""OK""};";
				put "   Here:ActionCodeList[ArryIndex][1]=expr(Here:launchSubRe(1));";
				put "   Here:ActionButtonDesc[ArryIndex]=""Filter Markers/Individuals Based on the Criteria Below"";";
				put ");";				
			%end;

		%end;
/**********end Marker Properties only code ***********/ 		

		%if &ProcessName = TDT %then %do;
			%if %upcase(&MendCellPlot)=YES and &i=0 %then %do;
			 put "Here:cellplot = Function({},{default local},";
			 put "Here:datac=open(""&ClientOutPath.&fsname..sas7bdat"", invisible);";
			%let _dsid_ = %sysfunc(open(OutLib.&fsname));
			%let ntotvars = %sysfunc(attrn(&_dsid_,NVARS));
			%let errvarlist=;
			%do v=1 %to &ntotvars;
				%if %index(%sysfunc(varname(&_dsid_,&v)),%quote(&PedID._))=1 %then
					%let errvarlist=&errvarlist %str(:)%sysfunc(varname(&_dsid_,&v))%str(,);
					put "Column(""%sysfunc(varname(&_dsid_,&v))"")<<Set Modeling Type(""Ordinal"");";
					put "Column(""%sysfunc(varname(&_dsid_,&v))"")<<Set Property(""Value Colors"",{""No errors""=21,""Mendelian error""=19});";  		 		
				%end;
			%let rc=%sysfunc(close(&_dsid_));  
			 put " ScrollBox(size(450,450),";
			 put " vlistbox(";
			 put '  panelbox("Legend",';
			 put '   t1=Textbox("Mendelian error"),';
			 put '   t1<<Font Color(3),';
			 put '   t1<<Bullet Point(1);';
			 put '   t3=Textbox("No errors"),';
			 put '   t3<<Font Color(5),';
			 put '   t3<<Bullet Point(1);';
			 put '  ),';
				put "Cell Plot(";
				put "	Y(";
				put "		&errvarlist";
				put "	),";
				put "	By(:&AnnoGroup),";
				put "	Label(:&AnnoLabel),";
				put " Scale Uniformly";
				put "))));";	

				put "InsertInto(Here:TabButtonList,""Cell Plot"");";
				put "InsertInto(TabCodeList,expr(cellplot));";
				put "InsertInto(TabAfterCodeList,"" "");";
				put "InsertInto(TabOpenList,1);";
				put "InsertInto(DataList,{{""Here:datac""}});";	
			%end;			
		%end;
		
		%if %index(&ProcessName,MetaAnalysis) and &i=0 %then %do;
			%if %index(&Weights,sample size)=0 %then %do; 
			/* Show Heterogeneity stats in Distribution */
				put 'Here:hetdist=Function({},{default local},';
				put "if(isEmpty(Here:data0&_suf_),";
				put " Here:data0&_suf_=open(""&ClientOutpath.&GWASMA_OutDataName..sas7bdat"", invisible),";
				put " Current Data Table(Here:data0&_suf_));";
				put ":&AnnoLabel << label(1);";

				put 'hlistbox(';
				put 'hetd=Distribution( Stack(1), Automatic Recalc(1),';
				put '  Continuous Distribution(';
				put "    Column(""Cochran_Q""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';		
				put '    Vertical(0)';
				put '  ),';
				put '  Continuous Distribution(';
				put "    Column(""Prob_Cochran_Q""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';
				put '    Vertical(0)';
				put '  ),' ;
				put '  Continuous Distribution(';
				put "    Column(""I_Squared""),";
				put '    Quantiles(1),';
				put '    Moments(0),';
				put '    Horizontal Layout(1),';
				put '    Count Axis(1),';
				put '    Vertical(0)';
				put '  ),' @;			
				put "),";
				/*S1149704 sbrlsi %if &NumGps > 1 %then %do;*/
				%if &NumGps > 1 or %upcase(&ProcessName) = QKMIXEDMODEL %then %do; 
					%if %length(&AnnoGroup) %then %do;
						put "column(""&AnnoGroup"")<<data type(character);";
					%end;
					put "      hetd << Local Data Filter(""on right"",";
					put " 		Conditional(1),";
					put "           Add Filter( Columns(:Name(""&AnnoGroup"")), Display(:Name(""&AnnoGroup""), Size(210, min(400,&NumGps*20)), List Display)),";
					put "		    Mode( Show(1), Include(1) )";
					put "      )";	
				%end;

				put "));";
				put "InsertInto(Here:TabButtonList,""Heterogeneity Statistics"");";
				put "InsertInto(Here:TabOpenList,%eval(&NumGps=1));";
				put "InsertINto(Here:TabCodeList, Here:hetdist());";
				put "InsertInto(Here:TabAfterCodeList,"" "");";
				put "InsertInto(Here:DataList,{{""Here:data0&_suf_""}});";

			%end;
		%end;	/* end MetaAnalysis Heterogeneity plots */
	%end; /*end loop over NumGps*/
	
	%outofloop:;
	
	/* Volcano plot */
	%if &_volplot or &_lsvp %then %do;

		put "////////// volp   ";
		put "InsertInto(DataList,{{""Here:data0&_suf_""}});";	
		%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
			%if %length(&ByVars)>0 or &_lsvp %then %do;
				put "InsertInto(Here:TabButtonList,""Volcano Plots &HaploTest"");";
			%end;
			%else %do;
				put "InsertInto(Here:TabButtonList,""Volcano Plot &HaploTest"");";		
			%end;		
		%end;
		%else %do;
			%if %length(&ByVars)>0 or &_lsvp %then %do;
				put "InsertInto(Here:TabButtonList,""Volcano Plots"");";
			%end;
			%else %do;
				put "InsertInto(Here:TabButtonList,""Volcano Plot"");";		
			%end;
		%end;
		put "Here:volp=function({},{default local},";
		put "if(isempty(Here:data0&_suf_),";
		put "if(jg:drill_down == 0,";
		put " Here:data0&_suf_=open(""&ClientOutPath.&_OutDataName..sas7bdat"", invisible);";
		put ");";
		%if %upcase(&OnlySig)=YES %then %do;
			%if %length(&Conv)=0 %then %let gtlt=%str(>);
			%else %let gtlt=%str(<);
			put "Here:data0&_suf_<<select where(" @;
			put ":Name(""&PVar1"") &gtlt &r" @;
			%do p=2 %to &NPVars;
			put "& :Name(""&&PVar&p"") &gtlt &r" @;
			%end;

			put ")<<hide;";
		%end;  
		put "colnames = Here:data0&_suf_<<GetColumnNames(String);";
		put 'if(Contains(colnames,"_excluded")>0,';
		put "	Here:data0&_suf_<<select where(:_excluded==1)<<hide);";
		put "Here:data0&_suf_ << exclude;";
		put "nhidden=0;";
		put "for each row(";
		put "  if(hidden(), nhidden=nhidden+1));";
		put "if(nhidden==nrow(Here:data0&_suf_),";
		put " throw(""All p-values non-significant in this data set.""));";
		put "Here:data0&_suf_ << Clear Select;";
		%if %length(&AnnoAccession) %then %do;
			put "column(""&AnnoAccession"")<<data type(character);";
		%end;
		%if %length(&AnnoGroup) %then %do;
			put "column(""&AnnoGroup"")<<data type(character);";
		%end;
		put "column(""&AnnoLoc"")<<data type(numeric);";
		put "column(""&AnnoLabel"") << label(1);";		
		%if &_pvalplots %then %do;
			put "column(""CumOrder"") << modeling type(nominal);";
		%end;		
		%if &ProcessName=%str(HaplotypeEstimation) %then put "column(""WindowID"")<<label(1);";;
		put ", //else";
		put "  currentdatatable(Here:data0&_suf_);";
		put "); //end isempty(data&_suf_)";
		put "Data Filter Context Box(Hlistbox(";
		put "Here:vpvlb=Vlistbox(";
		%if &_volplot %then %do;
			put "Here:vp=Graph Builder(";
			put "	Show Control Panel( 0 ), Auto Stretching(0), Lock Scales( 1 ), ";
			put "	Graph Spacing( 3 ),";
			put "	Spacing Borders( 1 ),";
			put "	Show Legend( 0 ),";
			%if &_forestplot=0 %then %do;
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					put "Variables( X( :Estimate_Single), ";
				%end;
				%else %do;
					put "Variables( X( :Estimate_Trend), ";
				%end;
			%end;
			%else %do;
				put "Variables( X( :OverallEffect), ";		
			%end;
			%let u=1;
			%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
				%do p=1 %to &NPVars;
					%if %index(&&PVar&p,Prob) and %index(&&PVar&p,Single) %then %do;
						put "             Y(:&&PVar&p),";
						%let ProbTrendVar&u = &&PVar&p;
						%let u= %sysevalf(&u + 1); 
					%end;
				%end;	
			%end;
			%else %do;
				%do p=1 %to &NPVars;
					%if %index(&&PVar&p,ProbTrend) or &_forestplot %then %do;
						put "             Y(:&&PVar&p),";
						%let ProbTrendVar&u = &&PVar&p;
						%let u= %sysevalf(&u + 1); 
					%end;	
				%end;		
			%end;
			%let nvp= %sysevalf(&u - 1); 
			put '            ' @;
			%if &NumGps>1 %then %do;
				put "Color(&AnnoGroup)";
			%end;
			put "),";				
			%do p=1 %to &nvp;
				put "      Elements(Position(1,&p), Points( X( 1 )" @;
				%if %length(&AnnoGroup) and &NumGps>1 and &AnnoGroup ^= "_group_" %then %do;
					put ", X( 2 )" @;
				%end;
				put ", Y, Legend( &p + 6 ) ) ),";			
			%end;			
			%if %length(&ByVars)>0 %then %do;
				put "            , By(&JBy)";
				put "            , SendToReport(";
				put "              Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
				%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
					put "              Dispatch({}, ""X title"", TextEditBox, {Set Text(""Estimate of Haplotype Effect"")}),";							
				%end;
				%else %do;
					put "              Dispatch({}, ""X title"", TextEditBox, {Set Text(""Estimate of %sysfunc(propcase(&two)) Allele Genotype Effect"")}),";			
				%end;
				put "            )";
			%end;	
			%else %do;
				put ",	SendToReport(";
				put "		Dispatch(";
				put "			{},";
				put "			""Graph Builder"",";
				put "			OutlineBox,";
				%if &_forestplot=0 %then %do;
					put "			{Set Title( ""Genotype Effect from Trend Test for &TraitVar"" )}";
				%end;
				%else %do;
					/*put "			{Set Title( ""&ProbTrendVar1 vs Overall &EffectVar"" )}";*/
					put "			{Set Title( ""Significance Test vs Overall &EffectVar"" )}";
				%end;
				put "     ),";
				%if &_forestplot %then %do;
					put "Dispatch({}, ""X title"", TextEditBox, {Set Text(""Overall &EffectVar"")}),";
				%end;
				%else %do;
					%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
						put "Dispatch({}, ""X title"", TextEditBox, {Set Text(""Estimate of Haplotype Effect"")}),";							
					%end;
					%else %do;
						put "Dispatch({}, ""X title"", TextEditBox, {Set Text(""Estimate of %sysfunc(propcase(&two)) Allele Genotype Effect"")}),";			
					%end;
				%end;
				put "     Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} )";
				put "    )";
			%end;
			put ",   SendtoReport(";
			
			%do p=1 %to &nvp;
				put "		Dispatch(";
				put "	    {},";
				put "	    ""&&ProbTrendVar&p"",";
				put "	    ScaleBox,";
				put "	    {Format( ""Fixed Dec"", 9, 2 ), Inc(%sysevalf((&AxisMax-&AxisMin)/6)), Min( &AxisMin), Max( &AxisMax ),  Minor Ticks( 0 ),Add Ref Line( &r, Dashed, ""Red"" ),";
				put "	    Rotated Labels( ""Automatic"" )}";
				put "    ),";
			%end;
			
			%if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) or %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
				%let u=0;
				%do p=1 %to &NPVars;
					%if %index(&&PVar&p,Prob) and %index(&&PVar&p,Single) %then %do;
						%if &u=0 %then %let k=;
					
						/*Test*/
						%let testflag= 0;
						%let ytest=;
						%if %sysfunc(find(&&PVar&p, Single)) %then %do;
							%let ytest= %str(Single Haplotype Test);
							%let testflag= 1;
						%end;
						%else %do;
							%let ytest= &&PVar&p;
							%let testflag= 0;
						%end;			

						/*MTC Correction*/
						%let ymtc=;
						/*
						%let currSubPVar = &&PVar&p;
						%if %sysfunc(find(&&PVar&p, _Bon)) %then %do;
							%let ymtc= %str(Bonferroni Corrected);
							%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _Bon, ));
						%end;
						%else %if %sysfunc(find(&&PVar&p, _FDR)) %then %do;
							%let ymtc= %str(FDR Corrected);
							%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _FDR, ));
						%end;
						%else %if %sysfunc(find(&&PVar&p, _Sid)) %then %do;
							%let ymtc= %str(Sidak Corrected);
							%let currSubPVar = %sysfunc(tranwrd(&&PVar&p, _Sid, ));
						%end;
						*/

						/*p-value adjustment*/	
						%let yadjs= ;
						%let ybwth= ;
						/*				
						%if %sysfunc(find(&currSubPVar, _F)) %then %do;

							%let bg= %sysfunc(find(&currSubPVar, _F));
							%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
							%if &ed %then %do;
								%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
							%end;
							%else %do;
								%let bwth= %substr(&currSubPVar, &bg + 2);
							%end;
							%let ybwth= %str(Bandwith &bwth);
							%let yadjs= %str(Fisher &ybwth Adj);
						%end;
						%else %if %sysfunc(find(&currSubPVar, _S)) %then %do;

							%let bg= %sysfunc(find(&currSubPVar, _S));
							%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
							%if &ed %then %do;
								%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
							%end;
							%else %do;
								%let bwth= %substr(&currSubPVar, &bg + 2);
							%end;
							%let ybwth= %str(Bandwith &bwth);				
							%let yadjs= %str(Simes &ybwth Adj);
						%end;
						%else %if %sysfunc(find(&currSubPVar, _T)) %then %do;

							%let bg= %sysfunc(find(&currSubPVar, _T));
							%let ed= %sysfunc(find(&currSubPVar, %str(_), &bg+1));
							%if &ed %then %do;
								%let bwth= %substr(&currSubPVar, &bg + 2, &ed - &bg - 2);
							%end;
							%else %do;
								%let bwth= %substr(&currSubPVar, &bg + 2);
							%end;
							%let ybwth= %str(Bandwith &bwth);
							%let yadjs= %str(TPM &ybwth Adj);					
						%end;
						*/
						/*%put ybwth= &ybwth;*/
						/*%put yadjs= &yadjs;*/
					
						/*p-value conversion*/
						%let yconv=;
						%if %sysfunc(find(%upcase(&&PVar&p), NEGLOG10)) %then %do;
							%let yconv= %str(Negative Log10 P-Value );
						%end;
						%else %if %sysfunc(find(%upcase(&&PVar&p), NEGLOG)) %then %do; 
							%let yconv= %str(Negative Log P-Value );
						%end;
						%else %do;
							%let yconv= %str(P-Value );
						%end;

						put "			Dispatch(";
						put "			{},";
						put "			""Y &k title"",";
						put "			TextEditBox,";
						put "			{Set Text("" ";
						%if &testflag %then %do;
							put "			&ytest ";
							%if %length(&ymtc) %then %do;
								put "			&ymtc ";
							%end;
							%if %length(&yadjs) %then %do;
								put "			&yadjs ";
							%end;
							%if %length(&yconv) %then %do;
								put "			&yconv ";
							%end;					
						%end;
						%else %do;
							put "			&ytest ";
						%end;
						put "			 "")}";
						put "			),";										
									
						%let k= %sysevalf(&k + 1); 
						%let u= %sysevalf(&u + 1); 
					%end;
				%end;	
			%end;
			%else %do;
				%let u=0;
				%do p=1 %to &NPVars;
					%if %index(&&PVar&p,ProbTrend) or &_forestplot %then %do;
						%if &u=0 %then %let k=;

						/*Test*/
						%let testflag= 0;
						%let ytest=;
						%if %sysfunc(find(&&PVar&p, Trend)) %then %do;
							%let ytest= %str(Trend Test);
							%let testflag= 1;
						%end;
						%else %if %sysfunc(find(&&PVar&p, ProbMetaIV)) %then %do;
							%let ytest= %str(Inverse Variance Test);
							%let testflag= 1;
						%end;	
						%else %if %sysfunc(find(&&PVar&p, ProbMetaWSS)) %then %do;
							%let ytest= %str(Weighted Square Root Test);
							%let testflag= 1;
						%end;						
						%else %do;
							%let ytest= &&PVar&p;
							%let testflag= 0;
						%end;			

						/*MTC Correction*/
						%let ymtc=;

						/*p-value adjustment*/						
						%let yadjs= ;
						%let ybwth= ;

						/*p-value conversion*/
						%let yconv=;
						%if %sysfunc(find(%upcase(&&PVar&p), NEGLOG10)) %then %do;
							%let yconv= %str(Negative Log10 P-Value );
						%end;
						%else %if %sysfunc(find(%upcase(&&PVar&p), NEGLOG)) %then %do; 
							%let yconv= %str(Negative Log P-Value );
						%end;
						%else %do;
							%let yconv= %str(P-Value );
						%end;

						put "			Dispatch(";
						put "			{},";
						put "			""Y &k title"",";
						put "			TextEditBox,";
						put "			{Set Text("" ";
						%if &testflag %then %do;
							put "			&ytest ";
							%if %length(&ymtc) %then %do;
								put "			&ymtc ";
							%end;
							%if %length(&yadjs) %then %do;
								put "			&yadjs ";
							%end;
							%if %length(&yconv) %then %do;
								put "			&yconv ";
							%end;					
						%end;
						%else %do;
							put "			&ytest ";
						%end;
						put "			 "")}";
						put "			),";

						%let k= %sysevalf(&k + 1); 
						%let u= %sysevalf(&u + 1); 
						
					%end;
				%end;
			%end;
			%*if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
				put "		Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
			%*end;	
			put "		Dispatch( {}, ""400"", LegendBox, {Sides( ""Left"" ), Set Wrap( 10 )} ),";
			put '    ),'; 
			put '   ),'; 
		%end;
		
		%if &_lsvp %then %do;
			/* Volcano plots of LS Diffs from Genotype test */
		
			put "try(Column(""LSDiff_1_0""); ";
			put "Here:lsvp1=Graph Builder(";
			put "	Size( 350, 200 ),Show Control Panel( 0 ), Auto Stretching(0), Lock Scales( 1 ), ";
			put "	Graph Spacing( 3 ),";
			put "	Spacing Borders( 1 ),";
			put "	Show Legend( 0 ),";
			put "Variables( X( :LSDiff_1_0), Y( %scan(&LSPVars,1)) ";
			%if &NumGps>1 %then %do;
				put ",Color(&AnnoGroup)";
			%end;			
			put '),' @;
			put "Elements( Points( X, Y, Legend( 2 ) ) ),";
			%if %length(&ByVars)>0 %then %do;
				put "            , By(&JBy)";
				put "           ,	SendToReport(Dispatch( {}, ""graph title"", TextEditBox, {Set Text( ""LS Diff of Genotype 1 - 0"" )} ))";
			%end;	
			%else %do;
				put ",	SendToReport(";
				put "		Dispatch(";
				put "			{},";
				put "			""Graph Builder"",";
				put "			OutlineBox,";
				put "			{Set Title( ""LS Diff of Genotype 1 - 0"" )}";
				put "     ),";
				put "     Dispatch({}, ""X title"", TextEditBox, {Set Text(""LS Diff of Genotype 1 - 0"")}),";			
				put "     Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} )";
				put "    )";
			%end;	
			put ",   SendtoReport(";
			put "		Dispatch(";
			put "	    {},";
			put "	    ""%scan(&LSPVars,1)"",";
			put "	    ScaleBox,";
			put "	    {Format( ""Fixed Dec"", 9, 2 ), Inc(%sysevalf((&AxisMax-&AxisMin)/6)), Min( &AxisMin), Max( &AxisMax ), Minor Ticks( 0 ),Add Ref Line( &r, Dashed, ""Red"" ),";
			put "	    Rotated Labels( ""Automatic"" )}";
			put "    ),";
			put "		Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";			
			put '    ),'; 
			
			put '   )),'; 	
			put "try(Column(""LSDiff_2_0"");"; 
			put "Here:lsvp2=Graph Builder(";
			put "	Size( 350, 200 ),Show Control Panel( 0 ), Auto Stretching(0), Lock Scales( 1 ), ";
			put "	Graph Spacing( 3 ),";
			put "	Spacing Borders( 1 ),";
			put "	Show Legend( 0 ),";
			put "Variables( X( :LSDiff_2_0), Y( %scan(&LSPVars,2)) ";
			%if &NumGps>1 %then %do;
				put ",Color(&AnnoGroup)";
			%end;			
			put '),' @;
			put "Elements( Points( X, Y, Legend( 2 ) ) ),";
			%if %length(&ByVars)>0 %then %do;
				put "            , By(&JBy)";
				put "           ,	SendToReport(Dispatch( {}, ""graph title"", TextEditBox, {Set Text( ""LS Diff of Genotype 2 - 0"" )} ))";
			%end;	
			%else %do;
				put ",	SendToReport(";
				put "		Dispatch(";
				put "			{},";
				put "			""Graph Builder"",";
				put "			OutlineBox,";
				put "			{Set Title( ""LS Diff of Genotype 2 - 0"" )}";
				put "     ),";
				put "     Dispatch({}, ""X title"", TextEditBox, {Set Text(""LS Diff of Genotype 2 - 0"")}),";			
				put "     Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} )";
				put "    )";
			%end;
			put ",   SendtoReport(";
			put "		Dispatch(";
			put "	    {},";
			put "	    ""%scan(&LSPVars,2)"",";
			put "	    ScaleBox,";
			put "	    {Format( ""Fixed Dec"", 9, 2 ), Inc(%sysevalf((&AxisMax-&AxisMin)/6)), Min( &AxisMin), Max( &AxisMax ),  Minor Ticks( 0 ),Add Ref Line( &r, Dashed, ""Red"" ),";
			put "	    Rotated Labels( ""Automatic"" )}";
			put "    ),";
			put "		Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
			put '    ),'; 
			put '   )),'; 			
		
		%end;		
		%if (&GpType = N) %then %do;
			put "column(""&AnnoGroup"")<<data type(character);";
		%end;
		%else %do;
			%if &ProcessName=JMPGenomicsBrowser %then %do;
				put "column(""&AnnoGroup"") << Set Property(""Value Ordering"",{""chr0"",""chr1"",""chr2"",""chr3"",""chr4"",""chr5"",";
				put "   ""chr6"",""chr7"",""chr8"",""chr9""});";
			%end;
		%end;
		/*
		%if &NumGps>1 %then %do;
			put "Here:data0&_suf_ << Color by Column(";
			put "&AnnoGroup, Color( 1 ),	Marker( 0 ),Continuous Scale( 0 ));";
		%end;		
		*/
		put "),";		
		%if &NumGps>1 %then %do;
			%if &_volplot %then %do;
				/* Change to local data filter when available in Graph Builder 
				put "Here:vp<<Local Data Filter(""on right"",Auto clear(0),No Outlinebox(0),";
				*/
				put "Here:data0&_suf_<< Data Filter(";
				put " Conditional(1),";
				%if %length(&ByVars)=0 %then %do;
					put " Local,";			
				%end;
				put " Add Filter(Columns(""&AnnoGroup"")), Display(:Name(""&AnnoGroup""), Size(210, min(400,&NumGps*20)), List Display), Mode(Show(1), Include(1))";
				put ");";
			%end;
			%if &_lsvp and &_volplot=0 %then %do;
				/* Change to local data filter when available in Graph Builder 
				put "try(Here:lsvp1<<Local Data Filter(""on right"",Auto clear(0),No Outlinebox(0),";
				*/
				put "Here:data0&_suf_<< Data Filter(";
				put " Conditional(1),";
				%if %length(&ByVars)=0 %then %do;
					put " Local,";			
				%end;			
				put " Add Filter(Columns(""&AnnoGroup"")), Display(:Name(""&AnnoGroup""), Size(210, min(400,&NumGps*20)), List Display), Mode(Show(1), Include(1))";
				put ");";			

				/* Change to local data filter when available in Graph Builder 
				put "try(Here:lsvp2<<Local Data Filter(""on right"",Auto clear(0),No Outlinebox(0),";
				put " Add Filter(Columns(""&AnnoGroup"")), Display(:Name(""&AnnoGroup""), Size(210, min(400,&NumGps*20)), List Display), Mode(Show(1), Include(1))";
				put "));";	*/
			
			%end;
		%end;
		put "),";
		put ") //end Data Filter Context Box";			
		put "); //end volp function";

		put "////////volp_after ";
		put "Here:volp_after= Function({},{default local},";
		put "Here:dt = Today();";
		%if &_volplot %then %do;
			put "Eval(Parse(EvalInsert(""\[vpr^dt^ = Here:vp << Report;]\"")));";	
		%end;
		%if &_lsvp %then %do;
			put "try(Eval(Parse(EvalInsert(""\[lsvpr1^dt^ = Here:lsvp1 << Report;]\""))));";		
			put "try(Eval(Parse(EvalInsert(""\[lsvpr2^dt^ = Here:lsvp2 << Report;]\""))));";				
		%end;

		put "Here:ppfsize = 800;";
		
		%if &NbyVal > 1 %then %do;
			/*
			put "for(_b=1, _b<=&NbyVal, _b++,";
			put "   for(_i=1, _i<=&NPVars, _i++,";
			put '    try(';
			put "Eval(Parse(EvalInsert(""\[";
			put "      vpr^dt^[_b][framebox(_i)] << FrameSize(650,175);";
			put "      vpr^dt^[_b][framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
			put "      vpr^dt^[_b][framebox(_i)]  << Marker Size(3);"; 
			put "      vpr^dt^[_b][framebox(_i)]  << Marker Drawing Mode(Normal);"; 		
			%do p=1 %to &nvp;
				put "      vpr^dt^[_b][Axisbox(&p+1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(0), Max(&AxisMax)};";	
			%end;
			put ']\"';
			put ")))";
			put ")));";
			*/		
			put "Eval(Parse(EvalInsert(""\[";
			put "	for(_b=1, _b<=&NbyVal, _b++,";
			put "   for(_i=1, _i<=&NPVars, _i++,";
			put '      try(';
			%if &_volplot %then %do;
				put "        vpr^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
			%end;
			%if &_lsvp %then %do;
				put "        lsvpr1^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "        lsvpr2^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
			%end;		
			put '  ));';
			put ');';
			%if &_volplot %then %do;
				put "   vpvlb << Prepend(";
				put "      VListBox(";
				%if ^%index(&ProcessName,MetaAnalysis) %then %do;
					put "      HListBox(";
					put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
					put "	   ),";
					put "   HListBox(";
					put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
					put "	),";				
					put "      HListBox(";
					put "   	Text Box( "" "" ),";
					put "	   ),";
				%end;
				put "      HListBox(TextBox("" Frame Size for Graphs Below""),";
				put "      frameSizeSlider^dt^ = SliderBox(num(200),num(800),Here:ppfsize,";
				put "		for(_b=1, _b<=&NbyVal, _b++,";
				put "   		for(_i=1, _i<=&NPVars, _i++,";
				put "      			try(";
				put "       	 			vpr^dt^[_b][framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "  			));";
				put "		);";	
				put "	)";
				put ")));";
			%end;			
			%*if &_forestplot=0 and ^&_lsvp %then %do;
				put "	for(_b=1, _b<=&NbyVal, _b++,";	
				put "            try(";
				put "            //show(vpr^dt^[_b][OutLineBox(1)]);";
				put "            gtitle= vpr^dt^[_b][OutLineBox(1)]<< Get Text;";
				put "            //show(gtitle);";
				put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
				put "            //show(gtitle);";
				put "            pos= Munger(gtitle, 1, ""Where("");";
				put "            //show(pos);";
				put "            if(pos, ";
				put "            	del= Trim(Substr(gtitle, pos - 6));";
				put "            	vpr^dt^[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
				put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";				
				put "            );";
				put "            //show(gtitle);";
				put "            vpr^dt^[_b][OutLineBox(1)] << {Set Title(""Genotype Effect from Trend Test for ""||gtitle), Image Export Display( Normal )};";
				put "        );";
				put "    );";
			%*end;		
			put ']\"';
			put "));";
			put ");";	
		%end;		
		%else %do;
			/*
			put "   for(_i=1, _i<=&NPVars, _i++,";
			put '    try(';
			put "Eval(Parse(EvalInsert(""\[";
			put "      vpr^dt^[framebox(_i)] << FrameSize(650,175);";
			put "      vpr^dt^[framebox(_i)] << add graphics script(pencolor(""red"");linestyle(""dashed"");hline(&r));";
			put "      vpr^dt^[framebox(_i)]  << Marker Size(3);"; 
			put "      vpr^dt^[framebox(_i)]  << Marker Drawing Mode(Normal);"; 		
			%do p=1 %to &nvp;
				put "      vpr^dt^[Axisbox(&p+1)] << {Format( ""Fixed Dec"", 9, 2 ),Min(0), Max(&AxisMax)};";	
			%end;
			put ']\"';
			put ")))";
			put "));";
			*/		
			put "Eval(Parse(EvalInsert(""\[";				
			put "   for(_i=1, _i<=&NPVars, _i++,";
			put '      try(';
			%if &_volplot %then %do;
				put "        vpr^dt^[framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
			%end;
			%if &_lsvp %then %do;
				put "        lsvpr1^dt^[framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "        lsvpr2^dt^[framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
			%end;		
			put '  ));';	

			%if &_volplot %then %do;
				put "   vpvlb << Prepend(";
				put "      VListBox(";
				%if ^%index(&ProcessName,MetaAnalysis) %then %do;
					put "      HListBox(";
					put "   	Text Box( "" Multiple testing correction: &mtcmethod"", << set wrap(800) ),";
					put "	   ),";
					put "   HListBox(";
					put "   	Text Box( "" P-value adjustment method: &adjmethod"", << set wrap(800) ),";
					put "	),";				
					put "      HListBox(";
					put "   	Text Box( "" "" ),";
					put "	   ),";	
				%end;
				put "   HListBox(TextBox("" Frame Size for Graphs Below""),";
				put "   frameSizeSlider^dt^ = SliderBox(num(200),num(800),Here:ppfsize,";
				put "   for(_i=1, _i<=&NPVars, _i++,";
				put "      try(";
				put "        vpr^dt^[framebox(_i)] << FrameSize(ppfsize,0.35*ppfsize);";
				put "  ));";	
				put " )";
				put ")));";
			%end;
					
			/*
			%*if &_forestplot=0 and ^&_lsvp %then %do;
				put "            try(";
				put "            //show(vpr^dt^[OutLineBox(1)]);";
				put "            gtitle= vpr^dt^[OutLineBox(1)]<< Get Text;";
				put "            //show(gtitle);";
				put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Graph Builder"", """"));";
				put "            //show(gtitle);";
				put "            pos= Munger(gtitle, 1, ""Where("");";
				put "            //show(pos);";
				put "            if(pos, ";
				put "            	del= Trim(Substr(gtitle, pos - 6));";
				put "            	vpr^dt^[OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
				put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";				
				put "            );";
				put "            //show(gtitle);";
				put "            vpr^dt^[OutLineBox(1)] << {Set Title(""Genotype Effect from Trend Test for ""||gtitle), Image Export Display( Normal )};";
				put "        );";						
			%*end;
			*/		
			put ']\"';
			put "));";
			put ");";

		%end;

		put ");";
		put "InsertInto(TabCodeList,expr(volp));";
		put "InsertInto(TabAfterCodeList,expr(volp_after));"; 
		/*S1149704 sbrlsi changed Volcano Plot Tab to pop up open as default*/
		put "InsertInto(TabOpenList,1);";
		/*
		%if %upcase(&ProcessName) = QTLSINGLEMARKERANALYSIS %then %do;
			put "InsertInto(TabOpenList,1);";
		%end;
		%else %do;
			put "InsertInto(TabOpenList,%eval(&NumGps = 1));";
		%end;	
		*/	
	%end;				
run;

* create string of sas data sets needed for script;
%let depdata = &_OutDataName..sas7bdat;
%AddDatatoStudy(&_OutDataName,_type=anno);

%let maxon = %eval(31-&maxlenAG);
%if %length(%trim(&_OutDataName)) > &maxon %then 
	%let _name = %substr(&_OutDataName,1,&maxon);
%else %let _name = &_OutDataName;
%if &NPVars>0 and &_summchart %then %do;
	%let depdata = &depdata, %substr(&_name,1,28)_sbg.sas7bdat;
	%AddDatatoStudy(%substr(&_name,1,28)_sbg,_type=other);
	/* 
	%PackageInsertDataset(Summary data from &ProcessName, outlib, %substr(&_name,1,28)_sbg, yes, no);
	*/
%end;
/*
%do i=1 %to &NumGps; 
	%PackageInsertDataset(&AnnoGroup &&SGroupn&i data from &ProcessName, outlib, &_name._&&SGroupn&i, yes, no);
%end;
*/
%if &ProcessName = TDT %then %do;
	%if %upcase(&MendCellPlot)=YES %then %do;
		%let depdata = &depdata, &fsname..sas7bdat;
		%AddDatatoStudy(&fsname,_type=other);
	%end;
%end;
%if &ProcessName = PCAforPS %then %do;
	%let depdata = &depdata, &PCAIncludeDS;
%end;
%if &ProcessName = MarkerProperties %then %do;
	%if %upcase(&ChartFreqs)=YES %then %do;
		%let depdata = &depdata, &AlleleFreqDataName..sas7bdat, &GenotypeFreqDataName..sas7bdat;
		%AddDatatoStudy(&AlleleFreqDataName,_type=other);
		%AddDatatoStudy(&GenotypeFreqDataName,_type=other);		
	%end;
	%if %upcase(&CreateCellPlot)=YES %then %do;
		%let depdata = &depdata, &NumGenoDataName..sas7bdat;
		%AddDatatoStudy(&NumGenoDataName,_type=wide);				
	%end;
	%let depdata = &depdata, &PropMissDataName..sas7bdat;
	%AddDatatoStudy(&PropMissDataName,_type=other);				
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &_JSLFileName, "&depdata",text/plain, yes);
%end;
%else %if %upcase(&ProcessName) = QKMIXEDMODEL %then %do;
	/*S1168164: sbrlsi added qkdepdata to fix issue of file not found on the server.
	Send macro variable qkdepdata to QKMixedModel with the list of data sets that the process depends on*/
	%let qkdepdata= &depdata;
%end;
%else %if %upcase(&ProcessName) = %upcase(HaplotypeQKMixedModel) %then %do;
	%let HQKDepData= &depdata;
%end;
%else %if %upcase(&ProcessName) = %upcase(HaplotypeTrendRegression) %then %do;
	%let HRDepData= &depdata;
%end;
%else %if &_pve=0 %then %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &_JSLFileName, "&depdata", text/plain, yes);
%end;

%mend;

/***********************************************
  Subset annotation data set and remove 
  corresponding variables from input data set
  according to AnnoWhereStmt.
  Added in reordering as well if ReorderingVars macro
  variable exists.
***********************************************/
%macro GenSubset(WhereClause);

	%if %length(&WhereClause)=0 %then %let WhereClause=1;
	/* Use XLVARs for X-linked stmt in TDT, also for filtering Null SNPs in PCA for PS */
	%if %symexist(XLVarStmt)=1 %then %do;
		%if %length(&XLVarStmt)=0 %then %let XLVarStmt=0;
	%end;
	%else %let XLVarStmt=0;
	
	/* Only do this to check for invalid variables in Where Stmt */
	data _null_;
		set mapdata(where=(&WhereClause));
	run;
	
	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exerr;
	%end;
	
	%if &XLVarStmt ne 0 %then %do;
		data _null_;
			set mapdata(where=(%unquote(&XLVarStmt)));
		run;

		%if &syserr>4 %then %do;
			%let exiterror=1;
			%goto exerr;
		%end;	
	%end;
	
	data mapdata_orig;
	 	set mapdata;
		if (%unquote(&XLVarStmt)) then _xlvar=1;
		else _xlvar=0;
		drop _oldorder_;
	run;

	data mapdata_orig_keep;
		set mapdata_orig;
		where &WhereClause;
	run;
	
	data mapdata_orig;
		merge mapdata_orig mapdata_orig_keep(in=kt);
		by _order_;
		rename _order_=_oldorder_;
		keepthis=kt;
		_excluded=1-keepthis;
	run;	
	
	proc datasets nolist;
		delete mapdata_orig_keep;
	quit;

	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exerr;
	%end;
	
	data mapdata;
	 set mapdata_orig;
	 if keepthis;
	 _order_+1;
	 drop keepthis;
	run;

	proc sort data=mapdata out=mapdata(drop=_xlvar);
		%if %upcase(&ProcessName) = %upcase(GenomicHeritability) %then %do;
			/* 
			sbrsi: Need to include ReorderingVars in the by statement to make sure 
			genotype and map data have the same order when running Genomic Heritability 
			(GenomicHeritability). Because the sorting is done here, the sort statement 
			in the code that follows a few lines below need to be taken out.
			Need to allow this as the standard for all ProcessName.
			*/
			by _xlvar %if %symexist(ReorderingVars) %then &ReorderingVars; _order_;
		%end;
		%else %do;
			by _xlvar _order_;
		%end;
	run;
	
	/* Need this for PCA for PS */
	data mapdata_null;
		set mapdata;
		_nullorder_ = _n_;
	run;

	data order;
	 set mapdata_orig;
	 _order_=_oldorder_;
	 keep _order_ keepthis _xlvar %if %symexist(ReorderingVars) %then &ReorderingVars;;
	run;

	%if %length(&GenoCol) %then %let nm=&nummark;
	%else %let nm=&n2snps;
	
	%let BlockSize=%sysfunc(min(&nm,50000));
	%let first = 1;
	%let last = &BlockSize;

	data _null_;
		/* Make sure AlleleVars are in consecutive variables */
		set &ds(drop=&AlleleVars);
		set &ds(keep=&AlleleVars);
		array getname{*} &AlleleVars;
		%let Block=1;
		%do %until (&first > &nm);	

			_f = vname(getname(&first));
			_l = vname(getname(&last));

			call symput("firstv&Block",_f);
			call symput("lastv&Block",_l);

			%let first = %eval(&last + 1);
			%let last = %eval(&first + &BlockSize - 1);
			%let Block = %eval(&Block + 1);
			%if &last > &nm %then %let last = &nm;
		%end;
	run;
	
	%let NBlocks = %eval(&Block-1);	
	
	options label;
	data sub_notrans(drop=&AlleleVars) %do b=1 %to &NBlocks; sub_trans&b(keep=&&firstv&b--&&lastv&b) %end;;
		/* Make sure AlleleVars are in consecutive variables */
		set &ds(drop=&AlleleVars);
		set &ds(keep=&AlleleVars);
	run;

	%removedata(trans);
	%do b=1 %to &NBlocks;
		proc transpose data=sub_trans&b out=trans_ prefix=_COL_;
			var &&firstv&b--&&lastv&b;
		run;
		
		%if &syserr>4 %then %let exiterror=1;
		
		%if &b=1 %then %do;
			data trans_;
				length _name_ $32;
				set trans_;
			run;
		%end;
		
		proc datasets nolist;
			%if &b>1 %then %do;
				append base=trans data=trans_ force nowarn;
			%end;
			%else %do;
				change trans_=trans;
			%end;
			delete sub_trans&b;
		quit;
		
	%end;

	data trans2;
	 set trans;
	 %if %length(&GenoCol) %then %do;
	 	_order_=_n_;
	 %end;
	 %else %do;
	 	_order_=int((_n_+1)/2);
	 %end;
	run;
	
	proc datasets nolist;
		delete trans trans_;
	quit;

	data subset;
	 merge order trans2;
	 by _order_;
	 if keepthis;
	run;

	data trans2;
	 delete;
	run;
	
	proc sort data=subset overwrite; 
		by _xlvar %if %symexist(ReorderingVars) %then &ReorderingVars; _order_;
	run;

	data subset;
	 set subset end=last;
	 by _xlvar;
	 retain firstx;
	 if _xlvar=0 then do;
	  if first._xlvar then call symput('firstvar',strip(_name_));
	 	if last._xlvar and _xlvar=0 then call symput('lastvar',strip(_name_));
	 end;
	 else do;
	 	if first._xlvar then do;
	 		call symput('firstxvar',strip(_name_));
	 		firstx=_n_;
	 	end;	 
	 	if last._xlvar then do;
	 		call symput('lastxvar',strip(_name_));
	 		numxmark=_n_ - firstx + 1;
	 		%if %length(&GenoCol)=0 %then numxmark = numxmark / 2;;
	 		call symput('numxm',strip(numxmark));
	 	end;
	 end;
	 n__=_n_;
	 drop numxmark firstx;
	run;
	
	%global numxmark;
	%if %symexist(firstxvar) %then %let numxmark = &numxm;
	%else %let numxmark=0;
	
	%if %symexist(firstvar) and %symexist(lastvar) %then %let AlleleVars = %str(&firstvar--&lastvar);
	%else %let AlleleVars=;
	%if %symexist(firstxvar) and %symexist(lastxvar) %then %let XLVars = %str(&firstxvar--&lastxvar);
	%else %let XLVars = ;

	/* Gets number of markers */
	%let dsid=%sysfunc(open(subset));
	%let n2snps=%sysfunc(attrn(&dsid,NOBS));
	%if %length(&GenoCol) %then %do;
		%let n2snps = %eval(2*&n2snps);
	%end;
	%let ret=%sysfunc(close(&dsid));
	%let nummark=%eval(&n2snps/2);

	%if &nummark=0 %then %do;
		%put ERROR: There are no markers that satisfy the Filter to Include Markers.;
		%let exiterror=1;
	%end;

	%let dsid=%sysfunc(open(subset,i));
	%let labelexists=%sysfunc(varnum(&dsid,_LABEL_));
	%let rc=%sysfunc(close(&dsid));

	%if %length(&GenoCol) %then %let nm=&nummark;
	%else %let nm=&n2snps;

	%let BlockSize=%sysfunc(min(&nm,50000));
	
	%let fobs = 1;
	%let lobs=&BlockSize;
	%let Block = 1;
	
	%do %until (&fobs > &nm);
	
		proc transpose data=subset(drop=keepthis where=(&fobs<=n__<=&lobs)) out=retrans_(drop=_name_);
		 id _name_;
		 var _COL_:;
		 %if &labelexists > 0 %then %do;
			idlabel _LABEL_;
		 %end;
		run;
		
		%if &syserr>4 %then %let exiterror=1;
		
		%if &Block=1 %then %do;
			proc datasets nolist;
				change retrans_=retrans;
			quit;
		%end;
		%else %do;
			data retrans;
				merge retrans retrans_;
			run;
		%end;
		
		%let fobs = %eval(&lobs + 1);
		%let lobs = %eval(&fobs + &BlockSize - 1);
		%if &lobs > &nm %then %let lobs = &nm;

		%let Block = %eval(&Block + 1);		
	%end;
	
	%if %symexist(ReorderingVars) and %length(&ReorderingVars)>0 %then %do;
		data _reord;
		 merge sub_notrans retrans;
		run;
		%let ds=_reord;
		%if %upcase(&ProcessName) ^= %upcase(GenomicHeritability) %then %do;
			/* 
			sbrsi: Because the ReorderingVars was included in the by statement (a few lines above) 
			to make sure genotype and map data have the same order when running Genomic Heritability,
			the sort statement here needs to be taken out. 
			Need to allow this as the standard for all ProcessName.
			*/
			proc sort data=mapdata;
				by &ReorderingVars;
			run;
		%end;
	%end;
	%else %do;
		%*let sInData=%substr(%unquote(&InData),1,28);
		%let lInData= %length(%unquote(&InData)); *needed to avoid errors with &syswarningtext*;
		%let sInData=%substr(%unquote(&InData),1,%sysfunc(min(28,&lInData)));
		data &sInData._sub;
		 merge sub_notrans retrans;
		run;
		%let ds=&sInData._sub;
	%end;
	
	proc datasets nolist;
		delete retrans retrans_ subset;
	quit;
	
	data count_old;
		set count;
	run;
	
	data count; 
	 set &ds(keep=&AlleleVars); %if %length(&XLVars) %then set &ds(keep=&XLVars);;
	 if _n_=1 then output;
	 else stop;
	run;
	
	%exerr:;
	options nolabel;
%mend;

/* called by SNP-Trait Assoc, and Case-Control for dom/rec/MAX test  */
%macro CreateStackedNumericDS(alleleds,_2=minor,_del=1);
		%global exiterr;
		%let exiterr=0;
		
		%if %qupcase(&MarkerFmt) ^= %quote(NUMERIC GENOTYPES) %then %do;

			%if %length(&AnnoMajAllele)=0 %then %do;
				ods listing close;
				ods output AlleleFreq=af ;
				proc allele data=&alleleds &GenoCol prefix=g;  
					 var &&firstvar&Block -- &&lastvar&Block;  
				run;

				%if &syserr>4 %then %do;
					%let exiterr=1;
					%goto exit;
				%end;

				ods listing;

				data _null_;
				 length maxaf_allele $16;
				 set af;
				 by Locus notsorted;
				 retain maxaf maxaf_allele;
				 mnum = substr(locus,2) + 0;
				 if first.Locus then do;
					maxaf = freq;
					maxaf_allele =  Allele;	
				 end;
				 else do;
					if freq > maxaf then do;
						maxaf = freq;
						maxaf_allele = allele;
					end;
				 end;
				 if last.Locus then do;
					call symput('MajAllele'||strip(mnum),trim(maxaf_allele));
				 end;
				run;
				%let OutputMajAllele = 1;
			%end;
			%else %do;
			 /* We already have Major Allele */
				data _null_;
					set mapdata;
					if _n_ >= &&BlockStart&Block and _n_ <= &&BlockEnd&Block then do;
						_obs = _n_ - &&BlockStart&Block + 1;
						call symput('MajAllele'||strip(_obs),trim(&AnnoMajAllele));
					end;
				run;
			%end;

		%end;

		/**Transpose data****/
		proc transpose data=&alleleds out=trans prefix=Ind;
			var &&firstvar&Block -- &&lastvar&Block;
		run;

		/* Convert genotypes to numeric coding and output in "stacked" format */
		data NumGenoData;
			length maja _w1 _w2 $16;
			set trans;
			array _col[*] Ind:;
			/* Genotype format */
			%if %qupcase(&MarkerFmt)=GENOTYPES or %qupcase(&MarkerFmt)=%str(NONDELIMITED GENOTYPES) %then %do;
				_order_= _n_ + &first - 1;
				if symexist('MajAllele'||strip(_n_)) then do;
					maja = trim(symget('MajAllele'||strip(_n_)));
					do _c_=1 to dim(_col);
						%if %length(&GenoDelim) %then %do;
							_w1=strip(scan(_col[_c_],1,"&GenoDelim"));
							_w2=strip(scan(_col[_c_],2,"&GenoDelim"));	
						%end;
						%else %do;
						/* Nondelimited genotype */
							_w1=substr(_col[_c_],1,1);
							_w2=substr(_col[_c_],2,1);
						%end;
						*if missing(_w1) or missing(_w2) then Genotype=.;
						if missing(_w1) or missing(_w2) or trim(left(_w1)) = '.' or trim(left(_w2)) = '.' then Genotype=.;
						else Genotype= 2 - ((_w1=maja)+(_w2=maja));	
						output;
					end;
				end;
				else do _c_=1 to dim(_col);
					Genotype=.;
					output;
				end;
			%end;
			/* Allele format */
			%else %if %qupcase(&MarkerFmt)=ALLELES %then %do;;
				_order_= round(_n_,2)/2 + &first - 1;
				_aorder_=round(_n_,2)/2;
				_i=_n_;
				if symexist('MajAllele'||strip(_aorder_)) then do;
					maja = strip(symget('MajAllele'||strip(_aorder_)));		
					do _c_=1 to dim(_col);
						_w1=strip(lag(_col[_c_]));
						if mod(_i,2)=0 then do;
							_w2=strip(_col[_c_]);
							*put '_n_ ' _n_ ' ' _col[_c_]  '_w1 ' _w1 '_w2 ' _w2;
							if missing(_w1) or missing(_w2) or trim(left(_w1)) = '.' or trim(left(_w2)) = '.' then Genotype=.;
							*if missing(_w1) or missing(_w2) then Genotype=.;
							else Genotype= 2 - ((_w1=maja)+(_w2=maja));
							output;
						end;
					end;
				end;
				else do _c_=1 to dim(_col);
					Genotype=.;
					output;
				end;				
			%end;	
			/* Numeric genotype format */
			%else %do;
				/* check for all missing for a SNP */
				_nmiss_ = nmiss(of Ind:);
				_order_=_n_ + &first - 1;
				if _nmiss_ ne dim(_col) then do;
					do _c_=1 to dim(_col);
						Genotype=_col[_c_];	
						output;
					end;
				end;
				else do _c_=1 to dim(_col);
					Genotype=.;
					output;
				end;				
			%end;
			keep _order_ maja Genotype;
			rename maja=MajorAllele;
		run;	
		
		data genotypes;
			set traits; set NumGenoData;
			%if &_2=%str(major) %then %do;
				Genotype = 2 - Genotype;
			%end;
		run;	

		%if &_del %then %do;
			proc datasets nolist;
				delete NumGenoData trans;
			quit;		
		%end;
		
		%exit:;
		
%mend;

