/*
JMP Genomics Analytical Process 

Name: Basic Single Cell RNA-Seq Workflow
Author: Meijian Guan, SAS Institute Inc.

Description: 
		This process runs a basic workflow
		for expression data, including options
		for quality control and BasicExpressionWorkflow.
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData testaddgene InGenes inputGenes testanova ExpInData topGenes testTsne ClusterMethod minClust maxClust
numClust minFeature maxFeature minCell MtPercent CalculationMethod min_disp max_disp min_mean max_mean OutData 
OutData OutPath nullqclog nullqc basicinfo ServerOutPath MacroPath VariableGeneGraphScript varName OutDataScript OutDataScriptName 
syserr exiterror clockrunning NumSvd rmMt rmRib EmbDim Perp Iter Nneighbors Epoc minDist;

/*%macro CheckandCreateDir(dir);
options noxwait;
	%local rc fileref;
	%let rc=%sysfunc(filename(fileref,&dir));
	%if %sysfunc(fexist(&fileref)) %then
	%do;
		%put The directory "&dir" already exists;
		%sysexec rmdir /Q /S "&dir";
		%sysexec mkdir "&dir";
	%end;
	%else
	%do;
		%sysexec mkdir "&dir";
	%end;
	%if &sysrc eq 0 %then %put The directory &dir has been created;
	%else %put There was a problem while creating the directory &dir;
%mend;
*/
%macro runScRNAseqWorkflow;
	
	%let OutDataScript=;
	%let OutDataScriptName=;
	%let ProcessName=;
	
	
	%if %length(&numClust)=0 %then %let numClust=15;
	%if %length(&minClust)=0 %then %let minClust=2;
	%if %length(&maxClust)=0 %then %let maxClust=15;
	
	%if %length(&NumSvd)=0 %then %let NumSvd=20;
	
	*ProcessVariables;
	*run macro;
	/*%CheckandCreateDir(&OutPath);*/
	%include "&MacroPath./VariableGenes.sas" / nosource;
	**%include "C:\PerforceWorkspaces\JMP\LifeSciences\LifeSciences\4100\JMP\MacroLib\VariableGenes.sas";

	%let ProcessName=BasicScRNASeqWorkflow;
	%PrepOutFile(OutDataScript,OutDataScriptName,&ProcessName Script for &OutData..jsl);
	%put &varName;
	data _null_;
		file "&OutDataScript" lrecl=32767;
		put '//!';
		put "Names Default to Here(1);";
		put "UpSt=include(""$JMP_HOME\LifeSciences\JSLFiles\UpdateScript.jsl"");";
		put "dynamicVars=Associative Array();";
		put "dynamicVars[""addGeneTest""]=""&testaddgene"";";
		put "dynamicVars[""inputGenePath""]=""&InGenes"";";
		put "dynamicVars[""inputGeneList""]=""&inputGenes"";";
		put "dynamicVars[""launchAnova""]=""&testanova"";";
		put "dynamicVars[""launchTsne""]=""&testTsne"";";
		put "dynamicVars[""ExpInDataPath""]=""&ExpInData"";";
		put "dynamicVars[""vargeneMethod""]=""&CalculationMethod"";";
		put "dynamicVars[""minGeneCount""]=""&minFeature"";";
		put "dynamicVars[""maxGeneCount""]=""&maxFeature"";";
		put "dynamicVars[""minCellCount""]=""&minCell"";";
		put "dynamicVars[""mtPercentage""]=""&MtPercent"";";
		put "dynamicVars[""minDis""]=""&min_disp"";";
		put "dynamicVars[""maxDis""]=""&max_disp"";";
		put "dynamicVars[""minMean""]=""&min_mean"";";
		put "dynamicVars[""maxMean""]=""&max_mean"";";
		put "dynamicVars[""numTopGene""]=""&topGenes"";";
		put "dynamicVars[""svdNum""]=""&NumSvd"";";
		put "dynamicVars[""numClusters""]=""&numClust"";";
		put "dynamicVars[""ClustMeth""]=""&ClusterMethod"";";
		put "dynamicVars[""minClustNum""]=""&minClust"";";
		put "dynamicVars[""maxClustNum""]=""&maxClust"";";
		put "dynamicVars[""EmbDim""]=""&EmbDim"";";
		put "dynamicVars[""Perp""]=""&Perp"";";
		put "dynamicVars[""Iter""]=""&Iter"";";
		put "dynamicVars[""Nneighbors""]=""&Nneighbors"";";
		put "dynamicVars[""Epoc""]=""&Epoc"";";
		put "dynamicVars[""minDist""]=""&minDist"";";
		/*put "dynamicVars[""rungtex""]=""&gtex"";";*/
		put "dynamicVars[""resPath""]=""&OutPath"";";
		put "dynamicVars[""pathtoMeanVar""]=""&OutPath.&varName..sas7bdat"";";
		put "dynamicVars[""varName""]=""&varName"";";
		put "dynamicVars[""pathtoVariableGenes""]=""&OutPath.&OutData._lg.sas7bdat"";";
		put "dynamicVars[""VariableGenesName""]=""&OutData"";";
		put "dynamicVars[""pathtoVariableGenesT""]=""&OutPath.&OutData._lg_t.sas7bdat"";";
		put "dynamicVars[""pathtoQCedDt""]=""&OutPath.&nullqclog..sas7bdat"";";
		put "dynamicVars[""pathtoQCedRaw""]=""&OutPath.&nullqc..sas7bdat"";";
		put "dynamicVars[""statsTable""]=""&OutPath.&basicinfo..sas7bdat"";";
		put "dynamicVars[""reportTitle""]=""Variable Gene Selection"";";
		put "myScript=UpSt:init(""$JMP_HOME\LifeSciences\JSLFiles\scRNAPipeline.jsl"");";
		put "newScript=UpSt:update(nameexpr(myScript),dynamicVars);";
		put "UpSt:evalScript(newScript);";
	run;

	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &OutDataScriptName, , text/plain, yes);
	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend;
%runScRNAseqWorkflow;



