/*
JMP Genomics Analytical Process

Name: Compare QTL Plots
Author: Luciano Silva
History:
	sbrlsi 09/10/14 Initial Coding
Description:
		Compare QTL Plots takes as input the output data sets from the QTL IM, CIM, and MIM process
		and overlays the QTL profiles of multiple variables on a single plot. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

%global StudyName InData0 KeyChromVar0 KeyGridVar0 KeyMarkerLab0 KeyChromLab0
	QTLTestVars0 QTLEffVars0 QTLPosVars0 Subset BaseSubset OutPath
	InData1 KeyChromVar1 KeyGridVar1 KeyMarkerLab1 QTLTestVars1 QTLEffVars1 QTLPosVars1
	InData2 KeyChromVar2 KeyGridVar2 KeyMarkerLab2 QTLTestVars2 QTLEffVars2 QTLPosVars2
	InData3 KeyChromVar3 KeyGridVar3 KeyMarkerLab3 QTLTestVars3 QTLEffVars3 QTLPosVars3
	InData4 KeyChromVar4 KeyGridVar4 KeyMarkerLab4 QTLTestVars4 QTLEffVars4 QTLPosVars4
	InData5 KeyChromVar5 KeyGridVar5 KeyMarkerLab5 QTLTestVars5 QTLEffVars5 QTLPosVars5
	InData6 KeyChromVar6 KeyGridVar6 KeyMarkerLab6 QTLTestVars6 QTLEffVars6 QTLPosVars6
	InData7 KeyChromVar7 KeyGridVar7 KeyMarkerLab7 QTLTestVars7 QTLEffVars7 QTLPosVars7
	InData8 KeyChromVar8 KeyGridVar8 KeyMarkerLab8 QTLTestVars8 QTLEffVars8 QTLPosVars8
	InData9 KeyChromVar9 KeyGridVar9 KeyMarkerLab9 QTLTestVars9 QTLEffVars9 QTLPosVars9
	InData10 KeyChromVar10 KeyGridVar10 KeyMarkerLab10 QTLTestVars10 QTLEffVars10 QTLPosVars10
	InData11 KeyChromVar11 KeyGridVar11 KeyMarkerLab11 QTLTestVars11 QTLEffVars11 QTLPosVars11
	InData12 KeyChromVar12 KeyGridVar12 KeyMarkerLab12 QTLTestVars12 QTLEffVars12 QTLPosVars12
	DisplayMarkers DisplayQTL
	OutName MacroPath ServerOutPath CompareQTLPlots_OutData CompareQTLPlots_Script;

%macro CompareQTLPlots;

	**begin: temporary to be deleted**;
	%let DisplayQTL= No;
	%let LODThresh= 0;
	**end**;
	
	**ProcessName must be declared before including UtilityMacros**;
	%global FullAPName ProcessName;
	%let FullAPName = %str(Compare QTL Plots);	
	%let ProcessName = CompareQTLPlots;
	
	***begin: Including macros***;
	**The macro variable ClientOutpath is created upon including the UtilityMacros**;
	%include "&MacroPath./UtilityMacros.sas" / nosource; 
	***end: Including macros***;
	
	**Setting ods output**;
	ods html close;
	*options nolabel;
	
	**begin: auxiliary macros**;
	%macro CheckVarList(_Dsn=, DataType= ,VarList= );
		**Checking QTLTestVars variable type**;
		
		data VarsDat;
			set _null_;
			length _var_ _lab_ $100 _dat_ _ord_ 4;
		run;

		%put VarList= &VarList;
		%let i= 1;
		%let currVar= %scan(&VarList, &i);
		%do %while(%length(&currVar));
			%put currVar= &currVar;
			
			%VarAttributes(_Ds= &_Dsn,_Var=&currVar,_Len=,_Type=VType);
							
			%if &VType ^= N %then %do;
				%put ERROR: Variable &currVar from &DataType Input Data Set &k is not a numeric variable.;
				%let currVar= ;
				%let exiterror=1;
				%goto exitCVL;
			%end;
			%else %do;
				%let currLab= ;
				%CheckLabel(&_Dsn, currVar, _New= currLab);
				%put currLab= &currLab;
				%put currVar= &currVar;
				proc sql;
					insert into Varsdat
					set _var_ = "&currVar",
					    _lab_ = "&currLab",
					    _dat_= &k,
					    _ord_= %sysevalf(&k*100 + &i);
				quit;

				%let i= %sysevalf(&i + 1);
				%let currVar= %scan(&VarList, &i);
			%end;
		%end;
				
		%exitCVL:;
	%mend CheckVarList;
	
	%macro ChangeRepeatedVars(dsn=, flagVar=, flagLab=);
	
		proc sql UNDO_POLICY= NONE;
			create table crossjoined as
			select f1.*, f2.*
			from 
			&dsn(rename=(_origvar_= _origvar1_ _var_= _var1_ _origlab_= _origlab1_ _lab_=_lab1_ _dat_= _dat1_ _ord_= _ord1_)) as f1 
			cross join 
			&dsn(rename=(_origvar_= _origvar2_ _var_= _var2_ _origlab_= _origlab2_ _lab_=_lab2_ _dat_= _dat2_ _ord_= _ord2_)) as f2
			where  _dat1_ ^= _dat2_ and f1._ord1_ < f2._ord2_
			order by _dat1_, _dat2_, _ord1_, _ord2_
			;
		quit;
		/*data outlib.&dsn.0; set &dsn; run;*/
				
		data crossjoined;
			set crossjoined;
			_varmatch_=0;
			_labmatch_=0;
			_newvar1_= _var1_;
			_newvar2_= _var2_;
			_newlab1_= _lab1_;
			_newlab2_= _lab2_;
			if upcase(trim(left(_var1_))) = upcase(trim(left(_var2_))) then do;
				_varmatch_=1;
				call symputx("&flagVar", 1);
				if length(trim(left(_var1_))) >= 27 then do;
					if _dat1_ = 0 then _newvar1_= substr(trim(left(_var1_)),1,27)||"_b"||trim(left(_dat1_));
					else _newvar1_= substr(trim(left(_var1_)),1,27)||"_m"||trim(left(_dat1_));
				end;
				else do;
					if _dat1_ = 0 then _newvar1_= trim(left(_var1_))||"_b"||trim(left(_dat1_));
					else _newvar1_= trim(left(_var1_))||"_m"||trim(left(_dat1_));
				end;
			end;
			if upcase(trim(left(_lab1_))) = upcase(trim(left(_lab2_))) then do;
				_labmatch_=1;
				call symputx("&flagLab", 1);
				if _dat1_ = 0 then _newlab1_= trim(left(_lab1_))||"_b"||trim(left(_dat1_));			
				else _newlab1_= trim(left(_lab1_))||"_m"||trim(left(_dat1_));
			end;
		run;
		
		**variable names**;
		proc sort data= crossjoined out=newdatvars;
			by _dat1_ _ord1_ descending _varmatch_;
		run;
		/*data outlib.newdatvars0; set newdatvars; run;*/
		
		data newdatvars;
			set newdatvars end= _eof_;
			by _dat1_ _ord1_;
			if _eof_ then call symputx("lastdat", trim(left(_dat2_)));
			*if first._ord1_ then output;
			keep _origvar1_ _newvar1_ _origvar2_ _newvar2_ _dat1_ _dat2_ _ord1_ _ord2_ _varmatch_;
		run;
		/*data outlib.newdatvars1; set newdatvars; run;*/
		
		%put lastdat= &lastdat;
		/*
		**for code testing pourpose**;
		proc sql UNDO_POLICY= NONE;
			create table outlib.newdatvars_final_ as
			select a1.*, a2.*, 
				(case when a2._dat2_ = &lastdat then a2._dat2_ else a1._dat1_ end) as _dat_,
				(case when a2._dat2_ = &lastdat then a2._ord2_ else a1._ord1_ end) as _ord_,
				(case when a2._dat2_ = &lastdat then a2._newvar2_ else a1._newvar1_ end) as _var_,
				(case when a2._dat2_ = &lastdat then a2._origvar2_ else a1._origvar1_ end) as _origvar_,
				(case when a2._dat2_ = &lastdat then 0 else a1._varmatch_ end) as _vmatch_,
				monotonic() as _mono_
			from 
			newdatvars(keep= _origvar1_ _newvar1_ _dat1_ _ord1_ _varmatch_) as a1
			full join
			newdatvars(keep= _origvar2_ _newvar2_ _dat2_ _ord2_) as a2
			on a1._ord1_= a2._ord2_
			group by _dat_, _ord_, _vmatch_
			having max(_mono_) = _mono_
			order by _dat_, _ord_, _vmatch_ 
			;
		quit;
		*/
		
		proc sql UNDO_POLICY= NONE;
			create table newdatvars_final(keep= _dat_ _ord_ _origvar_ _var_ _vmatch_) as
			/*create table newdatvars_final as*/
			select a1.*, a2.*, 
				(case when a2._dat2_ = &lastdat then a2._dat2_ else a1._dat1_ end) as _dat_,
				(case when a2._dat2_ = &lastdat then a2._ord2_ else a1._ord1_ end) as _ord_,
				(case when a2._dat2_ = &lastdat then a2._newvar2_ else a1._newvar1_ end) as _var_,
				(case when a2._dat2_ = &lastdat then a2._origvar2_ else a1._origvar1_ end) as _origvar_,
				(case when a2._dat2_ = &lastdat then 0 else a1._varmatch_ end) as _vmatch_,
				monotonic() as _mono_
			from 
			newdatvars(keep= _origvar1_ _newvar1_ _dat1_ _ord1_ _varmatch_) as a1
			full join
			newdatvars(keep= _origvar2_ _newvar2_ _dat2_ _ord2_) as a2
			on a1._ord1_= a2._ord2_
			group by _dat_, _ord_, _vmatch_
			having max(_mono_) = _mono_
			order by _dat_, _ord_, _vmatch_ 
			;
		quit;
		
		data newdatvars_final;
			set newdatvars_final;
			by _dat_ _ord_;
			if last._ord_ then output;
		run;
		/*data outlib.newdatvars_final; set newdatvars_final; run;*/
			
		**variable labels**;
		proc sort data= crossjoined out=newdatlabs;
			by _dat1_ _ord1_ descending _labmatch_;
		run;
		/*data outlib.newdatlabs0; set newdatlabs; run;*/
		
		data newdatlabs;
			set newdatlabs end= _eof_;
			by _dat1_ _ord1_;
			if _eof_ then call symputx("lastdat", trim(left(_dat2_)));
			*if first._ord1_ then output;
			keep _origlab1_ _newlab1_ _origlab2_ _newlab2_ _dat1_ _dat2_ _ord1_ _ord2_ _labmatch_;
		run;
		/*data outlib.newdatlabs1; set newdatlabs; run;*/
		
		%put lastdat= &lastdat;
		proc sql UNDO_POLICY= NONE;
			create table newdatlabs_final(keep= _dat_ _ord_ _origlab_ _lab_ _lmatch_) as
			/*create table newdatlabs_final as*/
			select a1.*, a2.*, 
				(case when a2._dat2_ = &lastdat then a2._dat2_ else a1._dat1_ end) as _dat_,
				(case when a2._dat2_ = &lastdat then a2._ord2_ else a1._ord1_ end) as _ord_,
				(case when a2._dat2_ = &lastdat then a2._newlab2_ else a1._newlab1_ end) as _lab_,
				(case when a2._dat2_ = &lastdat then a2._origlab2_ else a1._origlab1_ end) as _origlab_,
				(case when a2._dat2_ = &lastdat then 0 else a1._labmatch_ end) as _lmatch_,
				monotonic() as _mono_
			from 
			newdatlabs(keep= _origlab1_ _newlab1_ _dat1_ _ord1_ _labmatch_) as a1
			full join
			newdatlabs(keep= _origlab2_ _newlab2_ _dat2_ _ord2_) as a2
			on a1._ord1_= a2._ord2_
			group by _dat_, _ord_, _lmatch_
			having max(_mono_) = _mono_
			order by _dat_, _ord_, _lmatch_
			;
		quit;
		
		data newdatlabs_final;
			set newdatlabs_final;
			by _dat_ _ord_;
			if last._ord_ then output;
		run;
		/*data outlib.newdatlabs_final; set newdatlabs_final; run;*/
		
		proc sql UNDO_POLICY= NONE;
			create table &dsn(drop= _dattmp_ _ordtmp_) as
			select a1.*, a2.*
			from 
			newdatvars_final(drop= _vmatch_) as a1,
			newdatlabs_final(rename= (_dat_= _dattmp_ _ord_= _ordtmp_) drop= _lmatch_) as a2
			where a1._dat_ = a2._dattmp_ and a1._ord_= a2._ordtmp_
			order by _dat_, _ord_
			;
		quit;

	%mend ChangeRepeatedVars;
	
	%macro GetMaxValue(_ds=, _varList=, _maxValue=, _minValue=);
		proc means data=&_ds max min noprint;
			var &_varList;
			output out=maxDat;
		run;
		
		proc transpose data=maxDat out=maxDatT;
			var &_varList;
			id _STAT_;
		run;
		
		proc sort data=maxDatT;
			by descending MAX;
		run;
		
		data _null_;
			set maxdatt(obs=1);
			call symputx("&_maxValue", max);
		run;
		
		proc sort data=maxDatT;
			by MIN;
		run;
		
		data _null_;
			set maxdatt(obs=1);
			call symputx("&_minValue", min);
		run;
	%mend;
	**end: auxiliary macros**;
	
	******processing input parameters from the xml file******;	
	**Checking Base Data Set and creaging OutName in case it is blank**;
	%GetOrigFileName(InData0,OrigInData0);
	%CheckDataName(_name=OutName,_default=&OrigInData0);

	**Checking Path;
	%PathName(&Indata0);
	%let InData0 = &TmpName;
	libname InLib0 "&TmpPath";
	%put InData0= &InData0;
		
	%put Subset= &Subset;
	%put BaseSubset= &BaseSubset;
	%if %index(%upcase(&Subset), NO) and %index(%upcase(&BaseSubset), NO) %then %do;
		%put ERROR: Either Delete Nonmatching Rows or Delete Rows Not in Base Input SAS Data Set must be checked in the General Tab. Please, return to this tab and check one of these two options.;
		%let exiterror= 1;
		%goto exit;
	%end;	
	
	**checking if variables KeyChromVar0 and KeyChromLab0 are the same**;
	%put KeyChromVar0 = &KeyChromVar0;
	%put KeyChromLab0 = &KeyChromLab0;
	%if &KeyChromVar0 = &KeyChromLab0 %then %let KeyChromLab0= ;
	%let ChromVar0Type= ;
	%put InData0= &InData0;
	%VarAttributes(_Ds=InLib0.&InData0,_Var=&KeyChromVar0,_Len=,_Type=ChromVar0Type);
			
	**these variables are used within the ChangeRepeatedVars macro**;
	%let flagQTLTestVars= 0;
	%let flagQTLTestLabs= 0;
	%let flagQTLEffVars= 0;
	%let flagQTLEffLabs= 0;
	%let flagQTLPosVars= 0;
	%let flagQTLPosLabs= 0;
	
	**data sets to hold variables names and labels*;
	proc datasets nolist;
		delete QTLTestNames_all QTLEffNames_all QTLPosNames_all;
	run;
	
	**checking variables in the base and merge data sets*;
	%let appendedQTLTest= 0;
	%let appendedQTLEff= 0;
	%let appendedQTLPos= 0;
	%let flagQTLEffExist= 0;
	%let flagQTLPosExist= 0;

	%let VType=;
	%let TotInData= 12;
	%do k= 0 %to &TotInData;
	 	%if %length(&&Indata&k) %then %do;
	 	
	 		%if &k= 0 %then %do;
				%let KeyChromLab&k= &KeyChromLab0;
				%let DataType&k= Base;	
			%end;
			%else %do;
				%let KeyChromLab&k= ;
				%let DataType&k= Merge;	

		 		**Checking Path;
				%PathName(&&Indata&k);
				%if &exiterror %then %do;
					%put ERROR: The &&DataType&k Input Data Set &k specification is invalid.;
					%goto exit;
				%end;
				%let InData&k = &TmpName;
				libname InLib&k "&TmpPath";
				%put InData&k= &&InData&k;
			%end;
			
			**Checking KeyChromVar variable type*;
			%VarAttributes(_Ds=InLib&k..&&InData&k,_Var=&&KeyChromVar&k,_Len=,_Type=VType);
			%let VTypeChVar= &VType;
			%put VTypeChVar= &VTypeChVar;
			%if &k= 0 %then %do;
				%let VTypeChVarRef= &VTypeChVar;
				%put VTypeChVarRef= &VTypeChVarRef;
				
				%if &VTypeChVarRef = N %then %do;
					%let string0= numeric;
				%end;
				%else %do;
					%let string0= character;
				%end;
			%end;
			%else %do;
				%if &VTypeChVar = N %then %do;
					%let string1= numeric;
				%end;
				%else %do;
					%let string1= character;
				%end;
			%end;
			
			%if &VTypeChVar ^= &VTypeChVarRef %then %do;
				%put ERROR: Variable &&KeyChromVar&k from &&DataType&k Input Data Set &k is of type &string1.. This variable must be of the same type (&string0) as the variable &&KeyChromVar0 from Base Input Data Set.;
				%let exiterror=1;
				%goto exit;
			%end;
			
			**Checking KeyGridVar variable type*;
			%VarAttributes(_Ds=InLib&k..&&InData&k,_Var=&&KeyGridVar&k,_Len=,_Type=VType);
			%if &VType ^= N %then %do;
				%put ERROR: Variable &&KeyGridVar&k from &&DataType&k Input Data Set &k is not a numeric variable.;
				%let exiterror=1;
				%goto exit;
			%end;
			
			**Checking KeyMarkerLab variable type*;
			%VarAttributes(_Ds=InLib&k..&&InData&k,_Var=&&KeyMarkerLab&k,_Len=,_Type=VType);
			%if &VType ^= C %then %do;
				%put ERROR: Variable &&KeyMarkerLab&k from &&DataType&k Input Data Set is not a character variable.;
				%let exiterror=1;
				%goto exit;
			%end;
			
			**Checking QTLTestVars variable type*;
			%CheckVarList(_Dsn=InLib&k..&&InData&k, DataType=&&DataType&k, VarList= &&QTLTestVars&k);
			%if &exiterror=1 %then %do;
				%goto exit;
			%end;
			
			data VarsDat;
				set VarsDat;
				_origvar_= _var_;
				_origlab_= _lab_;
			run;
							
			%if &appendedQTLTest= 0 %then %do;
				data QTLTestNames_all;
					set VarsDat;
				run;
				
				%let appendedQTLTest= 1;
			%end;
			%else %do;
			
				proc append base=QTLTestNames_all data=VarsDat;
								
				%ChangeRepeatedVars(dsn=QTLTestNames_all, flagVar= flagQTLTestVars, flagLab= flagQTLTestLabs);
			
				/*data outlib.QTLTestnewvars_final&k; set newdatvars_final; run;*/
				/*data outlib.QTLTestnewlabs_final&k; set newdatlabs_final; run;*/
			%end;
			/*data outlib.QTLTestNames&k; set VarsDat; run;*/
			/*data outlib.QTLTestNames_all&k; set QTLTestNames_all; run;*/
			
			proc datasets nolist;
				delete VarsDat;
			run;

			**Checking QTLEffVars variable type*;
			%put QTLEffVars&k= &&QTLEffVars&k;
			%if %length(&&QTLEffVars&k) %then %do;
				%let flagQTLEffExist= 1;
							
				%CheckVarList(_Dsn=InLib&k..&&InData&k, DataType=&&DataType&k, VarList= &&QTLEffVars&k);
				%if &exiterror=1 %then %do;
					%goto exit;
				%end;
				
				data VarsDat;
					set VarsDat;
					_origvar_= _var_;
					_origlab_= _lab_;
				run;
				
				%if &appendedQTLEff= 0 %then %do;
					data QTLEffNames_all;
						set VarsDat;
					run;
					
					%let appendedQTLEff= 1;
				%end;
				%else %do;
				
					proc append base=QTLEffNames_all data=VarsDat;
										
					%ChangeRepeatedVars(dsn=QTLEffNames_all, flagVar= flagQTLEffVars, flagLab= flagQTLEffLabs);

				%end;
				/*data outlib.QTLEffNames&k; set VarsDat; run;*/
				/*data outlib.QTLEffNames_all&k; set QTLEffNames_all; run;*/
				
				proc datasets nolist;
					delete VarsDat;
				run;
			%end;
			
			**Checking QTLPosVars variable type*;
			%if %length(&&QTLPosVars&k) %then %do;
				%let flagQTLPosExist= 1;
	
				%CheckVarList(_Dsn=InLib&k..&&InData&k, DataType=&&DataType&k, VarList= &&QTLPosVars&k);
				%if &exiterror=1 %then %do;
					%goto exit;
				%end;
					
				data VarsDat;
					set VarsDat;
					_origvar_= _var_;
					_origlab_= _lab_;
				run;
				
				%if &appendedQTLPos= 0 %then %do;
					data QTLPosNames_all;
						set VarsDat;
					run;
					
					%let appendedQTLPos= 1;
				%end;
				%else %do;
				
					proc append base=QTLPosNames_all data=VarsDat;
										
					%ChangeRepeatedVars(dsn=QTLPosNames_all, flagVar= flagQTLPosVars, flagLab= flagQTLPosLabs);

				%end;
				/*data outlib.QTLPosNames&k; set VarsDat; run;*/
				/*data outlib.QTLPosNames_all&k; set QTLPosNames_all; run;*/
				
				proc datasets nolist;
					delete VarsDat;
				run;
			%end;
			
			**Checking KeyChromLab variable type*;
			/*
			%if %length(&&KeyChromLab&k)^=0 %then %do;
				%VarAttributes(_Ds=InLib&k..&&InData&k,_Var=&&KeyChromLab&k,_Len=,_Type=VType);
			%end;
			*/
			
		%end; /*end if*/
	%end; /*end loop*/
	/*data outlib.QTLTestNames_all; set QTLTestNames_all; run;*/
	/*
	%if &flagQTLEffExist %then %do;
		data outlib.QTLEffNames_all; set QTLEffNames_all; run;
	%end;
	%if &flagQTLPosExist %then %do;
		data outlib.QTLPosNames_all; set QTLPosNames_all; run;
	%end;
	*/
	
	**create newnames to rename and relabel repeated variables*;	
	%let TotQTLTestVars= 0;
	%let QTLTestVarsList= ;
	%put flagQTLTestVars= &flagQTLTestVars;	
	%put flagQTLTestLabs= &flagQTLTestLabs;
	/*%if &flagQTLTestVars or &flagQTLTestLabs %then %do;*/
		%do k=0 %to &TotInData;
		
			%let OrigQTLTestVars&k= ;
			%let renameQTLTestVars&k= ;
			%let OrigQTLTestLabs&k= ;
			%let QTLTestLabs&k= ;
			%let relabeQTLTestVars&k= ;
				
			%if %length(&&QTLTestVars&k) %then %do;
				data _null_;
					set QTLTestNames_all end= _eof_;
					where _dat_= &k;
					call symputx("origvars"||trim(left(_n_)),trim(left(_origvar_)));
					call symputx("newvars"||trim(left(_n_)),trim(left(_var_)));
					call symputx("origlabs"||trim(left(_n_)),trim(left(_origlab_)));
					call symputx("newlabs"||trim(left(_n_)),trim(left(_lab_)));
					if _eof_ then call symputx("nrows", trim(left(_n_)));
				run;
				
				%let QTLTestVars&k= ;
				%do i=1 %to &nrows;
					%let OrigQTLTestVars&k= &&OrigQTLTestVars&k &&origvars&i;
					%let QTLTestVars&k= &&QTLTestVars&k &&newvars&i;
					%let OrigQTLTestLabs&k= &&OrigQTLTestLabs&k &&origlabs&i;
					%let QTLTestLabs&k= &&QTLTestLabs&k @$@&&newlabs&i;
					
					%let renameQTLTestVars&k= &&renameQTLTestVars&k &&origvars&i= &&newvars&i;
					%let relabeQTLTestVars&k= &&relabeQTLTestVars&k &&origvars&i= "&&newlabs&i";
					
					%let QTLTestVarsList= &QTLTestVarsList &&newvars&i;
				%end;
				%put NewQTLTestVars&k= &&QTLTestVars&k;
				%put NewQTLTestLabs&k= &&QTLTestLabs&k;
				%put renameQTLTestVars&k= &&renameQTLTestVars&k;
				%put relabeQTLTestVars&k= &&relabeQTLTestVars&k;
				
				%let TotQTLTestVars= %sysevalf(&TotQTLTestVars + &nrows);
			%end;
		%end;
	/*%end;
	%else %do;
		%do k=0 %to &TotInData;
			%let OrigQTLTestVars&k= &&QTLTestVars&k;
		%end;
	%end;
	*/
	
	%let TotQTLEffVars= 0;
	%let QTLEffVarsList= ;
	%put flagQTLEffVars= &flagQTLEffVars;	
	%put flagQTLEffLabs= &flagQTLEffLabs;
	/*%if &flagQTLEffVars or &flagQTLEffLabs %then %do;*/
		%do k=0 %to &TotInData;
			%let OrigQTLEffVars&k= ;
			%let renameQTLEffVars&k= ;
			%let OrigQTLEffLabs&k= ;
			%let QTLEffLabs&k= ;
			%let relabeQTLEffVars&k= ;
				
			%if %length(&&QTLEffVars&k) %then %do;
			
				data _null_;
					set QTLEffNames_all end= _eof_;
					where _dat_= &k;
					call symputx("origvars"||trim(left(_n_)),trim(left(_origvar_)));
					call symputx("newvars"||trim(left(_n_)),trim(left(_var_)));
					call symputx("origlabs"||trim(left(_n_)),trim(left(_origlab_)));
					call symputx("newlabs"||trim(left(_n_)),trim(left(_lab_)));
					if _eof_ then call symputx("nrows", trim(left(_n_)));
				run;
				
				%let QTLEffVars&k= ;
				%do i=1 %to &nrows;
					%let OrigQTLEffVars&k= &&OrigQTLEffVars&k &&origvars&i;
					%let QTLEffVars&k= &&QTLEffVars&k &&newvars&i;
					%let OrigQTLEffLabs&k= &&OrigQTLEffLabs&k &&origlabs&i;
					%let QTLEffLabs&k= &&QTLEffLabs&k @$@&&newlabs&i;
					
					%let renameQTLEffVars&k= &&renameQTLEffVars&k &&origvars&i= &&newvars&i;
					%let relabeQTLEffVars&k= &&relabeQTLEffVars&k &&origvars&i= "&&newlabs&i";
				
					%let QTLEffVarsList= &QTLEffVarsList &&newvars&i;
				%end;
				%put NewQTLEffVars&k= &&QTLEffVars&k;
				%put NewQTLEffLabs&k= &&QTLEffLabs&k;
				%put renameQTLEffVars&k= &&renameQTLEffVars&k;
				%put relabeQTLEffVars&k= &&relabeQTLEffVars&k;
				
				%let TotQTLEffVars= %sysevalf(&TotQTLEffVars + &nrows);
			%end;
		%end;
	/*%end;	
	%else %do;
		%do k=0 %to &TotInData;
			%let OrigQTLEffVars&k= &&QTLEffVars&k;
		%end;
	%end;
	*/

	%let TotQTLPosVars= 0;
	%let QTLPosVarsList= ;
	%put flagQTLPosVars= &flagQTLPosVars;	
	%put flagQTLPosLabs= &flagQTLPosLabs;
	/*%if &flagQTLPosVars or &flagQTLPosLabs %then %do;*/
		%do k=0 %to &TotInData;
		
			%let OrigQTLPosVars&k= ;
			%let renameQTLPosVars&k= ;
			%let OrigQTLPosLabs&k= ;
			%let QTLPosLabs&k= ;
			%let relabeQTLPosVars&k= ;
				
			%if %length(&&QTLPosVars&k) %then %do;
						
				data _null_;
					set QTLPosNames_all end= _eof_;
					where _dat_= &k;
					call symputx("origvars"||trim(left(_n_)),trim(left(_origvar_)));
					call symputx("newvars"||trim(left(_n_)),trim(left(_var_)));
					call symputx("origlabs"||trim(left(_n_)),trim(left(_origlab_)));
					call symputx("newlabs"||trim(left(_n_)),trim(left(_lab_)));
					if _eof_ then call symputx("nrows", trim(left(_n_)));
				run;
				
				%put nrows= &nrows;
				%let QTLPosVars&k= ;
				%do i=1 %to &nrows;
					%let OrigQTLPosVars&k= &&OrigQTLPosVars&k &&origvars&i;
					%let QTLPosVars&k= &&QTLPosVars&k &&newvars&i;
					%let OrigQTLPosLabs&k= &&OrigQTLPosLabs&k &&origlabs&i;
					%let QTLPosLabs&k= &&QTLPosLabs&k @$@&&newlabs&i;
					
					%put newlabs&i= &&newlabs&i;
					%let renameQTLPosVars&k= &&renameQTLPosVars&k &&origvars&i= &&newvars&i;
					%let relabeQTLPosVars&k= &&relabeQTLPosVars&k &&origvars&i= "&&newlabs&i";
				
					%let QTLPosVarsList= &QTLPosVarsList &&newvars&i;
				
				%end;
				%put here;
				%put NewQTLPosVars&k= &&QTLPosVars&k;
				%put NewQTLPosLabs&k= &&QTLPosLabs&k;
				%put renameQTLPosVars&k= &&renameQTLPosVars&k;
				%put relabeQTLPosVars&k= &&relabeQTLPosVars&k;
				
				%let TotQTLPosVars= %sysevalf(&TotQTLPosVars + &nrows);
			%end;
		%end;
	/*%end;	
	%else %do;
		%do k=0 %to &TotInData;
			%let OrigQTLPosVars&k= &&QTLPosVars&k;
		%end;
	%end;
	*/
	
	**create initial merged data set;
	data MergedData;
		set InLib0.&InData0(
			keep= &KeyChromVar0 &KeyChromLab0 &KeyGridVar0 &KeyMarkerLab0 
				&OrigQTLTestVars0 
				%if &flagQTLEffExist %then &OrigQTLEffVars0; 
				%if &flagQTLPosExist %then &OrigQTLPosVars0;
			);
		/*%if &flagQTLTestVars or &flagQTLTestLabs %then %do;*/
			label &relabeQTLTestVars0;
			rename &renameQTLTestVars0;
		/*%end;
		%if &flagQTLEffExist and (&flagQTLEffVars or &flagQTLEffLabs) %then %do;*/
		%if &flagQTLEffExist %then %do;
			label &relabeQTLEffVars0;
			rename &renameQTLEffVars0;	
		%end;
		/*%if &flagQTLPosExist and (&flagQTLPosVars or &flagQTLPosLabs) %then %do;*/
		%if &flagQTLPosExist %then %do;
			label &relabeQTLPosVars0;
			rename &renameQTLPosVars0;	
		%end;
	run;
	/*data outlib.MergedData0; set MergedData; run;*/
		
	**begin: one-to-one merge*;
	%let OrigQTLEff_=;
	%let OrigQTLPos_=;
	data _null_;
		%if &flagQTLEffExist %then call symputx("OrigQTLEff_", '&&OrigQTLEffVars&k');; 
		%if &flagQTLPosExist %then call symputx("OrigQTLPos_", '&&OrigQTLPosVars&k');;
	run;
	%let OrigQTLEff= %SUPERQ(OrigQTLEff_);
	%let OrigQTLPos= %SUPERQ(OrigQTLPos_);
	%put OrigQTLEff= &OrigQTLEff;
	%put OrigQTLPos= &OrigQTLPos;
	
	%if &ChromVar0Type = C %then
		%let onwherest= upcase(trim(left(a1.&KeyChromVar0)))= upcase(trim(left(a2._KeyChromTmp_))) and round(a1.&KeyGridVar0,0001)= round(a2._KeyGridTmp_,0001) and upcase(trim(left(a1.&KeyMarkerLab0)))= upcase(trim(left(a2._KeyMarkerTmp_)));
	%else
		%let onwherest= a1.&KeyChromVar0= a2._KeyChromTmp_ and round(a1.&KeyGridVar0,0001)= round(a2._KeyGridTmp_,0001) and upcase(trim(left(a1.&KeyMarkerLab0)))= upcase(trim(left(a2._KeyMarkerTmp_)));

	%do k=1 %to &TotInData; 
		%if %length(&&Indata&k) %then %do;
			/*data outlib.InData_&k; set InLib&k..&&InData&k; run;*/
		
			**creating proc sql from statement*;
			%if %index(%upcase(&Subset), YES) %then %do;
				%let fromstmt= 	%nrbquote( 
							MergedData as a1, 
							InLib&k..&&InData&k(
								keep= &&KeyChromVar&k &&KeyGridVar&k &&KeyMarkerLab&k &&OrigQTLTestVars&k &OrigQTLEff &OrigQTLPos
								rename= (&&KeyChromVar&k= _KeyChromTmp_ &&KeyGridVar&k= _KeyGridTmp_ &&KeyMarkerLab&k= _KeyMarkerTmp_)
								) as a2
							where &onwherest
						);
			%end;
			%else %if %index(%upcase(&BaseSubset), YES) %then %do;
				%let fromstmt= 	%nrbquote( 
						MergedData as a1
						left join
						InLib&k..&&InData&k(
							keep= &&KeyChromVar&k &&KeyGridVar&k &&KeyMarkerLab&k &&OrigQTLTestVars&k &OrigQTLEff &OrigQTLPos
							rename= (&&KeyChromVar&k= _KeyChromTmp_ &&KeyGridVar&k= _KeyGridTmp_ &&KeyMarkerLab&k= _KeyMarkerTmp_)
							) as a2
						on &onwherest
						);
			%end;
			%put fromstmt= %unquote(&fromstmt);
			
			proc sql UNDO_POLICY=NONE;
				create table MergedData(drop=_KeyChromTmp_ _KeyGridTmp_ _KeyMarkerTmp_ _mono_) as
				select a1.*, a2.*, monotonic() as _mono_
				from &fromstmt
				group by a1.&KeyChromVar0, a1.&KeyGridVar0, a1.&KeyMarkerLab0
				having max(_mono_) = _mono_
				order by &KeyChromVar0, &KeyGridVar0, a1.&KeyMarkerLab0
				;
			quit;
			
			**S1121540: To avoid problems in downstream analaysis*;
			%NOBsVars(MergedData);
			%if &nobs = 0 %then %do;
				%put ERROR: Merging the Merge Input Data Set &k left the merged data set with zero observations. This means that Merge Input Data Set does not have any observations in common with other data sets. Please, remove this data set or change the merging key variables.;
				%let k= %sysevalf(&TotInData + 1);
				%let exiterror= 1;
				%goto exit;
			%end;
			
			%put _relabeQTLTestVars&k= &&relabeQTLTestVars&k;
			%put _relabeQTLEffVars&k= &&relabeQTLEffVars&k;
			%put _relabeQTLPosVars&k= &&relabeQTLPosVars&k;
			data MergedData;
				set MergedData;
				/*%if &flagQTLTestVars or &flagQTLTestLabs %then %do;*/
					label &&relabeQTLTestVars&k;
					rename &&renameQTLTestVars&k;
				/*%end;
				%if &flagQTLEffExist and (&flagQTLEffVars or &flagQTLEffLabs) %then %do;*/
				%if &flagQTLEffExist %then %do;
					label &&relabeQTLEffVars&k;
					rename &&renameQTLEffVars&k;
				%end;
				/*%if &flagQTLPosExist and (&flagQTLPosVars or &flagQTLPosLabs) %then %do;*/
				%if &flagQTLPosExist %then %do;
					label &&relabeQTLPosVars&k;
					rename &&renameQTLPosVars&k;
				%end;
			run;
			/*data outlib.MergedData&k; set MergedData; run;*/
		%end;
	%end;
	**end: one-to-one merge*;
	/*data outlib.MergedData; set MergedData; run;*/
	
	**begin: adding marker indicator for the graphic results*;
	%if %index(%upcase(&DisplayMarkers), YES) or %index(%upcase(&DisplayQTL), YES) %then %do;

		data MergedData;
			set MergedData;
			_sindex_ = _n_;
		run;

		proc sort data=MergedData;
			by &KeyMarkerLab0;
		run;

		data MergedData;
			set MergedData;
			by &KeyMarkerLab0;
			MarkerIndicator= 0;
			if first.&KeyMarkerLab0 then MarkerIndicator= 1;
			label MarkerIndicator= "Marker Indicator";
		run;

		proc sort data=MergedData out=MergedData(drop= _sindex_);
			by _sindex_;
		run;

		proc means data=MergedData noprint;
			/*var MarkerIndicator %if %index(%upcase(&DisplayQTL), YES) %then QTLIndicator1;;????*/
			var MarkerIndicator;
			by &KeyChromVar0;
			/*output out=nmbych sum=nm %if %index(%upcase(&DisplayQTL), YES) %then nQTL;;?????*/
			output out=nmbych sum=nm;
		run;
		/*data outlib.nmbych; set nmbych; run;*/

		data _null_;
			set nmbych;
			call symputx("mbych"||trim(left(_n_)),nm);
			/*??????
			%if %index(%upcase(&DisplayQTL), YES) %then %do;
				call symputx("qtlbych"||trim(left(_n_)),nQTL);
			%end;
			*/
		run;
	%end;
	**end: adding marker indicator*;
	
	**getting max value of QTLTestVars*;
	%let maxQTLTest= ;
	%let minQTLTest= ;
	%GetMaxValue(_ds= MergedData,_varlist= &QTLTestVarsList, _maxValue= maxQTLTest, _minValue= minQTLTest);
	%let maxQTLTest= %sysevalf(&maxQTLTest + &maxQTLTest*0.05); 
	%put maxQTLTest= &maxQTLTest;
	
	**getting max value of QTLEffVars*;
	%let maxQTLEff= 0;
	%let minQTLEff= 0;
	%if &flagQTLEffExist %then %do;
		%GetMaxValue(_ds= MergedData,_varlist= &QTLEffVarsList, _maxValue= maxQTLEff, _minValue= minQTLEff);
		%let maxQTLEff= %sysfunc(max(%sysfunc(abs(&maxQTLEff)), %sysfunc(abs(&minQTLEff))));
		%let maxQTLEff= %sysevalf(&maxQTLEff + &maxQTLEff*0.05); 
		%put maxQTLEff= &maxQTLEff;
	%end;
	
	%*goto exit;
	
	**Prep MergedData for output*;
	%PrepOutDS(CompareQTLPlots_OutData,CompareQTLPlots_OutDataName,&OutName,_cqp);
	data OutLib.&CompareQTLPlots_OutDataName;
		set MergedData;
	run;

	%*goto exit;
	
	**Getting chromosomes values*;
	%let tChromNum=;
	%GetLevels(_Ds= MergedData, _Var= &KeyChromVar0, _LPrefix= ChrVars, _nLvls= tChromNum, _vType= ChrVarType);
	%put tChromNum= &tChromNum;
	
	**Getting chromosome labels*;
	%if %length(&KeyChromLab0) %then %do;
		%let tChromLab=;
		%GetLevels(_Ds= MergedData, _Var= &KeyChromLab0, _LPrefix= ChrLabel, _nLvls= tChromLab, _vType= ChrLabType);
		%put tChromLab= &tChromLab;
	%end;
	
	**create JSL file*;	
	%PrepOutFile(CompareQTLPlots_Script,CompareQTLPlots_ScriptName,&ProcessName Script for &OutName..jsl);
	%global _mainjslfile_;
	%let _mainjslfile_ = %quote(&CompareQTLPlots_Script);
	
	data _null_;
		file "&CompareQTLPlots_Script" lrecl=32767;
		put '//!';
		put "Names Default to Here(1);";

		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";	
		put 'wait(0.5);';

		put "Here:QTLFn = Function({},{default local}," ;
		put "If(IsEmpty(Here:qtldata),";
		put "	Here:qtldata=open(""&ClientOutpath.&CompareQTLPlots_OutDataName..sas7bdat"", invisible);";

		put "  column(""&KeyChromVar0"")<<Set Modeling Type(Ordinal);";
		put "  column(""&KeyMarkerLab0"")<<Label(1);";
		put "  column(""&KeyGridVar0"")<<data type(numeric);";
		%if %length(&KeyChromLab0) %then %do;
			put "  column(""&KeyChromLab0"")<<data type(character);";
		%end;
		put ","; /*else for Is empty check*/
		put "	 Current Data Table(Here:qtldata);";
		put ");";
		put "	Here:maxQTLTest= &maxQTLTest; ";
		**sbrlsi getting marker indicator*;
		%if %index(%upcase(&DisplayMarkers), YES) %then %do;
			put "	mi = :Name(""Marker Indicator"") << Get Values; ";
			put "	nm = sum(mi); ";
			put "	Here:ym = J(nm,1,-.1);";
			put "	Here:xm = J(nm,1,0);";
			put "	nr= N Row(mi);";
			put "	i= 1;";
			put "	for (j =1, j <= nr, j++,";
			put "		if (mi[j]==1,";
			put "			Here:xm[i]= :Name(""&KeyGridVar0"")[j];";
			put "			i= i + 1;";
			put "		)";
			put "	);";		
		%end;
		**sbrlsi getting QTL indicator*;
		%if %index(%upcase(&DisplayQTL), YES) %then %do;
			put "	Here:nqtl = 0; ";		
		%end;
		**end getting QTL indicator*;
		put "  Here:vlb = VListBox(";
		put "  Here:olb = outlineBox(""QTL Test Size Profile"", ";

		%macro qtlplots();	
			put "  Here:hlbq = hlistbox(";
			%let bem=1;
			%let enm=0;
			%if %index(%upcase(&DisplayQTL), YES) %then %do;
				put "Here:chrom = :Name(""&KeyChromVar0"") << Get Values;";
				put "Here:qtli = :Name(""QTL Position Indicator &TraitVar"") << Get Values;";
				put "nr= N Row(Here:qtli);";
			%end;
			%do i=1 %to &tChromNum %by 1;
				put "o=OutlineBox("@;
				%if %length(&KeyChromLab0) %then %do;
					put " ""&KeyChromLab0 &&ChrLabel&i"",";
				%end;
				%else %do;
					put " ""&KeyChromVar0 &&ChrVars&i"",";
				%end;
				put " Vlistbox(op=Graph Builder(";
				put "	Show Control Panel(0),";
				put "	Show Legend(1),";
				put " 	Variables(";
				put " 		X(:&KeyGridVar0),";
				%let l= 1;
				%do k= 0 %to &TotInData;
					%if %length(&&Indata&k) %then %do;
						%let j= 1;
						%let currLab= %scan(&&QTLTestLabs&k, &j, @$@);
						%do %while(%length(&currLab));
							%if &l=1 %then %do;
								put " Y(:&currLab)";
							%end;
							%else %do;
								put " Y(:&currLab, Position(1))";
							%end;
							%if &l < &TotQTLTestVars %then %do;
								put ",";
							%end;
							%let j= %sysevalf(&j + 1);
							%let currLab= %scan(&&QTLTestLabs&k, &j, @$@);
							%let l= %sysevalf(&l + 1);
						%end;
					%end;
				%end;
				put "),";
				put "Elements(";
				put "Points( X,"; 
				%let l= 1;
				%do k= 0 %to &TotInData;
					%if %length(&&Indata&k) %then %do;
						%let j= 1;
						%let currLab= %scan(&&QTLTestLabs&k, &j, @$@);
						%do %while(%length(&currLab));
							put " Y(&l)"@;
							%if &l < &TotQTLTestVars %then %do;
								put ","@;
							%end;
							%let j= %sysevalf(&j + 1);
							%let currLab= %scan(&&QTLTestLabs&k, &j, @$@);
							%let l= %sysevalf(&l + 1);
						%end;
					%end;
				%end;	
				put "  ,Legend( 5 )";
				put " ),";
				put "Line( X,"; 
				%let l= 1;
				%do k= 0 %to &TotInData;
					%if %length(&&Indata&k) %then %do;
						%let j= 1;
						%let currLab= %scan(&&QTLTestLabs&k, &j, @$@);
						%do %while(%length(&currLab));
							put " Y(&l)"@;
							%if &l < &TotQTLTestVars %then %do;
								put ","@;
							%end;
							%let j= %sysevalf(&j + 1);
							%let currLab= %scan(&&QTLTestLabs&k, &j, @$@);
							%let l= %sysevalf(&l + 1);
						%end;
					%end;
				%end;	
				put "  ,Legend( 9 )";
				put " ),";				
				put "),";				
				%if &tChromNum > 1 %then %do;
					%if %length(&KeyChromLab0) %then %do;
						put ", Where(uppercase(:&KeyChromLab0)==uppercase(""&&ChrLabel&i""))";
					%end;
					%else %do;
						%put VTypeChVarRef= &VTypeChVarRef;
						%if &VTypeChVarRef = N %then %do;
							put ", Where(:Name(""&KeyChromVar0"")==&&ChrVars&i)";
						%end;
						%else %do;
							put ", Where(:Name(""&KeyChromVar0"")==""&&ChrVars&i"")";
						%end;
					%end;
				%end;
				put ",SendToReport(";
				put "	Dispatch(";
				put "		{},";
				put "		""Graph Builder"",";
				put "		OutlineBox,";
				put "		{Set Title( ""LOD Profiles"" ), Image Export Display( Normal )}";
				put "	),";
				put "	Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
				put "	Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""LOD"" )} ),";
				put "),";
				put "),";
				put ")),";
				put "if(type(op)==""Scriptable"" | type(op)==""List"",";
				put "rop = op << report;";
				%if &tChromNum > 1 %then %do;
					put 'try(o[TextBox(1)] << Set Text("") << Set Font Size(1));';
				%end;	
				put "try(";
				%if &LODThresh > 0 %then %do;
					put "rop[framebox(1)] << add graphics script(hline(&LODThresh));";
				%end;
				**sbrlsi including marker indicator*;
				%if %index(%upcase(&DisplayMarkers), YES) %then %do;
					%let bem= %sysevalf(&enm + 1);
					%let enm= %sysevalf(&bem + &&mbych&i - 1);
					%*put bem= &bem;
					%*put enm= &enm;
					put "rop[framebox(1)] << add graphics script(";
					put "				Marker(16,";
					put "					ColorState(""black""),";
					put "					Here:xm[&bem::&enm],";
					put "					Here:ym[&bem::&enm] ";
					put "				)";
					put "			);";
				%end;
				**end including marker indicator*;
				**sbrlsi including qtl indicator*;
				%if %index(%upcase(&DisplayQTL), YES) %then %do;
					put "Here:nqtl&i = 0;";
					put "k=1;";
					put "for(j=1 , j <= nr, j++,";
					put "	if (Here:chrom[j]==&&ChrVars&i & Here:qtli[j]==1,";
					put "		if(k==1, ";
					put "			Here:xqtl&i= :Name(""&KeyGridVar0"")[j];";
					put "			Here:yqtl&i= 0.5;";
					put "		,";
					put "			Here:xqtl&i= Here:xqtl&i || :Name(""&KeyGridVar0"")[j];";
					put "			Here:yqtl&i= Here:yqtl&i || 0.5;";			
					put "		);";
					put "		Here:nqtl&i= Here:nqtl&i + 1;";
					put "		k= k + 1;";
					put "	);";
					put ");	";			
					put "if(Here:nqtl&i > 0, ";
					put 'Eval(Parse(EvalInsert("\[';
					put "				rop[framebox(1)] << add graphics script(";
					put "					Marker(21,";
					put "						ColorState(""red""),";
					put "						[^Here:xqtl&i^],";
					put "						[^Here:yqtl&i^] ";
					put "					)";
					put "				)";
					put '	]\")';
					put "		)";
					put "	);";
					put ");";
				%end;
				**end including qtl indicator*;
				put "rop[framebox(1)] << MarkerSize(2);";
				put "rop[framebox(1)] << MarkerDrawingMode(Normal);";		
				put "//rop[Axisbox(2)] << RemoveAxisLabel;";
				put "rop[Axisbox(2)] << Min(-0.5);";
				put "rop[Axisbox(2)] << Max(Here:maxQTLTest);";
				put "rop[Axisbox(2)] << Inc(2);";
				put "rop[Axisbox(2)] << Minor Ticks(0);";
				put "//rop[Axisbox(2)] << AddAxisLabel(""Y"");";
				%if %Upcase(&KeyGridVar0) = CM %then %do;
					put "rop[Axisbox(1)] << Min(-5);";
				%end;
				put "rop[framebox(1)] << FrameSize(275,175);";
				put "));";
				put " ";
			%end;
			put "	);";	
		%mend; 
		%qtlplots();	
		put "	);";					
		put ");";
		put ");";

		put "InsertInto(DataList,{{""Here:qtldata""}});";
		put "InsertInto(TabButtonList, ""Results"");";
		put "InsertInto(TabCodeList,expr(Here:QTLFn()));";
					
		%if &flagQTLEffExist %then %do;
			put ;
			put "//append effects plot";
			put "  Here:AppendEffFn = Function({},{default local}," ;
			put "	Here:maxQTLEff= &maxQTLEff; ";
			**begin: insert effect profile into the jsl from the first run*;
			put "  Here:olbe = outlineBox(""QTL Effect Size Profile"", ";
			%macro effplots();	
				put "  Here:hlbe = hlistbox(";
				%do i=1 %to &tChromNum %by 1;
					put "o=OutlineBox("@;
					%if %length(&KeyChromLab0) %then %do;
						put " ""&KeyChromLab0 &&ChrLabel&i"",";
						%*put KeyChromLab0 = &KeyChromLab0;
					%end;
					%else %do;
						put " ""&KeyChromVar0 &&ChrVars&i"",";
					%end;
					put "Vlistbox(op= Graph Builder(";
					put "	Show Control Panel(0),";
					put "	Show Legend(1),";
					put " 	Variables(";
					put " 		X(:&KeyGridVar0),";					
					%let l= 1;
					%do k= 0 %to &TotInData;
						%if %length(&&Indata&k) %then %do;
							%let j= 1;
							%let currLab= %scan(&&QTLEffLabs&k, &j, @$@);
							%do %while(%length(&currLab));
								%if &l=1 %then %do;
									put "Y(:&currLab)";
								%end;
								%else %do;
									put " Y(:&currLab, Position(1))";
								%end;
								%if &l < &TotQTLEffVars %then %do;
									put ",";
								%end;
								%let j= %sysevalf(&j + 1);
								%let currLab= %scan(&&QTLEffLabs&k, &j, @$@);
								%let l= %sysevalf(&l + 1);
							%end;
						%end;
					%end;
					put "),";
					put "Elements(";
					put "Points( X,"; 
					%let l= 1;
					%do k= 0 %to &TotInData;
						%if %length(&&Indata&k) %then %do;
							%let j= 1;
							%let currLab= %scan(&&QTLEffLabs&k, &j, @$@);
							%do %while(%length(&currLab));
								put " Y(&l)"@;
								%if &l < &TotQTLEffVars %then %do;
									put ","@;
								%end;
								%let j= %sysevalf(&j + 1);
								%let currLab= %scan(&&QTLEffLabs&k, &j, @$@);
								%let l= %sysevalf(&l + 1);
							%end;
						%end;
					%end;	
					put "  ,Legend( 5 )";
					put " ),";
					put "Line( X,"; 
					%let l= 1;
					%do k= 0 %to &TotInData;
						%if %length(&&Indata&k) %then %do;
							%let j= 1;
							%let currLab= %scan(&&QTLEffLabs&k, &j, @$@);
							%do %while(%length(&currLab));
								put " Y(&l)"@;
								%if &l < &TotQTLEffVars %then %do;
									put ","@;
								%end;
								%let j= %sysevalf(&j + 1);
								%let currLab= %scan(&&QTLEffLabs&k, &j, @$@);
								%let l= %sysevalf(&l + 1);
							%end;
						%end;
					%end;	
					put "  ,Legend( 9 )";
					put " ),";				
					put "),";				
				
					%if &tChromNum > 1 %then %do;
						%if %length(&KeyChromLab0) %then %do;
							put ", Where(uppercase(:&KeyChromLab0)==uppercase(""&&ChrLabel&i""))";
						%end;
						%else %do;
							%if &VTypeChVarRef = N %then %do;
								put ", Where(:Name(""&KeyChromVar0"")==&&ChrVars&i)";
							%end;
							%else %do;
								put ", Where(:Name(""&KeyChromVar0"")==""&&ChrVars&i"")";
							%end;
						%end;
					%end;
					put ",SendToReport(";
					put "	Dispatch(";
					put "		{},";
					put "		""Graph Builder"",";
					put "		OutlineBox,";
					put "		{Set Title( ""QTL Effect Profiles"" ), Image Export Display( Normal )}";
					put "	),";
					put "	Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
					put "	Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""QTL Effect"" )} ),";
					put "),";					
					put "),";
					put "),";
					put "if(type(op)==""Scriptable"" | type(op)==""List"",";
					put "rop = op << report;";
					%if &tChromNum > 1 %then %do;
						put 'try(o[TextBox(1)] << Set Text("") << Set Font Size(1));';
					%end;	
					put "try(";
					put "rop[framebox(1)] << add graphics script(hline(0));";		
					put "rop[framebox(1)] << MarkerSize(2);";
					put "rop[framebox(1)] << MarkerDrawingMode(Normal);";		
					put "//rop[Axisbox(2)] << RemoveAxisLabel;";
					put "rop[Axisbox(2)] << Min(-Here:maxQTLEff);";
					put "rop[Axisbox(2)] << Max(Here:maxQTLEff);";
					put "rop[Axisbox(2)] << Inc(.1);";
					put "rop[Axisbox(2)] << Minor Ticks(0);";
					put "//rop[Axisbox(2)] << AddAxisLabel(""Y"");";
					%if %Upcase(&KeyGridVar0) = CM %then %do;
						put "rop[Axisbox(1)] << Min(-5);";
					%end;
					put "rop[framebox(1)] << FrameSize(275,175);";
					put "));";
					put "),";
				%end;
				put "	);";	
			%mend; 
			%effplots();
			put "	);";
			put "	Here:vlb << Append(Here:olbe);" ; 
			put " );" ;
			put "InsertInto(TabAfterCodeList, expr(Here:AppendEffFn())); ";
			put "InsertInto(TabOpenList,1); ";
			**end: insert effect profile into the jsl from the first run*;
		%end;
		%else %do;
			put "InsertInto(TabAfterCodeList, "" ""); ";
			put "InsertInto(TabOpenList,1); ";			
		%end;		
	run;
	
	%PackageInsertFile(JSL Code from &ProcessName, &OutPath, &CompareQTLPlots_ScriptName, "&CompareQTLPlots_OutDataName..sas7bdat", yes);

	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend CompareQTLPlots;
%CompareQTLPlots;