/*
JMP Genomics Analytical Process 

Name: Basic Expression Workflow
Author: Russ Wolfinger, SAS Institute Inc.
History:
	sasrdw 08Jan08 initial coding 

Description: 
		This process runs a basic workflow
		for expression data, including options
		for quality control and BasicExpressionWorkflow.
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/



*ProcessVariables;

%global StudyName WFOutName InData Variables ListVars LabelVar BaseMergeVars
/*AnnoInData AnnoMergeVars AnnoChrVar AnnoPosVar AnnoKeepVars MaxLength 
AnnoOrganism AnnoAccession AnnoLabel AnnoGeneSymbol AnnoDescription AnnoGeneId AnnoSnpId */
ExpInData ClassVars ColorVars ExpLabelVar GroupVars WhereEddmt
AccProportion VCFirstPC VCLastPC PCAFixedEffects PCARandomEffects
ReplaceLow LowReplaceValue LowPct LowCut LowStd
ReplaceHigh HighReplaceValue HighPct HighCut HighStd
DelByNMiss DelByMean DelByMedian DelByPctl Percentile DelByStd DelByIQR DeleteCheck
FilGroupVars GroupPercent InDataNew
Dist CorrPCA CorrGroup StandardizationMethod QCTime
FixedEffects RandomEffects LsmeansEffects EstInFile
ClusterLSM LSMeansDiffSet LSMeansControlValues
MultipleTestingMethod Alpha NegLog10pCutoff 
OutData OutPath ServerOutPath MacroPath;

*ProcessBody;

%macro ExpressionQCWorkflow;

* initialization and argument checking;
%let ProcessName = ExpressionQCWorkflow; 
%global FullAPName;
%let FullAPName = %str(Expression QC Workflow);

%include "&MacroPath./UtilityMacros.sas" / nosource;

%if ((&ReplaceLow = yes) and ^%length(%trim(&LowReplaceValue))) or
	((&ReplaceHigh = yes) and ^%length(%trim(&HighReplaceValue))) %then %do;
	%put ERROR: Replacement value must be numeric or missing (.).;
	%let exiterror = 1;
	%goto exit;
%end;

* check methods applied;
%if &Dist=No & &CorrPCA=No & &CorrGroup=No & &StandardizationMethod=NONE &
	^%length(&FixedEffects) & ^%length(&RandomEffects) %then %do;
	%put ERROR: There is no Analysis Process selected.  Please check your settings on the last two tabs.;
	%goto exit;
%end;

%GetOrigFileName(InData,OrigInData);

* check name for OutData;
%if %length(&OrigInData) < 29 %then %let OID = &OrigInData;
%else %let OID = %substr(&OrigInData,1,28);
%PrepOutDS(OutData,OutDataName,&OID,_fin);

%let InDataNew= &OID._fin;

%let eid = %nrbquote(&ExpInData);
%put eid = &eid;
%if %length(&eid) %then %let ed = 1;
%else %let ed = 0;
%CheckInData(_ExpFlag=&ed);
%if &exiterror %then %goto exit;

%if ^%length(&StudyName) %then %do;
	%if %length(&WFOutName) %then %let StudyName=&WFOutName;
	%else %let StudyName=&InData;
%end;

%if ^%length(&Variables) and ^%length(&ListVars) %then %do;
/*	%if %length(&eid) %then %do;
		%do i = 1 %to &_nAllColChunk_;
			%if &i=1 %then %let Variables = &&_AllCol_&i._;
			%else %let Variables = Variables &&_AllCol_&i._ ;
		%end;
	%end;
	%else %do;*/
	%if ^%length(&eid) %then %do;
		/*%put NOTE: Filtering intensities over all numeric variables in &InData..;
		%let Variables = _numeric_;*/
		%global _nAllColChunk_ ; 
		%let _nAllColChunk_=1;
		%global _AllCol_&_nAllColChunk_._;
		%let _AllCol_&_nAllColChunk_._=_numeric_;
	%end;
%end;
%else %do;
	%CheckInData2(InLib.&InData,_Vars=&Variables,_VarListStyle=&ListVars);
%end;

/*%if (&InPath ^= &OutPath) %then %do;
	data OutLib.&InData;
		set InLib.&InData;
	run;
%end;*/

%put ExpInPath = &ExpInPath;
%put OutPath = &OutPath;

%if (&ExpInPath ^= &OutPath) & ^%length(&WhereEddmt) %then %do;
	data OutLib.&ExpInData;
		set ExpInLib.&ExpInData;
	run;
%end;
%if (&syserr>4 | &exiterror) %then %goto exit;

%if &ed and %length(&FilGroupVars) %then %do;
	data ed;
		set ExpInLib.&ExpInData;
	run;
	%GroupMacroVars(data=ed,vars=&FilGroupVars,prefix=g,maxnamelen=24);
	%if GroupPercent=0 %then %let GroupPercent=.01;
%end;
%if &exiterror %then %goto exit;

%if (%scan(&DeleteCheck,2,' ')= all) | (%scan(&DeleteCheck,2,' ')= any) %then %do;
	%if (%scan(&DeleteCheck,1,' ')=all) %then 
		%let DeleteCheckFlag = 1;
	%else %let DeleteCheckFlag = 0;
%end;

%put DeleteCheckFlag=&DeleteCheckFlag;

/* QC and Normalization */
%if (&Dist=Yes) %then %let ezdist = 1;
%else %let ezdist = 0;

%if (&CorrPCA=Yes) %then %let ezpca = 1;
%else %let ezpca = 0;

%if (&CorrGroup=Yes) %then %let ezgroup = 1;
%else %let ezgroup = 0;

/* Filter rows */
* compute column stats and save in macro variables;
%if (&ReplaceLow = yes) or (&ReplaceHigh = yes) %then %do;
	data dsix;
		set InLib.&InData(obs=0);
		keep %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
		drop _character_;
	run;
	
	%let _dsix_ = %sysfunc(open(dsix));
	%let nix = %sysfunc(attrn(&_dsix_,NVARS));

	%if (&ReplaceLow = yes) and (&LowPct > 0) %then %do;

		proc univariate data=InLib.&InData noprint; 
			var %do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end; ;
	
			output out=pOut
					 pctlpts  = &LowPct 
					 pctlpre  = %do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end;
					 pctlname = P; 
		data _null_;
			set pOut;
			%do v=1 %to &nix; 
				%let vname = %sysfunc(varname(&_dsix_,&v));
				call symputx("LowPctl&v",&vname.P);
			%end;
		run; 
		 
	%end;

	%if (&ReplaceHigh = yes) and (&HighPct < 100) %then %do;
		proc univariate data=InLib.&InData noprint; 
			var %do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end; ;
	
			output out=pOut
					 pctlpts  = &HighPct 
					 pctlpre  = %do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end;
					 pctlname = P; 
		data _null_;
			set pOut;
			%do v=1 %to &nix; 
				%let vname = %sysfunc(varname(&_dsix_,&v));
				call symputx("HighPctl&v",&vname.P);
			%end;
		run; 
		
	%end;

	%if ((&ReplaceLow = yes) and (&LowStd < 100)) or
		 ((&ReplaceHigh = yes) and (&HighStd < 100)) %then %do;

		proc means data=InLib.&InData noprint; 
			var %do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end; ;
	
			output out=mOut(drop=_type_ _freq_)
				mean(%do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end;) = 
					%do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end; ;
		run;
		data _null_;
			set mOut;
			%do v=1 %to &nix; 
				call symputx("Mean&v",%sysfunc(varname(&_dsix_,&v)));
			%end;
		run; 

		proc means data=InLib.&InData noprint; 
			var %do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end; ;
		
			output out=sOut(drop=_type_ _freq_)
				std(%do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end;) = 
					%do v=1 %to &nix; %sysfunc(varname(&_dsix_,&v)) %end; ;
		run;
		data _null_;
			set sOut;
			%do v=1 %to &nix; 
				call symputx("Std&v",%sysfunc(varname(&_dsix_,&v)));
			%end;
		run; 
	%end;
%end;

* replace values and filter rows;
data &OutData(drop=%do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;) vars(keep=%do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
   set InLib.&InData;
 
	* replace lowest values;
	%if (&ReplaceLow = yes) %then %do;
		%do v=1 %to &nix;
			%let cv = %sysfunc(varname(&_dsix_,&v));
			%if (&LowPct > 0) %then %do;
				 if &cv < &&LowPctl&v then &cv = &LowReplaceValue;
			%end;
			
			if &cv < &LowCut then &cv = &LowReplaceValue;
			
			%if (&LowStd < 100) %then %do;
				 if (&cv - &&Mean&v)/&&Std&v < -&LowStd then &cv = &LowReplaceValue;
			%end;	
		%end;
	%end;

	* replace highest values;
	%if (&ReplaceHigh = yes) %then %do;
		%do v=1 %to &nix;
			%let cv = %sysfunc(varname(&_dsix_,&v));
			%if (&HighPct < 100) %then %do;
				 if &cv > &&HighPctl&v then &cv = &HighReplaceValue;
			%end;
			
			if &cv > &HighCut then &cv = &HighReplaceValue;
			
			%if (&HighStd < 100) %then %do;
				 if (&cv - &&Mean&v)/&&Std&v > &HighStd then &cv = &HighReplaceValue;
			%end;	
		%end;
	%end;
	
	* filter rows;
	%if (%scan(&DeleteCheck,2,' ')= all) | (%scan(&DeleteCheck,2,' ')= any) %then %do;
		%if (&DeleteCheckFlag) %then %do;
			* using "and" logic here, that is, all conditions must be met before the row is deleted;
			* user can apply "or" logic by running the AP multiple times;
			if %length(&DelByNMiss) | %length(&DelByMean) | %length(&DelByMedian) | %length(&DelByPctl) 
				| %length(&DelByStd) | %length(&DelByIQR) then _deleteit = 1;
			else _deleteit = 0;
			
			%if %length(&FilGroupVars) %then %do;
				%if %length(&DelByNMiss) %then %do;
					%let _gcount = 0;
					%do j=1 %to &gNum;
						NMISS = nmiss(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByNMiss then _gcount+1;
					%end;
					if (100*_gcount/&gNum) < &GroupPercent then _deleteit = 0;
					drop NMISS;
				%end;
				%if %length(&DelByMean) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						MEAN = mean(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByMean then _gcount+1;
					%end;
					if (100*_gcount/&gNum) < &GroupPercent then _deleteit = 0;
					drop MEAN;
				%end;
				%if %length(&DelByMedian) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						MEDIAN = median(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByMedian then _gcount+1;
					%end;
					if (100*_gcount/&gNum) < &GroupPercent then _deleteit = 0;
					drop MEDIAN;
				%end;
				%if %length(&DelByPctl) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						PCTL = pctl(&Percentile,of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByPctl then _gcount+1;
					%end;
					if (100*_gcount/&gNum) < &GroupPercent then _deleteit = 0;
					drop PCTL;
				%end;
				%if %length(&DelByStd) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						STD = std(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByStd then _gcount+1;
					%end;
					if (100*_gcount/&gNum) < &GroupPercent then _deleteit = 0;
					drop STD;
				%end;
				%if %length(&DelByIQR) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						IQR = iqr(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByIQR then _gcount+1;
					%end;
					if (100*_gcount/&gNum) < &GroupPercent then _deleteit = 0;
					drop IQR;
				%end;
				drop _gcount;
			%end;
			%else %do;
				%if %length(&DelByNMiss) %then %do;
					NMISS = nmiss(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
					if not (&DelByNMiss) then _deleteit = 0;
				%end;
				%if %length(&DelByMean) %then %do;
					MEAN = mean(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
					if not (&DelByMean) then _deleteit = 0;
				%end;
				%if %length(&DelByMedian) %then %do;
					MEDIAN = median(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
					if not (&DelByMedian) then _deleteit = 0;
				%end;
				%if %length(&DelByPctl) %then %do;
					PCTL = pctl(&Percentile,of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
					if not (&DelByPctl) then _deleteit = 0;
				%end;
				%if %length(&DelByStd) %then %do;
					STD = std(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
					if not (&DelByStd) then _deleteit = 0;
				%end;
				%if %length(&DelByIQR) %then %do;
					IQR = iqr(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
					if not (&DelByIQR) then _deleteit = 0;
				%end;
			%end;
			if _deleteit then delete;
			drop _deleteit;
		%end;
		%else %do;
			_deleteit2 = 0;
			%if %length(&FilGroupVars) %then %do;
				%if %length(&DelByNMiss) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						NMISS = nmiss(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByNMiss then _gcount+1;
					%end;
					if (100*_gcount/&gNum) > &GroupPercent then _deleteit2 = 1;
					drop NMISS;
				%end;
				%if %length(&DelByMean) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						MEAN = mean(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByMean then _gcount+1;
					%end;
					if (100*_gcount/&gNum) > &GroupPercent then _deleteit2 = 1;
					drop MEAN;
				%end;
				%if %length(&DelByMedian) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						MEDIAN = median(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByMedian then _gcount+1;
					%end;
					if (100*_gcount/&gNum) > &GroupPercent then _deleteit2 = 1;
					drop MEDIAN;
				%end;
				%if %length(&DelByPctl) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						PCTL = pctl(&Percentile,of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByPctl then _gcount+1;
					%end;
					if (100*_gcount/&gNum) > &GroupPercent then _deleteit2 = 1;
					drop PCTL;
				%end;
				%if %length(&DelByStd) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						STD = std(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByStd then _gcount+1;
					%end;
					if (100*_gcount/&gNum) > &GroupPercent then _deleteit2 = 1;
					drop STD;
				%end;
				%if %length(&DelByIQR) %then %do;
					_gcount = 0;
					%do j=1 %to &gNum;
						IQR = iqr(of %do k=1 %to &&gNum&j; &&G&j.R&k %end;);
						if &DelByIQR then _gcount+1;
					%end;
					if (100*_gcount/&gNum) > &GroupPercent then _deleteit2 = 1;
					drop IQR;
				%end;
				if _deleteit2 then delete;
				drop _deleteit2;
				drop _gcount;
			%end;
			%else %do;
			   %if %length(&DelByNMiss) %then %do;
			   	NMISS = nmiss(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
			   	if &DelByNMiss then delete;
			   %end;
	 		  %if %length(&DelByMean) %then %do;
	   			MEAN = mean(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
	   			if &DelByMean then delete;
	 		  %end;
	 		  %if %length(&DelByMedian) %then %do;
	   			MEDIAN = median(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
	  		 	if &DelByMedian then delete;
	 		  %end;
			   %if %length(&DelByPctl) %then %do;
	  		 	PCTL&Percentile = pctl(&Percentile,of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
	 		  	if &DelByPctl then delete;
	 		  %end;
			   %if %length(&DelByStd) %then %do;
			   	STD = std(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
	 		  	if &DelByStd then delete;
	 		  %end;
	 		  %if %length(&DelByIQR) %then %do;
	 		  	IQR = iqr(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
	 		  	if &DelByIQR then delete;
			   %end;
			   drop _deleteit2;
			%end;
		%end;
	%end;
run;


%if (&ReplaceLow = yes) or (&ReplaceHigh = yes) or (%scan(&DeleteCheck,2,' ')= all) or (%scan(&DeleteCheck,2,' ')= any)  %then %do;

	data nonmiss;
		set vars;
		if sum(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;) = . then delete;
	run;
	
	/*data OutLib.nonmiss; set nonmiss; run;*/
	
	%NObsVars(nonmiss);
	%if &nobs < 1 %then %do;
		%put ERROR: There are no valid observations after replacing high and/or low values and/or deleting rows.;
		%let exiterror = 1;
		%goto exit;
	%end;

%end;

data &OutData;
	merge &OutData vars;
run;

%NObsVars(&OutData);
%if (&nobs=0) %then %do;
	%put ERROR: No observations passed the delete filter(s).  You may want to relax or change the criteria.;
	%let exiterror = 1;
	%goto exit;
%end;

%let ezanova = 0;
%let ezStats=1;
%let OutName = &StudyName;
%if ^%length(&NegLog10pCutoff) %then %let NegLog10pCutoff = 0;

%if (&ezpca = 1) %then %do;
	%let FixedTmp = %sysfunc(translate(%nrbquote(&PCAFixedEffects)," ","("," ",")"," ","|"," ","*"));
	%let RandomTmp = %sysfunc(translate(%nrbquote(&PCARandomEffects)," ","("," ",")"," ","|"," ","*"));
	/*
	%if %index(%nrbquote(&PCAFixedEffects),%quote(|)) %then %let FixedTmp = %sysfunc(tranwrd(%nrbquote(&PCAFixedEffects),%quote(|), ));
	%else %if %index(%nrbquote(&PCAFixedEffects),%quote(*)) %then %let FixedTmp = %sysfunc(tranwrd(%nrbquote(&PCAFixedEffects),%quote(*), ));
	%else %let FixedTmp = &PCAFixedEffects;
	%if %index(%nrbquote(&PCARandomEffects),%quote(|)) %then %let RandomTmp = %sysfunc(tranwrd(%nrbquote(&PCARandomEffects),%quote(|), ));
	%else %if %index(%nrbquote(&PCARandomEffects),%quote(*)) %then %let RandomTmp = %sysfunc(tranwrd(%nrbquote(&PCARandomEffects),%quote(*), ));
	%else %let RandomTmp=&PCARandomEffects;
	*/
	%if %index(%nrbquote(&FixedTmp),%quote(@)) %then %let FixedTmp2 = %substr(&FixedTmp, 1, %index(%nrbquote(&FixedTmp),%quote(@))-1);
	%else %let FixedTmp2 = &FixedTmp;
	%if %index(%nrbquote(&RandomTmp),%quote(@)) %then %let RandomTmp2 = %substr(&RandomTmp, 1, %index(%nrbquote(&RandomTmp),%quote(@))-1);
	%else %let RandomTmp2 = &RandomTmp;
	%let ClassVars2 = &FixedTmp2 &RandomTmp2 ;
%end;

* create JSL file;
%global BEW_Script;
%PrepOutFile(BEW_Script,BEW_ScriptName,&ProcessName Script for &InData..jsl);

* put JSL statements;
data _null_;
   file "&BEW_Script";
	put "//!";
	put "Names Default to Here(1);";	
	put "if(hostis(windows), _slash_ = ""\"", _slash_ = ""/"");";
	put "if(Host Is(Windows),";
	put "   jmpGenomicsDir = ConvertFilePath(""$LS_HOME"", absolute, windows),";
	put "   jmpGenomicsDir = ConvertFilePath(""$LS_HOME"", absolute, POSIX)";
	put ");";
	put "include(jmpGenomicsDir||""Generator""||_slash_||""WizardHelperFunctions.jsl"");";
	put "include(jmpGenomicsDir||""Generator""||_slash_||""WorkflowHelperFunctions.jsl"");";
	put "jg:_BEWF_ = 1;";
	put "here:EZ_ProcessName = ""&ProcessName"";";
	put "here:EZ_AE_StudyID = ""&StudyName"";";
	put "here:JMPG_WF_Flag =1;";
	put "EZ_AE_CHPIE = 0;";
	put "EZ_AE_CHPFormat = "" "";";
	put "CDFRadio = 0;";
	put "LabelVar = ""&LabelVar"";";
	put "BaseMergeVars = ""&BaseMergeVars"";";
	put "jg:AnnoRadio = 3;";
	put "jg:AnnoInData = "" "";";
	put "AnnoMergeVars = "" "";";
	put "AnnoLabel = "" "";";
	put "AnnoAccession = "" "";";
	put "jg:AnnoGeneSymbol = "" "";";
	put "jg:AnnoDescription = "" "";";
	put "AnnoGeneId = "" "";";
	put "jg:EZ_AE_OutPath = ""&OutPath"";";
	put "jg:EZ_AE_ClientOutPath = ""&ClientOutPath"";";
	put "jg:EZ_AE_ServerOutPath = ""&ServerOutPath"";";
	put "EZ_AE_statistics = 1;";
	put "EZ_AE_hcluster = 0;";
	put "EZ_AE_kmcluster = 0;";
	put "EZ_AE_PCA = 0;";
	put "jg:EZ_AE_InData = ""&OutDataName"";";
	put "EZ_AE_ExpInPath = ""&ExpInPath"";";
	put "EZ_AE_ExpInData = ""&ExpInData"";";
	put "EZ_AE_AP = 1;";
	put "ColorVars = ""&ColorVars"";";
	put "ExpLabelVar = ""&ExpLabelVar"";";
	put "GroupVars = ""&GroupVars"";";
	put "FilGroupVars = ""&FilGroupVars"";";
	put "Variables = ""&Variables"";";
	put "ListVars = ""&ListVars"";";
	put "FixedEffects = ""&FixedEffects"";";
	put "RandomEffects = ""&RandomEffects"";";
	%if (&ezpca = 1) %then %do;
		put "AccProportion = ""&AccProportion"";";
		put "PCARandomEffects = ""&PCARandomEffects"";";
		put "PCAFixedEffects = ""&PCAFixedEffects"";";
		put "ClassVars2 = ""&ClassVars2"";";
		put "VCFirstPC = ""&VCFirstPC"";";
		put "VCLastPC = ""&VCLastPC"";";
	%end;
	
	%if %index(&QCTime,Before) %then %do;	
		put "jg:EZ_AE_Dist = &ezdist;";
		put "jg:EZ_AE_CorrPCA = &ezpca;";
		put "jg:EZ_AE_CorrGroup = &ezgroup;";
		put "jg:EZ_AE_QC2 = 0;";
		put "jg:EZ_AE_Dist2 = 0;";
		put "jg:EZ_AE_CorrPCA2 = 0;";
		put "jg:EZ_AE_CorrGroup2 = 0;";
	%end;
	%else %if %index(&QCTime,After) %then %do;;
		put "jg:EZ_AE_Dist = 0;";
		put "jg:EZ_AE_CorrPCA = 0;";
		put "jg:EZ_AE_CorrGroup = 0;";		
		put "jg:EZ_AE_QC2 = 1;";
		put "jg:EZ_AE_Dist2 = &ezdist;";
		put "jg:EZ_AE_CorrPCA2 = &ezpca;";
		put "jg:EZ_AE_CorrGroup2 = &ezgroup;";
	%end;
	%else %do;
		put "jg:EZ_AE_Dist = &ezdist;";
		put "jg:EZ_AE_CorrPCA = &ezpca;";
		put "jg:EZ_AE_CorrGroup = &ezgroup;";
		put "jg:EZ_AE_QC2 = 1;";
		put "jg:EZ_AE_Dist2 = &ezdist;";
		put "jg:EZ_AE_CorrPCA2 = &ezpca;";
		put "jg:EZ_AE_CorrGroup2 = &ezgroup;";	
	%end;
	put "jg:NormMethod = ""&StandardizationMethod"";";
	put "jg:EZ_AE_ANOVA = &ezanova;";
	put "ClassVars = ""&ClassVars"";";
	
	%if (&ezanova = 1) %then %do;
		%if (&oneway = 1) %then %do;
			put "jg:EZ_AE_OneWay = 1;";
			put "FixedEffects = ""&bf"";";
			put "BlockVars = ""&br"";";
		%end;
		%else %do;
			put "jg:EZ_AE_OneWay = 0;";
			put "ByVars = """";";
		%end;
		put "LSMeansEffects = ""&LSMeansEffects"";";
		put "EZ_AE_MeanDiff = ""&LSMeansDiffSet"";";
		put "ClusterLSM = ""&ClusterLSM"";";
		put "EZ_AE_ControlLevel1 = {};";
		%let len = %length(&LSMeansControlValues);
		%put LSMeansControlValues = &LSMeansControlValues;
		%put len = &len;
		%if &len %then %do;
			%let LSM = %quote(&LSMeansControlValues);
			%put LSM = &LSM;
			%let LSMCV = ;
			%do i=1 %to &len;
				%let c1 = %qsubstr(&LSM,&i,1);
				%put i=&i c1=&c1;
				%if ^%length(&c1.&c1) %then %do;
					%let LSMCV = &LSMCV.%str( );
				%end;
				%else %if &c1.&c1 = %str(%"%") %then %do;
					%let LSMCV = &LSMCV.\!&c1;
				%end;
				%else %do;
					%let LSMCV = &LSMCV.&c1;
				%end;
			%end;
			%put LSMCV = &LSMCV;
			put "c1 = ""&LSMCV"";";
			put "lc1 = length(c1);";
			put "c2 = substr(c1,2,lc1-2);";
			put "InsertInto(EZ_AE_ControlLevel1,c2);";
		%end;
		put "EZ_AE_ControlLevel2 = {};";
		put "EstInFile = ""&EstInFile"";";
		put "MultiTest = ""&MultipleTestingMethod"";";
		put "EZ_AE_Alpha = &Alpha;";
		put "Cutoff = &NegLog10pCutoff;";
	%end;
	put "EZ_AE_uploadGene = 0;";
	put "_CreateWorkflow;";
	put 'jg:_BEWF_=.;';
	put "here:JMPG_WF_Flag =0;";
run;

%if (%scan(&DeleteCheck,2,' ')= all) | (%scan(&DeleteCheck,2,' ')= any) %then %do;
	%PackageInsertDataset(Filtered data from &ProcessName, OutLib, &OutDataName,_type=other);
%end;
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &BEW_ScriptName, , text/plain, yes);



%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ExpressionQCWorkflow;


* call the macro;
%ExpressionQCWorkflow;


