/*
JMP Genomics Analytical Process

Name: GxE Interaction

*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData InMeans QTVar GenotypeVar EnvVar RepVar WhereStmt 
InRepN InMSL InMSB InMSE InDFMSE Test HetResid ModelType Alpha OutAMMIPred OutPath 
ServerOutPath OutName MacroPath 
GxEInteraction_OutDataMeans GxEInteraction_OutDataPCA GxEInteraction_OutDataScree GxEInteractionScript GxEInteraction_OutDataStab 
GxEInteraction_OutDataLSM GxEInteraction_OutDataSS GxEInteraction_OutDataSSP GxEInteraction_OutDataAMMIPred;

*ProcessBody;

%macro GxEInteraction;

*initialization and argument checking;
%let ProcessName = GxEInteraction; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%GetOrigFileName(InData,OrigInData);

%let TraitVars = &QTVar;
%let t=1;
%let maxtrtnamelen = 0;
%do %while(%length(%scan(&TraitVars,&t)));
	%if %length(%scan(&TraitVars,&t)) > &maxtrtnamelen %then %let maxtrtnamelen = %length(%scan(&TraitVars,&t));
	%let t=%eval(&t+1);
%end;
%let NTraits = %eval(&t-1);

%if &NTraits > 1 %then %let maxl = %eval(28 -  (&maxtrtnamelen+1));
%else %let maxl=28;

* check name for OutData;
%CheckDataName(_name=OutName,_default=&OrigInData);
%if %length(&OutName)>&maxl %then %let OutName=%substr(&OutName,1,&maxl);

%PathName(&InData);
%if &exiterror %then %do;
   %put ERROR: Input dataset specification is invalid.;
   %goto exit;
%end;
%let InData = &TmpName;
libname InLib "&TmpPath";

libname OutLib "&OutPath";

%let dsid=%sysfunc(open(InLib.&InData,i));
%let gvartype=%sysfunc(vartype(&dsid,%sysfunc(varnum(&dsid,&GenotypeVar))));
%let i=1;
%do %while(%length(%scan(&EnvVar,&i)));
	%let evartype&i=%sysfunc(vartype(&dsid,%sysfunc(varnum(&dsid,%scan(&EnvVar,&i)))));   
	%let i=%eval(&i+1);
%end;
%let nEnvVar= %eval(&i-1);
%let rc=%sysfunc(close(&dsid));

ods listing close;

**filtering observations with missing entries for either Env or Geno or Reps**;
data filtermiss;
	set Inlib.&InData;
	if 
		%let i=1;
		%do %while(%length(%scan(&EnvVar,&i)));
			missing(%scan(&EnvVar,&i))=0 and
			%let i=%eval(&i+1);
		%end;
		missing(&GenotypeVar)=0 %if %upcase(&InMeans) = NO %then and missing(&RepVar)=0;
	;
	%if %length(&WhereStmt) %then %do;
	  	where &WhereStmt;
 	%end;
run;

**Error checking: number of obsertaions must be greater than zero after applying first filters**;
%Nobsvars(filtermiss);
%let totalobs = &nobs;
%if &totalobs = 0 %then %do;
	%put ERROR: Number of observations in the data set is zero after removing observations with missing entries for &EnvVar &RepVar &WhereStmt..;
	%let exiterror=1;
	%goto exit;
%end;

**creating a new numeric ID variable for genotypes**;
proc sort data=filtermiss;
	by &GenotypeVar;
run;

proc sort data=filtermiss nodupkey out=countgen;
	by &GenotypeVar;
run;
/*data OutLib.countgen0; set countgen; run;*/

data countgen;
	set countgen(keep= &GenotypeVar);
	_gennum_ = _n_;
run;		
/*data OutLib.countgen1; set countgen; run;*/		

data filtermiss;
	merge filtermiss countgen;
	by &GenotypeVar;
run;

%let ng1dropenv= 0;
%if %upcase(&InMeans) = YES %then %do;
	
	**counting number of Env**;
	proc sort data=filtermiss nodupkey out=countenv;
		by &EnvVar;
	run;
	
	%Nobsvars(countenv);
	%let env_n = &nobs;
	%put env_n= &env_n;
	
	** Remove genotypes that are not present in all environments **;
	proc sort data=filtermiss nodupkey out=genenv;
		by &GenotypeVar &EnvVar;
	run;
	
	/*data OutLib.ge_genenv; set genenv; run;*/
	
	proc freq data=genenv;
		tables &GenotypeVar / out=freqs(keep=&Genotypevar count rename=(count=_ct_));
	run;
	
	/*data OutLib.ge_freqs; set freqs; run;*/
	
	proc sort data=filtermiss;
		by &GenotypeVar;
	run;
	
	data filtermiss g1dropenv;
		merge filtermiss freqs;
		by &Genotypevar;
		*if _ct_ ne &env_n then delete;
		if _ct_ ne &env_n then output g1dropenv;
		else output filtermiss;
		drop _ct_;
	run;
	/*data OutLib.ge_g1dropenv; set g1dropenv; run;*/
	/*data OutLib.ge_filtermiss01; set filtermiss; run;*/

	%Nobsvars(g1dropenv);
	%let ng1dropenv= &nobs;
	%if &ng1dropenv %then %do;
		proc sort data=g1dropenv(keep= &GenotypeVar &EnvVar) nodupkey;
			by &GenotypeVar &EnvVar;
		run;
	%end;

	**counting number of observations**;
	%Nobsvars(filtermiss);
	%let totalobs = &nobs;
	
	%if &totalobs = 0 %then %do;
		%put ERROR: Number of observations in the data set is zero after removing genotypes that are not present in all environments.;
		%put ERROR: There are &env_n environments in the data set. Please, check if each genotype appears in all environments.; 
		%let exiterror=1;
		%goto exit;
	%end;
	
	**re-counting number of Env**;
	proc sort data=filtermiss nodupkey out=countenv;
		by &EnvVar;
	run;
	
	%Nobsvars(countenv);
	%let env_n = &nobs;
	
	**counting number of Geno**;
	proc sort data=filtermiss nodupkey out=countgen;
		by &GenotypeVar;
	run;
	/*data OutLib.countgen0; set countgen; run;*/
	
	%Nobsvars(countgen);
	%let gen_n = &nobs;
	
	**S1268333: Adding a check to verify if there is only one entry of each genotype per environment**;
	%if &totalobs gt %eval(&env_n * &gen_n) %then %do;
		%put ERROR: Some genotype entries were detected more than once in some environments. When the option Input Data Set Contains Means of Genotypes on Each Environment is checked, the input data set must have exactly one entry of each genotype per environment.;
		%let exiterror= 1;
		%goto exit;
	%end;
	%else %if &totalobs lt %eval(&env_n * &gen_n) %then %do;
		%put ERROR: Some genotype entries were not detected in some environments. When the option Input Data Set Contains Means of Genotypes on Each Environment is checked, the input data set must have exactly one entry of each genotype per environment.;
		%let exiterror= 1;
		%goto exit;
	%end;
	
	**checking if there is missing trait values when input data has means**;
	proc means data=filtermiss nmiss;
		var &TraitVars;
		output out=missdat nmiss= &TraitVars;
	run;
	
	%let miss_flag= 0;
	data _null_;
		set missdat;
		%do t=1 %to &Ntraits; 
			if %scan(&TraitVars,&t) > 0 then do;
				call symputx("tmiss&t", 1);
				call symputx("miss_flag", 1);
			end;
			else do;
				call symputx("tmiss&t", 0);
			end;
		%end;
	run;
	%put tmiss1= &tmiss1;
	
	%if &miss_flag %then %do;
		%let miss_trts= ;
		%let count_mtrts= 0;
		%do t=1 %to &Ntraits; 
			%if &&tmiss&t %then %do;
				%let miss_trts= &miss_trts %scan(&TraitVars,&t);
				%let count_mtrts= %sysevalf(&count_mtrts + 1);
			%end;
		%end;
		
		%let miss_trts= %sysfunc(tranwrd(&miss_trts, %str( ), %str(, )));
		%if &count_mtrts = 1 %then %do;
			%put ERROR: At least one missing value was found in the quantitative trait: &miss_trts.. When the input data set contains means of genotypes, missing values is not admitted. Please, remove all genotypes with missing values from all environments, keeping the same number of genotypes across all environments. You may use the Filter to Include Observations from the General tab.;
		%end;
		%else %do;
			%put ERROR: At least one missing value was found in each of the quantitative traits: &miss_trts.. When the input data set contains means of genotypes, missing values is not admitted. Please, remove all genotypes with missing values from all environments, keeping the same number of genotypes across all environments. You may use the Filter to Include Observations from the General tab. If genotypes with missing values do not coincide across environments, then it is advised to run a trait-by-trait analysis.;
		%end;
		%let exiterror= 1;
		%goto exit;
	%end;
%end;

/*data OutLib.ge_filtermiss; set filtermiss; run;*/

data alldropgeno;
	delete;
run;

%let backupdat=0;
%let dropgenoflag=0;

%do t=1 %to &Ntraits;
	/*data OutLib.ge_orgfiltermiss&t; set filtermiss; run;*/

	** Loop over QTVars **;
	%let QTVar = %scan(&TraitVars,&t);
	
	** Filter out genotypes for which all reps are missing for the current trait in any given environment combination**;
	proc sort data=filtermiss;
		by &GenotypeVar &EnvVar;
	run;

	proc means data=filtermiss nmiss noprint;
		var &QTVar;
		by &GenotypeVar &EnvVar;
		output out=missdat nmiss= _nmiss_;
	run;

	%let ng1droptrait= 0;
	data missdat;
		length _envcomb_ $500;
		set missdat end=_eof_;
		retain _count_;
		if _n_ = 1 then _count_=0;
		_allmissing_= 0;
		if _freq_ = _nmiss_ then do; 
			_allmissing_= 1;
			_count_ = _count_ + 1;
			call symputx("mgen"||trim(left(_count_)), &GenotypeVar);
			%let i=1;
			%do %while(%length(%scan(&EnvVar,&i)));
				%if &i=1 %then %let lk=%str();
				%else %let lk=%str(|);
				_envcomb_= trim(left(_envcomb_))||" &lk "||"%scan(&EnvVar,&i)= "||trim(left(%scan(&EnvVar,&i)));
				%let i=%eval(&i+1);
			%end;	
			call symputx("menvcomb"||trim(left(_count_)), _envcomb_);
		end;
		if _eof_ then call symputx("ng1droptrait", _count_);
		drop _envcomb_ _type_ _freq_ _nmiss_ _count_;
	run;
	%put ng1droptrait= &ng1droptrait;
	
	%if &ng1droptrait %then %do;	
		%do c=1 %to &ng1droptrait;		
			%put WARNING: Genotype (&GenotypeVar= &&mgen&c) in the environment (&&menvcomb&c) has no non-missing values for trait &QTVar., and it will be removed.;
		%end;
		
		data _filtermiss_ g1droptrait(keep= &GenotypeVar &EnvVar);
			merge filtermiss missdat;
			by &GenotypeVar &EnvVar;
			if _allmissing_ eq 0 then output _filtermiss_;
			else output g1droptrait;
			drop _allmissing_;
		run;
		/*data OutLib.ge_g1droptrait&t; set g1droptrait; run;*/
		/*data OutLib.ge_filtermiss; set _filtermiss_; run;*/	

		%if &backupdat = 0 %then %do;
			%let backupdat= 1;
			
			data origfiltermiss;
				set filtermiss;
			run;			
		%end;

		data filtermiss;
			set _filtermiss_;
		run;
		
		proc sort data=g1droptrait nodupey;
			by &GenotypeVar &EnvVar;
		run;	
	%end;
	
	%*goto exit;
	
	**counting number of Env**;
	proc sort data=filtermiss nodupkey out=countenv;
		by &EnvVar;
	run;
	
	%Nobsvars(countenv);
	%let env_n = &nobs;	

	** Remove genotypes that are not present in all environments **;
	proc sort data=filtermiss nodupkey out=genenv;
		by &GenotypeVar &EnvVar;
	run;

	proc freq data=genenv;
		tables &GenotypeVar / out=freqs(keep=&Genotypevar count rename=(count=_ct_));
	run;

	proc sort data=filtermiss;
		by &GenotypeVar;
	run;
	
	data _filtermiss_ g2dropenv;
		merge filtermiss freqs;
		by &Genotypevar;
		*if _ct_ ne &env_n then delete;
		if _ct_ ne &env_n then output g2dropenv;
		else output _filtermiss_;
		drop _ct_;
	run;
	/*data OutLib.ge_g2dropenv&t; set g2dropenv; run;*/
	/*data OutLib.ge_filtermiss; set _filtermiss_; run;*/

	%Nobsvars(g2dropenv);
	%let ng2dropenv= &nobs;
	%if &ng2dropenv %then %do;	
		proc sort data=g2dropenv(keep= &GenotypeVar &EnvVar) nodupkey;
			by &GenotypeVar &EnvVar;
		run;
		
		%if &backupdat = 0 %then %do;
			%let backupdat= 1;
			
			data origfiltermiss;
				set filtermiss;
			run;			
		%end;		
	%end;	

	data filtermiss;
		set _filtermiss_;
	run;	
	
	/*data OutLib.ge_filtermiss&t; set filtermiss; run;*/
	
	**counting number of observations**;
	%Nobsvars(filtermiss);
	%let totalobs = &nobs;	

	%if &totalobs = 0 %then %do;
		%put ERROR: Number of observations in the data set is zero after removing genotypes that are not present in all environments.;
		%put ERROR: There are &env_n environments in the data set. Please, check if each genotype appears in all environments.;
		%let exiterror=1;
		%goto exit;
	%end;

	**re-counting number of Env**;
	proc sort data=filtermiss nodupkey out=countenv;
		by &EnvVar;
	run;

	%Nobsvars(countenv);
	%let env_n = &nobs;	

	/*
	**counting number of observations in the second Env **;
	%if %length(%scan(&EnvVar,2)) %then %do;
	        ** Need this for Parallel Plots **;
		proc sort data=filtermiss nodupkey out=env2;
			by 
			%let i=2; 
			%do %while(%length(%scan(&EnvVar,&i)));
				%scan(&EnvVar,&i)
				%let i=%eval(&i+1);
			%end;
			;
		run;
		%Nobsvars(env2);
		%let env2_n = %eval(&nobs + 1);
	%end;
	%else %do;
		%let env2_n = 1;
	%end;
	%put env2_n= &env2_n;
	*/

	**counting number of Geno**;
	proc sort data=filtermiss nodupkey out=countgen;
		by &GenotypeVar;
	run;

	%Nobsvars(countgen);
	%let gen_n = &nobs;

	**counting number of Reps**;
	%if %upcase(&InMeans) = YES %then %do;
		%let rep_n = &InRepN;
	%end;
	%else %do;
		%if %length(&RepVar) %then %do;
			proc sort data=filtermiss nodupkey out=countrep;
				by &RepVar;
			run;
		
			%Nobsvars(countrep);
			%let rep_n = &nobs;
			
			**re-counting number of Reps when number of obersavations does not match its expected value**;
			%if &totalobs ne %eval(&env_n * &gen_n * &rep_n) %then %let rep_n = %sysevalf(&totalobs / (&env_n * &gen_n));	
		%end;
		%else %do;
			%let rep_n = 1;
		%end;
	%end;	
	
	%if &NTraits=1 %then %do;
		%PrepOutDS(GxEInteraction_OutDataMeans,MeansDS,&OutName._gem);
		%PrepOutDS(GxEInteraction_OutDataPCA,PCADS,&OutName._gep);
		%PrepOutDS(GxEInteraction_OutDataScree,ScreeDS,&OutName._ges);
		%PrepOutDS(GxEInteraction_OutDataLSM,LSMDS,&OutName._gel);
		%PrepOutDS(GxEInteraction_OutDataStab,StabDS,&OutName._gee);
		%PrepOutDS(GxEInteraction_OutDataSS,SSDS,&OutName._gea);
		%PrepOutDS(GxEInteraction_OutDataSSP,SSPDS,&OutName._gen);
		%PrepOutDS(GxEInteraction_OutDataAMMIPred,AMMIPred,&OutName._gei);
		%PrepOutFile(GxEInteractionScript,GxEInteractionScriptName,&ProcessName Script for &OutName..jsl);	
	%end;
	%else %do;
		%let GxEInteraction_OutDataMeans=;
		%let GxEInteraction_OutDataPCA=;
		%let GxEInteraction_OutDataScree=;
		%let GxEInteraction_OutDataLSM=;
		%let GxEInteraction_OutDataStab=;
		%let GxEInteraction_OutDataSS=;
		%let GxEInteraction_OutDataSSP=;
		%let GxEInteraction_OutDataAMMIPred=;
		%let GxEInteractionScript=;
				
		%PrepOutDS(GxEInteraction_OutDataMeans,MeansDS,&OutName._&QTVar._gem);
		%PrepOutDS(GxEInteraction_OutDataPCA,PCADS,&OutName._&QTVar._gep);
		%PrepOutDS(GxEInteraction_OutDataScree,ScreeDS,&OutName._&QTVar._ges);
		%PrepOutDS(GxEInteraction_OutDataLSM,LSMDS,&OutName._&QTVar._gel);
		%PrepOutDS(GxEInteraction_OutDataStab,StabDS,&OutName._&QTVar._gee);
		%PrepOutDS(GxEInteraction_OutDataSS,SSDS,&OutName._&QTVar._gea);
		%PrepOutDS(GxEInteraction_OutDataSSP,SSPDS,&OutName._&QTVar._gen);
		%PrepOutDS(GxEInteraction_OutDataAMMIPred,AMMIPred,&OutName._&QTVar._gei);
		%PrepOutFile(GxEInteractionScript,GxEInteractionScriptName,&ProcessName Script for &OutName._&QTVar..jsl);
		%if &t=1 %then	%do;
			%global jrnfile;
			%PrepOutFile(jrnfile,jrnfilename,GxE Interaction Journal for &OutName..jsl);
		%end;
	%end;
	
	title "Results for Trait &QTVar";
	
  	%let EnvVarInt = %sysfunc(translate(&EnvVar,%str(*),%str( )));
  	%let EnvVarFact = &GenotypeVar|%sysfunc(translate(&EnvVar,%str(|),%str( )));
	
	%if %upcase(&InMeans) = NO %then %do;
		** Get heritability **;
		ods listing close;
		ods output covparms = cp;
		proc mixed data= filtermiss;
		 	class &EnvVar &GenotypeVar &RepVar;
		 	model &QTVar= ;
  			random &EnvVarFact &RepVar(&EnvVarInt);
		run;
	
		%if &syserr>4 %then %goto exit;
		
		data cp ;
			set cp end=_end;;
			retain S2_G S2_P 0;
			if CovParm="&GenotypeVar" then do;
				S2_G = estimate;
				S2_P + estimate;
			end;
			else if index(CovParm,"&GenotypeVar") then S2_P + estimate;
			else if CovParm = "Residual" then S2_P + estimate;
			if _end then do;
				Heritability = S2_G / S2_P;
				output;
				call symputx("Herit&t",heritability);
			end;
			keep s2_G s2_p heritability;
		run;
		
		ods listing;
		proc print data=cp; run;
		ods listing close;	
	%end;
	
	********** Stability betas  *************;
	**grant mean**;
	proc means data=filtermiss noprint mean;
		var &QTVar;
		output out=yppp mean=yppp;
	run;
	/*data OutLib.yppp; set yppp; run;*/

	**env means (e_bar)**;
	proc sort data=filtermiss out=sorte; 
		by &EnvVar;
	run;

	proc means data=sorte mean noprint; 
		by &EnvVar;
		var &QTVar;
		output out=ypjp mean=ypjp;
	run;
	/*data OutLib.ypjp; set ypjp; run;*/

	**env deviations from grant mean**;
	data enveff;
		merge ypjp yppp;
		by _TYPE_;
		ei=ypjp-yppp; /*environmental index: Fernandez, G.C.J. (HORTSCIENCE, VOL. 26(8), AUGUST 1991)*/
		call symputx("yppp",yppp);
	run;
	/*data OutLib.enveff; set enveff; run;*/

	*** Get genotype and env means for Wricke Wi ***;
	proc sort data=filtermiss out=sortg;
		by &GenotypeVar &EnvVar;
	run;

	**geno means (g_bar)**;
	proc means data=sortg mean noprint;
		by &GenotypeVar;
		var &QTVar;
		id &GenotypeVar _gennum_; /*Luciano*/
		output out=yipp mean=yipp;
	run;	
	/*data OutLib.yipp; set yipp; run;*/

	**geno means within each env (ge_bar)**;
	proc means data=sortg mean noprint;
		by &GenotypeVar &EnvVar;
		var &QTVar;
		id &GenotypeVar _gennum_; /*Luciano*/
		output out=yijp mean=yijp;
	run;
	/*data OutLib.yijp; set yijp; run;*/

	data means_yijp_yipp;
		merge yijp yipp;
		by &GenotypeVar;
	run;

	proc sort data=means_yijp_yipp;
		by &EnvVar;
	run;

	data means_yijp_yipp_ypjp;
		merge means_yijp_yipp ypjp;
		by &EnvVar;
		w = (yijp - ypjp - yipp + &yppp)**2;
	run;

	proc sort data=means_yijp_yipp_ypjp;
		by &GenotypeVar;
	run;
	/*data Outlib.means_yijp_yipp_ypjp; set means_yijp_yipp_ypjp; run;*/

	data wricke;
		set means_yijp_yipp_ypjp;
		by &GenotypeVar;
		if first.&GenotypeVar then W_i=w;
		else W_i+w;
		if last.&GenotypeVar then do;
			X_i = yipp;
			output;
		end;
		keep &GenotypeVar W_i X_i;
	run;
	/*data OutLib.wricke; set wricke; run;*/

	%if %upcase(&InMeans) = NO %then %do;
		**Full model: Get Mean Squares**;
		ods listing ;
		/*
		ods listing exclude lsmeans lsmeancl;;
		ods output modelanova=ma overallanova=ova lsmeancl=lsm;;
		proc glm data= filtermiss namelen=200;
			class &EnvVar &GenotypeVar &RepVar;
			%if %length(%scan(&EnvVar,2)) %then %do;
				%let e=1;
				%let EnvLSM=;
				%do %while(%length(%scan(&EnvVar,&e)));
			  		%let EnvLSM = &EnvLSM &GenotypeVar*%scan(&EnvVar,&e);
			  	 	%let e=%eval(&e+1);
				%end;
				model &QTVar=&EnvVarFact &RepVar(&EnvVarInt);
 	      			lsmeans &GenotypeVar &EnvLSM &GenotypeVar*&EnvVarInt / cl alpha=&alpha;;  	
			%end;
			%else %do;
		   		model &QTVar=&EnvVar &RepVar(&EnvVar) &GenotypeVar &GenotypeVar*&EnvVar;
		  		lsmeans &GenotypeVar &GenotypeVar*&EnvVar / cl alpha=&alpha;;
			%end;
		run;
		quit;
		
		data OutLib.ma0; set ma; run;
		data OutLib.ova0; set ova; run;
		data OutLib.lsm; set lsm; run;		
		
		ods listing select all;
		ods listing close;
		data ma;	
			length source $300;
			set ma;
			if hypothesistype=3;
			*sbrlsi: including an empty space after the star in teh GenotypeVar to make sure Genotype source is out of the summation*;
			*if indexw(upcase(source),"%upcase(&GenotypeVar)","*")=0 and indexw(upcase(source),"%upcase(&RepVar)","*()")=0 then msl + ms;
			if indexw(upcase(source),"%upcase(&GenotypeVar)","* ")=0 and indexw(upcase(source),"%upcase(&RepVar)","*()")=0 then msl + ms;
			else if upcase(source)="%qupcase(&RepVar(&EnvVarInt))" then call symputx("MSB",MS);
			drop dependent hypothesistype msl;
			source = "   "||strip(source);
			call symputx("MSL",msl);
			label SS="Sum of Squares";
		run;
		data OutLib.ma1; set ma; run;
		%put MSL= &MSL;		
		*/
		
		/*data OutLib.ge_filtermiss; set filtermiss; run;*/
		
		**start: hpmixed**;
		ods listing exclude test lsmeans;
		ods output tests3=ma overallanova=ova lsmeans=lsm;
		proc hpmixed data= filtermiss namelen=200;
			class &EnvVar &GenotypeVar &RepVar;
			%if %length(%scan(&EnvVar,2)) %then %do;
				%let e=1;
				%let EnvLSM=;
				%do %while(%length(%scan(&EnvVar,&e)));
			  		%let EnvLSM = &EnvLSM &GenotypeVar*%scan(&EnvVar,&e);
			  	 	%let e=%eval(&e+1);
				%end;
				model &QTVar=&EnvVarFact &RepVar(&EnvVarInt);
 	      			lsmeans &GenotypeVar &EnvLSM &GenotypeVar*&EnvVarInt / cl alpha=&alpha;; 
 	      			test &EnvVarFact &RepVar(&EnvVarInt);
			%end;
			%else %do;
		   		model &QTVar=&EnvVar &RepVar(&EnvVar) &GenotypeVar &GenotypeVar*&EnvVar;
		  		lsmeans &GenotypeVar &GenotypeVar*&EnvVar / cl alpha=&alpha;;
		  		test &EnvVar &RepVar(&EnvVar) &GenotypeVar &GenotypeVar*&EnvVar;
			%end;
		run;
		quit;
		
		data _null_;
			set ova;
			where upcase(source)="ERROR";
			call symputx("mse", ms);
			*call symputx("ssedf", df);
		run;
		
		data ma;
			set ma;
			ms= &mse*FValue;
			ss= ms*DenDF;
			rename effect=source;
			label ms="Mean Square";
		run;
		
		data lsm;
			set lsm;
			rename estimate= LSMean Lower= LowerCL Upper=UpperCL;
			label estimate= "&QTVar LSMean";
		run;
		
		/*data OutLib.ge_ma0; set ma; run;*/
		/*data OutLib.ge_ova0; set ova; run;*/
		/*data OutLib.ge_lsm; set lsm; run;*/
		**end: hpmixed**;	
		
		ods listing select all;
		ods listing close;
		data ma;	
			length source $300;
			set ma;
			*if hypothesistype=3;
			*sbrlsi: including an empty space after the star in teh GenotypeVar to make sure Genotype source is out of the summation*;
			*if indexw(upcase(source),"%upcase(&GenotypeVar)","*")=0 and indexw(upcase(source),"%upcase(&RepVar)","*()")=0 then msl + ms;
			if indexw(upcase(source),"%upcase(&GenotypeVar)","* ")=0 and indexw(upcase(source),"%upcase(&RepVar)","*()")=0 then msl + ms;
			else if upcase(source)="%qupcase(&RepVar(&EnvVarInt))" then call symputx("MSB",MS);
			drop dependent hypothesistype msl;
			source = "   "||strip(source);
			call symputx("MSL",msl);
			label SS="Sum of Squares";
		run;
		/*data OutLib.ma1; set ma; run;*/
		%put MSL= &MSL;	
		
		data oa1 oa2 oa3;
			set ova;
			if source="Error" then do;
				call symputx("MSE",MS);
				call symputx("dfMSE",df);
			end;
			drop dependent;
			if source="Model" then output oa1;
			else if source="Error" then output oa2;
			else output oa3;
		run;
		
		/*data OutLib.oa1; set oa1; run;*/
		/*data OutLib.oa2; set oa2; run;*/
		/*data OutLib.oa3; set oa3; run;*/
	%end;
	%else %do;
		%let MSB= &InMSB;
		%let MSL= &InMSL;
		%let MSE= &InMSE;
		%let dfMSE= &InDFMSE;
	%end;
	%put InMSB= &InMSB;
	%put InMSL= &InMSL;
	%put InMSE= &InMSE;
	%put InDFMSE= &InDFMSE;
	
	%if (%upcase(&InMeans) = NO) or	(%upcase(&InMeans) = YES and &InMSL > 0 and &InMSE > 0) %then %do;	

		/*
		proc glm data=yijp;
			class &EnvVar &GenotypeVar;
			*model yijp=&EnvVarInt &RepVar(&EnvVar) &GenotypeVar;
			model yijp=&EnvVarInt &GenotypeVar;
			output out=pred r=zij;
		run;
		*/

		proc hpmixed data=yijp;
			class &EnvVar &GenotypeVar;
			*model yijp=&EnvVarInt &RepVar(&EnvVar) &GenotypeVar;
			model yijp=&EnvVarInt &GenotypeVar;
			output out=pred resid=zij;
		run;
		
		/*data OutLib.pred; set pred; run;*/

		proc sort data=pred;
			by &EnvVar;
		run;

		proc sort data=enveff;
			by &EnvVar;
		run;	

		data Ze;
			merge pred enveff;
			by &EnvVar;
		run;

		proc sort data=Ze out=GEI;
			by &GenotypeVar;
		run;
		/*data OutLib.GEI; set GEI; run;*/

		proc reg data=GEI outest=estimate;
			by &GenotypeVar; 
			model zij=ei / noint;
		run;
		/*data OutLib.estimate; set estimate; run;*/

		data &GxEInteraction_OutDataStab(keep= &GenotypeVar x_i w_i beta_i MSD_i S2d_i alpha_i lambda_i) 
				/*pooleddev(keep=&GenotypeVar MSD_i)*/ 
				pmsd(keep= pooledmsd)
			;
			merge estimate wricke;
			by &GenotypeVar;
			Beta_i= ei;
			MSD_i = _rmse_**2; 
			S2d_i = MSD_i - &MSE; 
			Alpha_i = Beta_i*&MSL/(&MSL-&MSB);
			Lambda_i = (MSD_i / (&gen_n-1)) / (&MSE/(&gen_n*&rep_n));
			ssd = MSD_i * (&env_n-2);
			pooledmsd + ssd;
			if _n_=&gen_n then output pmsd;
			output &GxEInteraction_OutDataStab;
			/*output pooleddev;*/
		run;
		/*data OutLib.pmsd0; set pmsd; run;*/	

		%if (%upcase(&InMeans) = NO) %then %do;
			data pmsd;
				length Source $150;
				merge pmsd oa2;
				Source="Pooled Deviation";
				dfe = df;
				mse = ms;
				df= &gen_n*(&env_n-2);
				ss = pooledmsd;
				ms = pooledmsd/df;
				FValue = ms / mse;
				probf = 1-probf(fvalue,df,dfe);
				drop dfe pooledmsd mse;
			run;
			/*data OutLib.pmsd0; set pmsd; run;*/

			data &GxEInteraction_OutDataSS;
				set oa1 ma pmsd oa2 oa3;
			run;

			/**** Table of means and ANOVA model ***;
			ods output lsmeans=lsm;* diffs=outlib.lsdiffs;
			%if %qupcase(&ModelType) ne %str(ANOVA) %then %do;
				ods output solutionr=solr ;
			%end;

			*ods graphics on;
			proc mixed data=filtermiss; 
				class  &RepVar &GenotypeVar &EnvVar;
				%if %qupcase(&ModelType)=ANOVA %then %do;
					model &QTVar =  &EnvVar &GenotypeVar &RepVar(&EnvVar) &GenotypeVar*&EnvVar / ; 

					lsmeans &GenotypeVar &GenotypeVar*&EnvVar /  cl alpha=&alpha;

				%end;
				%else %do;
					model &QTVar = &GenotypeVar / ;
					random &EnvVar &RepVar(&EnvVar) &GenotypeVar*&EnvVar / solution;
					lsmeans &GenotypeVar / cl alpha=&alpha;
				%end;
			run;

			%if &syserr>4 %then %goto exit;
			*/
		%end;
	%end;
	
	%if (%upcase(&InMeans) = NO) %then %do;	
		%if %qupcase(&ModelType)=%str(ANOVA) %then %do;
			**This IF STATAMENT is always true because ModelType is hidden in the xml file**;
			%let tdata=lsint;
			%let PlotStat = %str(LSMeans);
			%let varvar = lsmean;
			%let varl=%quote(&QTVar LSMEAN);

			data &GxEInteraction_OutDataLSM(drop=&EnvVar) lsint;
				set lsm;
				if 
				%let e=1;
				%do %while(%length(%scan(&EnvVar,&e)));
					%if &e>1 %then and;
					missing(%scan(&EnvVar,&e)) 
					%let e=%eval(&e+1);
				%end;
				then output &GxEInteraction_OutDataLSM;
				else do;
					%let e=1;
					%do %while(%length(%scan(&EnvVar,&e)));
						%if &&evartype&e=N %then %do;
							%unquote(%scan(&EnvVar,&e)n) = strip(%scan(&EnvVar,&e))+0;
							drop %scan(&EnvVar,&e);
							rename %scan(&EnvVar,&e)n = %scan(&EnvVar,&e);
						%end;
						%let e=%eval(&e+1);
					%end;
					output lsint;
					drop effect dependent;
				end;
			run;

		%end;
		%else %do;
			%let tdata=solr;
			%let PlotStat = BLUPs;

			data &GxEInteraction_OutDataLSM;
				set lsm;
			run;		

			%let varvar=estimate;
			%let varl = Estimate;

		%end;

		proc sort data=&tdata out=sortsol;
			by &EnvVar &GenotypeVar;
		run;	
		/*data OutLib.ge_sortsol; set sortsol; run;*/

		proc transpose data=sortsol out=&GxEInteraction_OutDataMeans(drop=_name_ _label_);
			var &varvar;
			by &EnvVar;
			id &GenotypeVar;
		run;
	%end;
	%else %do;
		options label;
		data &GxEInteraction_OutDataLSM;
			set yipp;
			rename yipp= Mean;
			label yipp= &QTVar Mean;
		run;
		
		%let PlotStat = %str(Mean);
		%let varl=%quote(&QTVar Mean);
		
		proc sort data=yijp out=yijp_s;
			by &EnvVar &GenotypeVar;
		run;	
		
		proc transpose data=yijp_s(rename=(yijp=&QTVar)) out=&GxEInteraction_OutDataMeans(drop=_name_ _label_);
			var &QTVar;
			by &EnvVar;
			id &GenotypeVar;
		run;	
	%end;
	
	data genonames;
		set &GxEInteraction_OutDataMeans(obs=1);
		drop &EnvVar;
	run;
	
	%let gnid = %sysfunc(open(genonames));
	%let numgeno = %sysfunc(attrn(&gnid,NVARS));
	%do i=1 %to &numgeno;	
		%let gname&i = %sysfunc(varname(&gnid,&i));
	%end;
	%let rc = %sysfunc(close(&gnid));
	
	%if &syserr> 4 %then %goto exit;

	%let biplot=0;
	%if %qupcase(&Test) ne %str(ANOVA) and %qupcase(&Test) ne %str(LINEAR MIXED MODEL) %then %do;
		**This if statement is always TRUE because Test is either AMMI, GREG, SREG (GGE), SHMM, COMM**;
		%let biplot=1;

		*** AMMI ***;
		**************TO GET RESIDUALS FROM THE ADDITIVE MODEL  *************;

		%let dat= filtermiss;
		%let gen_num= _gennum_;
		
		ods listing close;
	
		%let EnvVarFact = %sysfunc(translate(&EnvVar,%str(|),%str( )));
		%if %quote(&Test)=AMMI %then %let PredVars = &EnvVarFact &GenotypeVar;
		%else %if %index(&Test,SREG) %then %let PredVars = &EnvVarFact;
		%else %if %quote(&Test)=GREG %then %let PredVars = &GenotypeVar;
		%else %let PredVars=; /*SHMM and COMM*/
		
		/*data OutLib.dat01; set &dat; run;*/

		**model without GxE interaction**;
		/*
		proc glm data=&dat;
			class &GenotypeVar &EnvVar %if %upcase(&InMeans)= NO %then &RepVar;;
			model &QTVar = &PredVars %if %upcase(&InMeans)= NO %then &RepVar(&EnvVarInt); / %if %quote(&Test)=COMM %then noint;;
			id &GenotypeVar &EnvVar %if %upcase(&InMeans)= NO %then &RepVar; &gen_num;
			output out=resid1 r=resid %if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then p=predic;;
		run;
		*/

		proc hpmixed data=&dat;
			class &GenotypeVar &EnvVar %if %upcase(&InMeans)= NO %then &RepVar;;
			model &QTVar = &PredVars %if %upcase(&InMeans)= NO %then &RepVar(&EnvVarInt); / %if %quote(&Test)=COMM %then noint;;
			id &GenotypeVar &EnvVar %if %upcase(&InMeans)= NO %then &RepVar; &gen_num;
			output out=resid1 resid=resid %if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then pred=predic;;
		run;
		
		/*data OutLib.resid1; set resid1; run;*/

		proc sort data=resid1;
			by &EnvVar &GenotypeVar;
		run;
		
		%if %upcase(&InMeans)= NO %then %do;
			************* COMPUTE AVERAGE OF RESIDUALS TO OBTAIN THE E MATRIX ******;
			proc means data=resid1 mean noprint;
				by &EnvVar &GenotypeVar;
				var resid %if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then predic;;
				id &gen_num;
				output out=resid2 mean=res %if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then pred;;
			run;

			/*
			proc transpose data=resid2 out=outlib.resid2 prefix =&GenotypeVar._;
				var res;
				*by &env;
				by &EnvVar;
				*id &gencod;
				id &GenotypeVar;
			run;
			*/
		%end;
		%else %do;
			data resid2;
				set resid1;
				rename resid= res predic=pred;
			run;
		%end;
		/*data OutLib.resid2; set resid2; run;*/
		
		%put env_n= &env_n;
		%put gen_n= &gen_n;
		%put rep_n= &rep_n;

		%let pca_n= %sysfunc(min(&env_n,&gen_n));
		proc iml;
			use resid2 var{&EnvVar &gen_num res %if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then &GenotypeVar pred;};
			read all var{res} into r;
			%if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then %do;
				read all var{pred} into p;
			%end;
			
			e=shape(r,&env_n);

			call svd(envvec,lambda,genvec,e);
			lambda_sq=lambda#lambda*&rep_n;

			propeig= lambda_sq/lambda_sq[+];
			propacum=propeig;
			do i=2 to nrow(lambda_sq);
				propacum[i]=propacum[i-1]+propeig[i];
			end;

			print 'Proportions of each component', lambda_sq propeig propacum;

			scree=lambda_sq||propeig||propacum;

			labe='env_1':"env_&env_n";
			labg='gen_1':"gen_&gen_n";

			create vece from envvec (|colname=labe|);
			append from envvec;

			create vecg from genvec (|colname=labg|);
			append from genvec;

			create scree from scree;
			append from scree;

			matcomb = envvec` || genvec` || lambda;
			label1="env1":"env&env_n";
			label2="gen1":"gen&gen_n";
			label3={"lambda"};
			label=label1||label2||label3;
			create matrix from matcomb (|colname=label|);
			append from matcomb;
			
			%if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then %do;
				**Gauch et al. Crop Sci. 51:96113 (2011)**;
				p=shape(p,&env_n);
				
				label_pred= "ammi0":"ammi&pca_n";
				
				ammi_all= colvec(p);
			
				%let k=1;
				%do i=1 %to &pca_n;
					ammi= p + envvec[,1:&k]*diag(lambda[1:&k])*genvec[,1:&k]`;
					ammi_all= ammi_all || colvec(ammi); 
					%let k=%sysevalf(&k + 1);
				%end;
								
				create ammi_pred from ammi_all (|colname=label_pred|);
				append from ammi_all;				
			%end;
			
		quit;
		ods listing;
		
		/*
		data OutLib.scree; set scree; run;
		data OutLib.vece; set vece; run;
		data OutLib.vecg; set vecg; run;
		data OutLib.matrix; set matrix; run;
		*/
		
		%if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then %do;
			/*data OutLib.ammi_pred; set ammi_pred; run;*/
			
			data &GxEInteraction_OutDataAMMIPred;	
				merge resid2(keep= &EnvVar &gen_num &GenotypeVar) ammi_pred;
			run;
		%end;

		data &GxEInteraction_OutDataScree(drop=ss source df ms fvalue probf) pcass(keep=source SS df ms fvalue probf);
			length number 4;
			set scree;
			rename col1=Eigenvalue col2=VarianceProportion col3=CumVarianceProportion;
			Number = _n_;
			Source="      IPCA"||strip(_n_);
			SS=col1;
			df = (&gen_n-1) + (&env_n-1) - (2*Number-1);
			MS = SS/df;
			Fvalue = MS / &MSE;
			ProbF = 1-probf(fvalue,df,&dfMSE);
			if number <=3 then output pcass;
			output &GxEInteraction_OutDataScree;
		run;

		%if %upcase(&InMeans) = NO %then %do;
			/*	
			ods output modelanova=ma ;
			proc glm data=&dat;
			  *class &gencod &env &rep;
			  class &GenotypeVar &EnvVar &rep; 
			  *model &var = &env &gencod &rep(&env) &gencod*&env;
			  model &QTVar = &EnvVar &GenotypeVar &RepVar(&EnvVar) &GenotypeVar*&EnvVar  ;
			  *random &rep(&env);
			  random &RepVar(&EnvVar);
			  %if %upcase(&HetResid)=YES %then %do;
				 *repeated/group=&ENV;
				 repeated/group=&EnvVar:
			  %end;  
			run;
			*/	
						
			data &GxEInteraction_OutDataSSP;
				length Source $150;
				set oa1 ma pcass oa2 oa3;
			run;	
		%end;
		%else %if (%upcase(&InMeans) = YES) %then %do;
			data oa2;
				Source= "MSE";
				df= &dfMSE;
				MS= &MSE;	
			run;
			
			data &GxEInteraction_OutDataSSP;
				length Source $150;
				set pcass oa2;
				
			run;		
		%end;

		**merge env means and SVD env variables**;
		data enva;
			merge ypjp(rename=(ypjp= Mean)) vece;
			PC1=env_1;
			PC2=env_2;
			PC3=env_3;
		run;

		**merge geno means and SVD geno variables**;
		data gena;
			merge yipp(rename=(yipp= Mean)) vecg;
			PC1=gen_1;
			PC2=gen_2;
			PC3=gen_3;
		run;

		data &GxEInteraction_OutDataPCA;
			length Score $11 _Label_ $32;
			set enva(in=_env_) gena(in=_gen_);
			if _env_ then do;
				Score="Environment";
				_Label_=
				%let e=1; 
				%do %while(%length(%scan(&EnvVar,&e)));
					%if &e>1 %then ||"_"||;
					strip(%scan(&EnvVar,&e))
				 	%let e=%eval(&e+1);
				%end;
				;
			end;
			else do;
				Score="Genotype";
				*_Label_=strip(&gencod);
				_Label_=strip(&GenotypeVar);
			end;
			label Mean=;
		run;
	%end; 
	**end if statement for Test**;

	* create JSL file;
	%global _mainjslfile_ FullAPName;
	%let _mainjslfile_ = &GxEInteractionScript;
	%if &NTraits > 1 %then %do;
		%let FullAPName=%str(GxE Interaction for Trait &QTVar);
	%end;
	%else %let FullAPName=%str(GxE Interaction);
	
	%put GenotypeVarIn= &GenotypeVar;
	%let OldGenotypeVar = &GenotypeVar;
	%if (%upcase(&InMeans) = NO) or (%upcase(&InMeans) = YES and &InMSL > 0 and &InMSE > 0) %then %do;
		%CheckLabel(&GxEInteraction_OutDataStab,GenotypeVar);
	%end;
	%put GenotypeVarOut= &GenotypeVar;
	
	**counting number of observations in the second Env **;
	%if %length(%scan(&EnvVar,2)) %then %do;
	        ** Need this for Parallel Plots **;
		proc sort data=&GxEInteraction_OutDataMeans nodupkey out=env2;
			by 
			%let i=2; 
			%do %while(%length(%scan(&EnvVar,&i)));
				%scan(&EnvVar,&i)
				%let i=%eval(&i+1);
			%end;
			;
		run;
		%Nobsvars(env2);
		%let env2_n = %eval(&nobs);
	%end;
	%else %do;
		%let env2_n = 1;
	%end;
	%put env2_n= &env2_n;	
	
	%let PlotEnvVar1 = %scan(&EnvVar,1);
	%CheckLabel(&GxEInteraction_OutDataMeans,PlotEnvVar1);
	%put PlotEnvVar1= &PlotEnvVar1;	
	
	%let nbyenv= 0;
	%let i=2; 
	%let b=1;
	%do %while(%length(%scan(&EnvVar,&i)));
		%let PlotEnvVarTmp = %scan(&EnvVar,&i);
		%CheckLabel(&GxEInteraction_OutDataMeans,PlotEnvVarTmp);
		%let PlotByVar&b= &PlotEnvVarTmp;	
		%let b=%eval(&b+1);
		%let i=%eval(&i+1);
	%end;	
	%let nbyenv= %eval(&b-1);
	
	%put env_n= &env_n;
	* put JSL statements;
	data _null_;
		file "&GxEInteractionScript";
		put '//!';
		put "Names Default to Here(1);";  

		/* Function for Stability plot */
		%if (%upcase(&InMeans) = NO) or (%upcase(&InMeans) = YES and &InMSL > 0 and &InMSE > 0) %then %do;
			put 'Here:lss=Function({},{default local},';
			put " if(isEmpty(Here:datas),";
			put "  Here:datas=open(""&ClientOutpath.&StabDS..sas7bdat"", invisible);";
			put " , //else";
			put "  Current Data Table(Here:datas);";
			put " );";
			put " Column(""&GenotypeVar"") << Set Property( Row Order Levels, 1 );";	

			put " Vlistbox(";
			put "  Graph Builder(";
			put "  Show Control Panel( 0 ),";
			put "  Auto Stretching(0),";
			put "  	Variables(";
			put "  		X( :Name(""&GenotypeVar"")),";
			put "  		Y( :W_i ),";
			put "  		Y( :Beta_i, Position( 1 ) ),";
			put " 		Y( :MSD_i, Position( 1 ) ),";
			put " 		Y( :S2d_i, Position( 1 ) ),";
			put " 		Y( :Alpha_i, Position( 1 ) ),";
			put " 		Y( :Lambda_i, Position( 1 ) )";
			put " 	),";
			put " 	Elements(";
			put " 		Points( X, Y( 1 ), Y( 2 ), Y( 3 ), Y( 4 ), Y( 5 ), Y( 6 ), Legend( 1 ), Jitter( 0 ) ),";
			put " 		Line(";
			put " 			X,Y( 1 ),Y( 2 ),Y( 3 ),Y( 4 ),Y( 5 ),Y( 6 ),Legend( 2 ),Row order( 0 ),";
			put " 			Summary Statistic( ""Mean"" )";
			put " 		)";
			put " 	),";
			put " 	SendToReport(";
			put " 		Dispatch(";
			put " 			{},""Graph Builder"",OutlineBox,";
			put " 			{Set Title( ""Stability Measures"" )}";
			put " 		),";
			put " 		Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put " 		Dispatch( {}, ""Y title"", TextEditBox, {Set Text( """" )} ),	";
			put " 		Dispatch(";
			put " 			{},";
			put " 			""400"",";
			put " 			LegendBox,";
			put " 			{Set Title( ""Measures"" ), Position( {-1, -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 5} )}";
			put " 		),";
			put " 		Dispatch(";
			put " 			{},";
			put " 			""&GenotypeVar"",";
			put " 			ScaleBox,";
			put " 			{Label Row( Label Orientation( ""Vertical"" ) )}";
			put " 		),";			
			put " 	)";
			put " );";
			put "));";
		%end;

		/* Function for Genotype LS means plot */
		put 'Here:lsg=Function({},{default local},';
		put "if(isEmpty(Here:datal),";
		put " Here:datal=open(""&ClientOutpath.&LSMDS..sas7bdat"", invisible);";
		put " Current Data Table(Here:datal);";
		put ", //else";
		put " Current Data Table(Here:datal);";
		put ");";
		put " Column(""&GenotypeVar"") << Set Property( Row Order Levels, 1 );";

		put " Vlistbox(";	
		put "  Graph Builder(";
		put "  	Size( 650, 500 ),";
		put "  	Show Control Panel( 0 ),";
		put "  	Show Legend( 0 ),";		
		put "	Variables(";
		put " 	X( :Name(""&GenotypeVar"") ),";
		%if %upcase(&InMeans) = NO %then %do;
			put " 	Y(:&varl), Y( :LowerCL, Position( 1 )), Y(:UpperCL, Position( 1 )),";
		%end;
		%else %do;
			put " 	Y(:&varl),";		
		%end;
		put "	),";
		
		%if %upcase(&InMeans) = NO %then %do;	
			put " 	Elements(";
			put " 		Line( X, Y( 1 ), Legend( 12 ), Missing Values( ""No Connection"" ) ),";
			put " 		Points( X, Y( 1 ), Legend( 9 ) ),";
			put " 		Bar( X, Y( 2 ), Y( 3 ), Legend( 2 ), Bar Style( ""Interval"" ) ),";
			/*put " 		Points( X, Y( 2 ), Y( 3 ), Legend( 1 ) ),";*/
			put " 	),";
		%end;	
		%else %do;
			put " 	Elements(";
			put " 		Line( X, Y( 1 ), Legend( 12 ), Missing Values( ""No Connection"" ) ),";
			put " 		Points( X, Y( 1 ), Legend( 9 ) )";
			put " 	),";	
		%end;
		
		put " 	SendToReport(";
		%if %upcase(&InMeans) = NO %then %do;
			%let ci = %sysevalf(100-100*&alpha);
			put " 		Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( ""&QTVar LS Means for &GenotypeVar with &ci% Confidence Limits"" )} ),";
			put " 		Dispatch( {}, ""Y  title"", TextEditBox, {Set Text( ""LS Means"" )} ),";
		%end;
		%else %do;
			put " 		Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( ""&QTVar Means for &GenotypeVar"" )} ),";
			put " 		Dispatch( {}, ""Y  title"", TextEditBox, {Set Text( ""Mean"" )} ),";
		%end;
		put " 		Dispatch(";
		put " 			{},";
		put " 			""&GenotypeVar"",";
		put " 			ScaleBox,";
		put " 			{Min( -0.5 ), Max( %eval(&numgeno+1) ), Inc( 1 ), Minor Ticks( 1 ), Show Major Ticks( 0 ),";
		put " 			Show Minor Ticks( 0 ), Rotated Labels( 1 )}";
		put " 		),";
		put "   	Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";		
		put " 		Dispatch( {}, ""Graph Builder"", FrameBox, {Frame Size( 435, 387 ),Marker Size(3)} ),";
		put " 	)";
		put "  );	";
		put " )";
		put "); //end function";

		/* Function for GxE Blups or LS means plot */
		put 'Here:lsge=Function({},{default local},';
		put "if(isEmpty(Here:dataq),";
		put " Here:dataq=open(""&ClientOutpath.&MeansDS..sas7bdat"", invisible);";
		put ", //else";
		put " Current Data Table(Here:dataq);";
		put ");";
		put "// Column(""&GenotypeVar"") << Set Property( Row Order Levels, 1 );";
		put "//evar1 = ""%scan(&EnvVar,1)"";";
		%if %length(%scan(&EnvVar,2)) %then %do;	
			put "//evar2 = ""%scan(&EnvVar,2)"";";
		%end;
		put " hlistbox(vlistbox(";
		put "Here:ppp=Parallel Plot(";
		put "	Scale Uniformly( 1 ),";
		put "	Center at zero( 0 ),";
		put "	Y(";
		%do g=1 %to &numgeno;
			put "		:&&gname&g,";
		%end;
		put "	),";
		%if %length(%scan(&EnvVar,2)) %then %do;
			/*put " by(Column(evar2)),";*/
			put " by(Column(""&PlotByVar1""),";
			%do i=2 %to &nbyenv;
				put " Column(""&&PlotByVar&i""),";
			%end;
			put " ),";
		%end;
		/*
		put "	SendToReport(";
		put "		Dispatch( {}, ""Parallel Plot"", OutlineBox, {Set Title( ""GxE Plot"" )} ),";
		%do j=1 %to &env2_n;
			put "		Dispatch(";
			put "			{},";
			put "			""Parallel Coord"",";
			put "			FrameBox(&j),";
			put "			{Frame Size( 650, 450 ), Row Legend(";
			put "				//Column(evar1),";
			put "				Column(""&PlotEnvVar1""),";
			put "				Color( 1 ),";
			put "				Color Theme( "" ),";
			put "				Marker( 0 ),";
			put "				Marker Theme( "" ),";
			put "				Continuous Scale( 0 ),";
			put "				Reverse Scale( 0 ),";
			put "				Excluded Rows( 0 )";
			put "			)}";
			put "		),";
			put "		Dispatch( {}, """", NomAxisBox, {Rotated Tick Labels( 1 )} ),";
		%end;
		put "	)";
		*/		
		put "),";
		/*
		%if %length(%scan(&EnvVar,2)) %then %do;	
			put "pppa=Parallel Plot(";
			put "	Scale Uniformly( 1 ),";
			put "	Center at zero( 0 ),";
			put "	Y(";
			%do g=1 %to &numgeno;
				put "		:&&gname&g,";
			%end;
			put "	),";
			put "	SendToReport(";
			put "		Dispatch({},""Parallel Plot"",OutlineBox,{Set Title( ""&QTVar &PlotStat for &GenotypeVar x &EnvVar"" )}	),";
			put "		Dispatch(";
			put "			{},";
			put "			""Parallel Coord"",";
			put "			FrameBox,";
			put "			{Frame Size( 435, 387 ) }";
			put "		)";
			put "	)";
			put "),";		
		%end;
		*/
		put "Here:dt = Today();";
		put "Eval(Parse(EvalInsert(""\[rppp^dt^ = Here:ppp << Report;]\"")));";
		put "Here:dt = Today();";
		put "Eval(Parse(EvalInsert(""\[rppp^dt^ = Here:ppp << Report;]\"")));";
		put "Eval(Parse(EvalInsert(""\[";
		%if %length(%scan(&EnvVar,2)) %then %do;

  			put "        for(_b=1, _b<=&env2_n, _b++,";
  			put "            //try() will return false if env2_n=1;";
  			put "            try(";
  			put "            //show(rppp^dt^[_b][OutLineBox(1)]);";
  			put "            gtitle= rppp^dt^[_b][OutLineBox(1)]<< Get Text;";
  			put "            //show(gtitle);";
  			put "            gtitle = Trim(Munger(Trim(gtitle), 1, ""Parallel Plot"", """"));";
  			put "            //show(gtitle);";
			put "            pos= Munger(gtitle, 1, ""Where("");";
      			put "            //show(pos);";
      			put "            if(pos, ";
      			put "            	del= Trim(Substr(gtitle, pos - 6));";
      			put "            	rppp^dt^[_b][OutLineBox(1)] << Dispatch( {}, del, TextBox, {Hide( 1 )} );";       		
      			put "            	gtitle= Trim(Substr(gtitle, 1, pos - 6));";
      			put "            );";
      			put "            //show(gtitle);";
			put "      	rppp^dt^[_b][OutLineBox(1)] << {Set Title(""Parallel Plot for ""||gtitle), Image Export Display( Normal )};";					
  			put "      	rppp^dt^[_b][NomAxisBox(1)] << {Rotated Tick Labels( 1 )};"; 
  			put "      	rppp^dt^[_b][FrameBox(1)] << {Frame Size( 650, 450 )};";
  			put "      	rppp^dt^[_b][FrameBox(1)] << Row Legend(Column(""&PlotEnvVar1""), Color( 1 ),Color Theme( """" ),Marker( 0 ),Marker Theme( """" ),Continuous Scale( 0 ),Reverse Scale( 0 ),Excluded Rows( 0 ));";  					
  			put "        ));";			
		%end;
		%else %do;
			put "	try(";
			put "      	rppp^dt^[OutLineBox(1)] << {Set Title(""GxE Plot""), Image Export Display( Normal )};";
			put "      	rppp^dt^[NomAxisBox(1)] << {Rotated Tick Labels( 1 )};";
			put "      	rppp^dt^[FrameBox(1)] << {Frame Size( 650, 450 )};";
  			put "      	rppp^dt^[FrameBox(1)] << Row Legend(Column(""&PlotEnvVar1""), Color( 1 ),Color Theme( """" ),Marker( 0 ),Marker Theme( """" ),Continuous Scale( 0 ),Reverse Scale( 0 ),Excluded Rows( 0 ));";  			
			put "	);";
		%end;
		put ']\"';				
		put ")));";
		
		put "),";
		put "),";
		put ");";
		
		%if (%upcase(&InMeans) = NO) or (%upcase(&InMeans) = YES and &InMSL > 0 and &InMSE > 0) %then %do;
			/*put "Here:TabButtonList={""Stability"",""G LSMeans"",""GxE &PlotStat""};";*/
			put "Here:TabButtonList={""Stability"",""G &PlotStat"",""GxE &PlotStat""};";
			put "Here:TabOpenList={1,1,1};";
			put "Here:TabCodeList={Here:lss(),Here:lsg(),Here:lsge()};";
			put "Here:TabAfterCodeList={"""","""",""""};";
			put "Here:DataList={{""Here:datas""},{""Here:datal""},{""Here:dataq""}};";
		%end;
		%else %do;
			/*put "Here:TabButtonList={""G LSMeans"",""GxE &PlotStat""};";*/
			put "Here:TabButtonList={""G &PlotStat"",""GxE &PlotStat""};";
			put "Here:TabOpenList={1,1};";
			put "Here:TabCodeList={Here:lsg(),Here:lsge()};";
			put "Here:TabAfterCodeList={"""",""""};";
			put "Here:DataList={{""Here:datal""},{""Here:dataq""}};";
		%end;

		%if &biplot %then %do;
			put "InsertInto(Here:TabButtonList,""2D Biplots"");";
			put "InsertInto(Here:TabOpenList,1);";
			%if &env_n > 2 %then %do;
				put "InsertInto(Here:TabButtonList,""3D Biplot"");";
				put "InsertInto(Here:TabOpenList,1);";
			%end;
			put "InsertInto(Here:TabButtonList,""PC1 x Mean &QtVar"");";
			put "InsertInto(Here:TabOpenList,1);";
			put " QTVar = ""&QTVar"";";
			put " Here:dsv = ""&ClientOutpath.&PCADS..sas7bdat"";";
			put " Here:datav=open(dsv, invisible);";
			put " Here:dse = ""&ClientOutPath.&ScreeDS..sas7bdat"";";
			put " Here:eigdata=open(dse, invisible);";
			%if &env_n > 2 %then %do;
				put " Here:twodtitle = ""2D Biplots for PC1, PC2, PC3"";";
				put " Here:threedtitle = ""3D Biplot of PC1, PC2, PC3"";";
				put " Here:yvars = {:PC1, :PC2, :PC3};";
			%end;
			%else %do;
				put " Here:twodtitle = ""2D Biplot for PC1, PC2"";";
				put " Here:yvars = {:PC1, :PC2};";
			%end;
			put " include(jg:LSpath[2]||""/JSLFiles/GxEPlots.jsl"");";

			put "InsertInto(Here:TabCodeList,expr(Here:PCPlots()));";
			%if &env_n > 2 %then put "InsertInto(Here:TabCodeList,expr(Here:S3D()));";;
			put "InsertInto(Here:TabCodeList,expr(Here:MeanxPC1()));";
			put "InsertInto(Here:TabAfterCodeList,"""");";
			%if &env_n > 2 %then put "InsertInto(Here:TabAfterCodeList,"""");";;
			put "InsertInto(Here:TabAfterCodeList,"""");";

			put "InsertInto(Here:DataList,{{""Here:datav""}});";
			%if &env_n > 2 %then put "InsertInto(Here:DataList,{{""Here:datav""}});";;
			put "InsertInto(Here:DataList,{{""Here:datav""}});";

			put "//newmat = fit_ammi(Column(""Residual"")<<GetAsMatrix,&env_n,&gen_n,3);";

			put "InsertInto(DataList,{{""Here:eigdata""}});";
			put "InsertInto(TabButtonList, ""Scree Plot"");";
			put "InsertInto(TabCodeList, expr(pcScree()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";	

			/* JSL for Principal components on outlib.resid2 data set 	
			Principal Components(
				Y(
					:geno_Florman,
					:geno_Tegua,
					:geno_manf393,
					:geno_mf447,
					:geno_mf478,
					:geno_mf480,
					:geno_mf484,
					:geno_mf485,
					:geno_mf487,
					:geno_mf489
				),
				Estimation Method( "Row-wise" ),
				on Covariances,
				Covariance Matrix( 1 ),
				Eigenvalues( 1 ),
				Biplot( 3 ),
				SendToReport(
					Dispatch( {}, "Summary Plots", OutlineBox, {Close( 1 )} ),
					Dispatch( {"Biplot"}, "PCA BiPlot", FrameBox, {Marker Size( 3 )} ),
				)
			);
			*/
		%end;
	run;
	
	%let GenotypeVar= &OldGenotypeVar;
	
	%if &NTraits > 1 %then %do;
		%let launch=NO;
		%let OpenSASOutputTab = 0;
	%end;		
	%else %do;
		%let launch=YES;
		%let OpenSASOutputTab = 1;
	%end;
	

	%if (%upcase(&InMeans) = NO) or (%upcase(&InMeans) = YES and &InMSL > 0 and &InMSE > 0) %then %do;
		%if &biplot=0 %then %PackageInsertFile(jsl code from &ProcessName, &OutPath, &GxEInteractionScriptName, "&StabDS..sas7bdat,&MeansDS..sas7bdat,&LSMDS..sas7bdat",text/plain, yes, &launch);
		%else %PackageInsertFile(jsl code from &ProcessName, &OutPath, &GxEInteractionScriptName, "&StabDS..sas7bdat,&MeansDS..sas7bdat, &LSMDS..sas7bdat, &PCADS..sas7bdat, &ScreeDS..sas7bdat",text/plain, yes, &launch);
	%end;
	%else %do;
		%if &biplot=0 %then %PackageInsertFile(jsl code from &ProcessName, &OutPath, &GxEInteractionScriptName, "&MeansDS..sas7bdat,&LSMDS..sas7bdat",text/plain, yes, &launch);
		%else %PackageInsertFile(jsl code from &ProcessName, &OutPath, &GxEInteractionScriptName, "&MeansDS..sas7bdat, &LSMDS..sas7bdat, &PCADS..sas7bdat, &ScreeDS..sas7bdat",text/plain, yes, &launch);		
	%end;
		
	%*PackageInsertFile(jsl code from &ProcessName, &OutPath, &GxEInteractionScriptName, &QuantDS..sas7bdat, text/plain, yes);
	
	%if %upcase(&InMeans) = NO %then %do;
		%PackageInsertDataset(ANOVA data from &ProcessName, outlib, &SSDS, 1, _type=other);
	%end;
	%PackageInsertDataset(PCA data from &ProcessName, outlib, &PCADS, 1, _type=other);

	%PackageInsertDataset(ANOVA for &test data from &ProcessName, outlib, &SSPDS, 1, _type=other);
	
	%if %quote(&Test)=AMMI and %upcase(&OutAMMIPred= YES) %then %do;
		%PackageInsertDataset(Predicted values for AMMI models from &ProcessName, outlib, &AMMIPred, 1, _type=other);
	%end;
	
	%if &t=1 %then %do;
		%global jsl_after_tabbedreport;
		%global SummaryScript;
		%global GxEInteraction_OutDataDp;	
		%PrepOutDS(GxEInteraction_OutDataDp,GxEInteraction_OutDataDpName,&OutName._gdp);
	%end; 
	
	%let jsl_after_tabbedreport = ;
	%let jsl_after_tabbedreport = &clientoutpath.&ProcessName Message for &OutName._&QTVar..jsl;
	%let SummaryScript = ;
	%PrepOutFile(SummaryScript,SummaryScriptName,&ProcessName Message for &OutName._&QTVar..jsl);	
	%let GxEInteraction_OutDataDp = ;
	%PrepOutDS(GxEInteraction_OutDataDp,GxEInteraction_OutDataDpName,&OutName._gdp);
	
	%if &ng1droptrait or &ng1dropenv or &ng2dropenv %then %do;
		%let dropgenoflag= 1;
		
		%if &ng1dropenv %then %do;
			data alldropgeno;
				length Trait 
					%if &gvartype= C %then &GenotypeVar; 
					%let i=1;
					%do %while(%length(%scan(&EnvVar,&i))); 
						%if &&evartype&i= C %then %scan(&EnvVar,&i); 
						%let i=%eval(&i+1);
					%end; 
				$50;
				set alldropgeno g1dropenv(in=b);
				if b then Trait="All Traits";
			run;	
		%end;	
		
		%if &ng2dropenv %then %do;
			data alldropgeno;
				length Trait 
					%if &gvartype= C %then &GenotypeVar; 
					%let i=1;
					%do %while(%length(%scan(&EnvVar,&i))); 
						%if &&evartype&i= C %then %scan(&EnvVar,&i); 
						%let i=%eval(&i+1);
					%end; 
				$50;
				set alldropgeno g2dropenv(in=b);
				if b then Trait="&QTVar";
			run;	
		%end;	
		
		%if &ng1droptrait %then %do;
			data alldropgeno;
				length Trait 
					%if &gvartype= C %then &GenotypeVar; 
					%let i=1;
					%do %while(%length(%scan(&EnvVar,&i))); 
						%if &&evartype&i= C %then %scan(&EnvVar,&i); 
						%let i=%eval(&i+1);
					%end; 
				$50;
				set alldropgeno g1droptrait(in=b);
				if b then Trait="&QTVar";
			run;	
		%end;		
	
		* put JSL statements;
		data _null_;
		   file "&SummaryScript";
		   put '//!';
		   put "Names Default to Here(1);";
		   put "wait(0.5);";
		   put "Here:gewarn_&t=New Window(""WARNING message running &ProcessName."",showmenu(0),showtoolbars(0),";
		   put "	Vlistbox(";
		   %if &ng1droptrait %then %do;		   
			   put "		Textbox(""WARNING: Some genotypes were removed from the analysis of trait &QTVar because they have no non-missing values in some environments."", Set Wrap(500), Set Font Size(10), Font Color(5)),";
		   %end;
		   %if &ng1dropenv or &ng2dropenv %then %do;
			   put "		Textbox(""WARNING: Some genotypes were removed from the analysis because they do not present in all environments."", Set Wrap(500), Set Font Size(10), Font Color(5)),";
		   %end;
		   put "		Textbox(""WARNING: A file with the removed genotypes is located in the output folder with name &GxEInteraction_OutDataDpName..sas7bdat."", Set Wrap(500), Set Font Size(10), Font Color(5)),";
		   put "		ButtonBox(""Open File"",";
          	   put "		jg:clientoutpath = ""&clientoutpath"";";
		   put "		//if(!isEmpty(Here:uselabelcb),";
		   put "			Here:uselabels = 1;";
		   put "		//);";
		   put "		if(hostis(windows),";
		   put "			jg:OpenWithSASNames(""&clientoutpath.\&GxEInteraction_OutDataDpName..sas7bdat"", 0, ""Here:gdpdata"");";
		   put "			if(!IsEmpty(Here:DataList), InsertInto(DataList,{{""Here:gdpdata""}}));,";
		   put "			SASDataSetTextOpen(""&clientoutpath.\&GxEInteraction_OutDataDpName..sas7bdat"",jg:jmpLSDir[2]||""ProcessResults/"",1)";
		   put "		)),";		    
		   put "));";
		   put "if(!IsEmpty(Here:TabWinList),";
		   put "    InsertInto(Here:TabWinList,0);";
		   put "    Here:TabWinList[Here:nwcounter]=Here:gewarn_&t;";
		   put "    Here:nwcounter++; ";		
		   put ");";
		run;		
	%end;
	%else %do;
		* put JSL statements;
		data _null_;
		   file "&SummaryScript";
		   put '//!';
		   put "Names Default to Here(1);";
		   put "wait(0.5);";
		   put "Here:gewarn_&t=New Window(""Message running &ProcessName."",showmenu(0),showtoolbars(0),";
		   put "	Vlistbox(";
		   put "		Textbox(""No genotypes were removed from the analysis of trait &QTVar."", Set Wrap(500), Set Font Size(10), Font Color(5)),";
		   put "));";
		   put "if(!IsEmpty(Here:TabWinList),";
		   put "    InsertInto(Here:TabWinList,0);";
		   put "    Here:TabWinList[Here:nwcounter]=Here:gewarn_&t;";
		   put "    Here:nwcounter++; ";		
		   put ");";
		run;		
	
	%end;
		
	%if &t=&Ntraits and &dropgenoflag %then %do;
		proc sort data= alldropgeno; 
			by Trait &GenotypeVar &EnvVar; 
		run;
		
		data OutLib.&GxEInteraction_OutDataDpName; 
			set alldropgeno; 
		run;
			
		%*PackageInsertDataset(Removed genotypes from &ProcessName, outlib, &GxEInteraction_OutDataDpName, 1, _type=other);			
	%end;	
	%*PackageInsertFile(jsl code from &ProcessName, &OutPath, &SummaryScriptName, , text/plain, yes, no);		

	%put backupdat = &backupdat;
	%if &backupdat = 1 %then %do;	
		** Reset input data because filtering is done for each trait**;
		data filtermiss;
			set origfiltermiss;
		run;			
	%end;
	
	%let _processname = GxEInteraction;
	%PackageEnd;
	%let _pkn = 0; /* reset for next trait so data sets aren't accumulated */
			
%end; /* end loop over QT Vars */

%PackageBegin(&ProcessName);	

%if &NTraits > 1 %then %do;
	* construct journal;
	data _null_;
		file "&JRNFile";
		put "//!";

		put "Here:geij = New Window(""Results Journal: GxE Interaction for &OutName"",";
		put "   Outline Box(""GxE Interaction Results"",";
		put "      Vlistbox( ";
		%do t=1 %to &NTraits;
			%let QTVar = %scan(&TraitVars,&t);
			put "      Button Box(""Trait &QTVar"",";
			put "         include(""&ClientOutpath.&ProcessName Script for &OutName._&QTVar..jsl"",<<new context);";
			put "      ),";
			put "      Text Box(""Heritability = %sysfunc(round(&&Herit&t,0.001))""),";
			put "      Text Box("" ""),";			
		%end;
		put "      )";
		put "   )";
		put ");";
		put "jg:SetIcon(Here:geij,""GxEInteraction"");";
	run;
	
	%PackageInsertFile(Journal from GxE Interaction, &OutPath, &JRNFileName, , text/plain, yes, yes);
%end;

%let _mainjslfile_ = ;
%let QTVar= &TraitVars;

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend GxEInteraction;

* run the macro;
%GxEInteraction;