/*
JMP Genomics Illumina SNP Input Engine

Name: IlluminaInputEngine
Author: Wendy Czika, SAS Institute Inc.
Support: Wendy Czika, SAS Institute Inc.
-----------------------------------------------------------
Copyright (C) 2006 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName NewStudyName InPath FileFilter GenInFile SampInFile MapInFile FileType CDelim OutPath ServerOutPath OutData GRows PrefixFDT AlleleType MapData FROutData SortData Recode GCScoreCutoff ProbeTypes Prefix SampLen MacroPath;

*ProcessBody;

%macro IlluminaIE;

%let ProcessName = IlluminaInputEngine;
%global FullAPName;
%let FullAPName=%str(Illumina SNP Input Engine);

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./InputEngineMacros.sas" / nosource;
%include "&MacroPath./Reorder.sas" / nosource;

* create output libaray;
libname OutLib "&OutPath";

%CheckEndSlash(InPath);
%let f = 1;
%let fLen = 0;
%let FileList = %trim(%left(&GenInFile)); 
%*put Infiles=&FileList;
%do %while(%length(&FileList));
	* extract one file name;
	%let flag = 0;
	%let name = ;
	%do %While(^&flag);
		%let e = %index(&FileList,%str(|));
		%if ^&e %then %do;
			%let name = %left(&name &FileList);
			%let flag = 1;
			%let FileList = ;
		%end;
		%else %do;
			%let name = %left(&name %substr(&FileList,1,&e-1));
			%if %index(&name,.) %then %let flag = 1;
			%let FileList = %substr(&FileList,&e+1);
		%end;
	%end;
	%let InFile&f = &name;
	%if %index(&name,.) %then %do;
		%let ext = %scan(&name,-1);
		%let InName&f = %substr(&name,1,%length(&name)-%length(&ext)-1);
	%end;
	%else %do;
		%let InName&f = &name;
	%end;

	%CheckDataName(_name=InName&f);

	%let f = %eval(&f+1);
	
%end;
%let nFiles = %eval(&f-1);	

%let FileList = %trim(%left(&SampInFile)); 
%let f = 1;
%let fLen = 0;
%if %length(&FileList) %then %do;
	%do %while(%length(&FileList));
		* extract one file name;
		%let flag = 0;
		%let name = ;
		%do %While(^&flag);
			%let e = %index(&FileList,%str(|));
			%if ^&e %then %do;
				%let name = %left(&name &FileList);
				%let flag = 1;
				%let FileList = ;
			%end;
			%else %do;
				%let name = %left(&name %substr(&FileList,1,&e-1));
				%if %index(&name,.) %then %let flag = 1;
				%let FileList = %substr(&FileList,&e+1);
			%end;
		%end;
		%let SampInFile&f = &name;
		%if %index(&name,.) %then %do;
			%let ext = %scan(&name,-1);
			%let SampInName&f = %substr(&name,1,%length(&name)-%length(&ext)-1);
		%end;
		%else %do;
			%let SampInName&f = &name;
		%end;
		%let f = %eval(&f+1);

	%end;
	%let NSampFiles = %eval(&f-1);
	/*
	%if &NFiles ne %eval(&f-1) %then %do;
		%put ERROR: There must be the same number of Genotype and Sample files specified.;
		%let exiterror=1;
		%goto exit;
	%end;
	*/
%end;

%CheckDataName(_name=OutData,_default=&InName1);
%if %length(&MapData)=0 %then %let OutData=%substr(&OutData,1,28);
%CheckDataName(_name=MapData,_default=&OutData._map);

%if %Length(&MapInFile)=0 and &NFiles > 1 %then %do;
	%put ERROR: You must specify a Map File when multiple Genotype Files are selected.;
	%let exiterror=1;
	%goto exit;
%end;

%let chrvar=;
%let MapExists=1;

%if %upcase(&CDelim)=TAB or %length(&CDelim)=0 %then %do;
	%let DLT = '090D'x;
	%let FType = TAB;
	%let IDelim=%str('090D'x);
%end;
%else %do;
	%let DLT = %quote(",");
	%let FType = CSV;
	%let IDelim=%str('2C0D'x);	
%end;

%let snpcolvar=;

%if %quote(&FileType)=%str(Full Data Table) %then %do;
	%do f=1 %to &nFiles;
		/*******************************************************
		Genotype file 
		*******************************************************/

		%skip_header_import_2(DataFile=%quote(&InPath.&&InFile&f),DataType=&FType,OutSet=illumina,FirstRow=2,VarNameRow=1,GuessRows=&GRows);

		%if &syserr>4 %then %do;
			%put ERROR: Error reading in genotype data.;
			%goto exit;
		%end;

		%let dsid=%sysfunc(open(illumina,i));
		%let chrnum=%sysfunc(varnum(&dsid,CHR));
		%if chrnum>0 %then %let chrtype=%sysfunc(vartype(&dsid,&chrnum));
		%else %let chrtype=N;
		%let posnum=%sysfunc(varnum(&dsid,POSITION));
		%let rc=%sysfunc(close(&dsid));

		data illumina_ns;
			%if &chrtype=C %then length Chr $2;;
			set illumina;
			by name notsorted;
			if first.name then order+1;
			%if &chrtype=C %then if anydigit(chr,1)=1 and lengthn(chr)=1 then chr="0"||chr;;
		run;

		%if &chrnum=0 or &posnum=0 %then %let SortData = NO;

		%if %length(&MapInFile) %then %do;
			%if &f=1 %then %do;
			
				%skip_header_import_2(DataFile=%quote(&InPath.&MapInFile),DataType=&FType,OutSet=OutLib.&MapData,FirstRow=2,VarNameRow=1,GuessRows=&GRows,UniqueVarNameFlag=1);

				%let dsid=%sysfunc(open(OutLib.&MapData,i));
				%let chrnum=%sysfunc(varnum(&dsid,CHROMOSOME));
				%let posnum=%sysfunc(varnum(&dsid,Position));
				%if &chrnum>0 %then %let chrvar=CHROMOSOME;
				%if &chrnum=0 %then %do;
					%let chrnum=%sysfunc(varnum(&dsid,CHR));
					%if &chrnum>0 %then %let chrvar=CHR;
				%end;
				%if &chrnum>0 %then %let chrtype=%sysfunc(vartype(&dsid,&chrnum));	
				%else %do;
					%let chrtype=N;
					%let SortData=NO;
				%end;
				%let rc=%sysfunc(close(&dsid));

				%if &chrnum>0 and &chrtype=C %then %do;
					data OutLib.&MapData;
						length &chrvar $2;
						set OutLib.&MapData;
						if anydigit(&chrvar,1)=1 and lengthn(&chrvar)=1 then &chrvar="0"||&chrvar;
					run;
				%end;		
			%end;

			%if %upcase(&SortData)=YES %then %do;
				%if &f=1 %then %do;
					proc sort data=OutLib.&MapData;
						by &chrvar position;
					run;
				%end;
				
				proc sort data=illumina_ns;
					by chr position;
				run;			
			%end;

			%if &f=1 %then %do;
				%let dsid=%sysfunc(open(OutLib.&MapData,i));
				%let snpvar=;
				%let snpnum=%sysfunc(varnum(&dsid,SNP_Name));
				%if &snpnum>0 %then %do;
					%let snpvar=SNP_Name;
				%end;
				%else %do;
					%let snpnum=%sysfunc(varnum(&dsid,Name));
					%if &snpnum>0 %then %do;
						%let snpvar=Name;
					%end;
				%end;
				%let rc=%sysfunc(close(&dsid));		

				%if %length(&PrefixFDT) %then %do;
					data OutLib.&MapData;
						length SNP_Name SNP_Column_Name $32;
						set OutLib.&MapData;
						%if &snpvar=SNP_Name %then %do;
							SNP_Column_Name = "&PrefixFDT" || substr(SNP_name,1,32-%length(&PrefixFDT));
						%end;
						%else %if &snpvar=Name %then %do;
							SNP_Column_Name = "&PrefixFDT" || substr(name,1,32-%length(&PrefixFDT));
							SNP_Name = name;
							drop name;
						%end;
					run;
					%let snpcolvar = SNP_Column_Name;
				%end;
			%end;

		%end;	
		%else %if &f=1 %then %do;

			%if %upcase(&SortData)=YES %then %do;
				proc sort data=illumina_ns;
					by chr position;
				run;			
			%end;	

			data OutLib.&MapData;
			 length SNP_Name %if %length(&PrefixFDT) %then SNP_Column_Name; $32;
			 set illumina_ns;
			 if lengthn(name)>0;
			 %if %length(&PrefixFDT) %then %do;
				SNP_Column_Name = "&PrefixFDT" || substr(name,1,32-%length(&PrefixFDT));
				%let snpcolvar = SNP_Column_Name;
			 %end;
			 SNP_Name=name;
			 keep SNP_: chr position;
			run;
		%end;
		%let did = %sysfunc(open(illumina));
		%global nvars;
		%let nvars = %sysfunc(attrn(&did,NVARS));
		%let gvars=;
		%do i=1 %to %eval(&nvars);
			%let vn=%sysfunc(varname(&did,&i));
			%if %index(&vn,%str(GType))>0 %then %let gvars=&gvars &vn;
		%end; 
		%let rc = %sysfunc(close(&did));

		data genodelim;
		 length &gvars $3;
		 format &gvars $3.;
		 set illumina_ns;
		 array g{*} &gvars;
		 do i=1 to dim(g);
			temp=g{i};
			if temp="--" then g{i}="";
			else g{i}=compress(substr(temp,1,1)||"/"||substr(temp,2));
		 end;
		 keep name &gvars;
		run;

		proc transpose data=genodelim out=OutData&f name=Sample_ID %if %length(&PrefixFDT) %then prefix=&PrefixFDT;;
		 id name; 
		 var &gvars;
		run;

		%if &syserr>4 %then %goto exit;

		data OutData&f;
		 set OutData&f;
		 Sample_ID = tranwrd(Sample_ID,"_GType","");
		 label Sample_ID="Sample ID";
		run;

	%end; /* end loop over files */
	
	data OutLib.&OutData;
		set OutData1;
	run;

	%if &NFiles > 1 %then %do;
		%do f=2 %to &NFiles;
			data OutLib.&OutData;
				set OutLib.&OutData OutData&f;
			run;
		%end;
	%end;	
	
	%if %length(&SampInFile) %then %do;
		%do f=1 %to &nSampFiles;
	
			%skip_header_import_2(DataFile=%quote(&InPath.&&SampInFile&f),DataType=&FType,OutSet=sampledata,FirstRow=2,VarNameRow=1,GuessRows=&GRows);

			* extract format, informat;
			proc contents data=sampledata out=format noprint;
			run;
			
			/*data OutLib._1format_&f; set format; run;*/
			
			proc sort data=format;
				by VARNUM;
			run;			

			%let ndvars= 0;
			data format;
				set format end=_eof_;
				_category_= fmtinfo(trim(left(format)), 'cat');
				_isdate_= 0;
				if _n_ = 1 then _ndvar_=0;
				if formatd then _format_ = trim(format)||strip(formatl)||'.'||strip(formatd);
				else if formatl then _format_ = trim(format)||strip(formatl)||'.';
				else _format_ = trim(format)||'.';
					if index(_category_, 'date') then do;
					_isdate_= 1;
					_ndvar_= _ndvar_ + 1;
					if formatd=0 and formatl=0 then _format_= trim(format)||'16.';
					call symputx("dvarname"||trim(left(_ndvar_)), NAME);
					call symputx("dvarform"||trim(left(_ndvar_)), _format_);
					call symputx("dvarlen"||trim(left(_ndvar_)), informl);
				end; 
				call symputx("varname"||trim(left(_n_)),NAME);
				if _eof_ then do;
					call symputx("dvarflag", _isdate_);
					call symputx("ndvars", trim(left(_ndvar_)));			
					call symputx("nvars", trim(left(_n_)));
				end;
				retain _ndvar_;
			run;
			%put dvarflag= &dvarflag;
			%put ndvars= &ndvars;

			%do j=1 %to &ndvars; 
				%put &&dvarname&j &&dvarform&j &&dvarlen&j; 
			%end;

			%if &ndvars %then %do;
				data sampledata;
					retain %do i=1 %to &nvars; &&varname&i %end;; 
					length %do j=1 %to &ndvars; &&dvarname&j $ &&dvarlen&j %end;;
					set sampledata(rename=(%do j=1 %to &ndvars; &&dvarname&j=_tmpvar&j %end;));
					%do j=1 %to &ndvars; 
						&&dvarname&j=put(_tmpvar&j,&&dvarform&j); 
						drop _tmpvar&j;
					%end;
				run;
			%end;

			%let dsid=%sysfunc(open(sampledata,i));
			%let sampidnum=%sysfunc(varnum(&dsid,sample_id));
			%let rc=%sysfunc(close(&dsid));

			data sampdata;
				set %if &f>1 %then sampdata; sampledata(in=new);
				if new then do;
					%if &sampidnum=0 %then %do;
						sample_id=strip(id);
						drop id;
					%end;
				end;
				%if &f=&NSampFiles %then _order=_n_;;
			run;
		%end;
				
		proc sort data=sampdata;
			by Sample_ID;
		run;

		proc sort data=OutLib.&OutData;
			by Sample_ID;
		run;

		data mergesamp;
			merge sampdata OutLib.&OutData;
			by Sample_ID;
		run;

		proc sort data=mergesamp out=OutLib.&OutData(drop=_order);
			by _order;
		run;

	%end;	
	%let Prefix=RS;	
%end;
/*************************************************************/
/************************ Final Report file ******************/
%else %if %quote(&FileType)=%str(Final Report) %then %do;
	%do f=1 %to &nFiles;

		%let FROutDataName = %substr(&OutData,1,29)_fr;
		%let DataFile=%quote(&InPath.&&InFile&f);
		%let DataType=&Ftype;
		%let OutSet=stackdata;
		%let FirstRow=11; 
		%let VarNameRow=10;
		%let GuessRows=&GRows;	
		%let _Varlist=;
		%let _Lenlist=;

		%let VarLen=32767;
		%let VarLabelRow=0;
		%tryagain:

		***** Added optional delimiter statement to handle bogus end-of-line characters;
		%if ^%length(&_VarList) %then %let _VarList = _TmpList1;
		%if ^%length(&_LenList) %then %let _LenList = _TmpList2;

		*Retrieve variable names;

		* adjust naming convention for text file;
		%let workpath=%sysfunc(pathname(work));
		filename tmpfile "&workpath.&_slash_.tmpfile072304.txt";
		data _null_;
			infile "&DataFile" truncover dsd lrecl=&VarLen;
			length x $&VarLen;
			input x 1-&VarLen;
			if _n_=&VarNameRow then do;
				file tmpfile lrecl=&VarLen;
				put x;
			end;
			if _n_ > &VarNameRow then stop;
		run;

		proc import out=tmpe 
			datafile= "&workpath.&_slash_.tmpfile072304.txt"
			dbms=&DataType replace;
			Getnames=No;
			DataRow=1;
			GUESSINGROWS=&GuessRows;
			Delimiter=%quote(&IDelim);
		run;

		%if %sysfunc(exist(tmpe))=0 %then %do;
			%put ERROR: Import not successful.  See SAS log for more details.;
			%goto exit;
		%end;

		%nObsVars(tmpe); 

		%if &Server=Local %then %let MaxNVars=39;
		%else %let MaxNVars=40;

		%if &nvars > &MaxNVars %then %do;
			%put ERROR: This appears to be a Full Data Table.  Please run again selecting Full Data Table as the Type of File.;
			%goto exit;
		%end;

		%let extrarow=0; 
		data _null_;
			set tmpe;
			if var1="[Data]" then call symputx("extrarow",1);
		run;
		%if &extrarow=1 %then %do;
			%let FirstRow=12; 
			%let VarNameRow=11;		
			%goto tryagain;
		%end;

		%if %sysfunc(fexist(tmpfile)) %then %let frc=%sysfunc(fdelete(tmpfile));
		proc transpose data=tmpe out=ttmp(drop=_name_);
			var var:;
		run;
		data ttmp;
			length Col1 $32.;
			set ttmp;
		run;

		%naming(_Dset=ttmp,_AllVars=Col1);

		data _null_;
			set ttmp;
			name='TmpName'||strip(_n_);
			call symput(name,col1);
		run;

		* Read full data;

	  %let workpath=%sysfunc(pathname(work));
	  filename tmpfile "&workpath.&_slash_.tmpfile072204.txt";
	  * output a partial text file contianing data only for determining variable attributes;
	  data _null_;
		 infile "&DataFile" truncover dsd lrecl=&VarLen;
			length x $&VarLen;
			input x 1-&VarLen;
			if _n_ > %eval(&FirstRow+&GuessRows-1) then stop;
			if _n_>=&FirstRow then do;
				file tmpfile lrecl=&VarLen;
				put x;
			end;
	  run;
	  * Import data again for getting variable attributes;
	  %put GuessRows = &GuessRows;

	  proc import out=&OutSet
		datafile= "&workpath.&_slash_.tmpfile072204.txt"
		 dbms=&DataType replace;
			GETNAMES=no; 
			DataRow = 1; 
			GUESSINGROWS=&GuessRows;
			Delimiter=%quote(&IDelim);			
		run; 

		%*if %sysfunc(fexist(tmpfile)) %then %let frc=%sysfunc(fdelete(tmpfile));
		* Read data final time;
		%let &_VarList = ;
		%let &_LenList = Sample_ID $&SampLen SNP_Name $24;
		%let tid = %sysfunc(open(&OutSet));
		%let nv_OutSet = %sysfunc(attrn(&tid,NVARS)); 
		%do i = 1 %to &nv_OutSet;
			%let _name&i = %sysfunc(VARNAME(&tid,&i));
			%let _len&i = %sysfunc(VARLEN(&tid,&i));
			%let _type&i = %sysfunc(VARTYPE(&tid,&i)); 

			%if &&_type&i = N %then %do;
				%if %quote(&&Tmpname&i) ne %str(Sample_ID) and %quote(&&Tmpname&i) ne %str(SNP_Name) %then %let &_LenList = &&&_LenList &&Tmpname&i &&_len&i...;
				%let &_VarList = &&&_VarList &&Tmpname&i ??;
			%end;
			%else %do;
				%if %quote(&&Tmpname&i) ne %str(Sample_ID) and %quote(&&Tmpname&i) ne %str(SNP_Name) %then %let &_LenList = &&&_LenList &&Tmpname&i $&&_len&i...;
				%let &_VarList = &&&_VarList &&Tmpname&i $;
			%end;
		%end;
		%let tid = %sysfunc(close(&tid));
		* apply data step to control variable attributes;

		data &OutSet;
			infile "&datafile" delimiter=&DLT MISSOVER DSD lrecl=&VarLen firstobs=&firstrow;
			length &&&_LenList;
			input &&&_VarList;
		run;	
		/*** end skip_header_import_2 code ***/	

		%if %upcase(&FROutData)=YES %then %do;
			data OutLib.&FROutDataName;
				set %if &f>1 %then OutLib.&FROutDataName; stackdata(in=new);
				if new then Sample_ID = tranwrd(Sample_ID,"_GType","");
				label Sample_ID="Sample ID";
			run;
		%end;

		%let dsid=%sysfunc(open(stackdata,i));
		%let chrnum=%sysfunc(varnum(&dsid,CHR));
	  %if &chrnum>0 %then %let chrtype=%sysfunc(vartype(&dsid,&chrnum));	
	  %else %let chrtype=N;
		%let posnum=%sysfunc(varnum(&dsid,POSITION));
		%let a1num=%sysfunc(varnum(&dsid,Allele1___&AlleleType));
		%let a2num=%sysfunc(varnum(&dsid,Allele2___&AlleleType));
		%let rc=%sysfunc(close(&dsid));
		%if &a1num=0 or &a2num=0 %then %do;
			%put ERROR: Allele1__&AlleleType and/or Allele2__&AlleleType columns not found.  Please select a different option for Alleles to Use for Genotypes.;
			%goto exit;
		%end;

		%let Probes=;
		%if %index(%quote(&ProbeTypes),%str(All))=0 %then %do;
			%if %index(%quote(&ProbeTypes),%str(rs))>0 %then %let Probes=%str("RS");
			%if %index(%quote(&ProbeTypes),%str(SNP))>0 %then %let Probes=&Probes %str( "SN");	
			%if %index(%quote(&ProbeTypes),%str(cnv))>0 %then %let Probes=&Probes %str( "CN");
			%if %index(%quote(&ProbeTypes),%str(mito))>0 %then %let Probes=&Probes %str( "MI");	
			%let Probes=%sysfunc(translate(%cmpres(&Probes),%str(,),%str( )));
		%end;

		%if (%index(%quote(&Probes),%str(,))>0 or %length(&Probes)=0) and %length(&Prefix)>0 %then %let mult=1;
		%else %let mult=0;

		data stackdata;
			%if &chrnum>0 and &chrtype=C %then %do;
				length chr $2;
			%end;
			set stackdata;
			array at_1{1} %unquote(Allele1___&AlleleType);
			array at_2{1} %unquote(Allele2___&AlleleType);
			%if %length(&Probes) %then if substr(upcase(SNP_Name),1,2) in (&Probes);;
			if at_1{1}="-" then at_1{1}="";
			if at_2{1}="-" then at_2{1}="";
			if lengthn(at_1{1}) and lengthn(at_2{1}) then 
				Genotype = compress(substr(at_1{1},1,1) || "/" || substr(at_2{1},1,1));
			else Genotype="";
			
			%if %sysevalf(&GCScoreCutoff > 0) %then %do;
				if GC_Score < &GCScoreCutoff then do;
					at_1{1}="";
					at_2{1}="";
					Genotype="";
				end;
			%end;
			%if &chrnum>0 and &chrtype=C %then %do;
				if anydigit(chr,1)=1 and lengthn(chr)=1 then chr="0"||chr;
		 %end;
		run;
		%Nobsvars(stackdata);
		%if &nobs=0 %then %do;
			%put ERROR: There are no SNPs with the Prefix that you selected.;
			%let exiterror=1;
			%goto exit;
		%end;

		%if &chrnum=0 or &posnum=0 %then %let SortData=NO;

		%if %length(&MapInFile) %then %do;
			%if &f=1 %then %do;
				%skip_header_import_2(DataFile=%quote(&InPath.&MapInFile),DataType=&Ftype,OutSet=OutLib.&MapData,FirstRow=2,VarNameRow=1,GuessRows=&GRows,UniqueVarNameFlag=1);

				%let dsid=%sysfunc(open(OutLib.&MapData,i));
				%let chrnum=%sysfunc(varnum(&dsid,CHROMOSOME));
				%let posnum=%sysfunc(varnum(&dsid,Position));
				%let SNPNamenum=%sysfunc(varnum(&dsid,SNP_Name));
				%let Namenum=%sysfunc(varnum(&dsid,Name));
				%if &chrnum>0 %then %do;
					%let chrtype=%sysfunc(vartype(&dsid,&chrnum));	
					%let chrvar=Chromosome;
				%end;
				%else %do;
					%let chrtype=N;
					%let SortData=NO;
				%end;
				%let rc=%sysfunc(close(&dsid));

				data OutLib.&MapData;
					%if &mult=1 %then %do;
						length SNP_Name SNP_Column_Name $32;
					%end;
					set OutLib.&MapData;
					%if %length(&Probes) %then if substr(upcase(Name),1,2) in (&Probes);;				
					%if &chrnum>0 and &chrtype=C %then %do;
						if anydigit(chromosome,1)=1 and lengthn(chromosome)=1 then chromosome="0"||chromosome;
					%end;
					%if &mult=1 %then %do;
						%if &Namenum>0 %then %do;
							SNP_Column_Name = "&Prefix" || substr(name,1,32-%length(&Prefix));
							SNP_Name = name;
						%end;
						%else %do;
							name =length(SNP_Name) + length("&Prefix");
							if (length(name)>32) then SNP_Column_Name = "&Prefix" || substr(SNP_Name,1,32-%length(&Prefix));
							else SNP_Column_Name = "&Prefix" || SNP_Name;
						%end;
						drop name;
						%let snpcolvar = SNP_Column_Name;
					%end;
					%else %do;
						rename Name=SNP_Name;
					%end;
				run;
			%end;

			%if %upcase(&SortData)=YES %then %do;
				%if &f=1 %then %do;
					proc sort data=OutLib.&MapData;
						by chromosome position;
					run;
				%end;
				proc sort data=stackdata out=stackdata;
					by Sample_ID chr position;
				run;
			%end;
			%else %do;
				proc sort data=stackdata out=stackdata;
					by Sample_ID;
				run;
			%end;		
		%end;
		%else %if &f=1 %then %do;

			%if %upcase(&SortData)=YES %then %do;
				proc sort data=stackdata out=stackdata;
					by Sample_ID chr position;
				run;
			%end;
			%else %do;
				proc sort data=stackdata out=stackdata;
					by Sample_ID;
				run;
			%end;	

			data OutLib.&MapData;
			%if &mult=1 %then %do;
				length SNP_Name SNP_Column_Name $32;
			%end;		
			 set stackdata;
			 by Sample_ID;
			 if first.Sample_ID then group+1;
			 if group=1;
			%if &mult=1 %then %do;
				SNP_Column_Name = "&Prefix" || substr(SNP_name,1,32-%length(&Prefix));
				%let snpcolvar = SNP_Column_Name;
			%end;		 
			 keep SNP_: %if &chrnum>0 %then chr; %if &posnum>0 %then position;;
			run;
		%end;

		proc transpose data=stackdata out=OutData&f(drop=_name_) let %if &mult=1 %then prefix=&Prefix;;
			by Sample_ID;
			var Genotype;
			id SNP_Name;
		run;
		%if &f=1 %then %do;
			data OutLib.&OutData;
				set OutData1;
			run;
		%end;
		%else %do;
			data OutLib.&OutData;
				set OutLib.&OutData OutData&f;
			run;
		%end;		
	%end;

	%if %length(&SampInFile) %then %do;
		%do f=1 %to &NSampFiles;

			%skip_header_import_2(DataFile=%quote(&InPath.&&SampInFile&f),DataType=&FType,OutSet=sampledata,FirstRow=2,VarNameRow=1,GuessRows=&GRows);

			* extract format, informat;
			proc contents data=sampledata out=format noprint;
			run;
			
			/*data OutLib._2format_&f; set format; run;*/
			
			proc sort data=format;
				by VARNUM;
			run;

			%let ndvars= 0;
			data format;
				set format end=_eof_;
				_category_= fmtinfo(trim(left(format)), 'cat');
				_isdate_= 0;
				if _n_ = 1 then _ndvar_=0;
				if formatd then _format_ = trim(format)||strip(formatl)||'.'||strip(formatd);
				else if formatl then _format_ = trim(format)||strip(formatl)||'.';
				else _format_ = trim(format)||'.';
					if index(_category_, 'date') then do;
					_isdate_= 1;
					_ndvar_= _ndvar_ + 1;
					if formatd=0 and formatl=0 then _format_= trim(format)||'16.';
					call symputx("dvarname"||trim(left(_ndvar_)), NAME);
					call symputx("dvarform"||trim(left(_ndvar_)), _format_);
					call symputx("dvarlen"||trim(left(_ndvar_)), informl);
				end; 
				call symputx("varname"||trim(left(_n_)),NAME);
				if _eof_ then do;
					call symputx("dvarflag", _isdate_);
					call symputx("ndvars", trim(left(_ndvar_)));			
					call symputx("nvars", trim(left(_n_)));
				end;
				retain _ndvar_;
			run;
			%put dvarflag= &dvarflag;
			%put ndvars= &ndvars;

			%do j=1 %to &ndvars; 
				%put &&dvarname&j &&dvarform&j &&dvarlen&j; 
			%end;

			%if &ndvars %then %do;
				data sampledata;
					retain %do i=1 %to &nvars; &&varname&i %end;; 
					length %do j=1 %to &ndvars; &&dvarname&j $ &&dvarlen&j %end;;
					set sampledata(rename=(%do j=1 %to &ndvars; &&dvarname&j=_tmpvar&j %end;));
					%do j=1 %to &ndvars; 
						&&dvarname&j=put(_tmpvar&j,&&dvarform&j); 
						drop _tmpvar&j;
					%end;
				run;
			%end;

			%let dsid=%sysfunc(open(sampledata,i));
			%let sampidnum=%sysfunc(varnum(&dsid,sample_id));
			%let rc=%sysfunc(close(&dsid));

			data sampdata;
				set %if &f>1 %then sampdata; sampledata;
				%if &sampidnum=0 %then %do;
					sample_id=strip(id);
					drop id;
				%end;
			run;
		%end;
		
		proc sort data=sampdata;
			by Sample_ID;
		run;
		
		proc sort data=OutLib.&OutData;
			by Sample_ID;
		run;

		data OutLib.&OutData;
			merge sampdata OutLib.&OutData;
			by Sample_ID;
		run;

	%end;	

	%if &syserr>4 %then %do;
		%put ERROR: Unable to create genotype data set from Final Report file.;
		%goto exit;
	%end;

	%if %length(&Prefix)=0 %then %let Prefix=%quote(&ProbeTypes);
%end;
%else %do;
	/**************Custom JMP G report****************************/
	%do f=1 %to &NFiles;
		%if %length(&MapInFile) and &f=1 %then %do;
			%skip_header_import_2(DataFile=%quote(&InPath.&MapInFile),DataType=TAB,OutSet=OutLib.&MapData,FirstRow=2,VarNameRow=1,GuessRows=&GRows,UniqueVarNameFlag=1);

			%let dsid=%sysfunc(open(OutLib.&MapData,i));
			%let chrnum=%sysfunc(varnum(&dsid,CHROMOSOME));
			%let posnum=%sysfunc(varnum(&dsid,Position));
			%if &chrnum>0 %then %do;
				%let chrtype=%sysfunc(vartype(&dsid,&chrnum));	
				%let chrvar=Chromosome;
			%end;
			%else %let chrtype=N;
			%let rc=%sysfunc(close(&dsid));

			data OutLib.&MapData;
				set OutLib.&MapData;
				%if &chrnum>0 and &chrtype=C %then %do;
					if anydigit(chromosome,1)=1 and lengthn(chromosome)=1 then chromosome="0"||chromosome;
				%end;
				%if %upcase(&Recode)=YES %then %do;
					if AA_Freq>=BB_Freq then call symputx("MajA"||strip(Name),"A",'G');
					else call symputx("MajA"||strip(Name),"B",'G');	
				%end;
				rename Name=SNP_Name;
			run;

			%if &syserr>4 %then %do;
				%put ERROR reading in SNP map file.;
				%goto exit;
			%end;

			%if %upcase(&SortData)=YES %then %do;
				proc sort data=OutLib.&MapData;
					by chromosome position;
				run;
			%end;

			%Naming(_Dset=OutLib.&MapData,_AllVars=SNP_Name);
		%end;
		%else %if &f=1 %then %let MapExists=0;

		/***** Read in Genotype data using code from DataImportEngine.sas *****/
		%let CR = '0D0A'x;

		* get loc for 1st data line;
		%let skipbyte_d = ;	
		%GetLineLoc(&InPath.&&InFile&f,2,skipbyte_d);
		%if &skipbyte_d %then %let skipbyte_d = %eval(&skipbyte_d-1);	

		%ExtractColumnHeader_Text(TextFile=&InPath.&&InFile&f,FileFormat=Win,
					Delimiter='090D'x,VarNameRow=1,vName_OutData=_vName2_);
		%if &exiterror %then %goto exit;

		* finding # of columns;
		%let did = %sysfunc(open(_vName2_)); 
		%let nv = %sysfunc(attrn(&did,NOBS));
		%let rc = %sysfunc(close(&did));

		* step 2 create a short file for scan;
		%let MaxLen = 32767;
		%let workpath=%sysfunc(pathname(work));
		filename tmpfile "&workpath.&_slash_.tmpfile071607.txt";
		%*if %sysfunc(fexist(tmpfile)) %then %let frc=%sysfunc(fdelete(tmpfile));

		data _null_;
			infile "&InPath.&&InFile&f" delimiter='090D'x MISSOVER TRUNCOVER DSD lrecl=&MaxLen;
			file tmpfile recfm=n lrecl=&MaxLen;
			length x $&MaxLen..;
			if _n_ = 1 then input #1 @;
			input x 1-&MaxLen; 
			/* v3.1 code
				Note: scan function count two adjcent tabs as one tab
			do i = 1 to &ScanColumns;
				y=scan(x,i,&DLT);
				put y @;
				if i < &ScanColumns then put +(-1) &Dilimiter @;
				else put +(-1) &CR;
			end;
			*/
			c = 1;
			s = 1;
			do i = 1 to &MaxLen;
				if i > length(x) then do;
					y = substr(x,s);
					put y +(-1) &CR;
					i = &MaxLen+20;
				end;
				else do;
					if index(substr(x,i,1),'09'x) then do;
						e = i - 1;
						if e>=s then do;
							y = substr(x,s,e-s+1);
							put y +(-1) @;
						end;
						if c < 2 then do;
							put '09'x @;
						end;
						else do;
							put &CR;
							i = &MaxLen;
						end;
						s = i + 1;
						c = c + 1;
					end;
				end;
			end;
			if _n_ > &GRows then stop;
		run;

		* step 3 call Proc Import for detecting variable attributes;
		PROC IMPORT OUT= WORK._tmp1 
				DATAFILE= "&workpath.&_slash_.tmpfile071607.txt" 
				DBMS=&FType REPLACE;
			GETNAMES=YES; 
			DATAROW=2;
			guessingrows=&GRows;
		RUN;
		%*let frc=%sysfunc(fdelete(tmpfile));

		* extract format, informat;
		proc contents data=_tmp1 out=format noprint;
		run;

		data format;
			set format;
			if formatd then format = trim(format)||strip(formatl)||'.'||strip(formatd);
			else if formatl then format = trim(format)||strip(formatl)||'.';
			else format = trim(format)||'.';
			if informd then informat = trim(informat)||strip(informl)||'.'||strip(informd);
			else if informl then informat = trim(informat)||strip(informl)||'.';
			else informat = trim(informat)||'.';
			flag = varnum;
			keep flag format informat length type;
			rename length=vLength format=vFormat informat=vInformat type=vType;
		run;

		proc sort data=format;
			by flag;
		run;

		data _null_;
			set format end=_e_; 
			if _e_ then do;
				call symput('_L_',vlength);
				call symput('_F_',vformat);
				call symput('_IF_',vinformat);
				call symput('_T_',vtype);
				call symput('_n_',_n_);
			end;
		run; 

		* step 4 check naming convention for all vars;
		%Naming(_Dset=_vName2_,_AllVars=vTmp,_UniqueRow=1); 

		data _vName2_;	
			merge _vName2_ format; 
			by flag;
			if _n_ > &_n_ then do;
				vLength = &_L_;
				vFormat = "&_F_";
				vInformat = "&_IF_";
				vType = &_T_;
			end;
		run;

		* step 5 read full data;
		%let did2 = %sysfunc(open(_vName2_));
		%let vNum = %sysfunc(VARNUM(&did2,vTmp));	
		%let lNum = %sysfunc(VARNUM(&did2,vLabel));
		%let fNum = %sysfunc(VARNUM(&did2,vFormat));
		%let ifNum = %sysfunc(VARNUM(&did2,vInformat));
		data OutData&f;
			infile "&InPath.&&InFile&f" RECFM=N delimiter='090D'x	DSD LRECL=32767; 
			%if %upcase(&Recode)=YES %then %do;
				array _var{%eval(&nv-1)} $3 _TEMPORARY_;
			%end;
			%let rc=%sysfunc(fetchobs(&did2,1));
			%do i = 1 %to &nv;
				%let name = %sysfunc(getvarc(&did2,&vNum)); 

				%if %upcase(&Recode)=YES and &i>1 %then %do;
					length &name 4;
				%end;
				%else %do;
					%let Format = %sysfunc(getvarc(&did2,&fNum)); 
					%let Informat = %sysfunc(getvarc(&did2,&ifNum)); 
					informat &name &Informat; 
					format &name &Format;
				%end;

				%let rc=%sysfunc(fetch(&did2));
			%end;

			label 
				%do i = 1 %to &nv;  
					%if &i = 1 %then %let rc=%sysfunc(fetchobs(&did2,1));
					%let name = %sysfunc(getvarc(&did2,&vNum)); 
					%let label = %sysfunc(getvarc(&did2,&lNum));
					&name = "&label" 
					%let rc=%sysfunc(fetch(&did2));
				%end;
			;
			if _n_ = 1 then input +&skipbyte_d @;

			input 
				%do i = 1 %to &nv;
					%if &i = 1 %then %let rc=%sysfunc(fetchobs(&did2,1));
					%if %upcase(&Recode)=YES and &i>1 %then %do;
						_var{%eval(&i-1)}
					%end;
					%else %do;
						%sysfunc(getvarc(&did2,&vNum))
					%end;
					%let rc=%sysfunc(fetch(&did2));
				%end;
			@;
			input +1 @;
			%if %upcase(&Recode)=YES %then %do;
				%do i = 2 %to &nv;
					%if &i = 2 %then %let rc=%sysfunc(fetchobs(&did2,2));
					%let i2=%eval(&i-1);
					%let currvar=%sysfunc(getvarc(&did2,&vNum));
					%if &&MajA&currvar=A %then %do;
						if _var{&i2}="A/A" then &currvar=0;
						else if _var{&i2}="A/B" then &currvar=1;
						else if _var{&i2}="B/B" then &currvar=2;
						else &currvar=.;
					%end;
					%else %do;
						if _var{&i2}="A/A" then &currvar=2;
						else if _var{&i2}="A/B" then &currvar=1;
						else if _var{&i2}="B/B" then &currvar=0;
						else &currvar=.;
					%end;			
					%let rc=%sysfunc(fetch(&did2));
				%end;
			%end;
			rename Sample=Sample_ID;
		run;

		%let rc = %sysfunc(close(&did2));	
		/***** End code from DataImportEngine.sas *****/

		%if &syserr>4 %then %do;
				%put ERROR: Error reading in genotype data.;
				%goto exit;
		%end;

		%if %length(&MapInFile) %then %do;
			%Reorder(OutData&f,rs:,OutLib.&MapData,SNP_Name,OutData&f,0);		
			%if &exiterror=1 %then %goto exit;			

		%end;
		
		data OutLib.&OutData;
			set %if &f>1 %then OutLib.&OutData; OutData&f;
		run;
	%end;

	%if %length(&SampInFile) %then %do;
	
		%do f=1 % to &NSampFiles;
			%skip_header_import_2(DataFile=%quote(&InPath.&&SampInFile&f),DataType=&FType,OutSet=sampledata,FirstRow=2,VarNameRow=1,GuessRows=&GRows);

			* extract format, informat;
			proc contents data=sampledata out=format noprint;
			run;
			
			/*data OutLib._3format_&f; set format; run;*/
			
			proc sort data=format;
				by VARNUM;
			run;			

			%let ndvars= 0;
			data format;
				set format end=_eof_;
				_category_= fmtinfo(trim(left(format)), 'cat');
				_isdate_= 0;
				if _n_ = 1 then _ndvar_=0;
				if formatd then _format_ = trim(format)||strip(formatl)||'.'||strip(formatd);
				else if formatl then _format_ = trim(format)||strip(formatl)||'.';
				else _format_ = trim(format)||'.';
					if index(_category_, 'date') then do;
					_isdate_= 1;
					_ndvar_= _ndvar_ + 1;
					if formatd=0 and formatl=0 then _format_= trim(format)||'16.';
					call symputx("dvarname"||trim(left(_ndvar_)), NAME);
					call symputx("dvarform"||trim(left(_ndvar_)), _format_);
					call symputx("dvarlen"||trim(left(_ndvar_)), informl);
				end; 
				call symputx("varname"||trim(left(_n_)),NAME);
				if _eof_ then do;
					call symputx("dvarflag", _isdate_);
					call symputx("ndvars", trim(left(_ndvar_)));			
					call symputx("nvars", trim(left(_n_)));
				end;
				retain _ndvar_;
			run;
			%put dvarflag= &dvarflag;
			%put ndvars= &ndvars;

			%do j=1 %to &ndvars; 
				%put &&dvarname&j &&dvarform&j &&dvarlen&j; 
			%end;

			%if &ndvars %then %do;
				data sampledata;
					retain %do i=1 %to &nvars; &&varname&i %end;; 
					length %do j=1 %to &ndvars; &&dvarname&j $ &&dvarlen&j %end;;
					set sampledata(rename=(%do j=1 %to &ndvars; &&dvarname&j=_tmpvar&j %end;));
					%do j=1 %to &ndvars; 
						&&dvarname&j=put(_tmpvar&j,&&dvarform&j); 
						drop _tmpvar&j;
					%end;
				run;
			%end;

			%let dsid=%sysfunc(open(sampledata,i));
			%let sinum=%sysfunc(varnum(&dsid,Sample_ID));
			%if &sinum=0 %then %do;
				%let idnum=%sysfunc(varnum(&dsid,ID));
				%if &idnum=0 %then %do;
					%put ERROR: Sample file must contain a column named Sample_ID or ID.;
					%goto exit;
				%end;
				%else %let sitype=%sysfunc(vartype(&dsid,&idnum));
			%end;
			%else %do;
				%let sitype=%sysfunc(vartype(&dsid,&sinum));
			%end;
			%let rc=%sysfunc(close(&dsid));

			data sampdata;
				set %if &f>1 %then sampdata; sampledata(in=new);
				if new then do;
					%if &sitype=N %then %do;
						%if &sinum>0 %then %do;
							charSI = strip(Sample_ID);
							drop Sample_ID;
							rename charSI = Sample_ID;
						%end;
						%else %do;
							Sample_ID = strip(ID);
						%end;
					%end;
					%else %if &sinum=0 %then %do;
						rename ID=Sample_ID;
					%end;
				end;
				%if &f=&NSampFiles %then _order=_n_;;
			run;
			
			%if &syserr>4 %then %do;
				%put ERROR: Sample file could not be imported properly.  Please be sure you are creating this file	using the JMP G Custom Report.;
				%goto exit;
			%end;
		
		%end;
		
		proc sort data=sampdata;
			by Sample_ID;
		run;

		proc sort data=OutLib.&OutData;
			by Sample_ID;
		run;

		data mergesamp;
			merge sampdata OutLib.&OutData;
			by Sample_ID;
		run;

		proc sort data=mergesamp out=OutLib.&OutData(drop=_order);
			by _order;
		run;

	%end;	
	%let Prefix=RS;

%end;

%PackageInsertDataset(Genotype data from &ProcessName, OutLib, &OutData, no);
%if &MapExists=1 %then %do;
	%PackageInsertDataset(Map data from &ProcessName, OutLib, &MapData, no, _type=anno);

	%if &chrnum>0 %then %do;
		%if %length(&chrvar)=0 %then %let chrvar=CHR;
	%end;
	%else %let chrvar=;

	%if &posnum>0 %then %let posvar=Position;
	%else %let posvar=;

	%let dsid=%sysfunc(open(OutLib.&MapData,i));
	%let snpvar=;
	%let snpnum=%sysfunc(varnum(&dsid,SNP_Name));
	%if &snpnum>0 %then %do;
		%let snpvar=SNP_Name;
	%end;
	%else %do;
		%let snpnum=%sysfunc(varnum(&dsid,Name));
		%if &snpnum>0 %then %do;
			%let snpvar=Name;
		%end;
	%end;
	%let rc=%sysfunc(close(&dsid));
%end;

%if %length(&PrefixFDT) %then %let Prefix=&PrefixFDT;
%else %if %length(&Prefix)=0 %then %let Prefix=RS;

%if %length(&snpcolvar)=0 %then %let snpcolvar=&snpvar;

%if &MapExists=1 %then 
%LaunchWorkflow(GEN,&OutData,_AnnoData=&MapData,_prefix=&Prefix,_MNameVar=&snpcolvar,_ChrVar=&chrvar,_PosVar=&posvar,_LabelVar=&snpvar,jslfname=IlluminaSNPLaunchScript);
%else %LaunchWorkflow(GEN,&OutData,_prefix=&Prefix,jslfname=IlluminaSNPLaunchScript);

%if %upcase(&FROutData)=YES %then %do;
	%PackageInsertDataset(Final report data from &ProcessName, OutLib, &FROutDataName, no);
%end;
%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend IlluminaIE;
%IlluminaIE;
