/*
JMP Genomics Analytical Process

Name: Linkage Groups and Recombination
Author: Kelci Miclaus and Luciano Silva
History: kemicl Initial Coding
	 sbrlsi 11/14/19 LOD Estimation

Description:
		Recombination and Linkage Groups caluclates a matrix of pairwise recombination rates based on an experimental cross 
		design and clusters the matrix to identify linkage groups for linkage mapping and QTL analysis. Segregation ratios 
		are also calculated for each marker.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ListMarkerVars WhereStmt 
AnnoWhereStmt AnnoInData AnnoMName SkeletonVar MarkerFmt OldNumCode GenoDelim GenoMcode 
FrameInData FrameMName SkeletonVarF SkOrderVar
CrossType GenNum RanMaNum RILBiasCorr SegRatios MultipleTestingMethod Alpha MaxR 
GroupingMethod GroupR GroupLOD LGmin 
ClusteringMethod MakeHPlots LGNum AClusteringMethod DefineClust ALGNum RCutoff
SASCLusters LinkageGroups_body LinkageGroups_frame LinkageGroups_contents Plot_OutTree 
KMeans MarkerReductionMethod NumberOfClusters CorrRadius AppendDroppedMarkers ClusterSkeletonMarker
OutPath ServerOutPath OutName MacroPath LinkageGroupsRec_OutData LinkageGroupsLod_OutData
LinkageGroupsSeg_OutData LinkageGroupsAn_Outdata LinkageGroupsScript
;

%let LinkageGroups_body=; 
%let LinkageGroups_frame=;
%let LinkageGroups_contents=;
%let Plot_OutTree=;
%let LinkageGroupsRec_OutData=;
%let LinkageGroupsLod_OutData=;
%let LinkageGroupsSeg_OutData=;
%let LinkageGroupsAn_Outdata=;
%let LinkageGroupsScript=;

/* Set to blank vars needed for GenMacros */
%global FullInPath ByVars AnnoLabel AnnoMergeVars AnnoGroup AnnoLoc AnnoAccession XLVARSTMT;
 %let ByVars = ;
 %let AnnoLabel = &AnnoMName;
 %let AnnoGroup = ;
 %let AnnoLoc = ;
 %let AnnoAccession = ;
 %let FullInPath = &InData;
 %let XLVARSTMT=; 
*ProcessBody;

%macro LinkageGroups;

%include "&MacroPath./GenMacros.sas" / nosource;
%include "&MacroPath./qtlUtilityMacros.sas" / nosource;
%include "&MacroPath./MultipleTestingAdjustment.sas" / nosource;

%GenInit(LinkageGroups,28);
%if &exiterror=1 %then %goto exit;

%let ReorderingVars = ; * set this macro to empty to avoid certain conflict when call %GenSubset;

%global FullAPName;
%let FullAPName=%str(Recombination and Linkage Groups);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if (%length(&SkeletonVar) or %length(&SkeletonVarF)) and %index(%upcase(&GroupingMethod),MINIMUM) = 0 %then %do;
 %put ERROR: The Minimum Recombination Grouping method must be used for linkage grouping when a Framework Linkage Group Variable is specified.;
 %put ERROR: Please go to the Linkage Grouping Method on the Linkage Groups Tab and specify the Minimum Grouping Recombination Threshold for joining new markers to groups.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&SkeletonVar)=0 and %length(&SkeletonVarF)=0 and %upcase(&ClusterSkeletonMarker)= YES %then %do;
	%*let ClusterSkeletonMarker= No;
	%let string1= The option to include framework markers in the K-Means clustering analysis from the Marker Reduction Tab was checked.;
	%let string2= However, no framework marker has been declared in the Framework Linkage Group Variables field of either the Annotation or Framework Map Tabs.;
	%let string3= Please, return to the Marker Reduction Tab and uncheck the option to include framework markers in the K-Means clustering analysis.;
	%put ERROR: &string1 &string2 &string3;
	%let exiterror=1;
	%goto exit;
%end;

/*begin: sbrlsi added SFn and IRILsib, and renamed crosstypes*/
%global Parent Cross; *sbrlsi: will be used in rfest() macro*;
%let OrigAlleleVars = &AlleleVars;
%let AlleleVars = &AlleleVars &ListMarkerVars;
%let Parent = 0;
%if %index(%upcase(&CrossType),BC) %then %do;
	%let cross = 1;
	%if %upcase(&CrossType) = BC1 %then %let Parent = 1;
	%else %let Parent = 2;
%end;
%else %if %upcase(&CrossType) = SFN  %then %let cross = 22;
%else %if %upcase(&CrossType) = RI1 %then %let cross = 3; /*RILself*/
%else %if %upcase(&CrossTYpe) = RI2 %then %let cross = 4; /*RILsib*/
%else %if %upcase(&CrossType) = RI0 %then %let cross = 5; /*DH*/
%else %if %upcase(&CrossType) = RFN %then %let cross = 6; /* goes with SFn, also known as AIL*/
%else %if %upcase(&CrossType) = IRILSELF %then %let cross = 7; /* goes with RILSELF */
%else %if %upcase(&CrossType) = IRILSIB %then %let cross = 8; /* goes with RILSIB */

%let OrigGenNum= &GenNum;
%if (%length(&GenNum)=0 or &GenNum < 2) and (%upcase(&CrossType) = SFN or %upcase(&CrossType) = RFN) %then %do;
	%let string1= Genotyping Generation (n) must be specified when crosstype is &CrossType.;
	%let string2= Please go to the Genotyping Generation (n) on the Recombination Tab and specify a value for this parameter.;
	%put ERROR: &string1.. &string2;
	%let exiterror=1;
	%goto exit;
%end;
%else %if ^%length(&GenNum) %then 
	%let GenNum = 2;

%let OrigRanMaNum= &RanMaNum;
%if (%length(&RanMaNum)=0 or &RanMaNum < 2) and (%upcase(&CrossType) = IRILSELF or %upcase(&CrossType) = IRILSIB) %then %do;
	%let string1= Random Mating Generation (t) Prior to Inbreeding must be specified when crosstype is &CrossType.;
	%let string2= Please go to the Random Mating Generation (t) Prior to Inbreeding on the Recombination Tab and specify a value for this parameter.;
	%put ERROR: &string1.. &string2;
	%let exiterror=1;
	%goto exit;
%end;
%else %if ^%length(&RanMaNum) %then 
	%let RanMaNum = 2;
	
/*
For both SFn and RFn, RanMaNum is empty and GenNum is NOT empty.
For both IRILself and IRILsib, RanMaNum is NOT empty and GenNum is empty.
In the equations, RanMaNum= GenNum, therefore we optioned to use GenNum in the
entire code.
*/
%if (%upcase(&CrossType) = IRILSELF or %upcase(&CrossType) = IRILSIB) 
	%then %let GenNum = &RanMaNum;

*to run the analysis of f2 in the old code written solely for it;
%if &cross = 22 and &GenNum = 2 %then %let cross = 2; 
%put cross= &cross;
/*end: sbrlsi*/

/* if AIL and no spec'd Seg ratios, create an indicator to not perform test right now */ 
%global no_st; *sbrlsi: will be used in rfest() macro*;
%let no_st = 0;
%if %upcase(&CrossType) = RFN and ^%length(&SegRatios) 
	%then %let no_st = 1;

/* Segregation Ratios Input */
%global AA_ef AB_ef BB_ef; *sbrlsi: will be used in rfest() macro*;
%if %length(&SegRatios) %then %do;
	%if ^%length(%scan(&SegRatios,2,' ')) %then %let SegRatios = &SegRatios 0;
	%if ^%length(%scan(&SegRatios,3,' ')) %then %let SegRatios = &SegRatios 0;
	%let AA_ef = %scan(&SegRatios,1,' ');
	%let AB_ef = %scan(&SegRatios,2,' ');
	%let BB_ef = %scan(&SegRatios,3,' ');
%end;
%else %do;
	%if &cross = 2 %then %do;
		%let AA_ef = 0.25;
		%let AB_ef = 0.50;
		%let BB_ef = 0.25;
	%end;
	%else %if &cross = 22 %then %do;	
		%let AA_ef = %sysevalf(0.5 - 1/(2**&GenNum));
		%let AB_ef = %sysevalf(1/(2**(&GenNum - 1)));
		%let BB_ef = &AA_ef;
	%end;	
	%else %do;
		%let AA_ef = 0.50;
		%let AB_ef = 0.50;
		%let BB_ef = 0.50;
	%end;
%end;

%put Seg Ratios  = &AA_ef &AB_ef &BB_ef;
%let Cutoff = %sysevalf(-%sysfunc(log10(&Alpha)));
%put Negative Log10 Cutoff for segragation tests = &Cutoff;

%let OrigRILBiasCorr= &RILBiasCorr;
%if %Upcase(&RILBiasCorr) = YES %then %let RILBiasCorr =1;
%else %let RILBiasCorr = 0;

%put Cross type = &CrossType, Cross Number = &cross;

/* data with recombination matrix in it */
%PrepOutDS(LinkageGroupsRec_OutData, LinkageGroupsRec_OutDataName, &OutName,_lgr);
/*data with LOD matrix in it */
%PrepOutDS(LinkageGroupsLod_OutData, LinkageGroupsLod_OutDataName, &OutName,_lod);

%PrepOutFile(LinkageGroupsScript,LinkageGroupsScriptName,&ProcessName Script for &OutName..jsl);
 
 %if %index(%upcase(&GroupingMethod),AUTOMATED) %then %let SASClusters = Yes;
 %else %let SASClusters = No;
 
 *make name for Cluster Plot and Check name for Cluster OutData;
 %if  %UPCASE(&SASClusters)=YES %then %do;
 	%let Plot_OutTree=&LinkageGroupsRec_OutData;
 	%CheckDataName(_name=Plot_OutTree,_default=&OrigInData);
 %end;
 
* AnnoWhereStmt statement applied in this macro *;
%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

%if %length(&FrameInData) %then %do;
	%PathName(&FrameInData);
	%if &exiterror %then %do;
		%put ERROR: The Framework Data Set specification is invalid.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let FrameData = &TmpName;
	libname FrameLib "&TmpPath";
	
	%if %length(&SkOrderVar) %then %do;
		data tmp;
			set framelib.&FrameData;
			where 0;
			keep &SkOrderVar;
		run;
		
		%let dsid = %sysfunc(open(tmp));
		%let SkType = %sysfunc(vartype(&dsid, 1));
		%let rc = %sysfunc(close(&dsid));
		%if &SkType = C %then %do;
			%put ERROR: The Framework Order Variable must be numeric to ensure that it is used correctly in the Linkage Map Order follow-up process.;
			%let exiterror = 1;
			%goto exit;
		%end;
	%end;
%end;

%if %length(&WhereStmt) %then %do;
 data countobs; 
 	set &ds;
  where &WhereStmt;
 run;
 %Nobsvars(countobs);
 %let ds=countobs;
%end;
%else %Nobsvars(&ds);

%if &nobs=0 %then %do;
	%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
	%goto exit;
%end;

/* gendat data set will be used in iml, only genotypes. 
nongendat will be merged with representative markers from K-Means clusterin*/
data gendat(keep= &AlleleVars) nongendat(drop= &AlleleVars); 
	set &ds;  
run;

data mnames; 
	set gendat;
	where 0;
run;

%*if %length(&ListMarkerVars) %then %do;
	/*get first and last marker name and create new liststyle that will not */
	%let tid = %sysfunc(open(mnames));
	%let nobs = %sysfunc(attrn(&tid,NOBS));
	%let nvars = %sysfunc(attrn(&tid,NVARS));
	%put nObs =&nObs nVars=&nVars;
	%let fmvar = %sysfunc(varname(&tid,1));
	%let lmvar = %sysfunc(varname(&tid,&nvars));
	
	%*let ListMarkerVars = &fmvar.--&lmvar;
	
	/*sbrlsi: fixing issue of duplicated markers listed in the AlleleVars and ListMarkerVars*/
	*S1098339: The text expression length (65539) exceeds maximum length (655534).*;
	%*let AlleleVars = &OrigAlleleVars &ListMarkerVars;
	%let nvarblock= 0;
	%let AlleleVars= ;
	%do k=1 %to &nvars;
		%if %length(&AlleleVars) le 32700 %then %do;
			%let AlleleVars = &AlleleVars %sysfunc(varname(&tid,&k));
		%end;
		%else %do;
			%let nvarblock= %sysevalf(&nvarblock + 1);
			%let AlleleVars&nvarblock = &AlleleVars %sysfunc(varname(&tid,&k));
			%let AlleleVars = ;
		%end;
	%end;
	%if &nvarblock = 0 %then %do;
		%let nvarblock= %sysevalf(&nvarblock + 1);
		%let AlleleVars&nvarblock = &AlleleVars;
		%let AlleleVars = ;
	%end;
	%else %if &nvarblock > 0 and %length(&AlleleVars) > 0 %then %do;
		%let nvarblock= %sysevalf(&nvarblock + 1);
		%let AlleleVars&nvarblock = &AlleleVars;
		%let AlleleVars = ;
	%end;
	
	%let AlleleVars = &fmvar.--&lmvar; *needed in the OldGenoCheck macro*;
	%let tid = %sysfunc(close(&tid));
%*end;
%*else %do;
	%*nobsvars(mnames);
%*end;

%if %length(&ListMarkerVars) %then %do;
	%let ListMarkerVars = &fmvar.--&lmvar;
%end;

%if &nvars<2 %then %do;
	%if &nvars = 0 %then %do;
		%put ERROR: The data set contains no markers to be analyzed.  At least two markers are needed to calculate linkage groups.;
	%end;
	%else %do;
		%put ERROR: The data set only contains one marker to be analyzed.  At least two markers are needed to calculate linkage groups.;
	%end;
	%goto exit;
%end;

/* marker recoding macro to convert to numeric for IML code */
%RecodeGenos;
/* Add a check for numeric format for negative values and throw error if found */
%OldGenoCheck;
%if &exiterror %then %goto exit;

%if &syserr>4 %then %do;
	%put ERROR: The genotype data could not be converted to numeric format for recombination computation.;
	%put Please check the Marker Format coding of your data for possible errors.;
 	%let exiterror=1;
	 %goto exit;
%end;

/* add here and make marker list data set.  if no annotation is used, and a framework map is used build skdata from markerlist and framework*/
%let _tid_ = %sysfunc(open(mnames));
%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));

data markerlist;
	length _MarkerName_ $100; 
	%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
		%let _nm_ = %sysfunc(floor(%eval(&_nVars_/2)));
		%put There are &_nm_ Markers;
		%if %length(&AnnoInData) %then %do;
			%do i = 1 %to &_nm_;
				_MarkerName_ = "&&AMName&i";
				output;
			%end;			
		%end;
		%else %do;
			%do i = 1 %to &_nm_;
				_MarkerName_ = "Marker_&i";
				output;
			%end;
		%end;
	%end;
	%else %do;
		%do i = 1 %to &_nVars_;
			_MarkerName_ = "%sysfunc(varname(&_tid_,&i))";
			output;
		%end;
	%end;
run;

/*S1051771. sbrlsi: When markers are in a format other than numeric, %recodegenos will return 
marker names as _geno_1 _geno_2 and so on. Therefore, I need to rename these markers to 
their orginal names. It is important to notice that Linkage Groups AP does not accept
allele marker format, and for that reason I can use the code to rename markers as follows:*/
%if %index(%Upcase(&MarkerFmt),NUMERIC) = 0  %then %do;
	proc datasets library=work nolist;
		modify gendat;
		rename 
		%do i=1 %to &_nVars_;
			_geno_&i = %sysfunc(varname(&_tid_,&i))
		%end;
		;
	quit;
%end;

%if %index(%upcase(&KMeans),YES) %then %do;
	%let vnamfirst = %sysfunc(varname(&_tid_,1));
	%let vnamlast = %sysfunc(varname(&_tid_,&_nVars_));
	%let ListMarkers= &vnamfirst.--&vnamlast;
	%put ListMarkers= &ListMarkers;
%end;
%let _rc_ = %sysfunc(close(&_tid_)); *close opened file*;

/*data outlib.gendat; set gendat;run;*/

*kemicl added to make data set for anno to avoid errors with allele format *;
%let Annodat = ;
%if %length(&AnnoInData) %then %do;
	%let Annodat = AnnoLib.&AnnoData;
	%if %length(&AnnoWhereStmt) %then %do;
		data adat; 
			set AnnoLib.&AnnoData;
			where &AnnoWhereStmt;
		run;
		%let Annodat = adat;
	%end;
%end;

%if %length(&FrameInData) %then %do;
	%if ^%length(&AnnoInData) %then %do;
		%let Annodat = markerlist;
		%let AnnoMName = _MarkerName_;
	%end;
	data adat; set &Annodat;
		ord = _n_;
		drop &SkeletonVar &SkeletonVarF &SkOrderVar;
	run;
		
	**S1115689: sbrlsi added monotonic() to the select statement to avoid problems with duplicated rows in the FrameData*;
	proc sql UNDO_POLICY=NONE; 
		create table adat(drop = ord _mono_) as 
			select *, monotonic() as _mono_ 
			from adat as d1 left join FrameLib.&FrameData(keep = &SkeletonVarF &SkOrderVar &FrameMName) as d2
			on d1.&AnnoMName = d2.&FrameMName
			group by d1.&AnnoMName
			having max(_mono_)= _mono_
			order by ord;
	quit;
	%let Annodat = adat;
	%let SkeletonVar = &SkeletonVarF;
%end;

/*data outlib.Annodat; set &Annodat; run;*/
%*goto exit;

/*make skeleton map for input to constrained MST for grouping */
%if %length(&SkeletonVar) %then %do;
	/* get skeleton markers and groups out and create a 1 to s valued _skeleton_ variable*/
	data grouped ungrouped; 
		set &Annodat;
		if not missing(&SkeletonVar) then output grouped;
		else output ungrouped;
		keep &AnnoMName &SkeletonVar;
	run;

	proc sort data=grouped out=grpnum nodupkey;
		by &SkeletonVar;
	run;
	%nobsvars(grpnum);
	%let NSkeletonGrps = &nobs;
	%if &nobs = 0 %then %do;
		%put ERROR:  No markers have framework linkage groups based on the values of the variables specified from the Framework Map data set.;
		%put ERROR:  Please check that the Framework Marker variable values matches with marker column names in the input data set. ;
		%goto exit;
	%end;
	
	%nobsvars(ungrouped);
	%if &nobs = 0 %then %do;
		%put ERROR:  All Markers already belong to linkage groups as specified by the Framework Linkage Group Variable. No further grouping can be done.;
		%goto exit;
	%end;

	data grpnum;
		set grpnum;
		_Skeleton_ = _n_;
	run;

	/* create separate component_id values from s+1 for ungrouped markers */
	data ungrouped;
		set ungrouped;
		_Skeleton_ = _n_ + &NSkeletonGrps;
	run;

	data skdata;
		if 0 then set grpnum;
		if _N_ = 1 then do;
			declare hash h(dataset:"grpnum");
			h.defineKey("&SkeletonVar");
			h.defineData("&SkeletonVar","_Skeleton_");
			h.defineDone();
		end;
		set &Annodat;
		if h.find() = 0;
		keep &AnnoMName &SkeletonVar _Skeleton_;
	run;	

	data skdata; 
		set skdata ungrouped; 
		rename &AnnoMName = _MarkerName_;
	run;

	/*data outlib.skdata; set skdata; run;*/
%end;

/*data outlib.gendat; set gendat; run;*/
/*data outlib.annodat; set &annodat; run;*/

/*sbrlsi: included code to do marker reduction via k-means clustering*/
%if %index(%upcase(&KMeans),YES) %then %do;

	%if %length(&AnnoInData) = 0 %then %do;
		%let Annodat = markerlist;
		%let AnnoMName = _MarkerName_;
	%end;

	%put Annodat= &Annodat;
	data AnnoTmp;
		set &Annodat;
		*_sindex_=_n_;
	run;
	/*data OutLib.AnnoTmp; set AnnoTmp; run;*/
	/*data OutLib.markerlist; set markerList; run;*/
	
	proc sort data=AnnoTmp;
		by &AnnoMName;
		*by _MarkerName_;
	run;
	
	/*data outlib.AnnoTmp;set AnnoTmp;run;*/
	/*
	%let vnamfirst = %sysfunc(varname(&_tid_,1));
	%let vnamlast = %sysfunc(varname(&_tid_,&_nVars_));
	%let ListMarkers= &vnamfirst.--&vnamlast;
	%put ListMarkers= &ListMarkers;
	*/
	
	/*S1051771. When markers are in a format other than numeric, %recodegenos will return 
	marker names as _geno_1 _geno_2 and so on. Therefore, I need to rename these markers to 
	their orginal names. It is important to notice that Linkage Groups AP does not accept
	allele marker format, and for that reason I can use the code to rename markers as follows:*/
	/*
	%if %index(%Upcase(&MarkerFmt),NUMERIC) = 0  %then %do;
		proc datasets library=work nolist;
			modify gendat;
			rename 
			%do i=1 %to &nvars;
				_geno_&i = %sysfunc(varname(&_tid_,&i))
			%end;
			;
		quit;
	%end;
	%let _rc_ = %sysfunc(close(&_tid_)); *close opened file*;
	*/
	
	/*gendat is a data set with input genoytpes coded*/
	/*transpose input data set to conform with KMeansClustering format*/
	proc transpose data=gendat out=gendat_trans prefix= Ind;
		var &ListMarkers;
	run;
	%let fclusdat = gendat_trans;
	data &fclusdat;
		set &fclusdat;
		_ordg_=_n_;
	run;
			
	proc sort data=&fclusdat;
		by _name_;
	run;

	*Merge fclusdat with Annodat if SkeletonVar is present in either Annotation or Framework Map data sets*;
	data &fclusdat;
		merge &fclusdat(in=a) AnnoTmp(rename=(&AnnoMName=_name_) keep=&AnnoMName %if %length(&SkeletonVar) %then &SkeletonVar; %if %length(&SkOrderVar) %then &SkOrderVar;);
		*merge &fclusdat(in=a) AnnoTmp(rename=(_MarkerName_=_name_) keep=_MarkerName_ %if %length(&SkeletonVar) %then &SkeletonVar; %if %length(&SkOrderVar) %then &SkOrderVar;);
		by _name_;
		if a;
	run;
	
	proc sort data=&fclusdat out=orgfclusdat;
		by _ordg_;
	run;
	
	%if %length(&SkeletonVar) %then %do;
		%if %upcase(&ClusterSkeletonMarker)= YES %then %do;
			*needed to be sorted by SkeletonVar for the by statement in proc fastclus*;
			proc sort data=&fclusdat;
				by &SkeletonVar; 
			run;
		%end;
		%else %do;
			*removing framework markers*;
			data &fclusdat;
				set &fclusdat;
				where missing(&SkeletonVar);
			run;		
		%end;
	%end;
		
	/*create variables to be used by KMeansClustering in what follows*/
	%NObsVars(gendat);
	%let nind= &nobs;
	%global Variables ListVars;
	%let ListVars= Ind1--Ind&nind;
	%put ListVars= &ListVars;
	
	/*columns become rows after transpositon. Centering and/or scaling markers. It is a good 
	practice to center and scale rows to avoid differences in clustering outcomes. If, for 
	instance, markers in a backcross data were coded as 1 and 2, or 0 and 1, these would
	produce different results if they were not centered and scaled.*/
	%let CenterRows = Yes; 
	%let ScaleRows = Yes; 
	
	* center and/or scale rows;
	%if &CenterRows=Yes or &ScaleRows=Yes %then %do;
		data ind;
			set &fclusdat;
			
			%if &CenterRows=Yes %then %do;
				_rowmean_ = mean(of &ListVars);
			%end;
			%else %do;
				_rowmean_ = 0;
			%end;
	
			%if &ScaleRows=Yes %then %do;
				_rowstd_ = std(of &ListVars);
			%end;
			%else %do;
				_rowstd_ = 1;
			%end;
			
			array v{*} &ListVars;
			do i=1 to dim(v);
				if _rowstd_ > 0 then v[i] = (v[i] - _rowmean_)/_rowstd_;
				else v[i] = .;
			end;
			drop i;
		run;	
		%let fclusdat = ind;		
	%end;	
	
	%global LinkageGroupsKMD_OutData LinkageGroupsKMC_OutData;
	%PrepOutDS(LinkageGroupsKMD_OutData, LinkageGroupsKMD_OutDataName, &OutName,_kmd);
	%PrepOutDS(LinkageGroupsKMC_OutData, LinkageGroupsKMC_OutDataName, &OutName,_kmc);
	
	%let FullInPath = &ClientOutPath.&LinkageGroupsKMC_OutDataName..sas7bdat;
		
	* determine radius as a function of a correlation;
	%if %index(&MarkerReductionMethod,Radius) %then %do;
		data std;
			set &fclusdat;
			_rowuss_ = uss(of &ListVars);
			keep _rowuss_;
		run;
		proc means data=std noprint;
			var _rowuss_;
			output out=stats(drop=_TYPE_);
		run;
		data stats;
			set stats;
			if _STAT_ in ("MEAN");
			call symputx('mean_uss',_rowuss_);
		run;
		
		data _null_;
			call symputx('Radius',sqrt(2 * &mean_uss * (1 - &CorrRadius)));
		run;
		
		%put mean_uss = &mean_uss;
	%end;
	
	/*data outlib.fclusdat; set &fclusdat; run;*/

	ods exclude all;
	ods noresults;
	proc fastclus data=&fclusdat out=kmd(rename=(cluster=_KMCluster_ distance=_KMClusterDistance_)) outseed=kmc(rename=(cluster=_KMCluster_))
		%if %index(&MarkerReductionMethod,Radius) %then %do; 
			%put radius=&Radius;
			radius=&Radius 
			maxclusters=100000
		%end;
		%else %do; 
			%put maxclusters=&NumberOfClusters;
			maxclusters=&NumberOfClusters 
		%end; 
		/*&ProcFastclusOptions*/;
		%if %length(&SkeletonVar) %then 
			by &SkeletonVar;;
	   var &ListVars; 
	run;
	ods exclude none;
	ods results;
	
	%if &syserr %then %do;
		%let exiterror = 1;
		%goto exit;
	%end;	
	
	/*data outlib.kmd0; set kmd; run;*/
	/*data outlib.kmc0; set kmc; run;*/
		
	/*This is a more general code than need for the marker reduction within the linkage groups AP.
	In the linkage groups AP it is mandatory to replace centroids with the original marker values,
	otherwise the remaining code will produce errors. Nevertheless, this following code has options
	to either replace the means of the centroids with values of distances from the input data, or to 
	use the centroid mean values them selves.*/
	%let UseOrginalValues = Yes;
	%let ReplaceMeans = No;
	%if %upcase(&UseOrginalValues)=YES %then %do;
		/*data outlib.orgfclusdat; set orgfclusdat; run;*/
		
		proc sql UNDO_POLICY=NONE;
			create table kmd(drop=_name2_) as
			select *
			from orgfclusdat(rename=(_name_= _name2_)) as a right join kmd(keep = %if %length(&SkeletonVar) %then &SkeletonVar; _name_ _KMCluster_ _KMClusterDistance_) as b
			on a._name2_= b._name_;
		quit;
		
		/*data outlib.kmd1;set kmd;run;*/
		
		*proc sort data=kmd out=os;
		proc sort data=kmd;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _name_;
		run;
					
		data os kmd;
			*set os;
			set kmd;			
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _name_;
			_KMClusterCenter_= 0;
			if first._KMCluster_ then do;
				_KMClusterCenter_= 1;
				output os;
			end;
			output kmd;
		run;
		
		/*data outlib.os; set os; run;*/
		
		*merging by row number*;
		%let misscluster= 0;
		data kmc;
			merge kmc(drop= &ListVars) os;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_;
			if missing(_KMCluster_) then do;
				call symputx("misscluster", 1);
				delete;
				*delete missing(_KMCluster_) is necessary to eliminate clusters of monomorphic markers*;
			end;
			if _freq_ = 0 then delete;
		run;
		%if &misscluster %then %do;
			%put WARNING: Monomorphic markers were found in the Input Data Set and they were removed from the analysis.;
		%end;
	%end;
	/*
	%else %do;
		proc sql UNDO_POLICY=NONE;
			create table kmd as
			select *
			from orgfclusdat(keep = _name_ _ordg_) as a left join kmd as b
			on a._name_= b._name_;
		quit;
		
		proc sort data=kmd out=os;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _name_;
		run;
		
		data os;
			set os;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _name_;
			if not first._KMCluster_ then delete;
		run;
						
		%if %upcase(&ReplaceMeans)=YES %then %do;	
			*merging by row number*;
			data kmc;
				merge kmc(drop=&ListVars) os;
				by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_;
			run;					
		%end;
		%else %do;
			*merging by row number*;
			data kmc;
				merge kmc os(keep= _name_ _KMCluster_);
				by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_;
			run;
		%end;
	%end;
	*/
	
	/*data outlib.kmcout;set kmc;run;*/
	/*data outlib.kmdout;set kmd;run;*/
	
	%if %length(&SkeletonVar) %then %do;
		%if %upcase(&ClusterSkeletonMarker)= NO %then %do;
			%NObsVars(kmc);
			%let ncluster= &nobs;
		
			proc append base=kmd data=orgfclusdat force;
			   where not missing(&SkeletonVar);
			run;
			
			data kmd;
				retain _count_ 0;
				set kmd;
				if not missing(&SkeletonVar) then do;
					_count_ + 1;
					_KMCluster_= &ncluster + _count_;
					_KMClusterDistance_= 0;
					_KMClusterCenter_= 1;
				end;
				else _count_ + 0;
				drop _count_;
			run;
			
			proc append base=kmc data=orgfclusdat force;
			   where not missing(&SkeletonVar);
			run;
			
			data kmc;
				retain _count_ 0;
				set kmc;
				if not missing(&SkeletonVar) then do;
					_count_ + 1;
					_KMCluster_= &ncluster + _count_;
					_KMClusterDistance_= 0;
					_KMClusterCenter_= 1;
				end;
				else _count_ + 0;
				drop _count_;
			run;
			
		%end;
		%else %do;
			data kmd;
				retain _count_ 0;
				set kmd;
				by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_;
				if first._KMCluster_ then do;
					 _count_ + 1;
					_KMCluster_= _count_;
				end;
				else do; 
					_count_ + 0;
					_KMCluster_= _count_;
				end;
				drop _count_;
			run;
			
			*S1099823: renumbering clusters*;
			data kmc;
				retain _count_ 0;
				set kmc;
				by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_;
				if first._KMCluster_ then do;
					 _count_ + 1;
					_KMCluster_= _count_;
				end;
				else do; 
					_count_ + 0;
					_KMCluster_= _count_;
				end;
				drop _count_;
			run;
		%end;
		
		/*data outlib.kmdout2;set kmd;run;*/
		/*data outlib.kmcout2;set kmc;run;*/
	%end;
	
	*need to create new markerlist with selected markers only.;
	data markerlist;
		set kmc(keep=_name_ _KMCluster_ _KMClusterDistance_ _KMClusterCenter_);
		rename _name_=_MarkerName_;
	run;
	
	%NObsVars(markerlist);
	%if &nobs<2 %then %do;
		%if &nvars = 0 %then %do;
			%put ERROR: Marker reduction lead to a the data set that contains no markers to be analyzed.  At least two markers are needed to calculate linkage groups.;
		%end;
		%else %do;
			%put ERROR: Marker reduction lead to a the data set that contains one marker to be analyzed.  At least two markers are needed to calculate linkage groups.;
		%end;
		%goto exit;
	%end;
	
	/*data outlib.markerlist;set markerlist;run;*/
	/*data outlib.kmc2;set kmc;run;*/
	
	/*get representative markers from each cluster and save them to a new gendat.*/
	%put ListVars= &ListVars;
	proc transpose data=kmc out=gendat(drop=_name_);
		var &ListVars;
		id _name_;
	run;
	
	/*data outlib.gendat2;set gendat;run;*/
		
	*need to redirect to new gendat because only a subset of marker will be used from now on;
	data mnames2;
		set gendat;
		where 0;
	run;
	
	/*data outlib.mnames2;set mnames2;run;*/

	%let _tid2_ = %sysfunc(open(mnames2));
	%let _nVars2_ = %sysfunc(attrn(&_tid2_,NVARS));	
	*S1098339: The text expression length (65539) exceeds maximum length (655534).*;
	%let nvarblock= 0;
	%let AlleleVars= ;
	%do k=1 %to &_nVars2_;
		%if %length(&AlleleVars) le 32700 %then %do;
			%let AlleleVars= &AlleleVars %sysfunc(varname(&_tid2_,&k));
		%end;
		%else %do;
			%let nvarblock= %sysevalf(&nvarblock + 1);
			%let AlleleVars&nvarblock = &AlleleVars %sysfunc(varname(&_tid2_,&k));
			%let AlleleVars = ;
		%end;
	%end;
	%if &nvarblock = 0 %then %do;
		%let nvarblock= %sysevalf(&nvarblock + 1);
		%let AlleleVars&nvarblock = &AlleleVars;
		%let AlleleVars = ;
	%end;
	%else %if &nvarblock > 0 and %length(&AlleleVars) > 0 %then %do;
		%let nvarblock= %sysevalf(&nvarblock + 1);
		%let AlleleVars&nvarblock = &AlleleVars;
		%let AlleleVars = ;
	%end;
	%*let OrigAlleleVars= &AlleleVars;

	%let _fv2_= %sysfunc(varname(&_tid2_,1));
	%let _lv2_= %sysfunc(varname(&_tid2_,&_nVars2_));
	/*%if %length(&ListMarkerVars) %then */
		%let ListMarkerVars= &_fv2_.--&_lv2_;
	%let _rc2_ = %sysfunc(close(&_tid2_)); *close opened file*;
		
	/*data outlib.ds; set &ds;run;*/
		
	/*Save original representative markers to &LinkageGroupsKMC_OutData. Must use &ds data set 
	to output the same input format of genotype when format is not NUMERIC GENOTYPE.*/	
	data OutLib.&LinkageGroupsKMC_OutDataName;
		*array gen{*} &AlleleVars;
		retain %do z=1 %to &nvarblock;
				&&AlleleVars&z
			%end;
			;
		set &ds(keep= %do z=1 %to &nvarblock;
				&&AlleleVars&z
				%end;
			);	
	run;

	*merge representative markers and non-marker variables from the input data*;
	data OutLib.&LinkageGroupsKMC_OutDataName;
		set nongendat;
		set OutLib.&LinkageGroupsKMC_OutDataName;	
	run;
		
	*create annotation data set with markers, clusters, and distance*;
	%if %length(&AnnoIndata) %then %do;		
		proc sort data=kmd out=kmd(rename=(_name_= &AnnoMName));
		*proc sort data=kmd out=kmd(rename=(_name_= _MarkerName_));
			by _name_;
		run;
		
		*data kmdgen OutLib.&LinkageGroupsKMD_OutDataName(drop= &ListVars _ordg_ %if %upcase(&AppendDroppedMarkers)= YES %then _sindex_;);
		data kmdgen OutLib.&LinkageGroupsKMD_OutDataName(drop= &ListVars _ordg_);
			merge AnnoTmp kmd(in=a);
			by &AnnoMName;
			*by _MarkerName_;
			if a;
		run;
		
		proc sort data=OutLib.&LinkageGroupsKMD_OutDataName;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName;
			*by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _MarkerName_;
		run;
	%end;
	%else %do;
		data kmdgen(rename=(_name_= _MarkerName_)) OutLib.&LinkageGroupsKMD_OutDataName(keep=_MarkerName_ _KMCluster_ _KMClusterDistance_ _KMClusterCenter_ %if %length(&SkeletonVar) %then &SkeletonVar;);
			set kmd(rename=(_name_= _MarkerName_));
		run;
	%end;
	
*end: k-means clustering*;
%end;
%else %do;
	%let _rc_ = %sysfunc(close(&_tid_)); *close opened file*;
%end;

/*sbrlsi moved the code to estimate recombination frequency from here to qtlUtilityMacros, because it is 
need to add dropped markers back in the Linkage Map Order AP.*/

%rfest(_gendata_=gendat, _annodata_=&Annodat, _RandLG_AP_=1, _runchisq_=1, _swindow_=0, _wsize_=);
%if &exiterror %then %goto exit;

/*data outlib.&OutName._rfmat0; set rfmat; run;*/
/*data outlib.&OutName._lodmat0; set lodmat; run;*/
/*data outlib.&OutName._srmat0; set srmat; run;*/

%*goto exit;

/*Make an output annotation data set to use for interactive filtering to rerun process with */
%PrepOutDS(LinkageGroupsAn_OutData, LinkageGroupsAn_OutDataName, &OutName,_an0);
%if %index(%upcase(&KMeans),NO) %then %do;
	%if %length(&AnnoInData) %then %do;
		options nolabel;
		data Outlib.&LinkageGroupsAn_OutDataName; 
			set AnnoLib.&AnnoData;
		run;
		options label;
	%end;
	%else %do;
		options nolabel;
		data Outlib.&LinkageGroupsAn_OutDataName; 
			set markerlist;
		run;
		options label;
		%let AnnoMName = _MarkerName_;
	%end;
%end;
%else %do;
	data Outlib.&LinkageGroupsAn_OutDataName; 
		set Outlib.&LinkageGroupsKMD_OutDataName;
		/*%if %upcase(&AppendDroppedMarkers)= NO %then _sindex_=_n_;;*/
	run;
	
	proc sort data=Outlib.&LinkageGroupsAn_OutDataName;
		by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName;
		*by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _MarkerName_;
	run;
	
	options nolabel;
 	data Outlib.&LinkageGroupsAn_OutDataName;
		set Outlib.&LinkageGroupsAn_OutDataName;
		by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName;
		*by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ _MarkerName_;
		%if %upcase(&AppendDroppedMarkers)= NO %then %do;
			%if %length(&SkeletonVar) %then %do;
				%if %upcase(&ClusterSkeletonMarker)= NO %then %do;
					if missing(&SkeletonVar) then do;
						*outputing first non-framework marker of each cluster*;
						if first._KMCluster_ and not missing(_KMCluster_) then output;
					end;
					else do;
						*outputing all framework markers*;
						output;
					end;
				%end;
				%else %do;
					*outputing first marker of each cluster*;
					if first._KMCluster_ and not missing(_KMCluster_) then output;
				%end;
			%end;
			%else %do;
				*outputing first marker of each cluster*;
				if first._KMCluster_ and not missing(_KMCluster_) then output;				
			%end;
		%end;
		%else %do;
			%if %length(&SkeletonVar) %then %do;
				%if %upcase(&ClusterSkeletonMarker)= NO %then %do;
					*deleting markers with missing cluster and skeletonvar*;
					if missing(_KMCluster_) and missing(&SkeletonVar) then delete;
				%end;
				%else %do;
					*deleting markers with missing cluster*;
					if missing(_KMCluster_) then delete;
				%end;
			%end;
			%else %do;
				if missing(_KMCluster_) then delete;
			%end;
		%end;
		*missing(_KMCluster_) is necessary to eliminate clusters of monomorphic markers*;
	run;
	options label;

%end;

/*kemicl added for minimum spanning tree grouping via OptModel */
%if %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
	
	/*Check Anno Skeleton Var, if so merge in Anno to create component_id data set for PROC OPTMODEL*/
	%if %length(&SkeletonVar) %then %do;
		data inskeleton; 
			if 0 then set skdata;
			if _N_ = 1 then do;
				declare hash h(dataset:"skdata");
				h.defineKey("_MarkerName_");
				h.defineData("_MarkerName_","_Skeleton_");
				h.defineDone();
			end;
			set markerlist;
			if h.find() = 0;
			keep _MarkerName_ _Skeleton_;
		run;		
	%end;
	
	/*data outlib.inskeleton; set inskeleton; run;*/
			
	/* create a data set of names for output when MDS isn't run to get stacked data 
	data _null_; set markerlist end=_e_;
		call symput("MVar"||trim(left(_n_)), trim(left(_MarkerName_)));
		if _e_ then call symput("Mtot",trim(left(_n_)));
	run;
	
	data stackedmlist;
		length _MarkerName_ $32 _MarkerName_2 $32;
		%do k = 1 %to &Mtot;
			%do m = 1 %to &k;
				_MarkerName_ = "&&MVar&m";
				_MarkerName_2 = "&&MVar&k";
				output;
			%end;
		%end;
	run;	
	*/
	
	/* kemicl CRP: SAS crash for creating stacked marker list with many markers.  change to SQL */
	data namesformerge; set markerlist;
		row = _n_;
	run;
	
	proc sql;
		create table stackedmlist as
		select f1._MarkerName_, f1.row1, f2._MarkerName_2, f2.row2
		from namesformerge(rename = (row=row1)) as f1 
		cross join namesformerge(rename = (_MarkerName_=_MarkerName_2 row=row2)) as f2
		where f1.row1 <= f2.row2
		order by f2.row2, f1.row1
	;
	quit;

	/* data outlib.stackedmlist; set stackedmlist; run; */
	
	data rstack;
		merge stackedmlist rstack(rename = (col1 = Recombination));
		label Recombination = "Recombination";
		if _MarkerName_ = _MarkerName_2 then delete;
		drop row1 row2;
	run;
	
	data lodstack;
		merge stackedmlist lodstack(rename = (col1 = LOD));
		label LOD = "LOD";
		if _MarkerName_ = _MarkerName_2 then delete;
		drop row1 row2;
	run;	
	
	/* data outlib.rstack; set rstack; run; */
		
	options mstored sasmstore=orlib;
	%if %length(%sysfunc(pathname(orlib))) = 0 %then %do;
		libname orlib "&MacroPath.";
	%end;
	
	/* if a skeleton map exists, use this in grouping */
	%if %length(&SkeletonVar) %then %do;
		%SkeletonMST(rf_indata=rstack, lod_indata=lodstack, sdata=inskeleton, marki=_MarkerName_, 
			markj=_MarkerName_2, cost=Recombination, rfthresh=&GroupR, lodcost=LOD, 
			lodthresh=&GroupLOD, lgthresh=&NSkeletonGrps);
	%end;
	%else %do;
		%MinimumSpanningTree(rf_indata=rstack, lod_indata=lodstack, marki=_MarkerName_, 
		markj=_MarkerName_2, cost=Recombination, rfthresh = &GroupR, lodcost=LOD, 
		lodthresh=&GroupLOD, lgthresh= &LGmin);
	%end;
	
	%if &syserr>4 %then %do;
		%put ERROR: The Minimum Recombination Grouping Method could not be completed due to errors.;
		%let exiterror=1;
		 %goto exit;
	%end;	
	
	%if %length(%nrbquote(&syswarningtext)) %then %do;
		%if %index(%quote(%upcase(&syswarningtext)),%quote(SKELETONMST NOT RESOLVED)) or %index(%quote(%upcase(&syswarningtext)),%quote(MINIMUMSPANNINGTREE NOT RESOLVED)) %then %do;
			%put syswarningtext= %nrbquote(&syswarningtext);
			%put ERROR: The stored macro catalog was not found in "&MacroPath.". This file should be copied from "&MacroPath.\Stored32" or "&MacroPath.\Stored64" depending on your OS.;
			%let exiterror=1;
			 %goto exit;
		%end;	
	%end;
				
	proc sort data=outgroups out=grpnum nodupkey;
		by LGroupnum;
	run;
	%nobsvars(grpnum);
	%let NumLinkageGrps = &nobs;
	
	data grpnum;
		set grpnum;
		LGroup = _n_;
	run;

	data groups;
		if 0 then set grpnum;
		if _N_ = 1 then do;
			declare hash h(dataset:"grpnum");
			h.defineKey("LGroupNum");
			h.defineData("LGroupNum","LGroup");
			h.defineDone();
		end;
		set outgroups;
		if h.find() = 0;
		drop LGroupNum;
	run;
	
	data markerlist;
		if 0 then set groups;
		if _N_ = 1 then do;
			declare hash h(dataset:"groups");
			h.defineKey("_MarkerName_");
			h.defineData("_MarkerName_","LGroup");
			h.defineDone();
		end;
		set markerlist;
		if h.find() = 0;
	run;	
%end;

proc datasets library=Outlib nolist;
	delete &LinkageGroupsRec_OutDataName &LinkageGroupsLod_OutDataName;
run;

%if %length(&AnnoDat) %then %do;
	/***** preparing recombination output data *****/
	data recomb;
		merge markerlist srmat rfmat;
		_ord = _n_;
	run;
	
	/*data outlib.recomb; set recomb;run;*/
	
	proc sort data = recomb; 
		by _MarkerName_;
	run;
	
	/*data outlib.annodat; set &Annodat;run;*/
	
	proc sort data = &Annodat out = annom;
		by &AnnoMName;
	run;
	
	data annom;
		set annom;
		_MarkerName_ = &AnnoMName;
	run;
	
	%let mid = %sysfunc(open(markerlist));
	%let MLen=%sysfunc(varlen(&mid,%sysfunc(varnum(&mid,_MarkerName_))));
	%let rc=%sysfunc(close(&mid));	
	
	data outds; 
		length _MarkerName_ $&MLen;
		merge annom recomb(in=a);
		by _MarkerName_;
		if a;
	run;
	
	proc sort data=outds out=Outlib.&LinkageGroupsRec_OutDataName(drop=_ord);
		by _ord;
	run;
	
	%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
	       	*needs to be verified*;
		%let OrigAlleleVars = ;
		%do j=1 %to &_nm_; 
			%let OrigAlleleVars = &OrigAlleleVars &&AMName&j; 
		%end;;
	%end;
	
	/***** preparing LOD output data *****/
	data lodmat;
		merge markerlist srmat lodmat;
		_ord = _n_;
	run;
	
	/*data outlib.lod; set lod; run;*/
	
	proc sort data = lodmat; 
		by _MarkerName_;
	run;
	
	data lodmat; 
		length _MarkerName_ $&MLen;
		merge annom lodmat(in=a);
		by _MarkerName_;
		if a;
	run;
	
	proc sort data=lodmat out=Outlib.&LinkageGroupsLod_OutDataName(drop=_ord);
		by _ord;
	run;	
%end;
%else %do;
	data Outlib.&LinkageGroupsRec_OutDataName; 
		merge markerlist %if %index(%upcase(&GroupingMethod),MINIMUM) %then groups; srmat rfmat;
	run;
	
	data Outlib.&LinkageGroupsLod_OutDataName; 
		merge markerlist %if %index(%upcase(&GroupingMethod),MINIMUM) %then groups; srmat lodmat;
	run;	
	
	%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
		*needs to be verified*;
		%let OrigAlleleVars = ;
		%do j=1 %to &_nm_; 
			%let OrigAlleleVars = &OrigAlleleVars Marker_&j; 
		%end;
	%end;
%end;

%let flag_appendclus= 0;
%if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers) = YES %then %do;

	*get number of clusters*;
	proc sort data=kmdgen;
		by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName;
	run;
		
	%let _skeltype=;
	data _null_;
		length _clus_ $50;
		retain _nclus_ 0;
		set kmdgen end=_eof_;
		by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName; 
		if first._KMCluster_ and not missing(_KMCluster_) then do;
			_nclus_ + 1;
			_clus_ = "_clusv" || strip(_nclus_);
			call symputx(_clus_,strip(_KMCluster_));
			call symputx("numbclus",strip(_nclus_));
			%if %length(&SkeletonVar) %then %do;
				_skel_ = "_skelv" || strip(_nclus_);
				call symputx(_skel_,strip(&SkeletonVar));
			%end;
		end;
		else _nclus_= _nclus_;
		if _eof_ then do;
			%if %length(&SkeletonVar) %then %do;
				call symputx("_skeltype", vtype(&SkeletonVar));
			%end;
		end;
	run;
	%put numbclus= &numbclus;
	
	data rfmatclus_all lodmatclus_all gendatclus_all;
		set _null_;
	run;
		
	%do b=1 %to &numbclus;
	
		%if %length(&SkeletonVar) and %upcase(&_skeltype)= C %then %do; 
			%let _skelv&b= "&&_skelv&b";
		%end;
	
		*create data sets for each cluster*;
		%put _clusv&b= &&_clusv&b;
		%if %length(&SkeletonVar) %then %do;
			%put _skelv&b= &&_skelv&b;
		%end;
		data _gendatclus_;
			set kmdgen;
			where _KMCluster_= &&_clusv&b %if %length(&SkeletonVar) %then and &SkeletonVar=&&_skelv&b;;
		run;
		
		/*data outlib._gendatclus_&b; set _gendatclus_; run;*/
		
		%NObsVars(_gendatclus_);
		%if &nobs > 1 %then %do;
			
			%let flag_appendclus= 1;
			
			proc transpose data=_gendatclus_ out=_gendatclust_(drop=_name_);
				var &ListVars;
				id &AnnoMName;
			run;
			
			/*data outlib._gendatclust_&b; set _gendatclust_; run;*/
		
			%rfest(_gendata_= _gendatclust_, _annodata_= _gendatclus_, _RandLG_AP_=1, _runchisq_=1, _swindow_=0, _wsize_=);
			%if &exiterror %then 
				%goto exit;
			 
			/*data outlib.rfmat&b; set rfmat; run;*/
			/*data outlib.srmat&b; set srmat; run;*/
			
			data rfmatclus;
				*merge _gendatclus_(drop=&ListVars _sindex_ _ordg_) srmat rfmat;
				merge _gendatclus_(drop=&ListVars _ordg_) srmat rfmat;
				%if %length(&AnnoDat) %then _MarkerName_ = &AnnoMName;;
			run;
			
			data lodmatclus;
				*merge _gendatclus_(drop=&ListVars _sindex_ _ordg_) srmat lodmat;
				merge _gendatclus_(drop=&ListVars _ordg_) srmat lodmat;
				%if %length(&AnnoDat) %then _MarkerName_ = &AnnoMName;;
			run;
			
			
			/*
			%if %length(&AnnoDat) %then %do;
				%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
					%let OrigAlleleVars = ;
					%do j=1 %to &_nm_; 
						%let OrigAlleleVars = &OrigAlleleVars &&AMName&j; 
					%end;;
				%end;
			%end;
			%else %do;				
				%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
					%let OrigAlleleVars = ;
					%do j=1 %to &_nm_; 
						%let OrigAlleleVars = &OrigAlleleVars Marker_&j; 
					%end;
				%end;
			%end;
			*/
			
			data rfmatclus_all;
				set rfmatclus_all rfmatclus;
			run;
			
			data lodmatclus_all;
				set lodmatclus_all lodmatclus;
			run;
			
			data mnclust;
				set _gendatclust_;
				where 0;
			run;
			
			%let _tid3_ = %sysfunc(open(mnclust));
			%let _nVars3_ = %sysfunc(attrn(&_tid3_,NVARS));
			%let ClusAlleleVars= ;
			%do k=1 %to &_nVars3_;
				%let ClusAlleleVars= &ClusAlleleVars %sysfunc(varname(&_tid3_,&k));;
			%end;
			%let _rc3_ = %sysfunc(close(&_tid3_));
			
			data gendatclus_all;
				*merge gendatclus_all _gendatclust_;
				merge gendatclus_all &ds(keep=&ClusAlleleVars);
				*must use &ds in order to handle non-numeric marker format*;
			run;
		%end;		
	%end;
%end;
%else %if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers)=NO %then %do;
	data Outlib.&LinkageGroupsRec_OutDataName;
		set Outlib.&LinkageGroupsRec_OutDataName(drop=_KMClusterCenter_);
	run;
	
	data Outlib.&LinkageGroupsLod_OutDataName;
		set Outlib.&LinkageGroupsLod_OutDataName(drop=_KMClusterCenter_);
	run;	
%end;

/*data Outlib.lgr0; set Outlib.&LinkageGroupsRec_OutDataName; run;*/

%put flag_appendclus= &flag_appendclus;
%if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers)=YES and &flag_appendclus=1 %then %do;
	*creating a macro that merges dropped out markers to representative markers*;
	%macro mergeLGandClus;
		
		/*data Outlib.lgr1; set Outlib.&LinkageGroupsRec_OutDataName; run;*/
		/*data outlib.rfmatclus_all1; set rfmatclus_all; run;*/
		/*data outlib.lodmatclus_all1; set lodmatclus_all; run;*/
		/*data Outlib.lgr2; set Outlib.&LinkageGroupsRec_OutDataName;run;*/
		
		*appending recomb matrix of dropped out markers to recomb of representative markers*;
		data rfmatclus_all; 
			set Outlib.&LinkageGroupsRec_OutDataName rfmatclus_all(in=a);
			*if a then _c_=1;
			_ord_= _n_;
		run;

		*removing representative markers that were included in the recomb of dropped markers*;
		proc sort data=	rfmatclus_all;
			by &AnnoMName _ord_;
		run;

		data rfmatclus_all; 
			set rfmatclus_all;
			by &AnnoMName _ord_;
			if first.&AnnoMName then output;
		run;

		*begin: reordering recomb matrix*;
		proc sort data=	rfmatclus_all;
			*by _KMCluster_ _ord_;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName; 
		run;

		data _null_;
			set rfmatclus_all end = last; 
			call symputx("clusMName"||trim(left(_n_)), &AnnoMName);
			if last then call symputx("Totlgrmk",_n_);
		run;
		%put Totlgrmk= &Totlgrmk;
		
		%if %index(%upcase(&GroupingMethod),INTERACTIVE)=0 %then %do;
			ods html close;
			ods listing close;
			ods output OneWayFreqs= nomisslgroups(keep=LGroup);
			proc freq data=rfmatclus_all(keep=LGroup);
			run;
			
			data _null_;
				set nomisslgroups end=_eof_;
				call symputx("nomissLG"||trim(left(_n_)), trim(left(LGroup)));
				if _eof_ then call symputx("numbnomissLG", trim(left(_n_)));
			run;
		%end;

		*get names of dropped out and representative markers*;
		%let KMCnvarblock= &nvarblock;
		%do z=1 %to &KMCnvarblock;
			%let KMCAlleleVars&z= &&AlleleVars&z;
		%end;
		
		*S1098339: The text expression length (65539) exceeds maximum length (655534).*;
		%let nvarblock= 0;
		%let AlleleVars= ;
		%do i=1 %to &Totlgrmk;
			%if %length(&AlleleVars) le 32700 %then %do;
				%let AlleleVars= &AlleleVars &&clusMName&i;
			%end;
			%else %do;
				%let nvarblock= %sysevalf(&nvarblock + 1);
				%let AlleleVars&nvarblock = &AlleleVars &&clusMName&i;
				%let AlleleVars = ;
			%end;
		%end;
		%if &nvarblock = 0 %then %do;
			%let nvarblock= %sysevalf(&nvarblock + 1);
			%let AlleleVars&nvarblock = &AlleleVars;
			%let AlleleVars = ;
		%end;
		%else %if &nvarblock > 0 and %length(&AlleleVars) > 0 %then %do;
			%let nvarblock= %sysevalf(&nvarblock + 1);
			%let AlleleVars&nvarblock = &AlleleVars;
			%let AlleleVars = ;
		%end;		
		%*let OrigAlleleVars= &AlleleVars;
		
		%let ListMarkerVars= &&clusMName1.--&&clusMName&Totlgrmk;
		
		proc transpose data=rfmatclus_all out=rfmatclus_all2;
			var 	%do z=1 %to &nvarblock;
					&&AlleleVars&z
				%end;
			;
			id &AnnoMName;
		run;
		
		proc transpose data=rfmatclus_all2 out=rfmatclus_all3 prefix=_col;
			var 	%do z=1 %to &nvarblock;
					&&AlleleVars&z
				%end;
			;
		run;
				
		/*data outlib.rfmatclus_all; set rfmatclus_all; run;*/
		/*data outlib.rfmatclus_all2; set rfmatclus_all2; run;*/
		/*data outlib.rfmatclus_all3; set rfmatclus_all3; run;*/
		/*
		%do z=1 %to &nvarblock;
			%put AlleleVars&z= &&AlleleVars&z;
		%end;
		*/

		*output new _lgr data set*;
		data Outlib.&LinkageGroupsRec_OutDataName;
			merge rfmatclus_all(drop=%do i=1 %to &Totlgrmk; &&clusMName&i %end;) 
				rfmatclus_all2(drop=_name_)
				rfmatclus_all3(drop=_name_);
			array rf2{*} 	%do z=1 %to &nvarblock;
						&&AlleleVars&z
					%end;
			;
			array rf3{*} _col:;
			do j=1 to _n_;
				do k=1 to j;
					rf2[j]=rf3[k];
				end;
				if missing(rf2[j]) then rf2[j]=.55;
			end;
			*do j=1 to dim(rf2);
			*	if missing(rf2[j]) then rf2[j]=-.55;
			*end;
			drop _ord_ j k _col:;
		run;
		*end: reordering recomb matrix*;
		
		/*data outlib.gendatclus_all; set gendatclus_all;run;*/
		
		*merging dropped out markers to representative markers and reordering them to match the _lgr data set*;
		data gendatclus_all; 
			*array g{*} &AlleleVars;
			retain %do z=1 %to &nvarblock;
					&&AlleleVars&z
				%end;
			;
			merge Outlib.&LinkageGroupsKMC_OutDataName gendatclus_all;
			keep 	%do z=1 %to &nvarblock;
					&&AlleleVars&z
				%end;
			;
		run;
				
		*output new _kmc data set by moving non-markers to first columns*;
		data Outlib.&LinkageGroupsKMC_OutDataName; 
			merge Outlib.&LinkageGroupsKMC_OutDataName(drop= 	%do z=1 %to &KMCnvarblock;
											&&KMCAlleleVars&z
										%end;) 
				gendatclus_all;
		run;
		
		*appending LOD matrix of dropped out markers to LOD of representative markers*;
		data lodmatclus_all; 
			set Outlib.&LinkageGroupsLod_OutDataName lodmatclus_all(in=a);
			*if a then _c_=1;
			_ord_= _n_;
		run;

		*removing representative markers that were included in the LOD of dropped markers*;
		proc sort data=	lodmatclus_all;
			by &AnnoMName _ord_;
		run;

		data lodmatclus_all; 
			set lodmatclus_all;
			by &AnnoMName _ord_;
			if first.&AnnoMName then output;
		run;

		*begin: reordering LOD matrix*;
		proc sort data=	lodmatclus_all;
			*by _KMCluster_ _ord_;
			by %if %length(&SkeletonVar) %then &SkeletonVar; _KMCluster_ _KMClusterDistance_ &AnnoMName; 
		run;

		/*
		data _null_;
			set lodmatclus_all end = last; 
			call symputx("clusMName"||trim(left(_n_)), &AnnoMName);
			if last then call symputx("Totlgrmk",_n_);
		run;
		%put Totlgrmk= &Totlgrmk;
		
		%if %index(%upcase(&GroupingMethod),INTERACTIVE)=0 %then %do;
			ods html close;
			ods listing close;
			ods output OneWayFreqs= nomisslgroups(keep=LGroup);
			proc freq data=lodmatclus_all(keep=LGroup);
			run;
			
			data _null_;
				set nomisslgroups end=_eof_;
				call symputx("nomissLG"||trim(left(_n_)), trim(left(LGroup)));
				if _eof_ then call symputx("numbnomissLG", trim(left(_n_)));
			run;
		%end;

		*get names of dropped out and representative markers*;
		%let KMCnvarblock= &nvarblock;
		%do z=1 %to &KMCnvarblock;
			%let KMCAlleleVars&z= &&AlleleVars&z;
		%end;
		
		*S1098339: The text expression length (65539) exceeds maximum length (655534).*;
		%let nvarblock= 0;
		%let AlleleVars= ;
		%do i=1 %to &Totlgrmk;
			%if %length(&AlleleVars) le 32700 %then %do;
				%let AlleleVars= &AlleleVars &&clusMName&i;
			%end;
			%else %do;
				%let nvarblock= %sysevalf(&nvarblock + 1);
				%let AlleleVars&nvarblock = &AlleleVars &&clusMName&i;
				%let AlleleVars = ;
			%end;
		%end;
		%if &nvarblock = 0 %then %do;
			%let nvarblock= %sysevalf(&nvarblock + 1);
			%let AlleleVars&nvarblock = &AlleleVars;
			%let AlleleVars = ;
		%end;
		%else %if &nvarblock > 0 and %length(&AlleleVars) > 0 %then %do;
			%let nvarblock= %sysevalf(&nvarblock + 1);
			%let AlleleVars&nvarblock = &AlleleVars;
			%let AlleleVars = ;
		%end;		
		%*let OrigAlleleVars= &AlleleVars;
		
		%let ListMarkerVars= &&clusMName1.--&&clusMName&Totlgrmk;
		
		*/
		proc transpose data=lodmatclus_all out=lodmatclus_all2;
			var 	%do z=1 %to &nvarblock;
					&&AlleleVars&z
				%end;
			;
			id &AnnoMName;
		run;
		
		proc transpose data=lodmatclus_all2 out=lodmatclus_all3 prefix=_col;
			var 	%do z=1 %to &nvarblock;
					&&AlleleVars&z
				%end;
			;
		run;
				
		/*data outlib.lodmatclus_all; set lodmatclus_all; run;*/
		/*data outlib.lodmatclus_all2; set lodmatclus_all2; run;*/
		/*data outlib.lodmatclus_all3; set lodmatclus_all3; run;*/
		/*
		%do z=1 %to &nvarblock;
			%put AlleleVars&z= &&AlleleVars&z;
		%end;
		*/

		*output new _lod data set*;
		data Outlib.&LinkageGroupsLod_OutDataName;
			merge lodmatclus_all(drop=%do i=1 %to &Totlgrmk; &&clusMName&i %end;) 
				lodmatclus_all2(drop=_name_)
				lodmatclus_all3(drop=_name_);
			array lod2{*} 	%do z=1 %to &nvarblock;
						&&AlleleVars&z
					%end;
			;
			array lod3{*} _col:;
			do j=1 to _n_;
				do k=1 to j;
					lod2[j]=lod3[k];
				end;
				if missing(lod2[j]) then lod2[j]=.0;
			end;
			*do j=1 to dim(lod2);
			*	if missing(lod2[j]) then lod2[j]=-.0;
			*end;
			drop _ord_ j k _col:;
		run;
		*end: reordering LOD matrix*;		
		
	%mend;
%end;

/* kemicl added for automated clustering */
/*Code for running Cluster Analysis via PROC Cluster */			
%if %UPCASE(&SASClusters)=YES %then %do;	
	ods html close;
	ods graphics off;
	/*
	%PrepHTML(LinkageGroups_body, LinkageGroups_frame, LinkageGroups_contents,&Plot_OutTree._Body.html,&Plot_OutTree._Frame.html,&Plot_OutTree._Contents.html);

	options PageSize=max;
	ods exclude avdist ClusterHistory;
	ods html
		body = "&LinkageGroups_Body"  (url="&bodyfile")
		frame = "&LinkageGroups_Frame"    (url="&framefile")
		contents = "&LinkageGroups_Contents"  (url="&contentsfile")
		gpath = "&OutPath"
	;
	
	ods listing close;
	title1 "Automated Clustering of Recombination Matrix";
	ods graphics on /imagename="&Plot_OutTree._pft";
	*/
	proc cluster data=&LinkageGroupsRec_OutData(Type=Distance) noprint nosquare nonorm 
		method=&AClusteringMethod outtree=tree plots=none; 
			%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
				var &OrigAlleleVars;
			%end;
			%else %do;
				var 	
					%do z=1 %to &nvarblock;
						&&AlleleVars&z
					%end;
				;
			%end;
			id _MarkerName_;
	run;
	/*
	title1;
	ods graphics off;

	filename plotout "&OutPath";
	goptions reset=all
				device=png
				gsfname=plotout;
	*/
	
	/*data Outlib.lg_tree; set tree; run;*/
	
	/*  name="&Plot_OutTree._tre" horizontal  lines=(color=blue width=1) */ 		
	proc tree data=tree noprint Out=Treeout height=H
		%if  %index(%UPCASE(&DefineClust),NUMBER) %then nclusters=&ALGNum; 
		%else %if %index(%UPCASE(&DefineClust),RECOMBINATION) %then level=&RCutoff;
		;
		id _MarkerName_;
	run;
	title1;
	
	/*data outlib.treeout;set treeout;run;*/

	/*For transfer of plots over to client in Client Server Mode 
	%if &Server^=Local %then %do;	
		%PackageInsertFile(PlotCriteria from &ProcessName, &OutPath, &Plot_OutTree._pft.png, , png, yes,no);
		%PackageInsertFile(ClusterTree from &ProcessName, &OutPath, &Plot_OutTree._tre.png, , png, yes,no);
	%end;

	ods html close;
	*/
	%if &syserr>4 %then %goto exit;

	/* Code to add cluster to the output recombination matrix */
	%if %sysfunc(exist(work.treeout)) %then %do;
		/*Code using Hash object to add in cluster when there is an UNIQUE ID variable to match to */
		data &LinkageGroupsRec_OutData;
			if _N_ = 1 then do;
				declare hash h(dataset:"treeout");
				h.defineKey("_MarkerName_");
				h.defineData("Cluster");
				h.defineDone();
			end;
			do until (eof_tree);
			set work.treeout end=eof_tree;
			label Cluster = "Linkage Group";
			rc=h.add();
			end;
			do until(eof_dset);
				set &LinkageGroupsRec_OutData end=eof_dset;
				rs=h.find();
				output;
			end;
			drop clusname  rc rs;
		run;

		proc datasets library=OutLib nolist;
			modify &LinkageGroupsRec_OutDataName;
			label Cluster = ;
			rename cluster=LGroup;
		quit;
	%end;
	%else %do;
		%put ERROR: Linkage Group Assignment via PROC CLUSTER and PROC TREE could not be completed.;
		%put Please check the SAS log for possible errors.;
		%let exiterror = 1;
		%goto exit;
	%end;
	
 	%if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers)=YES and &flag_appendclus=1 %then %do;
		%mergeLGandClus;	
	%end;
	
	%if &notests = 0 %then %do;
		/* another data set with only seg ratios in it to do volcano plot*/
		%PrepOutDS(LinkageGroupsSeg_OutData, LinkageGroupsSeg_OutDataName, &OutName,_srt);
		
		data &LinkageGroupsSeg_OutData;
			set &LinkageGroupsRec_OutData;
			%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
				drop &OrigAlleleVars;
			%end;
			%else %do;
				drop 	%do z=1 %to &nvarblock;
						&&AlleleVars&z
					%end;
				;
			%end;
 	%end;
 	 	
	%PackageInsertDataset(Linkage Groups and Recombination Matrix from &ProcessName, OutLib, &LinkageGroupsRec_OutDataName,no);
%end;	
%else %do;
	%if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers)=YES and &flag_appendclus=1 %then %do;
		%mergeLGandClus;	
	%end;
%end;

%PackageInsertDataset(Linkage Groups and LOD Matrix from &ProcessName, OutLib, &LinkageGroupsLod_OutDataName,no);

%*goto exit;

/*sbrlsi added to output K-Means clusters*/
%if %index(%upcase(&KMeans),YES) %then %do;
	%PackageInsertDataset(K-Means Clusters of Markers from Marker Reduction, OutLib, &LinkageGroupsKMD_OutDataName,no);
	%PackageInsertDataset(Representative Markers from K-Means Clusters from Marker Reduction, OutLib, &LinkageGroupsKMC_OutDataName,no);
%end;

%if %length(&SkeletonVar) %then %do;
	/* create label for jsl of skeleton var */
	%let SkeletonLab = &SkeletonVar;
	%CheckLabel(&LinkageGroupsRec_OutData,SkeletonLab);
%end;
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = %quote(&LinkageGroupsScript);
%let FullAPName = %str(Recombination and Linkage Groups);

*need new mnames in case KMCluster is used and AppendDroppedMarkers=Yes*;
data mnames4; 
	%if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers)=YES 
		and &flag_appendclus=1 and %index(%upcase(&GroupingMethod),INTERACTIVE)=0 %then
		set gendatclus_all;
	%else
		set gendat;
	;
	where 0;
run;
%let _tid4_ = %sysfunc(open(mnames4));
%let _nVars4_ = %sysfunc(attrn(&_tid4_,NVARS));	

data _null_;
	file "&LinkageGroupsScript" lrecl=32767;
	put '//!';	
	put "Names Default to Here(1);";

	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";
	put "ClientOutPath = ""&ClientOutPath"";";
	put "OutDataName = ""&LinkageGroupsRec_OutDataName"";";	
	/* make expression for saving cluster */
	put "if(host is(Windows),";
	put "   include( ConvertFilePath(""$GENOMICS_HOME/JSLFiles/Linkage Group Expressions.jsl"", absolute, windows)),";
	put "   include( ConvertFilePath(""$GENOMICS_HOME/JSLFiles/Linkage Group Expressions.jsl"", absolute, POSIX))";
	put ");";	
	
	%if %index(%upcase(&GroupingMethod),INTERACTIVE) or  %UPCASE(&MakeHPlots) = YES %then %do;	
		/* Make Heatmap of matrix */ 
		%if %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
			put "fhsize = 575;";
			put "fvsize = 575;";
			%let CFlag = 1;
		%end;
		%else %do;
			put "fhsize = 300;";
			put "fvsize = 300;";
			%let CFlag = 0;
			%let LGNum = 1;
			%if %index(%upcase(&GroupingMethod),AUTOMATED) %then %let ClusteringMethod = &AClusteringMethod;
			%else %let ClusteringMethod = Single;
		%end;
		put "Here:Htmp = Function({},{default local},";
		put "	If(IsEmpty(Here:rfdat),";
		put " 	Here:rfdat = open(""&ClientOutPath.&LinkageGroupsRec_OutDataName..sas7bdat"",invisible);";
		%if %UPCASE(&MakeHPlots) = YES %then %do;
			put "		Column(""LGroup"")<<Set Modeling Type(Nominal);";
			put "		Here:rfdat << Color Or Mark by Column(:Name(""LGroup""));";
			%if %length(&SkeletonVar) %then %do;
				put "		Column(""&SkeletonLab"")<<Set Modeling Type(Nominal);";
				put "		Here:rfdat << Select Where(!Is Missing(:Name(""&SkeletonLab"")[]));";
				put "		Here:rfdat << Markers(11);";
				put "		Here:rfdat << Clear Select();";
			%end;
		%end;
		put " ,"; /* else */
		put "		Current Data Table(Here:rfdat);";
		put "	);";
		put " VListBox(";
		%if %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
			put "		HListBox(";
			put "		PanelBox(""Frame Size Tools"",";
			put "      HListBox(";
			put '         HListBox(TextBox("  Horizontal Frame Size"),';
			put '            Here:HSlider = SliderBox(num(100),num(2000),fhsize,';
			put "               try(Here:hcrep = (Here:hc<<report));";	
			put "					  try(Here:hcrep[CellPlotBox(2)]<<FrameSize(fhsize, fvsize));";
			put "				  );";
			put "			  ),";
			put '         HListBox(TextBox("  Vertical Frame Size"),';
			put '            Here:VSlider = SliderBox(num(100),num(2000),fvsize,';
			put "               try(Here:hcrep = (Here:hc<<report));";	
			put "					  try(Here:hcrep[CellPlotBox(2)]<<FrameSize(fhsize, fvsize));";
			put "				  );";
			put "			  ),";
			put "			  ButtonBox(""Reset Frame Size"","; 
			put "					Here:HSlider<< Set(575);";
			put "					Here:VSlider<< Set(575);";
			put "             try(Here:hcrep = (Here:hc<<report));";	
			put "					try(Here:hcrep[CellPlotBox(2)]<<FrameSize(575, 575));";
			put "			  )";
			put "		   )";  
			put "		),";
			put "		PanelBox(""Zoom Tools"",";
			put "			HListBox(";
			put "				ButtonBox(""Zoom to Selected Rows"", Here:hc<<Zoom to Selected Rows),";
			put "				TextBox(""  ""),";
			put "				ButtonBox(""Release Zoom"", Here:hc<<Release Zoom;),";
			put "          TextBox(""     ""),";		
			put "			)";
			put "		),";  /* close panel box*/
			put "		),";
		%end;
		%else %do;
			put "   HListBox(";
		%end;
			put "   Here:hc = Hierarchical Cluster(Y(";
			/* KEMICL REMOVED OPTION OF ALLELES HERE
			%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
				%if %length(&AnnoInData) %then %do;
					%do i=1 %to %eval(&_nm_ - 1);
						put "		:Name(""&&AMName&i""), ";
					%end;
					put "		:Name(""&&AMName&i"")), ";
				%end;			
				%else %do;
					%do i=1 %to %eval(&_nm_ - 1);
						put "		:Name(""Marker_&i""), ";
					%end;
					put "		:Name(""Marker_&i"")), ";
				%end;
			%end;
			%else %do;
			*/
			%do i=1 %to %eval(&_nVars4_ - 1);
				put "		:Name(""%sysfunc(varname(&_tid4_,&i))""), ";
			%end;
			put "		:Name(""%sysfunc(varname(&_tid4_,&i))"")), ";
			%if %UPCASE(&MakeHPlots) = YES %then %do;
				put "		 	Automatic Recalc( 1 ),";
			%end;
			put	"      Label(:_MarkerName_),"/
				"      Method(&ClusteringMethod), Standardize(0), Distance Matrix(1), Hide Distance Messages(1), Color Clusters( &Cflag )," /
				"      Two way clustering, Number of Clusters(&LGNum), Dendrogram Scale(""Distance Scale""), Color Map(Reverse Spectral), Legend(1)," /
			;
			put "		More Color Map Columns( " ;
				%let j = 1;
				%do %while (%length(%scan(&JMPCList,&j,' ')));
					put "		:Name(""%scan(&JMPCList,&j,' ')""),";
					%let j = %eval(&j+1);
				%end;
				put " ),"; /* close color map columns */
			put '      SendToReport(Dispatch({}, "Dendrogram", OutlineBox, Set Title("Linkage Groups and Segregation Ratios")),'/
				 '         Dispatch( {"Dendrogram"}, "", CellPlotBox, {Graph Type( "Dot Plot" )} ),'/
				 '         Dispatch({"Dendrogram"},"", CellPlotBox(2), Frame Size(fhsize,fvsize)),'/
				 '         Dispatch({"Dendrogram"},"Clust Dendro", FrameBox(2), Frame Size(140,fvsize)),'/  
				 '         Dispatch({"Dendrogram"},"Clust Dendro", FrameBox(3), Frame Size(fhsize,140)),'/
				 '       )';
			put "   );";
			%if %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
				/* save initial cluster back to sas data set */
				put "Here:init = 1;";
				put "Here:SaveGroup;";
				put "Here:init = 0;";
			%end;
			%else %do;
				put " 	Here:hc<<Local Data Filter(""on right"", ";
				/*sbrlsi removed Where(:LGroup = 1) because it causes problem in the display if 
				LGgroup = 1 has only one marker.
				put "			Add Filter(Columns(:LGroup), Where(:LGroup = 1)),";
				*/
				put "			Add Filter(Columns(:LGroup),";
				%if %index(%upcase(&KMeans),YES) and %upcase(&AppendDroppedMarkers)=YES 
					and &flag_appendclus=1 %then %do;
					put "			Where(:LGroup=1,";
					%do u=2 %to &numbnomissLG;
						put "			:LGroup= &&nomissLG&u,";
					%end;
					put "				),";
				%end;
				put "			),";
				put "			Mode(  Show(1), Include(1)),";
				put "			Animation( Animate Column( :LGroup ) )";
				put "		);";
				put "	);"; /* Close hlist*/
			%end;

		put " );"; /* close Vlist */
		put ");"; /* close function */

		put "InsertInto(DataList,{{""Here:rfdat""}});";
		put "InsertInto(TabButtonList, ""Linkage Group Results"");";
		put "InsertInto(TabCodeList, expr(Here:Htmp()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
	%end;
	%if &notests = 0 %then %do;
		/* Segregation Ratio Test Volcano Plot */
		put "msize = 4;";
		put "fsize =475;";
		put "cutoff = &Cutoff;";
		put "Here:vp=Function({},{default local},";
		put "	If(IsEmpty(Here:rfdat),";
		*S1100645: Fixing error due to &LinkageGroupsSeg_OutDataName..sas7bdat not found*;
		%*if %UPCASE(&MakeHPlots)=YES or %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
		%if %UPCASE(&MakeHPlots)=YES or %index(%upcase(&GroupingMethod),INTERACTIVE) or %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
			put " 	Here:rfdat = open(""&ClientOutPath.&LinkageGroupsRec_OutDataName..sas7bdat"",invisible);";
			%if %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
				%if %length(&SkeletonVar) %then %do;
					put "		Column(""&SkeletonLab"")<<Set Modeling Type(Nominal);";
					put "		Here:rfdat << Select Where(!Is Missing(:Name(""&SkeletonLab"")[]));";
					put "		Here:rfdat << Markers(11);";
					put "		Here:rfdat << Clear Select();";
				%end;
				put "		Column(""LGroup"")<<Set Modeling Type(Nominal);";
				put "		Here:rfdat << Color Or Mark by Column(:Name(""LGroup""));";	
			%end;
		%end;
		%else %do;
			put " 	Here:rfdat = open(""&ClientOutPath.&LinkageGroupsSeg_OutDataName..sas7bdat"",invisible);";
			put "		Column(""LGroup"")<<Set Modeling Type(Nominal);";
			put "		Here:rfdat << Color Or Mark by Column(:Name(""LGroup""));";
		%end;
		put " ,"; /* else */
		put "		Current Data Table(Here:rfdat);";
		put "	);";		
		put " Column(""_MarkerName_"")<<label(1);";
		%if %length(&SkeletonVar) %then %do;
			put "	Column(""&SkeletonLab"")<<label(1);";
		%end;
		
		put "	 HListBox(";
		put "   VListBox(";
		%if &cross = 2 or %length(&SegRatios) %then %do;
			put " 	  TextBox(""    Testing segregation ratios of AA = &AA_ef, AB = &AB_ef, BB = &BB_ef"", Set Wrap( 500 )),";
		%end;
		%else %if &cross = 1 and &Parent =1 %then %do;
			put " 	  TextBox(""    Testing segregation ratios of AA = &AA_ef, AB = &AB_ef, BB = 0"", Set Wrap( 500 )),";
		%end;
		%else %if &cross = 1 and &Parent =2 %then %do;
			put " 	  TextBox(""    Testing segregation ratios of AA = 0, AB = &AB_ef, BB = &BB_ef"", Set Wrap( 500 )),";
		%end;	
		%else %do;
			put " 	  TextBox(""    Testing segregation ratios of AA = &AA_ef, AB = 0, BB = &BB_ef"", Set Wrap( 500 )),";
		%end;			
		put "      TextBox(""    Line drawn represents significance at alpha = &Alpha"", Set Wrap( 500 )),";
		put "      HListBox(";
		put '         HListBox(TextBox("     Marker Size"),';
		put '            markerSizeSlider = SliderBox(num(-1),num(10.5),msize,';
		put "               try(Here:cbivrep = (Here:cbiv<<report));";	
		put "					  try(Here:cbivrep[framebox(1)]<<MarkerSize(msize));";
		put "   			  );";
		put "			  ),";
		put '         HListBox(TextBox("     Frame Size"),';
		put '            frameSizeSlider = SliderBox(num(100),num(700),fsize,';
		put "               try(Here:cbivrep = (Here:cbiv<<report));";	
		put "					  try(Here:cbivrep[framebox(1)]<<FrameSize(fsize, 0.75*fsize));";
		put "				  );";
		put "			  )";
		put "		  ),";
		put "  	  Here:cbiv = Bivariate(";
		put "				Y(:Name(""&Col7Name"")),";
		put "				X(:Name(""A_Pct minus B_Pct"")),";
		put '               SendToReport(';
		put '                  Dispatch(';
		put '                     {},';
		put '                     "2",';
		put '                     ScaleBox,';
		put '                     {Scale( Linear ), Format( "Best" ),';
		put '                     Add Ref Line( cutoff, Dashed, Red )}';
		put '                  )';
		%if %UPCASE(&SASClusters) = YES or %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
			put "						,Dispatch({},""Bivar Plot"",FrameBox,";
			put "							{Row Legend(LGroup,Color( 1 ))";
			put "							}";
			put "						)";			
		%end;
		put '               )';	
		put "		 	);";
		put "       try((cbiv<<report)[framebox(1)]<<FrameSize(fsize,0.75*fsize));";
		put "       try((cbiv<<report)[framebox(1)]<<MarkerSize(msize));";
		put "       try((cbiv<<report)[framebox(1)]<<Marker Drawing Mode(Normal));";	
		put "  )"; /*close vlistbox*/
		put ",Distribution(";
		put "Nominal Distribution( Column( :LGroup ), Show Counts( 1 ) ),";
		%if %length(&SkeletonVar) %then %do;
			put "Nominal Distribution( Column( :Name(""&SkeletonLab"") ), Show Counts( 1 ) ),";
		%end;
		put "	SendToReport(";
		put "		Dispatch({},""Distributions"",OutlineBox,";
		put "			{Set Title( ""Linkage Group Distribution"" )}";
		put "		),";
		put "		Dispatch({""LGroup""},""Distrib Nom Hist"",FrameBox,";
		put "			{Frame Size( 200, 385 )}";
		put "		),";
		put "		Dispatch( {""LGroup""}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
		%if %length(&SkeletonVar) %then %do;
			put "		,Dispatch( {""&SkeletonLab""}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
		%end;
		put "	)";
		put ");";
		put " ));"; /* close hlistbox and function */

		put "InsertInto(DataList,{{""Here:rfdat""}});";
		put "InsertInto(TabButtonList, ""Segregation and Linkage Groups"");";
		put "InsertInto(TabCodeList, expr(Here:vp()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";	
	%end;
		/*
		put"New Window(""Linkage Groups WARNING"",showmenu(0),showtoolbars(0),";
		put " TextBox(""WARNING: Statistical tests for segregation ratios could not be computed. This could be due to a division by 0. Please check that the specified segregation ratios match your experimental cross."", Set Wrap(450), Font Color(red)),";
		put " TextBox(""Segregation Ratios tested: AA = &AA_ef, AB = &AB_ef, BB = &BB_ef"", Set Wrap( 450 ), Font Color(red))";
			put ");";
		*/
	
	%if %UPCASE(&SASClusters)=NO and %index(%upcase(&GroupingMethod),MINIMUM)=0 %then %do;	
		put "Here:ActionButtonDesc={""Use the Diamond in the Dendrogram to Set Groups""};";
		put "Here:ActionButtonList={{""Save Current Linkage Group Membership""}};";
		put "Here:ActionCodeList ={{Here:SaveGroup}};";
	%end;
	/* end for if not automated mode SASCLusters NO */		

	put "Here:ExcludeExpr = Expr(";
	put "	Here:CreateExclusionAnno(""&AnnoMName"",""&ClientOutPath"",""&OutName"");";
	put "   Here:_OpenAPwithSetting_;";
	/*begin K-Means. sbrlsi*/
	%if %index(%upcase(&KMeans),YES) %then %do;
		put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""InData""),""&FullInPath"");";
		/*S1108223: Setting ListMarkerVars and AlleleVars*/
		put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""ListMarkerVars""),""&ListMarkerVars"");";
		put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""AlleleVars""),"" "");";
		%if %upcase(&AppendDroppedMarkers) = YES %then %do;
			put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""KMeans""),""&KMeans"");";
			put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""MarkerReductionMethod""),""&MarkerReductionMethod"");";
			put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""NumberOfClusters""),""&NumberOfClusters"");";
			put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""CorrRadius""),""&CorrRadius"");";
			put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""AppendDroppedMarkers""),""&AppendDroppedMarkers"");";
		%end;
		put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""KMeans""),""NO"");";
	%end;
	%else %do;
		put "	jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""KMeans""),""NO"");";
	%end;
	/*end K-Means. sbrlsi*/	
	put "	  jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""AnnoInData""),""&ClientOutPath.&OutName._an1.sas7bdat"");";	
	put "     jg:LinkageGroups_updateDeps;";	
	put "	  jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""AnnoMName""),""&AnnoMName"");";
	put "	  jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""AnnoWhereStmt""),""_ExcludeMarker_ = 0"");";
	/*put "	  jg:setCurrentParmValue(2, ""LinkageGroups"", jg:getParmNum(""LinkageGroups"",""OutName""),""&OutName.ex"");";	*/
	put "     jg:_diagpos_ = jg:LinkageGroups_mainDialog << Get Window Position; jg:runProcess(2,""LinkageGroups"");";
	/*
	put "  For(nwc = 1, nwc < Here:nwcounter, nwc++, If(!Is Empty(Here:tabwinlist[nwc]), Here:tabwinlist[nwc] << Close Window));";
	put "  Here:APresults << Close Window; Wait(0); If(!Is Empty(Here:DataList), ";
	put "  For(iii = 1, iii <= N Items(DataList), iii++, For(nn = 1, nn <= N Items(DataList[iii]), nn++, If(!Is Empty(Parse(DataList[iii][nn])), ";
	put "  Try(Close(Eval(Parse(DataList[iii][nn])), NoSave))))))";
	*/
	put "	fromx = 0; closeallexpr;";
	put ");";
	
	%if %UPCASE(&SASClusters)=NO and %index(%upcase(&GroupingMethod),MINIMUM)=0 %then %do;	;
		put "InsertInto(Here:ActionButtonDesc,""Click the Button to Remove Selected Markers"");";
		put "InsertInto(Here:ActionButtonList,{{""Exclude Markers and Rerun Analysis""}});";
		put "InsertInto(Here:ActionCodeList,{{Here:ExcludeExpr}});";	
	%end;
	%else %do;
		put "Here:ActionButtonDesc={""Click the Button to Remove Selected Markers""};";
		put "Here:ActionButtonList={{""Exclude Markers and Rerun Analysis""}};";
		put "Here:ActionCodeList ={{Here:ExcludeExpr}};";
	%end;
	
	put "if(host is(Windows),";
	put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
	put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
	put "	";
	
	put "lobuttonwindow=Function({},{default local},";
	put "		VListBox(";
	put "			TextBox(""Click the button below to launch the Linkage Map Order process.""),";
	put "			ButtonBox(""Linkage Map Order"",";
	put "				if(host is(Windows),";
	put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/LinkageMapOrder.jsl"", absolute, windows)),";
	put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/LinkageMapOrder.jsl"", absolute, POSIX) ));";
	put "	";
	/* Set input data set and Output Folder */
	put "				jg:setDefaultParmValues(2, ""LinkageMapOrder"");";				
	put "				jg:LinkageMapOrder_updateDeps;";
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""InData""),""&ClientOutPath.&LinkageGroupsRec_OutDataName..sas7bdat"");";		
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""OutPath""),""&ClientOutPath"");";										
	put "				jg:LinkageMapOrder_updateDeps;";				
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""LGroupVar""),""LGroup"");";	
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""IDVar""),""_MarkerName_"");";
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""SkOrderVar""),""&SkOrderVar"");";
	%if %length(&OrigAlleleVars) le 32767 and %index(%upcase(&KMeans),NO) %then %do;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""DistVars""),""&OrigAlleleVars"");";
	%end;
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""ListDistVars""),""&ListMarkerVars"");";
	%if %upcase(&AppendDroppedMarkers)=YES %then %do;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""KeepVars""),""_KMCluster_ _KMClusterDistance_ _KMClusterCenter_"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""ClusterVar""),""_KMCluster_"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""ClusterCenterVar""),""_KMClusterCenter_"");";		
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""WhereStmt""),""_KMClusterCenter_=1"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""AddDroppedMarkers""),""Yes"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""OldNumCode""),""&OldNumCode"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""CrossType""),""&CrossType"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""GenNum""),""&OrigGenNum"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""RanMaNum""),""&OrigRanMaNum"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""RILBiasCorr""),""&OrigRILBiasCorr"");";		
	%end;
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""LODInData""),""&ClientOutPath.&LinkageGroupsLOD_OutDataName..sas7bdat"");";												
	%if %length(&OrigAlleleVars) le 32767 and %index(%upcase(&KMeans),NO) %then %do;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""LODVars""),""&OrigAlleleVars"");";
	%end;
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""LODIDVar""),""_MarkerName_"");";	
	put "				jg:LinkageMapOrder_updateDeps;";
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""ListLODVars""),""&ListMarkerVars"");";
	put "				jg:LinkageMapOrder_updateDeps;";	
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""SNPInData""),""&FullInPath"");";
	%if %length(&SkOrderVar) = 0 %then %do;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""Algorithm""),""Accelerated Map Order Optimization"");";
	%end;
	%else %do;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""Algorithm""),""Map Order Optimization"");";
	%end;
	put "				jg:LinkageMapOrder_updateDeps;";
	%if %length(&OrigAlleleVars) le 32767 and %index(%upcase(&KMeans),NO) %then %do;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""AlleleVars""),""&OrigAlleleVars"");";
	%end;
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""ListMarkerVars""),""&ListMarkerVars"");";
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""MarkerFmt""),""&MarkerFmt"");";
	put "				jg:LinkageMapOrder_updateDeps;";
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""GenoDelim""),""&GenoDelim"");";
	put "				jg:setCurrentParmValue(2, ""LinkageMapOrder"", jg:getParmNum(""LinkageMapOrder"",""GenoMcode""),""&GenoMcode"");";
	put "			)";
	put "		)";
	put ");";
		
	put "Here:LaunchCode = lobuttonwindow();";
	
	%if %index(%upcase(&KMeans),YES) %then %do;
		%if &misscluster %then %do;
			put "ActionButtonMessage=NewWindow(""&ProcessName Warning Message"", showmenu(0), showtoolbars(0),";
			put "	VListBox( ";
			put "		tb=TextBox(""Warning message: Monomorphic markers were found in the Input Data Set and they were removed from the analysis."", ";
			put "			<<Font Color(5),<<Set Font Size(10),<<Set Wrap(500)";
			put "		),";
			put "		TextBox(" "),";
			put "		ButtonBox(""OK"",ActionButtonMessage<<CloseWindow);";
			put "	);";
			put ");";
		%end;
	%end;
run;
%let _rc4_ = %sysfunc(close(&_tid4_));	

%PackageInsertFile(JSL Code from &ProcessName, &OutPath, &LinkageGroupsScriptName, &LinkageGroupsRec_OutDataName..sas7bdat, text/plain, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%exitcheck:;

%mend;

%LinkageGroups;