/*
JMP Recombination and Linkage Groups for Outcross

Name: Linkage Groups and Recombination for Outcross
Author: Kelci Miclaus and Luciano Silva
	

Description:
		Recombination and Linkage Groups for Outcross caluclates a matrix of pairwise recombination rates based on an 
		biparetal outcross and clusters the matrix to identify linkage groups for linkage mapping and QTL analysis. 
		This process uses the OneMap R Package and SAS code.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ListMarkerVars WhereStmt 
	AnnoWhereStmt AnnoInData AnnoMName SkeletonVar MarkerType  
	FrameInData FrameMName SkeletonVarF SkOrderVar
	GroupingMethod GroupR LGmin 
	ClusteringMethod MakeHPlots LGNum AClusteringMethod DefineClust ALGNum RCutoff
	SASCLusters LinkageGroups_body LinkageGroups_frame LinkageGroups_contents Plot_OutTree
	OutPath ServerOutPath OutName MacroPath LinkageGroups_OutData LinkageGroupseg_OutData 
	LinkageGroupAn_Outdata LinkageGroupsScript PhasesRFandLod_OutData 
	GroupLOD IncreaseRMemory RMemorySize;

%let LinkageGroups_body=;
%let LinkageGroups_frame=;
%let LinkageGroups_contents=;
%let Plot_OutTree=;
%let LinkageGroups_OutData=;
%let LinkageGroupseg_OutData=;
%let LinkageGroupAn_Outdata=;
%let LinkageGroupsScript=;

/* Set to blank vars needed for GenMacros */
%global FullInPath ByVars AnnoLabel AnnoMergeVars AnnoGroup AnnoLoc AnnoAccession;
%let ByVars = ;
%let AnnoLabel = &AnnoMName;
%let AnnoGroup = ;
%let AnnoLoc = ;
%let AnnoAccession = ;
%let FullInPath = &InData;

/*sbrlsi*/
%global MarkerFmt GenoDelim GenoMCode CrossType XLVARSTMT;
%let MarkerFmt= Outcross Nondelimited Genotypes;
%let GenoDelim= ;
%let GenoMCode= ;
%let CrossType= Outcross;
%let notests= 1;/*to be deleted after inclusion of segregation test*/
%let XLVARSTMT=;
/*sbrlsi*/

*ProcessBody;

%macro LinkageGroupsOutcross;

	%let rc=;

	/*sbrlsi*/
	%let GenoDataObjName= genodata.obj; 
   	%let AnnoDataObjName= annodata.obj;
	%let OutCrossObjName= outcross.obj;
	%let TwoptsObjName= twopts.obj;
	%let BestRFMatrixObjName= rfmat.obj;
	%let BestRFMatrixDataName= rfmat;
	%let BestLODMatrixObjName= lodmat.obj;
	%let BestLODMatrixDataName= lodmat;
	%let BestPhMatrixObjName= phmat.obj;
	%let BestPhMatrixDataName= phmat;
	%let PhasesObjName= phasesmat.obj;
	%let PhasesDataName= phasesmat;
	%let LGObjName= lgroupmat.obj;
	%let LGDataName= lgroupmat;
	
	%include "&MacroPath./OutcrossIMLModules.sas" / nosource;  
        
	%let replacethis=\;
	%let replacewiththis=/;
	%let RSourcePath= %sysfunc(tranwrd(&MacroPath,&replacethis,&replacewiththis));
	/*sbrlsi*/
	
        %include "&MacroPath./GenMacros.sas" / nosource;
        %include "&MacroPath./qtlUtilityMacros.sas" / nosource;
        %*include "&MacroPath./MultipleTestingAdjustment.sas" / nosource;
        
	/*sbrlsi*/
	/*
	%PathName(&InData);
	%if &exiterror %then %do;
		%put ERROR: The Framework Data Set specification is invalid.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let InDataName = &TmpName;	
	%let InDataPath = &TmpPath;
	*/
	/*sbrlsi*/        
        
	%GenInit(LinkageGroupsOutcross,28);
	%if &exiterror=1 %then %goto exit;

	%*goto exitcheck;

	%let ReorderingVars = ; * set this macro to empty to avoid certain conflict when call %GenSubset;

        %global FullAPName;
        %let FullAPName=%str(Recombination and Linkage Groups for Outcross);
        	
	%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
		%put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
		%let exiterror=1;
		%goto exit;
	%end;
        
        %if (%length(&SkeletonVar) or %length(&SkeletonVarF)) and %index(%upcase(&GroupingMethod),MINIMUM) = 0 %then %do;
            %put ERROR: The Minimum Recombination Grouping method must be used for linkage grouping when a Framework Linkage Group Variable is specified.;
            %put ERROR: Please go to the Linkage Grouping Method on the Linkage Groups tab and specify the Minimum Grouping Recombination Threshold for joining new markers to groups.;
            %let exiterror=1;
            %goto exit;
        %end;       
	
	%let OrigAlleleVars = &AlleleVars;
	%let AlleleVars = &AlleleVars &ListMarkerVars;
        
	/* data with recombination matrix in it */
	%PrepOutDS(LinkageGroups_OutData, LinkageGroups_OutDataName, &OutName,_lgr);
	%PrepOutFile(LinkageGroupsScript, LinkageGroupsScriptName,&ProcessName Script for &OutName..jsl);
                        
        %if %index(%upcase(&GroupingMethod),AUTOMATED) %then %do;
            %let SASClusters = Yes;
        %end;
        %else %do; 
            %let SASClusters = No;
        %end;
        
	%*goto exitcheck;
            
        *make name for Cluster Plot and Check name for Cluster OutData;
        %if %UPCASE(&SASClusters)=YES %then %do;
            %let Plot_OutTree=&LinkageGroups_OutData;
            %CheckDataName(_name=Plot_OutTree,_default=&OrigInData);
        %end;
            
	 /* AnnoWhere statement applied in this macro */
	%PrepMapandBy;
	run;
	%if &exiterror=1 %then %goto exit;

	%if %length(&FrameInData) %then %do;
		%PathName(&FrameInData);
		%if &exiterror %then %do;
			%put ERROR: The Framework Data Set specification is invalid.;
			%let exiterror=1;
			%goto exit;
		%end;
		%let FrameData = &TmpName;
		libname FrameLib "&TmpPath";
		
		%if %length(&SkOrderVar) %then %do;
			data tmp;
				set framelib.&FrameData;
				where 0;
				keep &SkOrderVar;
			run;
			
			%let dsid = %sysfunc(open(tmp));
			%let SkType = %sysfunc(vartype(&dsid, 1));
			%let rc = %sysfunc(close(&dsid));
			%if &SkType = C %then %do;
				%put ERROR: The Framework Order Variable must be numeric to ensure that it is used correctly in the Linkage Map Order follow-up process.;
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
	%end;
	
	%put ds= &ds;
	%if %length(&WhereStmt) %then %do;
		data countobs; 
			set &ds;
			where &WhereStmt;
		run;
		%Nobsvars(countobs);
		%let ds=countobs;
	%end;
	%else %Nobsvars(&ds);

	%if &nobs=0 %then %do;
		%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
		%goto exit;
	%end;
	
	/* data set to use in iml, only genotypes */
	data gendat; 
		set &ds;  
		keep &AlleleVars;
	run;
	
	data mnames; 
		set gendat;
		where 0;
	run;
	
	%if %length(&ListMarkerVars) %then %do;
		/*get first and last marker name and create new liststyle that will not */
		%let tid = %sysfunc(open(mnames));
		%let nobs = %sysfunc(attrn(&tid,NOBS));
		%let nvars = %sysfunc(attrn(&tid,NVARS));
		%put nObs =&nObs nVars=&nVars;
		%let fmvar = %sysfunc(varname(&tid,1));
		%let lmvar = %sysfunc(varname(&tid,&nvars));
		%let tid = %sysfunc(close(&tid));
		
		%let ListMarkerVars = &fmvar.--&lmvar;
		%put &ListMarkerVars;
		%let AlleleVars = &OrigAlleleVars &ListMarkerVars;
	%end;
	%else %do;
		%nobsvars(mnames);
	%end;
		
	%if &nvars<2 %then %do;
		%if &nvars = 0 %then %do;
			%put ERROR: The data set contains no markers to be analyzed.  At least two markers are needed to calculate linkage groups.;
		%end;
		%else %do;
			%put ERROR: The data set only contains one marker to be analyzed.  At least two markers are needed to calculate linkage groups.;
		%end;
		%goto exit;
	%end;
	
	/* add here and make marker list data set.  if no annotation is used, and a framework map is used build skdata from markerlist and framework*/
	%let _tid_ = %sysfunc(open(mnames));
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));
	
	data markerlist;
		length _MarkerName_ $32; 
		%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
			%let _nm_ = %sysfunc(floor(%eval(&_nVars_/2)));
			%put There are &_nm_ Markers;
			%if %length(&AnnoInData) %then %do;
				%do i = 1 %to &_nm_;
					_MarkerName_ = "&&AMName&i";
					output;
				%end;			
			%end;
			%else %do;
				%do i = 1 %to &_nm_;
					_MarkerName_ = "Marker_&i";
					output;
				%end;
			%end;
		%end;
		%else %do;
			%do i = 1 %to &_nVars_;
				_MarkerName_ = "%sysfunc(varname(&_tid_,&i))";
				output;
			%end;
		%end;
	run;

        %*goto exitcheck;
        
        %let Annodat = ;
        /*kemicl added to make data set for anno to avoid errors with allele format */
        %if %length(&AnnoInData) %then %do;
            %let Annodat = AnnoLib.&AnnoData;
            %if %length(&AnnoWhereStmt) %then %do;
		data adat; 
			set AnnoLib.&AnnoData;
			where &AnnoWhereStmt;
		run;
		%let Annodat = adat;
            %end;
        %end;
        
        %*goto exitcheck;

        %if %length(&FrameInData) %then %do;
            %if ^%length(&AnnoInData) %then %do;
            	%let Annodat = markerlist;
            	%let AnnoMName = _MarkerName_;
            %end;
            data adat; set &Annodat;
            	ord = _n_;
            	drop &SkeletonVar &SkeletonVarF &SkOrderVar;
            run;   
            
            **S1115689: sbrlsi added monotonic() to the select statement to avoid problems with duplicated rows in the FrameData*;
            proc sql UNDO_POLICY=NONE; 
            	create table adat(drop = ord _mono_) as 
            		select *, monotonic() as _mono_ 
            		from adat as d1 left join FrameLib.&FrameData(keep = &SkeletonVarF &SkOrderVar &FrameMName) as d2
			on d1.&AnnoMName = d2.&FrameMName
			group by d1.&AnnoMName
			having max(_mono_)= _mono_
			order by ord;
            quit;
            %let Annodat = adat;
            %let SkeletonVar = &SkeletonVarF;
        %end;
       
	/*make skeleton map for input to constrained MST for grouping */
	%if %length(&SkeletonVar) %then %do;
		/* get skeleton markers and groups out and create a 1 to s valued _skeleton_ variable*/
		data grouped ungrouped; 
			set &Annodat;
			if not missing(&SkeletonVar) then output grouped;
			else output ungrouped;
			keep &AnnoMName &SkeletonVar;
		run;
	
		proc sort data=grouped out=grpnum nodupkey;
			by &SkeletonVar;
		run;
		%nobsvars(grpnum);
		%let NSkeletonGrps = &nobs;
		%if &nobs = 0 %then %do;
			%put ERROR:  No markers have framework linkage groups based on the values of the variables specified from the Framework Map data set.;
			%put ERROR:  Please check that the Framework Marker variable values matches with marker column names in the input data set. ;
			%goto exit;
		%end;
		
		%nobsvars(ungrouped);
		%if &nobs = 0 %then %do;
			%put ERROR:  All Markers already belong to linkage groups as specified by the Framework Linkage Group Variable. No further grouping can be done.;
			%goto exit;
		%end;
	
		data grpnum;
			set grpnum;
			_Skeleton_ = _n_;
		run;
	
		/* create separate component_id values from s+1 for ungrouped markers */
		data ungrouped;
			set ungrouped;
			_Skeleton_ = _n_ + &NSkeletonGrps;
		run;
	
		data skdata;
			if 0 then set grpnum;
			if _N_ = 1 then do;
				declare hash h(dataset:"grpnum");
				h.defineKey("&SkeletonVar");
				h.defineData("&SkeletonVar","_Skeleton_");
				h.defineDone();
			end;
			set &Annodat;
			if h.find() = 0;
			keep &AnnoMName &SkeletonVar _Skeleton_;
		run;	
		
		%*goto exitcheck;
	
		data skdata; 
			set skdata ungrouped; 
			rename &AnnoMName = _MarkerName_;
		run;
	%end;	
	
	/*data outlib.annodat; set &Annodat; run;*/
       
       /*begin estimation of recombination*/
	options nonotes;
	ods _all_ close;
	*ods html body="&Outpath.&OutName..html";
	proc iml; 
		/*NOTES:
		(1) Must keep module calls in the order they have been written here because
			the output from one module may be used in the next.
		(2) R objects created by a module can be accessed by any other module as far as 
			modules calls are keept within the same block defined by "Proc iml"	and "quit".
	        This fact can be used to avoid having to exchange matrices or datasets between 
			R and IML.
		*/
		
		/*loading modules*/
		load module=(RemoveRObjs);
		load module=(SetRMemory);
		load module=(LoadRPkgs);
		load module=(LoadRFunc);
		load module=(ExportDataToR);
		load module=(RebuildOutcrossObj);
		load module=(TwoPts_RF);
		load module=(GetTwopts_Analysis);
		load module=(Groups_LG);
		
		%if %upcase(&IncreaseRMemory)= YES %then %do;
			/*sets R max memory size*/
			run SetRMemory(&RMemorySize);
		%end;
		
		/*loads R packages*/
		run LoadRPkgs;
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;

		/*loads R functions*/
		run LoadRFunc("&RSourcePath");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;

		/*exports genotypic data from SAS to R*/
		run ExportDataToR("gendat", "&GenoDataObjName");

		/*exports annotation data from SAS to R*/
		run ExportDataToR("&Annodat", "&AnnoDataObjName");

		/*rebuilds outcross object in R*/
		run RebuildOutcrossObj("&GenoDataObjName", "&AnnoDataObjName", "&OutCrossObjName", "&MarkerType");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
		
		/*removes objects in R*/
		run RemoveRObjs("&GenoDataObjName, &AnnoDataObjName");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
		
		/*estimates pairwise recombinations*/
		run TwoPts_RF("NULL", "NULL", "&OutCrossObjName", "&TwoptsObjName");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
		
		/*get estimates of recombination fraction and LOD for each linkage phase*/
		run GetTwopts_Analysis("&TwoptsObjName", "&BestRFMatrixObjName", "&BestRFMatrixDataName", 
							"&BestLODMatrixObjName", "&BestLODMatrixDataName",
							"&BestPhMatrixObjName", "&BestPhMatrixDataName",
							"&PhasesObjName", "&PhasesDataName");		
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
		
		%if %index(%upcase(&GroupingMethod),TRANSITIVE) %then %do;
			/*estimate linkage groups*/
			run Groups_LG(&GroupLOD, &GroupR, "&TwoptsObjName", "&LGObjName", "&LGDataName");		
			if ^isOK then do;
				call symputx("exiterror", 1);
				abort;
			end;
			
			/*data outlib.LGDataName; set &LGDataName; run;*/
		%end;
	quit; 
	
	/*data outlib.BestRFMatrixDataName0; set &BestRFMatrixDataName; run;*/
	data &BestRFMatrixDataName;
		set &BestRFMatrixDataName;
		array rf{*} col:;
		do k=1 to dim(rf);
			if k < _n_ then do;
				*reset missing recombination*;
				if missing(rf[k]) then rf[k]=.55; 
			end;
		end;
		drop k;
	run;
	/*data outlib.BestRFMatrixDataName; set &BestRFMatrixDataName; run;*/
	
	/*data outlib.BestLODMatrixDataName0; set &BestLODMatrixDataName; run;*/
	data &BestLODMatrixDataName;
		set &BestLODMatrixDataName;
		array lod{*} col:;
		do k=1 to dim(lod);
			if k < _n_ then do;
				*reset missing lod*;
				if missing(lod[k]) then lod[k]=0; 
			end;
		end;
		drop k;
	run;	
	/*data outlib.BestLODMatrixDataName; set &BestLODMatrixDataName; run;*/
		
	%if %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
		proc iml;
			use &BestRFMatrixDataName;
			read all into Rf;
			rstack = symsqr(Rf);	
			nr= nrow(rstack);
			do j=1 to nr;
				*reset missing recombination*;
				if missing(rstack[j]) then rstack[j]=.55; 
			end;
			create rstack from rstack;
			append from rstack;
			close &BestRFMatrixDataName;
			
			use &BestLODMatrixDataName;
			read all into LOD;
			lodstack = symsqr(LOD);	
			nr= nrow(lodstack);
			do j=1 to nr;
				if missing(lodstack[j]) then lodstack[j]=0;
			end;
			create lodstack from lodstack;
			append from lodstack;
			close &BestLODMatrixDataName;	
		quit;
	%end;
	options notes;
	/*end estimation of recombination*/
       
       	%*goto exitcheck;
       
 	%if &syserr>4 or &exiterror = 1 %then %do;
 		%put ERROR: Calculation of pairwise recombination could not be completed.;
 		%put ERROR: Please be sure the appropriate columns were selected as Marker Variables and Marker Type. Also, check coding of genotypes for possible errors.;
 	 	%let exiterror=1;
 		 %goto exit;
	%end;
	
	/*sbrlsi: data with recombination and lod for all linkage phases in it */	
	%PrepOutDS(PhasesRFandLod_OutData, PhasesRFandLod_OutDataName, &OutName,_tps);
	
	data Outlib.&PhasesRFandLod_OutDataName;
		length Marker1 Marker2 $32 rf1-rf4 lod1-lod4 8;
		set &PhasesDataName(rename=(rf1=_rf1 rf2=_rf2 rf3=_rf3 rf4=_rf4 lod1=_lod1 lod2=_lod2 lod3=_lod3 lod4=_lod4));
		rf1= _rf1;
		rf2= _rf2;
		rf3= _rf3;
		rf4= _rf4;	
		lod1= _lod1;
		lod2= _lod2;
		lod3= _lod3;
		lod4= _lod4;	
		drop _rf: _lod:;	
	run;	
	
	%PackageInsertDataset(Recombination and LOD tall data from &ProcessName, OutLib, &PhasesRFandLod_OutDataName,no);
	/*sbrlsi*/
 
 	%*goto exitcheck;
 
	%if %index(%Upcase(&MarkerFmt), ALLELES) and %length(&AnnoInData) %then %do;
		data _null_;
			set &Annodat end = last;
			call symputx("AMName"||trim(left(_n_)), &AnnoLabel);
			if last then call symputx("Totnames",_n_);
		run;
	%end;

	options nonotes;
	proc datasets library=work nolist;
		modify rfmat;
		%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
			%let _nm_ = %sysfunc(floor(%eval(&_nVars_/2)));
			%put There are &_nm_ Markers;
			%if %length(&AnnoInData) %then %do;
				%if &Totnames >= &_nm_ %then %do;
					rename %do i=1 %to &_nm_; Col&i = &&AMName&i %end;;
				%end;
				%else %do;
					%put ERROR: The number of markers formed from the alleles in the Input Data Set do not match with the the number of marker names in the Annotation Data Set.;
					%let exiterror =1;
					%goto exit;
				%end;
			%end;
			%else %do;
				rename %do i=1 %to &_nm_; Col&i = Marker_&i %end;;
			%end;
		%end;
		%else %do;
			rename %do i=1 %to &_nVars_; Col&i = %sysfunc(varname(&_tid_,&i)) %end;;
		%end;
		*modify srmat;
		*rename %do i=1 %to 5; Col&i = %scan(&CountList,&i,' ') %end; Col6 = SegRatio_Chisq Col7 = &Col7Name Col8 = A_minus_B;
		modify phmat;
		rename %do i=1 %to &_nVars_; pcol&i = ph_%sysfunc(varname(&_tid_,&i)) %end;;
	quit;
	           
	options notes label;
      
	/*Make an output annotation data set to use for interactive filtering to rerun process with */
	%PrepOutDS(LinkageGroupAn_OutData, LinkageGroupAn_OutDataName, &OutName,_an0);
	%if %length(&AnnoInData) %then %do;
		options nolabel;
		data Outlib.&LinkageGroupAn_OutDataName; 
			set AnnoLib.&AnnoData;
		run;
		options label;
	%end;
	%else %do;
		options nolabel;
		data Outlib.&LinkageGroupAn_OutDataName; 
			set markerlist;
		run;
		options label;
		%let AnnoMName = _MarkerName_;
	%end;

	/*kemicl added for minimum spanning tree grouping via OptModel */
	%if %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
	
		/*Check Anno Skeleton Var, if so merge in Anno to create component_id data set for PROC OPTMODEL*/
		%if %length(&SkeletonVar) %then %do;
			data inskeleton;
				if 0 then set skdata;
				if _N_ = 1 then do;
					declare hash h(dataset:"skdata");
					h.defineKey("_MarkerName_");
					h.defineData("_MarkerName_","_Skeleton_");
					h.defineDone();
				end;
				set markerlist;
				if h.find() = 0;
				keep _MarkerName_ _Skeleton_;
			run;		
		%end;

		/*data outlib.inskeleton; set inskeleton; run;*/

		data _null_; set markerlist end=_e_;
			call symput("MVar"||trim(left(_n_)), trim(left(_MarkerName_)));
			if _e_ then call symput("Mtot",trim(left(_n_)));
		run;

		/* create a data set of names for output when MDS isn't run to get stacked data */
		data stackedmlist;
			length _MarkerName_ $32 _MarkerName_2 $32;
			%do k = 1 %to &Mtot;
				%do m = 1 %to &k;
					_MarkerName_ = "&&MVar&m";
					_MarkerName_2 = "&&MVar&k";
					output;
				%end;
			%end;
		run;	

		data rstack;
			merge stackedmlist rstack(rename = (col1 = Recombination)) lodstack(rename=(col1 = LOD));
			label Recombination = "Recombination";
			label LOD= "LOD";
			if _MarkerName_ = _MarkerName_2 then delete;
		run;
		
		/*data outlib.rstack; set rstack; run; */
		
		%*goto exitcheck;

		options mstored sasmstore=orlib;
		%if %length(%sysfunc(pathname(orlib))) = 0 %then %do;
			libname orlib "&MacroPath.";
		%end;

		/* if a skeleton map exists, use this in grouping */
		%if %length(&SkeletonVar) %then %do;
			%SkeletonMST(rf_indata=rstack, lod_indata=, sdata=inskeleton, marki=_MarkerName_, 
				markj=_MarkerName_2, cost=Recombination, rfthresh=&GroupR, lodcost=, 
				lodthresh=, lgthresh=&NSkeletonGrps);
		%end;
		%else %do;
			%MinimumSpanningTree(rf_indata=rstack, lod_indata=, marki=_MarkerName_, 
				markj=_MarkerName_2, cost=Recombination, rfthresh = &GroupR, lodcost=, 
				lodthresh=, lgthresh= &LGmin);
		%end;

		%if &syserr>4 %then %do;
			%put ERROR: The Minimum Recombination Grouping Method could not be completed due to errors.;
			%let exiterror=1;
			 %goto exit;
		%end;	
		%if %length(&syswarningtext) %then %do;
			%if %index(%quote(%upcase(&syswarningtext)),%quote(SKELETONMST NOT RESOLVED)) or %index(%quote(%upcase(&syswarningtext)),%quote(MINIMUMSPANNINGTREE NOT RESOLVED)) %then %do;
				%put &syswarningtext;
				%put ERROR: The stored macro catalog was not found in "&MacroPath.". This file should be copied from "&MacroPath.\Stored32" or "&MacroPath.\Stored64" depending on your OS.;
				%let exiterror=1;
				 %goto exit;
			%end;	
		%end;

		proc sort data=outgroups out=grpnum nodupkey;
			by LGroupnum;
		run;
		%nobsvars(grpnum);
		%let NumLinkageGrps = &nobs;

		data grpnum;
			set grpnum;
			LGroup = _n_;
		run;

		data groups;
			if 0 then set grpnum;
			if _N_ = 1 then do;
				declare hash h(dataset:"grpnum");
				h.defineKey("LGroupNum");
				h.defineData("LGroupNum","LGroup");
				h.defineDone();
			end;
			set outgroups;
			if h.find() = 0;
			drop LGroupNum;
		run;

		data markerlist;
			if 0 then set groups;
			if _N_ = 1 then do;
				declare hash h(dataset:"groups");
				h.defineKey("_MarkerName_");
				h.defineData("_MarkerName_","LGroup");
				h.defineDone();
			end;
			set markerlist;
			if h.find() = 0;
		run;	
	%end;
	%else %if %index(%upcase(&GroupingMethod),TRANSITIVE) %then %do;
		data markerlist;
			merge markerlist &LGDataName(keep=LGroup);
		run;
	%end;

	proc datasets library=Outlib;
		delete &LinkageGroups_OutDataName;
	run;
	
	/*data outlib.markerlist; set markerlist;run;*/
	/*data outlib.rfmat; set rfmat;run;*/
	
	%if %length(&AnnoDat) %then %do;
		data recomb;
			*merge markerlist srmat rfmat;
			merge markerlist rfmat; /*sbrlsi*/
			_ord = _n_;
		run;
		
		proc sort data = recomb; 
			by _MarkerName_;
		run;
		
		proc sort data = &Annodat out = annom;
			by &AnnoMName;
		run;
		
		data annom;
			set annom;
			_MarkerName_ = &AnnoMName;
		run;
		
		/*begin S1078887: problem was generated when annotation data set had more markers than input data set*/
		%let mid = %sysfunc(open(recomb));
		%let MR_Len=%sysfunc(varlen(&mid,%sysfunc(varnum(&mid,_MarkerName_))));
		%let rc=%sysfunc(close(&mid));	
		
		%let mid = %sysfunc(open(annom));
		%let MA_Len=%sysfunc(varlen(&mid,%sysfunc(varnum(&mid,_MarkerName_))));
		%let rc=%sysfunc(close(&mid));	
		
		%let MLen= %sysfunc(max(&MR_Len, &MA_Len));
		%put MLen=&MLen;
		/*end S1078887*/
		
		data outds; 
			length _MarkerName_ $&MLen;
			merge annom recomb(in=a);
			by _MarkerName_;
			if a;
		run;
		
		proc sort data=outds out=Outlib.&LinkageGroups_OutDataName(drop=_ord);
			by _ord;
		run;
		
		%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
			%let OrigAlleleVars = ;
			%do j=1 %to &_nm_; 
				%let OrigAlleleVars = &OrigAlleleVars &&AMName&j; 
			%end;;
		%end;
	%end;
	%else %do;
		*data Outlib.&LinkageGroups_OutDataName; merge markerlist %if %index(%upcase(&GroupingMethod),MINIMUM) %then groups; srmat rfmat;
		/*sbrlsi*/
		data Outlib.&LinkageGroups_OutDataName; 
			merge markerlist %if %index(%upcase(&GroupingMethod),MINIMUM) %then groups; rfmat;
		run;
		/*sbrlsi*/
		
		%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
			%let OrigAlleleVars = ;
			%do j=1 %to &_nm_; 
				%let OrigAlleleVars = &OrigAlleleVars Marker_&j; 
			%end;
		%end;
	%end;
	
	/* kemicl added for automated clustering */
	/*Code for running Cluster Analysis via PROC Cluster */			
	%if %UPCASE(&SASClusters)=YES %then %do;	
		ods html close;
		ods graphics off;
		/*
		%PrepHTML(LinkageGroups_body, LinkageGroups_frame, LinkageGroups_contents,&Plot_OutTree._Body.html,&Plot_OutTree._Frame.html,&Plot_OutTree._Contents.html);

		options PageSize=max;
		ods exclude avdist ClusterHistory;
		ods html
			body = "&LinkageGroups_Body"  (url="&bodyfile")
			frame = "&LinkageGroups_Frame"    (url="&framefile")
			contents = "&LinkageGroups_Contents"  (url="&contentsfile")
			gpath = "&OutPath"
		;

		ods listing close;
		title1 "Automated Clustering of Recombination Matrix";
		ods graphics on /imagename="&Plot_OutTree._pft";
		*/
		
		/*sbrlsi added because of missing values in the recombination matrix of outcross*/
		%put LinkageGroups_OutData= &LinkageGroups_OutData;
		data Linkage_OutData_Tmp(drop=i);
			set &LinkageGroups_OutData;
			%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
				array a{*} &OrigAlleleVars;
			%end;
			%else %do;
				array a{*} &AlleleVars;
			%end;
			do i=1 to dim(a);
				if missing(a{i}) then a{i}=.5;
			end;
		run;
		
		%*goto exitcheck;
		/*data outlib.Linkage_OutData_Tmp; set Linkage_OutData_Tmp; run;*/
		
		*proc cluster data=&LinkageGroups_OutData(Type=Distance) noprint nosquare nonorm method=&AClusteringMethod outtree=tree plots=none; 
		proc cluster data=Linkage_OutData_Tmp(Type=Distance) noprint nosquare nonorm method=&AClusteringMethod outtree=tree plots=none; 
				%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
					var &OrigAlleleVars;
				%end;
				%else %do;
					var &AlleleVars;
				%end;
				id _MarkerName_;
		run;
		
		%*goto exitcheck;
		
		/*
		title1;
		ods graphics off;

		filename plotout "&OutPath";
		goptions reset=all
					device=png
					gsfname=plotout;
		*/
		/*  name="&Plot_OutTree._tre" horizontal  lines=(color=blue width=1) */ 
		proc tree data=tree noprint Out=Treeout height=H
			%if  %index(%UPCASE(&DefineClust),NUMBER) %then nclusters=&ALGNum; 
			%else %if %index(%UPCASE(&DefineClust),RECOMBINATION) %then level=&RCutoff;
			;
			id _MarkerName_;
		run;
		title1;

		%*goto exitcheck;

		/*For transfer of plots over to client in Client Server Mode 
		%if &Server^=Local %then %do;	
			%PackageInsertFile(PlotCriteria from &ProcessName, &OutPath, &Plot_OutTree._pft.png, , png, yes,no);
			%PackageInsertFile(ClusterTree from &ProcessName, &OutPath, &Plot_OutTree._tre.png, , png, yes,no);
		%end;

		ods html close;
		*/
		%if &syserr>4 %then %goto exit;

		/* Code to add cluster to the output recombination matrix */
		%if %sysfunc(exist(work.treeout)) %then %do;
		/*Code using Hash object to add in cluster when there is an UNIQUE ID variable to match to */
		  data &LinkageGroups_OutData;
			if _N_ = 1 then do;
				declare hash h(dataset:"treeout");
				h.defineKey("_MarkerName_");
				h.defineData("Cluster");
				h.defineDone();
			end;
			do until (eof_tree);
			set work.treeout end=eof_tree;
			label Cluster = "Linkage Group";
			rc=h.add();
			end;
			do until(eof_dset);
				set &LinkageGroups_OutData end=eof_dset;
				rs=h.find();
				output;
			end;
			drop clusname  rc rs;
		  run;

		  proc datasets library=OutLib;
			modify &LinkageGroups_OutDataName;
			label Cluster = ;
			rename cluster=LGroup;
		  quit;
		%end;
		%else %do;
			%put ERROR: Linkage Group Assignment via PROC CLUSTER and PROC TREE could not be completed.;
			%put Please check the SAS log for possible errors.;
			%let exiterror = 1;
			%goto exit;
		%end;

		%if &notests = 0 %then %do;
			/* another data set with only seg ratios in it to do volcano plot*/
			%PrepOutDS(LinkageGroupseg_OutData, LinkageGroupseg_OutDataName, &OutName,_srt);

			data &LinkageGroupseg_OutData;
				set &LinkageGroups_OutData;
				%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
					drop &OrigAlleleVars;
				%end;
				%else %do;
					drop &AlleleVars;
				%end;
		%end;
	%end;	

	%PackageInsertDataset(Linkage Groups and Recombination Matrix from &ProcessName, OutLib, &LinkageGroups_OutDataName,no);

	%if %length(&SkeletonVar) %then %do;
		/* create label for jsl of skeleton var */
		%let SkeletonLab = &SkeletonVar;
		%CheckLabel(&LinkageGroups_OutData,SkeletonLab);
	%end;
	%global _mainjslfile_ FullAPName;
	%let _mainjslfile_ = %quote(&LinkageGroupsScript);
	%let FullAPName = %str(Recombination and Linkage Groups);

	/*sbrlsi*/
	*data &LinkageGroups_OutData;
		*merge &LinkageGroups_OutData &BestPhMatrixDataName;
		*call symputx("mktype_"||strip(_n_),&MarkerType,'L');
	*run;	
	data _null_;
		set &LinkageGroups_OutData;
		call symputx("mktype_"||strip(_n_),&MarkerType,'L');
	run;	
	/*sbrlsi*/
	
	%*goto exitcheck;

	data _null_;
		file "&LinkageGroupsScript" lrecl=32767;
		put '//!';	
		put "Names Default to Here(1);";

		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";
		put "ClientOutPath = ""&ClientOutPath"";";
		put "OutDataName = ""&LinkageGroups_OutDataName"";";	
		/* make expression for saving cluster */
		put "if(host is(Windows),";
		put "   include( ConvertFilePath(""$GENOMICS_HOME/JSLFiles/Linkage Group Expressions.jsl"", absolute, windows)),";
		put "   include( ConvertFilePath(""$GENOMICS_HOME/JSLFiles/Linkage Group Expressions.jsl"", absolute, POSIX))";
		put ");";	
	
		%if %index(%upcase(&GroupingMethod),INTERACTIVE) or  %UPCASE(&MakeHPlots) = YES %then %do;	
			/* Make Heatmap of matrix */ 
			%if %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
				put "fhsize = 575;";
				put "fvsize = 575;";
				%let CFlag = 1;
			%end;
			%else %do;
				put "fhsize = 300;";
				put "fvsize = 300;";
				%let CFlag = 0;
				%let LGNum = 1;
				%if %index(%upcase(&GroupingMethod),AUTOMATED) %then %let ClusteringMethod = &AClusteringMethod;
				%else %let ClusteringMethod = Single;
			%end;
			put "Here:Htmp = Function({},{default local},";
			put "	If(IsEmpty(Here:rfdat),";
			put " 	Here:rfdat = open(""&ClientOutPath.&LinkageGroups_OutDataName..sas7bdat"",invisible);";
			%if %UPCASE(&MakeHPlots) = YES %then %do;
				put "		Column(""LGroup"")<<Set Modeling Type(Nominal);";
				put "		Here:rfdat << Color Or Mark by Column(:Name(""LGroup""));";
				%if %length(&SkeletonVar) %then %do;
					put "		Column(""&SkeletonLab"")<<Set Modeling Type(Nominal);";
					put "		Here:rfdat << Select Where(!Is Missing(:Name(""&SkeletonLab"")[]));";
					put "		Here:rfdat << Markers(11);";
					put "		Here:rfdat << Clear Select();";
				%end;
			%end;
			put " ,"; /* else */
			put "		Current Data Table(Here:rfdat);";
			put "	);";
			put " VListBox(";
			%if %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
				put "		HListBox(";
				put "		PanelBox(""Frame Size Tools"",";
				put "      HListBox(";
				put '         HListBox(TextBox("  Horizontal Frame Size"),';
				put '            Here:HSlider = SliderBox(num(100),num(2000),fhsize,';
				put "               try(Here:hcrep = (Here:hc<<report));";	
				put "					  try(Here:hcrep[CellPlotBox(2)]<<FrameSize(fhsize, fvsize));";
				put "				  );";
				put "			  ),";
				put '         HListBox(TextBox("  Vertical Frame Size"),';
				put '            Here:VSlider = SliderBox(num(100),num(2000),fvsize,';
				put "               try(Here:hcrep = (Here:hc<<report));";	
				put "					  try(Here:hcrep[CellPlotBox(2)]<<FrameSize(fhsize, fvsize));";
				put "				  );";
				put "			  ),";
				put "			  ButtonBox(""Reset Frame Size"","; 
				put "					Here:HSlider<< Set(575);";
				put "					Here:VSlider<< Set(575);";
				put "             try(Here:hcrep = (Here:hc<<report));";	
				put "					try(Here:hcrep[CellPlotBox(2)]<<FrameSize(575, 575));";
				put "			  )";
				put "		   )";  
				put "		),";
				put "		PanelBox(""Zoom Tools"",";
				put "			HListBox(";
				put "				ButtonBox(""Zoom to Selected Rows"", Here:hc<<Zoom to Selected Rows),";
				put "				TextBox(""  ""),";
				put "				ButtonBox(""Release Zoom"", Here:hc<<Release Zoom;),";
				put "          TextBox(""     ""),";		
				put "			)";
				put "		),";  /* close panel box*/
				put "		),";
			%end;
			%else %do;
				put "   HListBox(";
			%end;
				put "   Here:hc = Hierarchical Cluster(Y(";
				/* KEMICL REMOVED OPTION OF ALLELES HERE
				%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
					%if %length(&AnnoInData) %then %do;
						%do i=1 %to %eval(&_nm_ - 1);
							put "		:Name(""&&AMName&i""), ";
						%end;
						put "		:Name(""&&AMName&i"")), ";
					%end;			
					%else %do;
						%do i=1 %to %eval(&_nm_ - 1);
							put "		:Name(""Marker_&i""), ";
						%end;
						put "		:Name(""Marker_&i"")), ";
					%end;
				%end;
				%else %do;
				*/
				%do i=1 %to %eval(&_nVars_ - 1);
					put "		:Name(""%sysfunc(varname(&_tid_,&i))""), ";
				%end;
				put "		:Name(""%sysfunc(varname(&_tid_,&i))"")), ";
				%if %UPCASE(&MakeHPlots) = YES %then %do;
					put "		 	Automatic Recalc( 1 ),";
				%end;
				put	"      Label(:_MarkerName_),"/
					"      Method(&ClusteringMethod), Standardize(0), Distance Matrix(1), Hide Distance Messages(1), Color Clusters( &Cflag )," /
					"      Two way clustering, Number of Clusters(&LGNum), Dendrogram Scale(""Distance Scale""), Color Map(Reverse Spectral), Legend(1)," /
				;
				put "	More Color Map Columns( " ;
				put "	:Name(""&MarkerType""),";
				put " ),";  /* close color map columns */			

				put '      SendToReport(Dispatch({}, "Dendrogram", OutlineBox, Set Title("Linkage Groups and Marker Type")),'/
					 '         Dispatch( {"Dendrogram"}, "", CellPlotBox, {Graph Type( "Color Map" )} ),'/
					 '         Dispatch({"Dendrogram"},"", CellPlotBox(2), Frame Size(fhsize,fvsize)),'/
					 '         Dispatch({"Dendrogram"},"Clust Dendro", FrameBox(2), Frame Size(140,fvsize)),'/  
					 '         Dispatch({"Dendrogram"},"Clust Dendro", FrameBox(3), Frame Size(fhsize,140)),'/
					 '       )';
				put "   );";
				%if %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
					/* save initial cluster back to sas data set */
					put "Here:init = 1;";
					put "Here:SaveGroup;";
					put "Here:init = 0;";
				%end;
				%else %do;
					put " 	Here:hc<<Local Data Filter(""on right"", ";
					/*sbrlsi removed Where(:LGroup = 1) because it causes problem in the display if 
					LGgroup = 1 has only one marker.
					put "			Add Filter(Columns(:LGroup), Where(:LGroup = 1)),";
					*/
					put "			Add Filter(Columns(:LGroup)),";
					put "			Mode(  Show(1), Include(1)),";
					put "			Animation( Animate Column( :LGroup ) )";
					put "		);";
					put "	);"; /* Close hlist*/
				%end;

			put " );"; /* close Vlist */
			put ");"; /* close function */

			put "InsertInto(DataList,{{""Here:rfdat""}});";
			put "InsertInto(TabButtonList, ""Linkage Group Results"");";
			put "InsertInto(TabCodeList, expr(Here:Htmp()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
		%end;
		
		%if &notests = 0 %then %do;
			/* Segregation Ratio Test Volcano Plot */
			put "msize = 4;";
			put "fsize =475;";
			put "cutoff = &Cutoff;";
			put "Here:vp=Function({},{default local},";
			put "	If(IsEmpty(Here:rfdat),";
			*S1100645: Fixing error due to &LinkageGroupsSeg_OutDataName..sas7bdat not found.*;
			%*if %UPCASE(&MakeHPlots)=YES or %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;
			%if %UPCASE(&MakeHPlots)=YES or %index(%upcase(&GroupingMethod),INTERACTIVE) 
				or %index(%upcase(&GroupingMethod),MINIMUM) 
				or %index(%upcase(&GroupingMethod),TRANSITIVE)
			%then %do;
				put " 	Here:rfdat = open(""&ClientOutPath.&LinkageGroups_OutDataName..sas7bdat"",invisible);";
				%if %index(%upcase(&GroupingMethod),MINIMUM) %then %do;
					%if %length(&SkeletonVar) %then %do;
						put "		Column(""&SkeletonLab"")<<Set Modeling Type(Nominal);";
						put "		Here:rfdat << Select Where(!Is Missing(:Name(""&SkeletonLab"")[]));";
						put "		Here:rfdat << Markers(11);";
						put "		Here:rfdat << Clear Select();";
					%end;
					put "		Column(""LGroup"")<<Set Modeling Type(Nominal);";
					put "		Here:rfdat << Color Or Mark by Column(:Name(""LGroup""));";	
				%end;
			%end;
			%else %do;
				put " 	Here:rfdat = open(""&ClientOutPath.&LinkageGroupseg_OutDataName..sas7bdat"",invisible);";
				put "		Column(""LGroup"")<<Set Modeling Type(Nominal);";
				put "		Here:rfdat << Color Or Mark by Column(:Name(""LGroup""));";
			%end;
			put " ,"; /* else */
			put "		Current Data Table(Here:rfdat);";
			put "	);";		
			put " Column(""_MarkerName_"")<<label(1);";
			%if %length(&SkeletonVar) %then %do;
				put "	Column(""&SkeletonLab"")<<label(1);";
			%end;

			put "	 HListBox(";
			put "   VListBox(";
			%if &cross = 2 or %length(&SegRatios) %then %do;
				put " 	  TextBox(""    Testing segregation ratios of AA = &AA_ef, AB = &AB_ef, BB = &BB_ef"", Set Wrap( 500 )),";
			%end;
			%else %if &cross = 1 and &Parent =1 %then %do;
				put " 	  TextBox(""    Testing segregation ratios of AA = &AA_ef, AB = &AB_ef, BB = 0"", Set Wrap( 500 )),";
			%end;
			%else %if &cross = 1 and &Parent =2 %then %do;
				put " 	  TextBox(""    Testing segregation ratios of AA = 0, AB = &AB_ef, BB = &BB_ef"", Set Wrap( 500 )),";
			%end;	
			%else %do;
				put " 	  TextBox(""    Testing segregation ratios of AA = &AA_ef, AB = 0, BB = &BB_ef"", Set Wrap( 500 )),";
			%end;			
			put "      TextBox(""    Line drawn represents significance at alpha = &Alpha"", Set Wrap( 500 )),";
			put "      HListBox(";
			put '         HListBox(TextBox("     Marker Size"),';
			put '            markerSizeSlider = SliderBox(num(-1),num(10.5),msize,';
			put "               try(Here:cbivrep = (Here:cbiv<<report));";	
			put "					  try(Here:cbivrep[framebox(1)]<<MarkerSize(msize));";
			put "   			  );";
			put "			  ),";
			put '         HListBox(TextBox("     Frame Size"),';
			put '            frameSizeSlider = SliderBox(num(100),num(700),fsize,';
			put "               try(Here:cbivrep = (Here:cbiv<<report));";	
			put "					  try(Here:cbivrep[framebox(1)]<<FrameSize(fsize, 0.75*fsize));";
			put "				  );";
			put "			  )";
			put "		  ),";
			put "  	  Here:cbiv = Bivariate(";
			put "				Y(:Name(""&Col7Name"")),";
			put "				X(:Name(""A_Pct minus B_Pct"")),";
			put '               SendToReport(';
			put '                  Dispatch(';
			put '                     {},';
			put '                     "2",';
			put '                     ScaleBox,';
			put '                     {Scale( Linear ), Format( "Best" ),';
			put '                     Add Ref Line( cutoff, Dashed, Red )}';
			put '                  )';
			%if %UPCASE(&SASClusters) = YES or %index(%upcase(&GroupingMethod),MINIMUM) or %index(%upcase(&GroupingMethod),TRANSITIVE) %then %do;
				put "						,Dispatch({},""Bivar Plot"",FrameBox,";
				put "							{Row Legend(LGroup,Color( 1 ))";
				put "							}";
				put "						)";			
			%end;
			put '               )';	
			put "		 	);";
			put "       try((cbiv<<report)[framebox(1)]<<FrameSize(fsize,0.75*fsize));";
			put "       try((cbiv<<report)[framebox(1)]<<MarkerSize(msize));";
			put "       try((cbiv<<report)[framebox(1)]<<Marker Drawing Mode(Normal));";	
			put "  )"; /*close vlistbox*/
			put ",Distribution(";
			put "Nominal Distribution( Column( :LGroup ), Show Counts( 1 ) ),";
			%if %length(&SkeletonVar) %then %do;
				put "Nominal Distribution( Column( :Name(""&SkeletonLab"") ), Show Counts( 1 ) ),";
			%end;
			put "	SendToReport(";
			put "		Dispatch({},""Distributions"",OutlineBox,";
			put "			{Set Title( ""Linkage Group Distribution"" )}";
			put "		),";
			put "		Dispatch({""LGroup""},""Distrib Nom Hist"",FrameBox,";
			put "			{Frame Size( 200, 385 )}";
			put "		),";
			put "		Dispatch( {""LGroup""}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
			%if %length(&SkeletonVar) %then %do;
				put "		,Dispatch( {""&SkeletonLab""}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
			%end;
			put "	)";
			put ");";
			put " ));"; /* close hlistbox and function */

			put "InsertInto(DataList,{{""Here:rfdat""}});";
			put "InsertInto(TabButtonList, ""Segregation and Linkage Groups"");";
			put "InsertInto(TabCodeList, expr(Here:vp()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";	
		%end;
		/*
		put"New Window(""Linkage Groups WARNING"",showmenu(0),showtoolbars(0),";
		put " TextBox(""WARNING: Statistical tests for segregation ratios could not be computed. This could be due to a division by 0. Please check that the specified segregation ratios match your experimental cross."", Set Wrap(450), Font Color(red)),";
		put " TextBox(""Segregation Ratios tested: AA = &AA_ef, AB = &AB_ef, BB = &BB_ef"", Set Wrap( 450 ), Font Color(red))";
			put ");";
		*/

		%if %UPCASE(&SASClusters)=NO and %index(%upcase(&GroupingMethod),MINIMUM)=0 and %index(%upcase(&GroupingMethod),TRANSITIVE)=0 %then %do;	
			put "Here:ActionButtonDesc={""Use the Diamond in the Dendrogram to Set Groups""};";
			put "Here:ActionButtonList={{""Save Current Linkage Group Membership""}};";
			put "Here:ActionCodeList ={{Here:SaveGroup}};";
		%end;
		/* end for if not automated mode SASCLusters NO */		

		put "Here:ExcludeExpr = Expr(";
		put "	  Here:CreateExclusionAnno(""&AnnoMName"",""&ClientOutPath"",""&OutName"");";
		put "   Here:_OpenAPwithSetting_;";
		put "	  jg:setCurrentParmValue(2, ""LinkageGroupsOutcross"", jg:getParmNum(""LinkageGroupsOutcross"",""AnnoInData""),""&ClientOutPath.&OutName._an1.sas7bdat"");";	
		put "   jg:LinkageGroupsOutcross_updateDeps;";	
		put "	  jg:setCurrentParmValue(2, ""LinkageGroupsOutcross"", jg:getParmNum(""LinkageGroupsOutcross"",""AnnoMName""),""&AnnoMName"");";
		put "	  jg:setCurrentParmValue(2, ""LinkageGroupsOutcross"", jg:getParmNum(""LinkageGroupsOutcross"",""AnnoWhereStmt""),""_ExcludeMarker_ = 0"");";
		put "   jg:_diagpos_ = jg:LinkageGroupsOutcross_mainDialog << Get Window Position; jg:runProcess(2,""LinkageGroupsOutcross"");";
		/*
		put "  For(nwc = 1, nwc < Here:nwcounter, nwc++, If(!Is Empty(Here:tabwinlist[nwc]), Here:tabwinlist[nwc] << Close Window));";
		put "  Here:APresults << Close Window; Wait(0); If(!Is Empty(Here:DataList), ";
		put "  For(iii = 1, iii <= N Items(DataList), iii++, For(nn = 1, nn <= N Items(DataList[iii]), nn++, If(!Is Empty(Parse(DataList[iii][nn])), ";
		put "  Try(Close(Eval(Parse(DataList[iii][nn])), NoSave))))))";
		*/
		put "	fromx = 0; closeallexpr;";
		put ");";

		%if %UPCASE(&MakeHPlots) = YES or %index(%upcase(&GroupingMethod),INTERACTIVE) %then %do;	
			%if %UPCASE(&SASClusters)=NO and %index(%upcase(&GroupingMethod),MINIMUM)=0 and %index(%upcase(&GroupingMethod),TRANSITIVE)=0 %then %do;
				put "InsertInto(Here:ActionButtonDesc,""Click the Button to Remove Selected Markers"");";
				put "InsertInto(Here:ActionButtonList,{{""Exclude Markers and Rerun Analysis""}});";
				put "InsertInto(Here:ActionCodeList,{{Here:ExcludeExpr}});";	
			%end;
			%else %do;
				put "Here:ActionButtonDesc={""Click the Button to Remove Selected Markers""};";
				put "Here:ActionButtonList={{""Exclude Markers and Rerun Analysis""}};";
				put "Here:ActionCodeList ={{Here:ExcludeExpr}};";
			%end;
		%end;
		
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";

		put "lobuttonwindow=Function({},{default local},";
		put "		VListBox(";
		put "			TextBox(""Click the button below to launch the Linkage Map Order process.""),";
		put "			ButtonBox(""Linkage Map Order"",";
		put "				if(host is(Windows),";
		put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/LinkageMapOrderOutcross.jsl"", absolute, windows)),";
		put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/LinkageMapOrderOutcross.jsl"", absolute, POSIX) ));";
		put "	";
		/* Set input data set and Output Folder */
		put "				jg:setDefaultParmValues(2, ""LinkageMapOrderOutcross"");";				
		put "				jg:LinkageMapOrderOutcross_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""InData""),""&ClientOutPath.&LinkageGroups_OutDataName..sas7bdat"");";		
		%if %upcase(&CrossType) = OUTCROSS %then %do;
			put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""MarkerType""),""&MarkerType"");";
			put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""LinkPhaseInData""),""&ClientOutPath.&PhasesRFandLod_OutDataName..sas7bdat"");";	
		%end;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""OutPath""),""&ClientOutPath"");";										
		put "				jg:LinkageMapOrderOutcross_updateDeps;";				
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""LGroupVar""),""LGroup"");";	
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""IDVar""),""_MarkerName_"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""SkOrderVar""),""&SkOrderVar"");";
		/*
		%if %length(&OrigAlleleVars) le 32767 %then %do;
			put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""DistVars""),""&OrigAlleleVars"");";
		%end;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""ListDistVars""),""&ListMarkerVars"");";
		*/
		
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""SNPInData""),""&FullInPath"");";
		/*
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""SNPInData""),""&InDataPath.&InDataName..sas7bdat"");";
		*/
		put "				jg:LinkageMapOrderOutcross_updateDeps;";
		%if %length(&OrigAlleleVars) le 32767 %then %do;
			put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""AlleleVars""),""&OrigAlleleVars"");";
		%end;
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""ListMarkerVars""),""&ListMarkerVars"");";
		/*
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""MarkerFmt""),""&MarkerFmt"");";
		put "				jg:LinkageMapOrderOutcross_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""GenoDelim""),""&GenoDelim"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapOrderOutcross"", jg:getParmNum(""LinkageMapOrderOutcross"",""GenoMcode""),""&GenoMcode"");";
		*/
		put "			)";
		put "		)";
		put ");";

		put "Here:LaunchCode = lobuttonwindow();";
	run;
	
	%let rc=%sysfunc(close(&_tid_));	
	%PackageInsertFile(JSL Code from &ProcessName, &OutPath, &LinkageGroupsScriptName, &LinkageGroups_OutDataName..sas7bdat, text/plain, yes);

	%*goto exitcheck;
		
	%exit:;
	%put rc= &rc;
	%if %length(&rc) %then
		%let rc=%sysfunc(close(&_tid_));
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

	%exitcheck:; *---sbrlsi used it while testing code ---*;
	quit;

%mend LinkageGroupsOutcross;
%LinkageGroupsOutcross;
