/*
JMP Genomics Analytical Process

Name: Missing Genotype Summary
Author: Wendy Czika

*/


/*
-----------------------------------------------------------
Copyright (C) 2007 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------*/

*ProcessVariables;

%global StudyName InData AlleleVars MarkerFmt GenoDelim TraitVar ByVars ListMarkerVars ListTraitVars WhereStmt 
AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoAccession AnnoWhereStmt
Perms Seed Conv MTC PValAdj Tau Bandwidth Alpha OnlySig OutPath ServerOutPath OutName MacroPath MGS_OutData MGSScript MGSScript2;

*ProcessBody;

%macro MissingGenotypeSummary;

%include "&MacroPath./GenMacros.sas" / nosource;

%GenInit(MissingGenotypeSummary,29);
%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(Missing Genotype by Trait Summary);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;
%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%global OldAlleleVars;
%let OldAlleleVars = &AlleleVars;
%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%if %length(&Seed)=0 %then %let Seed=0;

%PrepOutDS(MGS_OutData,MGS_OutDataName,&OutName._mgs);
%PrepOutFile(MGSScript,MGSScriptName,&ProcessName Script for &OutName..jsl);

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

data outcc;
 delete;
run;

data outt;
	delete;
run;

data a1;
	set &ds(obs=1);
run;

proc transpose data=a1 out=at;
	var &TraitVar;
run;

%if &syserr %then %goto exit;

%NObsVars(at);
%global Ntrait;
%let Ntrait = &nobs;

%do i=1 %to &Ntrait; 
	%global _tv&i;
%end; 

proc sort data=at;
	by _name_;
run;

/*data OutLib.mg_at; set at; run;*/

data _null_;
	set at;
	length is $ 16;
	is = "_tv" || strip(_n_);
	call symput(is,strip(_name_));
run;

%let flagsubds= 0;
%if %length(&WhereStmt) %then %do;
	data countobs; 
 		set &ds;
  		where &WhereStmt;
	run;
 	%Nobsvars(countobs);
 	%let ds=countobs;
 	%let flagsubds= 1;
%end;
%else %Nobsvars(&ds);

**start: recoding binary trait variables to 0 and 1 scale to avoid errors in the ploting of missing values**;
%let flagnumeric = 0;
%let flagcharcter = 0;
%let _tid_ = %sysfunc(open(&ds));
%do i=1 %to &Ntrait;
	%let tvarnum&i= %sysfunc(varnum(&_tid_,&&_tv&i));
	%put tvarnum&i= &&tvarnum&i;
	%let tvartype&i= %sysfunc(vartype(&_tid_,&&tvarnum&i));
	%if &&tvartype&i = N %then %do;
		%let flagnumeric = 1;
	%end;
	%else %do;
		%let flagcharcter = 1;
	%end;
	%put tvartype&i= &&tvartype&i;
%end;
%let _tid_ = %sysfunc(close(&_tid_));
%put flagnumeric = &flagnumeric;
%put flagcharcter = &flagcharcter;

%global nrectraits;
%let flagrec= 0;
%let k= 1;
%do i=1 %to &Ntrait; 
	proc freq data=&ds noprint;
		tables &&_tv&i / out=t&i;
	run;
	
	proc sort data=t&i;
		by &&_tv&i;
	run;
	
	data t&i;
		set t&i;
		if missing(&&_tv&i) then delete;
	run;
	
	/*data OutLib.mg_t&i; set t&i; run;*/
	
	%NObsVars(t&i);	
	%let nlevels&i= &nobs;

	data _null_;
		set t&i;
		if _n_ = 1 then call symputx("min&i", &&_tv&i);
		%if &&nlevels&i > 1 %then %do;
			if _n_ = 2 then call symputx("max&i", &&_tv&i);
		%end;
	run;
	
	%if &&nlevels&i < 2 %then %do;
		%let max&i= &&min&i;
	%end;
	
	%let flagrec&i= 0;
	%if &&tvartype&i = N %then %do;
		%if &&min&i ^= 0 or &&max&i ^= 1 %then %do;
			%let flagrec&i= 1;
			%let flagrec= 1;
			%let TextSummary&k= The values of trait %upcase(&&_tv&i) were recoded (old=new) as &&min&i=0 and &&max&i=1;
			%let k= %eval(&k+1);
		%end;
	%end;
	%else %do;
		%let flagrec&i= 1;
		%let flagrec= 1;
		%let TextSummary&k= The values of trait %upcase(&&_tv&i) were recoded (old=new) as &&min&i=0 and &&max&i=1;
		%let k= %eval(&k+1);
	%end;
	%put flagrec&i= &&flagrec&i;
%end;
%let nrectraits= %eval(&k-1);
	
%if &flagrec %then %do;

	%if &flagsubds %then %do;
		%let recds= &ds;
	%end;
	%else %do;
		%let recds= rectraits;
	%end;

	data &recds;
		set &ds(rename= (%do i=1 %to &Ntrait; 
					%if &&flagrec&i %then %do;
						&&_tv&i= _tmp_&i
					%end;
				%end;
				)
			);
		%do i=1 %to &Ntrait; 
			%if &&tvartype&i = N %then %do;
				%if &&flagrec&i %then %do;
					if _tmp_&i = &&min&i then &&_tv&i= 0;
					else if _tmp_&i = &&max&i then &&_tv&i= 1;
					else _tmp_&i= .;
				%end;
			%end;
			%else %do;
				%if &&flagrec&i %then %do;
					if _tmp_&i = "&&min&i" then &&_tv&i= 0;
					else if _tmp_&i = "&&max&i" then &&_tv&i= 1;
					else _tmp_&i= " ";
				%end;			
			%end;
		%end;
		;
	run;	

	%let ds= &recds;
	
	/*data OutLib.mg_ds; set &ds; run;*/
%end;
**end: recoding binary trait variables to 0 and 1 scale to avoind errors in the ploting of missing values**;

proc format;
	value numtochar 0='A/A'
	2='B/B'
	1='A/B'
	other=''
	;
run;

/******
data OutLib.missinggeno;
	set &ds;
	array allvars{*} &AlleleVars;
	array mi{&nummark};
	do _i=1 to &nummark;
		%if %length(&GenoCol) %then %do;
			if missing(allvars{_i}) %if %upcase(&MarkerFmt)=GENOTYPES %then or allvars{_i}="/"; then mi{_i}=1;
			else mi{_i}=0;
		%end;
		%else %do;
			if missing(allvars{2*_i-1}) or missing(allvars{2*_i}) then mi{_i}=1;
			else mi{_i}=0;		
		%end;
	end;
	drop &AlleleVars _i;
run;		

%let errvarlist=:mi1;
%do i=2 %to &nummark;
	%let errvarlist=&errvarlist, :mi&i;
%end;

data _null_;
	file "&MGSScript2";
	put "datacp=open("".\missinggeno.sas7bdat"");";
	put "Cell Plot(";
	put "	Y(";
	put "		&errvarlist";
	put "	),";
	put "	By(:&AnnoGroup),";
*	put "	Label(:&AnnoLabel)";
	put ");";
run;
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &MGSScriptName2, "missinggeno.sas7bdat", text/plain);
******/	
			
%if &nobs=0 %then %do;
	%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
	%goto exit;
%end;%let BlockSize = %sysfunc(min(100000,%sysevalf(50000000/&nobs,floor)));

%let first = 1;
%let BlockSize=%sysfunc(min(&BlockSize,&nummark));
%let last = &BlockSize;
%let Block = 1;

data _null_;
	set &ds;
	array getname{*} &AlleleVars;
	%let Block=1;
	%do %until (&first > &nummark);	
		%if %length(&GenoCol) %then %do;
			_f = vname(getname(&first));
			_l = vname(getname(&last));
		%end;
		%else %do;
			_f = vname(getname(%eval(2*&first-1)));
			_l = vname(getname(%eval(2*&last)));
		%end;

		call symput("firstvar&Block",_f);
		call symput("lastvar&Block",_l);

		%let first = %eval(&last + 1);
		%let last = %eval(&first + &BlockSize - 1);
		%let Block = %eval(&Block + 1);
		%if &last > &nummark %then %let last = &nummark;
	%end;
run;	
%let NBlocks = %eval(&Block-1);
%let NChunks=%eval((&NBlocks-1)/100 + 1);
%let ChunkSize = %sysfunc(min(100,&NBlocks)); /* Number of blocks in chunk */

%let first = 1; /* first marker in block */
%let last=&BlockSize; /* last marker in block */
%let Block = 1;	

%let skipmerge=0;
%if &NTrait=1 and %length(&ByVars)=0 %then %do;
	%let dsid=%sysfunc(open(&ds,i));
	%let trnum=%sysfunc(varnum(&dsid,&TraitVar));
	%let fmnum=%sysfunc(varnum(&dsid,&firstvar1));
	%let lmnum=%sysfunc(varnum(&dsid,&&lastvar&NBlocks));	
	%let rc=%sysfunc(close(&dsid));
	%if &trnum < &fmnum or &trnum > &lmnum %then %let skipmerge=1;
%end;	

%do c=1 %to &NChunks;

	/* Last block in current chunk */
	%let BinC = %sysfunc(min(%eval(&Block + &ChunkSize -1),&NBlocks));

	data %do b=&Block %to &BinC; avars&b(keep=&ByVars &TraitVar &&firstvar&b--&&lastvar&b) %end;;
		%if &skipmerge=0 %then %do;
			merge &ds(keep=&ByVars &TraitVar) &ds(keep=&AlleleVars);
		%end;
		%else %do;
			set &ds;
		%end;
	run;
	
	%do %while (&Block le &BinC);
	
		data missind;
			set avars&Block; 
			array _missind{&BlockSize};
			%if %length(&GenoCol) %then %do;
				array _old{*} &&firstvar&Block--&&lastvar&Block;
				do _i=1 to &BlockSize;
					if missing(_old{_i}) %if %upcase(&MarkerFmt)=GENOTYPES %then or _old{_i}="&GenoDelim"; then _missind{_i}=1;
			%end;
			%else %do;
				array _old{*} &&firstvar&Block--&&lastvar&Block;
				do _i=1 to &BlockSize;
					if missing(_old{2*_i-1}) or missing(_old{2*_i}) then _missind{_i}=1;
			%end;
					else _missind{_i}=0;
			end;
			drop &&firstvar&Block--&&lastvar&Block _i;
		run;

		proc datasets nolist;
			delete avars&Block;
		quit;

		%do i=1 %to &Ntrait;

			proc means data=missind noprint;
				var _missind:;
				class &&_tv&i;
				output out=Means sum= / autoname;
				%if %length(&ByVars) %then %do; 
					by &ByVars;
				%end;				
			run;

			proc transpose data=Means(where=(missing(&&_tv&i)=0)) out=misssum prefix=NMiss;
				var _missind:;
				id &&_tv&i;
				%if %length(&ByVars) %then %do; 
					by &ByVars;
				%end;			
			run;

			**start S1573196: removing by groups for which the trait variable has only one level to avoid error in proc casecontrol.**;			
			%if %length(&ByVars) %then %do;
				proc sort data=missind;
					by 
					%let j=1;
					%do %while(%length(%scan(&ByVars,&j))); 
							%scan(&ByVars,&j)
							%let j=%eval(&j + 1);
					%end;
					;
				run;

				proc freq data=missind noprint;
					tables &&_tv&i * 
					%let j=1;
					%do %while(%length(%scan(&ByVars,&j)));
							%if &j>1 %then %do;
								%str(*)
							%end; 
							%scan(&ByVars,&j)
							%let j=%eval(&j + 1);
					%end;
					/out=fby1
					;		
				run;

				data fby1;
					set fby1;
					where not missing(&&_tv&i);
				run;

				proc freq data=fby1 noprint;
					tables 
					%let j=1;
					%do %while(%length(%scan(&ByVars,&j)));
							%if &j>1 %then %do;
								%str(*)
							%end; 
							%scan(&ByVars,&j)
							%let j=%eval(&j + 1);
					%end;
					/out=fby2
					;		
				run;

				proc sort data=fby2;
					by COUNT;
				run;

				proc sql UNDO_POLICY= NONE;
					create table missind as
					select a.*, b.*
					from 
					missind as a left join fby2(drop=PERCENT rename=(count=_count_)) as b
					on 
					%let j=1;
					%do %while(%length(%scan(&ByVars,&j)));
						    %if &j>1 %then %do;
								%str(and)
							%end; 
							a.%scan(&ByVars,&j)=b.%scan(&ByVars,&j)
							%let j=%eval(&j + 1);
					%end;
					;
				quit;

				data missind;
					set missind;
					where _count_ > 1;
					drop _count_;
				run;
				
				%NObsVars(missind);
				%if &nobs = 0 %then %do;
					%put ERROR: Trait &&_tv&i cannot be analyzed because it has only one level.;
					%let exiterror=1;
					%goto exit;
				%end;
			%end;
			**end: S1573196**;

			%CheckBinaryChar(missind,&&_tv&i);

			options nolabel;
			%put Fitting trait &i of &Ntrait: &&_tv&i;
			/*data OutLib.mg_missind; set missind; run;*/
			proc casecontrol data=missind genocol genotype ndata=mapdata(rename=(&AnnoLabel=name) firstobs=&first obs=&last) perms=&Perms %if %length(&Seed) %then seed=&Seed;
											outstat=outccm(rename=(ChiSqGenotype=ChiSqMissing dfGenotype=dfMissing ProbGenotype=ProbMissing));
				%if %length(&ByVars) %then %do; 
					by &ByVars;
				%end;
				var _missind:;  
				trait &&_tv&i;
				format _missind: numtochar.;
			run;

			data outccm;
				merge outccm misssum(drop=_name_);
			run;

			options label;
			data outcc;
			 length Trait $32;
			 set outcc outccm(in=new);
			 if new then do;
					if ProbMissing=. then ProbMissing=1;
				*	NMissDiff = NMiss&cval2 - NMiss&cval1;
				*	NNonMiss&cval1 = NumTrait&cval1 - NMiss&cval1;
				*	NNonMiss&cval2 = NumTrait&cval2 - NMiss&cval2;
				*	NNonMissDiff = NNonMiss&cval2 - NNonMiss&cval1;	
					Total&cval1=NumTrait&cval1;
					Total&cval2=NumTrait&cval2;
					TotalMissing = NMiss&cval1 + Nmiss&cval2;

					TraitNum = &i;
					Trait="&&_tv&i";
					label NMiss&cval1 = "Number of Missing for Trait=&cval1";
					label NMiss&cval2 = "Number of Missing for Trait=&cval2";
			 end;
			 drop NumTrait:;
			run;		
			
			/*data OutLib.mg_outcc; set outcc; run;*/
		%end;
		
		%let first = %eval(&last + 1);
		%let last = %eval(&first + &BlockSize - 1);
		%if &last > &nummark %then %do;
			%let last = &nummark;
			%let BlockSize = %eval(&last - &first + 1);
		%end;
		%let Block = %eval(&Block + 1);		

	%end;
%end; /* end chunk */

%global JByAnno;

%if &NTrait>1 %then %do;
	%let TraitBy = Trait;
	%let JTraitBy = Trait;
%end;
%else %do;
	%let TraitBy=;
	%let JTraitBy=;
%end;
%let ByVars = &TraitBy &ByVars;
%let JByAnno=;

%if %length(&ByVars)>0 %then %do;
 %let JBy = %quote(&JTraitBy,&JBy);
 %if &NumGps>1 %then %let JByAnno = %quote(&AnnoGroup,&JBy);
%end;
%else %do;
 %let JBy = %quote(&JTraitBy);
 %if &NumGps>1 %then %let JByAnno = &AnnoGroup;
%end;
%if &NumGps=1 %then %let JByAnno = &JBy;

%let Nbyval = %eval(&Nbyval*&Ntrait);

%if (&syserr>4) %then %goto exit;

proc sort data=outcc;
 by Locus;
run;

%if (&syserr>4) %then %goto exit;

proc sort data=mapdata;
 by &AnnoLabel;
run;

data temp;
 merge mapdata outcc(in=k rename=(Locus=&AnnoLabel));
 by &AnnoLabel;
 label &AnnoLoc= ' ';
 if k;
run;
%let mgdata=temp;
	
proc sort data=temp out=&MGS_OutData;
 by &ByVars &AnnoGroup &AnnoLoc;
run;

%Psmooth(Missing, &MGS_OutData, &MGS_OutDataName);
%if (&syserr>4) %then %goto exit;

%if &NByVal > 1 %then %let ssr=0;
%else %let ssr=1;
%JSLPvalMap(Missing Genotype by Trait, &MGSScript, &MGSScriptName, &MGS_OutDataName, _subsetdd=&ssr);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend MissingGenotypeSummary;

* run the macro;
%MissingGenotypeSummary;
