/*
JMP Genomics Nanostring Expression Input Engine


*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

OPTIONS nospool nolabel;

*ProcessVariables;

%global StudyName NewStudyName DataInPath ExpInFile  FileType TransMethod OutData ExpData AnnoData 
ExpDataPrefix OutPath ServerOutPath MacroPath QCProbeOutData ExpOutData OutDataWide OutDataAnno
ShiftFactor ImagingQC BindingQC PositiveQC DetectionQC /*NormMethod*/ scalingfactor RmControl;

*ProcessBody;

%macro NanostringInputEngine;

*initialization;
*options mprint;
%let ProcessName = NanostringInputEngine;
%global FullAPName _mainjslfile_;
%let FullAPName=%str(Nanostring Input Engine);

%include "&MacroPath./UtilityMacros.sas" / nosource;

%include "&MacroPath./InputEngineMacros.sas" / nosource;
%let ColumnNameAlias = ColumnName;

* create output libaray;
libname OutLib "&OutPath";

%let ExpType = ;
%CheckFileType(&ExpInFile,ExpType);

* Check if &OutData and &ExpOutData are valid SAS data set names;
%PathName(&ExpInFile);
%put ExpType= &ExpType;
%CheckEndSlash(&DataInPath);
%checkdataname(_name=ExpOutData,_default=&tmpname,_SDS=1);
%checkdataname(_name=OutData,_default=%trim(&ExpOutData)_data,_SDS=1);

%if %length(&ExpOutData) >27 %then %let qcpbName = %substr(&ExpOutData,1,27);
%else %let qcpbName =&ExpOutData;
%checkdataname(_name=QCProbeOutData,_default=%trim(&qcpbName)_qcpb,_SDS=1);
%checkdataname(_name=OutDataWide,_default=%trim(&qcpbName)_wide,_SDS=1);
%checkdataname(_name=OutDataAnno,_default=%trim(&qcpbName)_anno,_SDS=1);


%if %upcase(&ExpType) = SAS %then %do;
	%PathName(&ExpInFile);
	libname tmp "&TmpPath";
	libname library "&TmpPath";
	%if "%sysfunc(pathname(Tmp))" ne "%sysfunc(pathname(OutLib))" |
		&ExpOutData ne &TmpName %then %do;
		data OutLib.&ExpOutData;
			set tmp.&TmpName;
		run;
	%end;
%end;
%else %do;
	* Check Excel file and OS system;
	%CheckExcel(&ExpType,the Experimental Design Table);
	%if &exiterror %then %goto exitdesignparse;
	***** Added optional delimiter statement to handle bogus end-of-line characters;
	%if %upcase(&ExpType) = CSV %then %let delimiter = DELIMITER='2C0D'x;
	%else %if %upcase(&ExpType) = DLM %then %let delimiter = DELIMITER='200D'x;
	%else %if %upcase(&ExpType) = TAB %then %let delimiter = DELIMITER='090D'x;
	%else %let delimiter = ;
	%if &ExpType = EXCEL2007 %then %do;
		%if %index(&SYSSCPL, X64) %then %do;
			PROC IMPORT OUT=OutLib.&ExpOutData 
				datafile= "&ExpInFile"
				dbms=xlsx replace;
				GETNAMES=YES;
			RUN;
		%end;
		%else %do;
			PROC IMPORT OUT=OutLib.&ExpOutData 
				datafile= "&ExpInFile"
				dbms=EXCEL replace;
				GETNAMES=YES;
			RUN;
		%end;
	%end;
	%else %do;
		PROC IMPORT OUT=OutLib.&ExpOutData 
			DATAFILE="&ExpInFile"
			DBMS=&ExpType REPLACE;
			GETNAMES=YES;
			&delimiter;
		RUN;
	%end;
%end;

%let CheckVar = &ColumnNameAlias;
%CheckExpVarExist(&CheckVar,OutLib.&ExpOutData,A column with name);
%if &exiterror = 1 %then %do; 
   %goto exit; 
%end;

* check if all the files exist;
%let tid = %sysfunc(open(OutLib.&ExpOutData));
%let nObs = %sysfunc(attrn(&tid,NOBS));
%let nVars = %sysfunc(attrn(&tid,NVARS)); 
%do i = 1 %to &nVars;
	%let CK_Var&i = %sysfunc(varname(&tid,&i)); 
	%if %upcase(&&CK_Var&i) = FILE | %upcase(&&CK_Var&i) = FILENAME %then
   %let FileAlias = &&CK_Var&i;
%end;
%let tid = %sysfunc(close(&tid));

data ETable;
   set OutLib.&ExpOutData;
   if &FileAlias ne '';
run;
* extract file names;
data _null_;
   set ETable end=_e_;
   if _e_ then call symput('nRec',_n_);
   name = 'File'||trim(left(_n_));
   call symput(name,&FileAlias);
run;
%if %sysfunc(exist(OutLib.&OutData)) %then %do;
   proc datasets library=OutLib nolist; 
      delete &OutData; 
   quit;
%end;
%if %sysfunc(exist(OutLib.&QCProbeOutData)) %then %do;
   proc datasets library=OutLib nolist; 
      delete &QCProbeOutData; 
   quit;
%end;
%nobsvars(ETable);
%let nChunk = 1;
%let nCELInChunk = 0;
%let ChunkSize=300;
%let VarLabelRow=0;

%let nRawFiles = &nobs;
%let ScanRows=1000;
%let DataType=CSV;
%let DLT = '2C0D'x;
%let VarNameRow=0;
%let DataStartRow=%eval(&VarNameRow+1);
%let posSum=0;
%let negSum=0;
%let geoSum=0;

%do iArray = 1 %to &nRawFiles;  * loop through each raw file;

	%global Arrayname&iArray BindingDensity&iArray FovCount&iArray FovCounted&iArray Imaging&iArray PositiveQC&iArray geoScale&iArray posScale&iArray;

	* get filename and columnname one by one;
	data _null_;
      set ETable;
      if _n_ = &iArray;
      put "Start to import column number: " _n_; 
      call symput('Tab',trim(left(&FileAlias)));
      call symput('ColName',trim(left(&ColumnNameAlias)));
   run;
	%let DataFile = &DataInPath.&Tab; 
	%let Arrayname&iArray=&ColName;

	%let _Skip = 0;
	
	* check if the raw file exist;
	%if ^%sysfunc(fileexist(&DataFile)) %then %do;
		%let _Skip = 1;
		%put ERROR: &DataFile does not exist.;
   %end;
   %if ^&_Skip %then %do;
   	%let DataRow = -1;
   	%let BindingDensity = -1;
   	%let FovCounted = -1;
   	%let FovCount = -1; 
   
   	*get data start row;
   	data _null_;
   		infile "&DataFile" TRUNCOVER EXPANDTABS lrecl=32767 firstobs=1;
   		LENGTH var1 $ 256;
   		input var1 1-256;
   		var1 = upcase(var1);
   		if index(var1,"<CODE_SUMMARY>") ^= 0 then call symput('DataRow',trim(left(_n_)));
   		if index(var1,"BINDINGDENSITY") ^= 0 then call symput('BindingDensity',trim(left(var1)));
   		if index(var1,"FOVCOUNTED") ^= 0 then call symput('FovCounted',trim(left(var1)));
   		if scan(var1,1,',')="FOVCOUNT" then call symput('FovCount',trim(left(var1)));
   	run;
   	%if &DataRow = -1 | &BindingDensity = -1 | &FovCounted = -1 | &FovCount = -1 %then %do;
   		%put ERROR: &DataFile is not a valid RCC file.;
		%let exiterror = 1;
		%goto exit;   		
   	%end;
   	
   	%let VarNameRow=%eval(&DataRow+1);
   	%let DataStartRow=%eval(&VarNameRow+1);
  		%put VarNameRow=&VarNameRow DataStartRow=&DataStartRow &BINDINGDENSITY &FovCounted &FovCount; 
  		
  		* calculate bindingdensity QC, Image QC;
  		%let BindingDensity&iArray=%scan(%nrbquote(&BindingDensity),2,',');
  		%let FovCount&iArray=%scan(%nrbquote(&FovCount),2,',');
  		%let FovCounted&iArray=%scan(%nrbquote(&FovCounted),2,',');
  		%*put FovCount&iArray= &&FovCount&iArray FovCounted&iArray=&&FovCounted&iArray;
  		
  		%let FovQC&iArray=%sysfunc(round(%sysevalf(&&FovCounted&iArray/&&FovCount&iArray),0.01));
  		%*put FovQC&iArray=&&FovQC&iArray;
  		
  		%if &iArray = 1 %then %do;
   		%let _FileFormat_ = ;
   		%GetFileFormat(&DataFile,_FileFormat_);
   		
   		* get variable name;
   		%ExtractColumnHeader_Text(TextFile=&DataFile,FileFormat=&_FileFormat_,
			 	Delimiter=&DLT,VarNameRow=&VarNameRow,vName_OutData=_vname_);
      	%if &exiterror %then %goto exit;
      	%naming(_Dset=_vname_,_AllVars=vTmp);
			
			data _null_;
				set _vname_ end=_e_;
				name='TmpName'||trim(left(_n_));
				call symput(name,vTmp);
				if _e_ then call symput('_nVars_',_n_);
			run;

         proc import out=table
         	datafile= "&DataFile"
         	dbms=&DataType replace;
         	GETNAMES=no; 
         	DataRow = &DataStartRow; 
         	GUESSINGROWS=&ScanRows;
         run;
			%if &syserr >= 4 %then %do;
				%put ERROR: Imported data is suspect.  It may help to increase the number of scanning rows.;
				%if &_exitwitherror=1 %then %do;
					%let exiterror = 1;
					%goto exit;
				%end;
			%end;  
			%let tid = %sysfunc(open(table));
			%let nv_OutSet = %sysfunc(attrn(&tid,NVARS)); 
			%do i = 1 %to &nv_OutSet;
				%let _name&i = %sysfunc(VARNAME(&tid,&i));
				%put _name&i =&&_name&i ;
			%end;
			%let tid = %sysfunc(close(&tid));
			
			%let nMax = %sysfunc(min(&nv_OutSet,&_nVars_));
			%let _lib = work;
      	%let _dset = table;
			proc datasets lib=&_lib nolist;
				modify &_dset;      
				%do i = 1 %to &nMax;
					%if &VarNameRow %then %do;
						%if (&&_name&i ne _V&i)  %then %do;
               	   label &&_name&i = "&&TmpName&i";
               	   rename &&_name&i = _V&i;
 						%end;
					%end;
				%end; 
			quit;
			proc datasets lib=&_lib nolist;
				modify &_dset;      
				%do i = 1 %to &nMax;
					%if &VarNameRow %then %do;
						%if (_V&i ne &&TmpName&i) %then %do;
							rename _V&i = &&TmpName&i;
						%end;
					%end;
				%end; 
			quit;
			%let tid = %sysfunc(open(table));
			%let nv_OutSet = %sysfunc(attrn(&tid,NVARS)); 
			%let _maxLen=0;
			%do i = 1 %to &nv_OutSet;
				%let _name&i = %sysfunc(VARNAME(&tid,&i));
				%let _len&i = %sysfunc(VARLEN(&tid,&i));
				%let _type&i = %sysfunc(VARTYPE(&tid,&i));
				%let _vnameLen&i=%length(&&_name&i);
				%if &&_len&i > &_maxLen %then %let _maxLen=&&_len&i; 
				%put _name&i=&&_name&i _vnameLen&i=&&_vnameLen&i _maxLen=&_maxLen _type&i=&&_type&i _len&i=&&_len&i;
			%end;
			%let tid = %sysfunc(close(&tid));
			data VarListTable;
				length VarLabel $&_maxLen.;
				%do i = 1 %to &nv_OutSet;
					%if &&_type&i = C %then %do;
						VarLabel="&&_name&i $";
					%end;
					%else %do;
						VarLabel="&&_name&i";
					%end;
					LenList=&&_len&i;
					output;
				%end;
			run;     			
   	
   	%end;
   	%else %do;
			data _null_;
				set VarListTable  end=_e_;
				name = "VarList" || trim(left(_n_));
				nameL = "LenList" || trim(left(_n_));
				call symput(name,VarLabel);
				call symput(nameL,LenList);
				if _e_ then call symput("nVVar",_n_);
			run;
			* length can range from 3 to 8 under UNIX because of the IEEE floating-point format;
			%if &Server ne Local %then %do;
				%do i =1 %to &nVVar; 
					%if &&LenList&i < 3 %then %let LenList&i =3;
				%end;
			%end;
			data Table;
				infile "&datafile" DELIMITER=&DLT MISSOVER DSD lrecl=32767 firstobs=&DataStartRow;
				length  %do i =1 %to &nVVar; &&VarList&i &&LenList&i... %end;;
				input %do i =1 %to &nVVar; &&VarList&i %end;;
			run;
		%end; 
		 /* import table formated files */
		%let nCELInChunk = %eval(&nCELInChunk+1);
		%if (&nCELInChunk > &ChunkSize) %then %do; 
         %let nChunk = %eval(&nChunk + 1);
         %let nCELInChunk = %eval(&nCELInChunk-&ChunkSize);
      %end;

		data Table;
			set Table;
			if CodeClass="" then delete;
			if index(CodeClass,"<") ^=0 then delete;
		run;
      data TmpTable&iArray;
         set Table;
			&ColName=Count ;
			drop Count;
      run;
		%if &DetectionQC=yes %then %do;
			*get negative genes first;
			data negative;
				set TmpTable&iArray;
				if index(CodeClass, "Negative") =0 then delete;
			*	drop CodeClass Name Accession;
			run;
			*calculate the average of the negative controls for each sample;
			proc summary data=negative;
				var &ColName;
				output out=_n2;
			run;

			* collect the mean value for each sample;
			data _null_;
				set _n2;
				if _STAT_="MEAN" then call symput ('negMean',trim(left(&ColName)));
				else if _STAT_="STD" then call symput ('negStd',trim(left(&ColName)));
			run;

			%let negMean&iArray=&negMean;
			%let negSum = %sysevalf(&negSum + &negMean);
			%let LOD&iArray= %sysevalf(&&negMean + 2*&negStd);
		%end;
		
	*normalization positive control;
	/*
	%if %index(&NormMethod,Positive) %then %do;
		
		data positive;
			set TmpTable&iArray;
			if index(CodeClass, "Positive") =0 then delete;
		run;
	
		proc summary data=positive;
			var &ColName;
			output out=_p2 sum=;
		run;
	
		data _p3;
			set _p2;
			drop _TYPE_ _FREQ_;
		run;

		
		data _null_;
			set _p3;
			call symput ('posMean',trim(left(&ColName)));
		run;
	
		%let posMean&iArray=&posMean;
		%let posSum = %sysevalf(&posSum + &posMean);
		
	
		data log_transformed;
			set positive;
			&ColName=log( &ColName);
		run;
		proc means data=log_transformed noprint;
			var &ColName;
			output out=mean_log mean=mean_log_x;
		run;
		data geomean;
			set mean_log;
			geomean_x = exp(mean_log_x);
			call symput ('geomean',geomean_x);
		run; 
		%let geoMean&iArray=&geomean;
		%let geoSum = %sysevalf(&geoSum + &geomean);

	%end;
	*/
		* collect quality control information;
		%if &ImagingQC = yes | &BindingQC=yes | &PositiveQC=yes | &DetectionQC=yes %then %do;
			
			data qc&iArray;				
				length Columnname $ 30 ;
				%if &BindingQC = yes %then %do;
					BindingDensity=&&BindingDensity&iArray;
				%end;
				%if &ImagingQC = yes %then %do;
					ImagingQC=&&FovQC&iArray;
				%end;
				%if &DetectionQC=yes %then %do;
					DetectionQC=&&LOD&iArray;
				%end;
				Columnname ="&ColName";
			run;
		%end;
		
      * merge within a chunk;
      %if (&nCELInChunk = &ChunkSize) | (&iArray = &nRawFiles) %then %do;
      	%ClockStart(Merge Chunk &nChunk);
      	%let CelInChunk = ;
      	%do i = 1 %to &nCelInChunk; 
         	%let j = %eval((&nChunk-1)*&ChunkSize+&i);
         	%if &i =1 %then %do;
         		data Chunk&nChunk;
         			set TmpTable&j;
         		run;
         		%if &ImagingQC = yes | &BindingQC=yes | &PositiveQC=yes | &DetectionQC=yes %then %do;
         			data QCT&nChunk;
         				set qc&j;
         			run;
         		%end;
         	%end;
         	%else %do;
         		proc sort data =Chunk&nChunk;
         			by Accession;
         		run;
          		proc sort data =TmpTable&j;
         			by Accession;
         		run;        		
         		data Chunk&nChunk;
         			merge Chunk&nChunk TmpTable&j;
         			by Accession;
         		run;
         		%if &ImagingQC = yes | &BindingQC=yes | &PositiveQC=yes | &DetectionQC=yes %then %do;
         			data QCT&nChunk;
         				set QCT&nChunk qc&j;
         			run;
         		%end;
         	%end;
      	%end;
         %if &nChunk = 1 %then %do;
         	%let ChunkList = Chunk1; 
         	%let ChunkList2 = QCT1; 
         %end;
         %else %do;
         	%let ChunkList = &ChunkList Chunk&nChunk;
         	%let ChunkList2 = &ChunkList2 QCT&nChunk;
         %end;
      	%ClockStop(Merge Chunk &nChunk);
      %end;
      
   %end;
%end;/*end of looping through files*/

data OutLib.&OutData;
   %let i = 1;
   %do %while(%length(%scan(&ChunkList,&i)));
      set %scan(&ChunkList,&i);
      %let i = %eval(&i+1);
   %end;
run;

*normalization positive control;
/*
%if %index(&NormMethod,Positive) %then %do;
	%do i=1 %to &nRawFiles;
		%let posScale&i=%sysfunc(round(%sysevalf(&posSum/(&&posMean&i*&nRawFiles)),0.01));
		%*put posSum=&posSum posMean&i=&&posMean&i posScale&i = &&posScale&i ;
		%let geoScale&i=%sysfunc(round(%sysevalf(&geoSum/(&&geomean&i*&nRawFiles)),0.01));
		%*put geoScale&i=&&geoScale&i posScale&i=&&posScale&i;
	%end;
	
	%if %index(&scalingfactor,Geometric) %then %do;
		data OutLib.&OutData;
			set OutLib.&OutData;
			%do i=1 %to &nRawFiles; 
				&&Arrayname&i=&&Arrayname&i*&&geoScale&i;
			%end;
		run;
	%end;
	%else %do;
		data OutLib.&OutData;
			set OutLib.&OutData;
			%do i=1 %to &nRawFiles; 
				&&Arrayname&i=&&Arrayname&i*&&posScale&i;
			%end;
		run;
	%end;
%end;
*/

%if &ImagingQC = yes | &BindingQC=yes | &PositiveQC=yes | &DetectionQC=yes %then %do;
	data OutLib.&QCProbeOutData;
		set &ChunkList2;
	run;
%end;
%RemoveData(&ChunkList); 

* calculate correlation coefficient;
data correla;
	set OutLib.&OutData;
	if index(CodeClass, "Positive") =0 then delete;
	constr =scan(scan(Name,2,'('),1,')');
	concentration=constr+0;
	drop CodeClass Name Accession constr;
run;

/*data correla;
	set correla;
	%do i=1 %to &nRawFiles;
		&&Arrayname&i=log2(&&Arrayname&i);
	%end;
run;*/

proc sort data= correla;
	by Concentration;
run;
options noprint;
proc corr data =correla pearson out=corr noprint;
	var %do i=1 %to &nRawFiles; &&Arrayname&i %end; ;
	with Concentration;
run;

*transform the correlation table;
proc transpose data = corr out = corr1;
	var %do i=1 %to &nRawFiles; &&Arrayname&i %end; ;
run;

data corr1;
	set corr1;
	ColumnName=_Name_;
	rename concentration=PositiveControlQC;
	drop _name_;
run;

proc sort data=corr1;
	by ColumnName;
run;

%if &ImagingQC = yes | &BindingQC=yes | &PositiveQC=yes | &DetectionQC=yes %then %do;
	proc sort data=OutLib.&QCProbeOutData;
		by ColumnName;
	run;

	*merge correlation coefficient qc to the qc table;
	data OutLib.&QCProbeOutData;
		merge OutLib.&QCProbeOutData corr1;
	run;
	proc sort data=OutLib.&ExpOutData;
		by ColumnName;
	run;
	proc sort data=OutLib.&QCProbeOutData;
		by ColumnName;
	run;
	data OutLib.&ExpOutData;
		merge OutLib.&ExpOutData OutLib.&QCProbeOutData;
		by ColumnName;
	run;
%end;

%if %upcase(&RmControl) = YES %then %do;
	
	data OutLib.&OutData;
	modify OutLib.&OutData;
	if find(upcase(Name), "POS_") then remove;
	if find(upcase(Name),"NEG_") then remove;
	run;
%end;

*log2 transformation;
%if %upcase(&TransMethod) = LOG2 %then %do;
	data OutLib.&OutData;
		set OutLib.&OutData;
		%do i=1 %to &nRawFiles;
			%if &&Arrayname&i >0 %then %do;
				&&Arrayname&i =log2(&&Arrayname&i);
			%end;
			%else %do;
				&&Arrayname&i =.;
			%end;
		%end;
	run;
%end;
%else %if %upcase(&TransMethod) = SHIFTED LOG2 %then %do;
	data OutLib.&OutData;
		set OutLib.&OutData;
		%do i=1 %to &nRawFiles;
			%if &&Arrayname&i >0 %then %do;
				&&Arrayname&i =log2(&&Arrayname&i+&ShiftFactor);
			%end;
			%else %do;
				&&Arrayname&i =.;
			%end;
		%end;
	run;
%end;


* create a wide data set for reference normalization;
%let in_the_house = 0;
data h;
	length WideName $ 32;
	label WideName = "Wide Name";
	set outlib.&outdata;
	if upcase(substr(codeclass,1,5)) = "HOUSE" then do;
		WideName = cats("HOUSE_",name);
		call symputx("in_the_house",1);
	end;
	else WideName = name;
run;

%naming(_dset=h, _allvars = WideName,_uniquerow=1);

/* Create SAS valid name and label var for wide transpose and include in anno*/
data Outlib.&OutDataAnno;
	set h;
	OrigNameOrder = _n_;
	drop _: vlabel %do i=1 %to &nRawFiles; &&Arrayname&i %end;;
run;

proc transpose data=h out=o;
	var %do i=1 %to &nRawFiles; &&Arrayname&i %end;;
	id WideName;
	idlabel name;
run;

data o;
	set o;
	label _NAME_ = ColumnName;
	rename _NAME_ = ColumnName;
run;

* rearrange columns so special ones are first;
data o1;
	set o;
	keep columnname house_:;
run;

data o2;
	set o;
	keep neg_:;
run;

data o3;
	set o;
	keep pos_:;
run;

data o4;
	set o;
	drop columname house_: neg_: pos_:;
run;

data Outlib.&OutDataWide;
	merge o1 o2 o3 o4;
run;


%PackageInsertDataset(Experimental design data from &ProcessName,outlib,&ExpOutData,Yes,_type=EDDS);
%PackageInsertDataset(Intensity data from &ProcessName,outlib,&OutData,Yes,_type=tall);
%if &ImagingQC = yes | &BindingQC=yes | &PositiveQC=yes | &DetectionQC=yes %then %do;
	%PackageInsertDataset(Quality control data from &ProcessName,outlib,&QCProbeOutData,Yes,_type=tall);
%end;
%PackageInsertDataset(Intensity wide data from &ProcessName,outlib,&OutDataWide,Yes,_type=wide);
%PackageInsertDataset(Annotation data from &ProcessName,outlib,&OutDataAnno,Yes,_type=Anno);


* create JSL file;
%global IEButtonsScript;
%let IEButtonsScript = ;
%let jslfname=NanostringLaunchScript_&OutData;
%PrepOutFile(IEButtonsScript,IEButtonsScriptName,&jslfname..jsl);
filename jslbtns "&IEButtonsScript";

%global _iewindow_ _mainjslfile_;
%let _iewindow_=iebuttonswindow;
%let _mainjslfile_ = %quote(&IEButtonsScript);

data _null_;
	file jslbtns;
	put "//!";
	put "Names Default to Here(1);";
	put "wait(0.5);";
	put "if(host is(Windows),";
	put " include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
	put " include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
	put " ";
	put "iebuttonswindow=Function({},{default local},";
	put "    VListBox(";
	put "       TextBox(""Click on a button below to launch the Nanostring Normalization Process using these data sets.""),";
	put "       TextBox("" ""),";
	put "       ButtonBox(""Reference Gene Normalization"",";
   put "          if(host is(Windows),";
   put "             Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/ReferenceNormalization.jsl"", absolute, windows)),";
   put "             Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/ReferenceNormalization.jsl"", absolute, POSIX) ));";
   put " ";  
   put "          jg:setCurrentParmValue(2, ""ReferenceNormalization"", jg:getParmNum(""ReferenceNormalization"",""InData""),""&OutPath.&OutDataWide..sas7bdat"");";
   put "          jg:setCurrentParmValue(2, ""ReferenceNormalization"", jg:getParmNum(""ReferenceNormalization"",""IDVar""),""ColumnName"");";
   %if &in_the_house %then %do;
   	put "          jg:setCurrentParmValue(2, ""ReferenceNormalization"", jg:getParmNum(""ReferenceNormalization"",""ListVars""),""house_:"");";
   %end;
   put "          jg:setCurrentParmValue(2, ""ReferenceNormalization"", jg:getParmNum(""ReferenceNormalization"",""OutPath""),""&ClientOutPath"");";
   put "          jg:setCurrentParmValue(2, ""ReferenceNormalization"", jg:getParmNum(""ReferenceNormalization"",""AnnoInData""),""&OutPath.&OutDataAnno..sas7bdat"");";
   put "	         jg:setCurrentParmValue(2, ""ReferenceNormalization"", jg:getParmNum(""ReferenceNormalization"",""AnnoNameVar""),""Name"");";
   put "          jg:ReferenceNormalization_updateDeps;";
	put "       )";
   put "    )";
   put ");";     
   put "Here:LaunchCode = iebuttonswindow();";
   put "Here:DataList = {};";
 run;
  
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &IEButtonsScriptName, "&OutDataWide..sas7bdat", text/plain, yes);
%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

options label;
%mend;
%NanostringInputEngine;


