/*
JMP Genomics Analytical Process

Name: PCA for Population Stratification
Author: Wendy Czika, SAS Institute Inc.
Support: Wendy Czika, SAS Institute Inc.
History:
   wehass 01Jul07 initial coding  
   kemicl 15Sep08 added Annotation Null SNP Filter 

*/

/*
-----------------------------------------------------------
Copyright (C) 2007 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData ContinuousVars TraitVar ListContinuousVars ListTraitVars WhereStmt
ColorVar LabelVar ToKeepVars PCAData NumberOfComponents ACCProportion DisplayPlots MergePCA TraitType Odds EigenCorr PCAlpha MultipleTestingMethod
AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoAccession AnnoWhereStmt NullSNPWhereStmt 
Conv MTC PValAdj Tau Bandwidth Alpha OnlySig OutPath ServerOutPath OutName MacroPath PCA_OutData PCA_OutData2 PCA_OutDataM PCAScript PCAScript2;


*ProcessBody;

%macro PCAforPopulationStratification;

*initialization and argument checking;

%global PredictorVars ClassVars ClassPredVars ListClassPredVars;

/* Initialization */
%include "&MacroPath./GenMacros.sas" / nosource;
%include "&MacroPath./PLS_PCA.sas";

%global MarkerFmt;
%let MarkerFmt =;
%GenInit(PCAforPS,28);
%if &exiterror=1 %then %goto exit;

%global FullAPName;
%let FullAPName = %str(PCA for Population Stratification);

%let TraitVar = &TraitVar &ListTraitVars;
%if %length(&TraitVar) and %length(&TraitType)=0 %then %do;
	%put ERROR: You must select a Type of Trait when Trait Variables are specified.;
	%let exiterror=1;
	%goto exit;
%end;

%let ByVars = ;

%global OldAlleleVars ListMarkerVars;
%let OldAlleleVars = &ContinuousVars;
%let ListMarkerVars = &ListContinuousVars;
%let AlleleVars = &ContinuousVars &ListContinuousVars;

%if %length(&AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;
%if %length(&PCAData) %then %do;
	%if %length(&TraitVar)=0 %then %do;
 		%put ERROR: Trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters when the PCA Data Set is specified.;
 		%let exiterror=1;
 		%goto exit;
 	%end;
%end;	

%if &NumberofComponents le 0 %then %do;
	%put ERROR: The Maximum Number of Principal Components must be positive.;
	%let exiterror=1;
	%goto exit;
%end;

%let ClassPredVars = ;
%let ListClassPredVars = ;
%let ContinuousVars = ;
%let ClassVars = ;
%global CenterCols CenterRows CenterMult ScaleCols ScaleRows;
%let CenterCols=Yes;
%let CenterRows=No;
%let CenterMult=No;
%let ScaleCols=No;
%let ScaleRows=No;
%global Plot Include3D ProcPLSOptions;
%let Plot=%str(Row Scores Only);
%let Include3D=Yes;
%let ProcPLSOptions=;

/*---output data set names---*/
%PrepOutDS(PCA_OutData,PCA_OutDataName,&OutName,_pca);
%PrepOutFile(PCAScript,PCAScriptName,&ProcessName Script for &OutName..jsl);
%if %upcase(&MergePCA)=YES %then %PrepOutDS(PCA_OutDataM,PCA_OutDataMName,&OutName,_pcm);

/* Use XLVar stuff in %GenSubset for Null SNP filter */
%if %length(&NullSNPWhereStmt) %then %do;
	%global XLVarStmt XLVars;
	%let XLVarStmt = &NullSNPWhereStmt;
%end;

/* Set up annotation data set and output data sets if Trait Var*/
%if %length(&TraitVar)>0 or %length(&NullSNPWhereStmt) %then %do;	
	%let GenoCol=1;
	%PrepMapandBy;
	run;
	%if &exiterror=1 %then %goto exit;
%end;
%else %let ds=InLib.&InData;

%if %length(&TraitVar)>0 %then %do;
	%PrepOutDS(PCA_OutData2,PCA_OutDataName2,&OutName,_pps);
	%PrepOutFile(PCAScript2,PCAScriptName2,&ProcessName Script for &OutName..jsl);
%end;

%if %length(&NullSNPWhereStmt) %then %do;
	%if %length(&XLVars)=0 %then %do;
		%put ERROR: The Filter to Select Null SNPs resulted in no null SNPs.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let NullSNPs = &XLVars;
	%let PredictorVars = &NullSNPs;
	%let PredVarsforAssoc = &AlleleVars &NullSNPs;
%end;
%else %do;
	%let PredictorVars = &AlleleVars;
	%let NullSNPs=;
%end;

%global ds_mg;
%if %length(&WhereStmt) %then %do;
	data ds_sub;  
		set InLib.&InData;
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
	run; 
	%let ds_mg= ds_sub; 
%end;
%else %do;
	%let ds_mg= InLib.&InData;
%end;

%if %length(&PCAData) %then %do;
	%global PCAIncludeDS;
	%let PCAIncludeDS =;
	%goto skip;
%end;

/*---Proc Princomp for calculating principal components / eigenvectors---*/
%let ProcessName = PrincipalComponentsAnalysis;
%global prefix;
%let prefix=pca;

%let ExpInData=;
%let ed = 0;

%PLS_PCA(PCA_OutData,PCA_OutDataName,PCAScript,PCAScriptName,_forps=1);
%let ProcessName = PCAforPS;
%if &exiterror=1 %then %goto exit;

/*----------------------------------------------------------------------------------------------------*/

%skip:

%if %length(&TraitVar)>0 %then %do;
	%ClockStart(Association testing using PCs);
	%include "&MacroPath./SNPTraitAssocMacro.sas" / nosource;
	
	options compress=binary label;

	%if %length(&NullSNPs) %then %let PredictorVars = &PredVarsforAssoc;
	data d_o(keep=&PredictorVars) nonpredvars(drop=&PredictorVars);  
		set &ds;
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
	run; 
	
	%Nobsvars(d_o);
	%if &syserr>4 %then %goto exit;	
	
	%if %length(&PCAData) %then %do;
		%PathName(&PCAData);
		%let PCAData = &TmpName;
		libname PCALib "&TmpPath";	
		
		%let dsid=%sysfunc(open(PCALib.&PCAData,i));
		%if &dsid=0 %then %do;
			%put ERROR: The specified PCA data set does not exist.;
			%goto exit;
		%end;		
	  %let cnnum=%sysfunc(varnum(&dsid,OldColName));
	  %if &cnnum>0 %then %let ColNameVar=OldColName;
	  %else %let ColNameVar=ColumnName;
	  %let pcanum=%sysfunc(varnum(&dsid,pca1));
	  %if &pcanum=0 %then %do;
	  	%put ERROR: Variable pca1 not found in the specified PCA data set.;
	  	%goto exit;
	  %end;
	  %let nPCAobs = %sysfunc(attrn(&dsid,NOBS));
	  %let rc=%sysfunc(close(&dsid));
	  
	  %if &nPCAobs ne &nobs %then %do;
	  	%put ERROR: There are a different number of observations in the input data set and PCA data set.;  
	  	%put Be sure you use the same Filter to Include Observations that was used in the run that created the PCA data set.;
	  	%goto exit;
	  %end;
	%end;
	
	%if %length(&ACCProportion)=0 %then %let ACCProportion=0;
	%if &ACCProportion>0 %then %do;
		%let propmet = 0;
		%let prevNofC = &NumberofComponents;
		
		data _null_;
			set OutLib.&EigData; 
			if Cumulative >= &AccProportion then do;
				call symputx('NumberofComponents',_n_);
				call symputx('propmet',1);
				stop;
			end; 
		run;		
		%if &propmet=0 %then %do;
			%put NOTE: The value specified for the cumulative proportion of variance to explain with principal components was not met.  The maximum number of principal components (&NumberofComponents) is used.;
		%end;
		%else %do;
			%put NOTE: The Number of Principal Components used to adjust the association tests is &NumberofComponents according to the cumulative proportion of variation specified.;
		%end;
	%end;
	
	data xwt;
		%if %length(&PCAData)=0 %then %do;
			set xwt;
			%if &foundcn>0 %then %do;
				_ind=substr(OldColName,2)+0;
			%end;
			%else _ind=substr(ColumnName,2)+0;;				
		%end;
		%else %do;
			set PCALib.&PCAData;
			_ind=substr(&ColNameVar,2)+0;		
		%end;
		keep pca1-pca&NumberofComponents _ind;		
	run;
	
	%if %length(&PCAData) %then %do;
		%let dsid=%sysfunc(open(xwt,i));
		%let nPCAvars = %sysfunc(attrn(&dsid,NVARS));
		%let rc=%sysfunc(close(&dsid));	
		
		%if %eval(&nPCAvars-1) < &NumberofComponents %then %let NumberofComponents=%eval(&nPCAvars-1);
	%end;	
	
	proc sort data=xwt out=xwt(drop=_ind);
		by _ind;
	run;
	
	options label;
	
	data numtrts;
		set &ds(obs=1 keep=&TraitVar);
	run;
	%Nobsvars(numtrts);
	
	%if &nvars > 1 and %upcase(&EigenCorr)=YES %then %do;
		%put ERROR: The EigenCorr method can only be performed when there is only one Trait Variable specified.  This option is ignored.;
		%let EigenCorr = No;
	%end;
	
	%if %upcase(&MergePCA)=YES and %upcase(&EigenCorr) ne YES %then %do;
		
		/*
		data OutLib.&PCA_OutDataMName;
			set nonpredvars;
			set xwt;
			set d_o;
		run;
		*/
		
		data OutLib.&PCA_OutDataMName;
			set &ds_mg;
			set xwt;
		run;		
		
		%PackageInsertDataset(Merged data from &ProcessName, OutLib, &PCA_OutDataMName);
		%let ds=OutLib.&PCA_OutDataMName;
	%end;
	%else %do;
		data mgdata;
			set nonpredvars;
			set xwt;
			set d_o;
		run;
		%let ds = mgdata;	
	%end;
		
	/* Only do this if single trait */
	%if %upcase(&EigenCorr)=YES %then %do;
		
		%let corrds = &ds;
		%let CorrTraitVar = &TraitVar;
	
		/* If character binary trait, convert to numeric */
		%let dsid=%sysfunc(open(&ds,i));
		%let trtnum = %sysfunc(varnum(&dsid,&TraitVar));
		%if (%sysfunc(vartype(&dsid,&trtnum)) = C) %then %do;
			%let tlen=%sysfunc(varlen(&dsid,&trtnum));
			%let rc=%sysfunc(close(&dsid));
			%CheckBinaryChar(&ds,&TraitVar,_len=&tlen);

			%if &nvalue=2 %then %do;
				%let tnum=%substr(%trim(&TraitVar),1,28)_num;
				%put Converting &TraitVar to numeric variable &tnum for EigenCorr, &cval1=0 &cval2=1.;
				data dsnum;
					set &ds;
					if &TraitVar="&cval1" then &tnum=0;
					else if &TraitVar="&cval2" then &tnum=1;
					else &tnum=.;
				run;
				
				%let CorrTraitVar=&tnum;
				%let corrds = dsnum;
			%end;
			%else %do;
				%put ERROR: Trait Variable(s) must be numeric.;
				%goto exit;
			%end;
		%end;
		%else %let rc=%sysfunc(close(&dsid));	
		
		%global PCA_OutDataE;
		%PrepOutDS(PCA_OutDataE,PCA_OutDataEName,&OutName,_pce);
		
		ods listing close;
		ods output fisherpearsoncorr=&PCA_OutDataE(keep=var withvar Nobs Corr pValue);
		proc corr data=&corrds fisher(alpha=&alpha) pearson;
			var pca1-pca&NumberofComponents;
			with &CorrTraitVar;
		run;

		%if %length(&MultipleTestingMethod) %then %do;
			%include "&MacroPath./MultipleTestingAdjustment.sas";
			%AdjPValueName(method=&MultipleTestingMethod);
			
			proc multtest inpvalues(pvalue)=&PCA_OutDataE &MultipleTestingMethod out=&PCA_OutDataE;
			run;
			
			%PackageInsertDataset(EigenCorr data from &ProcessName, OutLib, &PCA_OutDataEName,_type=other);
			
			%if %symexist(EigData) %then %do;
				data OutLib.&EigData;
					merge OutLib.&EigData &PCA_OutDataE;
				run;
			%end;
			
			proc sort data=&PCA_OutDataE(drop=pvalue) out=fpc(rename=(&mtpvar=pvalue));
				by &mtpvar;
			run;			

		%end;
		%else %do;
			proc sort data=&PCA_OutDataE out=fpc;
				by pvalue;
			run;
			
			%PackageInsertDataset(EigenCorr data from &ProcessName, OutLib, &PCA_OutDataEName,_type=other);
			
			%if %symexist(EigData) %then %do;
				data OutLib.&EigData;
					merge OutLib.&EigData &PCA_OutDataE;
				run;
			%end;			
		%end;	
		
		%let PCAList = ;
		%let dsid = %sysfunc(open(fpc));
		%let rc = %sysfunc(fetchobs(&dsid,1));
		%let currpval = %sysfunc(getvarn(&dsid,%sysfunc(varnum(&dsid,pValue)))); 
		%let next=2;
		%do %while(%sysevalf(&currpval < &PCAlpha) and &rc=0);
			%let currPCA = %sysfunc(getvarc(&dsid,%sysfunc(varnum(&dsid,Var)))); 
			%let PCAList = %str(&PCAList &currPCA);
			%let rc = %sysfunc(fetchobs(&dsid,&next));
			%let currpval = %sysfunc(getvarn(&dsid,%sysfunc(varnum(&dsid,pValue)))); 	
			%let next = %eval(&next + 1);
		%end;
		%let rc = %sysfunc(close(&dsid));
		
		%if %length(&PCAList)=0 %then %do;
			%if %length(&MultipleTestingMethod) %then %let mtm=%str( &MultipleTestingMethod.-adjusted);
			%else %let mtm=;
			
			%put ERROR: There were no PCs with a&mtm p-value below &PCAlpha.;
			/* Display PCA plots if checked? */
			%let exiterror=1;
			%goto exit;
		%end;
		
		%if %upcase(&MergePCA)=YES %then %do;
			/*
			data &PCA_OutDataM;
				set nonpredvars;
				set xwt(keep=&PCAList);
				set d_o;
			run;
			*/
			
			data OutLib.&PCA_OutDataMName;
				set &ds_mg;
				set xwt(keep=&PCAList);
			run;		
			
			%PackageInsertDataset(Merged data from &ProcessName, OutLib, &PCA_OutDataMName);
			%let ds=&PCA_OutDataM;
		%end;
	%end;
	%else %let PCAList = pca1-pca&NumberofComponents;
			
	%global SNPTraitAssoc_PET SNPTraitAssoc_ORT SNPTraitAssoc_OutData;
	%let SNPTraitAssoc_PET=;
	%let SNPTraitAssoc_ORT=;
	%let SNPTraitAssoc_OutData=;

	%PrepOutDS(SNPTraitAssoc_PET,SNPTraitAssoc_PETName,&OutName._pet);
	%PrepOutDS(SNPTraitAssoc_ORT,SNPTraitAssoc_ORTName,&OutName._ort);
	%PrepOutDS(SNPTraitAssoc_OutData,SNPTraitAssoc_OutDataName,&OutName._sta);
	
	%let geno=0;
	%let trend=1;
	%let AlleleVars = &PredictorVars;
	%let RandEff=;
	%let CensorVar=;
	%let StrataVars=;
	%let ClassVars=;
	%let FixedEff = &PCAList;
	%let IntEff = ;
	%let IntTestsOnly= No;
	%let ParmsStmt = ;
	%let MarkerFmt = Numeric Genotypes;
	/* Table name from all but MIXED */
	%global ParmEst EffName QTests;
	%let QTests=Trend;
	%let ParmEst=ParameterEstimates;
	%let EffName=Effect;

	%if %upcase(&TraitType)=CONTINUOUS %then %do;
		%let WhichProc=MIXED;
		%let TraitOpt= ;
		%let ProcOption= ;
		%let Stat=F; 
		%let Gtable=Tests3;
		%let TStmt=contrast;
		%let Ttable=Contrasts;
		%let usestrata=0;
		%if %length(&RandEff) %then %let ModOption=%quote(/ s ddfm=&DDFM); 
		%else %let ModOption=%str(/ s ddfm=residual);
		%let ClassOpt=;	
		%let teststmt=%str('genotypes' Genotype 1);
		%let ParmEst=SolutionF;
	%end;
	%else %do;
		%let WhichProc=LOGISTIC;
		%let TraitOpt=(Event=LAST);
		%let ProcOption = ;
		%let Stat=ChiSq;
		%*let Gtable=Type3;
		%let Gtable=ModelANOVA; /*S1146207: The logistic output formerly named Type3 is now named ModelANOVA*/
		%let Otable=OddsRatiosWald;
		%let TStmt=contrast;             
		%let Ttable=ContrastTest;        
		%let usestrata=1;
		%if %upcase(&TraitType)=NOMINAL %then %let ModOption=%str(/ link=glogit); 
		%else %if %length(StrataVars)>0 %then %let ModOption=/ GCONV=1E-6;       
		%else %let ModOption = ;

		%let lsmi=0;

		%let ClassOpt = %str(/ cprefix=8);
		%let teststmt=%str('genotypes' Genotype 1); 
		%let EffName = Variable;
	%end;
	
	%if %length(&NullSNPs) %then %do;
		data mapdata;
			set mapdata_null;
			_order_ = _nullorder_ ;
			drop _nullorder_;
		run;
	%end;
	
	%SNPTraitAssocMacro(lsm=0);
	%if &exiterror=1 %then %goto exit;

	proc sort data=ma_all out=&SNPTraitAssoc_OutData;
	 by &ByVars &AnnoGroup &AnnoLoc;
	run;

	%Psmooth(Trend, &SNPTraitAssoc_OutData, &SNPTraitAssoc_OutDataName);
	%if (&syserr>4) %then %do;
		%if %upcase(&DisplayPlots)=YES %then %PackageInsertFile(jsl code from &ProcessName, &OutPath, &PCAScriptName,"&PCAIncludeDS", text/plain, yes);
		%goto exit;
	%end;
	
	%PackageInsertDataset(P-value data from &ProcessName, outlib, &OutName._sta, yes, yes, _type=anno);
	
	%if &NByVal > 1 %then %let ssr=0;
	%else %let ssr=1;
	
	%if %upcase(&DisplayPlots)=YES %then %JSLPvalMap(PCA for PS Association, &PCAScript, &PCAScriptName, &SNPTraitAssoc_OutDataName, mod=1, _volplot=&volcanoplot, _subsetdd=&ssr, _plotinput=1);
	%else %JSLPvalMap(PCA for PS Association, &PCAScript2, &PCAScriptName2, &SNPTraitAssoc_OutDataName, _volplot=&volcanoplot, _subsetdd=&ssr, _plotinput=1);

%end; /* end for TraitVar */
%else %if %upcase(&MergePCA)=YES %then %do;
	/* Need to do this if they want to merge with no traitVar*/
	options label;
	
	%if %length(&NullSNPs) %then %let PredictorVars = &PredVarsforAssoc;
	data d_o(keep=&PredictorVars) nonpredvars(drop=&PredictorVars);;  
		set InLib.&InData;
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
	run; 
		
	%if %length(&ACCProportion)=0 %then %let ACCProportion=0;
	%if &ACCProportion>0 %then %do;
		%let propmet = 0;
		%let prevNofC = &NumberofComponents;
		
		data _null_;
			set OutLib.&EigData; 
			if Cumulative >= &AccProportion then do;
				call symputx('NumberofComponents',_n_);
				call symputx('propmet',1);
				stop;
			end; 
		run;		
		%if &propmet=0 %then %do;
			%put NOTE: The value specified for the cumulative proportion of variance to explain with principal components was not met.  The maximum number of principal components (&NumberofComponents) is used.;
		%end;
		%else %do;
			%put NOTE: The Number of Principal Components used to adjust the association tests is &NumberofComponents according to the cumulative proportion of variation specified.;
		%end;
	%end;	
	
	data xwt;
		set xwt;
		%if &foundcn>0 %then %do;
			_ind=substr(OldColName,2)+0;
		%end;
		%else _ind=substr(ColumnName,2)+0;;	
		keep pca1-pca&NumberofComponents _ind;		
	run;
	
	proc sort data=xwt out=xwt(drop=_ind);
		by _ind;
	run;	
	
	/*
	data OutLib.&PCA_OutDataMName;
		set nonpredvars;
		set xwt;
		set d_o;
	run;
	*/
	
	data OutLib.&PCA_OutDataMName;
		set &ds_mg;
		set xwt;
	run;	
	
	%PackageInsertDataset(Merged data from &ProcessName, OutLib, &PCA_OutDataMName);

%end;

%exit:
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend PCAforPopulationStratification;

* run the macro;
%PCAforPopulationStratification;


