/*
JMP Genomics Analytical Process

Name: Build Genotype Probability SAS data set for Outcross
Author: Shengchu Wang, Kelci Miclaus, and Luciano Silva

Description:

*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

%global StudyName Indata MarkerVars ListMarkerVars MapFunc OutPath ServerOutPath MacroPath 
	AnnoInData ChromNum ChromLab MarkerLab MarkerPos MarkerPos_BP	
	MarkerType LinkagePhase OutName QtlBuildGPdataset_OutData QtlBuildGPdatasetScript;

%global debugflag AnnoData tChromNum VType ChLab_Type MkBP_Type NonGNTypeFlag;
%let debugflag = 0;
	
/*Luciano*/
%global MarkerFmt GenoDelim GenoMCode OldNumCode CrossType ;
%let MarkerFmt= Outcross Nondelimited Genotypes;
%let GenoDelim= ;
%let GenoMCode= ;
%let OldNumCode= ; 
%let CrossType= Outcross;
/*Luciano*/
*ProcessBody;

%macro BuildGPdatasetOutcross;

	%global GProbsData;

	%let GenoDataObjName= geno.obj;
	%let AnnoDataObjName= anno.obj;
	%let OutCrossObjName= outcross.obj;
	%let LGroupsObjName= lgroups.obj;
	%let GProbsData = gprobsdat;

	%let replacethis=\;
	%let replacewiththis=/;
	%let RSourcePath= %sysfunc(tranwrd(&MacroPath,&replacethis,&replacewiththis));
	%let RdllPath= %sysfunc(tranwrd(&MacroPath,&replacethis,&replacewiththis));
	
	%let ProcessName = QtlBuildGPdatasetOutcross;
	%include "&MacroPath./UtilityMacros.sas" / nosource;
	%include "&MacroPath./qtlUtilityMacros.sas" / nosource;
	options nolabel;
	options nospool;
	*options mlogic;
	*options spool;	
	
	%include "&MacroPath./OutcrossIMLModules.sas" / nosource;
	
	%GetOrigFileName(InData,OrigInData);
	%let FullInData=&InData;	
		
	%PathName(&InData);
	%if &exiterror %then %do;
		%put ERROR: Input data set (cross information) specification is invalid.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let InData = &TmpName;
	libname InLib "&TmpPath";
	libname OutLib "&OutPath";

	%if %length(&AnnoInData)=0 %then %do;
		%put ERROR: Annotation data set (map information) must be specified.;
		%let exiterror=1;
		%goto exit;
	%end;

	%PathName(&AnnoInData);
	%if &exiterror %then %do;
		%put ERROR: Annotation data set (map information) specification is invalid.;
		%let exiterror=1;
		%goto exit;
	%end;

	%let AnnoData = &TmpName;
	libname AnnoLib "&TmpPath";
	
	* check name for OutName;
	%let maxnamelength = 29;
	%CheckDataName(_name=OutName,_default=&OrigInData);
	%if %length(&OutName)>&maxnamelength %then %let OutName=%substr(&OutName,1,&maxnamelength);
	
	%if %length(&ListMarkerVars &MarkerVars)=0 %then %do;
		%put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let OrigMarkerVars = &MarkerVars;
	%let MarkerVars = &MarkerVars &ListMarkerVars;
	
	%PrepOutDS(QtlBuildGPdataset_OutData,QtlBuildGPdataset_OutDataName,&OutName._GP);

	/*** 1. Deal with map data set - AnnoLib.&AnnoData ***/
	data nmapfile;
	 	set AnnoLib.&AnnoData;
	run;
	%let MapFile = nmapfile;

	%let ListMapVars = &ChromNum &ChromLab &MarkerLab &MarkerPos &MarkerPos_BP &MarkerType &LinkagePhase; /*Luciano*/
	%if &ChromNum = &ChromLab %then 
		%let ChromLab =;
	/* Test the type of chromNum etc. */
	%VarAttributes(_Ds=&MapFile,_Var=&ChromNum,_Len=,_Type=VType);
	%if &VType ^= N %then %do;
		%put ERROR: Variable (&ChromNum) is not a numeric variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%if %length(&ChromLab)^=0 %then %do;
		%VarAttributes(_Ds=&MapFile,_Var=&ChromLab,_Len=,_Type=ChLab_Type);
	%end;
	%VarAttributes(_Ds=&MapFile,_Var=&MarkerLab,_Len=,_Type=VType);
	%if &VType ^= C %then %do;
		%put ERROR: Variable (&MarkerLab) is not a character variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%VarAttributes(_Ds=&MapFile,_Var=&MarkerPos,_Len=,_Type=VType);
	%if &VType ^= N %then %do;
		%put ERROR: Variable (&MarkerPos) is not a numeric variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%if %length(&MarkerPos_BP)^=0 %then %do;
		%VarAttributes(_Ds=&MapFile,_Var=&MarkerPos_BP,_Len=,_Type=MkBP_Type);
	%end;
	%VarAttributes(_Ds=&MapFile,_Var=&MarkerType,_Len=,_Type=VType);
	%if &VType ^= C %then %do;
		%put ERROR: Variable (&MarkerType) is not a numeric variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%VarAttributes(_Ds=&MapFile,_Var=&LinkagePhase,_Len=,_Type=VType);
	%if &VType ^= N %then %do;
		%put ERROR: Variable (&LinkagePhase) is not a numeric variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	
	/* Check ChromNum, MarkerPos is in ascending order or not */
	%let _tmp_ = 1;
	data _null_;
		set &MapFile;
		if (&ChromNum*1000000+&MarkerPos/100)<(lag(&ChromNum)*1000000+lag(&MarkerPos)/100) then do;
			call symput('_tmp_',"-1");
		end;
	run;
	%if &_tmp_ = -1 %then %do;
		%put ERROR: In the annotation data set (map information), &ChromNum*100,000 + &MarkerPos/100 must be in ascending order for data matrix formatting.;
		%put ERROR: Your genetic distances in the &MarkerPos variable may be too large to produce a genotype probability data set for.;
		%let exiterror=1;
		%goto exit;
	%end;
	
	%if &debugflag = 1 %then %do;
		proc print data=&MapFile; var &ListMapVars; Title "Map File"; run;
	%end;
	
	/*** 2. Deal with cross data set - InLib.&InData ***/
	%let CrossFile = InLib.&InData;
	
	/* Produce MkData data set */
	%let MkFile = gendat;
	data &MkFile;
		set &CrossFile (keep = &MarkerVars);
	run;
	%if %length(&ListMarkerVars)^=0 %then %do;
	/* Re-construct MarkerVars */
		proc contents data=&MkFile out=_t0_ (keep=name varnum) noprint; run;
		proc sort data=_t0_; by varnum; run;
		%let _t1_ = _t0_;
		%getDataVarList(_t1_,name);  /*Kemicl need to change this doing mult datasteps to make macro to make a macro for each var!!*/
		%let MarkerVars = &dataVarList;
		%put MarkerVars = &MarkerVars;
	%end;
	%let MarkerVars = %upcase(&MarkerVars);
	
	/*** 3. Combine map (MapFile) and cross (&CrossFile = &MkFile) information - InLib.&InData ***/
	%let allmatch=1;
	/* %put MarkerVars = &MarkerVars; */
	/* Delete the value of MarkerVars that is not in MapFile.MarkerLab and orderly. */
	/* Create nMarkerVars : contents of &MapFile.MarkerLab */
	%if %index(%Upcase(&MarkerFmt), ALLELES) = 0 %then %do;
		%getDataVarList(MapFile,&MarkerLab); /*Kemicl need to change this doing mult datasteps to make macro to make a macro for each var!!*/
		%let nMarkerVars = &dataVarList;
	%end;
	/*
	%put MarkerVars = &MarkerVars;
	%put nMarkerVars = &nMarkerVars;
	 newMarkerVars is the nMarkerVars after delete */
	%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
		%let newMarkerVars = &nMarkerVars;
		%let MarkerVars = &nMarkerVars;
	%end;
	%else %do;
		%let newMarkerVars=;
		%let i = 1;
		%let vari = %scan(&nMarkerVars,&i);
		%let varj = %scan(&MarkerVars,&i);
		%do %while(%length(&varj));
			/*kemicl checks
			%put This is Vari = &Vari;
			%put this is Varj = &Varj;
			*/
			/****************************************************/
			%if %upcase(&vari) ne %upcase(&varj) %then %do;
				%let allmatch=0;
				%put For Variable &i, Marker Variable is &Varj and Marker Label in Annotation is &Vari.;
				%goto enddo;
			%end;
			%else %do;
				%let newMarkerVars = &newMarkerVars &Vari;
			%end;
			/***************************************************/
			%let i = %eval(&i+1);
			%let vari = %scan(&nMarkerVars,&i);
			%let varj = %scan(&MarkerVars,&i);
		%end;
		%if %length(&vari) %then %do;
			data &Mapfile;
				set &Mapfile;
				if _n_ < &i;
			run;
		%end;
	%enddo:;
	%end;
	%put allmatch = &allmatch;
		
	/* Kemicl - added check to see that all markers in input match to Marker Label (and order) in annotation;*/
	%if &allmatch=0 %then %do;  
		%put ERROR: The markers in the input data set do not match the annotation data set according to the variable selected as Marker Label Variable.;
		%put Please check that the Marker Label exactly matches the input data columns or run the Subset and Reorder Genetic Data AP to order these data sets correctly.;
	
		* create JSL file;
		%global SRButtonsScript;
		%PrepOutFile(SRButtonsScript,SRButtonsScriptName,launchAP_buttons_SR.jsl);
		filename jslbtns "&SRButtonsScript";

		options noquotelenmax;
		data _null_;
			file jslbtns lrecl=32767;
			put "//!";
			put "Names Default to Here(1);";
			put "wait(0.5);";
			put "if(host is(Windows),";
			put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
			put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
			put "	";
			put "srbuttonswindow=New Window(""Launch AP to Reorder Markers"",";
			put "		VListBox(";
			put "			TextBox(""Click on the button below to launch the Subset and Reorder Genetic Data process to reorder marker columns properly.""),";
			put "			TextBox(""Use the output data sets from that process as the new input and annotation data sets to .""),";

			put "			TextBox("" ""),";
			put "			ButtonBox(""Subset and Reorder Genetic Data"",";
			put "				if(host is(Windows),";
			put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
			put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
			put "	";

			/* Set input data set and Output Folder */
			put "				jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";				
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""%sysfunc(pathname(Inlib))\&InData..sas7bdat"");";
			put "				jg:SubsetandReorderGeneticData_updateDeps;";
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&MarkerVars"");";	
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""Outcross Nondelimited Genotypes"");";
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),""0"");";		
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""%sysfunc(pathname(AnnoLib))\&AnnoData..sas7bdat"");";					
			put "				jg:SubsetandReorderGeneticData_updateDeps;";				
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&MarkerLab"");";	
			put "			)";;

			put "		)";
			put ");";
		run;

		%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SRButtonsScriptName, , text/plain, yes);		
		%let exiterror=1;
		%goto exit;
	%end;
	
	/* check marker var to make sure that each chromosome at least have two markers */
	proc summary data = &MapFile; by &ChromNum; Output out = _tmp_; run;
	%let _flag_ = 1;
	data null;
		set _tmp_;
		if _FREQ_ = 1 then do;
			call symput('_flag_',"-1");
			call symputx('_chrom',&ChromNum);
		end;
	run;
	%if &_flag_ = -1 %then %do;
		%put ERROR: In the annotation data set (map information), each chromosome (&_chrom) must have more than one marker.;
		%let exiterror=1;
		%goto exit;
	%end;

	options nonotes;
	ods _all_ close;	
	ods html close;
	*ods html body="&Outpath.&OutName..html";	
	proc iml;
		/*NOTES:
		(1) Must keep module calls in the order they have been written here because
			the output from one module may be used in the next.
		(2) R objects created by a module can be accessed by any other module as far as 
			modules calls are keept within the same block defined by "Proc iml"	and "quit".
	        This fact can be used to avoid having to exchange matrices or datasets between 
			R and IML.
		*/
		
		/*loading modules*/
		load module=(LoadRPkgs);
		load module=(LoadRFunc);
		load module=(LoadRdll);
		load module=(UnLoadRdll);		
		load module=(ExportDataToR); 
		load module=(RebuildOutcrossObj);
		load module=(RebuildLGsObj); 
		load module=(BuildGP);
		
		/*loads R packages*/
		run LoadRPkgs;
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;

		/*loads R dll files*/
		run LoadRdll("&RdllPath");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;

		/*loads R functions*/
		run LoadRFunc("&RSourcePath");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;		

		/*exports genotypic data from SAS to R*/
		run ExportDataToR("&MkFile", "&GenoDataObjName");

		/*exports annotation data from SAS to R*/
		run ExportDataToR("&MapFile", "&AnnoDataObjName");
		
		/*rebuilds outcross object in R*/
		run RebuildOutcrossObj("&GenoDataObjName", "&AnnoDataObjName", "&OutCrossObjName", "&MarkerType");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
		
		/*rebuilds linkage group objects in R*/
		run RebuildLGsObj("&ChromNum", "&MarkerPos", "&MarkerLab", "&LinkagePhase", 
				"&MapFunc", "&GenoDataObjName", "&AnnoDataObjName", 
				"&OutCrossObjName", "&LGroupsObjName");		
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
			
		%*let exiterror=1;
		%*goto exit;
		
		/*estimates conditional probabilities of pseudomarkers*/
		run BuildGP("&Mapfunc", "&LGroupsObjName", "&GProbsData");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;
		
		/*unloads R dll files*/
		run UnLoadRdll("&RdllPath");
		if ^isOK then do;
			call symputx("exiterror", 1);
			abort;
		end;

	quit;

 	%if &syserr>4 or &exiterror = 1 %then %do;
 		%put ERROR: Calculation of probabilities for QTL genotypes could not be completed.;
 		%put ERROR: Please be sure the appropriate columns were selected as Marker Variables, Marker Type, and Linkage Phase of Adjacent Markers. Also, check coding of genotypes for possible errors.;
 	 	%let exiterror=1;
 		%goto exit;
	%end;
	
	/* kemicl - S0671459 change missing values due to 0/0 division to 0*/
	data &QtlBuildGPdataset_OutData; 
		*set &QtlBuildGPdataset_OutData;
		set &GProbsData; /*Luciano*/
		array d{*} COL:;
		do i=1 to dim(d);
			if d[i]=. then d[i]=0;
		end;
		drop i;
	run;
	
	/*************************************************************************/
	* Packaging;
	* create JSL file;
	%global BGPButtonsScript;
	%PrepOutFile(BGPButtonsScript,BGPButtonsScriptName,QTLBuildGP_launchbuttons.jsl);
	filename jslbtns "&BGPButtonsScript";
	
	%global _mainjslfile_ FullAPName;
	%let _mainjslfile_ = %quote(&BGPButtonsScript);
	%let FullAPName = %str(Build QTL Genotype Probability Data Set for Outcross);
	
	%global _iewindow_;
	%let _iewindow_=qbbuttonswindow;
		
	data _null_;
		file jslbtns lrecl=32767;
		put "//!";
		put "Names Default to Here(1);";
		
		put "wait(0.5);";
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";
		put "Here:qbbuttonswindow=Function({},{default local},";
		put "		VListBox(";
		put "			TextBox(""Click on the button below to launch the IM, CIM and MIM Analysis process using the output data set.""),";
		put "			TextBox("" ""),";
		put "			ButtonBox(""IM, CIM and MIM Analysis"",";
		put "				if(host is(Windows),";
		put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SingleQTLModel.jsl"", absolute, windows)),";
		put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SingleQTLModel.jsl"", absolute, POSIX) ));";
		put "	";
		put "				jg:setDefaultParmValues(2, ""SingleQTLModel"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""InData""),""&FullInData"");";
		put "				jg:SingleQTLModel_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""MarkerVars""),""&OrigMarkerVars"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""ListMarkerVars""),""&ListMarkerVars"");";
	
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""AnnoInData""),""&AnnoInData"");";
		put "				jg:SingleQTLModel_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""ChromLab""),""&ChromLab"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""ChromNum""),""&ChromNum"");";	
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""MarkerLab""),""&MarkerLab"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""MarkerPos""),""&MarkerPos"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""MarkerPos_BP""),""&MarkerPos_BP"");";
		%if %quote(&CrossType)=%str(Outcross) %then %do;
			put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""GKNumber""),""four"");";
		%end;
		%else %if %index(&CrossType,B)>0 or %quote(&CrossType)=%str(RI0) %then %do;
			put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""GKNumber""),""two"");";
		%end;
		%else %do;
			put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""GKNumber""),""three"");";
		%end;
		/*Luciano*/	
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""MAlgorithm""),""Haley-Knott Regression"");"; 	
		/*Luciano*/	
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""GPInData""),""&OutPath.&QtlBuildGPdataset_OutDataName..sas7bdat"");";
		put "				jg:SingleQTLModel_updateDeps;";	
		
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""MarkerFmt""),""&MarkerFmt"");";
		put "				jg:SingleQTLModel_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""GenoDelim""),""&GenoDelim"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""GenoMcode""),""&GenoMcode"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""OutName""),""&OutName"");";
		put "		  	   	jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""OldNumCode""),""&OldNumCode"");";
		put "				jg:setCurrentParmValue(2, ""SingleQTLModel"", jg:getParmNum(""SingleQTLModel"",""OutPath""),""&ClientOutPath"");";
		put "			)";;
		put "		),";
		put ");";
		
		put "Here: LaunchCode = Here:qbbuttonswindow();";
	run;

	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &BGPButtonsScriptName, , text/plain, yes);

	%PackageInsertDataset(Genotype probability data from &ProcessName,outlib,&QtlBuildGPdataset_OutDataName,No);

	%*goto exitcheck;

	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
				
	%exitcheck:; *---Luciano used it while testing code ---*;


%mend BuildGPdatasetOutcross;
%BuildGPdatasetOutcross;
