/*
JMP Genomics Analytical Process

Name: QTL Single Marker Analysis
Author: Shengchu Wang

Description:

	QTL Single Marker Analysis is a good way to quickly
    scan whole genome to get an idea of where the QTLs may
    reside. It does simple regression for each marker with 
    all trait values and find out the probability of QTL 
	evidence for these markers. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2007 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData MarkerVars AlleleVars TraitVar FactorVars FactorClassVars ClassVars ListMarkerVars ListTraitVars WhereStmt OutPath ServerOutPath MacroPath
AnnoInData ChromLab MarkerLab MarkerPos 
Alpha MarkerFmt OldNumCode GenoDelim GenoMcode OutName SingleMarkerAnalysis_OutData SingleMarkerAnalysisScript;

/* Set to blank vars needed for GenMacros */
%global ByVars AnnoMName AnnoLabel AnnoMergeVars AnnoGroup AnnoLoc AnnoAccession;
 %let ByVars = ;
 %let AnnoMName = &MarkerLab;
 %let AnnoLabel = &AnnoMName;
 %let AnnoGroup = &ChromLab;
 %let AnnoLoc = &MarkerPos;
 %let AnnoAccession = ;
 %let AlleleVars = &MarkerVars;
 %let ClassVars = &FactorClassVars;

%Global FixedEff;
%let FixedEff = &FactorVars &ClassVars;

/* set up other variables for SNPTrait code.  Kemicl: later add some back as options? */
%global TraitType StrataVars CensorVar CensorValues RandEff IntEff IntTestsOnly 
Parmsstmt OutputResid Conv MTC  PValAdj Tau Bandwidth  OnlySig
Odds RecodeAllele PerfPerm AdjPerm Seed;
%Let TraitType = Continuous;
%let StrataVars= ;
%let CensorVar= ;
%let CensorValues= ;
%let RandEff=%nrbquote();
%let IntEff=%nrbquote();
%let IntTestsOnly = No;
%let Parmsstmt = ;
%let OutputResid = No;
%let Conv = NegLog10;
%let MTC = ;
%let PValAdj = ;
%let Tau = ;
%let Bandwidth = ;
%let OnlySig = No;
%let Odds = No;
%let RecodeAllele = ;
%let PerfPerm = 0;
%let AdjPerm = No;
%let Seed = 12343;


*ProcessBody;

%macro QtlSingleMarkerAnalysis;

%include "&MacroPath./GenMacros.sas" / nosource;
%include "&MacroPath./SNPTraitAssocMacro.sas" / nosource;
%include "&MacroPath./qtlUtilityMacros.sas" / nosource;

%GenInit(QtlSingleMarkerAnalysis,28);

%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(QTL Single Marker Analysis);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%let SingleMarkerAnalysis_OutData = ;
%let SingleMarkerAnalysisScript = ;

%PrepOutDS(SingleMarkerAnalysis_OutData,SingleMarkerAnalysis_OutDataName,&OutName._sma);

%PrepOutFile(SingleMarkerAnalysisScript,SingleMarkerAnalysisScriptName,&ProcessName Script for &OutName..jsl);

%global QTests;
%let QTests=Trend;
%let geno=0;
%let trend = 1;

%global SNPTraitAssoc_PEG SNPTraitAssoc_PET SNPTraitAssoc_CPG SNPTraitAssoc_CPT SNPTraitAssoc_ORG SNPTraitAssoc_ORT SNPTraitAssoc_LSM;
%let SNPTraitAssoc_PEG=;
%let SNPTraitAssoc_PET=;
%let SNPTraitAssoc_CPG=;
%let SNPTraitAssoc_CPT=;
%let SNPTraitAssoc_ORG=;
%let SNPTraitAssoc_ORT=;
%let SNPTraitAssoc_LSM=;

/* don't output ls means */
%let lsmi=0;

%if &geno=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PEG,SNPTraitAssoc_PEGName,&OutName._peg);
	%PrepOutDS(SNPTraitAssoc_CPG,SNPTraitAssoc_CPGName,&OutName._cpg);
	%if %upcase(&OutputResid)=YES %then %PrepOutDS(SNPTraitAssoc_ORG,SNPTraitAssoc_ORGName,&OutName._org);
	%if &lsmi=1 %then %PrepOutDS(SNPTraitAssoc_LSM,SNPTraitAssoc_LSMName,&OutName._lsm);
%end;
%if &trend=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PET,SNPTraitAssoc_PETName,&OutName._pet);
	%PrepOutDS(SNPTraitAssoc_CPT,SNPTraitAssoc_CPTName,&OutName._cpt);
	%if %upcase(&OutputResid)=YES %then %PrepOutDS(SNPTraitAssoc_ORT,SNPTraitAssoc_ORTName,&OutName._ort);	
%end;

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;


%if %length(&WhereStmt) %then %do;
 data countobs; 
 	set &ds;
  where &WhereStmt;
 run;
 %Nobsvars(countobs);
 %let ds=countobs;
%end;
%else %Nobsvars(&ds);

%if &nobs=0 %then %do;
	%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
	%goto exit;
%end;

/* data set to use in iml, only genotypes */
data gendat(keep =&AlleleVars) odat(drop=&AlleleVars); set &ds;  
run;

data mnames; set gendat;
where 0;
run;

/* marker recoding macro to convert to numeric for IML code */
%RecodeGenos;
/* Add a check for numeric format for negative values and throw error if found */
%OldGenoCheck;

%if &exiterror %then %goto exit;
%if &syserr>4 %then %do;
	%put ERROR: The genotype data could not be converted to numeric format for recombination computation.;
	%put Please check the Marker Format coding of your data for possible errors.;
 	%let exiterror=1;
	 %goto exit;
%end;

%if %index(%Upcase(&MarkerFmt),NUMERIC) = 0  %then %do;
	%let _tid_ = %sysfunc(open(mnames));
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));

	proc datasets library=work nolist;
		modify gendat;
		%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
			%let _nm_ = %sysfunc(floor(%eval(&_nVars_/2)));
			%put There are &_nm_ Markers;	
			rename %do i=1 %to &_nm_; _geno_&i = Marker_&i %end;;
		%end;
		%else %do;
			rename %do i=1 %to &_nVars_; _geno_&i = %sysfunc(varname(&_tid_,&i)) %end;;
		%end;
	quit;

	%if %index(%Upcase(&MarkerFmt), ALLELES) %then %do;
		%let AlleleVars = Marker:;
	%end;
%end;

data gendat; merge odat gendat; run;
%let ds = gendat;

%let MarkerFmt = Numeric Genotypes;
%let Genocol = 1;

/* Table name from all but MIXED */
%global ParmEst EffName;
%let ParmEst=ParameterEstimates;
%let EffName=Effect;

/* Trait Type should be always be continuous */
%let WhichProc=MIXED;
 %let TraitOpt= ;
%let ProcOption= ;
%let Stat=F; 
%let Gtable=Tests3;
%let TStmt=contrast;
%let Ttable=Contrasts;
%let usestrata=0;
%if %length(&RandEff) %then %let ModOption=%quote(/ s ddfm=&DDFM); 
%else %let ModOption=%str(/ s ddfm=residual);
%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption residual outp=outres);	
%let ClassOpt=;	
%let teststmt=%str('genotypes' Genotype 1);
%let ParmEst=SolutionF;

%SNPTraitAssocMacro(lsm=&lsmi);
%if &exiterror=1 %then %goto exit;

proc sort data=ma_all out=&SingleMarkerAnalysis_OutData;
 by &ByVars &AnnoGroup &AnnoLoc;
run;

%Psmooth(&QTests, &SingleMarkerAnalysis_OutData, &SingleMarkerAnalysis_OutDataName);
 %if (&syserr>4) %then %goto exit;
 
%PackageInsertDataset(P-value data from &ProcessName, outlib, &OutName._sma, yes, yes);

%JSLPvalMap(QTL Single Marker Analysis, &SingleMarkerAnalysisScript, &SingleMarkerAnalysisScriptName, &SingleMarkerAnalysis_OutDataName, _volplot=1);


%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend QtlSingleMarkerAnalysis;

* run the macro;
%QtlSingleMarkerAnalysis;
