/*
JMP Genomics Analytical Process

Name: Recode Genotypes
Author: Wendy Czika

*/

/*
-----------------------------------------------------------
Copyright (C) 2007 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ListMarkerVars BreedingConversion GenotypeConversion 
PloidyNum MarkerFmt GenoDelim GenoMcode RVConversion MAF Prefix AppendPrefix WhereStmt 
AnnoInData AnnoLabel AnnoMajAllele AnnoMAF AnnoGroup2 OutPath ServerOutPath OutName 
OutAnnoName MacroPath NumGenoData NumGenoData_OutAnnoData;

*ProcessBody;

%macro RecodeGenotypes;

%include "&MacroPath./GenMacros.sas" / nosource;

%global ByVars AnnoGroup AnnoLoc AnnoAccession AnnoWhereStmt;
%let ByVars=;
%let AnnoGroup=&AnnoGroup2;
%let AnnoLoc=;
%let AnnoAccession=;
%let AnnoWhereStmt=;
%let OutWholeName = &OutName;

%GenInit(RecodeGenotypes,32);
%global FullAPName;
%let FullAPName = %str(Recode Genotypes);	

%if &exiterror=1 %then %goto exit;

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
	%put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
	%let exiterror=1;
	%goto exit;
%end;

%if %upcase(&BreedingConversion)= YES and ^%index(%upcase(&MarkerFmt), NUMERIC) %then %do;
	%put ERROR: Format of Marker Variables must be NUMERIC GENOTYPES when the option Genotype Recoding for Breeding is checked.;
	%let exiterror=1;
	%goto exit;
%end;

%if %length(&PloidyNum) = 0 %then %let PloidyNum= 2;

%if &PloidyNum > 2 and ^%Index(&MarkerFmt, %str(Nondelimited)) and %upcase(&BreedingConversion) ^= YES %then %do;
	/*Can use the above if conditon because BreedingConversion requires numeric genotypes*/
	%put ERROR: Ploidy Number greater than 2 can be analyzed if either of two conditions below are met:; 
	%put ERROR: (1) When markers have Nondelimited Genotypes.;
	%put ERROR: (2) When markers have Numeric Genotypes and Genotype Recoding for Breeding is selected.;
	%let exiterror=1;
	%goto exit;
%end;

/* A,H,B, or A,AB,B genotypes */
%if %substr(&MarkerFmt,1,1) = %str(S) %then %do;

	%let GenoDelim=%str(/);
	proc format;
		value $singlechar 'A'='A/A'
  	            		'B'='B/B'
  	            		'AB'='A/B'
  	            		'H'='A/B'
  	            		other=''
		;
	run;
	%let GenoCol = %str(genocol delimiter="/");
	%if %length(&AnnoMajAllele)=0 %then %do;
		%if %upcase(&GenoMcode)=%str(A ALLELE) %then %let AnnoMajAllele = %str("B");
		%else %let AnnoMajAllele = %str("A");
	%end;
%end;

%if %length(&MAF)=0 %then %let MAF = 0;

%let AlleleVars = &AlleleVars &ListMarkerVars;

%let pl=%eval(%length(&Prefix)+1);
%if %length(&OutWholeName)=0 %then %let OutName=%substr(&OutName,1,32-&pl)_%unquote(&Prefix);

%if %length(&AnnoInData) %then %do;
	%GetOrigFileName(AnnoInData,OrigAnnoInData);
	%PathName(&AnnoInData);
	%let AnnoData = &TmpName;
	libname AnnoLib "&TmpPath";
	%if %length(&OutAnnoName) %then %CheckDataName(_name=OutAnnoName);
	%else %let OutAnnoName=%sysfunc(compress(%substr(&AnnoData,1,32-&pl)_%unquote(&Prefix)));
%end;
%else %do;
	%if %length(&OutAnnoName) %then %CheckDataName(_name=OutAnnoName);
	%else %let OutAnnoName=%substr(&OutName,1,27)_anno;
%end;

%PrepOutDS(NumGenoData, NumGenoDataName, &OutName);
%PrepOutDS(NumGenoData_OutAnnoData, NumGenoData_OutAnnoDataName, &OutAnnoName);

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

%global nvars nobs;
%if %length(&WhereStmt) %then %do;
	data countobs(keep=&AlleleVars) subds; 
	 set &ds;
	 where &WhereStmt;
	run;
	
	%let ds=subds;
	
	%let dsid = %sysfunc(open(count,i));
	%let nobs = %sysfunc(attrn(&dsid,NOBS));
	%let mvartype=%sysfunc(vartype(&dsid,1));
	%let nvars = %sysfunc(attrn(&dsid,NVARS));
	%let firstavar=%sysfunc(varname(&dsid,1));
	%let lastavar=%sysfunc(varname(&dsid,&nvars));	
	%let rc = %sysfunc(close(&dsid));	
%end;
%else %do;
	%let dsid = %sysfunc(open(&ds,i));
	%let nobs = %sysfunc(attrn(&dsid,NOBS));
	%let rc = %sysfunc(close(&dsid));
	/* count data set created in GenMacros */
	%let dsid=%sysfunc(open(count,i));
	%let mvartype=%sysfunc(vartype(&dsid,1));
	%let nvars = %sysfunc(attrn(&dsid,NVARS));	
	%let firstavar=%sysfunc(varname(&dsid,1));
	%let lastavar=%sysfunc(varname(&dsid,&nvars));
	%let rc=%sysfunc(close(&dsid));
%end;

%if &nobs=0 %then %do;
	%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Filter to Include Observations.;
	%goto exit;
%end;

%let dsid=%sysfunc(open(&ds,i));
%let newvar=%sysfunc(varnum(&dsid,%unquote(&Prefix)1));
%let rc=%sysfunc(close(&dsid));

%if &newvar>0 %then %do;
	%put ERROR: Variables with the prefix &Prefix already exist in the data set.  Please use a different prefix.;
	%goto exit;
%end;

%if %substr(&MarkerFmt,1,1) = %str(A) %then %do;
	%let dsid = %sysfunc(open(count));
	%let nvars = %sysfunc(attrn(&dsid,NVARS));
	
	%let maxvarlen = 0;
	%do i=1 %to &nvars;
		%let curvarlen = %length(%sysfunc(varname(&dsid,&i)));
		%if &curvarlen > &maxvarlen %then %let maxvarlen = &curvarlen;
	%end;
	
	data anames;
		length _Allele1Col_ _Allele2Col_ $&maxvarlen;
		%do i=1 %to %eval(&nvars/2);
			%let i1=%eval(2*&i-1);
			%let i2=%eval(2*&i);
			_Allele1Col_ = "%sysfunc(varname(&dsid,&i1))";
			_Allele2Col_ = "%sysfunc(varname(&dsid,&i2))";
			output;
		%end;
	run;
	%let rc=%sysfunc(close(&dsid));
	data mapdata;
		set mapdata;
		set anames;
	run;
	
	proc datasets nolist;
		delete anames;
	quit;	
%end;

%if %upcase(&BreedingConversion) ^= YES %then %do;
	%if %substr(&GenotypeConversion,1,1) ne %str(G) %then %do;
	
		%put AnnoLabel= &AnnoLabel;
		/*data OutLib.rg_mapdata; set mapdata; run;*/
	
		/*Output: numeric additive, numeric dominant, numeric heterozygos, numeric recessive*/
		%let n2snps=&nvars;
		%if %length(&GenoCol) %then %do;
		 %let n2snps=%eval(2*&n2snps);
		%end;
		%let nm=%eval(&n2snps/2);
		%let NDigits = %sysevalf(%sysfunc(floor(%sysfunc(log10(&nm))))+1);
		%if %length(&AnnoGroup2) %then %let NDigits = %sysfunc(max(&NDigits,&GpLen+4));

		%let BlockSize = %sysfunc(min(100000,%sysevalf(20000000/&nobs,floor)));

		%if &nm < &BlockSize %then %let BlockSize = &nm;

		%let first = 1;
		%let last=&BlockSize;

		data _null_;
			set count;
			array getname{*} &AlleleVars;
			%let Block=1;
			%do %until (&first > &nm);	
				%if %length(&GenoCol) %then %do;
					_f = vname(getname(&first));
					_l = vname(getname(&last));
				%end;
				%else %do;
					_f = vname(getname(%eval(2*&first-1)));
					_l = vname(getname(%eval(2*&last)));
				%end;

				call symput("firstvar&Block",_f);
				call symput("lastvar&Block",_l);

				%let BlockStart&Block = &first;
				%let BlockEnd&Block = &last;			

				%let first = %eval(&last + 1);
				%let last = %eval(&first + &BlockSize - 1);
				%let Block = %eval(&Block + 1);
				%if &last > &nm %then %let last = &nm;
			%end;
		run;	

		%let NBlocks = %eval(&Block-1);
		%let NChunks=%eval((&NBlocks-1)/100 + 1);
		%let ChunkSize = %sysfunc(min(100,&NBlocks)); /* Number of blocks in chunk */

		%let first = 1; /* first marker in block */
		%let last=&BlockSize; /* last marker in block */
		%let Block = 1;	
		
		%NObsVars(&ds);
		%let nsamples= &nobs;

		ods listing close;

		%do c=1 %to &NChunks;

			/* Last block in current chunk */
			%let BinC = %sysfunc(min(%eval(&Block + &ChunkSize -1),&NBlocks));

			data 
				%do b=&Block %to &BinC; 
					avars&b(keep=&&firstvar&b--&&lastvar&b) 
				%end; 
				%if &c=1 %then other(drop=&firstvar1--&&lastvar&NBlocks);
				;
				/*Make sure Allele Vars are in consecutive columns */
				set &ds(drop=&AlleleVars);
				set &ds(keep=&AlleleVars);
				%if %qupcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
					format  &firstvar1--&&lastvar&NBlocks numtochar.;
				%end;		
				%if %substr(&MarkerFmt,1,1)=%str(S) %then %do;
					format &firstvar1--&&lastvar&NBlocks $singlechar.;
				%end;				
			run;
			
			/*data OutLib.rg_avars1; set avars1; run;*/
			
			%if &PloidyNum > 2 %then %do;
				%Nobsvars(avars1);
				%let nobs= &nobs;
			
				%do %while (&Block le &NBlocks);

					%*Nobsvars(avars&Block);

					proc transpose data=avars&Block out=trans_(drop=_name_) prefix=indiv;
						var &&firstvar&Block -- &&lastvar&Block;
					run;

					%if &syserr>4 %then %let exiterror=1;

					proc datasets nolist;
						%if &block>1 %then %do;
							append base=trans data=trans_ force;
							delete trans_;
						%end;
						%else %do;
							delete trans;
							change trans_=trans;
						%end;
						delete avars&Block;
					run;		

					%let first = %eval(&last + 1);
					%let last = %eval(&first + &BlockSize - 1);
					%if &last > &nm %then %let last = &nm;
					%let Block = %eval(&Block + 1);		

				%end;	
				
				/*data OutLib.rg_trans; set trans; run;*/
			%end;

			%if %length(&AnnoMajAllele)=0 or (%index(%quote(&RVConversion),%str(rare variants)) and %length(&AnnoMAF)=0) %then %do;
				%if &PloidyNum = 2 %then %do;
					%do %while (&Block le &BinC);

						ods output AlleleFreq=af_;
						proc allele data=avars&Block ndata=mapdata(rename=(&AnnoLabel=name) firstobs=&first obs=&last) &GenoCol;  
							 var &&firstvar&Block -- &&lastvar&Block;  
						run;

						%if &Block>1 %then %do;
							proc append base=af data=af_ force;
							run;
						%end;
						%else %do;
							%RemoveData(af);
							proc datasets nolist;
								change af_=af;
							quit;
						%end;

						%let first = %eval(&last + 1);
						%let last = %eval(&first + &BlockSize - 1);
						%if &last > &nm %then %let last = &nm;
						%let Block = %eval(&Block + 1);		
					%end;
				%end;
				%else %do;

					data af;
						set trans;
						array samp{*} indiv:;
						Total= dim(samp);
						*put Total;
						
						NIndiv=0; *total of nonmissing samples*;
						CountA=0; *total of allele A*;
						CountB=0; *total of allele B*;
						do i=1 to dim(samp);
							if not missing(samp[i]) and samp[i]^="." then do;
								CountA= CountA + countc(samp[i], "A"); 
								CountB= CountB + countc(samp[i], "B"); 
								NIndiv= NIndiv + 1;
							end;
						end;	
						*total of other alleles*;
						CountO= NIndiv * &PloidyNum - (CountA + CountB); 
						FreqA= 0;
						FreqB= 0;
						if NIndiv > 0 then do;
							FreqA= CountA/(NIndiv * &PloidyNum);
							FreqB= CountB/(NIndiv * &PloidyNum);
						end;
						_MAF_= 0;
						if CountA <= CountB then do;  
							*minor allele*;
							_MinorAllele_= "A"; 
							_MajorAllele_= "B";
							if NIndiv > 0 then do;	
								*minor allele frequency*;
								_MAF_= CountA/(NIndiv * &PloidyNum); 
							end;
						end;
						else do;	     
							*minor allele*;
							_MinorAllele_= "B"; 
							_MajorAllele_= "A";
							if NIndiv > 0 then do;		 
								*minor allele frequency*;
								_MAF_= CountB/(NIndiv * &PloidyNum); 
							end;
						end;
						if CountA > 0 and CountB > 0 and CountO > 0 then do;
							_NAllele_=.;
							_MinorAllele_ = "";
							_MajorAllele_ = "";
						end;
						else if CountA > 0 and CountB > 0 and CountO = 0 then do;
							_NAllele_=2;
						end;
						else if (CountA = 0 or CountB = 0) and CountO = 0 then do;
							_NAllele_=1;
							_MinorAllele_ = "";
							_MajorAllele_ = "";
						end;						
						drop i indiv:;
					run;

					data af;
						merge mapdata(rename=(&AnnoLabel= Locus) keep=&AnnoLabel) af;
						Allele= "A";
						Count= CountA;
						Freq= FreqA;
						output;
						Allele= "B";
						Count= CountB;
						Freq= FreqB;
						output;
						keep Locus Allele Count Freq _NAllele_
							%if ^%index(%quote(&RVConversion),%str(rare variants)) and %length(&AnnoMAF)=0 %then _MAF_;
						;
					run;
				
				%end;
				
				/*data OutLib.rg_af; set af; run;*/
			%end;
		%end; /* end chunk */

		ods listing;		
		%if &syserr>4 %then %goto exit;

		%if %length(&AnnoMajAllele)=0 or (%index(%quote(&RVConversion),%str(rare variants)) and %length(&AnnoMAF)=0) %then %do;

			proc sort data=mapdata out=mapsort;
				by &AnnoLabel;
			run;
			proc sort data=af;
				by locus descending freq;
			run;

			data afsub;
				merge af mapsort(rename=(&AnnoLabel=locus));
				by locus;
				if first.locus then do;
					%if %length(&AnnoMajAllele)=0 %then %do;
						retain _MajorAllele_ _MinorAllele_;
						_MajorAllele_ = Allele;
						%let AnnoMajAllele = _MajorAllele_;
					%end;
					%if %index(%quote(&RVConversion),%str(rare variants)) and %length(&AnnoMAF)=0 %then %do;
						retain _MinorAllele_ _MAF_;
						_MAF_ = 1 - freq;
						%let AnnoMAF=_maf_;
					%end;
					allct = 1;
				end;
				else allct + 1;
				%if %upcase(&AnnoGroup2)=%upcase(&AnnoLabel) %then %do;
					/* Fix for S0666352 */
					&AnnoGroup2 = locus;
				%end;
				if last.locus then do;
					if allct = 2 then _MinorAllele_ = Allele;
					else _MinorAllele_ = "";
					output;
				end;
				drop controlvar freq stderr count allct allele;
			run;
			
			/*data OutLib.rg_afsub; set afsub; run;*/

			proc datasets nolist;
				delete mapsort af;
			quit;

		%end;
		%else %do;
			proc sort data=mapdata out=afsub(rename=(&AnnoLabel=Locus));
				by &AnnoLabel;
			run;

			%if %upcase(&AnnoGroup2)=%upcase(&AnnoLabel) %then %do;
				/* Fix for S0666352 */
				data afsub;
					set afsub;
					&AnnoGroup2 = locus;
				run;
			%end;		
		%end;
		
		%if &PloidyNum = 2 %then %do;
			%Nobsvars(avars1);
			%let nobs= &nobs;
			
			%let first = 1; /* first marker in block */
			%let last=&BlockSize; /* last marker in block */
			%let Block = 1;				
		
			%do %while (&Block le &NBlocks);

				%*Nobsvars(avars&Block);

				proc transpose data=avars&Block out=trans_(drop=_name_) prefix=indiv;
					var &&firstvar&Block -- &&lastvar&Block;
				run;

				%if &syserr>4 %then %let exiterror=1;

				proc datasets nolist;
					%if &block>1 %then %do;
						append base=trans data=trans_ force;
						delete trans_;
					%end;
					%else %do;
						delete trans;
						change trans_=trans;
					%end;
					delete avars&Block;
				run;		

				%let first = %eval(&last + 1);
				%let last = %eval(&first + &BlockSize - 1);
				%if &last > &nm %then %let last = &nm;
				%let Block = %eval(&Block + 1);		

			%end;
		%end;

		proc sort data=afsub;
			by _order_;
		run;

		%if %length(&GenoCol)=0 %then %do;
			/* Converts allele format to Genotype format */
			data trans1(rename=(indiv1-indiv&nobs=findiv1-findiv&nobs)) trans2(rename=(indiv1-indiv&nobs=sindiv1-sindiv&nobs));
				set trans;
				retain maxlen 0;
				%do i=1 %to &nobs;
					if length(strip(indiv&i)) > maxlen then maxlen = length(strip(indiv&i));
				%end;
				call symputx("maxavlen",maxlen);
				if mod(_n_,2)=1 then output trans1;
				else output trans2;
			run;

			data trans;
				set trans1; set trans2;
				array indiv{&nobs} $%eval(2*&maxavlen + 1);
				array findiv{&nobs} findiv1-findiv&nobs;
				array sindiv{&nobs} sindiv1-sindiv&nobs;
				do _i=1 to &nobs;
					if missing(findiv{_i}) or missing(sindiv{_i}) then indiv{_i}="";
					else indiv{_i} = cats(findiv{_i},"/",sindiv{_i});
				end;
				keep indiv:;
			run;

			proc datasets nolist;
				delete trans1 trans2;
			quit;

			%let GenoDelim=%str(/);

		%end;	
		%else %do;
			data _null_;
				set trans;
				retain maxlen 0;
				array iarray{*} indiv1-indiv&nobs;
				do i=1 to dim(iarray);
					if length(strip(iarray{i})) > maxlen then maxlen = length(strip(iarray{i}));
				end;
				call symputx("maxavlen",maxlen);
			run;
			%let maxavlen = %sysevalf(&maxavlen/2,ceil);
		%end;				

		%let Block = 1;	
		%let fobs=1;
		%let lobs=&BlockSize;
		
		/*data OutLib.rg_trans; set trans; run;*/

		options compress=binary;

		%if %length(&AnnoGroup2) %then %do;
			data trans;
				set trans; set afsub;
				if &AnnoMAF <= &MAF then _rare_=1;
				else _rare_=0;
			run;

			proc sort data=trans ;
				by &AnnoGroup2 _rare_;
			run;
			
			/*data OutLib.rg_transrare; set trans; run;*/

		%end;

		%do c=1 %to &NChunks;	

			/* Last block in current chunk */
			%let BinC = %sysfunc(min(%eval(&Block + &ChunkSize -1),&NBlocks));
			%let anyblankmajall = 0;

			/* The code below is processing the entire data set every time, then outputting to a subset of blocks (if >1 chunk).  Not
			very efficient, but it makes it easier to output according to _order_ to keep everything ordered correctly - can revisit
			later if there is a better way */
			data %do b=&Block %to &BinC; mrge&b %end; %if &c=&NChunks and (%length(&AnnoGroup2) or %length(&AnnoMAF)) %then afsub(drop=_ng:);;
				length _w1 _w2 %do k= 3 %to &PloidyNum; _w&k %end; _a_ $&maxavlen;
				%if %length(&AnnoGroup2) %then %do;
					length _new_id locus_lab $32 _Coding_ $9;
					set trans;
					by &AnnoGroup2 _rare_;
				%end;
				%else %do;
					%if %length(&AnnoMAF) %then %do;
						length _Coding_ $9;
					%end;
					set trans; set afsub;
				%end;
				locus_lab = locus;
				array samp{&nobs} indiv:;
				array _ng{&nobs} 8 ;
				%if %length(&AnnoGroup2) %then %do;
					retain _ng 0;
				%end;

				%if %length(&AnnoGroup2) %then %do;
					if missing(&AnnoGroup2) then missAG + 1;
				%end;
				do _i=1 to &nobs;
					%if %qupcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
						_w1=substr(put(samp{_i},numtochar.),1,1);
						_w2=substr(put(samp{_i},numtochar.),3,1);					
					%end;
					%else %if %substr(&MarkerFmt,1,1)=%str(S) %then %do;
						_w1=substr(put(samp{_i},$singlechar.),1,1);
						_w2=substr(put(samp{_i},$singlechar.),3,1);				
					%end;	
					%else %if %length(&GenoDelim) %then %do;
						_w1=compress(scan(samp{_i},1,"&GenoDelim"));
						_w2=strip(trim(scan(samp{_i},2,"&GenoDelim")));
					%end;
					%else %do;
						/*nondelimited genotype*/
						%do k= 1 %to &PloidyNum;
							_w&k=substr(samp{_i},&k,1);
						%end;
					%end;				
					_a_=strip(&AnnoMajAllele);
					if lengthn(_a_)=0 or missing(_a_) or _a_="." then call symputx("anyblankmajall",1);

					_Coding_ = "%scan(&GenotypeConversion,2)";

					%if %length(&AnnoGroup2) %then %do;
						if _rare_=1 then do;
							_Coding_ = "Collapse";
							if first._rare_ or missing(&AnnoGroup2) then do; /* Don't want to group together SNPs with missing AnnoGroup2 */
								if missing(_w1) or missing(_w2) %do k= 3 %to &PloidyNum; or missing(_w&k) %end; then _ng{_i}=.;
								else _ng{_i} = 1 - (_w1=_a_ and _w2=_a_ %do k= 3 %to &PloidyNum; and _w&k = _a_ %end; );
							end;
							/* Use Dominant coding for rare variants */
							else if missing(_w1)=0 and missing(_w2)=0 %do k= 3 %to &PloidyNum; and missing(_w&k) = 0 %end; then _ng{_i} = max(_ng{_i}, 1 - (_w1=_a_ and _w2=_a_ %do k= 3 %to &PloidyNum; and _w&k = _a_ %end;));
						end;
						else do;
					%end;

					if missing(_w1) or _w1="." or missing(_w2) or _w2="." %do k= 3 %to &PloidyNum; or missing(_w&k) or _w&k="." %end; then _ng{_i}=.;

					%if %length(&AnnoMAF) %then %do;
						/* Use Dominant coding for rare variants */
						else if &AnnoMAF <= &MAF then do;
							_ng{_i} = 1 - (_w1=_a_ and _w2=_a_ %do k= 3 %to &PloidyNum; and _w&k = _a_ %end;);
							_Coding_ = "Dominant";
						end;
					%end;
					
					%if %substr(&GenotypeConversion,9,1)=%str(A) %then %do;
						else _ng{_i}= &PloidyNum - ((_w1=_a_) + (_w2=_a_) 
									%do k= 3 %to &PloidyNum;   
										+ (_w&k=_a_)
									%end;
									);
					%end;
					%else %if %substr(&GenotypeConversion,9,1)=%str(D) %then %do;
						else _ng{_i} = 1 - (
									_w1=_a_ and _w2=_a_ 
										%do k= 3 %to &PloidyNum;   
											and _w&k=_a_ 
										%end;
								    );
					%end;
					%else %if %substr(&GenotypeConversion,9,1)=%str(H) %then %do;
						else _ng{_i}= (
								_w1=_a_ or _w2=_a_
									%do k= 3 %to &PloidyNum;   
										or _w&k=_a_ 
									%end;							
								) - 
								(
								_w1=_a_ and _w2=_a_
									%do k= 3 %to &PloidyNum;   
										and _w&k=_a_ 
									%end;								
								);
					%end;
					%else %do;
						else _ng{_i}= 1 - (
									_w1=_a_ or _w2=_a_
										%do k= 3 %to &PloidyNum;   
											or _w&k=_a_ 
										%end;								
								   );
					%end;		
					%if %length(&AnnoGroup2) %then %do;
						end; /* end else for _rare_=1 */
					%end;
				end;
				drop indiv: _i _w1 _w2 %do k= 3 %to &PloidyNum; _w&k %end; _a_;

				%if %length(&AnnoGroup2) %then %do;
					%if %upcase(&AppendPrefix)=YES %then %do;
						_new_id = locus;
					%end;
					%else %do;
						_new_id = strip(_order_);
					%end;
					/* Only output last in group */
					if _rare_=0 or last.&AnnoGroup2 or missing(&AnnoGroup2) then do;
						if (last.&AnnoGroup2 or missing(&AnnoGroup2)) and _rare_=1 then do;
							if missing(&AnnoGroup2) then do;
								_new_id = cats("_RV_Missing",missAG);
								locus_lab = cat("RV for Missing",missAG);						
							end;
							else do;
								_new_id = cats("_RV_",&AnnoGroup2);
								locus_lab = cat("Collapsed RVs for ",strip(&AnnoGroup2));
							end;
						end;
						%if &c=&NChunks %then %do;
							drop _rare_ missag;
							output afsub;
						%end;					
				%end;

				if &fobs <= _order_ <= &lobs then output mrge&Block;
				%let Blockt=%eval(&Block+1);
				%let fobst=&lobs;
				%let lobst=%eval(&lobs+&BlockSize);
				%do %while (&Blockt le &BinC);

					else if &fobst <= _order_ <= &lobst then output mrge&Blockt;

					%let Blockt=%eval(&Blockt+1);
					%let fobst=%eval(&lobst+1);
					%let lobst = %eval(&lobst+&BlockSize);
				%end;
				%if %length(&AnnoGroup2) %then %do;
					end;	
				%end;
				%if &c=&NChunks and %length(&AnnoGroup2)=0 and %length(&AnnoMAF) %then %do;
					output afsub;
				%end;
			run;
			
			%put anyblankmajall= &anyblankmajall;
			%if &anyblankmajall=1 %then %do;
				%*put ERROR: The Annotation Major Allele is not defined or all alleles are missing for one or more markers.  Please check your input and/or annotation data sets.;
				%put WARNING: The Annotation Major Allele is not defined or all alleles are missing for one or more markers.  Please check your input and/or annotation data sets.;
			%end;

			options label;

			%let FirstBlock=&Block;

			%do %while (&Block le &BinC);

				proc sort data=mrge&Block;
					by _order_;
				run;

				proc transpose data=mrge&Block out=retrans&Block(drop=_name_) prefix=&Prefix;
					var _ng1 - _ng&nobs;
					%if %length(&AnnoGroup2) %then %do;
						id _new_id;
					%end;
					%else %do;
						%if %upcase(&AppendPrefix)=YES %then %do;
							id locus;
						%end;
						%else %do;
							id _order_;
						%end;
					%end;
					%if %length(&AnnoInData) %then %do;
						idlabel locus_lab;
					%end;
				run;
				%if &syserr>4 %then %let exiterror=1;

				proc datasets nolist;
					delete mrge&Block;
				quit;

				%if &Block=&BinC %then %do;

					data &NumGenoData;
						%if &c=1 %then %do;
							set other;
						%end;
						%else %do;
							set &NumGenoData;
						%end;
						%do b=&FirstBlock %to &Block;/* Separate SET statements instead of a single MERGE are processed much faster! */
							set retrans&b;
						%end;
						/*
						%if &c=&NChunks %then %do;
							label %do i=1 %to &nm; %unquote(&Prefix.&i)="&&MLabel&i" %end;; 					
						%end;			
						*/
					run;			

					proc datasets nolist;
						delete %do b=&FirstBlock %to &Block; retrans&b %end;;
					quit;
				%end;

				%let fobs = %eval(&lobs + 1);
				%let lobs = %eval(&fobs + &BlockSize - 1);
				%if &lobs > &nm %then %let lobs = &nm;	
				%let Block = %eval(&Block + 1);						
			%end;	
			options nolabel;
		%end; /* end chunk */

		%if %length(&AnnoGroup2) %then %do;
			proc sort data=afsub;
				by _order_;
			run;
		%end;

		proc datasets nolist;
			delete trans;
		quit;
	%end;
	%else %do;
		/*Output: genotype string*/
		%if %substr(&MarkerFmt,1,1) = %str(A) %then %let size=%eval(&nvars/2);
		%else %let size=&nvars;

		%let dropavars = &AlleleVars;
		%let colon=%index(&ListMarkerVars,%str(:));
		%if &colon %then %do;
			%let colonpref = %substr(&ListMarkerVars,1,%eval(&colon-1));
			%if %index(%upcase(&Prefix),%upcase(&colonpref))=1 %then %let dropavars = %quote(&firstavar--&lastavar);
		%end;

		%if %substr(&MarkerFmt,1,1) = %str(A) %then %let preflen = 33;
		%else %let preflen = %eval(2*&PloidyNum - 1);

		data OutLib.&NumGenoDataName;
			set &ds;
			%if %length(&WhereStmt) %then %do;
				where &WhereStmt;
			%end;
			%let annoid=%sysfunc(open(mapdata,i));
			%let labelnum=%sysfunc(varnum(&annoid,&AnnoLabel));
			array _markers{*} &AlleleVars;
			array &Prefix{&size} $&preflen;

			do _i=1 to dim(&Prefix);
				%if %substr(&MarkerFmt,1,1) = %str(S) %then %do;
					select(strip(_markers{_i}));
						when("B") &Prefix{_i}="B/B";
						when("H","AB") &Prefix{_i}="A/B";
						when("A") &Prefix{_i}="A/A";
						otherwise &Prefix{_i}="";
					end;
				%end;
				%else %if %qupcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
					select(_markers{_i});
						when(0) &Prefix{_i}="B/B";
						when(1) &Prefix{_i}="A/B";
						when(2) &Prefix{_i}="A/A";
						otherwise &Prefix{_i}="";
					end;
				%end;
				%else %if %substr(&MarkerFmt,1,1) = %str(A) %then %do;
					if missing(_markers{2*_i-1}) or missing(_markers{2*_i}) then &Prefix{_i}="";
					else &Prefix{_i}=cats(_markers{2*_i-1},"/",_markers{2*_i});
				%end;
				%else %if %length(&GenoDelim) %then %do;
					&Prefix{_i}=_markers{_i};
				%end;
				%else %do;
				/*nondelimted */
					if lengthn(_markers{_i})=&PloidyNum then 
						&Prefix{_i}=cats(substr(_markers{_i},1,1),"/",substr(_markers{_i},2,1) %do k=3 %to &PloidyNum; ,"/",substr(_markers{_i},&k,1) %end;);
					else &Prefix{_i}="";
				%end;
			end;

			%do _i=1 %to &size;
				%let rc=%sysfunc(fetchobs(&annoid,&_i));
				%let snplabel=%sysfunc(getvarc(&annoid,&labelnum));
				label %unquote(&Prefix.&_i)="&snplabel";
			%end;
			%let rc=%sysfunc(close(&annoid));

			/* need to not use &Allelevars in case SOMETHING: includes prefix */
			drop &dropavars _i;
		run;

		%let NDigits = %sysevalf(%sysfunc(floor(%sysfunc(log10(&nvars))))+1);

	%end;

	%if &syserr>4 %then %goto exit;

	%if %length(&AnnoLabel)>29 %then %let AnnoSubL = %substr(&AnnoLabel,29);
	%else %let AnnoSubL = &AnnoLabel;

	%if %length(&AnnoGroup2) %then %let addl=11;
	%else %let addl=0;

	%if %upcase(&AppendPrefix)=YES %then %do;
		%if %length(&AnnoGroup2) %then %let longest = %sysfunc(max(%eval(&NDigits+&addl),&annolen));
		%else %let longest=&annolen;
		%let newannolen = %eval(%length(&Prefix)+&longest);
	%end;
	%else %let newannolen = %eval(%length(&Prefix)+&NDigits+&addl);

	data OutLib.&NumGenoData_OutAnnoDataName;
		length &AnnoSubL._RG $&newannolen &AnnoLabel $&annolen; 
		%if %substr(&GenotypeConversion,1,1) ne %str(G) %then %do;
			set afsub(%if %upcase(&AnnoGroup2)=%upcase(&AnnoLabel) %then drop=&AnnoLabel; rename=(locus = &AnnoLabel));
			drop locus_lab;
		%end;
		%else %do;
			set mapdata;
		%end;
		%if %length(&AnnoGroup2) %then %do;
			&AnnoSubL._rg = "&prefix"||strip(_new_id);
			drop _new_id;
		%end;
		%else %do;
			%if %upcase(&AppendPrefix)=YES %then %do;
				&AnnoSubL._rg = "&prefix"||strip(&AnnoLabel);			
			%end;
			%else %do;
				&AnnoSubL._rg = "&prefix"||strip(_order_);
			%end;
		%end;
		drop _group_ _position_ _order_;
	run;

	%if %upcase(&AppendPrefix)=YES %then %Naming(_Dset=OutLib.&NumGenoData_OutAnnoDataName,_AllVars=&AnnoSubL._rg);

	%PackageInsertDataset(Recoded genotype data from &ProcessName, OutLib, &NumGenoDataName, no);
	%PackageInsertDataset(Annotation data from &ProcessName, OutLib, &NumGenoData_OutAnnoDataName, no, _type=anno);

%end;
%else %do;
	/*Genotype recoding for breeding*/
	data markernames;
		set &ds;
		where 0;
		keep &AlleleVars;
	run;

	%let _imn_= %sysfunc(open(markernames));
	%let _nmk_= %sysfunc(attrn(&_imn_,NVARS));
	%put _nmk_= &_nmk_;

	proc transpose data=markernames out=markernamest;
	run;

	data markernamest;
		set markernamest;
		addname= trim(left(_name_))||"_a";
		domname= trim(left(_name_))||"_d";
	run;

	proc transpose data=markernamest out= addnamedat(drop=_name_);
		*var addname;
		id addname;
	run;

	%let _amn_= %sysfunc(open(addnamedat));

	data dat_a;
		set &ds;
		array a {*} &AlleleVars;
		do i=1 to dim(a);
			if a[i] = 0 then a[i]=-1;
			else if a[i] = &PloidyNum then a[i]=1;
			else if a[i] ^=. then a[i]=0;
			else a[i]=.;
		end;
		rename 
			%do i=1 %to &_nmk_;
				%sysfunc(varname(&_imn_,&i)) = %sysfunc(varname(&_amn_,&i)) 
			%end; 
		;
		drop i;
		keep &AlleleVars;
	run;

	proc transpose data=markernamest out= domnamedat(drop=_name_);
		*var addname;
		id domname;
	run;

	%let _dmn_= %sysfunc(open(domnamedat));

	data dat_d;
		set &ds;
		array d {*} &AlleleVars;
		do i=1 to dim(d);
			if d[i]=0 then d[i]=0;
			else if d[i]=&PloidyNum then d[i]=0;
			else if d[i]^=. then d[i]=1;
			else d[i]=.;
			
		end;
		rename 
			%do i=1 %to &_nmk_;
				%sysfunc(varname(&_imn_,&i)) = %sysfunc(varname(&_dmn_,&i)) 
			%end; 
		;
		drop i;
		keep &AlleleVars;
	run;

	%let _rc_= %sysfunc(close(&_imn_));
	%let _rc_= %sysfunc(close(&_amn_));
	%let _rc_= %sysfunc(close(&_dmn_));

	data OutLib.&NumGenoDataName;;
		merge &ds(drop= &AlleleVars) dat_a dat_d;
	run;

	%PackageInsertDataset(Recoded genotype data from &ProcessName, OutLib, &NumGenoDataName, no);

%end;

%global RecodeGScript;
%PrepOutFile(RecodeGScript,RecodeGScriptName,&ProcessName Script for &OutName..jsl);

%global _mainjslfile_;
%let _mainjslfile_ = %quote(&RecodeGScript);

**creating a JSL file without code just to force the final jsl file to contain the process name on it instead of sasmessage**;
* create JSL code;
data _null_;
	file "&RecodeGScript" lrecl=32767;
	put '//!';
	put "Names Default to Here(1);";
	put "Here:DataList = {};";
run;
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &RecodeGScriptName, , text/plain, yes, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend;

* run the macro;
%RecodeGenotypes;
