/* 
JMP Genomics Analytical Process

Name: ReferenceNormalization
Author: Li Li, SAS Institute Inc.
Support: Li Li, SAS Institute Inc.
History:
   saslii 10Nov13 initial coding 

Description: 
The Reference (housekeeping) gene normalization is a 
straightforward normalization method for Nanostring Count data. 
It is divided the average by the geometric mean in each lane to get 
a lane-specific normalization factor.
*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData Vars IDVar ListVars TransMethod ShiftFactor ScalingFactor ControlGene ReferenceGene posMeth negMeth
	Scaling_OutData AnnoInData AnnoNameVar OutData OutPath ServerOutPath MacroPath StableGenesOut RmRef;

*ProcessBody;

%macro CalPosCtrl(scalingfactor);
	%let _tid_ = %sysfunc(open(&scalingfactor));
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS)); 
	%let _nObs_ = %sysfunc(attrn(&_tid_,NOBS));
	%do _i_ = 1 %to &_nVars_;
	%let _ExpVar_ = %sysfunc(varname(&_tid_,&_i_));
		%if %upcase(&_ExpVar_) = %upcase(&IDVar) %then %do;
			%do _j_ = 1 %to &_nObs_;
				%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
				%let _colName_&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&IDVar))));
				%put colName=&&_colName_&_j_;
			%end;
		%end;
	%end;
	%let _tid_ = %sysfunc(close(&_tid_));
	
	proc transpose data = &scalingfactor out = OutLib.&Scaling_OutData;
		var &posGenes;
		id &IDVar ;
	run;
	
	data OutLib.&Scaling_OutData;
		set OutLib.&Scaling_OutData;
		Label _NAME_=Gene;
		rename _NAME_=Gene;
	run;

	%if %index(&scalingMeth,Geometric) %then %do;
		data log_scaling;
			set OutLib.&Scaling_OutData;
			%do _j_ = 1 %to &_nObs_; 
				&&_colName_&_j_ =log(&&_colName_&_j_);
			%end;		
		run;
		proc means data=log_scaling noprint;
			var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
			output out=mean_log ;
		run;
		data mean_log;
			set mean_log;
			if _STAT_="MEAN";
			%do _j_ = 1 %to &_nObs_; 
				&&_colName_&_j_ =exp(&&_colName_&_j_);
			%end;
		run;
		data _null_;
			set mean_log;
			%do _j_ = 1 %to &_nObs_; 
				call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
				%put &&scaleFactor&_j_;
			%end;;
		run;
		%let sum_mean=0;
		%do _j_ = 1 %to &_nObs_;
			%let sum_mean=%sysevalf(&sum_mean+&&scaleFactor&_j_);
			%put &sum_mean;
		%end;
		%do _j_ = 1 %to &_nObs_;
			%let scaleFactor&_j_=%sysfunc(round(%sysevalf(&sum_mean/(&&scaleFactor&_j_*&_nObs_)),0.01));
			%put scaleFactor=&&scaleFactor&_j_;
		%end;
	%end;
	%else %do;
	*calculate arithmetic mean;
		proc summary data=OutLib.&Scaling_OutData;
			var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
			output out=_p1 sum=;
		run;
		/*
		data _p1;
			set _p1;
			if _STAT_="MEAN";
		run;
		*/

		data _p1;
			set _p1;
			drop _TYPE_ _FREQ_;
		run;

		data _null_;
			set _p1;
			%do _j_ = 1 %to &_nObs_; 
				call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
				%put &&scaleFactor&_j_;
			%end;;
		run;
		%let sum_mean=0;
		%do _j_ = 1 %to &_nObs_;
			%let sum_mean=%sysevalf(&sum_mean+&&scaleFactor&_j_);
		%end;

		%do _j_ = 1 %to &_nObs_;
			%let scaleFactor&_j_=%sysfunc(round(%sysevalf(&sum_mean/(&&scaleFactor&_j_*&_nObs_)),0.01));
			%put &&scaleFactor&_j_;
		%end;
	%end;

	/*
	proc transpose data = inlib.edf_wide out = OutData_trans;
		id &IDVar ;
	run;

	proc sql noprint;
	create table inlib.edf_wide_trans_refGenes as
	select *
	from inlib.edf_wide_trans
	where ((upcase(GeneName) like 'POS_%') | (upcase(GeneName) like 'NEG_%') | (upcase(GeneName) like 'HOUSE_%'));
	quit;
	
	proc sql noprint;
	create table inlib.edf_wide_trans_noRefGenes as
	select *
	from inlib.edf_wide_trans
	where ((upcase(GeneName) not like 'POS_%') & (upcase(GeneName) not like 'NEG_%') & (upcase(GeneName) not like 'HOUSE_%'));
	quit;*/

	data indata_trans_noRefGenes;
		set indata_trans_noRefGenes;
		%do _j_ = 1 %to &_nObs_; 
			&&_colName_&_j_=&&_colName_&_j_*&&scaleFactor&_j_;;	
 			%if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Log2) %then %do;
 				%if &&_colName_&_j_ >0 %then %do;	
				 	&&_colName_&_j_ = log2(&&_colName_&_j_);
				%end;
				%else %do;
					&&_colName_&_j_ =.;
				%end;
			 %end;
			 %else %if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Shifted log2) %then %do;
			 	%if &&_colName_&_j_ >0 %then %do;
			 		&&_colName_&_j_ = log2(&&_colName_&_j_+&ShiftFactor);
			 	%end;
			 	%else %do;
			 		&&_colName_&_j_.;
			 	%end;
			 %end;
		%end;
	run;

	data OutLib.&OutData;
		set indata_trans_refGenes indata_trans_noRefGenes;;
		Label GeneName="Wide Name";
		rename GeneName=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;

	%if %upcase(&ReferenceGene) = NONE %then %do;
		data OutLib.&Scaling_OutData;
			length ColumnName $ 20;
			%do _j_ = 1 %to &_nObs_; 
				ColumnName = "&&_colName_&_j_";
				ScalingFactor=&&scaleFactor&_j_;
				output;
			%end;
		run;
	%end;

%mend;


%macro CalNegCtrl(scalingfactor);
	%let _tid_ = %sysfunc(open(&scalingfactor));
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS)); 
	%let _nObs_ = %sysfunc(attrn(&_tid_,NOBS));
	%put &_nVars_;
	%do _i_ = 1 %to &_nVars_;
	%let _ExpVar_ = %sysfunc(varname(&_tid_,&_i_));
		%if %upcase(&_ExpVar_) = %upcase(&IDVar) %then %do;
			%do _j_ = 1 %to &_nObs_;
				%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
				%let _colName_&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&IDVar))));
				%put colName=&&_colName_&_j_;
			%end;
		%end;
	%end;
	%let _tid_ = %sysfunc(close(&_tid_));

	proc transpose data = &scalingfactor out = OutLib.&Scaling_OutData;
	var &negGenes;
	id &IDVar ;
	run;
	/*gene x sample*/
	data OutLib.&Scaling_OutData;
		set OutLib.&Scaling_OutData;
		Label _NAME_=Gene;
		rename _NAME_=Gene;
	run;
	
	%if %index(&negMeth,Mean+2Std) %then %do;
		proc summary data=OutLib.&Scaling_OutData;
		var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
		output out=_p1 std=;
		run;

		proc summary data=OutLib.&Scaling_OutData;
		var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
		output out=_p2 mean=;
		run;

		data _null_;
			set _p2;
			%do _j_ = 1 %to &_nObs_; 
				call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
			%end;;
		run;
		
		data _null_;
			set _p1;
			%do _j_ = 1 %to &_nObs_; 
				call symput ("scaleFactorStd&_j_",trim(left(&&_colName_&_j_ )));
			%end;;
		run;

		%do _j_ = 1 %to &_nObs_;
			%let scaleFactor&_j_=%sysfunc(round(%sysevalf((&&scaleFactor&_j_+2*&&scaleFactorStd&_j_)),0.01));
			%put scaleFactor=&&scaleFactor&_j_;
		%end;

	%end;
	%else %if %index(&negMeth,Mean) %then %do;
		proc summary data=OutLib.&Scaling_OutData;
		var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
		output out=_p1 mean=;
		run;
		
		data _null_;
		set _p1;
		%do _j_ = 1 %to &_nObs_; 
			call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
		%end;;
		run;

	%end;
	%else %if %index(&negMeth,Max) %then %do;
		proc summary data=OutLib.&Scaling_OutData;
		var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
		output out=_p1 max=;
		run;
		
		data _null_;
		set _p1;
		%do _j_ = 1 %to &_nObs_; 
			call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
		%end;;
		run;
	%end;

	/*proc transpose data = inlib.edf_wide out = OutData_trans;
		id &IDVar ;
	run;

	proc sql noprint;
	create table inlib.edf_wide_trans_refGenes as
	select *
	from inlib.edf_wide_trans
	where ((upcase(GeneName) like 'POS_%') | (upcase(GeneName) like 'NEG_%') | (upcase(GeneName) like 'HOUSE_%'));
	quit;
	
	proc sql noprint;
	create table inlib.edf_wide_trans_noRefGenes as
	select *
	from inlib.edf_wide_trans
	where ((upcase(GeneName) not like 'POS_%') & (upcase(GeneName) not like 'NEG_%') & (upcase(GeneName) not like 'HOUSE_%'));
	quit;*/
	
	data indata_trans_noRefGenes;
		set indata_trans_noRefGenes;
		%do _j_ = 1 %to &_nObs_; 
			/*&&_colName_&_j_=&&_colName_&_j_*&&scaleFactor&_j_;;*/
			&&_colName_&_j_=&&_colName_&_j_ - &&scaleFactor&_j_;
			
			/*%if &&_colName_&_j_<0 %then %do;
				&&_colName_&_j_=0;
			%end;
 			%if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Log2) %then %do;
			 	&&_colName_&_j_ = log2(&&_colName_&_j_);
			%end;
			%else %if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Shifted log2) %then %do;
			 	&&_colName_&_j_ = log2(&&_colName_&_j_+&ShiftFactor);
			%end;*/
		%end;
	run;
	
	**modify;
	data indata_trans_noRefGenes(drop=i);
		set indata_trans_noRefGenes;
		array _n(*) _numeric_;
		do i=1 to dim(_n);
			if _n(i)<0 then _n(i)=0;
		end;
		%do _j_ = 1 %to &_nObs_; 
			%if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Log2) %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_);
			%end;
			%else %if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Shifted log2) %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_+&ShiftFactor);
			%end;
				
		%end;
	run;
	/*
	data indata_trans_noRefGenes;
		set indata_trans_noRefGenes;
		%do _j_ = 1 %to &_nObs_; 
			%if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Log2) %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_);
			%end;
			%else %if %upcase(&ReferenceGene) = NONE & %index(&TransMethod,Shifted log2) %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_+&ShiftFactor);
			%end;
		
		%end;
	run;
	*/
	
	data OutLib.&OutData;
		set indata_trans_refGenes indata_trans_noRefGenes;;
		Label GeneName="Wide Name";
		rename GeneName=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;

	%if %upcase(&ReferenceGene) = NONE %then %do;
		data OutLib.&Scaling_OutData;
			length ColumnName $ 20;
			%do _j_ = 1 %to &_nObs_; 
				ColumnName = "&&_colName_&_j_";
				ScalingFactor=&&scaleFactor&_j_;
				output;
			%end;
		run;
	%end;

%mend;

%macro ReferenceNormalization;
*initialization and argument checking;
%let ProcessName = ReferenceNormalization;
%include "&MacroPath./UtilityMacros.sas" / nosource;

%GetOrigFileName(InData,OrigInData);

%CheckInData(_ExpFlag=0);

* check name for OutData;
%if %length(&OrigInData) < 22 %then %let OID = &OrigInData;
%else %let OID = %substr(&OrigInData,1,21);

* add listvars to vars;
%if %length(&ListVars) | %length(&Vars) %then %do;
	%CheckInData2(InLib.&InData,_Vars=&Vars,_VarListStyle=&ListVars);
	%let nV = &_nTallCol_;
%end;

%let InData1 = InLib.&InData;

%if %length(&InData) >22 %then %let ScalName = %substr(&InData,1,21);
%else %let ScalName =&InData;

%if %index(&ControlGene, Positive Control) & %index(&ReferenceGene,None) %then %let sufix=_pos;
%else %if %index(&ControlGene, Negative Control) & %index(&ReferenceGene,None) %then %let sufix=_neg;
%else %if %index(&ControlGene, None) & %index(&ReferenceGene,Stable Genes) %then %let sufix=_stbl;
%else %if %index(&ControlGene, None) & %index(&ReferenceGene,Housekeeping Genes) %then %let sufix=_hskp;
%else %if %index(&ControlGene, Positive Control) & %index(&ReferenceGene,Housekeeping Genes) %then %let sufix=_poshskp;
%else %if %index(&ControlGene, Negative Control) & %index(&ReferenceGene,Housekeeping Genes) %then %let sufix=_neghskp;
%else %if %index(&ControlGene, Positive Control) & %index(&ReferenceGene,Stable Genes) %then %let sufix=_posstbl;
%else %if %index(&ControlGene, Negative Control) & %index(&ReferenceGene,Stable Genes) %then %let sufix=_negstbl;
%else %if %index(&ControlGene, None) & %index(&ReferenceGene,None) %then %let sufix=_nonorm;

%if %index(&TransMethod,None)=0 %then %let sufix=&sufix._lg;
%else %let sufix=&sufix;

/*change the sufix names*/
%checkdataname(_name=OutData,_default=%trim(&ScalName)&sufix,_SDS=1);
%checkdataname(_name=Scaling_OutData,_default=%trim(&ScalName)_scf,_SDS=1);
%checkdataname(_name=StableGenesOut,_default=%trim(&ScalName)_stblgen,_SDS=1);

data OutLib.&OutData;	
	set InLib.&InData;
run;
%put Outside &ReferenceGene;

proc transpose data = OutLib.&OutData 
	out = inDataTmpT
	name=GeneName;
	id &IDVar;
run;

proc sql noprint;
create table indata_trans_refGenes as
select *
from inDataTmpT
where ((upcase(GeneName) like 'POS_%') | (upcase(GeneName) like 'NEG_%') | (upcase(GeneName) like 'HOUSE_%'));
quit;

proc sql noprint;
create table indata_trans_noRefGenes as
select *
from inDataTmpT
where ((upcase(GeneName) not like 'POS_%') & (upcase(GeneName) not like 'NEG_%') & (upcase(GeneName) not like 'HOUSE_%'));
quit;

/*add exceptions*/
%if %index(&ControlGene, None) AND %index(&ReferenceGene, None) %then %do;
	%if %index(&TransMethod,Log2) %then %do;
		data indata_trans_noRefGenes(drop=i);
		set indata_trans_noRefGenes;
		array _n(*) _numeric_;
		do i=1 to dim(_n);
			_n(i)=log2(_n(i));
		end;
		run;
		
		data OutLib.&OutData;
			set indata_trans_refGenes indata_trans_noRefGenes;;
			Label GeneName="Wide Name";
			rename GeneName=widename;
			label _LABEL_ = "Name";
			rename _LABEL_ = name;
		run;
	%end;
	%else %if %index(&TransMethod,Shifted log2) %then %do;
		data indata_trans_noRefGenes(drop=i);
		set indata_trans_noRefGenes;
		array _n(*) _numeric_;
		do i=1 to dim(_n);
			_n(i)=log2(_n(i)+&ShiftFactor);
		end;
		run;
		
		data OutLib.&OutData;
			set indata_trans_refGenes indata_trans_noRefGenes;;
			Label GeneName="Wide Name";
			rename GeneName=widename;
			label _LABEL_ = "Name";
			rename _LABEL_ = name;
		run;
	%end;
	%else %do;
		data OutLib.&OutData;
			set inDataTmpT;
			Label GeneName="Wide Name";
			rename GeneName=widename;
			label _LABEL_ = "Name";
			rename _LABEL_ = name;
		run;
	%end;
%end;

%if %index(&ControlGene, Positive Control) %then %do;
	proc sql noprint;
	select GeneName, count(*) into :posGenes separated by ' ', :posGeneNum
	from inDataTmpT
	where upcase(GeneName) like 'POS_%';
	quit;
	
	%if(%length(&posGenes)) %then %do;
		data ScalingFactorDt;	
			set InLib.&InData;
			keep 	&IDVar &posGenes;
		run;
	%end;
	%else %do;
		%put ERROR: No Positive Control Gene has been detected.;
		%goto exit;
	%end;
	
	%CalPosCtrl(ScalingFactorDt);
	

	* Multiply all the endogenous counts in a lane by that lanes normalization factor except control counts;
	/* KELCI - right now it's normalizing all factors.  Reassess */
	/*proc transpose data = InLib.&InData out = OutData_trans;
		id &IDVar ;
	run;
	
	data OutLib.&OutData;
		set OutData_trans;
		%do _j_ = 1 %to &_nObs_; 
			&&_colName_&_j_=&&_colName_&_j_*&&scaleFactor&_j_;;		 
		%end;
		Label _NAME_="Wide Name";
		rename _NAME_=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;*/
	
	
%end;

%if %index(&ControlGene, Negative Control) %then %do;
	proc sql noprint;
	select GeneName, count(*) into :negGenes separated by ' ', :negGeneNum
	from inDataTmpT
	where upcase(GeneName) like 'NEG_%';
	quit;
	
	%if(%length(&negGenes)) %then %do;
		data ScalingFactorDt;	
			set InLib.&InData;
			keep 	&IDVar &negGenes;
		run;
	%end;
	%else %do;
		%put ERROR: No Negative Control Gene has been detected.;
		%goto exit;
	%end;
	
	%CalNegCtrl(ScalingFactorDt);
	
	
	* Multiply all the endogenous counts in a lane by that lanes normalization factor except control counts;
	/* KELCI - right now it's normalizing all factors.  Reassess */
	/*proc transpose data = InLib.&InData out = OutData_trans;
		id &IDVar ;
	run;
	
	data OutLib.&OutData;
		set OutData_trans;
		%do _j_ = 1 %to &_nObs_; 
			&&_colName_&_j_=&&_colName_&_j_*&&scaleFactor&_j_;;		 
		%end;
		Label _NAME_="Wide Name";
		rename _NAME_=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;*/
%end;

%if %index(&ReferenceGene,None)=0 %then %do;
	%if %index(&ReferenceGene,Stable Genes) %then %do;
		%put Inside &ReferenceGene;
		/*
		proc transpose data = OutLib.&OutData 
			out = tmpStableGene
			name=GeneName;
			id &IDVar;
		run;
		*/
		
		%put After transpose;

		data indata_trans_noRefGenes;
		set indata_trans_noRefGenes;
		array _n(*) _numeric_;
		genemean=mean(of _n[*]);
		genecv=std(of _n[*])/mean(of _n[*]);
		run;

		proc sql noprint;
		select median(genemean), median(genecv) into :medianMean, :medianCV
		from indata_trans_noRefGenes;
		quit;

		%put &medianMean and &medianCV;

		proc sql noprint;
		create table candidategenes
		as select GeneName, genemean, genecv
		from indata_trans_noRefGenes
		where genemean>&medianMean AND genecv<&medianCV
		order by genecv;
		quit;
		
		Data indata_trans_noRefGenes;
			set indata_trans_noRefGenes(drop=genemean genecv);
		run;

		proc sql noprint;
		create table OutLib.&StableGenesOut
		as select GeneName from candidategenes(obs=10);
		quit;

		proc sql noprint;
		select GeneName, count(*) into :stablegenes separated by ' ', :genenum
		from candidategenes (obs=10);
		quit;
		
		%put &stablegenes;
	%end;
	
	%if %index(&ControlGene, None) %then %do;
	/*Keep housekeeping genes, add low cv genes*/
		%if %index(&ReferenceGene,Housekeeping Genes) & (%length(&ListVars) | %length(&Vars)) %then %do;
			data ScalingFactorDt;	
				set InLib.&InData;
				keep 	&IDVar %do i = 1 %to &_nAllColChunk_;  &&_AllCol_&i._;	%end;;
			run;
		%end;
		%else %if %index(&ReferenceGene,Stable Genes) & (%length(&stablegenes)) %then %do;
			data ScalingFactorDt;	
				set InLib.&InData;
				keep 	&IDVar &stablegenes;
			run;
		%end;
		%else %do;
			data ScalingFactorDt;	
				set InLib.&InData;
			run;
		%end;
	%end;
	%else %do;
		proc transpose data=OutLib.&OutData out = outdataT;
			id widename;
		run;
		data outdataT;
			set outdataT;
			Label _NAME_="ColumnName";
			rename _NAME_=ColumnName;
		run;

		/*Keep housekeeping genes, add low cv genes*/
		%if %index(&ReferenceGene,Housekeeping Genes) & (%length(&ListVars) | %length(&Vars)) %then %do;
			data ScalingFactorDt;	
				set outdataT;
				keep 	&IDVar %do i = 1 %to &_nAllColChunk_;  &&_AllCol_&i._;	%end;;
			run;
		%end;
		%else %if %index(&ReferenceGene,Stable Genes) & (%length(&stablegenes)) %then %do;
			data ScalingFactorDt;	
				set outdataT;
				keep 	&IDVar &stablegenes;
			run;
		%end;
		%else %do;
			data ScalingFactorDt;	
				set outdataT;
			run;
		%end;

	%end;
	
	/*
	libname c "C:\temp";
	data c.sf;
		set ScalingFactor;
	run;
	*/

	* get sample information;

	%let _tid_ = %sysfunc(open(ScalingFactorDt));
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS)); 
	%let _nObs_ = %sysfunc(attrn(&_tid_,NOBS));
	%do _i_ = 1 %to &_nVars_;
	%let _ExpVar_ = %sysfunc(varname(&_tid_,&_i_));
		%if %upcase(&_ExpVar_) = %upcase(&IDVar) %then %do;
			%do _j_ = 1 %to &_nObs_;
				%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
				%let _colName_&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&IDVar))));
				%put colName=&&_colName_&_j_;
			%end;
		%end;
	%end;
	%let _tid_ = %sysfunc(close(&_tid_));

	/*Get sample information wrapped*/
	/*%getSampleInfo(&ScalingFactorDt);*/

	/*The place to transpose subsetted genes into scaling Factor. Add low CV genes*/
	proc transpose data = ScalingFactorDt out = OutLib.&Scaling_OutData;
		%if %index(&ReferenceGene,Housekeeping Genes) & (%length(&ListVars) | %length(&Vars)) %then %do;
			var %do i = 1 %to &_nAllColChunk_;  &&_AllCol_&i._;%end;;
		%end;
		%else %if %index(&ReferenceGene,Stable Genes) & (%length(&stablegenes)) %then %do;
			var &stablegenes;
		%end;
		id &IDVar ;
	run;

	data OutLib.&Scaling_OutData;
		set OutLib.&Scaling_OutData;
		Label _NAME_=Gene;
		rename _NAME_=Gene;
	run;

	*calculate geometric mean;

	%if %index(&ScalingFactor,Geometric) %then %do;
		data log_scaling;
			set OutLib.&Scaling_OutData;
			%do _j_ = 1 %to &_nObs_; 
				&&_colName_&_j_ =log(&&_colName_&_j_);
			%end;		
		run;
		proc means data=log_scaling noprint;
			var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
			output out=mean_log ;
		run;
		data mean_log;
			set mean_log;
			if _STAT_="MEAN";
			%do _j_ = 1 %to &_nObs_; 
				&&_colName_&_j_ =exp(&&_colName_&_j_);
			%end;
		run;
		data _null_;
			set mean_log;
			%do _j_ = 1 %to &_nObs_; 
				call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
			%end;;
		run;
		%let sum_mean=0;
		%do _j_ = 1 %to &_nObs_;
			%let sum_mean=%sysevalf(&sum_mean+&&scaleFactor&_j_);
		%end;

		%do _j_ = 1 %to &_nObs_;
			%let scaleFactor&_j_=%sysfunc(round(%sysevalf(&sum_mean/(&&scaleFactor&_j_*&_nObs_)),0.01));
			%put scaleFactor=&&scaleFactor&_j_;
		%end;
	%end;
	%else %do;
		*calculate arithmetic mean;
		proc summary data=OutLib.&Scaling_OutData;
			var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
			output out=_p1 sum=;
		run;

		data _p1;
			set _p1;
			drop _TYPE_ _FREQ_;
		run;

		data _null_;
			set _p1;
			%do _j_ = 1 %to &_nObs_; 
				call symput ("scaleFactor&_j_",trim(left(&&_colName_&_j_ )));
			%end;;
		run;
		%let sum_mean=0;
		%do _j_ = 1 %to &_nObs_;
			%let sum_mean=%sysevalf(&sum_mean+&&scaleFactor&_j_);
		%end;

		%do _j_ = 1 %to &_nObs_;
			%let scaleFactor&_j_=%sysfunc(round(%sysevalf(&sum_mean/(&&scaleFactor&_j_*&_nObs_)),0.01));
			%put scaleFactor=&&scaleFactor&_j_;
		%end;
	%end;
	/*Calculate means wrapped*/
	/*%calcMeans(&ScalingFactor, &Scaling_OutData);*/

	* Multiply all the endogenous counts in a lane by that lanes normalization factor except control counts;
	/* KELCI - right now it's normalizing all factors.  Reassess */
	/**/
	/*
	%if %index(&ControlGene, None) %then %do;
		proc transpose data = InLib.&InData out = OutData_trans;
			id &IDVar ;
		run;
	%end;*/
	data indata_trans_noRefGenes;
		set indata_trans_noRefGenes;
		%do _j_ = 1 %to &_nObs_; 
			&&_colName_&_j_=&&_colName_&_j_*&&scaleFactor&_j_;;	
			%if %upcase(&ReferenceGene) ^= NONE & %index(&TransMethod,Log2) %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_);
			 %end;
			 %else %if %upcase(&ReferenceGene) ^= NONE & %index(&TransMethod,Shifted log2) %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_+&ShiftFactor);
			 %end;
		%end;
	run;
	
	data OutLib.&OutData;
		set indata_trans_refGenes indata_trans_noRefGenes;;
		Label GeneName="Wide Name";
		rename GeneName=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;
	
	
/*	
	data OutLib.&OutData;
		set OutData_trans;
		if scan(_NAME_,1,'_')="POS" or scan(_NAME_,1,'_')="NEG" then delete ; 
		Label _NAME_="Wide Name";
		rename _NAME_=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;

	data OutData_trans1;
		set OutData_trans;
		if scan(_NAME_,1,'_')="POS" or scan(_NAME_,1,'_')="NEG" ;
		Label _NAME_="Wide Name";
		rename _NAME_=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = Name;
	run;

	data OutLib.&OutData;
		set OutLib.&OutData OutData_trans1;
	run;
	*/
	/*
	data OutLib.&OutData;
		set OutData_trans;
		%do _j_ = 1 %to &_nObs_; 
			&&_colName_&_j_=&&_colName_&_j_*&&scaleFactor&_j_;;
			%if %upcase(&TransMethod) = LOG2 %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_);
			 %end;
			 %else %if %upcase(&TransMethod) = SHIFTED LOG2 %then %do;
				&&_colName_&_j_ = log2(&&_colName_&_j_+&ShiftFactor);
			 %end;			 
		%end;
		Label _NAME_="Wide Name";
		rename _NAME_=widename;
		label _LABEL_ = "Name";
		rename _LABEL_ = name;
	run;*/

	data OutLib.&Scaling_OutData;
		length ColumnName $ 20;
		%do _j_ = 1 %to &_nObs_; 
			ColumnName = "&&_colName_&_j_";
			ScalingFactor=&&scaleFactor&_j_;
			output;
		%end;
	run;
%end;
	
%if %length(&AnnoInData) %then %do;
	%if %sysfunc(fileexist(&AnnoInData)) = 0 %then %do;
		%let exiterror = 1;
		%put ERROR: Specified AnnoInData does not exist.;
		%goto exit;
	%end;
%PathName(&AnnoInData);
%if &exiterror %then %do;
 %put ERROR: Annotation data set specification is invalid.;
 %goto exit;
%end;
%let AnnoData = &TmpName;
%let AnnoPath = &TmpPath;
libname AnnoLib "&AnnoPath";

%let _ordervar_  = 0;
%let _ds_ = %sysfunc(open(AnnoLib.&AnnoData));
	%let _ordervar_  = %sysfunc(varnum(&_ds_,OrigNameOrder));
%let _rc_=%sysfunc(close(&_ds_));

	proc sql;
		create table OutLib.&OutData(%if &_ordervar_ %then drop =OrigNameOrder)  as
		select * from AnnoLib.&AnnoData as d1 left join OutLib.&OutData as d2 on d1.&AnnoNameVar=d2.Name
		%if &_ordervar_ %then %do; order by  OrigNameOrder; %end;;
	quit;

	%nobsvars(OutLib.&OutData);
	%if &nobs = 0 %then %do;
		%let exiterror = 1;
		%put ERROR: Merging the normalized gene table with the specified Annotation table and Annotation Gene Name Variable resulted in an empty table.;
		%put ERROR: Please check the annotation data table and that the Name variable matches the labels of the wide input data table for normalization.;
		%goto exit;
	%end;
%end;
/*
proc transpose data = OutLib.&OutData out = OutLib.&OutData;
	%if %length(&ListVars) | %length(&Vars) %then %do;
		var %do _j_ = 1 %to &_nObs_; &&_colName_&_j_ %end;;
	%end;
	id Gene ;
run;

data OutLib.&OutData;
	set OutLib.&OutData ;
	Label _NAME_=ColumnName;
	rename _NAME_=ColumnName;
run;*/
	
%if %upcase(&RmRef) = YES %then %do;
	
	data OutLib.&OutData;
	modify OutLib.&OutData;
	if find(upcase(widename), "POS_") then remove;
	if find(upcase(widename),"NEG_") then remove;
	if find(upcase(widename),"HOUSE_") then remove;
	run;
%end;


%*PrepOutDS(OutData,OutDataName,&OID,_tall);
%if %index(&ControlGene, None)=0 | %index(&ReferenceGene, None)=0 %then %do;
	%*PrepOutDS(Scaling_OutData,Scaling_OutDataName,&OID,_scf); * Scaling factors;
%end;


%if %index(&ReferenceGene,Stable Genes) %then %do;
	%*PrepOutDS(StableGenesOut,StableGenesOutName,&OID,_scf); * Stable genes;
%end;

%PackageInsertDataset(Reference normalized data from &ProcessName,outlib,&OutData,No,_type=tall);
%if %index(&ControlGene, None)=0 | %index(&ReferenceGene, None)=0 %then %do;
	%PackageInsertDataset(Reference Scaling data from &ProcessName,outlib,&Scaling_OutData,No,_type=tall);
%end;

%if %index(&ReferenceGene,Stable Genes) %then %do;
	%PackageInsertDataset(Reference Stable Genes from &ProcessName,outlib,&StableGenesOut,No,_type=tall);
%end;

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend;

%ReferenceNormalization;
