/*
JMP Genomics Analytical Process

Name: SNP-Trait Association
Author: Wendy Czika

*/

/*
-----------------------------------------------------------
Copyright (C) 2007 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ByVars WhereStmt ListMarkerVars ListTraitVars MarkerFmt GenoDelim TraitVar TraitType ClassVars ClusterVars WeightVar
FixedEff IntEff IntTestsOnly StrataVars  AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoMajAllele AnnoAccession AnnoWhereStmt  
QTests RateOption TotalOption Odds Listing Conv MTC  PValAdj Tau Bandwidth Alpha OnlySig OutPath ServerOutPath OutName MacroPath SNPTraitAssoc_OutData SNPTraitAssocScript;
 
*ProcessBody;

%macro SurveySNPTraitAssociation;

%include "&MacroPath./GenMacros.sas" / nosource;
%include "&MacroPath./SNPTraitAssocMacro.sas" / nosource;

%GenInit(SurveySNPTraitAssociation,28);
%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(Survey SNP-Trait Association);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%global OldAlleleVars;
%let OldAlleleVars = &AlleleVars;
%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%PrepOutDS(SNPTraitAssoc_OutData,SNPTraitAssoc_OutDataName,&OutName._sst);

%PrepOutFile(SNPTraitAssocScript,SNPTraitAssocScriptName,&ProcessName Script for &OutName..jsl);

%if (%length(&QTests)=0) %then %let QTests=Genotype Trend;
%let geno=0;
%if (%index(%upcase(&QTests),GENOTYPE)>0) %then %let geno=1;
%let trend=0;
%if (%index(%upcase(&QTests),TREND)>0) %then %let trend=1;
/*S1571734: Changing the test name RaoScott_ChiSq to RaoScott_C to avoid variable name longer than 32*/
%if (%index(%upcase(&QTests),RAOSCOTT_CHISQ)>0) %then %let QTests = %sysfunc(tranwrd(%quote(&QTests),%str(RaoScott_ChiSq),%str(RaoScott_C)));
%let raoscottchi=0;
%let raoscottf=0;
%if %upcase(&TraitType) ne CONTINUOUS %then %do;
	%if (%index(%upcase(&QTests),RAOSCOTT_F)>0) %then %let raoscottf=1;
	%if (%index(%upcase(&QTests),RAOSCOTT_C)>0) %then %let raoscottchi=1;
%end;
%else %do;
	%if (%index(%upcase(&QTests),RAOSCOTT_F)>0) %then %let QTests = %sysfunc(tranwrd(%quote(&QTests),%str(RaoScott_F),%str( )));
	%if (%index(%upcase(&QTests),RAOSCOTT_C)>0) %then %let QTests = %sysfunc(tranwrd(%quote(&QTests),%str(RaoScott_C),%str( )));
	%if %length(&QTests)=0 %then %do;
		%put ERROR: You must select the Genotype and/or Trend test for continuous trait variables.;
		%goto exit;
	%end;
%end;

%global SNPTraitAssoc_PEG SNPTraitAssoc_PET;
%let SNPTraitAssoc_PEG=;
%let SNPTraitAssoc_PET=;
%if &geno=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PEG,SNPTraitAssoc_PEGName,&OutName._peg);
%end;
%if &trend=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PET,SNPTraitAssoc_PETName,&OutName._pet);
%end;

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

%global RandEff CensorVar;
%let RandEff=;
%let CensorVar=;

%global ParmEst EffName;
%let ParmEst=ParameterEstimates;
%let EffName=Parameter;
%if %upcase(&TraitType)=CONTINUOUS %then %do;
	%let WhichProc=SURVEYREG;
	%let Stat=F; 
	%let Gtable=Effects;
	%let TStmt=;
	%let Ttable=Effects;
	%let ClassOpt=;	
	%let TraitType=SContinuous;
	%let ModOption=%str(/ solution); 	
	%let teststmt=;	
%end;
%else %do;
 %let WhichProc=SURVEYLOGISTIC;
 %let Stat=ChiSq;
 %let Gtable=Type3;
 %let TStmt=contrast;
 %let Ttable=ContrastTest;
 %let usestrata=1;
 %if %upcase(&TraitType)=NOMINAL %then %let ModOption=%str(/ link=glogit);
 %else %let ModOption = ;
 %let ClassOpt = %str(/ cprefix=8);
 %let teststmt=%str('genotypes' Genotype 1);	
 %let EffName=Variable;
%end;

%global RateTotal;
%let RateTotal=;
%if %length(&RateOption) %then %let RateTotal = %quote(RATE=&RateOption);
%else %if %length(&TotalOption) %then %let RateTotal = %quote(TOTAL=&TotalOption);

%global ProcOption TraitOpt;
%let ProcOption=;
%let TraitOpt=;

%global ParmsStmt;
%let ParmsStmt = ;

%SNPTraitAssocMacro(survey=1);
%if &exiterror=1 %then %goto exit;

proc sort data=ma_all out=&SNPTraitAssoc_OutData;
 by &ByVars &AnnoGroup &AnnoLoc;
run;
%Psmooth(&QTests, &SNPTraitAssoc_OutData, &SNPTraitAssoc_OutDataName);

 %if (&syserr>4) %then %goto exit;
%PackageInsertDataset(P-value data from &ProcessName, outlib, &OutName._sst, yes, yes, _type=anno);

%if &NByVal > 1 %then %let ssr=0;
%else %let ssr=1;
%if &MarkerFmt = %str(Numeric Genotypes) %then %let _pi = 1;
%else %let _pi = 0;
%JSLPvalMap(Survey SNP-Trait Association, &SNPTraitAssocScript, &SNPTraitAssocScriptName, &SNPTraitAssoc_OutDataName, _volplot=&volcanoplot, _subsetdd=&ssr, _plotinput= &_pi);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend;

* run the macro;
%SurveySNPTraitAssociation;
