/*
JMP Genomics VCF Input Engine

Name: VCFInputEngine

*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

OPTIONS nospool;
 
*ProcessVariables;

%global StudyName NewStudyName InPath FileFilter InFiles FileType
InputStmt LengthStmt VarNameList DataStartRow UniqueVarNameFlag 
ApplyOriginalName DataStepStatements ScanRows ScanColumns
OutDataPrefix OutputFormat SepThreshold ExcChr IncChr
GQFilter DPFilter FilterI FilterPercent FilterA MergetoOne
MaxColLen OutPath ServerOutPath MacroPath;

*ProcessBody;

%macro VCFInputEngine;

*initialization;
%let ProcessName = VCFInputEngine;
%put ProcessName = &ProcessName;
%global FullAPName;
%let FullAPName = %str(VCF Input Engine);

%include "&MacroPath./UtilityMacros.sas" / nosource;

%include "&MacroPath./InputEngineMacros.sas" / nosource;

%CheckEndSlash(InPath);
%let fLen = 0;
%let FileList = %trim(%left(&InFiles)); 
%put Infiles=&FileList;


%let nDig= 4; /*leave 3 charcters for the file number ranging from 1 up to 999 files + one charcter for the underscore*/
%let LMax= %sysevalf(32 - &nDig);
%let LPrefix= %length(&OutDataPrefix);

%let f = 1;
%do %while(%length(&FileList));
	* extract one file name;
	%let flag = 0;
	%let name = ;
	%do %While(^&flag);
		%let e = %index(&FileList,%str(|));
		%if ^&e %then %do;
			%let name = %left(&name &FileList);
			%let flag = 1;
			%let FileList = ;
		%end;
		%else %do;
			%let name = %left(&name %substr(&FileList,1,&e-1));
			%if %index(&name,.) %then %let flag = 1;
			%let FileList = %substr(&FileList,&e+1);
		%end;
	%end;
	%let InFile&f = &name;
	%if %index(&name,.) %then %do;
		%let ext = %scan(&name,-1);
		%let InName&f = %substr(&name,1,%length(&name)-%length(&ext)-1);
	%end;
	%else %do;
		%let InName&f = &name;
	%end;

	%if %length(&&InName&f) > %sysevalf(&LMax - 1) %then %do;
		%*let InName&f = %substr(&&InFile&f,1,&LMax - 1)_&f;
		%let InName&f = %substr(&&InFile&f,1,&LMax - 1);
	%end;
	%let LInName&f= %sysevalf(%length(&&InName&f) + 1); /*adding 1 to account for the underscore before the InName when concatenating*/
	%CheckDataName(_name=InName&f);
	
	%put InFile&f = &&InFile&f;
	%put InName&f = &&InName&f;
	
	%let f = %eval(&f+1);
	
%end;
%let nFiles = %eval(&f-1);

%if &nFiles = 0 %then %do;
   %put ERROR: There is no file selected to be imported.;
   %goto exit;
%end;

%put There are &nfiles file(s) selected to import.;

/*
%if &nFiles = 1 and %symexist(OutData) %then %do;
	%if %length(&OutData) %then %do;
		%let OutName1 = var;
		%CheckDataName(_name=OutName1);
	%end;
%end;
*/

%let FType = TAB;

%let Dilimiter = %str('09'x);

* define libnames;
libname OutLib "&OutPath";

%let GTFlag= 0;

* loop through all files;
%do nf = 1 %to &nFiles;

	%put InName&nf= &&InName&nf;
	%put &OutDataPrefix.._&&InName&nf;
	%if %length(&OutDataPrefix) %then %do;	
		%if %sysevalf(&LPrefix + &&LInName&nf) > (32 - &nDig) %then %do;
			%let OutName&nf = %substr(&OutDataPrefix._&&InName&nf,1,32 - &nDig)_&nf;
		%end;
		%else %do;
			%let OutName&nf = &OutDataPrefix._&&InName&nf.._&nf;
		%end;
	%end;
	%else %do;
		%if %sysevalf(&&LInName&nf) > (32 - &nDig) %then %do;
			%let OutName&nf = %substr(&&InName&nf,1,32 - &nDig)_&nf;
		%end;
		%else %do;
			%let OutName&nf = &&InName&nf.._&nf;
		%end;
	%end;
	%put OutName&nf= &&OutName&nf;
	%CheckDataName(_name=OutName&nf);
	
	/*
	%if &MergetoOne=Yes & &nFiles >1 %then %do;
		%if %length(&OutDataPrefix) %then %let OutNameCombined =&OutDataPrefix._comb;
		%else %let OutNameCombined=%substr(&InName1,1,20)._comb;
		%CheckDataName(_name=OutNameCombined);
	%end;
	*/

	%let DLT = %str('090D'x);
	%let DLT2 = %str('09'x);
	%let DLT3 = %str('090A'x);

	* step1: extract variable names and locate the 1st data line;	
	* get file format first;
	%let _FileFormat_ = ;
	%GetFileFormat(&&InPath.&&InFile&nf,_FileFormat_); 
	%put _FileFormat_&nf= &_FileFormat_;

	* get loc for 1st data line;
	%let skipbyte_d = ;	
	filename prodfile "&&InPath.&&InFile&nf";
	data _null_;
		inuse =fopen("prodfile");
		call symput("INUSE",trim(left(inuse)));
		if inuse then inuse2=fclose(inuse);
	run;
	%put insue_&nf= &INUSE;
	
	%if &INUSE = 0 %then %do;
		%goto exit;
	%end;
	 	
	**subtract 7 for the length of _header**;
	%if %length(&OutDataPrefix) %then %do;	
		%if %sysevalf(&LPrefix + &&LInName&nf) > (32 - &nDig - 7) %then %do;
			%let HeaderOutName&nf = %substr(&OutDataPrefix._&&InName&nf,1,32 - &nDig - 7)_header_&nf;
		%end;
		%else %do;
			%let HeaderOutName&nf = &OutDataPrefix._&&InName&nf.._header_&nf;
		%end;
	%end;
	%else %do;
		%if %sysevalf(&&LInName&nf) > (32 - &nDig - 7) %then %do;
			%let HeaderOutName&nf = %substr(&&InName&nf,1,32 - &nDig - 7)_header_&nf;
		%end;
		%else %do;
			%let HeaderOutName&nf = &&InName&nf.._header_&nf;
		%end;
	%end;
	%CheckDataName(_name=HeaderOutName&nf);
	
	data out;
		 infile "&&InPath.&&InFile&nf";
		 input;
		 file "&OutPath.&&HeaderOutName&nf...txt";
		 if index(_infile_,"##") ne 1 and index(_infile_,'"') ne 1 then stop;
		 header = _infile_;
		 output;
		 put _infile_;
	run;
	
	/*data Outlib._0Out_&nf; set out; run;*/
		
	%Nobsvars(out);
	%let VarNameRow = %eval(&nobs + 1);
	%let DataStartRow = %eval(&VarNamerow + 1);	
				
	%GetLineLoc(&&InPath.&&InFile&nf,&DataStartRow,skipbyte_d);
	
	%if &skipbyte_d %then %let skipbyte_d = %eval(&skipbyte_d-1);
	%put skipbyte to the 1st data line: &skipbyte_d;
	%put VarNameRow= &VarNameRow;
	%if &VarNameRow %then %do;
		%ExtractColumnHeader_Text(TextFile=&&InPath.&&InFile&nf,FileFormat=&_FileFormat_,
			Delimiter=&DLT,VarNameRow=&VarNameRow,vName_OutData=_vname_);
		%*if &exiterror %then %goto exit;
	%end;
	%else %do;
		*scan the 1st data row;
		%ExtractColumnHeader_Text(TextFile=&&InPath.&&InFile&nf,FileFormat=&_FileFormat_,
			Delimiter=&DLT,VarNameRow=&DataStartRow,vName_OutData=_vname_);
		%*if &exiterror %then %goto exit;
	%end;	
	
	/*data OutLib._0vname_&nf; set _vname_; run;*/
	
	%*let exiterror=1;
	%*goto exit;		

	%let TabFlag= 1;	
	%NObsVars(_vname_);
	%if &nobs = 1 %then %do;
		%let TabFlag= 0;
		
		data _vname_;
			set _vname_(rename=(vTmp= vTmp0 vLabel=vLabel0 flag=flag0));
			if _n_= 1 then do;
				flag=0;
				_i= 1;
				_var=scan(vLabel0,_i);
			end;
			do while(not missing(_var));
				vTmp= _var;
				vLabel= _var;
				if _i= 1 then vPos= 1;
				else vPos= index(vLabel0, trim(_var)) - 1;
				flag + 1;
				_i + 1;
				_var= scan(vLabel0,_i);
				output;
			end;
			retain flag;
			keep vTmp vLabel flag vPos;
		run;
	%end;
	%put TabFlag= &TabFlag;
	
	/*data OutLib._1vname_&nf; set _vname_; run;*/
	
	/*
	data _vname_;
		set _vname_;
		if index(vLabel, '#')=1 then vLabel= substr(vLabel, 2); 
	run;
	*/

	* finding # of columns;
	%let did = %sysfunc(open(_vname_)); 
	%let nv = %sysfunc(attrn(&did,NOBS));
	%let _nVars_ = %sysfunc(attrn(&did,NVARS));
	%let rc = %sysfunc(close(&did));

	* get column labels;
	%let did = %sysfunc(open(_vname_));
	%do _i_ = 1 %to &_nVars_;
		%let _colName_=%sysfunc(varname(&did,&_i_));
		%if %upcase(&_colName_) = VLABEL %then %do;
			%do _j_ = 1 %to &nv;
				%let rc =%sysfunc(fetchobs(&did,&_j_));
				%global vlabel&_j_;
				%let vlabel&_j_ = %sysfunc(getvarc(&did,%sysfunc(varnum(&did,vLabel))));
				%put vLabel&_j_=&&vLabel&_j_;
			%end;
		%end;
	%end;
	%let rc = %sysfunc(close(&did));
	
	* import raw data file;
	%let VarList = ;
	%let uFlag = 0;
	
	%if &TabFlag %then %do;
		*read full data;
		%let _FileFormat_ = ;
	   	%GetFileFormat(&InPath.&&InFile&nf,_FileFormat_);  
	   
	   	%ExtractColumnHeader_Text(TextFile=%quote(&InPath.&&InFile&nf),FileFormat=&_FileFormat_,
	         Delimiter=&DLT,VarNameRow=&VarNameRow,vName_OutData=_vname_);
	      	%if &exiterror %then %goto exit;      
	      	
		/*
		data _vname_;
			set _vname_;
			if index(vLabel, '#')=1 then vLabel= substr(vLabel, 2); 
		run;
		*/     	      
      	%end;
      	
   	%let RowDupFlag = ;
   	%let UniqueVarNameFlag=&uFlag;
   	
   	/*data Outlib._3vname_&nf; set _vname_; run;*/
   
   	%if &UniqueVarNameFlag %then %do;
		%naming(_Dset=_vname_,_AllVars=vTmp,_UniqueRow=1,_DupFlag=RowDupFlag);
   	%end;
   	%else %do;
   		%naming(_Dset=_vname_,_AllVars=vTmp);
   	%end;
   	%if ^%length(&RowDupFlag) %then %let RowDupFlag = 0;
  
   	/*data Outlib._4vname_&nf; set _vname_; run;*/

	*Retrieve variable labels; 	
   	data _null_;
      		set _vname_ end=_e_;
      		*if index(vLabel, '#')=1 then vLabel= substr(vLabel, 2); 
      		name='TmpName'||trim(left(_n_));
      		call symput(name,vTmp);
      		/*
      		%if &RowDupFlag %then %do;
      		   	Lname='TmpLabel'||trim(left(_n_));
      		   	call symput(Lname,vLabel);
      		%end;
      		*/
    		%if &TabFlag = 0 %then %do;
    			pname='TmpPos'||trim(left(_n_));
      			call symput(pname,vPos);
    		%end;
      		if _e_ then call symput('_nVars_',_n_);
   	run;

	/*
	data _null_;
		set _vname_ end=_e_;
		*if index(vLabel, '#')=1 then vLabel= substr(vLabel, 2); 
		lname = 'TmpLabel'||trim(left(_n_));
		call symput(lname,vLabel); 
   	run;   
   	*/
   		
   	%if &TabFlag = 1 %then %do;
		* Read full data;
		OPTIONS OBS=&ScanRows;
		proc import out=OutName_sub
	   		datafile= "&InPath.&&InFile&nf"
	   		dbms=&FType replace; 
	   		DataRow = &DataStartRow; 
	   		GETNAMES=no; 
	   		GUESSINGROWS=&ScanRows;
	   	run;	
	   	OPTIONS OBS=MAX;
   	%end;
   	%else %do;
		data OutName_sub;
			length %do j=1 %to &_nVars_; &&TmpName&j %end; $500;	
			infile "&InPath.&&InFile&nf" MISSOVER lrecl=32767 firstobs= &DataStartRow obs=%sysevalf(&DataStartRow + &ScanRows);
			input %do j=1 %to &_nVars_; @&&TmpPos&j &&TmpName&j %end;;	
		run;   	   	
   	%end;
    	
   	/*data Outlib._0OutName_sub; set OutName_sub; run;*/
   	  
	* get column labels and column length;
	%let Lenlist=;
	%let InputList=;
	%let chromNFlag=0;
	%let infoDPFlag =0;
	%let did = %sysfunc(open(OutName_sub)); 
	%*let nv2 = %sysfunc(attrn(&did,NOBS));
	%let _nVars_ = %sysfunc(attrn(&did,NVARS));
	%let rc = %sysfunc(close(&did));

	%let did = %sysfunc(open(OutName_sub));
	%do _i_ = 1 %to &_nVars_;
		%let _vName&_i_ = %sysfunc(VARNAME(&did,&_i_));
		%let vLen&_i_ = %sysfunc(VARLEN(&did,&_i_));
		%let _type&_i_ = %sysfunc(VARTYPE(&did,&_i_));
		%*put vLabel&_i_=&&vLabel&_i_ vLen&_j_=&&vLen&_i_ _type&_i_=&&_type&_i_;
		%if %nrbquote(&&vLabel&_i_) =%nrbquote(#CHROM) or %nrbquote(&&vLabel&_i_) =%nrbquote(__CHROM) %then %do;
		%*if %nrbquote(&&vLabel&_i_) =%nrbquote(CHROM) or %nrbquote(&&vLabel&_i_) =%nrbquote(__CHROM) %then %do;		
			%let vLen&_i_ = %eval(&&vLen&_i_ + 4);
			%let Lenlist=&Lenlist &&_vName&_i_ $&&vLen&_i_;
			%let ChromLen=&&vLen&_i_;
			%if &&_type&_i_ = N %then %do;
				%let chromNFlag=1;
			%end;
		%end;
		
		/*
		%if %nrbquote(&&vLabel&_i_) =%nrbquote(FILTER) %then %do;
			%let filterFlag=1;
		%end;
		%else %do;
			%let filterFlag=0;
		%end;
		*/
		
		%if &&_type&_i_ = C %then %do;
			%let Lenlist=&Lenlist &&_vName&_i_ $&&vLen&_i_;
			%*let InputList=&InputList &&_vName&_i_ $;
			%let ForLen&_i_ = format &&_vName&_i_ $&&vLen&_i_...;
			%let InforLen&_i_ =informat &&_vName&_i_ $&&vLen&_i_...;
		%end;
		%else %do;
			%let ForLen&_i_ = format &&_vName&_i_ best12.;
			%let InforLen&_i_ = informat &&_vName&_i_ best32.; 
		%end;
		/*%else %do;
			%if %nrbquote(&&vLabel&_i_) =%nrbquote(#CHROM) %then %do;
			%*if %nrbquote(&&vLabel&_i_) =%nrbquote(CHROM) %then %do;
				%let InputList=&InputList &&_vName&_i_ $;
			%end;
			%else %do;
				%let InputList=&InputList &&_vName&_i_;
			%end;
		%end;*/
		%let InputList=&InputList &&_vName&_i_;
		%put ForLen&_i_=&&ForLen&_i_ InforLen&_i_=&&InforLen&_i_;
	%end;
	%let rc = %sysfunc(close(&did)); 
	%put Lenlist=&Lenlist;
	%put InputList=&InputList;
	%put chromNFlag=&chromNFlag;

	%if &chromNFlag=1 %then %do;

		* rename variables;	
		%if &TabFlag %then %do;
			data &&OutName&nf ;
				length &Lenlist;
				infile "&InPath.&&InFile&nf" delimiter=&Dilimiter MISSOVER DSD lrecl=32767 firstobs=&DataStartRow;
				input &InputList;
			run;
		%end;
		%else %do;
			data &&OutName&nf ;
				length &Lenlist;
				infile "&InPath.&&InFile&nf" MISSOVER lrecl=32767  firstobs=&DataStartRow;
				input %do j=1 %to &_nVars_; @&&TmpPos&j &&TmpName&j %end;;				
			run;			
		%end;
		
		data &&OutName&nf;
			retain &InputList;
			set &&OutName&nf;
		run;

		%let OutSet=&&OutName&nf;
  		%let tid = %sysfunc(open(&OutSet));
  		%let nv_OutSet = %sysfunc(attrn(&tid,NVARS)); 
  		%let no_OutSet = %sysfunc(attrn(&tid,NOBS)); 
  		%do i = 1 %to &nv_OutSet;
  	   		%let _name&i = %sysfunc(VARNAME(&tid,&i));
  		%end;
   		%let tid = %sysfunc(close(&tid));
	
   		%let nMax = %sysfunc(min(&nv_OutSet,&_nVars_));
   		%if %scan(&OutSet,1) = &OutSet %then %do;
   			   %let _lib = work;
   			   %let _dset = &OutSet;
   		%end;
   		%else %do;
   			  %let _lib = %scan(&OutSet,1);
   			  %let _dset = %scan(&OutSet,2);
   		%end; 

   		proc datasets lib=&_lib nolist;
   			modify &_dset;      
   			%do i = 1 %to &nMax;
   				label &&_name&i = "%qtrim(&&TmpLabel&i)";;
				%*if (&&_name&i ne &&TmpName&i) & (&&TmpName&i ne _) %then %do;
				%if (&&_name&i ne _V&i)  %then %do;
					label &&_name&i = "%qtrim(&&TmpName&i)";
					rename &&_name&i = _V&i;
					%* rename &&_name&i = &&TmpName&i;
				%end; 
			%end;
   		quit;
   		
   		proc datasets lib=&_lib nolist;
   			modify &_dset;      
   			%do i = 1 %to &nMax;
   				%if &VarNameRow %then %do;
   					%if (_V&i ne &&TmpName&i) %then %do;
   						rename _V&i = &&TmpName&i;
   					%end;
   				%end;
   			%end; 
   		quit;	
   	%end;
   	%else %do;
		/*%skip_header_import_2(DataFile=%quote(&InPath.&&InFile&nf),DataType=&FType,
			OutSet=&&OutName&nf,VarLen=32767, FirstRow=&DataStartRow,VarNameRow=&VarNameRow,
			VarLabelRow=&VarNameRow,GuessRows=&ScanRows,_varlist=&VarList,
			UniqueVarNameFlag=&uFlag,_exitwitherror=0);*/
			
		%if &TabFlag %then %do;
			%let DataFile = %quote(&InPath.&&InFile&nf);
			%if %upcase(&FType) = TAB %then %let DLT = '090D'x;
			%let _FileFormat_ = ;
			%GetFileFormat(&DataFile,_FileFormat_);  
			%ExtractColumnHeader_Text(TextFile=%quote(&InPath.&&InFile&nf),FileFormat=&_FileFormat_,
				Delimiter=&DLT,VarNameRow=&VarNameRow,vName_OutData=_vname_);
			%if &exiterror %then %goto exit;
			
			/*
			data _vname_;
				set _vname_;
				if index(vLabel, '#')=1 then vLabel= substr(vLabel, 2); 
			run;
			*/
			
			%naming(_Dset=_vname_,_AllVars=vTmp);
			%if ^%length(&RowDupFlag) %then %let RowDupFlag = 0;
		%end;
   		
   		data _null_;
   		   set _vname_ end=_e_;
   		   *if index(vLabel, '#')=1 then vLabel= substr(vLabel, 2); 
   		   name='TmpName'||trim(left(_n_));
   		   call symput(name,vTmp);
   		   %*if &RowDupFlag %then %do;
   		   	Lname='TmpLabel'||trim(left(_n_));
   	    		call symput(Lname,vLabel);
   	   	   %*end;
   		   if _e_ then call symput('_nVars_',_n_);
   		run;
		
		/*
		data _null_;
			set _vname_ end=_e_;
			lname = 'TmpLabel'||trim(left(_n_));
			call symput(lname,vLabel); 
		run;
		*/
		
		* get column length;
		%if &TabFlag %then %do;
			* Read full data;
			data &&OutName&nf;
				infile "&InPath.&&InFile&nf" delimiter='09'x MISSOVER DSD lrecl=32767 firstobs=&DataStartRow;
				%do _i_ = 1 %to &_nVars_;
					&&InforLen&_i_;
				%end;
				%do _i_ = 1 %to &_nVars_;
					&&ForLen&_i_;
				%end;
				input &InputList;
				if _ERROR_ then call symputx('_EFIERR_',1);  /*set ERROR detection macro variable;*/
			run;
		%end;
		%else %do;
			data &&OutName&nf;
				infile "&InPath.&&InFile&nf" MISSOVER lrecl=32767 firstobs=&DataStartRow;
				%do _i_ = 1 %to &_nVars_;
					&&InforLen&_i_;
				%end;
				%do _i_ = 1 %to &_nVars_;
					&&ForLen&_i_;
				%end;
				input %do j=1 %to &_nVars_; @&&TmpPos&j &&TmpName&j %end;;
			run;
		%end;
		
		/*data OutLib._0OutName_&nf; set &&OutName&nf; run;*/

		* rename variables;
	   	%let tid = %sysfunc(open(&&OutName&nf));
	   	%let nv_OutSet = %sysfunc(attrn(&tid,NVARS)); 
	   	%let no_OutSet = %sysfunc(attrn(&tid,NOBS)); 
	   	%do i = 1 %to &nv_OutSet;
	      		%let _name&i = %sysfunc(VARNAME(&tid,&i));
	   	%end;
  		%let tid = %sysfunc(close(&tid));
  		%let nMax = %sysfunc(min(&nv_OutSet,&_nVars_));
  	   	
  	   	%if %scan(&&OutName&nf,1) = &&OutName&nf %then %do;
	      		%let _lib = work;
	      		%let _dset = &&OutName&nf;
	   	%end;
	   	%else %do;
	     		%let _lib = %scan(&&OutName&nf,1);
	     		%let _dset = %scan(&&OutName&nf,2);
   		%end; 

      		proc datasets lib=&_lib nolist;
         		modify &_dset;      
         		%do i = 1 %to &nMax;
            			label &&_name&i = "%qtrim(&&TmpLabel&i)";;
            			%if &VarNameRow %then %do;
               				%if (&&_name&i ne _V&i)  %then %do;
                  				label &&_name&i = "%qtrim(&&TmpName&i)";
                  				rename &&_name&i = _V&i;
               					%*rename &&_name&i = &&TmpName&i;
               				%end;
            			%end;
         		%end; 
      		quit;
      		
      		proc datasets lib=&_lib nolist;
         		modify &_dset;      
         		%do i = 1 %to &nMax;
         			%if &VarNameRow %then %do;
               				%if (_V&i ne &&TmpName&i) %then %do;
                  				rename _V&i = &&TmpName&i;
               				%end;
            			%end;
         		%end; 
     		quit;		
	%end;

	/*data OutLib._1OutName&nf; set &&OutName&nf; run;*/
	
   	%*let exiterror=1;
   	%*goto exit;	
	
	%let _tid_ = %sysfunc(open(&&OutName&nf));
	%let IDvarExist = 0;
	%put &&OutName&nf &_tid_;
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));
	%let IDvarExist =%sysfunc(varnum(&_tid_,ID));
	%put _nVars_= &_nVars_;
	%put IDvarExist= &IDvarExist;
	
	%*let rc = %sysfunc(close(&_tid_));
	%*let exiterror=1;
	%*goto exit;
	
	%if &IDvarExist %then %do;
		%let IDType = %sysfunc(vartype(&_tid_,&IdVarExist));
	%end;
	%else %do;
		%let IDType = C;
	%end;	
	
	%let _nLabels_ = 0;
	%do _i_ = 1 %to &_nVars_;
		%global vname&_i_ vlabel&_i_ vnameOrg&_i_;
		%let vname&_i_ = %sysfunc(varname(&_tid_,&_i_));
		%let vnameOrg&_i_ = &&vname&_i_;
		%if %length(&&vname&_i_)>22 %then %do;
			%let vname&_i_ = %substr(&&vname&_i_,1,22);
			%if %length(&&vLabel&_i_)>22 & %index(&&vLabel&_i_,%str(\)) %then %do;
				%let vname&_i_ = %trim(%left(%scan(&&vLabel&_i_,-1,%str(\))));
				%let vname&_i_= %sysfunc(translate(&&vname&_i_,%str(_),%str(.),%str(_),%str(-)));
			%end;
		%end;
		%put vname&_i_=&&vname&_i_ vLabel&_i_=&&vLabel&_i_  vnameOrg&_i_=&&vnameOrg&_i_;
	%end;
	%let rc = %sysfunc(close(&_tid_));
	
	%if &_nVars_ > 9 %then %let genotypes=1;
	%else %let genotypes=0;	

	%if &genotypes %then %do; /* in case proc import sets TIME. format to sample columns */
		/*********** data import code **********************/
		* extract format, informat;
		%if &chromNFlag=1 %then %do;
			proc contents data=OutName_sub out=format noprint;
			run;
		%end;
		%else %do;
			proc contents data=&&OutName&nf out=format noprint;
			run;
		%end;
		
		/*data OutLib._0format_&nf; set format; run;*/
		
		data format;
			set format;
			if formatd then format = trim(format)||trim(left(formatl))||'.'||trim(left(formatd));
			else if formatl then format = trim(format)||trim(left(formatl))||'.';
			else format = trim(format)||'.';
			if informd then informat = trim(informat)||trim(left(informl))||'.'||trim(left(informd));
			else if informl then informat = trim(informat)||trim(left(informl))||'.';
			else informat = trim(informat)||'.';
			flag = varnum;
			keep flag format informat length type;
			rename length=vLength format=vFormat informat=vInformat type=vType;
		run;
	
		proc sort data=format;
			by flag;
		run;
	
		data _null_;
			set format end=_e_; 
			if _e_ then do;
				call symput('_L_',vlength);
				call symput('_F_',vformat);
				call symput('_IF_',vinformat);
				call symput('_T_',vtype);
				call symput('_n_',_n_);
			end;
		run; 
		
		%if &chromNFlag=1 %then %do;
			* force CHROM column to be character;
			data _null_;
				set _vname_ end=_e_;
				if upcase(vlabel)="#CHROM" then call symput("vflag",trim(left(flag))); 
				*if upcase(vlabel)="CHROM" then call symput("vflag",trim(left(flag))); 
		  	run;  
		  	%put vflag=&vflag ChromLen=&ChromLen;
		  	data format;
		  		set format;
		  		if flag="&vflag" then do;
		  			vFormat="$&ChromLen..";
		  			vinformat="$&ChromLen..";
		  		end;
		  	run;
		%end;
		
		/*data OutLib._1format_&nf; set format; run;*/
		/*data OutLib._5vname_&nf; set _vname_; run;*/
	
		%let redo = 0;
		data _vname_;	
			merge _vname_ format; 
			by flag;
			if _n_ > 9 then do;
				if substr(vformat,1,1) ne "$" then call symputx("redo",1);
				vLength = &_L_;
				vFormat = "$32.";
				vInformat = "$32.";
				*vType = "C";
				if length(vTmp)>22 then do;
					vTmp=substr(vTmp,1,22);
					if index(vLabel,"\") then do;
						vTmp=trim(left(scan(vLabel,-1,"\")));
						vTmp=translate(vTmp,"_",".","_","-");
					end;
				end;
				/*
				if length(vLabel)>28 & index(vLabel,"\") then do;
						vTmp=trim(left((scan(vLabel,-1,"\"))));
						vTmp=translate(vTmp,"_",".","_","-");
				end;
				*/
			end;
		run;
		
		/*data OutLib._6vname_&nf; set _vname_; run;*/
		
		%put redo= &redo;
		%if &redo %then %do;
			**No need to use TabFlag because when TabFlag=0, then all variables will be character, and redo=0**;		
			* step 5 read full data;
			%let did2 = %sysfunc(open(_vname_));
			%let vNum = %sysfunc(VARNUM(&did2,vTmp));	
			%let lNum = %sysfunc(VARNUM(&did2,vLabel));
			%let fNum = %sysfunc(VARNUM(&did2,vFormat));
			%let ifNum = %sysfunc(VARNUM(&did2,vInformat));
			data &&OutName&nf(compress=yes);
				infile "&InPath.&&InFile&nf" RECFM=N delimiter=%if &_FileFormat_=Win %then &DLT; %else &DLT3;
					DSD LRECL=32767; 
				%let rc=%sysfunc(fetchobs(&did2,1));
				%do i = 1 %to &nv;
					%let name = %sysfunc(getvarc(&did2,&vNum)); 
					%let Format = %sysfunc(getvarc(&did2,&fNum)); 
					%let Informat = %sysfunc(getvarc(&did2,&ifNum)); 
					%put name Format Informat= &name &Format &Informat;
					informat &name &Informat; 
					format &name &Format;
					%let rc=%sysfunc(fetch(&did2));
				%end;
	
				label 
				%do i = 1 %to &nv;  
					%if &i = 1 %then %let rc=%sysfunc(fetchobs(&did2,1));
					%let name = %sysfunc(getvarc(&did2,&vNum)); 
					%let label = %sysfunc(getvarc(&did2,&lNum));
					&name = "&label" 
					%let rc=%sysfunc(fetch(&did2));
				%end;
				;
				if _n_ = 1 then input +&skipbyte_d @;
	
				input 
				%do i = 1 %to &nv;
					%if &i = 1 %then %let rc=%sysfunc(fetchobs(&did2,1));
					%sysfunc(getvarc(&did2,&vNum))
					%let rc=%sysfunc(fetch(&did2));
				%end;
				@;
				%if &_fileFormat_=Win %then input +1 @;;
			run;
	
			%let rc = %sysfunc(close(&did2));	
		%end;
	%end;
		
	%NObsVars(&&OutName&nf);
	%if (&nobs=0 | &nvars=0) %then %do;
		data OutLib.exitedwitherrors;
			error=1;
		run;
		%put ERROR: Error(s) were encountered creating output data set.;
		%let exiterror = 1;
		%goto exit;
	%end;
	
	/*
	**To be implemented: parsing INFO fields**;
	%let infofieldslist = %str(AA AC AD ADF ADR AF AN ASP BKPTID BQ CICN CICNADJ CIEND CIGAR);
	%let infofieldslist = &infofieldslist %str(CILEN CIPOS CN CNADJ DB DBRIPID DBVARID DGVID DP DPADJ END EVENT); 
	%let infofieldslist = &infofieldslist %str(H2 H3 HOMLEN HOMSEQ IMPRECISE INDEL MATEID MEINFO METRANS MQ MQ0); 
	%let infofieldslist = &infofieldslist %str(NOVEL NS PARID RSPOS SB SOMATIC SVLEN SVTYPE VALIDATED);
	%let ninfo= 46;

	%let InfoFields =;
	proc sort data=&&OutName&nf(keep=info) out=infolds nodupkey;
		by info;
	run;	
	
	data _null_;
		set infolds end=_e_;
		length InfoFields $300.;
		retain InfoFields;
		ff=1;
		if _n_=1 then InfoFields=translate(trim(INFO)," ",";");
		*else do;
			do while(lengthn(scan(INFO,ff,";")));
				field = scan(INFO,ff,";");	
				if ^index(InfoFields,trim(field)) then do;
					InfoFields=catx(' ',InfoFields ,field);
				end;					
				%do f=1 %to &ninfo;
					%let cf = %scan(&infofieldslist,&f);
					if field="&cf" then call symputx("&cf",1);		
				%end;
				ff+1;
			end;
		*end;
		if _e_ then call symput("InfoFields",trim(left(InfoFields)));
	run;	
	%put InfoFields= &InfoFields;
	*/
	
	/*
	%let GT=0;
	%let DP=0;
	%let DPI=0;
	%let FT=0;
	%let GL=0;
	%let GLE=0;
	%let GQ=0;
	%let HQ=0;
	%let PL=0;
	%let GP=0;
	%let PS=0;
	%let PQ=0;
	%let EC=0;
	%let nform= 13;
	*/
		
	%if &genotypes %then %do;
		proc sort data=&&OutName&nf(keep=FORMAT) out=frmlst nodupkey;
			by format;
		run;
		
		/*data OutLib._0frmlst; set frmlst; run;*/

		%let fmtfieldslist= ;
		%let fmtfieldslistc= ;
		data _null_;
			set frmlst end=_e_;
			length FormatFields $100.;
			retain FormatFields;
			ff=1;
			if _n_=1 then FormatFields=translate(trim(FORMAT)," ",":");
			do while(lengthn(scan(FORMAT,ff,":")));
				field = scan(FORMAT,ff,":");	
				if ^index(FormatFields,trim(field)) then do;
					FormatFields=catx(' ',FormatFields ,field);
				end;
				ff+1;
			end;
			if _e_ then do;
				call symput("fmtfieldslist",trim(left(FormatFields)));
				call symput("fmtfieldslistc", translate(trim(FormatFields),":"," "));
			end;
		run;
		%put fmtfieldslist= &fmtfieldslist;
		%put fmtfieldslistc= &fmtfieldslistc;
		
		%*goto exit;

		%let i=1;
		%let f= %scan(&fmtfieldslist, &i, " ");
		%let DP=0; /**need this because the code below uses DP**/
		%let GQ=0; /**need this because the code below uses GQ**/
		%do %while(%length(&f));
			%let &f= 0;
			%*put &f= &&&f;	

			%if %quote(&f)=%str(DP) %then %do;
				%let DP= 1;
			%end;
			%else %if %quote(&f)=%str(GQ) %then %do;
				%let GQ= 1;
			%end;		
			%let i= %sysevalf(&i + 1);
			%let f= %scan(&fmtfieldslist, &i, " ");
		%end;
		%let nform= %sysevalf(&i - 1);	
		
		%let Idx= ;
		**make sure selvnameOrg has non-missing value**;
		data flds;
			set &&OutName&nf(
					keep= FORMAT %do j=10 %to &_nVars_; &&vnameOrg&j %end; 
					firstobs=1 obs=%sysfunc(max(1000,&ScanRows))
					);
			array _a{*} %do j=10 %to &_nVars_; &&vnameOrg&j %end;;
			/*
			*if trim(left(FORMAT)) = "&fmtfieldslistc" then do; 
				do _i=1 to dim(_a);
					if not missing(_a[_i]) then do;
						call symputx("Idx", trim(left(_i)));
						output;
						stop;
					end;
				end;
			*end;
			*/
		run;
		%put Idx= &Idx;
		
		%if %length(&Idx) %then %do;
			%let Idx= %sysevalf(&Idx + 9);
		%end;
		%else %do;
			%let Idx=10;
		%end;
		%let selvnameOrg= &&vnameOrg&Idx;
		%put selvnameOrg= &selvnameOrg;
		
		/*
		proc sort data=&&OutName&nf(keep=format &selvnameOrg) out=flds nodupkey;
			by format;
		run;
		*/
		
		/*data OutLib._1flds_&nf; set flds; run;*/
		
		%*goto exit;
		
		%let FormatFields =;
		/*Where to assign GT value*/
		data flds;
			set flds(keep= FORMAT &selvnameOrg) end=_e_;
			length FormatFields $100.;
			retain FormatFields;
			%do f=1 %to &nform; 
				%let cf = %scan(&fmtfieldslist,&f);
				retain n&cf;
				if _n_ = 1 then n&cf=0; 
			%end;
			ff=1;
			if _n_=1 then FormatFields=translate(trim(FORMAT)," ",":");
			*else do;
				do while(lengthn(scan(FORMAT,ff,":")));
					field = scan(FORMAT,ff,":");	
					fieldvalue = scan(&selvnameOrg,ff,":");
					if ^index(FormatFields,trim(field)) then do;
						FormatFields=catx(' ',FormatFields ,field);
					end;	
					%do f=1 %to &nform;
						%let cf = %scan(&fmtfieldslist,&f);
						%*put cf= &cf;
						if field="&cf" then do;
							call symputx("&cf",1);
							gg=1;
							do while(lengthn(scan(fieldvalue,gg,",")));
								gg+1;
							end;
							if (gg - 1) > n&cf then n&cf= gg-1;
							*put n&cf;
						end;
					%end;					
					ff+1;
				end;
			*end;
			if _e_ then do;
				call symput("FormatFields",trim(left(FormatFields)));
				%do f=1 %to &nform; 
					%let cf = %scan(&fmtfieldslist,&f);
					call symputx("n&cf",n&cf);
				%end;
			end;
		run;
		
		/*data OutLib._2flds_&nf; set flds; run;*/

		/*
		%let newfields=;
		%let numfields =0;
		%do f=1 %to 11;
			%let cf=%scan(&fields,&f);
			%if %quote(&&&cf)=1 %then %do;
				%let newfields = %quote(&newfields &cf);
				%let numfields = %eval(&numfields+1);
			%end;
		%end;
		%let newfields=&FormatFields;
		*/

		%let numbsubfs=; 
		%let newfields=;
		%let numfields=0;
		%let g=1;
		%let tmp= %scan(&FormatFields,&g," ");
		%do %while(%length(&tmp));
			%do f=1 %to &nform;
				%let cf=%scan(&fmtfieldslist,&f, " ");
				%*put f g cf tmp= &f &g &cf &tmp;
				%if %quote(&tmp) = %quote(&cf) %then %do;
					%*put tmp cf = &tmp &cf;
					%let numbsubfs = &numbsubfs &&&n&cf; 
					%let newfields = &newfields &cf;
					%let numfields = %eval(&numfields+1);	
				%end;
			%end;
			%let g= %sysevalf(&g + 1);
			%let tmp= %scan(&FormatFields,&g," ");
		%end;
		%put newfields= &newfields;
		%put FormatFields= &FormatFields;
		%put numbsubfs= &numbsubfs;
		%put numfields= &numfields;
		
		%let newfields= &FormatFields;
		%put FormatFields=&FormatFields newfields=&newfields;
	%end;
	%else %do;
	      %let GT= 0;
	      %let DP= 0;
	%end;
	
	%if &GT = 1 %then %do;
		**Need this flag when there are multiple files, some with genotype columns and others not.**;
		**This flag will be used to output combined wide data set**;
		%let GTFlag= 1;
	%end;

	%*goto exit;
	
	%put DP= &DP;
	%if &DP = 0 %then %do;
		data _null_;
			set &&OutName&nf;
			if index(INFO,"DP=") ne 0 then do;
				call symput("infoDPFlag",trim(left(1)));
				stop;
			end;
		run;
	%end;
	
	%put IDType&nf= &IDType;
	
	/*data OutLib._1OutName&nf; set &&OutName&nf; run;*/	

	data %unquote(&&OutName&nf)(compress=yes);
		/*%if &IDType = C %then %do;*/
			length id $17;
			format id $17.;
			informat id $17.;
		/*%end;*/
		set &&OutName&nf(%if &IDvarExist %then rename=(ID=_ID););
		rename __chrom = chrom;
		%if &IDvarExist %then %do;
			%if &IDType=N %then %do;
				if not missing(_ID) then do;
					id= trim(left(put(_ID, $17.)));
				end;
				else do;
					id= "_Variant_"||strip(_n_);
				end;
			%end;
			%else %do;
				if not missing(_ID) then do;
					id= _ID;
				end;
				else do;
					id = "_Variant_"||strip(_n_);
				end;
			%end;
		%end;
		%else %do;
			id = "_Variant_"||strip(_n_);
		%end;
		
		%if &genotypes %then %do;
			ff=1;
			do while(lengthn(scan(FORMAT,ff,":")));
				field = scan(FORMAT,ff,":");	
				
				%*do f=1 %to &numfields;
				%let f=1;
				%do %while(%length(%scan(&newfields,&f)));
					%let cf = %scan(&newfields,&f);
					%let ncf = %scan(&numbsubfs,&f);
					if field="&cf" then do;
						%do j=10 %to &_nVars_;
							%if %quote(&cf) = %str(GT) %then %do;
								GTC&j = scan(&&vnameOrg&j,1,":");
								if countc(GTC&j,".")>0 or lengthn(GTC&j)=0 then &&vname&j.._GT = .;
								/* Homozygous for reference allele, or 1 reference allele on Y chrom, male X, etc */
								else if countc(GTC&j,"0")=2 or (countc(GTC&j,"0")=1 and lengthn(GTC&j)=1) then &&vname&j.._GT = 0;
								else if countc(GTC&j,"0")=1 and lengthn(trim(GTC&j))=3 then &&vname&j.._GT = 1;
								else &&vname&j.._GT = 2;
							%end;
							%else %do;
								noGT= scan(&&vnameOrg&j,ff,":");
								%if %quote(&cf) = %str(GQ) or 
									%quote(&cf)=%str(PQ) or 
									%quote(&cf)=%str(DP) or
									%quote(&cf)=%str(GL) or
									%quote(&cf)=%str(PL) or
									%quote(&cf)=%str(HQ) or
									%quote(&cf)=%str(PS) or
									%quote(&cf)=%str(PQ) or 
									%quote(&cf)=%str(EC) or
									%quote(&cf)=%str(MQ)	
								%then %do;
									** Numeric **;
									%*put ncf cf= &cf &ncf;
									%if &ncf=1 %then %do;
										&&vname&j.._&cf = noGT + 0;
									%end;
									%else %do;
										%do g=1 %to &ncf;
											if index(noGT,",") then do;
												&&vname&j.._&cf._&g = scan(noGT,&g,",") + 0;
											end;
											else if index(noGT,";") then do;
												&&vname&j.._&cf._&g = scan(noGT,&g,";") + 0;
											end;
										%end;
									%end;
								%end;
								%else %do;
									** Character **;
									%*put ncf cf= &cf &ncf;
									%if &ncf=1 %then %do;
										&&vname&j.._&cf = noGT;
									%end;
									%else %do;
										%do g=1 %to &ncf;
											if index(noGT,",") then do;
												&&vname&j.._&cf._&g = scan(noGT,&g,",");
											end;
											else if index(noGT,";") then do;
												&&vname&j.._&cf._&g = scan(noGT,&g,";");
											end;
										%end;
									%end;
								%end;							
							%end;
						
						%end;	
					end;
					%let f= %eval(&f+1);
				%end;
				*if _n_ = 1 then put ff;	
				ff + 1;
			end;
			
			%if &GT %then %do;
				
				%do j=10 %to &_nVars_;
					%if (%index(&FilterI,either) or %index(&FilterI,GQ)) and &GQ %then %do;
						if &&vname&j.._GQ ne . and &&vname&j.._GQ < &GQFilter then &&vname&j.._GT = .; 
					%end;
					%if (%index(&FilterI,either) or %index(&FilterI,DP)) and &DP %then %do;
						if &&vname&j.._DP ne . and &&vname&j.._DP < &DPFilter then &&vname&j.._GT = .; 
					%end;
					%if %index(&FilterI,both) and (&GQ or &DP) %then %do;
						if 
							%if &GQ %then &&vname&j.._GQ ne . and &&vname&j.._GQ < &GQFilter; 
							%if &GQ and &DP %then and; 
							%if &DP %then &&vname&j.._DP ne . and &&vname&j.._DP < &DPFilter; 
						then &&vname&j.._GT = .; 
					%end;
					
					%if &GQ %then _gqf&j = (&&vname&j.._GQ ne . and &&vname&j.._GQ < &GQFilter);;
					%if &DP %then _dpf&j = (&&vname&j.._DP ne . and &&vname&j.._DP < &DPFilter);;
					%if &DP or &GQ %then %do;
						_eith&j = (%if &GQ %then (&&vname&j.._GQ ne . and &&vname&j.._GQ < &GQFilter); %if &GQ and &DP %then or; %if &DP %then (&&vname&j.._DP ne . and &&vname&j.._DP < &DPFilter););
						_both&j = (%if &GQ %then (&&vname&j.._GQ ne . and &&vname&j.._GQ < &GQFilter); %if &GQ and &DP %then and; %if &DP %then (&&vname&j.._DP ne . and &&vname&j.._DP < &DPFilter););
					%end;
					%if &infoDPFlag %then %do;
						&&vname&j.._DP= scan(scan(substr(INFO,index(INFO,"DP=")),1,";"),2,"=");
						%if (%index(&FilterI,both) or %index(&FilterI,DP) or %index(&FilterI,either)) %then %do;
							if &&vname&j.._DP ne . and &&vname&j.._DP < &DPFilter then &&vname&j.._GT = .; 
						%end;
					%end;
				%end;
				
				%if %index(&FilterA,GQ) and &GQ %then %do;
					_gqf = sum(of %do j=10 %to &_NVars_; _gqf&j %end;);
					if 100*_gqf / %eval(&_nVars_ - 9) >= &FilterPercent then delete;
				%end;
				%if %index(&FilterA,DP) and &DP %then %do;
					_dpf = sum(of %do j=10 %to &_NVars_; _dpf&j %end;);				
					if 100*_dpf / %eval(&_nVars_ - 9) >= &FilterPercent then delete;
				%end;
				%if %index(&FilterA,both) and (&GQ or &DP) %then %do;
					_both = sum(of %do j=10 %to &_NVars_; _both&j %end;);				
					if 100*_both / %eval(&_nVars_ - 9) >= &FilterPercent then delete;				
				%end;
				%if %index(&FilterA,either) and (&GQ or &DP) %then %do;
					_eith = sum(of %do j=10 %to &_NVars_; _eith&j %end;);				
					if 100*_eith / %eval(&_nVars_ - 9) >= &FilterPercent then delete;				
				%end;				
				drop _gqf: _dpf: _both: _eith:;
				
			%end;

			drop ff GTC: field format noGT %do j=10 %to &_nVars_; &&vnameOrg&j %end;;
			
			/*
			%if &IDType = C %then %do;
				if lengthn(id)=0 then id = "_Variant_"||strip(_n_);
			%end;
			%else %do;
				if missing(id) then id = _n_;
			%end;
			*/
			
		%end;
		%if &IDvarExist %then drop _ID;;
	run;
	
	/*data OutLib._2OutName&nf; set &&OutName&nf; run;*/
		
	%let dsid=%sysfunc(open(&&OutName&nf));
	%let chromnum = %sysfunc(varnum(&dsid,chrom));
	%let chromtype = %sysfunc(vartype(&dsid,&chromnum));
	%let rc=%sysfunc(close(&dsid));	

	/*data OutLib.vcf_0OutName&nf; set &&OutName&nf; run;*/

	data &&OutName&nf (drop=INFO);
		set &&OutName&nf;
		%if %length(&ExcChr) %then %do;
			if
				%let ExcChr= %sysfunc(trim(%sysfunc(left(&ExcChr))));
				%let _i_=1;
				%let dropc= %upcase(%scan(&ExcChr, &_i_, " "));
				%*put dropc= &dropc;
				%if &chromtype=N %then %do;
					upcase(chrom) = &dropc
					%do %while(%length(&dropc));					
						 or upcase(chrom) = &dropc
						%let _i_= %sysevalf(&_i_ + 1);
						%let dropc= %scan(&ExcChr, &_i_, " ");
					%end;
				%end;
				%else %do;
					upcase(chrom) = "&dropc"
					%do %while(%length(&dropc));					
						 or upcase(chrom) = "&dropc"
						%let _i_= %sysevalf(&_i_ + 1);
						%let dropc= %upcase(%scan(&ExcChr, &_i_, " "));
					%end;								
				%end;
			then delete;
			if REF=ALT then delete;
			if missing (POS) then delete;
		%end;
		%else %if %length(&IncChr) %then %do;
			if
				%let IncChr= %sysfunc(trim(%sysfunc(left(&IncChr))));
				%let _i_=1;
				%let keepc= %upcase(%scan(&IncChr, &_i_, " "));
				%*put keepc= &keepc;
				%if &chromtype=N %then %do;
					upcase(chrom) not in (&keepc
					%do %while(%length(&keepc));					
						,&keepc 
						%let _i_= %sysevalf(&_i_ + 1);
						%let keepc= %scan(&IncChr, &_i_, " ");
					%end;
					)
				%end;
				%else %do;
					upcase(chrom) not in ("&keepc"
					%do %while(%length(&keepc));					
						,"&keepc"
						%let _i_= %sysevalf(&_i_ + 1);
						%let keepc= %upcase(%scan(&IncChr, &_i_, " "));
					%end;	
					)
				%end;
			then delete;
			if REF = ALT then delete;
			if missing (POS) then delete;
		%end;
		%else %do;
			if REF=ALT then delete;
			if missing (POS) then delete;		
		%end;
	run;	
	
	/*data OutLib.vcf_1OutName&nf; set &&OutName&nf; run;*/
	
	%*goto exit;
	
	%Nobsvars(%unquote(&&OutName&nf));
	%if (&nobs=0) %then %do;
		%put ERROR: Zero observations remained in the results after applying the Filter to Exclude Chromosomes. Please, review the filter specificied in the dialog.;
		%let exiterror = 1;
		%goto exit;
	%end;	
	
	%let curdsid = %sysfunc(open(&&OutName&nf));
	%if (&curdsid) %then %do;
		**Assuming FILTER column is the last column after dropping FORMAT and GENOTYPE columns**;
		%let filterpos= %sysfunc(varnum(&curdsid,%nrbquote(FILTER)));
		%if &filterpos %then %do;
			%let filterFlag=1;
			%let headervarlist=;
			%do i=1 %to &filterpos;
				%let headervarlist= &headervarlist %sysfunc(varname(&curdsid,&i)); 
			%end;			
		%end;
		%else %do;
			%let filterFlag=0;
		%end;
	%end;
	%let currc = %sysfunc(close(&curdsid));
	
	%if &filterFlag=1 %then %do;
		data &&OutName&nf;
			retain &headervarlist;
			set &&OutName&nf(rename=(FILTER=FILTERnum));	
			FILTER = put(FILTERnum, 7.);	
			drop FILTERnum;
		run;
	%end;
	
	%put Nobs SepThreshold= &Nobs &SepThreshold;
	%if &Nobs > &SepThreshold %then %do;
		%put Break output by chromosome ###;
		
		/**S1577092: need to create chroms data set when applying chromosome filter in the code above**/
		proc sort data=%unquote(&&OutName&nf)(keep=chrom) out=chroms nodupkey;
			by chrom;
		run;

		data _null_;
			set chroms end=last;
			%if &chromtype=N %then %do;
				if not missing(chrom) then do;
					call symput("chrom"||strip(_n_),substr(strip(chrom),1,30));
					call symput("chromfull"||strip(_n_),strip(chrom));
				end;
				else do;
					call symput("chrom"||strip(_n_),"_mis_");
					call symput("chromfull"||strip(_n_),"_mis_");
				end;
			%end;
			%else %do;
				if not missing(chrom) then do;
					call symput("chrom"||strip(_n_),substr(strip(chrom),1,30));
					call symput("chromfull"||strip(_n_),strip(chrom));
				end;
				else do;
					call symput("chrom"||strip(_n_),"_mis_");
					call symput("chromfull"||strip(_n_),"_mis_");
				end;
			%end;
			if last then call symputx("numchroms",_n_);
		run;
		
		%if &MergetoOne=Yes & &nFiles>1 %then %do;
			%put Single file combining multiple vcf files for each chromosome (one file for each chromosome) ###;
		
			data 
				%do c=1 %to &numchroms; 
					/*%unquote(&&OutNameN&nf.._&&chrom&c)(compress=yes)*/
					%unquote(chromdata&c)(compress=yes)
				%end;
				;
				set %unquote(&&OutName&nf);
				select(chrom);
				%do c=1 %to &numchroms;
					%if %quote(&&chrom&c)=%str(_) %then %do;
						when(.) output %unquote(chromdata&c.__);
					%end;
					%else %do;
						*when("&&chrom&c") output %unquote(&&OutNameN&nf.._&&chrom&c);
						when("&&chrom&c") output %unquote(chromdata&c);
					%end;
				%end;
				end;
			run;
	
			%if &nf = 1 %then %do;
				%do c=1 %to &numchroms;
					/*
					data outSingleone_&&chrom&c;
						set %unquote(&&OutNameN&nf.._&&chrom&c);
					run;
					*/
					data outSingleone_&c;
						set chromdata&c;
					run;
				%end;
			%end;
			%else %do;
				%do c=1 %to &numchroms;
					*proc sort data = outSingleone_&&chrom&c;
					proc sort data = outSingleone_&c;
						by chrom POS;
					run;
					
					*proc sort data = %unquote(&&OutNameN&nf.._&&chrom&c);
					proc sort data = chromdata&c;
						by chrom POS;
					run;
					
					data outSingleone_&c;
					*data outSingleone_&&chrom&c;
						*merge outSingleone_&&chrom&c %unquote(&&OutNameN&nf.._&&chrom&c);
						merge outSingleone_&c chromdata&c;
						by chrom pos;
					run;
				%end;
			%end;
			
			%let transvars&nf= ;
			%let transvarsGT&nf= ;
			%do j=10 %to &_nVars_; 
				%let c&nf= &&transvars&nf &&vname&j;
				%let transvarsGT&nf= &&transvarsGT&nf &&vname&j.._GT;
			%end;
			%put transvarsGT&nf= &&transvars&nf;			
	
			%if &nf = &nFiles %then %do;
			
				%do c=1 %to &numchroms;	
				
					%let LChr= %sysevalf(%length(&&chrom&c) + 1); /*adding 1 to account for the underscore before chrom when concatenating*/
					%if %length(&OutDataPrefix) %then %do;	
						**subtract 5 for the length of _w or _t + 3 digits for chromosome number**;
						%if %sysevalf(&LPrefix + &&LInName&nf + &LChr) > (32 /*- &nDig*/ - 5) %then %do;
							%let OutNameNCombined= %substr(&OutDataPrefix._&&InName&nf.._&&chrom&c,1,32 /*- &nDig*/ - 5)_t&c;
							%let OutNameNCombinedW= %substr(&OutDataPrefix._&&InName&nf.._&&chrom&c,1,32 /*- &nDig*/ - 5)_w&c;
						%end;
						%else %do;
							%let OutNameNCombined= &OutDataPrefix._&&InName&nf.._&&chrom&c.._t&c;
							%let OutNameNCombinedW= &OutDataPrefix._&&InName&nf.._&&chrom&c.._w&c;
						%end;

						**subtract 10 for the length of _anno + _t + 3 digits for chromosome number**;
						%if %sysevalf(&LPrefix + &&LInName&nf) > (32 /*- &nDig*/ - 10) %then %do;
							%let AnnoOutNameNCombined= %substr(&OutDataPrefix._&&InName&nf.._&&chrom&c,1,32 /*- &nDig*/ - 10)_anno_&c;
						%end;
						%else %do;
							%let AnnoOutNameNCombined= &OutDataPrefix._&&InName&nf.._&&chrom&c.._anno_&c;
						%end;				
					%end;
					%else %do;
						**subtract 5 for the length of _w or _t + 3 digits for chromosome number**;
						%if %sysevalf(&&LInName&nf + &LChr) > (32 /*- &nDig*/ - 5) %then %do;
							%let OutNameNCombined= %substr(&&InName&nf.._&&chrom&c,1,32 /*- &nDig*/ - 5)_t&c;
							%let OutNameNCombinedW= %substr(&&InName&nf.._&&chrom&c,1,32 /*- &nDig*/ - 5)_w&c;
						%end;
						%else %do;
							%let OutNameNCombined= &&InName&nf.._&&chrom&c.._t&c;
							%let OutNameNCombinedW= &&InName&nf.._&&chrom&c.._w&c;
						%end;	

						**subtract 10 for the length of _anno + _c + 3 digits for chromosome number**;
						%if %sysevalf(&&LInName&nf + &LChr) > (32 /*- &nDig*/ - 10) %then %do;
							%let AnnoOutNameNCombined= %substr(&&InName&nf.._&&chrom&c,1,32 /*- &nDig*/ - 10)_anno_&c;

						%end;
						%else %do;
							%let AnnoOutNameNCombined= &&InName&nf.._&&chrom&c.._anno_&c;
						%end;				
					%end;	
					
					%CheckDataName(_name=OutNameNCombined);
					%CheckDataName(_name=OutNameNCombinedW);
					%CheckDataName(_name=AnnoOutNameNCombined);
				
					data outlib.&OutNameNCombined;
						*set outSingleone_&&chrom&c;
						set outSingleone_&c;
					run;
				
					%PackageInsertDataset(Imported &OutNameNCombined chromosome &&chromfull&c tall %if &GTFlag=1 %then output; data from &ProcessName,outlib,%unquote(&OutNameNCombined),No,_type=tall);			

					data _GTvars_;
						*set outSingleone_&&chrom&c;
						set outSingleone_&c;
						keep 
						%do j=1 %to &nFiles; 
							%let k=1;
							%let vGT= %scan(&&transvarsGT&j,&k," ");						
							%do %while(%length(&vGT));
								&vGT
								%let k=%sysevalf(&k + 1);
								%let vGT= %scan(&&transvarsGT&j,&k," ");							
							%end;
						%end;
						;
						where 0;
					run;

					/*data OutLib._GTvars_; set _GTvars_; run;*/

					%let _did_ = %sysfunc(open(_GTvars_)); 
					%let _nGTvars_ = %sysfunc(attrn(&_did_,NVARS));
					%put _nGTvars_= &_nGTvars_;
						
					%if %upcase(&OutputFormat)=YES and &GTFlag=1 %then %do;
	
						options nolabel;
						proc transpose data=outlib.%unquote(&OutNameNCombined)(rename=(
							/*
							%do j=1 %to &nFiles; 
								%let k=1;
								%let vGT= %scan(&&transvarsGT&j,&k," ");
								%let v= %scan(&&transvars&j,&k," ");
								%do %while(%length(&vGT));
									&vGT=&v
									%let k=%sysevalf(&k + 1);
									%let vGT= %scan(&&transvarsGT&j,&k," ");
									%let v= %scan(&&transvars&j,&k," ");
								%end;
							%end;
							*/
							%do _j_=1 %to &_nGTvars_; 
								%let vGT= %sysfunc(strip(%sysfunc(varname(&_did_,&_j_))));
								&vGT = %substr(&vGT, 1, %length(&vGT)-3)
							%end;							
							))
							let 
							out=outlib.%unquote(&OutNameNCombinedW)/*(rename=(_name_= SampleID))*/
							name=SampleID /*%if &IDType = N %then %do; prefix = _Variant_ %end;*/
							;
							/*var %do j=1 %to &nFiles; &&transvars&j %end;;*/
							var 
							%do _j_=1 %to &_nGTvars_; 
								%let vGT= %sysfunc(strip(%sysfunc(varname(&_did_,&_j_)))); 
								%substr(&vGT, 1, %length(&vGT)-3) 
							 %end;
							;
							id id;
							label SampleID= "SampleID";
						run;
						options label;
						
						%PackageInsertDataset(Imported &OutNameNCombinedW chromosome &&chromfull&c wide output data from &ProcessName,outlib,%unquote(&OutNameNCombinedW),No);
					%end;	
					
					data OutLib.&AnnoOutNameNCombined;	
						set outlib.%unquote(&OutNameNCombined);
						%if &GTFlag=1 %then %do;	
							drop 
								%do _j_=1 %to &_nGTvars_; 
									%sysfunc(strip(%sysfunc(varname(&_did_,&_j_))))
							 	%end;
							;
						%end;
						*keep &headervarlist;
					run;
					%let _rc_ = %sysfunc(close(&_did_));
					
					%PackageInsertDataset(Imported &AnnoOutNameNCombined chromosome &&chromfull&c annotation data from &ProcessName,outlib,%unquote(&AnnoOutNameNCombined),No,_type=Anno);	
					
					%if &c=1 %then %do;
						%let dslist = &OutNameNCombined..sas7bdat;
					%end;
					%else %do;
						%let dslist = %unquote(&dslist |&OutNameNCombined..sas7bdat);
					%end;
					
				%end;
			%end;
			
		%end;
		%else %do;
			%put Single file for each file and each chromosome, one file for each vcf-chromosome combination ###;
				
			/*
			data 
				%do c=1 %to &numchroms; 
					outlib.%unquote(&&OutNameN&nf)(compress=yes)
				%end;
				;
				set %unquote(&&OutName&nf);
				select(chrom);
					%do c=1 %to &numchroms;
						%if %quote(&&chrom&c)=%str(_) %then %do;
							when(.) output outlib.%unquote(&&OutNameN&nf..__);
						%end;
						%else %do;
							*when("&&chrom&c") output outlib.%unquote(&&OutNameN&nf.._&&chrom&c);
							when("&&chrom&c") output outlib.%unquote(&&OutNameN&nf);
						%end;
					%end;
				end;
			run;
			*/

			%do c=1 %to &numchroms;
			
				%let LChr= %sysevalf(%length(&&chrom&c) + 1); /*adding 1 to account for the underscore before chrom when concatenating*/
				%if %length(&OutDataPrefix) %then %do;	
					**subtract 5 for the length of _w or _t + 3 digits for chromosome number**;
					%if %sysevalf(&LPrefix + &&LInName&nf + &LChr) > (32 - &nDig - 5) %then %do;
						%let OutNameN&nf = %substr(&OutDataPrefix._&&InName&nf.._&&chrom&c,1,32 - &nDig - 5)_t&c._&nf;
						%let OutNameNW&nf= %substr(&OutDataPrefix._&&InName&nf.._&&chrom&c,1,32 - &nDig - 5)_w&c._&nf;
					%end;
					%else %do;
						%let OutNameN&nf = &OutDataPrefix._&&InName&nf.._&&chrom&c.._t&c._&nf;
						%let OutNameNW&nf= &OutDataPrefix._&&InName&nf.._&&chrom&c.._w&c._&nf;
					%end;

					**subtract 8 for the length of _anno + 3 digits for chromosome number**;
					%if %sysevalf(&LPrefix + &&LInName&nf + &LChr) > (32 - &nDig - 8) %then %do;
						%let AnnoOutNameN&nf = %substr(&OutDataPrefix._&&InName&nf.._&&chrom&c,1,32 - &nDig - 8)_anno_&c._&nf;
					%end;
					%else %do;
						%let AnnoOutNameN&nf = &OutDataPrefix._&&InName&nf.._&&chrom&c.._anno_&c._&nf;
					%end;

				%end;
				%else %do;
					**subtract 5 for the length of _w or _t + 3 digits for chromosome number**;
					%if %sysevalf(&&LInName&nf + &LChr) > (32 - &nDig - 5) %then %do;
						%let OutNameN&nf = %substr(&&InName&nf.._&&chrom&c,1,32 - &nDig - 5)_t&c._&nf;
						%let OutNameNW&nf= %substr(&&InName&nf.._&&chrom&c,1,32 - &nDig - 5)_w&c._&nf;
					%end;
					%else %do;
						%let OutNameN&nf = &&InName&nf.._&&chrom&c.._t&c._&nf;
						%let OutNameNW&nf= &&InName&nf.._&&chrom&c.._w&c._&nf;
					%end;

					**subtract 8 for the length of _anno + 3 digits for chromosome number**;
					%if %sysevalf(&&LInName&nf + &LChr) > (32 - &nDig - 8) %then %do;
						%let AnnoOutNameN&nf = %substr(&&InName&nf.._&&chrom&c,1,32 - &nDig - 8)_anno_&c._&nf;
					%end;
					%else %do;
						%let AnnoOutNameN&nf = &&InName&nf.._&&chrom&c.._anno_&c._&nf;
					%end;
				%end;

				%CheckDataName(_name=OutNameN&nf);
				%CheckDataName(_name=OutNameNW&nf);
				%CheckDataName(_name=AnnoOutNameN&nf);				
			
				data outlib.%unquote(&&OutNameN&nf)(compress=yes);
					set %unquote(&&OutName&nf);
					where chrom = "&&chrom&c";
				run;
			
				%PackageInsertDataset(Imported &&OutNameN&nf chromosome &&chromfull&c./VCF&nf tall %if &GT %then output; data from &ProcessName,outlib,%unquote(&&OutNameN&nf),No,_type=tall);
				
				%if %upcase(&OutputFormat)=YES and &GT=1 %then %do;
					options nolabel;
					proc transpose data=outlib.%unquote(&&OutNameN&nf)(rename=(%do j=10 %to &_nVars_; &&vname&j.._GT=&&vname&j %end;)) 
						let 
						out=outlib.%unquote(&&OutNameNW&nf)/*(rename=(_name_= SampleID))*/  
						name=SampleID /*%if &IDType = N %then %do; prefix = _Variant_ %end;*/
						;
						var %do j=10 %to &_nVars_; &&vname&j %end;;
						id id;
						label SampleID= "SampleID";
					run;
					options label;
					
					%PackageInsertDataset(Imported &&OutNameNW&nf chromosome &&chromfull&c./VCF&nf wide output data from &ProcessName,outlib,%unquote(&&OutNameNW&nf),No);
				%end;	
				
				data OutLib.&&AnnoOutNameN&nf;
					set outlib.%unquote(&&OutNameN&nf);
					%if &GT %then %do;	
						drop %do j=10 %to &_nVars_; &&vname&j.._GT %end;;
					%end;
					*keep &headervarlist;
				run;
				
				%PackageInsertDataset(Imported &&AnnoOutNameN&nf chromosome &&chromfull&c./VCF&nf annotation data from &ProcessName,outlib,%unquote(&&AnnoOutNameN&nf),No,_type=Anno);
			
				%if &nf=1 and &c=1 %then %do;
					%let dslist = &&OutNameN&nf...sas7bdat;
				%end;
				%else %do;
					%let dslist = %unquote(&dslist |&&OutNameN&nf...sas7bdat);
				%end;				
			%end;		
		%end;	
	%end;
	%else %do;
		%put Do not break output by chromosome ###;
		
		/*
		%if %upcase(&OutputFormat)=YES %then %let maxl=30;
		%else %let maxl=30;
		
		%if %length(&&OutName&nf)>&maxl %then  %do;
			%let OutNameN&nf = %unquote(%substr(&&OutName&nf,1,&maxl));
			%let NewOutNameN&nf=%unquote(%substr(&&OutName&nf,1,22))_&nf;
			%let AnnoOutNameN&nf=%unquote(%substr(&&OutName&nf,1,22))_&nf._anno; 
		%end;
		%else %do ;
			%let OutNameN&nf = %unquote(&&OutName&nf);
			%let NewOutNameN&nf=&&OutName&nf;
			%let AnnoOutNameN&nf=%unquote(%substr(&&OutName&nf,1,22))_&nf._anno; 
		%end;
		*/
				
		%if &MergetoOne=Yes & &nFiles>1 %then %do;
			%put Single file combining all vcf files ###;
			
			%if &nf = 1 %then %do;
				data outSingleone;
					set %unquote(&&OutName&nf);
				run;
				
			%end;
			%else %do;
				proc sort data = outSingleone;
					by chrom POS;
				run;
				proc sort data = %unquote(&&OutName&nf);
					by chrom POS;
				run;
				data outSingleone;
					merge outSingleone %unquote(&&OutName&nf);
					by chrom pos;
				run;		
			%end;
			
			%let transvars&nf= ;
			%let transvarsGT&nf= ;
			%do j=10 %to &_nVars_; 
				%let transvars&nf= &&transvars&nf &&vname&j;
				%let transvarsGT&nf= &&transvarsGT&nf &&vname&j.._GT;
			%end;
			%put transvarsGT&nf= &&transvarsGT&nf;			
			
			%if &nf = &nFiles %then %do;
			
				%put LPrefix LInName&nf = &LPrefix &&LInName&nf;			
				%if %length(&OutDataPrefix) %then %do;	
					**subtract 6 for the length of _tcomb or _wcomb**;
					%if %sysevalf(&LPrefix + &&LInName&nf) > (32 /* - &nDig*/ - 6) %then %do;
						%let OutNameCombined= %substr(&OutDataPrefix._&&InName&nf,1,32 /*- &nDig*/ - 6)_tcomb;
						%let OutNameCombinedW= %substr(&OutDataPrefix._&&InName&nf,1,32 /*- &nDig*/ - 6)_wcomb;
					%end;
					%else %do;
						%let OutNameCombined= &OutDataPrefix._&&InName&nf.._tcomb;
						%let OutNameCombinedW= &OutDataPrefix._&&InName&nf.._wcomb;
					%end;

					**subtract 10 for the length of _comb + _anno**;
					%if %sysevalf(&LPrefix + &&LInName&nf) > (32 /*- &nDig*/ - 10) %then %do;
						%let AnnoOutNameCombined= %substr(&OutDataPrefix._&&InName&nf,1,32 /*- &nDig*/ - 10)_comb_anno;
					%end;
					%else %do;
						%let AnnoOutNameCombined= &OutDataPrefix._&&InName&nf.._comb_anno;
					%end;				
				%end;
				%else %do;
					**subtract 6 for the length of _tcomb or _wcomb**;
					%if %sysevalf(&&LInName&nf) > (32 /*- &nDig*/ - 6) %then %do;
						%let OutNameCombined= %substr(&&InName&nf,1,32 /*- &nDig*/ - 6)_tcomb;
						%let OutNameCombinedW= %substr(&&InName&nf,1,32 /*- &nDig*/ - 6)_wcomb;
					%end;
					%else %do;
						%let OutNameCombined= &&InName&nf.._tcomb;
						%let OutNameCombinedW= &&InName&nf.._wcomb;
					%end;	

					**subtract 10 for the length of _comb + _anno**;
					%if %sysevalf(&&LInName&nf) > (32 /*- &nDig*/ - 10) %then %do;
						%let AnnoOutNameCombined= %substr(&&InName&nf,1,32 /*- &nDig*/ - 10)_comb_anno;

					%end;
					%else %do;
						%let AnnoOutNameCombined= &&InName&nf.._comb_anno;
					%end;				
				%end;

				%put OutNameCombined= &OutNameCombined;
				%put OutNameCombinedW= &OutNameCombinedW;
				%put AnnoOutNameCombined= &AnnoOutNameCombined;

				%CheckDataName(_name=OutNameCombined);
				%CheckDataName(_name=OutNameCombinedW);
				%CheckDataName(_name=AnnoOutNameCombined);			
			
				data outlib.&OutNameCombined;
					set outSingleone;
				run;
				
				%PackageInsertDataset(Imported &OutNameCombined tall %if &GTFlag=1 %then output; data from &ProcessName,outlib,%unquote(&OutNameCombined),No,_type=tall);
					
				data _GTvars_;
					set outSingleone;
					keep 
					%do j=1 %to &nFiles; 
						%let k=1;
						%let vGT= %scan(&&transvarsGT&j,&k," ");						
						%do %while(%length(&vGT));
							&vGT
							%let k=%sysevalf(&k + 1);
							%let vGT= %scan(&&transvarsGT&j,&k," ");							
						%end;
					%end;
					;
					where 0;
				run;
				
				/*data OutLib._GTvars_; set _GTvars_; run;*/
				
				%let _did_ = %sysfunc(open(_GTvars_)); 
				%let _nGTvars_ = %sysfunc(attrn(&_did_,NVARS));
				%put _nGTvars_= &_nGTvars_;					
					
				%if %upcase(&OutputFormat)=YES and &GTFlag=1 %then %do;
					
					options nolabel;
					proc transpose data=outlib.&OutNameCombined(rename=(	
							/*
							%do j=1 %to &nFiles; 
								%let k=1;
								%let vGT= %scan(&&transvarsGT&j,&k," ");
								%let v= %scan(&&transvars&j,&k," ");
								%do %while(%length(&vGT));
									&vGT=&v
									%let k=%sysevalf(&k + 1);
									%let vGT= %scan(&&transvarsGT&j,&k," ");
									%let v= %scan(&&transvars&j,&k," ");
								%end;
							%end;
							*/
							%do _j_=1 %to &_nGTvars_; 
								%let vGT= %sysfunc(strip(%sysfunc(varname(&_did_,&_j_))));
								&vGT = %substr(&vGT, 1, %length(&vGT)-3)
							%end;							
							))
						let 					
						out=outlib.&OutNameCombinedW/*(rename=(_name_= SampleID))*/
						name=SampleID /*%if &IDType = N %then %do; prefix = _Variant_ %end;*/
						;
						/**var %do j=1 %to &nFiles; &&transvars&j %end;;*/
						var
						%do _j_=1 %to &_nGTvars_; 
							%let vGT= %sysfunc(strip(%sysfunc(varname(&_did_,&_j_)))); 
							%substr(&vGT, 1, %length(&vGT)-3) 
						 %end;
						;
						id id;
						label SampleID="SampleID";
					run;
					options label;
					
					%PackageInsertDataset(Imported &OutNameCombinedW wide output data from &ProcessName,outlib,%unquote(&OutNameCombinedW),No);
				%end;
							
				data OutLib.&AnnoOutNameCombined;
					set outSingleone;
					%if &GTFlag=1 %then %do;	
						drop 
							%do _j_=1 %to &_nGTvars_; 
								%sysfunc(strip(%sysfunc(varname(&_did_,&_j_))))
						 	%end;
						;
					%end;		
					*keep &headervarlist;
				run;
				%let _rc_ = %sysfunc(close(&_did_));
				
				%PackageInsertDataset(Imported &AnnoOutNameCombined annotation data from &ProcessName,outlib,%unquote(&AnnoOutNameCombined),No,_type=Anno);					

				%let dslist = &OutNameCombined..sas7bdat;
			%end;

		%end;
		%else %do;
			%put Do not merge multiple vcf files into one single file ###;
			
			%if %length(&OutDataPrefix) %then %do;	
				**subtract 2 for the length of _w or _t**;
				%if %sysevalf(&LPrefix + &&LInName&nf) > (32 - &nDig - 2) %then %do;
					%let NewOutNameN&nf = %substr(&OutDataPrefix._&&InName&nf,1,32 - &nDig - 2)_t_&nf;
					%let NewOutNameNW&nf= %substr(&OutDataPrefix._&&InName&nf,1,32 - &nDig - 2)_w_&nf;
				%end;
				%else %do;
					%let NewOutNameN&nf = &OutDataPrefix._&&InName&nf.._t_&nf;
					%let NewOutNameNW&nf= &OutDataPrefix._&&InName&nf.._w_&nf;
				%end;

				**subtract 5 for the length of _anno**;
				%if %sysevalf(&LPrefix + &&LInName&nf) > (32 - &nDig - 5) %then %do;
					%let NewAnnoOutNameN&nf = %substr(&OutDataPrefix._&&InName&nf,1,32 - &nDig - 5)_anno_&nf;
				%end;
				%else %do;
					%let NewAnnoOutNameN&nf = &OutDataPrefix._&&InName&nf.._anno_&nf;
				%end;
			
			%end;
			%else %do;
				**subtract 2 for the length of _w or _t**;
				%if %sysevalf(&&LInName&nf) > (32 - &nDig - 2) %then %do;
					%let NewOutNameN&nf = %substr(&&InName&nf,1,32 - &nDig - 2)_t_&nf;
					%let NewOutNameNW&nf= %substr(&&InName&nf,1,32 - &nDig - 2)_w_&nf;
				%end;
				%else %do;
					%let NewOutNameN&nf = &&InName&nf.._t_&nf;
					%let NewOutNameNW&nf= &&InName&nf.._w_&nf;
				%end;

				**subtract 5 for the length of _anno**;
				%if %sysevalf(&&LInName&nf) > (32 - &nDig - 5) %then %do;
					%let NewAnnoOutNameN&nf = %substr(&&InName&nf,1,32 - &nDig - 5)_anno_&nf;
				%end;
				%else %do;
					%let NewAnnoOutNameN&nf = &&InName&nf.._anno_&nf;
				%end;
			%end;
			
			%*put NewOutNameN&nf= &&NewOutNameN&nf;
			%*put NewOutNameNW&nf= &&NewOutNameNW&nf;
			%*put NewAnnoOutNameN&nf= &&NewAnnoOutNameN&nf;
			
			%CheckDataName(_name=NewOutNameN&nf);
			%CheckDataName(_name=NewOutNameNW&nf);
			%CheckDataName(_name=NewAnnoOutNameN&nf);				

			data outlib.%unquote(&&NewOutNameN&nf);
				set %unquote(&&OutName&nf);
			run;
			
			%PackageInsertDataset(Imported &&NewOutNameN&nf tall %if &GT %then output; data from &ProcessName,outlib,%unquote(&&NewOutNameN&nf),No,_type=tall);
						
			%put GT= &GT;
			%if %upcase(&OutputFormat)=YES and &GT=1 %then %do;

				options nolabel;
				proc transpose data=outlib.%unquote(&&NewOutNameN&nf)(rename=(%do j=10 %to &_nVars_; &&vname&j.._GT=&&vname&j %end;)) 
					let 
					out=outlib.%unquote(&&NewOutNameNW&nf)/*(rename=(_name_= SampleID))*/ 
					name=SampleID /*%if &IDType = N %then %do; prefix = _Variant_ %end;*/
					; 
					var %do j=10 %to &_nVars_; &&vname&j %end;;
					id id;
					label SampleID= "SampleID";
				run;
				options label;
				
				%PackageInsertDataset(Imported &&NewOutNameNW&nf wide output data from &ProcessName,outlib,%unquote(&&NewOutNameNW&nf),No);				
			%end;		
			
			data OutLib.&&NewAnnoOutNameN&nf;
				set outlib.%unquote(&&NewOutNameN&nf);
				%if &GT %then %do;	
					drop %do j=10 %to &_nVars_; &&vname&j.._GT %end;;
				%end;
				*keep &headervarlist;
			run;
			
			%PackageInsertDataset(Imported &&NewAnnoOutNameN&nf annotation data from &ProcessName,outlib,%unquote(&&NewAnnoOutNameN&nf),No,_type=Anno);
			
			%if &nf=1 %then %do;
				%let dslist = &&NewOutNameN&nf...sas7bdat;
			%end;
			%else %do;
				%let dslist = %unquote(&dslist |&&NewOutNameN&nf...sas7bdat);
			%end;

		%end;
	%end;
%end;

/* Button to launch IBS Sharing Regions */
* create JSL file;
%global IEButtonsScript;
%PrepOutFile(IEButtonsScript,IEButtonsScriptName,&ProcessName Script for &OutName1..jsl);
filename jslbtns "&IEButtonsScript";

%global _iewindow_ _mainjslfile_;
%let _iewindow_=cgbuttonswindow;
%let _mainjslfile_=%quote(&IEButtonsScript);
	
%if &GT and &NFiles=1 %then %do;

	data _null_;
		file jslbtns lrecl=32767;
		put "//!";
		put "Names Default to Here(1);";
		put "wait(0.5);";
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";
		put "Here:vcfbuttonswindow=Function({},{default local},";
		put "		VListBox(";
		put "			TextBox(""Click on the button below to launch the IBS Sharing Regions process using the output data sets.""),";
		put "			TextBox("" ""),";
		put "			ButtonBox(""IBS Sharing Regions"",";
		put "				if(host is(Windows),";
		put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/IBSSharingRegions.jsl"", absolute, windows)),";
		put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/IBSSharingRegions.jsl"", absolute, POSIX) ));";
		put "	";
		put "				jg:setDefaultParmValues(2, ""IBSSharingRegions"");";
		put "				jg:setCurrentParmValue(2, ""IBSSharingRegions"", jg:getParmNum(""IBSSharingRegions"",""InPath""),""&OutPath"");";
		put "				jg:IBSSharingRegions_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""IBSSharingRegions"", jg:getParmNum(""IBSSharingRegions"",""InDatas""),""&dslist"");";
		put "				jg:IBSSharingRegions_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""IBSSharingRegions"", jg:getParmNum(""IBSSharingRegions"",""ChrVar""),""CHROM"");";
		put "				jg:setCurrentParmValue(2, ""IBSSharingRegions"", jg:getParmNum(""IBSSharingRegions"",""PosVar""),""POS"");";
		put "				jg:setCurrentParmValue(2, ""IBSSharingRegions"", jg:getParmNum(""IBSSharingRegions"",""OutName""),""&OutName1"");";

		put "				jg:setCurrentParmValue(2, ""IBSSharingRegions"", jg:getParmNum(""IBSSharingRegions"",""OutPath""),""&ClientOutPath"");";
		put "			)";;
		put "		);";
		put ");";
		put "Here:LaunchCode = vcfbuttonswindow();";

	run;

%end;
%else %do;
	**creating a JSL file without code just to force the final jsl file to contain the process name on it instead of sasmessage**;
	* create JSL code;
	data _null_;
		file jslbtns lrecl=32767;
		put "//!";
		put "Names Default to Here(1);";
		put "Here:DataList = {};";
	run;
%end;
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &IEButtonsScriptName, , text/plain, yes);	

/* For Settings file */
%let OutName = &OutName1;

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend VCFInputEngine;
%VCFInputEngine;


