/*
JMP Genomics Analytical Process 

Name: ANOVA
Author: Russ Wolfinger, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
	sasrdw 25Oct04 initial coding from MixedModelAnalysis

Description: 
ANOVA fits a linear model to sequential
sets of observations of a pre-normalized intensity 
variable and creates numerous output displays. 
You can conduct similar and more advanced analyses
using MixedModelAnalysis. This process can be 
computationally intensive for large data sets.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/



*ProcessVariables;

%let StudyName=;
%let InData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Latin Square\affylatin_norm.sas7bdat);
%let ByVars= Probe_Set_ID;
%let BaseMergeVars= Probe_Set_ID;
%let LabelVar= ;
%let ChrVar= ;
%let PosVar= ;
%let InClassVars= Probe;
%let OutPath=%nrbquote($HOME/JMPG/ProcessResults/);
%let MacroPath= $GENOMICS_HOME\MacroLib;
%let ExpInData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Latin Square\affylatin_exp.sas7bdat);
%let ClassVars= Experiment Array;
%let DesignByVars= ;
%let SplitResults= ;
%let FixedEffects=%nrbquote(Probe);
%let RandomEffects=%nrbquote(Array(Experiment));
%let LsmeansEffects=%nrbquote(Experiment);
%let BaseSubjectVar= ;
%let BaselineVar= ;
%let BaselineValue=%nrbquote();
%let LSMeansDiffSet= Differences with a Control;
%let LSMeansControlValues=%nrbquote("a");
%let EstInFile=%nrbquote();
%let MultipleTestingFixedEffects=%nrbquote();
%let MultipleTestingMethod= FDR;
%let Alpha= ;
%let NegLog10pCutoff= 15;
%let LComponents= no;
%let PlotResiduals= no;
%let OutDataRes= ;
%let AnnoLabResid= ;
%let FilterType= False Positive Rate;
%let ResidualFalsePositive= 0.05;
%let ResidualCutoff= ;
%let MaxLoop= 3;
%let LSMeanStdMethod= STD;
%let ClusterLSM= yes;
%let Unbounded= no;
%let KenwardRoger= no;
%let UDiffs= no;
%let TStats= no;
%let PValues= no;
%let AdjPValues= no;
%let VolcanoUnifiedY= yes;
%let Speak= no;
%let JournalChrom= no;
%let ChunkSize= 1000;
%let OutData=%nrbquote();
%let JSLFile=%nrbquote();
%let OutDataComp=%nrbquote();
%let MMEIData=%nrbquote(affylatin_mmei);
%let AnnoInData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Latin Square\u95a_trim.sas7bdat);
%let AnnoMergeVars= Probe_Set_ID;
%let AnnoLabel= Probe_Set_ID;
%let AnnoChrVar= ;
%let AnnoPosVar= ;
%let AnnoWhereStmt=%nrbquote();
%let AnnoOnly= no;
%let AnnoAccession= Sequence_Derived_From;
%let AnnoGeneSymbol= Gene_Symbol;
%let AnnoDescription= Title;
%let AnnoGeneId= LocusLink;
%let AnnoSnpId= ;
%let AnnoKeepVars= ;
%let AnnoOrganism= Homo sapiens;
%let Anno2InData=%nrbquote();
%let Anno2MergeVars= ;
%let Anno2Label= ;
%let Anno2ChrVar= ;
%let Anno2PosVar= ;
%let Anno2WhereStmt= ;
%let Anno2Only= no;
%let TrackPath=%nrbquote();
%let TrackFilter= TrackGene.*\.sas;
%let TrackFiles= ;
%let GENOMICS_HOME=%nrbquote($GENOMICS_HOME\);
%let USER_HOME=%nrbquote($HOME\JMPG\);

*ProcessBody;

%include "$GENOMICS_HOME\ProcessLibrary\ANOVA.sas"; 