/*
SAS Scientific Discovery Solutions Analytical Process 

Name: ANOVA
Author: Russ Wolfinger, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
	sasrdw 25Oct04 initial coding from MixedModelAnalysis

Description: 
ANOVA fits a linear model to sequential
sets of observations of a pre-normalized intensity 
variable and creates numerous output displays. 
You can conduct similar and more advanced analyses
using MixedModelAnalysis. This process can be 
computationally intensive for large data sets.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/



*ProcessVariables;

%let StudyName=;
%let InData=%nrbquote($LS_HOME\Sample Data\Microarray\Scanalyze Drosophila\drosophilaaging_norm.sas7bdat);
%let ByVars= SPOT;
%let BaseMergeVars= SPOT;
%let InClassVars= ;
%let OutPath=%nrbquote($HOME/JMPG/ProcessResults/);
%let MacroPath= $GENOMICS_HOME\MacroLib;
%let AnnoInData=%nrbquote($LS_HOME\Sample Data\Microarray\Scanalyze Drosophila\drosophila_annotation.sas7bdat);
%let AnnoMergeVars= Spot;
%let AnnoAccession= Accession;
%let AnnoLabel= ShortDescription;
%let AnnoGeneSymbol= ;
%let AnnoDescription= ;
%let AnnoGeneId= ;
%let AnnoSnpId= ;
%let AnnoOrganism= Drosophila melanogaster;
%let ExpInData=%nrbquote($LS_HOME\Sample Data\Microarray\Scanalyze Drosophila\drosophilaaging_exp.sas7bdat);
%let ClassVars= Array Channel Line Sex Age;
%let DesignByVars= ;
%let SplitResults= ;
%let FixedEffects=%nrbquote(Line Sex Age Channel Line*Sex Line*Age Sex*Age);
%let LsmeansEffects=%nrbquote(Line*Sex*Age);
%let RandomEffects=%nrbquote(Array);
%let MultipleTestingMethod= FDR;
%let Alpha= 0.05;
%let NegLog10pCutoff= ;
%let PlotResiduals= yes;
%let OutDataRes=%nrbquote();
%let AnnoLabResid= no;
%let FilterType= False Positive Rate;
%let ResidualFalsePositive= 0.05;
%let ResidualCutoff= ;
%let MaxLoop= 3;
%let LSMeansDiffSet= All Pairwise Differences;
%let LSMeansControlValues= ;
%let LSMeanStdMethod= STD;
%let ClusterLSM= yes;
%let VolcanoUnifiedY= yes;
%let Unbounded= no;
%let KenwardRoger= no;
%let UDiffs= no;
%let TStats= no;
%let PValues= no;
%let AdjPValues= no;
%let Speak= no;
%let ChunkSize= 1000;
%let OutData=%nrbquote();
%let JSLFile=%nrbquote();
%let MMEIData=%nrbquote();

*ProcessBody;

%include "$GENOMICS_HOME\ProcessLibrary\ANOVA.sas"; 