/*
JMP Genomics Analytical Process 

Name: Basic Single Cell RNA-Seq Workflow
Author: Meijian Guan, SAS Institute Inc.

Description: 
		This process runs a basic workflow
		for expression data, including options
		for quality control and BasicExpressionWorkflow.
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/



*ProcessVariables;


%let StudyName=%nrbquote();
%let InData=%nrbquote($LS_HOME\Sample Data\scRNAseq\PBMC_dense.sas7bdat);
%let minFeature= 200;
%let maxFeature= 2500;
%let minCell= 3;
%let MtPercent= 5;
%let rmMt= No;
%let rmRib= No;
%let CalculationMethod=%nrbquote(Dispersion);
%let topGenes= 2000;
%let min_disp= 0.5;
%let max_disp= 100;
%let min_mean= 0.0125;
%let max_mean= 3;
%let NumSvd= 20;
%let ClusterMethod=%nrbquote(Hierarchical Clustering);
%let numClust= 9;
%let minClust= 3;
%let maxClust= 15;
%let OutData=%nrbquote(vargenes);
%let EmbDim= 2;
%let Perp= 30;
%let Iter= 500;
%let Nneighbors= 15;
%let Epoc= 200;
%let minDist= 0.1;
%let OutPath=%nrbquote($HOME\JMPG\ProcessResults\);
%let MacroPath= $LS_HOME\MacroLib;
%let testaddgene= Yes;
%let InGenes=%nrbquote($LS_HOME\Sample Data\scRNAseq\markergenes.sas7bdat);
%let testanova= Yes;
%let ExpInData=%nrbquote($LS_HOME\Sample Data\scRNAseq\PBMC_edf.sas7bdat);
%let testTsne= No;
%let Server= Local;
%let LS_HOME=%nrbquote($LS_HOME\);
%let CLIENT_LS_HOME=%nrbquote($LS_HOME\);
%let GENOMICS_HOME=%nrbquote($GENOMIC_HOME\);
%let USER_HOME=%nrbquote($HOME\JMPG\);
%let LSUSER_HOME=%nrbquote($HOME\JMPLS\);
%let CLIENT_USER_HOME=%nrbquote($HOME\JMPG\);
%let CLIENT_LSUSER_HOME=%nrbquote($HOME\JMPLS\);
%let Metapath=%nrbquote($HOME\JMPG\);
%let Starter_Index=%nrbquote(2);
%let LS_Product=%nrbquote(Genomics);
%let CLIENT_HOST= Windows;


*ProcessBody;
%macro runAP;  
filename customap "$HOME\JMPG\ProcessLibrary/BasicScRNASeqWorkflow.sas";
%if %sysfunc(fexist(customap)) %then %do;
   %include customap;
%end;
%else %do;
   %include "$GENOMIC_HOME\ProcessLibrary/BasicScRNASeqWorkflow.sas";
%end;
%mend;
%runAP;
      