/*
JMP Genomics Analytical Process 

Name: MixedModelAnalysis
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 28Aug02 initial coding
   sasrdw 01Jul03 added multiple testing methods
   sasrdw 24Mar04 added processing for lsmeans diff,
                  slice, and labels
   juzhan 02Mar05 added serval new annotation macro variable                  
	sasrdw 11Apr05 converted to tall input format                 

Description: 
MixedModelAnalysis fits a mixed linear model to
sets of observations of a pre-normalized intensity 
variable and creates numerous output displays.  Note: 
This process can be computationally intensive for
large data sets.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 
 
Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------

**/


*ProcessVariables;

%let StudyName=;
%let InData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Exon\huex_1_mn_trimmed.sas7bdat);
%let ByVars= transcript_cluster_id;
%let BaseMergeVars= ;
%let ChrVar= ;
%let PosVar= ;
%let OutPath=%nrbquote($HOME/JMPG/ProcessResults/);
%let MacroPath= $GENOMICS_HOME\MacroLib;
%let ExpInData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Exon\edf.sas7bdat);
%let DesignByVars= ;
%let SplitResults= ;
%let ProcMixedStatements=%nrbquote(class chip patient_number tissue_type probeset_id;
model response = tissue_type probeset_id tissue_type*probeset_id;
random patient_number chip;);
%let BaseSubjectVar= ;
%let BaselineVar= ;
%let BaselineValue=%nrbquote();
%let EstInFile=%nrbquote();
%let SimpleDiffs= no;
%let MultipleTestingFixedEffects=%nrbquote(Tissue_Type Tissue_Type*probeset_id);
%let MultipleTestingMethod= FDR;
%let Alpha= 0.05;
%let NegLog10pCutoff= ;
%let LComponents= yes;
%let PlotResiduals= no;
%let OutDataRes= ;
%let AnnoLabResid= ;
%let FilterType= None;
%let ResidualFalsePositive= ;
%let ResidualCutoff= ;
%let MaxLoop= ;
%let LSMeanStdMethod= STD;
%let ClusterLSM= no;
%let UDiffs= no;
%let TStats= no;
%let PValues= no;
%let AdjPValues= no;
%let VolcanoUnifiedY= yes;
%let Speak= no;
%let ListAll= No;
%let JournalChrom= no;
%let ProcMixedOptions= ;
%let ChunkSize= 60000;
%let OutData=%nrbquote();
%let JSLFile=%nrbquote();
%let OutDataComp=%nrbquote();
%let MMEIData=%nrbquote();
%let MixedModelBody= MixedModelBody.html;
%let MixedModelContents= MixedModelContents.html;
%let MixedModelFrame= MixedModelFrame.html;
%let AnnoInData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Exon\huex_1_0_st_v2_na30_hg19_tran_tm.sas7bdat);
%let AnnoMergeVars= transcript_cluster_id;
%let AnnoLabel= ;
%let AnnoChrVar= seqname;
%let AnnoPosVar= start;
%let AnnoWhereStmt=%nrbquote();
%let AnnoOnly= no;
%let AnnoAccession= ;
%let AnnoGeneSymbol= ;
%let AnnoDescription= ;
%let AnnoGeneId= ;
%let AnnoSnpId= ;
%let AnnoKeepVars= ;
%let AnnoOrganism= ;
%let Anno2InData=%nrbquote($LS_HOME\Sample Data\Microarray\Affymetrix Exon\huex_1_0_st_v2_na30_hg19_prob_tm.sas7bdat);
%let Anno2MergeVars= probeset_id;
%let Anno2Label= ;
%let Anno2ChrVar= seqname;
%let Anno2PosVar= start;
%let Anno2WhereStmt=%nrbquote();
%let Anno2Only= no;
%let TrackPath=%nrbquote($GENOMICS_HOME\Settings\);
%let TrackFilter= TrackGene.*\.sas;
%let TrackFiles= TrackGeneText__UCSC_hg18_knownGene.sas;
%let GENOMICS_HOME=%nrbquote($GENOMICS_HOME\);
%let USER_HOME=%nrbquote($HOME\JMPG\);

*ProcessBody;

%include "$GENOMICS_HOME\ProcessLibrary\MixedModelAnalysis.sas"; 